-- -------------------------------------------------------------------------- --
--         ____ _                ____                                         --
--        / ___| |__   __ _ _ __/ ___|  ___ __ _ _ __  _ __   ___ _ __        --
--       | |   | '_ \ / _` | '__\___ \ / __/ _` | '_ \| '_ \ / _ \ '__|       --
--       | |___| | | | (_| | |   ___) | (_| (_| | | | | | | |  __/ |          --
--        \____|_| |_|\__,_|_|  |____/ \___\__,_|_| |_|_| |_|\___|_|          --
--                                                                            --
-- -------------------------------------------------------------------------- --
--                            CharScanner by kunda                            --
-- -------------------------------------------------------------------------- --
--                                                                            --
-- CharScanner scans Basic Character Data, Talent Specifications, Equipment,  --
-- PvE/PvP Achievements, PvE Boss Kill Statistics and PvP Statistics from     --
-- single target or group and shows this data in a movable frame. Any scanned --
-- unit data is temporarily saved (in memory not in SavedVariables) and can   --
-- be viewed or deleted any time.                                             --
--                                                                            --
-- ONE click and ONE look shows the most important character data!            --
--                                                                            --
-- CharScanner does not rate, score or measure!                               --
-- CharScanner simply displays available ingame information in a nice way.    --
--                                                                            --
-- What you get in ~0.4 Seconds:                                              --
--  - A complete Talent scan (primary and secondary talent specification).    --
--  - A complete Gear scan.                                                   --
--  - A PvE dungeon and raid achievement/statistic scan.                      --
--  - A PvP achievement/statistic scan.                                       --
-- What you get in 50 Seconds if you scan your 25 player group:               --
--    ( 40 Seconds for Achievement/Talent scan restriction - Patch 3.3.5 )    --
--    ( 10 Seconds real scan time )                                           --
--  - All from above.                                                         --
--  - A nice Compare Tool.                                                    --
--                                                                            --
-- -------------------------------------------------------------------------- --
--                                                                            --
--      No SendAddonMessage(), no RealID/BN, no SavedVariables madness!       --
--                                                                            --
-- -------------------------------------------------------------------------- --
--                                                                            --
-- NOTES:                                                                     --
-- - Inspect range IS A MUST!                                                 --
-- - There is NO 'Inspect Glyphs API'! :-(                                    --
-- - Maximum duration for one scan is: 1 Second (default).                    --
--  ! Change this value in the options menu if you have a high latency or   ! --
--  ! if you get a 'Scan incomplete!' status message if you scan yourself.  ! --
-- - GroupScan:                                                               --
--    - Raid roster changes between a 'GroupScan'-Button click and end of     --
--      groupscan are discarded!                                              --
--    - GroupScan will stop if:                                               --
--        All group/raid members has been scanned. or                         --
--        You enter combat! or                                                --
--        You click on: 'STOP SCAN'                                           --
--    - 'GroupScanMissing'-Button will only scan party/raid members that were --
--       not scanned before (out-of-range/new/...).                           --
-- - Entering combat OR entering/leaving an instance OR zoning (with loading  --
--   screen) will stop any scan.                                              --
-- - Mainrole: (Lex DRUID)                                                    --
--    - DRUID TANK = Thick Hide + Natural Reaction >= 4                       --
-- - LDB is supported.                                                        --
--    But: LibStub, CallbackHandler-1.0 & LibDataBroker-1.1 is not included.  --
--         This is intended and I will not change this!                       --
-- - Scan duration:                                                           --
--  ! Blizzard added a Achievement/Talent scan restriction with Patch 3.3.5 ! --
--  ! Only 6 scans in a row are allowed, than a 10-second break is required ! --
--    Scan duration is mainly latency dependent! My tests:                    --
--    - SINGLESCAN:                                                           --
--      ~0.4 Seconds for one unit.                                            --
--    - GROUPSCAN:                                                            --
--      - A group with 25 player is normally scanned in less than 50 Seconds  --
--        if all members are in inspect range.                                --
--          player  1- 6: ~2.4 Seconds                                        --
--          10-second break: Achievement/Talent scan restriction              --
--          player  7-12: ~2.4 Seconds                                        --
--          10-second break: Achievement/Talent scan restriction              --
--          player 13-18: ~2.4 Seconds                                        --
--          10-second break: Achievement/Talent scan restriction              --
--          player 18-24: ~2.4 Seconds                                        --
--          10-second break: Achievement/Talent scan restriction              --
--          player 24-25: ~2.4 Seconds                                        --
--        40 Seconds for Achievement/Talent scan restriction                  --
--       ~10 Seconds real scan time                                           --
--      =~50 Seconds for a complete 25p-group scan                            --
--                                                                            --
-- -------------------------------------------------------------------------- --
--                                                                            --
-- TODO:                                                                      --
--  - Some frame improvements (undock/resize/move/scale).                     --
--  - More Compare.                                                           --
--                                                                            --
-- -------------------------------------------------------------------------- --
--                                                                            --
-- INCORRECT DATA:                                                            --
-- - Statistic id 4046/4045 (Anub'arak) bug was fixed with Patch 3.3.3.xxxxx, --
--   but the old 4045 statistic value which is returned from the server is    --
--   still the same (old wrong cumulative 4046-one).                          --
--   -> This means:                                                           --
--      The statistic value returned for id 4045 is NOT 100% CORRECT (!!!)    --
--      (armory too!).                                                        --
--      Look at local function BossStatistic() for the workaround.            --
-- - There is no boss kill statistics available ingame for Cataclysm. :(      --
--                                                                            --
-- -------------------------------------------------------------------------- --
--                                                                            --
-- NEVER EVER:                                                                --
-- - CharScanner will never ever save scanned data to SavedVariables.         --
--   - Most data will change every day, so it's completely useless to save    --
--     such data to SavedVariables.                                           --
--     The sheer amount of data makes such a feature a no go.                 --
-- - CharScanner will never ever autoscan a character or a group.             --
--   - You must actively initialize a single or group scan by clicking a      --
--     button.                                                                --
--     Triggering INSPECT_READY, INSPECT_TALENT_READY or                      --
--     INSPECT_ACHIEVEMENT_READY without user interaction is very bad and can --
--     cause massive lags.                                                    --
-- - CharScanner will never ever use SendAddonMessage()!!!                    --
--   - I hate SendAddonMessage()!                                             --
--   - EVERYBODY can fake any message that is send via SendAddonMessage().    --
--   - A messages from SendAddonMessage() is ALWAYS! an untrustable source!   --
--   - There is only one trustable source: The WoW server!                    --
-- - CharScanner will never ever use any RealID/BattleNet BN...() function!!! --
--   - NEVER EVER!!!                                                          --
--                                                                            --
-- -------------------------------------------------------------------------- --
--                                                                            --
-- slash commands: /cs - /scan - /charscanner                                 --
--        example: /scan         (CharScanner GUI & SINGLESCAN)               --
--        example: /scan options (CharScanner OPTIONS)                        --
--                                                                            --
-- -------------------------------------------------------------------------- --

-- ---------------------------------------------------------------------------------------------------------------------
CharScanner = CreateFrame("Frame")    -- container
CharScanner_Options = {}              -- SavedVariable options table

local _G                           = _G
local type                         = _G.type
local mod                          = _G.mod
local pairs                        = _G.pairs
local select                       = _G.select
local tonumber                     = _G.tonumber
local table_insert                 = _G.table.insert
local table_sort                   = _G.table.sort
local table_wipe                   = _G.table.wipe
local math_floor                   = _G.math.floor
local string_find                  = _G.string.find
local string_lower                 = _G.string.lower
local string_sub                   = _G.string.sub
local string_match                 = _G.string.match
local string_gsub                  = _G.string.gsub
local string_format                = _G.string.format
local GetTalentTabInfo             = _G.GetTalentTabInfo
local GetTalentInfo                = _G.GetTalentInfo
local GetTalentLink                = _G.GetTalentLink
local GetStatistic                 = _G.GetStatistic
local GetComparisonStatistic       = _G.GetComparisonStatistic
local GetAchievementComparisonInfo = _G.GetAchievementComparisonInfo
local GetAchievementInfo           = _G.GetAchievementInfo
local GetCategoryNumAchievements   = _G.GetCategoryNumAchievements
local UnitGUID                     = _G.UnitGUID
local GetTime                      = _G.GetTime
local GetInventoryItemLink         = _G.GetInventoryItemLink
local GetInventorySlotInfo         = _G.GetInventorySlotInfo
local GetInventoryItemTexture      = _G.GetInventoryItemTexture
local GetItemInfo                  = _G.GetItemInfo
local GetItemStats                 = _G.GetItemStats
local GetItemIcon                  = _G.GetItemIcon

local L                                 = _G.CharScanner_Localization          -- localization table
local PVEDUMP                           = _G.CharScanner_Data_PvE              -- PvE achievement & statistic data table
local PVPDUMP                           = _G.CharScanner_Data_PvP              -- PvP achievement & statistic data table
local CharScanner_GearStats_Enchantment = _G.CharScanner_GearStats_Enchantment -- Enchantment statistic data function
local CharScanner_GearStats_Reforge     = _G.CharScanner_GearStats_Reforge     -- Reforge statistic data function
local CharScanner_GearStats_Gem         = _G.CharScanner_GearStats_Gem         -- Gem statistic data function
local CharScanner_GearStats_Set         = _G.CharScanner_GearStats_Set         -- Set statistic data function

local GVAR = {}                       -- local UI Widgets
local TEMPLATE = {}                   -- local Templates

local mult2 = 10^(2 or 0)             -- x.xx
local mult4 = 10^(4 or 0)             -- x.xxx

local AchievementIDTable = {}         -- Achievement ID->name table (build after event PLAYER_LOGIN)
local StatisticIDTable = {}           -- Statistic ID->name table (build after event PLAYER_LOGIN)
local inCombat = false                -- false = out of combat | true = in combat
local inCombatGrpChk = false          -- party/raid member check if inCombat invite
local inPartyOrRaid = false           -- false = NOT in party or raid | true = in party or raid
local rosterhack = 0                  -- execute RAID_ROSTER_UPDATE block only if GetNumRaidMembers() has changed

local SingleScanIsActive = false      -- false = SingleScan is NOT active | true = SingleScan is active
local SingleScanTimerStart = 0        -- Timer (single scan) start (time for one scan)
local GroupScanIsActive = false       -- false = GroupScan is NOT active | true = GroupScan is active
local GroupScanTimerStart = 0         -- Timer (group scan) start (time for whole group scan)
local GroupScanNext = false           --
local GroupScanFirstGUID              -- value for GroupScan: first successful scan guid
local GroupMembers = {}               -- table with all current groupmembers (key = guid, value = unitID)
local GroupMissingMembers = {}        -- table with groupmembers that were not scanned (key = guid, value = unitID)
local GroupMissingNum = 0             -- number of missing group members (GroupMissingMembers count)
local GroupTabStatus = 1              -- group tab status: 1 = 'All' | 2 = 'Group' | 3 = 'Others'

local Event_INSPECT_ACHIEVEMENT_READY -- Achievement/Talent scan restriction: checkvalue
local Event_INSPECT_READY             -- Achievement/Talent scan restriction: checkvalue
local RestrictionTime = GetTime()     -- Achievement/Talent scan restriction: latest time event INSPECT_READY or INSPECT_ACHIEVEMENT_READY fired
local RestrictionAchNum = 0           -- Achievement/Talent scan restriction: count INSPECT_ACHIEVEMENT_READY event: 0 to 6
local RestrictionTalNum = 0           -- Achievement/Talent scan restriction: count INSPECT_READY event: 0 to 6
local RestrictionMaxTime = 10         -- Achievement/Talent scan restriction: max time to wait: 10 seconds
local RestrictionCountMax = 6         -- Achievement/Talent scan restriction: max scans in a row: 6
local RestrictionFirstTime            -- Achievement/Talent scan restriction: first time use (to allow scans after login)

local GUI_PvEstats_menu = {}              -- === PvE Achievement/Statistic collapse menu values
local GUI_PvPstats_menu = {}              -- === PvP Achievement/Statistic collapse menu values
local GUI_history_data = {}               -- === History (scanned units) data
local GUI_history_curRows = 25            -- current scroll frame rows
local GUI_history_minRows = 5             -- min. scroll frame rows
local GUI_history_maxRows = 25            -- max. scroll frame rows
local GUI_history_Count = 0               -- count
local GUI_history_SortBy = 1              -- sort by: 1 = class | 2 = name | 3 = time | 4 = mainrole | 5 = talents1 | 6 = talents2 | 7 = ItemLevel | 8 = Achievement | 9 = Boss | 10 = Kills
local GUI_history_SortOrder = 1           -- sort order: 0 = asc | 1 = desc
local GUI_history_RaidL1 = 0              -- raid level1
local GUI_history_RaidL2                  -- raid level2
local GUI_history_RaidL3                  -- raid level3
local GUI_achievementdetail_data = {}     -- === Achievement Detail data
local GUI_achievementdetail_curRows = 25  -- current scroll frame rows
local GUI_achievementdetail_minRows = 5   -- min. scroll frame rows
local GUI_achievementdetail_maxRows = 25  -- max. scroll frame rows
local GUI_achievementdetail_Count = 0     -- count
local GUI_achievementdetail_SortBy = 1    -- sort by: 1 = id | 2 = name
local GUI_achievementdetail_SortOrder = 1 -- sort order: 0 = asc | 1 = desc
local GUI_achievementdetail_Select = 1    -- 1 = ALL | 2 = COMPLETE | 3 = INCOMPLETE
local GUI_achievementdetail_Compare       -- id for Single Achievement Compare
local GUI_achievementdetail_L1            -- level1
local GUI_achievementdetail_L2            -- level2
local GUI_achievementdetail_L3            -- level3
local GUI_PvPachievementdetail_L1         -- PvP level1
local GUI_PvPachievementdetail_L2         -- PvP level2
local GUI_PvPachievementdetail_L3         -- PvP level3
local GUI_ABKsearch_data = {}             -- === ABK (stats) search result data
local GUI_ABKsearch_Count = 0             -- count
local GUI_ABKsearch_SortBy = 2            -- sort by: 1 = standard | 2 = name | 3 = kills
local GUI_ABKsearch_SortOrder = 1         -- sort order: 0 = asc | 1 = desc
local GUI_ABKsearch_curRows = 20          -- current scroll frame rows
local GUI_ABKsearch_minRows = 5           -- min. scroll frame rows
local GUI_ABKsearch_maxRows = 40          -- max. scroll frame rows

local GUIselectedGUID                 -- current shown char GUID
local NumScannedChars = 0             -- number of scanned characters
local playerGUID = ""                 -- player GUID (set after event PLAYER_LOGIN)
local playerFaction = ""              -- player Faction Horde/Alliance (set after event PLAYER_LOGIN)
local IsUnitInpected = false          -- scanned unit: true: "Inspect" | false: "Character" (it's me)
local SCANUNIT                        -- current scan Unit
local SCANGUID                        -- current scan GUID
local SCAN = {}                       -- scanned unit data (key = GUID)
local LatestScanIsComplete            -- true = last scan is complete | false/nil = last scan is NOT complete
local BossSearchQuery                 -- PvE boss name search query

local TalentTreeFrameIsShown          -- for mouseover
local GearFrameIsShown                -- for mouseover
local HistoryFrameIsShown             --
local CompareFrameIsShown             --
local AchievementDetailFrameMode      -- 'PvE' or 'PvP'

local myMAX_TIER = 7                  -- max. tier
local myMAX_COLUMN = 4                -- Talent Tree column
local myMAX_TALENT_TABS = 3           -- Blizzard default is 3 too -- need check at GVAR.TalentTreeFrame.Tree if this value changes
local myMAX_NUM_TALENTS = 28          -- Blizzard default is 28 too

local unitIDparty = {
	[1] = "party1",
	[2] = "party2",
	[3] = "party3",
	[4] = "party4"
}
local unitIDraid = {}
for i = 1, 40 do
	unitIDraid[i] = "raid"..i
end

-- magical 180 degree texture cut center rotation
local function rt(H,E,M,P)
	return E,P,E,M,H,P,H,M
end

local Textures = {
	CharScanner       = {path    = "Interface\\Addons\\CharScanner\\CharScanner-texture-button.tga"}, -- Textures.CharScanner.path
	CharScannerHL     = {path    = "Interface\\Addons\\CharScanner\\CharScanner-texture-button-highlight.tga"}, -- Textures.CharScannerHL.path
	CharScannerGUI    = {path    = "Interface\\Addons\\CharScanner\\CharScanner-texture-button-GUI.tga"}, -- Textures.CharScannerGUI.path
	CharScannerIcons  = {path    = "Interface\\Addons\\CharScanner\\CharScanner-texture-icons.tga"}, -- Textures.CharScannerIcons.path
	MinimapButton     = {--path    = CharScannerIcons -- Textures.MinimapButton
	                     coords    =    { 96/128, 128/128,   0/128,  32/128},
	                     width     = 32,
	                     height    = 32},
	-- --------------------------------------------------------------------------------
	ButtonHL          = {--path    = CharScannerIcons -- Textures.ButtonHL.
	                     coords    =    {109/128, 126/128,  37/128,  54/128}},
	SingleScan        = {--path    = CharScannerIcons -- Textures.SingleScan.
	                     coords    =    {  1/128,  32/128,  55/128,  72/128},
	                     width     = 31,
	                     height    = 17},
	GroupScanAll      = {--path    = CharScannerIcons -- Textures.GroupScanAll.
	                     coords    =    { 33/128,  64/128,  55/128,  72/128},
	                     width     = 31,
	                     height    = 17},
	GroupScanMissing  = {--path    = CharScannerIcons -- Textures.GroupScanMissing.
	                     coords    =    { 65/128,  96/128,  55/128,  72/128},
	                     width     = 31,
	                     height    = 17},
	Delete            = {--path    = CharScannerIcons -- Textures.Delete.
	                     coords    =    { 55/128,  72/128,   1/128,  18/128},
	                     width     = 17,
	                     height    = 17},
	History           = {--path    = CharScannerIcons -- Textures.History.
	                     coords    =    {  1/128,  18/128,   1/128,  18/128},
	                     width     = 17,
	                     height    = 17},
	Compare           = {--path    = CharScannerIcons -- Textures.Compare.
	                     coords    =    { 19/128,  36/128,   1/128,  18/128},
	                     width     = 17,
	                     height    = 17},
	Options           = {--path    = CharScannerIcons -- Textures.Options.
	                     coords    =    { 91/128, 108/128,  37/128,  54/128},
	                     width     = 17,
	                     height    = 17},
	Close             = {--path    = CharScannerIcons -- Textures.Close.
	                     coords    =    { 73/128,  90/128,   1/128,  18/128},
	                     width     = 17,
	                     height    = 17},
	Expand            = {--path    = CharScannerIcons -- Textures.Expand.
	                     coords    =    { 37/128,  54/128,   1/128,   18/128},
	                     width     = 17,
	                     height    = 17},
	Collapse          = {--path    = CharScannerIcons -- Textures.Collapse.
	                     coords    = {rt( 37/128,  54/128,   1/128,  18/128)},
	                     width     = 17,
	                     height    = 17},
	RightSide         = {--path    = CharScannerIcons -- Textures.RightSide.
	                     coords    =    { 97/128, 110/128,  55/128,  72/128},
	                     coordsOff = {rt( 97/128, 110/128,  55/128,  72/128)},
	                     width     = 13,
	                     height    = 17},
	FreeIcon          = {--path    = CharScannerIcons -- Textures.FreeIcon.
	                     coords    =    { 55/128,  72/128,  37/128,  54/128},
	                     width     = 17,
	                     height    = 17},
	DockedIcon        = {--path    = CharScannerIcons -- Textures.DockedIcon.
	                     coords    =    { 73/128,  90/128,  37/128,  54/128},
	                     width     = 17,
	                     height    = 17},
	-- --------------------------------------------------------------------------------
	ResizeUpDown      = {--path    = CharScannerIcons -- Textures.ResizeUpDown.
	                     coords    =    { 19/128,  36/128,  37/128,  54/128}},
	                     --qsize   = 17 -- sizeGeneral.SliderSize = 17
	ResizeCorner      = {--path    = CharScannerIcons -- Textures.ResizeCorner.
	                     coordsL   =    { 37/128,  54/128,  37/128,  54/128},
	                     coordsR   = {rt( 37/128,  54/128,  37/128,  54/128)}},
	                     --qsize   = 17 -- sizeGeneral.SliderSize = 17
	-- --------------------------------------------------------------------------------
	PlusMinus         = {--path      = CharScannerIcons -- Textures.PlusMinus.
	                     coordsPlus  =  { 92/128,  99/128, 113/128, 120/128},
	                     coordsMinus =  { 92/128,  99/128, 121/128, 128/128},
	                     qsize       = 7},
	AchievementDetail = {--path    = CharScannerIcons -- Textures.AchievementDetail.
	                     coords    =    {112/128, 128/128, 104/128, 128/128},
	                     width     = 8,  -- 16 scaled
	                     height    = 12},-- 24 scaled
	RaidSelect        = {--path    = CharScannerIcons -- Textures.RaidSelect.
	                     coords    =    { 19/128,  36/128,   1/128,  18/128},
	                     width     = 12, -- 17 scaled
	                     height    = 12},-- 17 scaled
	SortArrow         = {--path    = CharScannerIcons -- Textures.SortArrow.
	                     coords1   =    { 62/128,  69/128, 120/128, 128/128},
	                     coords2   = {rt( 62/128,  69/128, 120/128, 128/128)},
	                     width     = 7,
	                     height    = 8},
	SelectionArrow    = {--path    = CharScannerIcons -- Textures.SelectionArrow.
	                     coords    =    { 82/128,  91/128, 113/128, 128/128},
	                     width     = 7.2,-- scaled  9  7.2  6
	                     height    = 12},-- scaled 15 12   10
	Trashcan          = {--path    = CharScannerIcons -- Textures.Trashcan.
	                     coords    =    { 70/128,  81/128, 116/128, 128/128},
	                     width     = 11,
	                     height    = 12},
	-- --------------------------------------------------------------------------------
	ClassIcon         = {path    = "Interface\\WorldStateFrame\\Icons-Classes"}, -- Textures.ClassIcon.path
	                     --coords  = auto
	                     --width   = depends,
	                     --height  = depends},
	RoleIcon          = {[1] = {path = "Interface\\LFGFrame\\LFGRole",            coords = {32/64, 48/64, 0/16, 16/16}}, -- TANK  -- Textures.RoleIcon.path
	                     [2] = {path = "Interface\\LFGFrame\\LFGRole",            coords = {48/64, 64/64, 0/16, 16/16}}, -- HEAL
	                     [3] = {path = "Interface\\LFGFrame\\LFGRole",            coords = {16/64, 32/64, 0/16, 16/16}}, -- DAMAGE
	                     [4] = {path = "Interface\\GossipFrame\\ActiveQuestIcon", coords = {0, 1, 0, 1}},                -- UNKNOWN
	                     [5] = {path = "Interface\\LFGFrame\\LFGRole",            coords = {0/64, 1/64, 0/16, 1/16}},    -- NOTALENTS
	                     sWidth  = 16, -- status
	                     sHeight = 16, -- status
	                     width   = 12, -- 16 scaled
	                     height  = 12},-- 16 scaled
	TalentMiniIcons   = {--path    = auto, -- Textures.TalentMiniIcons.
	                     coords    = {5/64, 59/64, 5/64, 59/64},
	                     qsize     = 14},
	TalentIconTexture = {--path    = auto -- Textures.TalentIconTexture.
	                     --coords  = auto
	                     qsize     = 24},
	TalentRankBorder  = {--path    = CharScannerIcons -- Textures.TalentRankBorder.
	                     coords    =    {100/128, 111/128, 117/128, 128/128},
	                     qsize     = 11},
	GearIconTexture   = {--path    = auto -- Textures.GearIconTexture.
	                     --coords  = not used - default = {0, 1, 0, 1}
	                     qsize     = 24},
	GemIcon           = {--path    = CharScannerIcons -- Textures.GemIcon.
	                     coordsOK  =    { 44/128,  60/128,  91/128, 107/128}, -- 1 ok
	                     coordsUK  =    { 10/128,  26/128,  91/128, 107/128}, -- 2 unknown gem
	                     coordsNO  =    { 27/128,  43/128,  91/128, 107/128}, -- 3 socket, but no gem
	                     qsize     = 14},
	GemColorIcon      = {path      = "Interface\\ItemSocketingFrame\\UI-ItemSockets", -- Textures.GemColorIcon.path
	                     coordsBlu =    { 90/256, 133/256, 164/256, 207/256}, -- Blue
	                     coordsRed =    { 45/256,  88/256, 164/256, 207/256}, -- Red
	                     coordsYel =    {  0/256,  43/256, 164/256, 207/256}, -- Yellow
	                     coordsMet =    { 45/256, 100/256, 102/256, 157/256}, -- Meta
	                     qsize     = 20},
	EnchantmentIcon   = {--path    = CharScannerIcons -- Textures.EnchantmentIcon.
	                     coords    =    { 61/128,  77/128,  91/128, 107/128},
	                     qsize     = 16},
	PvPIcon           = {--path    = CharScannerIcons -- Textures.PvPIcon.
	                     coords    =    { 95/128, 111/128,  91/128, 107/128},
	                     qsize     = 12},
	ReforgeIcon       = {--path    = CharScannerIcons -- Textures.ReforgeIcon.
	                     coords    =    { 78/128,  94/128,  91/128, 107/128},
	                     qsize     = 16},
	SearchBoxIcons    = {--path    = CharScannerIcons -- Textures.SearchBoxIcons.
	                     coordsL   =    { 55/128,  60/128,  19/128,  36/128},
	                     coordsM   =    { 63/128,  64/128,  19/128,  36/128},
	                     coordsR   =    { 67/128,  72/128,  19/128,  36/128},
	                     coordsCLR =    { 73/128,  90/128,  19/128,  37/128},
	                     width     = 17,
	                     height    = 17},
	-- --------------------------------------------------------------------------------
	SliderKnob        = {--path    = CharScannerIcons -- Textures.SliderKnob.
	                     coords    =    { 50/128,  61/128, 111/128, 128/128},
	                     width     = 11,
	                     height    = 17},
	SliderBG          = {--path    = CharScannerIcons -- Textures.SliderBG.
	                     coordsL   =    { 62/128,  67/128, 109/128, 115/128},
	                     coordsM   =    { 70/128,  71/128, 109/128, 115/128},
	                     coordsR   =    { 74/128,  79/128, 109/128, 115/128}},
	ScrollFrameUp     = {--path    = CharScannerIcons -- Textures.ScrollFrameUp.
	                     coords    =    {  1/128,  18/128,  19/128,  36/128},
	                     coordsHL  =    { 19/128,  36/128,  19/128,  36/128},
	                     coordsDIS =    { 37/128,  54/128,  19/128,  36/128}},
	                     --qsize   = 17 -- sizeGeneral.SliderSize = 17
	ScrollFrameDown   = {--path    = CharScannerIcons -- Textures.ScrollFrameDown.
	                     coords    = {rt(  1/128,  18/128,  19/128,  36/128)},
	                     coordsHL  = {rt( 19/128,  36/128,  19/128,  36/128)},
	                     coordsDIS = {rt( 37/128,  54/128,  19/128,  36/128)}},
	                     --qsize   = 17 -- sizeGeneral.SliderSize = 17
	ScrollFrameKnob   = {--path    = CharScannerIcons -- Textures.ScrollFrameKnob.
	                     coords    =    {  1/128,  18/128,  37/128,  54/128}},
	                     --qsize   = 17 -- sizeGeneral.SliderSize = 17
}

-- fonts
local fonts = {
	GameFontNormal         = {name = "GameFontNormal"},         -- fonts.GameFontNormal.name
	GameFontNormalSmall    = {name = "GameFontNormalSmall"},    -- fonts.GameFontNormalSmall.name
	GameFontNormalLarge    = {name = "GameFontNormalLarge"},    -- fonts.GameFontNormalLarge.name
	GameFontWhiteSmall     = {name = "GameFontWhiteSmall"},     -- fonts.GameFontWhiteSmall.name
	GameFontHighlight      = {name = "GameFontHighlight"},      -- fonts.GameFontHighlight.name
	GameFontHighlightSmall = {name = "GameFontHighlightSmall"}, -- fonts.GameFontHighlightSmall.name
}
for key, value in pairs(fonts) do
	local _, fontHeight = _G[value.name]:GetFont()
	value.height = math_floor(fontHeight+0.5)
end

-- size General
local sizeGeneral = {}
sizeGeneral.MainWidth    = 450
sizeGeneral.Border       = 1
sizeGeneral.Space        = 2
sizeGeneral.Offset       = 5
sizeGeneral.SliderSize   = 17
sizeGeneral.BarHeight    = fonts.GameFontNormalSmall.height + sizeGeneral.Space -- 10+2=12
sizeGeneral.StatusHeight = sizeGeneral.Offset + fonts.GameFontNormalLarge.height + sizeGeneral.Offset -- 5++5=
sizeGeneral.StatusProgressBarWidth = 300

sizeGeneral.MainButtonHeight = sizeGeneral.Border + sizeGeneral.Space + Textures.SingleScan.height + sizeGeneral.Space + sizeGeneral.Border -- 23 - SingleScan.height is default button height
sizeGeneral.MainHeight       = sizeGeneral.Border + sizeGeneral.Offset + sizeGeneral.MainButtonHeight + sizeGeneral.Offset + sizeGeneral.Border -- 35
sizeGeneral.FreeBarHeight    = sizeGeneral.Border + sizeGeneral.Space + 12 + sizeGeneral.Space + sizeGeneral.Border -- 18

sizeGeneral.TabButtonHeight = sizeGeneral.Border + sizeGeneral.Space + fonts.GameFontNormalSmall.height + sizeGeneral.Space + sizeGeneral.Border -- 16
sizeGeneral.TabFrameHeight  = sizeGeneral.Border + sizeGeneral.Border + sizeGeneral.Space + sizeGeneral.TabButtonHeight -- 1 + 1 + 2 + 16 = 20
sizeGeneral.TabOffset       = 10 -- offset between the tabs -> autocalc | tab1 | TabOffset | tab2 | (TabOffset | tab3 |...) autocalc

-- size BaseCharFrame
local sizeBaseCharFrame_BaseHeight = sizeGeneral.Offset + fonts.GameFontNormal.height + sizeGeneral.Space + fonts.GameFontNormalSmall.height + sizeGeneral.Space + fonts.GameFontNormalSmall.height + sizeGeneral.Offset -- 46

-- size Talent-/TalentTreeFrame
local sizeTalentFrame = {}
sizeTalentFrame.BaseHeight           = sizeGeneral.Offset + Textures.TalentMiniIcons.qsize + Textures.TalentMiniIcons.qsize + sizeGeneral.Offset -- 5 + 12 + 12 + 5 = 34
sizeTalentFrame.TreeTopTextHeight    = fonts.GameFontWhiteSmall.height + sizeGeneral.Space + fonts.GameFontWhiteSmall.height + sizeGeneral.Space -- 10 + 2 + 10 + 2 == 24
sizeTalentFrame.TreeHeight           = sizeTalentFrame.TreeTopTextHeight + (myMAX_TIER*Textures.TalentIconTexture.qsize) + ((myMAX_TIER-1)*sizeGeneral.Offset) -- 24 + (11*20) + (10*5) = 294
sizeTalentFrame.TreeWidth            = (myMAX_COLUMN*Textures.TalentIconTexture.qsize) + ((myMAX_COLUMN-1)*sizeGeneral.Offset) -- (4*20) + (3*5) = 95
sizeTalentFrame.TreeFrameWidth       = (3*sizeTalentFrame.TreeWidth) + (4*sizeGeneral.TabOffset) -- (3*95) + (4*10) = 325 
sizeTalentFrame.TreeFrameHeight      = sizeGeneral.TabFrameHeight + sizeGeneral.Offset + sizeTalentFrame.TreeHeight + sizeGeneral.TabOffset -- 20 + 5 + 294 + 10 = 329
sizeTalentFrame.Tree1Left            = sizeGeneral.TabOffset -- 10
sizeTalentFrame.Tree2Left            = sizeTalentFrame.Tree1Left + sizeTalentFrame.TreeWidth + sizeGeneral.TabOffset --  10 + 95 + 10 = 115
sizeTalentFrame.Tree3Left            = sizeTalentFrame.Tree2Left + sizeTalentFrame.TreeWidth + sizeGeneral.TabOffset -- 115 + 95 + 10 = 220
sizeTalentFrame.GroupSelectionWidth  = (sizeTalentFrame.TreeFrameWidth/2) - (sizeGeneral.Offset+(sizeGeneral.Offset/2)) -- (325/2) - (5+(5/2)) = 155
sizeTalentFrame.GroupSelection1Left  = sizeGeneral.Offset
sizeTalentFrame.GroupSelection2Left  = sizeGeneral.Offset + sizeTalentFrame.GroupSelectionWidth + sizeGeneral.Offset

-- size Equipment-/GearFrame
local sizeEquipmentFrame = {}
sizeEquipmentFrame.BaseHeight    = sizeGeneral.Offset + fonts.GameFontNormalSmall.height + sizeGeneral.Space + sizeGeneral.Space + fonts.GameFontNormalSmall.height + sizeGeneral.Space + sizeGeneral.Space + fonts.GameFontNormalSmall.height + sizeGeneral.Offset -- 48
sizeEquipmentFrame.BaseHeightXXL = sizeGeneral.Offset + fonts.GameFontNormalSmall.height + sizeGeneral.Space + sizeGeneral.Space + fonts.GameFontNormalSmall.height + sizeGeneral.Space + sizeGeneral.Space + fonts.GameFontNormalSmall.height + fonts.GameFontNormalSmall.height + sizeGeneral.Offset -- 58

sizeEquipmentFrame.GearButtonWidth = Textures.GearIconTexture.qsize + 30 + sizeGeneral.Space + Textures.PvPIcon.qsize + sizeGeneral.Space + Textures.GearIconTexture.qsize + sizeGeneral.Space + Textures.EnchantmentIcon.qsize + sizeGeneral.Space + Textures.ReforgeIcon.qsize + sizeGeneral.Space + Textures.GemIcon.qsize + (sizeGeneral.Space/2) + Textures.GemIcon.qsize + (sizeGeneral.Space/2) + Textures.GemIcon.qsize + (sizeGeneral.Space/2) + Textures.GemIcon.qsize
sizeEquipmentFrame.GearPanelWidth = sizeGeneral.Offset + sizeEquipmentFrame.GearButtonWidth + sizeGeneral.Offset
sizeEquipmentFrame.GearWidth = sizeEquipmentFrame.GearPanelWidth*2

sizeEquipmentFrame.GearSummaryButtonWidth = sizeEquipmentFrame.GearButtonWidth
sizeEquipmentFrame.GearStatValueWidth = 60
sizeEquipmentFrame.GearStatKeyWidth = sizeEquipmentFrame.GearButtonWidth - sizeEquipmentFrame.GearStatValueWidth

sizeEquipmentFrame.GearHeight    = sizeGeneral.Offset + (17*Textures.GearIconTexture.qsize) + (14*sizeGeneral.Space) + (6*sizeGeneral.Offset) + sizeGeneral.Offset
sizeEquipmentFrame.GearSummaryHeight = sizeGeneral.Offset + (8*fonts.GameFontNormalSmall.height)+(7*sizeGeneral.Space) + sizeGeneral.Offset

-- size Achievement & Statistics
local sizeStatFrame = {}
sizeStatFrame.TabButtonWidth  = 120
sizeStatFrame.TabButton1Left  = (sizeGeneral.MainWidth - sizeStatFrame.TabButtonWidth - sizeStatFrame.TabButtonWidth - sizeGeneral.TabOffset)/2
sizeStatFrame.TabButton2Left  = sizeStatFrame.TabButton1Left + sizeStatFrame.TabButtonWidth + sizeGeneral.TabOffset

sizeStatFrame.OffsetTopSearch = sizeGeneral.Border + sizeGeneral.Space + Textures.SearchBoxIcons.height + sizeGeneral.Space -- 1+2+17+2 = 22
sizeStatFrame.BaseHeight      = sizeGeneral.TabFrameHeight + sizeStatFrame.OffsetTopSearch + (#PVEDUMP*sizeGeneral.BarHeight) + sizeGeneral.Offset -- 20+(3*12)+5 = 63
sizeStatFrame.LevelModifier   = Textures.PlusMinus.qsize + sizeGeneral.Space -- 7+2 = 9
sizeStatFrame.ButtonWidth     = sizeGeneral.MainWidth - sizeGeneral.Offset - sizeGeneral.Space - sizeGeneral.SliderSize - sizeGeneral.Space - sizeGeneral.Border
sizeStatFrame.SearchBoxWidth  = 80

sizeStatFrame.RaidSelectRight        = 0
sizeStatFrame.RaidSelectWidth        = Textures.RaidSelect.width
sizeStatFrame.BossesTotalRight       = sizeStatFrame.RaidSelectRight + sizeStatFrame.RaidSelectWidth + sizeGeneral.Space
sizeStatFrame.BossesTotalWidth       = 50
sizeStatFrame.StatisticRight         = sizeStatFrame.BossesTotalRight + sizeStatFrame.BossesTotalWidth + sizeGeneral.Space
sizeStatFrame.StatisticWidth         = 60
sizeStatFrame.AchievementDetailRight = sizeStatFrame.StatisticRight + sizeStatFrame.StatisticWidth + sizeGeneral.Space
sizeStatFrame.AchievementDetailWidth = Textures.AchievementDetail.width
sizeStatFrame.AchievementRight       = sizeStatFrame.AchievementDetailRight + sizeStatFrame.AchievementDetailWidth + sizeGeneral.Space
sizeStatFrame.AchievementWidth       = 60
sizeStatFrame.AchievementRightTitle  = sizeStatFrame.AchievementDetailRight + sizeStatFrame.AchievementDetailWidth + sizeGeneral.Space -- only title
sizeStatFrame.AchievementWidthTitle  = 80 -- only title

sizeStatFrame.NameLeft        = 0
sizeStatFrame.NameWidth       = sizeStatFrame.ButtonWidth - (sizeGeneral.Space + sizeStatFrame.AchievementWidth + sizeStatFrame.AchievementRight)

sizeStatFrame.NameWidthTitle  = sizeStatFrame.ButtonWidth - (sizeGeneral.Space + sizeStatFrame.AchievementWidthTitle + sizeStatFrame.AchievementRightTitle)
sizeStatFrame.NameWidthSearch = sizeStatFrame.ButtonWidth - (sizeGeneral.Space + sizeStatFrame.BossesTotalWidth + sizeStatFrame.BossesTotalRight)
sizeStatFrame.SearchSortStdLeft    = Textures.SearchBoxIcons.width + sizeGeneral.Space + sizeStatFrame.SearchBoxWidth + sizeGeneral.Space + Textures.SearchBoxIcons.width + sizeGeneral.Space + 40
sizeStatFrame.SearchSortStdWidth   = (sizeStatFrame.ButtonWidth - sizeStatFrame.SearchSortStdLeft - sizeGeneral.Space - sizeStatFrame.BossesTotalWidth - sizeStatFrame.BossesTotalRight - sizeGeneral.Space)/2
sizeStatFrame.SearchSortNameLeft   = sizeStatFrame.SearchSortStdLeft + sizeStatFrame.SearchSortStdWidth + sizeGeneral.Space
sizeStatFrame.SearchSortNameWidth  = sizeStatFrame.SearchSortStdWidth
sizeStatFrame.SearchSortKillsWidth = sizeStatFrame.BossesTotalWidth

sizeStatFrame.NameLeft_L1c    = sizeStatFrame.LevelModifier
sizeStatFrame.NameWidth_L1c   = sizeStatFrame.NameWidth - sizeStatFrame.LevelModifier
sizeStatFrame.NameLeft_L2     = sizeStatFrame.LevelModifier
sizeStatFrame.NameWidth_L2    = sizeStatFrame.NameWidth_L1c + sizeGeneral.Space + sizeStatFrame.AchievementWidth + sizeGeneral.Space + sizeStatFrame.AchievementDetailWidth + sizeGeneral.Space + sizeStatFrame.StatisticWidth
sizeStatFrame.NameLeft_L2c    = sizeStatFrame.LevelModifier * 2
sizeStatFrame.NameWidth_L2c   = sizeStatFrame.NameWidth_L1c - sizeStatFrame.LevelModifier
sizeStatFrame.NameLeft_L3     = sizeStatFrame.LevelModifier * 2
sizeStatFrame.NameWidth_L3    = sizeStatFrame.NameWidth_L2 - sizeStatFrame.LevelModifier
sizeStatFrame.NameLeft_L3c    = sizeStatFrame.LevelModifier * 3
sizeStatFrame.NameWidth_L3c   = sizeStatFrame.NameWidth_L2c - sizeStatFrame.LevelModifier
sizeStatFrame.NameLeft_L4     = sizeStatFrame.LevelModifier * 3
sizeStatFrame.NameWidth_L4    = sizeStatFrame.NameWidth_L3 - sizeStatFrame.LevelModifier

sizeStatFrame.SF_curHeight    = (GUI_ABKsearch_curRows*sizeGeneral.BarHeight) -- (20*12)
sizeStatFrame.SFpvp_curHeight = (GUI_ABKsearch_curRows*sizeGeneral.BarHeight) -- (20*12)
sizeStatFrame.Stats_curHeight = (GUI_ABKsearch_curRows*sizeGeneral.BarHeight) + sizeGeneral.TabFrameHeight + sizeStatFrame.OffsetTopSearch + sizeGeneral.Offset -- (20*12) + 20 + 5
sizeStatFrame.Stats_minHeight = (GUI_ABKsearch_minRows*sizeGeneral.BarHeight) + sizeGeneral.TabFrameHeight + sizeStatFrame.OffsetTopSearch + sizeGeneral.Offset -- (5*12) + 20 + 5
sizeStatFrame.Stats_maxHeight = (GUI_ABKsearch_maxRows*sizeGeneral.BarHeight) + sizeGeneral.TabFrameHeight + sizeStatFrame.OffsetTopSearch + sizeGeneral.Offset -- (40*12) + 20 + 5

sizeStatFrame.PvPHonorableKillsRight    = sizeStatFrame.RaidSelectRight + sizeStatFrame.RaidSelectWidth + sizeGeneral.Space
sizeStatFrame.PvPHonorableKillsWidth    = 50
sizeStatFrame.PvPWinLossPercentRight    = sizeStatFrame.PvPHonorableKillsRight + sizeStatFrame.PvPHonorableKillsWidth + sizeGeneral.Space
sizeStatFrame.PvPWinLossPercentWidth    = 50
sizeStatFrame.PvPWinLossRight           = sizeStatFrame.PvPWinLossPercentRight + sizeStatFrame.PvPWinLossPercentWidth + sizeGeneral.Space
sizeStatFrame.PvPWinLossWidth           = 70
sizeStatFrame.PvPAchievementDetailRight = sizeStatFrame.PvPWinLossRight + sizeStatFrame.PvPWinLossWidth + sizeGeneral.Space
sizeStatFrame.PvPAchievementDetailWidth = Textures.AchievementDetail.width
sizeStatFrame.PvPAchievementRight       = sizeStatFrame.PvPAchievementDetailRight + sizeStatFrame.PvPAchievementDetailWidth + sizeGeneral.Space
sizeStatFrame.PvPAchievementWidth       = 60
sizeStatFrame.PvPAchievementRightTitle  = sizeStatFrame.PvPAchievementDetailRight + sizeStatFrame.PvPAchievementDetailWidth + sizeGeneral.Space -- only title
sizeStatFrame.PvPAchievementWidthTitle  = 80 -- only title
sizeStatFrame.PvPWinLossPerRightTitle   = sizeStatFrame.PvPHonorableKillsRight + sizeStatFrame.PvPHonorableKillsWidth + sizeGeneral.Space -- only title
sizeStatFrame.PvPWinLossPerWidthTitle   = sizeStatFrame.PvPWinLossPercentWidth + sizeGeneral.Space + sizeStatFrame.PvPWinLossWidth -- only title

sizeStatFrame.PvPNameLeft        = 0
sizeStatFrame.PvPNameWidth       = sizeStatFrame.ButtonWidth - (sizeGeneral.Space + sizeStatFrame.PvPAchievementWidth + sizeStatFrame.PvPAchievementRight)

sizeStatFrame.PvPNameWidthTitle  = sizeStatFrame.ButtonWidth - (sizeGeneral.Space + sizeStatFrame.PvPAchievementWidthTitle + sizeStatFrame.PvPAchievementRightTitle)

sizeStatFrame.PvPNameLeft_L1c    = sizeStatFrame.LevelModifier
sizeStatFrame.PvPNameWidth_L1c   = sizeStatFrame.PvPNameWidth - sizeStatFrame.LevelModifier
sizeStatFrame.PvPNameLeft_L2     = sizeStatFrame.LevelModifier
sizeStatFrame.PvPNameWidth_L2    = sizeStatFrame.PvPNameWidth_L1c + sizeGeneral.Space + sizeStatFrame.PvPAchievementWidth + sizeGeneral.Space + sizeStatFrame.PvPAchievementDetailWidth + sizeGeneral.Space + sizeStatFrame.PvPWinLossWidth + sizeGeneral.Space + sizeStatFrame.PvPWinLossPercentWidth
sizeStatFrame.PvPNameLeft_L2c    = sizeStatFrame.LevelModifier * 2
sizeStatFrame.PvPNameWidth_L2c   = sizeStatFrame.PvPNameWidth_L1c - sizeStatFrame.LevelModifier
sizeStatFrame.PvPNameLeft_L3     = sizeStatFrame.LevelModifier * 2
sizeStatFrame.PvPNameWidth_L3    = sizeStatFrame.PvPNameWidth_L2 - sizeStatFrame.LevelModifier
sizeStatFrame.PvPNameLeft_L3c    = sizeStatFrame.LevelModifier * 3
sizeStatFrame.PvPNameWidth_L3c   = sizeStatFrame.PvPNameWidth_L2c - sizeStatFrame.LevelModifier
sizeStatFrame.PvPNameLeft_L4     = sizeStatFrame.LevelModifier * 3
sizeStatFrame.PvPNameWidth_L4    = sizeStatFrame.PvPNameWidth_L3 - sizeStatFrame.LevelModifier

-- size AchievementDetailFrame
local sizeAchievementDetail = {}
sizeAchievementDetail.SortHeight   = 12
sizeAchievementDetail.IdLeft       = 0
sizeAchievementDetail.IdWidth      = 35
sizeAchievementDetail.NameLeft     = sizeAchievementDetail.IdLeft + sizeAchievementDetail.IdWidth + sizeGeneral.Space
sizeAchievementDetail.NameWidth    = 220
sizeAchievementDetail.CompareLeft  = sizeAchievementDetail.NameLeft + sizeAchievementDetail.NameWidth + sizeGeneral.Space
sizeAchievementDetail.CompareWidth = Textures.RaidSelect.width
sizeAchievementDetail.ButtonWidth  = sizeAchievementDetail.CompareLeft + sizeAchievementDetail.CompareWidth
sizeAchievementDetail.Width        = sizeGeneral.Offset + sizeAchievementDetail.ButtonWidth + sizeGeneral.Space + sizeGeneral.SliderSize + sizeGeneral.Space + sizeGeneral.Border
sizeAchievementDetail.SortLine1Top    = sizeGeneral.Offset
sizeAchievementDetail.SortLine1Height = sizeGeneral.Border + sizeGeneral.Border + sizeGeneral.SliderSize + sizeGeneral.Border + sizeGeneral.Border --21
sizeAchievementDetail.SortLine2Top    = sizeAchievementDetail.SortLine1Top + sizeAchievementDetail.SortLine1Height + sizeGeneral.Space
sizeAchievementDetail.SortLine2Height = sizeGeneral.BarHeight
sizeAchievementDetail.SortLineXTop    = sizeAchievementDetail.SortLine2Top + sizeAchievementDetail.SortLine2Height + sizeGeneral.Space
sizeAchievementDetail.Height          = sizeAchievementDetail.SortLineXTop + (GUI_achievementdetail_curRows*sizeGeneral.BarHeight) + sizeGeneral.Offset
sizeAchievementDetail.InfoTexture       = 24
sizeAchievementDetail.InfoTextWidth     = sizeAchievementDetail.Width - (3*sizeGeneral.Offset) - sizeAchievementDetail.InfoTexture

-- size SummaryFrame
local sizeSummaryFrame_BaseHeight = sizeGeneral.Offset + sizeGeneral.BarHeight + sizeGeneral.BarHeight + sizeGeneral.Offset

-- size History/CompareFrame
local sizeCompareFrame = {}
sizeCompareFrame.RaidSelectLeft   = sizeGeneral.Space + sizeGeneral.SliderSize + sizeGeneral.Space + sizeGeneral.Border
sizeCompareFrame.TrashcanLeft     = 0
sizeCompareFrame.TrashcanWidth    = Textures.Trashcan.width
sizeCompareFrame.ArrowLeft        = sizeCompareFrame.TrashcanLeft + sizeCompareFrame.TrashcanWidth + sizeGeneral.Space
sizeCompareFrame.ArrowWidth       = Textures.SelectionArrow.width
sizeCompareFrame.ClassiconLeft    = sizeCompareFrame.ArrowLeft + sizeCompareFrame.ArrowWidth + sizeGeneral.Space
sizeCompareFrame.ClassiconWidth   = sizeGeneral.BarHeight
sizeCompareFrame.NameLeft         = sizeCompareFrame.ClassiconLeft + sizeCompareFrame.ClassiconWidth + sizeGeneral.Space
sizeCompareFrame.NameWidth        = 120
sizeCompareFrame.TimeLeft         = sizeCompareFrame.NameLeft + sizeCompareFrame.NameWidth + sizeGeneral.Space
sizeCompareFrame.TimeWidth        = 40
sizeCompareFrame.MainroleLeft     = sizeCompareFrame.TimeLeft + sizeCompareFrame.TimeWidth + sizeGeneral.Space
sizeCompareFrame.MainroleWidth    = 12
sizeCompareFrame.Talents1Left     = sizeCompareFrame.MainroleLeft + sizeCompareFrame.MainroleWidth + sizeGeneral.Space
sizeCompareFrame.Talents1Width    = 150
sizeCompareFrame.Talents2Left     = sizeCompareFrame.Talents1Left + sizeCompareFrame.Talents1Width + sizeGeneral.Space
sizeCompareFrame.Talents2Width    = 40
sizeCompareFrame.EnchLeft         = sizeCompareFrame.Talents2Left + sizeCompareFrame.Talents2Width + sizeGeneral.Space
sizeCompareFrame.EnchWidth        = 25
sizeCompareFrame.PvPLeft          = sizeCompareFrame.EnchLeft + sizeCompareFrame.EnchWidth + sizeGeneral.Space
sizeCompareFrame.PvPWidth         = 25
sizeCompareFrame.EquipLeft        = sizeCompareFrame.PvPLeft + sizeCompareFrame.PvPWidth + sizeGeneral.Space
sizeCompareFrame.EquipWidth       = 50
sizeCompareFrame.AchievementLeft  = sizeCompareFrame.EquipLeft + sizeCompareFrame.EquipWidth + sizeGeneral.Space
sizeCompareFrame.AchievementWidth = 80
sizeCompareFrame.BossLeft         = sizeCompareFrame.AchievementLeft + sizeCompareFrame.AchievementWidth + sizeGeneral.Space
sizeCompareFrame.BossWidth        = 80
sizeCompareFrame.KillsLeft        = sizeCompareFrame.BossLeft + sizeCompareFrame.BossWidth + sizeGeneral.Space
sizeCompareFrame.KillsWidth       = 50

sizeCompareFrame.SingleAchievementCompareLeft  = sizeCompareFrame.AchievementLeft
sizeCompareFrame.SingleAchievementCompareWidth = sizeCompareFrame.AchievementWidth + sizeGeneral.Space + sizeCompareFrame.BossWidth + sizeGeneral.Space + sizeCompareFrame.KillsWidth

sizeCompareFrame.HistoryButtonWidth = sizeCompareFrame.MainroleLeft + sizeCompareFrame.MainroleWidth
sizeCompareFrame.HistoryWidth       = sizeGeneral.Offset + sizeCompareFrame.HistoryButtonWidth + sizeGeneral.Space + sizeGeneral.SliderSize + sizeGeneral.Space + sizeGeneral.Border
sizeCompareFrame.CompareButtonWidth = sizeCompareFrame.KillsLeft + sizeCompareFrame.KillsWidth
sizeCompareFrame.CompareWidth       = sizeGeneral.Offset + sizeCompareFrame.CompareButtonWidth + sizeGeneral.Space + sizeGeneral.SliderSize + sizeGeneral.Space + sizeGeneral.Border
sizeCompareFrame.SortLine1Top     = sizeGeneral.Offset
sizeCompareFrame.SortLine1Height  = sizeGeneral.Border + sizeGeneral.Border + sizeGeneral.SliderSize + sizeGeneral.Border + sizeGeneral.Border -- 21
sizeCompareFrame.SortLine2Top     = sizeCompareFrame.SortLine1Top + sizeCompareFrame.SortLine1Height + sizeGeneral.Space
sizeCompareFrame.SortLine2Height  = sizeGeneral.BarHeight
sizeCompareFrame.SortLineXTop     = sizeCompareFrame.SortLine2Top + sizeCompareFrame.SortLine2Height + sizeGeneral.Space
sizeCompareFrame.HistoryHeight    = sizeCompareFrame.SortLineXTop + (GUI_history_curRows*sizeGeneral.BarHeight) + sizeGeneral.Offset

local DefaultPvEStatsViewData = { -- PvE Cataclysm default view
	[1] = {lvl = 1, name1 = "Cataclysm", index1 = 4, name2 = nil,        index2 = nil, name3 = nil, index3 = nil},
	[2] = {lvl = 2, name1 = "Cataclysm", index1 = 4, name2 = "dungeons", index2 = 1,   name3 = nil, index3 = nil},
	[3] = {lvl = 2, name1 = "Cataclysm", index1 = 4, name2 = "raids",    index2 = 2,   name3 = nil, index3 = nil}
}

local DefaultPvPStatsViewData = { -- PvP default view
	[1] = {lvl = 1, name1 = "Battlegrounds", index1 = 2, name2 = nil, index2 = nil, name3 = nil, index3 = nil},
	[2] = {lvl = 1, name1 = "Arena",         index1 = 3, name2 = nil, index2 = nil, name3 = nil, index3 = nil},
	[3] = {lvl = 1, name1 = "WorldPvP",      index1 = 4, name2 = nil, index2 = nil, name3 = nil, index3 = nil}
}

local color = { -- e.g.: color.White.hex | color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4]
	DarkGrey            = {rgba = {0.2,  0.2,  0.2,  1    }, hex = "333333"}, -- color.DarkGrey.
	Grey                = {rgba = {0.5,  0.5,  0.5,  1    }, hex = "808080"}, -- color.Grey.
	Red                 = {rgba = {1,    0,    0,    1    }, hex = "ff0000"}, -- color.Red.
	White               = {rgba = {1,    1,    1,    1    }, hex = "ffffff"}, -- color.White.
	Level1              = {rgba = {1,    1,    1,    1    }, hex = "ffffff"}, -- color.Level1.
	Level2              = {rgba = {0.8,  0.8,  0.8,  1    }, hex = "cccccc"}, -- color.Level2.
	Level3              = {rgba = {0.6,  0.6,  0.6,  1    }, hex = "999999"}, -- color.Level3.
	LightYellow         = {rgba = {1,    1,    0.49, 1    }, hex = "ffff7f"}, -- color.LightYellow.
	Yellow              = {rgba = {1,    0.82, 0,    1    }, hex = "ffd100"}, -- color.Yellow.
	Green               = {rgba = {0,    0.75, 0,    1    }, hex = "00bf00"}, -- color.Green.
	Brown               = {rgba = {0.78, 0.52, 0.28, 1    }, hex = "c78547"}, -- color.Brown.
	Firebrick           = {rgba = {0.8,  0.00, 0.00, 1    }, hex = "cd0000"}, -- color.Firebrick.
	SpellColor          = {rgba = {0.44, 0.83, 1.00, 1    }, hex = "71d5ff"}, -- color.SpellColor.
	Role         = {[1] = {rgba = {0.78, 0.52, 0.28, 1    }, hex = "c78547"}, -- color.Role[1]. - Brown     - TANK
	                [2] = {rgba = {0,    0.75, 0,    1    }, hex = "00bf00"}, -- color.Role[2]. - Green     - HEAL
	                [3] = {rgba = {0.8,  0.00, 0.00, 1    }, hex = "cd0000"}, -- color.Role[3]. - Firebrick - DAMAGE
	                [4] = {rgba = {1,    0.82, 0,    1    }, hex = "ffd100"}},-- color.Role[4]. - Yellow    - UNKNOWN
	FrameBG             = {rgba = {0,    0,    0,    1    }},                 -- color.FrameBG.
	FrameBorder         = {rgba = {0.4,  0.4,  0.4,  1    }},                 -- color.FrameBorder.
	ButtonBG            = {rgba = {0,    0,    0,    1    }},                 -- color.ButtonBG.
	ButtonBGActive      = {rgba = {0.25, 0.25, 0.25, 1    }},                 -- color.ButtonBGActive.
	ButtonHighlight     = {rgba = {1,    1,    1,    0.2  }},                 -- color.ButtonHighlight.
	ButtonHighlightLow  = {rgba = {0.6,  0.6,  0.6,  0.2  }},                 -- color.ButtonHighlightLow.
	ButtonHighlightRed  = {rgba = {0.55, 0,    0,    0.5  }},                 -- color.ButtonHighlightRed.
	ButtonBorder        = {rgba = {0.4,  0.4,  0.4,  1    }},                 -- color.ButtonBorder.
	ButtonBorderDisable = {rgba = {0.2,  0.2,  0.2,  1    }},                 -- color.ButtonBorderDisable.
	ButtonSortBG        = {rgba = {1,    1,    1,    0.1  }},                 -- color.ButtonSortBG.
	ButtonLineMarker    = {rgba = {0.06, 0.06, 0.06, 1    }},                 -- color.ButtonLineMarker.
}

local RoleTextShort = {
	[1] = "|cff"..color.Role[1].hex..L["T"].."|r", -- TANK
	[2] = "|cff"..color.Role[2].hex..L["H"].."|r", -- HEAL
	[3] = "|cff"..color.Role[3].hex..L["D"].."|r", -- DAMAGE
	[4] = "|cff"..color.Role[4].hex..L["?"].."|r"  -- UNKNOWN
}

local classcolors = {}
for class, color in pairs(RAID_CLASS_COLORS) do
	classcolors[class] = {r = color.r, g = color.g, b = color.b}
end
classcolors["OORCLASS"] = {r = 0.5, g = 0.5 , b = 0.5}

local classes = {-- the classes table is populated with talent data if a class is scanned the first time
	DEATHKNIGHT = {icon = {0.265625, 0.484375, 0.515625, 0.734375}, -- (68/256, 124/256, 132/256, 188/256)
	               roleNum = {[1] = 1,   -- TANK   | Blood
	                          [2] = 3,   -- DAMAGE | Frost
	                          [3] = 3}}, -- DAMAGE | Unholy
	DRUID       = {icon = {0.7578125, 0.9765625, 0.015625, 0.234375}, -- (194/256, 250/256, 4/256, 60/256)
	               roleNum = {[1] = 3,   -- DAMAGE | Balance
	                          [2] = 1,   -- TANK   | FeralCombat --> Thick Hide + Natural Reaction < 4 = DAMAGE
	                          [3] = 2}}, -- HEAL   | Restoration
	HUNTER      = {icon = {0.01953125, 0.23828125, 0.265625, 0.484375}, -- (5/256, 61/256, 68/256, 124/256)
	               roleNum = {[1] = 3,   -- DAMAGE | BeastMastery
	                          [2] = 3,   -- DAMAGE | Marksmanship
	                          [3] = 3}}, -- DAMAGE | Survival
	MAGE        = {icon = {0.265625, 0.484375, 0.015625, 0.234375}, -- (68/256, 124/256, 4/256, 60/256)
	               roleNum = {[1] = 3,   -- DAMAGE | Arcane
	                          [2] = 3,   -- DAMAGE | Fire
	                          [3] = 3}}, -- DAMAGE | Frost
	PALADIN     = {icon = {0.01953125, 0.23828125, 0.515625, 0.734375}, -- (5/256, 61/256, 132/256, 188/256)
	               roleNum = {[1] = 2,   -- HEAL   | Holy
	                          [2] = 1,   -- TANK   | Protection
	                          [3] = 3}}, -- DAMAGE | Combat
	PRIEST      = {icon = {0.51171875, 0.73046875, 0.265625, 0.484375}, -- (131/256, 187/256, 68/256, 124/256)
	               roleNum = {[1] = 2,   -- HEAL   | Discipline
	                          [2] = 2,   -- HEAL   | Holy
	                          [3] = 3}}, -- DAMAGE | Shadow
	ROGUE       = {icon = {0.51171875, 0.73046875, 0.015625, 0.234375}, -- (131/256, 187/256, 4/256, 60/256)
	               roleNum = {[1] = 3,   -- DAMAGE | Assassination
	                          [2] = 3,   -- DAMAGE | Combat
	                          [3] = 3}}, -- DAMAGE | Subtlety
	SHAMAN      = {icon = {0.265625, 0.484375, 0.265625, 0.484375}, -- (68/256, 124/256, 68/256, 124/256)
	               roleNum = {[1] = 3,   -- DAMAGE | ElementalCombat
	                          [2] = 3,   -- DAMAGE | Enhancement
	                          [3] = 2}}, -- HEAL   | Restoration
	WARLOCK     = {icon = {0.7578125, 0.9765625, 0.265625, 0.484375}, -- (194/256, 250/256, 68/256, 124/256)
	               roleNum = {[1] = 3,   -- DAMAGE | Curses
	                          [2] = 3,   -- DAMAGE | Summoning
	                          [3] = 3}}, -- DAMAGE | Destruction
	WARRIOR     = {icon = {0.01953125, 0.23828125, 0.015625, 0.234375}, -- (5/256, 61/256, 4/256, 60/256)
	               roleNum = {[1] = 3,   -- DAMAGE | Arms
	                          [2] = 3,   -- DAMAGE | Fury
	                          [3] = 1}}, -- TANK   | Protection
	OORCLASS    = {icon = {0, 0, 0, 0},
	               roleNum = {[1] = 4,   --
	                          [2] = 4,   --
	                          [3] = 4}}, --
}

local guildRank = {
	[1] = GUILD_RANK0_DESC, -- 0 | Guild Master
	[2] = GUILD_RANK1_DESC, -- 1 | Officer
	[3] = GUILD_RANK2_DESC, -- 2 | Veteran
	[4] = GUILD_RANK3_DESC, -- 3 | Member
	[5] = GUILD_RANK4_DESC  -- 4 | Initiate
}

local slotButton = {
	[1]  = {id =  1, name = "HeadSlot",          lname = _G[string.upper("HeadSlot")] or "bug",          texture="Interface\\Paperdoll\\UI-PaperDoll-Slot-Head"},
	[2]  = {id =  2, name = "NeckSlot",          lname = _G[string.upper("NeckSlot")] or "bug",          texture="Interface\\Paperdoll\\UI-PaperDoll-Slot-Neck"},
	[3]  = {id =  3, name = "ShoulderSlot",      lname = _G[string.upper("ShoulderSlot")] or "bug",      texture="Interface\\Paperdoll\\UI-PaperDoll-Slot-Shoulder"},
	[4]  = {id =  4, name = "BackSlot",          lname = _G[string.upper("BackSlot")] or "bug",          texture="Interface\\Paperdoll\\UI-PaperDoll-Slot-Chest"}, -- Back
	[5]  = {id =  5, name = "ChestSlot",         lname = _G[string.upper("ChestSlot")] or "bug",         texture="Interface\\Paperdoll\\UI-PaperDoll-Slot-Chest"},
	[6]  = {id =  6, name = "WristSlot",         lname = _G[string.upper("WristSlot")] or "bug",         texture="Interface\\Paperdoll\\UI-PaperDoll-Slot-Wrists"},
	[7]  = {id =  7, name = "HandsSlot",         lname = _G[string.upper("HandsSlot")] or "bug",         texture="Interface\\Paperdoll\\UI-PaperDoll-Slot-Hands"},
	[8]  = {id =  8, name = "WaistSlot",         lname = _G[string.upper("WaistSlot")] or "bug",         texture="Interface\\Paperdoll\\UI-PaperDoll-Slot-Waist"},
	[9]  = {id =  9, name = "LegsSlot",          lname = _G[string.upper("LegsSlot")] or "bug",          texture="Interface\\Paperdoll\\UI-PaperDoll-Slot-Legs"},
	[10] = {id = 10, name = "FeetSlot",          lname = _G[string.upper("FeetSlot")] or "bug",          texture="Interface\\Paperdoll\\UI-PaperDoll-Slot-Feet"},
	[11] = {id = 11, name = "Finger0Slot",       lname = _G[string.upper("Finger0Slot")] or "bug",       texture="Interface\\Paperdoll\\UI-PaperDoll-Slot-Finger"},
	[12] = {id = 12, name = "Finger1Slot",       lname = _G[string.upper("Finger1Slot")] or "bug",       texture="Interface\\Paperdoll\\UI-PaperDoll-Slot-Finger"},
	[13] = {id = 13, name = "Trinket0Slot",      lname = _G[string.upper("Trinket0Slot")] or "bug",      texture="Interface\\Paperdoll\\UI-PaperDoll-Slot-Trinket"},
	[14] = {id = 14, name = "Trinket1Slot",      lname = _G[string.upper("Trinket1Slot")] or "bug",      texture="Interface\\Paperdoll\\UI-PaperDoll-Slot-Trinket"},
	[15] = {id = 15, name = "MainHandSlot",      lname = _G[string.upper("MainHandSlot")] or "bug",      texture="Interface\\Paperdoll\\UI-PaperDoll-Slot-MainHand"}, -- middle
	[16] = {id = 16, name = "SecondaryHandSlot", lname = _G[string.upper("SecondaryHandSlot")] or "bug", texture="Interface\\Paperdoll\\UI-PaperDoll-Slot-SecondaryHand"}, -- middle
	[17] = {id = 17, name = "RangedSlot",        lname = _G[string.upper("RangedSlot")] or "bug",        texture="Interface\\Paperdoll\\UI-PaperDoll-Slot-Ranged"}, -- middle
}

local itemQuality = {
	[1] = {name = ITEM_QUALITY0_DESC, color = "9d9d9d", rgb = {0.615, 0.615, 0.615}}, -- 0 | poor
	[2] = {name = ITEM_QUALITY1_DESC, color = "ffffff", rgb = {1.000, 1.000, 1.000}}, -- 1 | common
	[3] = {name = ITEM_QUALITY2_DESC, color = "1eff00", rgb = {0.117, 1.000, 0.000}}, -- 2 | uncommon
	[4] = {name = ITEM_QUALITY3_DESC, color = "0070dd", rgb = {0.000, 0.439, 0.866}}, -- 3 | rare
	[5] = {name = ITEM_QUALITY4_DESC, color = "a335ee", rgb = {0.639, 0.207, 0.933}}, -- 4 | epic
	[6] = {name = ITEM_QUALITY5_DESC, color = "ff8000", rgb = {1.000, 0.501, 0.000}}, -- 5 | legendary
	[7] = {name = ITEM_QUALITY6_DESC, color = "e6cc80", rgb = {0.901, 0.800, 0.501}}, -- 6 | artifact (not in game)
	[8] = {name = ITEM_ACCOUNTBOUND,  color = "e6cc80", rgb = {0.901, 0.800, 0.501}}, -- 7 | account bound
	[9] = {name = UNKNOWN,            color = "666666", rgb = {0.400, 0.400, 0.400}}  -- 8 | unknown (item that is not in local cache or currently not availabe)
}
local itemQualityNum = #itemQuality
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local bonusStr = string_format(ITEM_SOCKET_BONUS, "") -- GearScan Gem Socket Bonus

local gemLocSocketsBLUE      = _G["EMPTY_SOCKET_BLUE"]
local gemLocSocketsRED       = _G["EMPTY_SOCKET_RED"]
local gemLocSocketsYELLOW    = _G["EMPTY_SOCKET_YELLOW"]
local gemLocSocketsMETA      = _G["EMPTY_SOCKET_META"]
local gemLocSocketsPRISMATIC = _G["EMPTY_SOCKET_PRISMATIC"]
local gemLocSocketsCOGWHEEL  = _G["EMPTY_SOCKET_COGWHEEL"]
local gemLocSocketsHYDRAULIC = _G["EMPTY_SOCKET_HYDRAULIC"]
local gemLocSocketsNO_COLOR  = _G["EMPTY_SOCKET_NO_COLOR"]

local gemSockets = {
	["EMPTY_SOCKET_BLUE"]      = 1,
	["EMPTY_SOCKET_RED"]       = 2,
	["EMPTY_SOCKET_YELLOW"]    = 3,
	["EMPTY_SOCKET_META"]      = 4,
	["EMPTY_SOCKET_PRISMATIC"] = 4,
	["EMPTY_SOCKET_COGWHEEL"]  = 4,
	["EMPTY_SOCKET_HYDRAULIC"] = 4,
	["EMPTY_SOCKET_NO_COLOR"]  = 4
}

local statName = {
	["ITEM_MOD_STRENGTH_SHORT"]            = 1,  -- STRENGTH     = 1
	["ITEM_MOD_AGILITY_SHORT"]             = 2,  -- AGILITY      = 2
	["ITEM_MOD_STAMINA_SHORT"]             = 3,  -- STAMINA      = 3
	["ITEM_MOD_INTELLECT_SHORT"]           = 4,  -- INTELLECT    = 4
	["ITEM_MOD_SPIRIT_SHORT"]              = 5,  -- SPIRIT       = 5

	["ITEM_MOD_ATTACK_POWER_SHORT"]        = 6,  -- ATTACKPOWER  = 6
	["ITEM_MOD_BLOCK_RATING_SHORT"]        = 7,  -- BLOCK        = 7
	["ITEM_MOD_DODGE_RATING_SHORT"]        = 8,  -- DODGE        = 8
	["ITEM_MOD_PARRY_RATING_SHORT"]        = 9,  -- PARRY        = 9
	["ITEM_MOD_HIT_RATING_SHORT"]          = 10, -- HIT          = 10
	["ITEM_MOD_CRIT_RATING_SHORT"]         = 11, -- CRIT         = 11
	["ITEM_MOD_HASTE_RATING_SHORT"]        = 12, -- HASTE        = 12
	["ITEM_MOD_EXPERTISE_RATING_SHORT"]    = 13, -- EXPERTISE    = 13
	["ITEM_MOD_MASTERY_RATING_SHORT"]      = 14, -- MASTERY      = 14
	["ITEM_MOD_SPELL_POWER_SHORT"]         = 15, -- SPELLPOWER   = 15

	["ITEM_MOD_SPELL_PENETRATION_SHORT"]   = 16, -- PENETRATION  = 16
	["ITEM_MOD_RESILIENCE_RATING_SHORT"]   = 17, -- RESILIENCE   = 17

	["RESISTANCE0_NAME"]                   = 18, -- ARMOR        = 18
	["RESISTANCE1_NAME"]                   = 19, -- HOLY_RES     = 19 - ITEM_MOD_HOLY_RESISTANCE_SHORT?  -> I have not seen this ingame
	["RESISTANCE2_NAME"]                   = 20, -- FIRE_RES     = 20 - ITEM_MOD_FIRE_RESISTANCE_SHORT   -> available ingame (via GetItemStats), but there is no global in GlobalStrings.lua
	["RESISTANCE3_NAME"]                   = 21, -- NATURE_RES   = 21 - ITEM_MOD_NATURE_RESISTANCE_SHORT -> available ingame (via GetItemStats), but there is no global in GlobalStrings.lua
	["RESISTANCE4_NAME"]                   = 22, -- FROST_RES    = 22 - ITEM_MOD_FROST_RESISTANCE_SHORT  -> available ingame (via GetItemStats), but there is no global in GlobalStrings.lua
	["RESISTANCE5_NAME"]                   = 23, -- SHADOW_RES   = 23 - ITEM_MOD_SHADOW_RESISTANCE_SHORT -> available ingame (via GetItemStats), but there is no global in GlobalStrings.lua
	["RESISTANCE6_NAME"]                   = 24, -- ARCANE_RES   = 24 - ITEM_MOD_ARCANE_RESISTANCE_SHORT -> available ingame (via GetItemStats), but there is no global in GlobalStrings.lua

	["ITEM_MOD_HEALTH_SHORT"]              = 25, -- HEALTH       = 25
	["ITEM_MOD_MANA_SHORT"]                = 26, -- MANA         = 26

	["ITEM_MOD_HASTE_RANGED_RATING_SHORT"] = 27, -- HASTE_RANGED = 27
	["ITEM_MOD_HIT_RANGED_RATING_SHORT"]   = 28, -- HIT_RANGED   = 28
	["ITEM_MOD_CRIT_RANGED_RATING_SHORT"]  = 29, -- CRIT_RANGED  = 29
}

local statGlobalName = {
	[1]  = "ITEM_MOD_STRENGTH_SHORT",            -- STRENGTH     = 1
	[2]  = "ITEM_MOD_AGILITY_SHORT",             -- AGILITY      = 2
	[3]  = "ITEM_MOD_STAMINA_SHORT",             -- STAMINA      = 3
	[4]  = "ITEM_MOD_INTELLECT_SHORT",           -- INTELLECT    = 4
	[5]  = "ITEM_MOD_SPIRIT_SHORT",              -- SPIRIT       = 5

	[6]  = "ITEM_MOD_ATTACK_POWER_SHORT",        -- ATTACKPOWER  = 6
	[7]  = "ITEM_MOD_BLOCK_RATING_SHORT",        -- BLOCK        = 7
	[8]  = "ITEM_MOD_DODGE_RATING_SHORT",        -- DODGE        = 8
	[9]  = "ITEM_MOD_PARRY_RATING_SHORT",        -- PARRY        = 9
	[10] = "ITEM_MOD_HIT_RATING_SHORT",          -- HIT          = 10
	[11] = "ITEM_MOD_CRIT_RATING_SHORT",         -- CRIT         = 11
	[12] = "ITEM_MOD_HASTE_RATING_SHORT",        -- HASTE        = 12
	[13] = "ITEM_MOD_EXPERTISE_RATING_SHORT",    -- EXPERTISE    = 13
	[14] = "ITEM_MOD_MASTERY_RATING_SHORT",      -- MASTERY      = 14
	[15] = "ITEM_MOD_SPELL_POWER_SHORT",         -- SPELLPOWER   = 15

	[16] = "ITEM_MOD_SPELL_PENETRATION_SHORT",   -- PENETRATION  = 16
	[17] = "ITEM_MOD_RESILIENCE_RATING_SHORT",   -- RESILIENCE   = 17

	[18] = "RESISTANCE0_NAME",                   -- ARMOR        = 18
	[19] = "RESISTANCE1_NAME",                   -- HOLY_RES     = 19 - ITEM_MOD_HOLY_RESISTANCE_SHORT?  -> I have not seen this ingame
	[20] = "RESISTANCE2_NAME",                   -- FIRE_RES     = 20 - ITEM_MOD_FIRE_RESISTANCE_SHORT   -> available ingame (via GetItemStats), but there is no global in GlobalStrings.lua
	[21] = "RESISTANCE3_NAME",                   -- NATURE_RES   = 21 - ITEM_MOD_NATURE_RESISTANCE_SHORT -> available ingame (via GetItemStats), but there is no global in GlobalStrings.lua
	[22] = "RESISTANCE4_NAME",                   -- FROST_RES    = 22 - ITEM_MOD_FROST_RESISTANCE_SHORT  -> available ingame (via GetItemStats), but there is no global in GlobalStrings.lua
	[23] = "RESISTANCE5_NAME",                   -- SHADOW_RES   = 23 - ITEM_MOD_SHADOW_RESISTANCE_SHORT -> available ingame (via GetItemStats), but there is no global in GlobalStrings.lua
	[24] = "RESISTANCE6_NAME",                   -- ARCANE_RES   = 24 - ITEM_MOD_ARCANE_RESISTANCE_SHORT -> available ingame (via GetItemStats), but there is no global in GlobalStrings.lua

	[25] = "ITEM_MOD_HEALTH_SHORT",              -- HEALTH       = 25
	[26] = "ITEM_MOD_MANA_SHORT",                -- MANA         = 26

	[27] = "ITEM_MOD_HASTE_RANGED_RATING_SHORT", -- HASTE_RANGED = 27
	[28] = "ITEM_MOD_HIT_RANGED_RATING_SHORT",   -- HIT_RANGED   = 28
	[29] = "ITEM_MOD_CRIT_RANGED_RATING_SHORT",  -- CRIT_RANGED  = 29
}

local statLocName = {
	[1]  = _G["ITEM_MOD_STRENGTH_SHORT"] or "Strength",               -- STRENGTH     = 1
	[2]  = _G["ITEM_MOD_AGILITY_SHORT"] or "Agility",                 -- AGILITY      = 2
	[3]  = _G["ITEM_MOD_STAMINA_SHORT"] or "Stamina",                 -- STAMINA      = 3
	[4]  = _G["ITEM_MOD_INTELLECT_SHORT"] or "Intellect",             -- INTELLECT    = 4
	[5]  = _G["ITEM_MOD_SPIRIT_SHORT"] or "Spirit",                   -- SPIRIT       = 5

	[6]  = _G["ITEM_MOD_ATTACK_POWER_SHORT"] or "Attackpower",        -- ATTACKPOWER  = 6
	[7]  = _G["ITEM_MOD_BLOCK_RATING_SHORT"] or "Block",              -- BLOCK        = 7
	[8]  = _G["ITEM_MOD_DODGE_RATING_SHORT"] or "Dodge",              -- DODGE        = 8
	[9]  = _G["ITEM_MOD_PARRY_RATING_SHORT"] or "Parry",              -- PARRY        = 9
	[10] = _G["ITEM_MOD_HIT_RATING_SHORT"] or "Hit",                  -- HIT          = 10
	[11] = _G["ITEM_MOD_CRIT_RATING_SHORT"] or "Crit",                -- CRIT         = 11
	[12] = _G["ITEM_MOD_HASTE_RATING_SHORT"] or "Haste",              -- HASTE        = 12
	[13] = _G["ITEM_MOD_EXPERTISE_RATING_SHORT"] or "Expertise",      -- EXPERTISE    = 13
	[14] = _G["ITEM_MOD_MASTERY_RATING_SHORT"] or "Mastery",          -- MASTERY      = 14
	[15] = _G["ITEM_MOD_SPELL_POWER_SHORT"] or "Spellpower",          -- SPELLPOWER   = 15

	[16] = _G["ITEM_MOD_SPELL_PENETRATION_SHORT"] or "Penetration",   -- PENETRATION  = 16
	[17] = _G["ITEM_MOD_RESILIENCE_RATING_SHORT"] or "Resilience",    -- RESILIENCE   = 17

	[18] = _G["RESISTANCE0_NAME"] or "Armor",                         -- ARMOR        = 18
	[19] = _G["RESISTANCE1_NAME"] or "Holy Resistance",               -- HOLY_RES     = 19 - ITEM_MOD_HOLY_RESISTANCE_SHORT?  -> I have not seen this ingame
	[20] = _G["RESISTANCE2_NAME"] or "Fire Resistance",               -- FIRE_RES     = 20 - ITEM_MOD_FIRE_RESISTANCE_SHORT   -> available ingame (via GetItemStats), but there is no global in GlobalStrings.lua
	[21] = _G["RESISTANCE3_NAME"] or "Nature Resistance",             -- NATURE_RES   = 21 - ITEM_MOD_NATURE_RESISTANCE_SHORT -> available ingame (via GetItemStats), but there is no global in GlobalStrings.lua
	[22] = _G["RESISTANCE4_NAME"] or "Frost Resistance",              -- FROST_RES    = 22 - ITEM_MOD_FROST_RESISTANCE_SHORT  -> available ingame (via GetItemStats), but there is no global in GlobalStrings.lua
	[23] = _G["RESISTANCE5_NAME"] or "Shadow Resistance",             -- SHADOW_RES   = 23 - ITEM_MOD_SHADOW_RESISTANCE_SHORT -> available ingame (via GetItemStats), but there is no global in GlobalStrings.lua
	[24] = _G["RESISTANCE6_NAME"] or "Arcane Resistance",             -- ARCANE_RES   = 24 - ITEM_MOD_ARCANE_RESISTANCE_SHORT -> available ingame (via GetItemStats), but there is no global in GlobalStrings.lua

	[25] = _G["ITEM_MOD_HEALTH_SHORT"] or "Health",                   -- HEALTH       = 25
	[26] = _G["ITEM_MOD_MANA_SHORT"] or "Mana",                       -- MANA         = 26

	[27] = _G["ITEM_MOD_HASTE_RANGED_RATING_SHORT"] or "Haste Range", -- HASTE_RANGED = 27
	[28] = _G["ITEM_MOD_HIT_RANGED_RATING_SHORT"] or "Hit Range",     -- HIT_RANGED   = 28
	[29] = _G["ITEM_MOD_CRIT_RANGED_RATING_SHORT"] or "Crit Range",   -- CRIT_RANGED  = 29
}

local statShortLocName = {
	[1]  = _G["ITEM_MOD_STRENGTH_SHORT"],              -- STRENGTH     = 1
	[2]  = _G["ITEM_MOD_AGILITY_SHORT"],               -- AGILITY      = 2
	[3]  = _G["ITEM_MOD_STAMINA_SHORT"],               -- STAMINA      = 3
	[4]  = _G["ITEM_MOD_INTELLECT_SHORT"],             -- INTELLECT    = 4
	[5]  = _G["ITEM_MOD_SPIRIT_SHORT"],                -- SPIRIT       = 5

	[6]  = _G["STAT_ATTACK_POWER"],                    -- ATTACKPOWER  = 6
	[7]  = _G["STAT_BLOCK"],                           -- BLOCK        = 7
	[8]  = _G["STAT_DODGE"],                           -- DODGE        = 8
	[9]  = _G["STAT_PARRY"],                           -- PARRY        = 9
	[10] = _G["STAT_HIT_CHANCE"],                      -- HIT          = 10
	[11] = _G["SPELL_CRIT_CHANCE"],                    -- CRIT         = 11
	[12] = _G["STAT_HASTE"],                           -- HASTE        = 12
	[13] = _G["STAT_EXPERTISE"],                       -- EXPERTISE    = 13
	[14] = _G["STAT_MASTERY"],                         -- MASTERY      = 14
	[15] = _G["STAT_SPELLPOWER"],                      -- SPELLPOWER   = 15

	[16] = _G["SPELL_PENETRATION"],                    -- PENETRATION  = 16
	[17] = _G["STAT_RESILIENCE"],                      -- RESILIENCE   = 17

	[18] = _G["RESISTANCE0_NAME"],                     -- ARMOR        = 18
	[19] = _G["RESISTANCE1_NAME"],                     -- HOLY_RES     = 19 - ITEM_MOD_HOLY_RESISTANCE_SHORT?  -> I have not seen this ingame
	[20] = _G["RESISTANCE2_NAME"],                     -- FIRE_RES     = 20 - ITEM_MOD_FIRE_RESISTANCE_SHORT   -> available ingame (via GetItemStats), but there is no global in GlobalStrings.lua
	[21] = _G["RESISTANCE3_NAME"],                     -- NATURE_RES   = 21 - ITEM_MOD_NATURE_RESISTANCE_SHORT -> available ingame (via GetItemStats), but there is no global in GlobalStrings.lua
	[22] = _G["RESISTANCE4_NAME"],                     -- FROST_RES    = 22 - ITEM_MOD_FROST_RESISTANCE_SHORT  -> available ingame (via GetItemStats), but there is no global in GlobalStrings.lua
	[23] = _G["RESISTANCE5_NAME"],                     -- SHADOW_RES   = 23 - ITEM_MOD_SHADOW_RESISTANCE_SHORT -> available ingame (via GetItemStats), but there is no global in GlobalStrings.lua
	[24] = _G["RESISTANCE6_NAME"],                     -- ARCANE_RES   = 24 - ITEM_MOD_ARCANE_RESISTANCE_SHORT -> available ingame (via GetItemStats), but there is no global in GlobalStrings.lua

	[25] = _G["ITEM_MOD_HEALTH_SHORT"],                -- HEALTH       = 25
	[26] = _G["ITEM_MOD_MANA_SHORT"],                  -- MANA         = 26

	[27] = _G["STAT_HASTE"].." (".._G["RANGED"]..")",        -- HASTE_RANGED = 27
	[28] = _G["STAT_HIT_CHANCE"].." (".._G["RANGED"]..")",   -- HIT_RANGED   = 28
	[29] = _G["SPELL_CRIT_CHANCE"].." (".._G["RANGED"]..")", -- CRIT_RANGED  = 29
}

local statDisplay = {
	--General = {
	[1]  =	1,  --= "ITEM_MOD_STRENGTH_SHORT",            -- STRENGTH     = 1
	[2]  =	2,  --= "ITEM_MOD_AGILITY_SHORT",             -- AGILITY      = 2
	[3]  =	3,  --= "ITEM_MOD_STAMINA_SHORT",             -- STAMINA      = 3
	[4]  =	4,  --= "ITEM_MOD_INTELLECT_SHORT",           -- INTELLECT    = 4
	[5]  =	5,  --= "ITEM_MOD_SPIRIT_SHORT",              -- SPIRIT       = 5

	[6]  =	14, --= "ITEM_MOD_MASTERY_RATING_SHORT",      -- MASTERY      = 14
	--},
	--Damage = {
	[7]  =	6,  --= "ITEM_MOD_ATTACK_POWER_SHORT",        -- ATTACKPOWER  = 6
	[8]  =	13, --= "ITEM_MOD_EXPERTISE_RATING_SHORT",    -- EXPERTISE    = 13

	[9]  =	15, --= "ITEM_MOD_SPELL_POWER_SHORT",         -- SPELLPOWER   = 15

	[10] =	12, --= "ITEM_MOD_HASTE_RATING_SHORT",        -- HASTE        = 12
	[11] =	27, --= "ITEM_MOD_HASTE_RANGED_RATING_SHORT", -- HASTE_RANGED = 27
	[12] =	11, --= "ITEM_MOD_CRIT_RATING_SHORT",         -- CRIT         = 11
	[13] =	29, --= "ITEM_MOD_CRIT_RANGED_RATING_SHORT",  -- CRIT_RANGED  = 29
	[14] =	10, --= "ITEM_MOD_HIT_RATING_SHORT",          -- HIT          = 10
	[15] =	28, --= "ITEM_MOD_HIT_RANGED_RATING_SHORT",   -- HIT_RANGED   = 28
	--},
	--Defense = {
	[16] =	18, --= "RESISTANCE0_NAME",                   -- ARMOR        = 18
	[17] =	7,  --= "ITEM_MOD_BLOCK_RATING_SHORT",        -- BLOCK        = 7
	[18] =	8,  --= "ITEM_MOD_DODGE_RATING_SHORT",        -- DODGE        = 8
	[19] =	9,  --= "ITEM_MOD_PARRY_RATING_SHORT",        -- PARRY        = 9
	--},
	--PvP = {
	[20] =	16, --= "ITEM_MOD_SPELL_PENETRATION_SHORT",   -- PENETRATION  = 16
	[21] =	17, --= "ITEM_MOD_RESILIENCE_RATING_SHORT",   -- RESILIENCE   = 17
	--},
	--Resistance = {
	[22] =	19, --= "RESISTANCE1_NAME",                   -- HOLY_RES     = 19 - ITEM_MOD_HOLY_RESISTANCE_SHORT?  -> I have not seen this ingame
	[23] =	20, --= "RESISTANCE2_NAME",                   -- FIRE_RES     = 20 - ITEM_MOD_FIRE_RESISTANCE_SHORT   -> available ingame (via GetItemStats), but there is no global in GlobalStrings.lua
	[24] =	21, --= "RESISTANCE3_NAME",                   -- NATURE_RES   = 21 - ITEM_MOD_NATURE_RESISTANCE_SHORT -> available ingame (via GetItemStats), but there is no global in GlobalStrings.lua
	[25] =	22, --= "RESISTANCE4_NAME",                   -- FROST_RES    = 22 - ITEM_MOD_FROST_RESISTANCE_SHORT  -> available ingame (via GetItemStats), but there is no global in GlobalStrings.lua
	[26] =	23, --= "RESISTANCE5_NAME",                   -- SHADOW_RES   = 23 - ITEM_MOD_SHADOW_RESISTANCE_SHORT -> available ingame (via GetItemStats), but there is no global in GlobalStrings.lua
	[27] =	24, --= "RESISTANCE6_NAME",                   -- ARCANE_RES   = 24 - ITEM_MOD_ARCANE_RESISTANCE_SHORT -> available ingame (via GetItemStats), but there is no global in GlobalStrings.lua
	--},
	--Misc = {
	[28] =	25, --= "ITEM_MOD_HEALTH_SHORT",              -- HEALTH       = 25
	[29] =	26, --= "ITEM_MOD_MANA_SHORT",                -- MANA         = 26
	--},
}

local PullDownMenu_ACI = {
	[1] = "|cff"..color.White.hex..ALL.."|r",
	[2] = "|cff"..color.Green.hex..COMPLETE.."|r",
	[3] = "|cff"..color.Red.hex..INCOMPLETE.."|r",
}

local x = 1
local PullDownMenu_PvEdump = {}
PullDownMenu_PvEdump[x] = {}
PullDownMenu_PvEdump[x].lvl = 1
PullDownMenu_PvEdump[x].name = L["Total"]
PullDownMenu_PvEdump[x].v1 = 0
x = x + 1
for i = 1, #PVEDUMP do
	if PVEDUMP[i].name then
		if not PullDownMenu_PvEdump[x] then PullDownMenu_PvEdump[x] = {} end
		PullDownMenu_PvEdump[x].lvl = 1
		PullDownMenu_PvEdump[x].name = L[ PVEDUMP[i].name ] or "ERROR"
		PullDownMenu_PvEdump[x].v1 = i
		x = x + 1
	end
	for j = 1, #PVEDUMP[i] do
		if PVEDUMP[i][j].name then
			if not PullDownMenu_PvEdump[x] then PullDownMenu_PvEdump[x] = {} end
			PullDownMenu_PvEdump[x].lvl = 2
			PullDownMenu_PvEdump[x].name = L[ PVEDUMP[i][j].name ] or "ERROR"
			PullDownMenu_PvEdump[x].v1 = i
			PullDownMenu_PvEdump[x].v2 = j
			x = x + 1
		end
		for k = 1, #PVEDUMP[i][j] do
			if PVEDUMP[i][j][k].name then
				if not PullDownMenu_PvEdump[x] then PullDownMenu_PvEdump[x] = {} end
				PullDownMenu_PvEdump[x].lvl = 3
				PullDownMenu_PvEdump[x].name = L[ PVEDUMP[i][j][k].name ] or "ERROR"
				PullDownMenu_PvEdump[x].v1 = i
				PullDownMenu_PvEdump[x].v2 = j
				PullDownMenu_PvEdump[x].v3 = k
				x = x + 1
			end
		end
	end
end
--for i = 1, #PullDownMenu_PvEdump do
--	print(i, PullDownMenu_PvEdump[i].lvl, PullDownMenu_PvEdump[i].name, PullDownMenu_PvEdump[x].v1, PullDownMenu_PvEdump[x].v2, PullDownMenu_PvEdump[x].v3)
--end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
-- -----------------------------------------------------------------------------
local function Print(...)
	print("|cff"..color.LightYellow.hex.."CharScanner:|r", ...)
end

local function ClassHexColor(class)
	local hex = string_format("%.2x%.2x%.2x", classcolors[class].r*255, classcolors[class].g*255, classcolors[class].b*255)
	return hex or "cccccc"
end

local function Desaturation(texture, desaturation)
	local shaderSupported = texture:SetDesaturated(desaturation)
	if not shaderSupported then
		if desaturation then
			texture:SetVertexColor(0.5, 0.5, 0.5)
		else
			texture:SetVertexColor(1.0, 1.0, 1.0)
		end
	end
end

local function SortedPairs(t, what)
	local ntab = {}
	for k,v in pairs(t) do
		table_insert(ntab,k)
	end
	if what == 1 then
		table_sort(ntab,
			function(a,b)
				if t[a].slevel == t[b].slevel then
					if t[a].index1 == t[b].index1 then
						if t[a].index2 == t[b].index2 then
							if t[a].index3 == t[b].index3 then
								if a < b then
									return true
						  	end
							elseif t[a].index3 < t[b].index3 then
								return true
							end
						elseif t[a].index2 < t[b].index2 then
							return true
						end
					elseif t[a].index1 < t[b].index1 then
						return true
					end
				elseif t[a].slevel < t[b].slevel then
					return true
				end
			end
		)
	--elseif what == 2 then
	end

	local i = 0
	local function _f(_s,_v)
		i = i + 1
		local k = ntab[i]
		if (k) then
			return k,t[k]
		end
	end
	return _f,nil,nil
end

local function cnum(number)
	local a = tostring(number)
	local p = string_find(a, ".", 1, true)
	if p then
		local r = string_sub(a, p+1)
		if string.len(r) == 1 then
			number = number.."0"
		end
	else
		number = number..".00"
	end
	return number -- string
end
-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
local function BuildIDTable()
	--[[ TEST code for future achievement/statistic updates
	for i=1, GetCategoryNumAchievements(-1) do
		local id, name = GetAchievementInfo(-1, i)--local id, name, _, _, _, _, _, description, _, icon, _ = GetAchievementInfo(-1, i)
		if id and name then
			if CharScanner_Options.AchievementTest == nil then CharScanner_Options.AchievementTest = {} end
			if not CharScanner_Options.AchievementTest[id] then
				CharScanner_Options.AchievementTest[id] = name
			end
		end
	end
	for i=1, GetCategoryNumAchievements(-2) do
		local id, name = GetAchievementInfo(-2, i)
		if id and name then
			if CharScanner_Options.StatisticTest == nil then CharScanner_Options.StatisticTest = {} end
			if not CharScanner_Options.StatisticTest[id] then
				CharScanner_Options.StatisticTest[id] = name
			end
		end
	end
	--]]

	-- achievement START
	-- - PvE START
	for i = 1, #PVEDUMP do
		if PVEDUMP[i].achievement then
			for key, value in pairs(PVEDUMP[i].achievement) do
				if value == 9999 then
					AchievementIDTable[4297] = select(2, GetAchievementInfo(4297))
					AchievementIDTable[4298] = select(2, GetAchievementInfo(4298))
				else
					AchievementIDTable[value] = select(2, GetAchievementInfo(value))
				end
			end
		end
		for j = 1, #PVEDUMP[i] do
			if PVEDUMP[i][j].achievement then
				for key, value in pairs(PVEDUMP[i][j].achievement) do
					if value == 9999 then
						AchievementIDTable[4297] = select(2, GetAchievementInfo(4297))
						AchievementIDTable[4298] = select(2, GetAchievementInfo(4298))
					else
						AchievementIDTable[value] = select(2, GetAchievementInfo(value))
					end
				end
			end
			for k = 1, #PVEDUMP[i][j] do
				if PVEDUMP[i][j][k].achievement then
					for key, value in pairs(PVEDUMP[i][j][k].achievement) do
						if value == 9999 then
							AchievementIDTable[4297] = select(2, GetAchievementInfo(4297))
							AchievementIDTable[4298] = select(2, GetAchievementInfo(4298))
						else
							AchievementIDTable[value] = select(2, GetAchievementInfo(value))
						end
					end
				end
			end
		end
	end
	-- - PvE END

	-- - PvP START
	for i = 1, #PVPDUMP do
		if PVPDUMP[i].achievement then
			for key, value in pairs(PVPDUMP[i].achievement) do
				if value.id then -- same id and same title
					AchievementIDTable[value.id] = select(2, GetAchievementInfo(value.id))
				elseif value.Aid then -- different ids and different title
					AchievementIDTable[value.Aid] = select(2, GetAchievementInfo(value.Aid))
					AchievementIDTable[value.Hid] = select(2, GetAchievementInfo(value.Hid))
				end
			end
		end
		for j = 1, #PVPDUMP[i] do
			if PVPDUMP[i][j].achievement then
				for key, value in pairs(PVPDUMP[i][j].achievement) do
					if value.id then -- same id and same title
						AchievementIDTable[value.id] = select(2, GetAchievementInfo(value.id))
					elseif value.Aid then -- different ids and different title
						AchievementIDTable[value.Aid] = select(2, GetAchievementInfo(value.Aid))
						AchievementIDTable[value.Hid] = select(2, GetAchievementInfo(value.Hid))
					end
				end
			end
		end
	end
	-- - PvP END
	-- achievement END

	-- statistic START
	-- PvE START
	for i = 1, #PVEDUMP do
		if PVEDUMP[i].statistic then
			for key, value in pairs(PVEDUMP[i].statistic) do
				StatisticIDTable[value] = select(2, GetAchievementInfo(value))
			end
		end
		for j = 1, #PVEDUMP[i] do
			if PVEDUMP[i][j].statistic then
				for key, value in pairs(PVEDUMP[i][j].statistic) do
					StatisticIDTable[value] = select(2, GetAchievementInfo(value))
				end
			end
			for k = 1, #PVEDUMP[i][j] do
				if PVEDUMP[i][j][k].statistic then
					for key, value in pairs(PVEDUMP[i][j][k].statistic) do
						StatisticIDTable[value] = select(2, GetAchievementInfo(value))
					end
				end
			end
		end
	end
	-- PvE END

	-- PvP START
	for i = 1, #PVPDUMP do
		if PVPDUMP[i].statistic then
			for key, value in pairs(PVPDUMP[i].statistic) do
				if value.matches then
					StatisticIDTable[value.matches] = select(2, GetAchievementInfo(value.matches))
				elseif value.wins then
					StatisticIDTable[value.wins] = select(2, GetAchievementInfo(value.wins))
				elseif value.kills then
					StatisticIDTable[value.kills] = select(2, GetAchievementInfo(value.kills))
				elseif value.misc then
					StatisticIDTable[value.misc] = select(2, GetAchievementInfo(value.misc))
				end
			end
		end
		for j = 1, #PVPDUMP[i] do
			if PVPDUMP[i][j].statistic then
				for key, value in pairs(PVPDUMP[i][j].statistic) do
					if value.matches then
						StatisticIDTable[value.matches] = select(2, GetAchievementInfo(value.matches))
					elseif value.wins then
						StatisticIDTable[value.wins] = select(2, GetAchievementInfo(value.wins))
					elseif value.kills then
						StatisticIDTable[value.kills] = select(2, GetAchievementInfo(value.kills))
					elseif value.misc then
						StatisticIDTable[value.misc] = select(2, GetAchievementInfo(value.misc))
					end
				end
			end
		end
	end
	-- PvP END
	-- statistic END

	--for key, value in pairs(AchievementIDTable) do print(key, value) end -- TEST
	--for key, value in pairs(StatisticIDTable) do print(key, value) end -- TEST
end
-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
local function CreateScanDataByGUID(guid)
	SCAN[guid] = {}
	SCAN[guid].oorScan = nil                   -- GroupScan and current group members only: out-of-range check value
	SCAN[guid].name = nil                      -- base info - name
	SCAN[guid].level = nil                     -- base info - level
	SCAN[guid].race = nil                      -- base info - race
	SCAN[guid].class = nil                     -- base info - class (localized)
	SCAN[guid].classEN = nil                   -- base info - class (english)
	SCAN[guid].faction = nil                   -- base info - faction (english)
	SCAN[guid].guildName = nil                 -- base info - guild name
	SCAN[guid].guildRankName = nil             -- base info - guild rank name
	SCAN[guid].guildRankIndex = nil            -- base info - guild rank index

	SCAN[guid].talentGroups = nil              -- talent - talent groups (count)
	SCAN[guid].activeTalentGroup = nil         -- talent - active talent group
	SCAN[guid].talents = {}                    -- all spend talent points
	SCAN[guid].mainRole = 5                    -- talent - main role | 1 = TANK, 2 = HEAL, 3 = DAMAGE, 4 = UNKNOWN (lvl>9 with all talentpoints unspent), 5 = NOTALENTS (lvl<=9)
	SCAN[guid].talentGroup1name = nil          -- talent - talent group 1 name with most points (eg: Affliction)
	SCAN[guid].talentGroup1role = nil          -- talent - talent group 1 role (class role: eg: PRIEST 2 -> HEAL)
	SCAN[guid].talentGroup1pointsString = nil  -- talent - talent group 1 points (eg: 55/11/6)
	SCAN[guid].talentGroup1pointsUnspent = nil -- talent - talent group 1 points unspent (eg: 0)
	SCAN[guid].talentGroup1icon = nil          -- talent - talent group 1 icon
	SCAN[guid].talentGroup2name = nil          -- talent - talent group 2 name with most points (eg: Destruction)
	SCAN[guid].talentGroup2role = nil          -- talent - talent group 2 role (class role: eg: PRIEST 3 -> DAMAGE)
	SCAN[guid].talentGroup2pointsString = nil  -- talent - talent group 2 points (eg: 3/10/58)
	SCAN[guid].talentGroup2pointsUnspent = nil -- talent - talent group 2 points unspent (eg: 1)
	SCAN[guid].talentGroup2icon = nil          -- talent - talent group 2 icon

	SCAN[guid].gear = {}                       -- item information from all gear slots
	SCAN[guid].gearStats = {}                  -- stats from all gear slots
	SCAN[guid].enchStats = {}                  -- sorted indexed table: enchantment stats from all gear slots
	SCAN[guid].enchTmpStats = {}               -- key/value temp table: enchantment stats from all gear slots
	SCAN[guid].refoMinStats = {}               -- reforge stats from all gear slots
	SCAN[guid].refoPluStats = {}               -- reforge stats from all gear slots
	SCAN[guid].gemStats = {}                   -- sorted indexed table: gem stats from all gear slots
	SCAN[guid].gemTmpStats = {}                -- key/value temp table: gem stats from all gear slots
	SCAN[guid].setStats = {}                   -- sorted indexed table: set stats from all gear slots
	SCAN[guid].setTmpStats = {}                -- key/value temp table: set stats from all gear slots

	SCAN[guid].itemlevelAvg = 0                -- itemlevel - average item level
	SCAN[guid].itemlevelMin = 0                -- itemlevel - lowest item level
	SCAN[guid].itemlevelMax = 0                -- itemlevel - highest item level
	SCAN[guid].itemlevelMiss = 0               -- itemlevel - number of unused Slots
	SCAN[guid].itemlevelQuality = nil          -- itemlevel - quality
	SCAN[guid].gearEnchCount = 0               -- gear - Enchantment count
	SCAN[guid].gearPvPCount = 0                -- gear - PvP item count
	SCAN[guid].gearTwoHandWeapon = nil         -- gear - TwoHandWeapon check
	SCAN[guid].gemBlue = 0                     -- gem color Blue
	SCAN[guid].gemRed = 0                      -- gem color Red
	SCAN[guid].gemYellow = 0                   -- gem color Yellow
	SCAN[guid].gemMeta = nil                   -- Meta gem requirements: OK/NOT OK
	SCAN[guid].gemTotal = 0                    -- total available gem sockets
	SCAN[guid].gemMissing = 0                  -- missing gem(s)

	SCAN[guid].PVEDATA = {}                    -- PvE achievement & statistic data
	SCAN[guid].PvEAchievementTotal = 0         -- PvE summary: achievements - total achievements
	SCAN[guid].PvEAchievementCount = 0         -- PvE summary: achievements - count
	SCAN[guid].PvEStatisticTotal = 0           -- PvE summary: statistics
	SCAN[guid].PvEStatisticCount = 0           -- PvE summary: statistics
	SCAN[guid].PvEStatisticBossKills = 0       -- PvE summary: statistics

	SCAN[guid].PVPDATA = {}                    -- PvP achievement & statistic data
	SCAN[guid].PvPAchievementTotal = 0         -- PvP summary: achievements - total achievements
	SCAN[guid].PvPAchievementCount = 0         -- PvP summary: achievements - count
	SCAN[guid].PvPStatisticTotalKills = 0      -- PvP summary:?
	SCAN[guid].PvPStatisticWins = 0            -- PvP summary:?
	SCAN[guid].PvPStatisticMatches = 0         -- PvP summary:?
	SCAN[guid].PvPStatisticKills = 0           -- PvP summary:?
	SCAN[guid].PvPBattlegroundWins = 0         -- PvP summary:?
	SCAN[guid].PvPBattlegroundMatches = 0      -- PvP summary:?
	SCAN[guid].PvPBattlegroundKills = 0        -- PvP summary:?
	SCAN[guid].PvPArenaWins = 0                -- PvP summary:?
	SCAN[guid].PvPArenaMatches = 0             -- PvP summary:?
	SCAN[guid].PvPArenaKills = 0               -- PvP summary:?

	SCAN[guid].time = nil                      -- scan time (eg.: '22:19')
	SCAN[guid].scanTime = nil                  -- time that elapsed for scan (eg.: 0.362)
end

local function CleanTempData()
	for k, v in pairs(SCAN) do -- clean
		if not v.time then
			SCAN[k] = nil
		end
	end
end

local function CleanOORData()
	local isShown
	for k, v in pairs(SCAN) do
		if v.oorScan then
			if k == GUIselectedGUID then
				isShown = true
			end
			SCAN[k] = nil
		end
	end
	if isShown then

		local showThis
		for k, v in pairs(SCAN) do
			if v.time then
				showThis = k
				break
			end
		end
		if showThis then
			CharScanner:PreShowSavedData(showThis)
		else
			table_wipe(SCAN)
			GUIselectedGUID = nil
			NumScannedChars = 0
			CharScanner:DeleteButtonToggle(false)
			CharScanner:ScanFrameToggle(nil, true)
			CharScanner:AchievementDetail_HideAll()
			CharScanner:ClearData()
		end

	end
end

-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
local function BossStatistic(id, inspect)
	-- workaround START
	-- bug       : Incorrect return value from GetComparisonStatistic()/GetStatistic().
	-- workaround: Check related achievement and return 0 or GetComparisonStatistic()/GetStatistic().
	-- NOTE      : Statistic id 4046/4045 (Anub'arak) bug was fixed with Patch 3.3.3.xxxxx, but the old 4045 statistic value which
	--             is returned from the server is still the same (old wrong cumulative 4046-one).
	--             -> This means: The statistic value returned for id 4045 is NOT 100% CORRECT (!!!) (armory too!).
	if id == 4045 then -- Times completed the Trial of the Grand Crusader (10 player)
		local x = GetAchievementComparisonInfo(3918, 1)
		if not x then
			return 0
		end
	end
	-- workaround END

	if inspect then
		return tonumber(GetComparisonStatistic(id)) or 0
	else
		return tonumber(GetStatistic(id)) or 0
	end
end
-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
local function GemSocketCheck(gemSocket1, gemSocket2, gemSocket3, gemSocket4, gem1IDsocket, gem2IDsocket, gem3IDsocket, gem4IDsocket)
	local hasBonus

	if gemSocket1 then
		if gem1IDsocket then
			if gemSocket1 == 1 then
				if gem1IDsocket == 1 or gem1IDsocket == 4 or gem1IDsocket == 6 or gem1IDsocket == 7 then
					hasBonus = true
				else
					return false
				end
			elseif gemSocket1 == 2 then
				if gem1IDsocket == 2 or gem1IDsocket == 5 or gem1IDsocket == 6 or gem1IDsocket == 7 then
					hasBonus = true
				else
					return false
				end
			elseif gemSocket1 == 3 then
				if gem1IDsocket == 3 or gem1IDsocket == 4 or gem1IDsocket == 5 or gem1IDsocket == 7 then
					hasBonus = true
				else
					return false
				end
			else
				hasBonus = true
			end
		else
			return false
		end

		if gemSocket2 then
			if gem2IDsocket then
				if gemSocket2 == 1 then
					if gem2IDsocket == 1 or gem2IDsocket == 4 or gem2IDsocket == 6 or gem2IDsocket == 7 then
						hasBonus = true
					else
						return false
					end
				elseif gemSocket2 == 2 then
					if gem2IDsocket == 2 or gem2IDsocket == 5 or gem2IDsocket == 6 or gem2IDsocket == 7 then
						hasBonus = true
					else
						return false
					end
				elseif gemSocket2 == 3 then
					if gem2IDsocket == 3 or gem2IDsocket == 4 or gem2IDsocket == 5 or gem2IDsocket == 7 then
						hasBonus = true
					else
						return false
					end
				else
					hasBonus = true
				end
			else
				return false
			end

			if gemSocket3 then
				if gem3IDsocket then
					if gemSocket3 == 1 then
						if gem3IDsocket == 1 or gem3IDsocket == 4 or gem3IDsocket == 6 or gem3IDsocket == 7 then
							hasBonus = true
						else
							return false
						end
					elseif gemSocket3 == 2 then
						if gem3IDsocket == 2 or gem3IDsocket == 5 or gem3IDsocket == 6 or gem3IDsocket == 7 then
							hasBonus = true
						else
							return false
						end
					elseif gemSocket3 == 3 then
						if gem3IDsocket == 3 or gem3IDsocket == 4 or gem3IDsocket == 5 or gem3IDsocket == 7 then
							hasBonus = true
						else
							return false
						end
					else
						hasBonus = true
					end
				else
					return false
				end

				if gemSocket4 then
					if gem4IDsocket then
						if gemSocket4 == 1 then
							if gem4IDsocket == 1 or gem4IDsocket == 4 or gem4IDsocket == 6 or gem4IDsocket == 7 then
								hasBonus = true
							else
								return false
							end
						elseif gemSocket4 == 2 then
							if gem4IDsocket == 2 or gem4IDsocket == 5 or gem4IDsocket == 6 or gem4IDsocket == 7 then
								hasBonus = true
							else
								return false
							end
						elseif gemSocket4 == 3 then
							if gem4IDsocket == 3 or gem4IDsocket == 4 or gem4IDsocket == 5 or gem4IDsocket == 7 then
								hasBonus = true
							else
								return false
							end
						else
							hasBonus = true
						end
					else
						return false
					end
				end
			end
		end
	end
	return hasBonus
end
-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
local function PullDownMenuFunc_ACI(value)
	CharScanner:AchievementDetail_Update(AchievementDetailFrameMode, nil, nil, nil, value)
end
local function PullDownMenuFunc_PvEdump(value1, value2, value3)
	CharScanner:History_GUISort("RAID", GUI_history_SortBy, value1, value2, value3)
end
-- -----------------------------------------------------------------------------


-- -----------------------------------------------------------------------------
local function ScrollFrameEnable(frame)
	frame.ScrollBarScrollUpButton:Enable()
	frame.ScrollBarScrollDownButton:Enable()
	frame.ScrollBarScrollUpButton:SetAlpha(1)
	frame.ScrollBarScrollDownButton:SetAlpha(1)
	frame.ScrollBarThumbTexture:SetAlpha(1)
end

local function ScrollFrameDisable(frame)
	frame.ScrollBarScrollUpButton:Disable()
	frame.ScrollBarScrollDownButton:Disable()
	frame.ScrollBarScrollUpButton:SetAlpha(0.2)
	frame.ScrollBarScrollDownButton:SetAlpha(0.2)
	frame.ScrollBarThumbTexture:SetAlpha(0.2)
end

local function ScrollFrameUpdate()
	-- 1 = PvE | 2 = PvP
	local statsHeight = math_floor(GVAR.MainFrame.Stats:GetHeight() + 0.5)
	for i = 1, 2 do
		-- PvE START ---------------
		local scrollFrame = GVAR.MainFrame.StatsPvE.ScrollFrame
		local contentHeight = sizeStatFrame.SF_curHeight
		-- PvE END -----------------
		-- PvP START ---------------
		if i == 2 then
			scrollFrame = GVAR.MainFrame.StatsPvP.ScrollFrame
			contentHeight = sizeStatFrame.SFpvp_curHeight
		end
		-- PvP END -----------------
	
		local scrollbarValue = scrollFrame.ScrollBar:GetValue()
	
		if contentHeight > (statsHeight - sizeGeneral.TabFrameHeight - sizeStatFrame.OffsetTopSearch - sizeGeneral.Offset) then
			ScrollFrameEnable(scrollFrame)
		else
			ScrollFrameDisable(scrollFrame)
		end
	
		if scrollbarValue == 0 then
			scrollFrame.ScrollBarScrollUpButton:Disable()
		else
			scrollFrame.ScrollBarScrollUpButton:Enable()
		end
	
		if (scrollbarValue - contentHeight) == 0 then
			scrollFrame.ScrollBarScrollDownButton:Disable()
		else
			scrollFrame.ScrollBarScrollDownButton:Enable()
		end
	end
end

local function ResizeRowUpdate()
	local height = math_floor(GVAR.MainFrame.Stats:GetHeight() + 0.5)
	local height_opt = (GUI_ABKsearch_curRows*sizeGeneral.BarHeight) + sizeGeneral.TabFrameHeight + sizeStatFrame.OffsetTopSearch + sizeGeneral.Offset -- (20*12) + 20 + 22 + 5
	if height < height_opt then
		local diff = height_opt - height
		local freq = math_floor(diff/12)
		if diff > 12 then
			local big = diff-(freq*12)
			if big > 6 then
				freq = freq + 1
			end
		else
			if diff < 6 then
				freq = freq - 1
			else
				freq = freq + 1
			end
		end
		if freq > 0 then
			GUI_ABKsearch_curRows = GUI_ABKsearch_curRows - freq
			if GUI_ABKsearch_curRows < GUI_ABKsearch_minRows then
				GUI_ABKsearch_curRows = GUI_ABKsearch_minRows
			elseif GUI_ABKsearch_curRows > GUI_ABKsearch_maxRows then
				GUI_ABKsearch_curRows = GUI_ABKsearch_maxRows
			end
			if BossSearchQuery then
				CharScanner:ABKsearch_FillList()
			end
		end
	else
		local diff = height - height_opt
		local freq = math_floor(diff/12)
		if diff > 12 then
			local big = diff-(freq*12)
			if big > 6 then
				freq = freq + 1
			end
		else
			if diff > 6 then
				freq = freq + 1
			else
				freq = freq - 1
			end
		end
		if freq > 0 then
			GUI_ABKsearch_curRows = GUI_ABKsearch_curRows + freq
			if GUI_ABKsearch_curRows < GUI_ABKsearch_minRows then
				GUI_ABKsearch_curRows = GUI_ABKsearch_minRows
			elseif GUI_ABKsearch_curRows > GUI_ABKsearch_maxRows then
				GUI_ABKsearch_curRows = GUI_ABKsearch_maxRows
			end
			if BossSearchQuery then
				CharScanner:ABKsearch_FillList()
			end
		end
	end
end

local function ResizeUpdate()
	GVAR.MainFrame.Stats:ClearAllPoints()
	GVAR.MainFrame.Stats:SetPoint("TOPLEFT", GVAR.MainFrame.Equipment, "BOTTOMLEFT", 0, sizeGeneral.Border)
	GVAR.MainFrame.Stats:SetWidth(sizeGeneral.MainWidth)

	local height = GVAR.MainFrame.Stats:GetHeight() - sizeGeneral.TabFrameHeight -- height - 20
	GVAR.MainFrame.StatsPvE:SetHeight(height)
	GVAR.MainFrame.StatsPvP:SetHeight(height)

	if BossSearchQuery then
		ResizeRowUpdate()
	end
	ScrollFrameUpdate()
	CharScanner:MainFrame_CalculateHeight()--MHIGH
end

local function ResizeEnd()
	GVAR.MainFrame.Stats:SetScript("OnUpdate", nil)
	GVAR.MainFrame.Stats:StopMovingOrSizing()
	GVAR.MainFrame.Stats:SetResizable(false)
	GVAR.MainFrame.Stats:ClearAllPoints()
	GVAR.MainFrame.Stats:SetPoint("TOPLEFT", GVAR.MainFrame.Equipment, "BOTTOMLEFT", 0, sizeGeneral.Border)
	GVAR.MainFrame.Stats:SetWidth(sizeGeneral.MainWidth)
	ResizeRowUpdate()

	local height = (GUI_ABKsearch_curRows*sizeGeneral.BarHeight) + sizeStatFrame.OffsetTopSearch + sizeGeneral.Offset -- (savedRows*12) + 16 + 22 + 5
	GVAR.MainFrame.Stats:SetHeight(sizeGeneral.TabFrameHeight + height) -- 20 + height
	GVAR.MainFrame.StatsPvE:SetHeight(height) -- height
	GVAR.MainFrame.StatsPvP:SetHeight(height) -- height

	ScrollFrameUpdate()
	CharScanner_Options.StatsCurRows = GUI_ABKsearch_curRows
	CharScanner:MainFrame_CalculateHeight()--MHIGH
end

local function ResizeStart()
	GVAR.MainFrame.Stats:SetResizable(true)
	GVAR.MainFrame.Stats:SetScript("OnUpdate", ResizeUpdate)
	GVAR.MainFrame.Stats:StartSizing()
end
-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
local function MoveStart(self)
	if self == GVAR.MainFrame.Status or self == GVAR.MainFrame.Headline.ButtonMover then
		GVAR.MainFrame.moveStrata = GVAR.MainFrame:GetFrameStrata()
		GVAR.MainFrame.moveLevel  = GVAR.MainFrame:GetFrameLevel() -- LVLFIX - this is stupid and I hate it, but this is the only way
		GVAR.MainFrame:SetFrameStrata("DIALOG")
		GVAR.MainFrame:StartMoving()
	elseif self == GVAR.OptionsFrame.Status or self == GVAR.OptionsFrame.Headline.ButtonMover then
		GVAR.OptionsFrame.moveCheck = GVAR.OptionsFrame:GetFrameStrata()
		GVAR.OptionsFrame:SetFrameStrata("DIALOG")
		GVAR.OptionsFrame:StartMoving()
	elseif self == GVAR.ScanButtonFrame.Mover then
		GVAR.ScanButtonFrame.moveCheck = GVAR.ScanButtonFrame:GetFrameStrata()
		GVAR.ScanButtonFrame:SetFrameStrata("DIALOG")
		GVAR.ScanButtonFrame:StartMoving()
	end
end

local function MoveStop(self)
	if self == GVAR.MainFrame.Status or self == GVAR.MainFrame.Headline.ButtonMover then
		GVAR.MainFrame:StopMovingOrSizing()
		GVAR.MainFrame:SetFrameStrata( GVAR.MainFrame.moveStrata )
		local newLvl = GVAR.MainFrame.moveLevel+50 -- LVLFIX - this is stupid and I hate it, but this is the only way
		if newLvl > 250 then
			newLvl = 250
		end
		GVAR.MainFrame:SetFrameLevel( newLvl )
		GVAR.MainFrame.moveStrata = nil
		GVAR.MainFrame.moveLevel  = nil
		CharScanner:Frame_SavePosition("CharScanner_MainFrame")
	elseif self == GVAR.OptionsFrame.Status or self == GVAR.OptionsFrame.Headline.ButtonMover then
		GVAR.OptionsFrame:StopMovingOrSizing()
		GVAR.OptionsFrame:SetFrameStrata(GVAR.OptionsFrame.moveCheck)
		GVAR.OptionsFrame.moveCheck = nil
		CharScanner:Frame_SavePosition("CharScanner_OptionsFrame")
	elseif self == GVAR.ScanButtonFrame.Mover then
		GVAR.ScanButtonFrame:StopMovingOrSizing()
		GVAR.ScanButtonFrame:SetFrameStrata(GVAR.ScanButtonFrame.moveCheck)
		GVAR.ScanButtonFrame.moveCheck = nil
		CharScanner:Frame_SavePosition("CharScanner_ScanButtonFrame")
	end
end
-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
local function NodeDisplay(mode, button, state)
	local temptab = GUI_PvEstats_menu
	local base = GVAR.MainFrame.StatsPvE.ScrollFrame
	if mode == "PvP" then
		temptab = GUI_PvPstats_menu
		base = GVAR.MainFrame.StatsPvP.ScrollFrame
	end

	if state then
		base[button]:Show()
	else
		base[button]:Hide()
	end
	for i = 1, #temptab do
		if temptab[i].name == button then
			temptab[i].collapse = state
			break
		end
	end

	local height = 0
	local visibleNode = {}
	local si = 1
	for i = 1, #temptab do
		if temptab[i].collapse then
			if temptab[i].slevel == 2 then
				visibleNode[si] = i
				si = si + 1
			elseif temptab[i].slevel == 3 then
				for j = 1, #temptab do
					if temptab[j].slevel == 2 and temptab[j].index1 == temptab[i].index1 and temptab[j].collapse then
						visibleNode[si] = i
						si = si + 1
					end
				end
			elseif temptab[i].slevel == 4 then

				for j = 1, #temptab do
					if temptab[j].slevel == 2 and temptab[j].index1 == temptab[i].index1 and temptab[j].collapse then--and
						for k = 1, #temptab do
							if temptab[k].slevel == 3 and temptab[k].index2 == temptab[i].index2 and temptab[k].collapse then
								visibleNode[si] = i
								si = si + 1
							end
						end
					end
				end

			end
		end
	end
	for i = 1, #visibleNode do
		height = height + temptab[ visibleNode[i] ].height
	end

	table_wipe(visibleNode)

	if mode == "PvE" then
		sizeStatFrame.SF_curHeight = height + (#PVEDUMP * sizeGeneral.BarHeight) -- height + 36
	else
		sizeStatFrame.SFpvp_curHeight = height + (#PVPDUMP * sizeGeneral.BarHeight) -- height + 48
	end
	ScrollFrameUpdate()
end
-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
local function GearTooltipHandler(self)
	if not self.comparing and IsModifiedClick("COMPAREITEMS") then
		self.shoppingTooltips = {ShoppingTooltip1, ShoppingTooltip2, ShoppingTooltip3}
		GameTooltip_ShowCompareItem()
		self.comparing = true
	elseif self.comparing and not IsModifiedClick("COMPAREITEMS") then
		for _, frame in pairs(self.shoppingTooltips) do
			frame:Hide()
		end
		self.comparing = false
	end
	if IsModifiedClick("DRESSUP") then
		ShowInspectCursor()
	else
		ResetCursor()
	end
end
-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
local MiniOptions_TimerStart
local function MiniOptionsTimer(self)
	if GetTime() >= MiniOptions_TimerStart+3 then
		GVAR.MiniOptionsFrame:Hide()
		GVAR.MiniOptionsFrame:SetScript("OnUpdate", nil)
	end
end
-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
local function SlashHandler(msg)
	msg = strtrim(string_lower(msg or ""))
	if msg == "options" then
		CharScanner:Frame_Toggle(GVAR.OptionsFrame)
	else
		CharScanner:Frame_Toggle(GVAR.MainFrame, 1)
		if not SingleScanIsActive and not GroupScanIsActive then
			CharScanner:ScanUnit("target")
		end
	end
end
-- -----------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
-- Template BorderTRBL START ----------------------------------------
TEMPLATE.BorderTRBL = function(frame) -- TRBL = Top-Right-Bottom-Left
	frame.FrameBorder = frame:CreateTexture(nil, "BORDER")
	frame.FrameBorder:SetPoint("TOPLEFT", sizeGeneral.Border, -sizeGeneral.Border)
	frame.FrameBorder:SetPoint("BOTTOMRIGHT", -sizeGeneral.Border, sizeGeneral.Border)
	frame.FrameBorder:SetTexture(color.FrameBG.rgba[1], color.FrameBG.rgba[2], color.FrameBG.rgba[3], color.FrameBG.rgba[4])
	frame.FrameBackground = frame:CreateTexture(nil, "BACKGROUND")
	frame.FrameBackground:SetPoint("TOPLEFT", 0, 0)
	frame.FrameBackground:SetPoint("BOTTOMRIGHT", 0, 0)
	frame.FrameBackground:SetTexture(color.FrameBorder.rgba[1], color.FrameBorder.rgba[2], color.FrameBorder.rgba[3], color.FrameBorder.rgba[4])
end
-- Template BorderTRBL END ----------------------------------------

-- Template BorderTcRBL START ----------------------------------------
TEMPLATE.BorderTcRBL = function(frame) -- TcRBL = Topcolor-Right-Bottom-Left
	frame.Border = frame:CreateTexture(nil, "BORDER")
	frame.Border:SetPoint("TOPLEFT", sizeGeneral.Border, 0)
	frame.Border:SetPoint("BOTTOMLEFT", 0, sizeGeneral.Border)
	frame.Border:SetPoint("TOPRIGHT", -sizeGeneral.Border, 0)
	frame.Border:SetPoint("BOTTOMRIGHT", -sizeGeneral.Border, -sizeGeneral.Border)
	frame.Border:SetTexture(color.FrameBG.rgba[1], color.FrameBG.rgba[2], color.FrameBG.rgba[3], color.FrameBG.rgba[4])
	frame.BorderTop = frame:CreateTexture(nil, "BORDER")
	frame.BorderTop:SetPoint("TOPLEFT", sizeGeneral.Border, 0)
	frame.BorderTop:SetPoint("TOPRIGHT", -sizeGeneral.Border, -sizeGeneral.Border)
	frame.BorderTop:SetHeight(sizeGeneral.Border) -- 1
	frame.BorderTop:SetTexture(color.DarkGrey.rgba[1], color.DarkGrey.rgba[2], color.DarkGrey.rgba[3], color.DarkGrey.rgba[4])
	frame.Background = frame:CreateTexture(nil, "BACKGROUND")
	frame.Background:SetPoint("TOPLEFT", 0, 0)
	frame.Background:SetPoint("BOTTOMRIGHT", 0, 0)
	frame.Background:SetTexture(color.FrameBorder.rgba[1], color.FrameBorder.rgba[2], color.FrameBorder.rgba[3], color.FrameBorder.rgba[4])
end
-- Template BorderTcRBL END ----------------------------------------

-- Template ScrollFrame START ----------------------------------------
TEMPLATE.ButtonTextureScrollFrame = function(button, pos)
	local textureNormal = button:CreateTexture(nil, "ARTWORK")
	textureNormal:SetWidth(sizeGeneral.SliderSize)
	textureNormal:SetHeight(sizeGeneral.SliderSize)
	textureNormal:SetPoint("LEFT", 0, 0)
	textureNormal:SetTexture(Textures.CharScannerIcons.path)
	textureNormal:SetTexCoord(unpack(Textures[pos].coords))
	button:SetNormalTexture(textureNormal)

	local texturePush = button:CreateTexture(nil, "ARTWORK")
	texturePush:SetWidth(sizeGeneral.SliderSize)
	texturePush:SetHeight(sizeGeneral.SliderSize)
	texturePush:SetPoint("TOPLEFT", 1, -1)
	texturePush:SetPoint("BOTTOMRIGHT", -1, 1)
	texturePush:SetTexture(Textures.CharScannerIcons.path)
	texturePush:SetTexCoord(unpack(Textures[pos].coords))
	button:SetPushedTexture(texturePush)

	local textureHighlight = button:CreateTexture(nil, "ARTWORK")
	textureHighlight:SetWidth(sizeGeneral.SliderSize)
	textureHighlight:SetHeight(sizeGeneral.SliderSize)
	textureHighlight:SetPoint("LEFT", 0, 0)
	textureHighlight:SetTexture(Textures.CharScannerIcons.path)
	textureHighlight:SetTexCoord(unpack(Textures[pos].coordsHL))
	button:SetHighlightTexture(textureHighlight)

	local textureDisable = button:CreateTexture(nil, "ARTWORK")
	textureDisable:SetWidth(sizeGeneral.SliderSize)
	textureDisable:SetHeight(sizeGeneral.SliderSize)
	textureDisable:SetPoint("LEFT", 0, 0)
	textureDisable:SetTexture(Textures.CharScannerIcons.path)
	textureDisable:SetTexCoord(unpack(Textures[pos].coordsDIS))
	button:SetDisabledTexture(textureDisable)
end

TEMPLATE.ScrollFrame = function(frame, child)

	frame.offset = 0
	frame:EnableMouseWheel(true)

	frame.ScrollBar = CreateFrame("Slider", nil, frame)
	frame.ScrollBar:SetWidth(sizeGeneral.SliderSize)
	frame.ScrollBar:SetHeight(0)
	frame.ScrollBar:SetPoint("TOPLEFT", frame, "TOPRIGHT", sizeGeneral.Space, -sizeGeneral.SliderSize)
	frame.ScrollBar:SetPoint("BOTTOMLEFT", frame, "BOTTOMRIGHT", sizeGeneral.Space, sizeGeneral.SliderSize)
	frame.ScrollBar:SetMinMaxValues(0, 0)
	frame.ScrollBar:SetValue(0)
	frame.ScrollBar:SetOrientation("VERTICAL")
	frame.ScrollBar:EnableMouseWheel(true)
	frame.ScrollBar:SetScript("OnValueChanged", function(self, value)
		frame:SetVerticalScroll(value)
	end)
	frame.ScrollBar:SetScript("OnMouseWheel", function(self, delta)
		if delta > 0 then
			frame.ScrollBar:SetValue(frame.ScrollBar:GetValue() - (frame.ScrollBar:GetHeight() / 2))
		else
			frame.ScrollBar:SetValue(frame.ScrollBar:GetValue() + (frame.ScrollBar:GetHeight() / 2))
		end
	end)

	frame.ScrollBG = frame.ScrollBar:CreateTexture(nil, "BACKGROUND")
	frame.ScrollBG:SetAllPoints(frame.ScrollBar)
	frame.ScrollBG:SetTexture(0, 0, 0, 0)

	frame.ScrollBarScrollUpButton = CreateFrame("Button", nil, frame.ScrollBar)
	frame.ScrollBarScrollUpButton:SetWidth(sizeGeneral.SliderSize)
	frame.ScrollBarScrollUpButton:SetHeight(sizeGeneral.SliderSize)
	frame.ScrollBarScrollUpButton:SetPoint("BOTTOM", frame.ScrollBar, "TOP", 0, 0)
	frame.ScrollBarScrollUpButton:SetScript("OnClick", function()
		frame.ScrollBar:SetValue(frame.ScrollBar:GetValue() - (frame.ScrollBar:GetHeight() / 2))
		PlaySound("UChatScrollButton")
	end)
	TEMPLATE.ButtonTextureScrollFrame(frame.ScrollBarScrollUpButton, "ScrollFrameUp")

	frame.ScrollBarScrollDownButton = CreateFrame("Button", nil, frame.ScrollBar)
	frame.ScrollBarScrollDownButton:SetWidth(sizeGeneral.SliderSize)
	frame.ScrollBarScrollDownButton:SetHeight(sizeGeneral.SliderSize)
	frame.ScrollBarScrollDownButton:SetPoint("TOP", frame.ScrollBar, "BOTTOM", 0, 0)
	frame.ScrollBarScrollDownButton:SetScript("OnClick", function()
		frame.ScrollBar:SetValue(frame.ScrollBar:GetValue() + (frame.ScrollBar:GetHeight() / 2))
		PlaySound("UChatScrollButton")
	end)
	TEMPLATE.ButtonTextureScrollFrame(frame.ScrollBarScrollDownButton, "ScrollFrameDown")

	frame.ScrollBarThumbTexture = frame.ScrollBar:CreateTexture(nil, "BORDER")
	frame.ScrollBarThumbTexture:SetWidth(sizeGeneral.SliderSize)
	frame.ScrollBarThumbTexture:SetHeight(sizeGeneral.SliderSize)
	frame.ScrollBarThumbTexture:SetTexture(Textures.CharScannerIcons.path)
	frame.ScrollBarThumbTexture:SetTexCoord(unpack(Textures.ScrollFrameKnob.coords))
	frame.ScrollBar:SetThumbTexture(frame.ScrollBarThumbTexture)

	frame:SetScript("OnScrollRangeChanged", function(self, xrange, yrange)
		-- ScrollFrame_OnScrollRangeChanged(self, xrange, yrange)

		if not yrange then
			yrange = self:GetVerticalScrollRange()
		end
		local value = frame.ScrollBar:GetValue()
		if value > yrange then
			value = yrange
		end
		frame.ScrollBar:SetMinMaxValues(0, yrange)
		frame.ScrollBar:SetValue(value)
		if math_floor(yrange) == 0 then
			frame.ScrollBarScrollDownButton:Disable()
			frame.ScrollBarScrollUpButton:Disable()
			frame.ScrollBarScrollDownButton:Show()
			frame.ScrollBarScrollUpButton:Show()
			--_G[scrollbar:GetName().."ThumbTexture"]:Hide()
		else
			frame.ScrollBarScrollDownButton:Show()
			frame.ScrollBarScrollUpButton:Show()
			frame.ScrollBar:Show()
			--_G[scrollbar:GetName().."ThumbTexture"]:Show()
			-- The 0.005 is to account for precision errors
			if yrange - value > 0.005 then
				frame.ScrollBarScrollDownButton:Enable()
			else
				frame.ScrollBarScrollDownButton:Disable()
			end
		end
	end)
	frame:SetScript("OnMouseWheel", function(self, delta)
		if delta > 0 then
			frame.ScrollBar:SetValue(frame.ScrollBar:GetValue() - (frame.ScrollBar:GetHeight() / 2))
		else
			frame.ScrollBar:SetValue(frame.ScrollBar:GetValue() + (frame.ScrollBar:GetHeight() / 2))
		end
	end)
	frame:SetScript("OnVerticalScroll", function(self, off)
		frame.ScrollBar:SetValue(off)
	end)

	if child then
		frame.ScrollChildFrame = CreateFrame("Frame", nil, frame)
		frame.ScrollChildFrame:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, 0)
		frame.ScrollChildFrame:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", 0, 0)
		--frame.ScrollChildFrame:SetHeight(0)
		--frame.ScrollChildFrame:SetWidth(0)
		frame:SetScrollChild(frame.ScrollChildFrame)
	end
end
-- Template ScrollFrame END ----------------------------------------

-- Template PullDownMenu START ----------------------------------------
TEMPLATE.DisablePullDownMenu = function(button)
	button.Border:SetTexture(0.4, 0.4, 0.4, 1)
	button:Disable()
end

TEMPLATE.EnablePullDownMenu = function(button)
	button.Border:SetTexture(0.8, 0.2, 0.2, 1)
	button:Enable()
end

TEMPLATE.PullDownMenu = function(button, contentName, buttonText, pulldownWidth, contentNum, func)
	local maxNum = contentNum
	if maxNum > 25 then
		maxNum = 25
	end
	button.PullDownButtonBG = button:CreateTexture(nil, "BORDER")
	button.PullDownButtonBG:SetPoint("TOPLEFT", 1, -1)
	button.PullDownButtonBG:SetPoint("BOTTOMRIGHT", -1, 1)
	button.PullDownButtonBG:SetTexture(0, 0, 0, 1)

	button.PullDownButtonBorder = button:CreateTexture(nil, "BACKGROUND")
	button.PullDownButtonBorder:SetPoint("TOPLEFT", 0, 0)
	button.PullDownButtonBorder:SetPoint("BOTTOMRIGHT", 0, 0)
	button.PullDownButtonBorder:SetTexture(0.4, 0.4, 0.4, 1)

	button.PullDownButtonNormal = button:CreateTexture(nil, "ARTWORK")
	button.PullDownButtonNormal:SetPoint("TOPLEFT", 2, -2)
	button.PullDownButtonNormal:SetPoint("BOTTOMRIGHT", -2, 2)
	button.PullDownButtonNormal:SetTexture(0, 0, 0, 1)

	button.PullDownButtonExpand = button:CreateTexture(nil, "ARTWORK")
	button.PullDownButtonExpand:SetHeight(sizeGeneral.SliderSize)
	button.PullDownButtonExpand:SetWidth(sizeGeneral.SliderSize)
	button.PullDownButtonExpand:SetPoint("RIGHT", button, "RIGHT", -(sizeGeneral.Space+sizeGeneral.Border), 0)
	button.PullDownButtonExpand:SetTexture(Textures.CharScannerIcons.path)
	button.PullDownButtonExpand:SetTexCoord(unpack(Textures.Expand.coords))
	button:SetNormalTexture(button.PullDownButtonExpand)

	button.PullDownButtonDisabled = button:CreateTexture(nil, "OVERLAY")
	button.PullDownButtonDisabled:SetPoint("TOPLEFT", 3, -3)
	button.PullDownButtonDisabled:SetPoint("BOTTOMRIGHT", -3, 3)
	button.PullDownButtonDisabled:SetTexture(0.6, 0.6, 0.6, 0.2)
	button:SetDisabledTexture(button.PullDownButtonDisabled)

	button.PullDownButtonHighlight = button:CreateTexture(nil, "OVERLAY")
	button.PullDownButtonHighlight:SetPoint("TOPLEFT", sizeGeneral.Border, -sizeGeneral.Border)
	button.PullDownButtonHighlight:SetPoint("BOTTOMRIGHT", -sizeGeneral.Border, sizeGeneral.Border)
	button.PullDownButtonHighlight:SetTexture(0.6, 0.6, 0.6, 0.2)
	button:SetHighlightTexture(button.PullDownButtonHighlight)

	button.PullDownButtonText = button:CreateFontString(nil, "ARTWORK", fonts.GameFontWhiteSmall.name)
	button.PullDownButtonText:SetWidth(pulldownWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border)
	button.PullDownButtonText:SetHeight(sizeGeneral.BarHeight)
	button.PullDownButtonText:SetPoint("LEFT", sizeGeneral.Offset+sizeGeneral.Space, 0)
	button.PullDownButtonText:SetJustifyH("LEFT")
	button.PullDownButtonText:SetText(buttonText)

	button.PullDownMenu = CreateFrame("Frame", nil, button)
	TEMPLATE.BorderTRBL(button.PullDownMenu)
	button.PullDownMenu:EnableMouse(true)
	button.PullDownMenu:SetToplevel(true)
	button.PullDownMenu:SetWidth(pulldownWidth)
	button.PullDownMenu:SetHeight(sizeGeneral.Offset+(maxNum*sizeGeneral.BarHeight)+sizeGeneral.Offset)
	button.PullDownMenu:SetPoint("TOPLEFT", button, "BOTTOMLEFT", 0, 1)
	button.PullDownMenu:Hide()
	button.PullDownMenu.content = contentName

	button.PullDownMenu.ScrollFrame = CreateFrame("ScrollFrame", nil, button.PullDownMenu)
	TEMPLATE.ScrollFrame(button.PullDownMenu.ScrollFrame)
	button.PullDownMenu.ScrollFrame:SetPoint("TOPLEFT", sizeGeneral.Offset, -sizeGeneral.Offset)
	button.PullDownMenu.ScrollFrame:SetPoint("BOTTOMRIGHT", -(sizeGeneral.Space+sizeGeneral.SliderSize+sizeGeneral.Space+sizeGeneral.Border), sizeGeneral.Offset)
	button.PullDownMenu.ScrollFrame:SetScript("OnVerticalScroll", function(self, value) CharScanner:PullDdownMenu_OnVerticalScroll(button.PullDownMenu, value) end)

	local function OnLeave()
		if not button:IsMouseOver() and not button.PullDownMenu:IsMouseOver() then
			button.PullDownMenu:Hide()
			button.PullDownButtonExpand:SetTexCoord(unpack(Textures.Expand.coords))
		end
	end

	for i = 1, maxNum do
		if not button.PullDownMenu.ScrollFrame.Button then button.PullDownMenu.ScrollFrame.Button = {} end
		button.PullDownMenu.ScrollFrame.Button[i] = CreateFrame("Button", nil, button.PullDownMenu)
		button.PullDownMenu.ScrollFrame.Button[i]:SetWidth(pulldownWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border)
		button.PullDownMenu.ScrollFrame.Button[i]:SetHeight(sizeGeneral.BarHeight)
		button.PullDownMenu.ScrollFrame.Button[i]:SetFrameLevel( button.PullDownMenu.ScrollFrame:GetFrameLevel() + 5 )
		button.PullDownMenu.ScrollFrame.Button[i]:Hide()
		if i == 1 then
			button.PullDownMenu.ScrollFrame.Button[i]:SetPoint("TOPLEFT", button.PullDownMenu.ScrollFrame, "TOPLEFT", 0,0)--sizeGeneral.Offset, -sizeGeneral.Offset)
		else
			button.PullDownMenu.ScrollFrame.Button[i]:SetPoint("TOPLEFT", button.PullDownMenu.ScrollFrame.Button[(i-1)], "BOTTOMLEFT", 0, 0)
		end

		button.PullDownMenu.ScrollFrame.Button[i].Text = button.PullDownMenu.ScrollFrame.Button[i]:CreateFontString(nil, "ARTWORK", fonts.GameFontWhiteSmall.name)
		button.PullDownMenu.ScrollFrame.Button[i].Text:SetWidth(pulldownWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border)
		button.PullDownMenu.ScrollFrame.Button[i].Text:SetHeight(sizeGeneral.BarHeight)
		button.PullDownMenu.ScrollFrame.Button[i].Text:SetPoint("LEFT", sizeGeneral.Space, 0)
		button.PullDownMenu.ScrollFrame.Button[i].Text:SetJustifyH("LEFT")

		button.PullDownMenu.ScrollFrame.Button[i]:SetScript("OnLeave", OnLeave)
		button.PullDownMenu.ScrollFrame.Button[i]:SetScript("OnClick", function()
			button.value1 = button.PullDownMenu.ScrollFrame.Button[i].value1
			button.value2 = button.PullDownMenu.ScrollFrame.Button[i].value2
			button.value3 = button.PullDownMenu.ScrollFrame.Button[i].value3
			button.PullDownButtonText:SetText( button.PullDownMenu.ScrollFrame.Button[i].Text:GetText() )
			button.PullDownMenu:Hide()
			button.PullDownButtonExpand:SetTexCoord(unpack(Textures.Expand.coords))
			if func then
				func(button.value1, button.value2, button.value3)
			end
		end)

		button.PullDownMenu.ScrollFrame.Button[i].Highlight = button.PullDownMenu.ScrollFrame.Button[i]:CreateTexture(nil, "ARTWORK")
		button.PullDownMenu.ScrollFrame.Button[i].Highlight:SetPoint("TOPLEFT", 0, 0)
		button.PullDownMenu.ScrollFrame.Button[i].Highlight:SetPoint("BOTTOMRIGHT", 0, 0)
		button.PullDownMenu.ScrollFrame.Button[i].Highlight:SetTexture(1, 1, 1, 0.2)
		button.PullDownMenu.ScrollFrame.Button[i]:SetHighlightTexture(button.PullDownMenu.ScrollFrame.Button[i].Highlight)
	end
	CharScanner:PullDdownMenu_FillList(button.PullDownMenu)

	button.PullDownMenu:SetScript("OnLeave", OnLeave)
	button.PullDownMenu:SetScript("OnHide", function(self) self:Hide() end) -- for esc close
	button.PullDownMenu.ScrollFrame:SetScript("OnLeave", OnLeave)
	button.PullDownMenu.ScrollFrame.ScrollBar:SetScript("OnLeave", OnLeave)
	button.PullDownMenu.ScrollFrame.ScrollBarScrollUpButton:SetScript("OnLeave", OnLeave)
	button.PullDownMenu.ScrollFrame.ScrollBarScrollDownButton:SetScript("OnLeave", OnLeave)

	button:SetScript("OnLeave", OnLeave)
	button:SetScript("OnClick", function()
		if button.PullDownMenu:IsShown() then
			button.PullDownMenu:Hide()
			button.PullDownButtonExpand:SetTexCoord(unpack(Textures.Expand.coords))
		else
			button.PullDownMenu:Show()
			button.PullDownButtonExpand:SetTexCoord(unpack(Textures.Collapse.coords))
		end
	end)
end
-- Template PullDownMenu END ----------------------------------------

-- Template TextButton START ----------------------------------------
TEMPLATE.DisableTextButton = function(button)
	button.Border:SetTexture(0.4, 0.4, 0.4, 1)
	button:Disable()
end

TEMPLATE.EnableTextButton = function(button, action)
	local buttoncolor
	if action == 1 then
		bordercolor = {0.73, 0.26, 0.21, 1}
	elseif action == 2 then
		bordercolor = {0.43, 0.32, 0.68, 1}
	elseif action == 3 then
		bordercolor = {0.24, 0.46, 0.21, 1}
	else
		bordercolor = {1, 1, 1, 1}
	end
	button.Border:SetTexture(bordercolor[1], bordercolor[2], bordercolor[3], bordercolor[4])
	button:Enable()
end

TEMPLATE.TextButton = function(button, text, action)
	local buttoncolor
	local bordercolor
	if action == 1 then
		button:SetNormalFontObject("GameFontNormal")
		button:SetDisabledFontObject("GameFontDisable") 
		buttoncolor = {0.38, 0, 0, 1}
		bordercolor = {0.73, 0.26, 0.21, 1}
	elseif action == 2 then
		button:SetNormalFontObject("GameFontWhiteSmall")
		button:SetDisabledFontObject("GameFontDisableSmall")
		buttoncolor = {0, 0, 0.5, 1}
		bordercolor = {0.43, 0.32, 0.68, 1}
	elseif action == 3 then
		button:SetNormalFontObject("GameFontWhiteSmall")
		button:SetDisabledFontObject("GameFontDisableSmall")
		buttoncolor = {0, 0.2, 0, 1}
		bordercolor = {0.24, 0.46, 0.21, 1}
	else
		button:SetNormalFontObject("GameFontNormal")
		button:SetDisabledFontObject("GameFontDisable")
		buttoncolor = {0, 0, 0, 1}
		bordercolor = {1, 1, 1, 1}
	end

	button.Background = button:CreateTexture(nil, "BORDER")
	button.Background:SetPoint("TOPLEFT", 1, -1)
	button.Background:SetPoint("BOTTOMRIGHT", -1, 1)
	button.Background:SetTexture(0, 0, 0, 1)

	button.Border = button:CreateTexture(nil, "BACKGROUND")
	button.Border:SetPoint("TOPLEFT", 0, 0)
	button.Border:SetPoint("BOTTOMRIGHT", 0, 0)
	button.Border:SetTexture(bordercolor[1], bordercolor[2], bordercolor[3], bordercolor[4])

	button.Normal = button:CreateTexture(nil, "ARTWORK")
	button.Normal:SetPoint("TOPLEFT", 2, -2)
	button.Normal:SetPoint("BOTTOMRIGHT", -2, 2)
	button.Normal:SetTexture(buttoncolor[1], buttoncolor[2], buttoncolor[3], buttoncolor[4])
	button:SetNormalTexture(button.Normal)

	button.Disabled = button:CreateTexture(nil, "OVERLAY")
	button.Disabled:SetPoint("TOPLEFT", 3, -3)
	button.Disabled:SetPoint("BOTTOMRIGHT", -3, 3)
	button.Disabled:SetTexture(0.6, 0.6, 0.6, 0.2)
	button:SetDisabledTexture(button.Disabled)

	button.Highlight = button:CreateTexture(nil, "OVERLAY")
	button.Highlight:SetPoint("TOPLEFT", 3, -3)
	button.Highlight:SetPoint("BOTTOMRIGHT", -3, 3)
	button.Highlight:SetTexture(0.6, 0.6, 0.6, 0.2)
	button:SetHighlightTexture(button.Highlight)

	button:SetPushedTextOffset(1, -1)	
	button:SetText(text)
end
-- Template TextButton END ----------------------------------------

-- Template IconButton START ----------------------------------------
TEMPLATE.DisableIconButton = function(button, bg, which)
	button:Disable()
	if which then
		button.BorderColor:SetTexture(color.ButtonBorderDisable.rgba[1], color.ButtonBorderDisable.rgba[2], color.ButtonBorderDisable.rgba[3], color.ButtonBorderDisable.rgba[4])
	else
		if bg then
			button.Background:SetTexture(color.ButtonBG.rgba[1], color.ButtonBG.rgba[2], color.ButtonBG.rgba[3], color.ButtonBG.rgba[4])
		end
		button.BorderColor:SetTexture(color.ButtonBorderDisable.rgba[1], color.ButtonBorderDisable.rgba[2], color.ButtonBorderDisable.rgba[3], color.ButtonBorderDisable.rgba[4])
	end
end

TEMPLATE.EnableIconButton = function(button, bg, which)
	button:Enable()
	if which then
		button.BorderColor:SetTexture(0.55, 0, 0, 1)
	else
		if bg then
			button.Background:SetTexture(color.ButtonBGActive.rgba[1], color.ButtonBGActive.rgba[2], color.ButtonBGActive.rgba[3], color.ButtonBGActive.rgba[4])
		end
		button.BorderColor:SetTexture(color.ButtonBorder.rgba[1], color.ButtonBorder.rgba[2], color.ButtonBorder.rgba[3], color.ButtonBorder.rgba[4])
	end
end

TEMPLATE.IconButton = function(button, which, height)
	local cut
	if which == 1 then
		cut = "GroupScanAll"
	elseif which == 2 then
		cut = "History"
	elseif which == 3 then
		cut = "Compare"
	elseif which == 4 then
		cut = "Delete"
	elseif which == 5 then
		cut = "GroupScanMissing"
	elseif which == 6 then
		cut = "Expand"
	elseif which == 7 then
		cut = "SingleScan"
	elseif which == 8 then
		cut = "Options"
	elseif which == 9 then
		cut = "Close"
	elseif which == 10 then
		cut = "RightSide"
	elseif which == 11 then
		cut = "FreeIcon"
	elseif which == 12 then
		cut = "DockedIcon"
	end

	button:SetWidth(sizeGeneral.Border + sizeGeneral.Space + Textures[cut].width + sizeGeneral.Space + sizeGeneral.Border)
	button:SetHeight(sizeGeneral.MainButtonHeight)

	if height then
		button:SetWidth(sizeGeneral.Border + sizeGeneral.Space + (Textures[cut].width/(Textures[cut].height/height)) + sizeGeneral.Space + sizeGeneral.Border)
		button:SetHeight(sizeGeneral.Border + sizeGeneral.Space + height + sizeGeneral.Space + sizeGeneral.Border)
	end

	button.Background = button:CreateTexture(nil, "BORDER")
	button.Background:SetPoint("TOPLEFT", 1, -1)
	button.Background:SetPoint("BOTTOMRIGHT", -1, 1)
	button.Background:SetTexture(color.ButtonBG.rgba[1], color.ButtonBG.rgba[2], color.ButtonBG.rgba[3], color.ButtonBG.rgba[4])
	button.BorderColor = button:CreateTexture(nil, "BACKGROUND")
	button.BorderColor:SetPoint("TOPLEFT", 0, 0)
	button.BorderColor:SetPoint("BOTTOMRIGHT", 0, 0)
	button.BorderColor:SetTexture(color.ButtonBorder.rgba[1], color.ButtonBorder.rgba[2], color.ButtonBorder.rgba[3], color.ButtonBorder.rgba[4])

	button.Normal = button:CreateTexture(nil, "ARTWORK")
	button.Normal:SetPoint("TOPLEFT", 3, -3)
	button.Normal:SetPoint("BOTTOMRIGHT", -3, 3)
	button.Normal:SetTexture(Textures.CharScannerIcons.path)
	button.Normal:SetTexCoord(unpack(Textures[cut].coords))
	button:SetNormalTexture(button.Normal)

	button.Push = button:CreateTexture(nil, "ARTWORK")
	button.Push:SetPoint("TOPLEFT", 4, -4)
	button.Push:SetPoint("BOTTOMRIGHT", -4, 4)
	button.Push:SetTexture(Textures.CharScannerIcons.path)
	button.Push:SetTexCoord(unpack(Textures[cut].coords))
	button:SetPushedTexture(button.Push)

	button.Disabled = button:CreateTexture(nil, "ARTWORK")
	button.Disabled:SetPoint("TOPLEFT", 3, -3)
	button.Disabled:SetPoint("BOTTOMRIGHT", -3, 3)
	button.Disabled:SetTexture(Textures.CharScannerIcons.path)
	button.Disabled:SetTexCoord(unpack(Textures[cut].coords))
	button:SetDisabledTexture(button.Disabled)

	Desaturation(button.Disabled, true)

	button.Highlight = button:CreateTexture(nil, "ARTWORK")
	button.Highlight:SetPoint("TOPLEFT", 3, -3)
	button.Highlight:SetPoint("BOTTOMRIGHT", -3, 3)
	button.Highlight:SetTexture(Textures.CharScannerIcons.path)
	button.Highlight:SetTexCoord(unpack(Textures.ButtonHL.coords))
	button:SetHighlightTexture(button.Highlight)
end
-- Template IconButton END ----------------------------------------

-- Template TabButton START ----------------------------------------
TEMPLATE.SetTabButton = function(button, enabled)
	if enabled then
		button.TabBottom:SetTexture(0, 0, 0, 1)
		button.TabBorder:SetTexture(color.ButtonBorder.rgba[1], color.ButtonBorder.rgba[2], color.ButtonBorder.rgba[3], color.ButtonBorder.rgba[4])
		button.TabText:SetTextColor(1, 1, 1, 1)
	else
		button.TabBottom:SetTexture(color.ButtonBorder.rgba[1], color.ButtonBorder.rgba[2], color.ButtonBorder.rgba[3], color.ButtonBorder.rgba[4])
		button.TabBorder:SetTexture(0.4, 0.4, 0.4, 0.4)
		button.TabText:SetTextColor(1, 1, 1, 0.5)
	end
end

TEMPLATE.DisableTabButton = function(button, withColor)
	if withColor then
		button.TabBottom:SetTexture(color.ButtonBorder.rgba[1], color.ButtonBorder.rgba[2], color.ButtonBorder.rgba[3], color.ButtonBorder.rgba[4])
		button.TabBorder:SetTexture(0, 0, 0, 0)
		button.TabText:SetTextColor(0, 0, 0, 0)
	end
	button:Disable()
end

TEMPLATE.EnableTabButton = function(button)
	button.TabBorder:SetTexture(0.4, 0.4, 0.4, 1)
	button.TabText:SetTextColor(1, 1, 1, 1)
	button:Enable()
end

TEMPLATE.TabButton = function(button, text, buttonwidth)
	if type(button) ~= "table" then button = {} end

	button.TabBackground = button:CreateTexture(nil, "BORDER")
	button.TabBackground:SetPoint("TOPLEFT", sizeGeneral.Border, -sizeGeneral.Border)
	button.TabBackground:SetPoint("BOTTOMRIGHT", -sizeGeneral.Border, sizeGeneral.Border)
	button.TabBackground:SetTexture(0, 0, 0, 1)

	button.TabBorder = button:CreateTexture(nil, "BACKGROUND")
	button.TabBorder:SetPoint("TOPLEFT", 0, 0)
	button.TabBorder:SetPoint("BOTTOMRIGHT", -sizeGeneral.Border, sizeGeneral.Border)
	button.TabBorder:SetPoint("TOPRIGHT", 0, 0)
	button.TabBorder:SetPoint("BOTTOMLEFT" ,sizeGeneral.Border, sizeGeneral.Border)
	button.TabBorder:SetTexture(color.ButtonBorder.rgba[1], color.ButtonBorder.rgba[2], color.ButtonBorder.rgba[3], color.ButtonBorder.rgba[4])

	button.TabBottom = button:CreateTexture(nil, "ARTWORK")
	button.TabBottom:SetPoint("TOPLEFT", button, "BOTTOMLEFT" ,sizeGeneral.Border, 2*sizeGeneral.Border)
	button.TabBottom:SetPoint("BOTTOMLEFT", sizeGeneral.Border, sizeGeneral.Border)
	button.TabBottom:SetPoint("TOPRIGHT", button, "BOTTOMRIGHT" ,-sizeGeneral.Border, 2*sizeGeneral.Border)
	button.TabBottom:SetPoint("BOTTOMRIGHT" ,-sizeGeneral.Border, sizeGeneral.Border)
	button.TabBottom:SetTexture(color.ButtonBorder.rgba[1], color.ButtonBorder.rgba[2], color.ButtonBorder.rgba[3], color.ButtonBorder.rgba[4])

	button.TabHighlight = button:CreateTexture(nil, "ARTWORK")
	button.TabHighlight:SetPoint("TOPLEFT", 3, -3)
	button.TabHighlight:SetPoint("BOTTOMRIGHT", -3, 3)
	button.TabHighlight:SetTexture(color.ButtonHighlight.rgba[1], color.ButtonHighlight.rgba[2], color.ButtonHighlight.rgba[3], color.ButtonHighlight.rgba[4])
	button:SetHighlightTexture(button.TabHighlight)

	button.TabText = button:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
	button.TabText:SetText(text)
	button.TabText:SetWidth(buttonwidth)
	button.TabText:SetHeight(12)
	button.TabText:SetPoint("CENTER", button, "CENTER", 0, 0)
	button.TabText:SetJustifyH("CENTER")
	button.TabText:SetTextColor(1, 1, 1, 1)
end
-- Template TabButton END ----------------------------------------

-- Template EditBox START ----------------------------------------
TEMPLATE.EditBox = function(button, width, funcSearch, funcClear)
	local boxwidth = width-(Textures.SearchBoxIcons.width+sizeGeneral.Space+sizeGeneral.Space+Textures.SearchBoxIcons.width)

	button.SearchIcon = button:CreateTexture(nil, "ARTWORK")
	button.SearchIcon:SetWidth(Textures.SearchBoxIcons.width) -- 17
	button.SearchIcon:SetHeight(Textures.SearchBoxIcons.height) --17 
	button.SearchIcon:SetPoint("LEFT", 0, 0)
	button.SearchIcon:SetTexture("Interface\\Minimap\\Tracking\\None")
	button.SearchIcon:SetTexCoord(3/32, 31/32, 1/32, 29/32)

	button.EditBox = CreateFrame("EditBox", nil, button)
	button.EditBox:SetWidth(boxwidth)
	button.EditBox:SetHeight(Textures.SearchBoxIcons.height)
	button.EditBox:SetPoint("LEFT", button.SearchIcon, "RIGHT", sizeGeneral.Space, 0)
	button.EditBox:SetAutoFocus(false)
	button.EditBox:SetFontObject(GameFontWhiteSmall)
	button.EditBox:SetTextInsets(sizeGeneral.Space, 0, 0, 0) 

	button.EditBoxLeft = button.EditBox:CreateTexture(nil, "BACKGROUND")
	button.EditBoxLeft:SetWidth(5)
	button.EditBoxLeft:SetHeight(Textures.SearchBoxIcons.height)
	button.EditBoxLeft:SetPoint("LEFT", 0, 0)
	button.EditBoxLeft:SetTexture(Textures.CharScannerIcons.path)
	button.EditBoxLeft:SetTexCoord(unpack(Textures.SearchBoxIcons.coordsL))

	button.EditBoxMiddle = button.EditBox:CreateTexture(nil, "BACKGROUND")
	button.EditBoxMiddle:SetWidth(boxwidth-5-5)
	button.EditBoxMiddle:SetHeight(Textures.SearchBoxIcons.height)
	button.EditBoxMiddle:SetPoint("LEFT", button.EditBoxLeft, "RIGHT", 0, 0)
	button.EditBoxMiddle:SetTexture(Textures.CharScannerIcons.path)
	button.EditBoxMiddle:SetTexCoord(unpack(Textures.SearchBoxIcons.coordsM))

	button.EditBoxRight = button.EditBox:CreateTexture(nil, "BACKGROUND")
	button.EditBoxRight:SetWidth(5)
	button.EditBoxRight:SetHeight(Textures.SearchBoxIcons.height)
	button.EditBoxRight:SetPoint("LEFT", button.EditBoxMiddle, "RIGHT", 0, 0)
	button.EditBoxRight:SetTexture(Textures.CharScannerIcons.path)
	button.EditBoxRight:SetTexCoord(unpack(Textures.SearchBoxIcons.coordsR))

	button.EditBoxClear = CreateFrame("Button", nil, button)
	button.EditBoxClear:SetWidth(Textures.SearchBoxIcons.width)
	button.EditBoxClear:SetHeight(Textures.SearchBoxIcons.height)
	button.EditBoxClear:SetPoint("LEFT", button.EditBox, "RIGHT", sizeGeneral.Space, 0)

	button.EditBoxClearNormal = button.EditBoxClear:CreateTexture(nil, "ARTWORK")
	button.EditBoxClearNormal:SetPoint("TOPLEFT", 0, 0)
	button.EditBoxClearNormal:SetPoint("BOTTOMRIGHT", 0, 0)
	button.EditBoxClearNormal:SetAlpha(0.85)
	button.EditBoxClearNormal:SetTexture(Textures.CharScannerIcons.path)
	button.EditBoxClearNormal:SetTexCoord(unpack(Textures.SearchBoxIcons.coordsCLR))
	button.EditBoxClear:SetNormalTexture(button.EditBoxClearNormal)

	button.EditBoxClearDisable = button.EditBoxClear:CreateTexture(nil, "ARTWORK")
	button.EditBoxClearDisable:SetPoint("TOPLEFT", 0, 0)
	button.EditBoxClearDisable:SetPoint("BOTTOMRIGHT", 0, 0)
	button.EditBoxClearDisable:SetTexture(Textures.CharScannerIcons.path)
	button.EditBoxClearDisable:SetTexCoord(unpack(Textures.SearchBoxIcons.coordsCLR))
	button.EditBoxClear:SetDisabledTexture(button.EditBoxClearDisable)

	Desaturation(button.EditBoxClearDisable, true)

	button.EditBox:SetScript("OnShow",         function(self) self:SetAutoFocus(false) end)
	button.EditBox:SetScript("OnEnterPressed", function(self) self:ClearFocus() end)
	button.EditBox:SetScript("OnTextChanged",  function() funcSearch() end)

	button.EditBoxClear:Disable()
	button.EditBoxClear:SetScript("OnClick", function() funcClear() end)
	button.EditBoxClear:SetScript("OnMouseDown", function()
		button.EditBoxClearNormal:SetPoint("TOPLEFT", 1, -1)
		button.EditBoxClearNormal:SetPoint("BOTTOMRIGHT", -1, 1)
	end)
	button.EditBoxClear:SetScript("OnMouseUp", function()
		button.EditBoxClearNormal:SetPoint("TOPLEFT", 0, 0)
		button.EditBoxClearNormal:SetPoint("BOTTOMRIGHT", 0, 0)
	end)
	button.EditBoxClear:SetScript("OnEnter", function()
		button.EditBoxClearNormal:SetAlpha(1)
	end)
	button.EditBoxClear:SetScript("OnLeave", function()
		button.EditBoxClearNormal:SetAlpha(0.85)
	end)
end
-- Template EditBox END ----------------------------------------

-- Template PvEStats START ----------------------------------------
TEMPLATE.PvEStats = function(button, collapse, level, l1, l2, l3, NOBOSS) --NOBOSS
	local collapseWidth = 0
	local nameWidth = sizeStatFrame.NameWidth
	local nameLeft = sizeStatFrame.NameLeft
	local achievementWidth = sizeStatFrame.AchievementWidth
	local col = "White"

	if level == "title" then
		col = "Yellow"
		nameWidth = sizeStatFrame.NameWidthTitle
		achievementWidth = sizeStatFrame.AchievementWidthTitle
	elseif level == "level1" then
		col = "Level1"
		if collapse then
			nameWidth = sizeStatFrame.NameWidth_L1c
			nameLeft = sizeStatFrame.NameLeft_L1c
			collapseWidth = 0
		end
	elseif level == "level2" then
		col = "Level2"
		nameWidth = sizeStatFrame.NameWidth_L2
		nameLeft = sizeStatFrame.NameLeft_L2
		if collapse then
			nameWidth = sizeStatFrame.NameWidth_L2c
			nameLeft = sizeStatFrame.NameLeft_L2c
			collapseWidth = sizeStatFrame.LevelModifier
		end
	elseif level == "level3" then
		col = "Level3"
		nameWidth = sizeStatFrame.NameWidth_L3
		nameLeft = sizeStatFrame.NameLeft_L3
		if collapse then
			nameWidth = sizeStatFrame.NameWidth_L3c
			nameLeft = sizeStatFrame.NameLeft_L3c
			collapseWidth = sizeStatFrame.LevelModifier*2
		end
	elseif level == "level4" then
		nameWidth = sizeStatFrame.NameWidth_L4
		nameLeft = sizeStatFrame.NameLeft_L4
	elseif level == "search" then
		nameWidth = sizeStatFrame.NameWidthSearch
	end

	button:SetFrameLevel( GVAR.MainFrame.Stats:GetFrameLevel()+1 )
	button:SetWidth(sizeStatFrame.ButtonWidth)
	button:SetHeight(sizeGeneral.BarHeight)

	button.Name = button:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
	button.Name:SetWidth(nameWidth)
	button.Name:SetHeight(fonts.GameFontNormalSmall.height)
	button.Name:SetPoint("LEFT", nameLeft, 0)
	button.Name:SetJustifyH("LEFT")
	button.Name:SetTextColor(color[col].rgba[1], color[col].rgba[2], color[col].rgba[3], color[col].rgba[4])

	button.BossesTotal = button:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
	button.BossesTotal:SetWidth(sizeStatFrame.BossesTotalWidth)
	button.BossesTotal:SetHeight(fonts.GameFontNormalSmall.height)
	button.BossesTotal:SetPoint("RIGHT", -sizeStatFrame.BossesTotalRight, 0)
	button.BossesTotal:SetJustifyH("RIGHT")
	button.BossesTotal:SetTextColor(color[col].rgba[1], color[col].rgba[2], color[col].rgba[3], color[col].rgba[4])

	if collapse then
		button.Collapse = button:CreateTexture(nil, "ARTWORK")
		button.Collapse:SetWidth(Textures.PlusMinus.qsize)
		button.Collapse:SetHeight(Textures.PlusMinus.qsize)
		button.Collapse:SetPoint("LEFT", collapseWidth, 0)
		button.Collapse:SetTexture(Textures.CharScannerIcons.path)
		button.Collapse:SetTexCoord(unpack(Textures.PlusMinus.coordsPlus))
		if NOBOSS then --NOBOSS
			button.Collapse:SetAlpha(0.2)
		end

		button.AchievementDetail = CreateFrame("Button", nil, button)
		button.AchievementDetail:SetWidth(achievementWidth + sizeStatFrame.AchievementDetailWidth + sizeGeneral.Space)
		button.AchievementDetail:SetHeight(sizeGeneral.BarHeight)
		button.AchievementDetail:SetPoint("RIGHT", -sizeStatFrame.AchievementDetailRight, 0)
		button.AchievementDetail:SetFrameLevel( GVAR.MainFrame.Stats:GetFrameLevel()+2 )

		button.AchievementDetailHighlight = button.AchievementDetail:CreateTexture(nil, "ARTWORK")
		button.AchievementDetailHighlight:SetPoint("TOPLEFT", button, "TOPRIGHT", -(sizeStatFrame.AchievementRight+achievementWidth), 0)
		button.AchievementDetailHighlight:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", -sizeStatFrame.AchievementRight, 0)
		button.AchievementDetailHighlight:SetTexture(0, 0, 0, 0)

		button.AchievementDetailIcon = button.AchievementDetail:CreateTexture(nil, "ARTWORK")
		button.AchievementDetailIcon:SetWidth(Textures.AchievementDetail.width)
		button.AchievementDetailIcon:SetHeight(Textures.AchievementDetail.height)
		button.AchievementDetailIcon:SetPoint("RIGHT", 0, 0)
		button.AchievementDetailIcon:SetTexture(Textures.CharScannerIcons.path)
		button.AchievementDetailIcon:SetTexCoord(unpack(Textures.AchievementDetail.coords))
		button.AchievementDetailIcon:SetAlpha(0.25) 

		button.Achievement = button.AchievementDetail:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
		button.Achievement:SetWidth(achievementWidth)
		button.Achievement:SetHeight(fonts.GameFontNormalSmall.height)
		button.Achievement:SetPoint("RIGHT", button.AchievementDetailIcon, "LEFT", -sizeGeneral.Space, 0)
		button.Achievement:SetJustifyH("RIGHT")
		button.Achievement:SetTextColor(color[col].rgba[1], color[col].rgba[2], color[col].rgba[3], color[col].rgba[4])

		button.AchievementDetail:SetScript("OnEnter", function() CharScanner:AchievementDetail_OnEnter("PvE", button, l1, l2, l3) end)
		button.AchievementDetail:SetScript("OnLeave", function() CharScanner:AchievementDetail_OnLeave("PvE", button, l1, l2, l3) end)
		button.AchievementDetail:SetScript("OnClick", function() CharScanner:AchievementDetail_OnClick("PvE", button, l1, l2, l3) end)

		button.RaidSelect = CreateFrame("Button", nil, button)
		button.RaidSelect:SetWidth(Textures.RaidSelect.width)
		button.RaidSelect:SetHeight(Textures.RaidSelect.height)
		button.RaidSelect:SetPoint("RIGHT", button, "RIGHT", -sizeStatFrame.RaidSelectRight, 0)
		button.RaidSelect:SetFrameLevel( GVAR.MainFrame.Stats:GetFrameLevel()+2 )
		button.RaidSelectTexture = button.RaidSelect:CreateTexture(nil, "ARTWORK")
		button.RaidSelectTexture:SetWidth(Textures.RaidSelect.width)
		button.RaidSelectTexture:SetHeight(Textures.RaidSelect.height)
		button.RaidSelectTexture:SetPoint("LEFT", 0, 0)
		button.RaidSelectTexture:SetTexture(Textures.CharScannerIcons.path)
		button.RaidSelectTexture:SetTexCoord(unpack(Textures.RaidSelect.coords))
		button.RaidSelectTexture:SetAlpha(0.25) 

		button.RaidSelect:SetScript("OnEnter", function() button.RaidSelectTexture:SetAlpha(1) end)
		button.RaidSelect:SetScript("OnLeave", function() button.RaidSelectTexture:SetAlpha(0.25) end)
		button.RaidSelect:SetScript("OnClick", function() CharScanner:CompareFrameShow() CharScanner:History_GUISort("RAID", GUI_history_SortBy, l1, l2, l3) end)
	end

	if level == "title" or level == "summary" then
		button.Achievement = button:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
		button.Achievement:SetWidth(achievementWidth)
		button.Achievement:SetHeight(fonts.GameFontNormalSmall.height)
		button.Achievement:SetPoint("RIGHT", -sizeStatFrame.AchievementRight, 0)
		button.Achievement:SetJustifyH("RIGHT")
		button.Achievement:SetTextColor(color[col].rgba[1], color[col].rgba[2], color[col].rgba[3], color[col].rgba[4])
	end
	if collapse or level == "title" or level == "summary" then
		button.Statistic = button:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
		button.Statistic:SetWidth(sizeStatFrame.StatisticWidth)
		button.Statistic:SetHeight(fonts.GameFontNormalSmall.height)
		button.Statistic:SetPoint("RIGHT", -sizeStatFrame.StatisticRight, 0)
		button.Statistic:SetJustifyH("RIGHT")
		button.Statistic:SetTextColor(color[col].rgba[1], color[col].rgba[2], color[col].rgba[3], color[col].rgba[4])
	end

	if level ~= "title" and level ~= "summary" then
		button.Highlight = button:CreateTexture(nil, "ARTWORK")
		button.Highlight:SetPoint("TOPLEFT", 0, 0)
		button.Highlight:SetPoint("BOTTOMRIGHT", 0, 0)
		button.Highlight:SetTexture(color.ButtonHighlight.rgba[1], color.ButtonHighlight.rgba[2], color.ButtonHighlight.rgba[3], color.ButtonHighlight.rgba[4])
		button:SetHighlightTexture(button.Highlight)
		button.FXtexture = button:CreateTexture(nil, "BACKGROUND")
		button.FXtexture:SetPoint("TOPLEFT", nameLeft, -5)
		button.FXtexture:SetPoint("BOTTOMRIGHT", -sizeStatFrame.BossesTotalRight, 5)
		button.FXtexture:SetTexture(color.ButtonLineMarker.rgba[1], color.ButtonLineMarker.rgba[2], color.ButtonLineMarker.rgba[3], color.ButtonLineMarker.rgba[4])
	end
--[[ TEST
	if button.Name then
		local texture = button:CreateTexture(nil, "ARTWORK")
		texture:SetPoint("LEFT", button, "LEFT", nameLeft, 0)
		texture:SetPoint("RIGHT", button, "LEFT", nameWidth+nameLeft, 0)
		texture:SetTexture(0.22, 0.52, 0.22, 1)
	end
	if button.Achievement then
		local texture = button:CreateTexture(nil, "ARTWORK")
		texture:SetPoint("LEFT", button, "RIGHT", -(sizeStatFrame.AchievementRight+achievementWidth), 0)
		texture:SetPoint("RIGHT", button, "RIGHT", -sizeStatFrame.AchievementRight, 0)
		texture:SetTexture(0.22, 0.22, 0.82, 1)
	end
	if button.AchievementDetail then
		local texture = button:CreateTexture(nil, "ARTWORK")
		texture:SetPoint("LEFT", button, "RIGHT", -(sizeStatFrame.AchievementDetailRight+sizeStatFrame.AchievementDetailWidth), 0)
		texture:SetPoint("RIGHT", button, "RIGHT", -sizeStatFrame.AchievementDetailRight, 0)
		texture:SetTexture(0.32, 0.22, 0.22, 1)
	end
	if button.Statistic then
		local texture = button:CreateTexture(nil, "ARTWORK")
		texture:SetPoint("LEFT", button, "RIGHT", -(sizeStatFrame.StatisticRight+sizeStatFrame.StatisticWidth), 0)
		texture:SetPoint("RIGHT", button, "RIGHT", -sizeStatFrame.StatisticRight, 0)
		texture:SetTexture(0.42, 0.22, 0.22, 1)
	end
	if button.BossesTotal then
		local texture = button:CreateTexture(nil, "ARTWORK")
		texture:SetPoint("LEFT", button, "RIGHT", -(sizeStatFrame.BossesTotalRight+sizeStatFrame.BossesTotalWidth), 0)
		texture:SetPoint("RIGHT", button, "RIGHT", -sizeStatFrame.BossesTotalRight, 0)
		texture:SetTexture(0.42, 0.12, 0.12, 1)
	end
	if button.RaidSelect then
		local texture = button:CreateTexture(nil, "ARTWORK")
		texture:SetPoint("LEFT", button, "RIGHT", -(sizeStatFrame.RaidSelectRight+sizeStatFrame.RaidSelectWidth), 0)
		texture:SetPoint("RIGHT", button, "RIGHT", -sizeStatFrame.RaidSelectRight, 0)
		texture:SetTexture(0.22, 0.72, 0.82, 1)
	end
--]]
end
-- Template PvEStats END ----------------------------------------

-- Template PvPStats START ----------------------------------------
TEMPLATE.PvPStats = function(button, collapse, level, l1, l2, l3)
	local collapseWidth = 0
	local nameWidth = sizeStatFrame.PvPNameWidth
	local nameLeft = sizeStatFrame.PvPNameLeft
	local achievementWidth = sizeStatFrame.PvPAchievementWidth
	local winlossWidth = sizeStatFrame.PvPWinLossWidth
	local winlosspercentWidth = sizeStatFrame.PvPWinLossPercentWidth
	local honorablekillsRight = sizeStatFrame.PvPHonorableKillsRight
	local col = "White"

	if level == "title" then
		col = "Yellow"
		nameWidth = sizeStatFrame.PvPNameWidthTitle
		achievementWidth = sizeStatFrame.PvPAchievementWidthTitle
		winlossWidth = 0
		winlosspercentWidth = sizeStatFrame.PvPWinLossPerWidthTitle
	elseif level == "level1" then
		col = "Level1"
		if collapse then
			nameWidth = sizeStatFrame.PvPNameWidth_L1c
			nameLeft = sizeStatFrame.PvPNameLeft_L1c
			collapseWidth = 0
		end
	elseif level == "level2" then
		col = "Level2"
		nameWidth = sizeStatFrame.PvPNameWidth_L2c + sizeStatFrame.LevelModifier
		nameLeft = sizeStatFrame.PvPNameLeft_L2c - sizeStatFrame.LevelModifier
		if collapse then
			nameWidth = sizeStatFrame.PvPNameWidth_L2c
			nameLeft = sizeStatFrame.PvPNameLeft_L2c
			collapseWidth = sizeStatFrame.LevelModifier
		end
	elseif level == "level3" then
		col = "Level3"
		honorablekillsRight = honorablekillsRight + 5 -- space for arena rating
		nameWidth = sizeStatFrame.PvPNameWidth_L3
		nameLeft = sizeStatFrame.PvPNameLeft_L3
		if collapse then
			nameWidth = sizeStatFrame.PvPNameWidth_L3c
			nameLeft = sizeStatFrame.PvPNameLeft_L3c
			collapseWidth = sizeStatFrame.LevelModifier*2
		end
	elseif level == "level4" then
		nameWidth = sizeStatFrame.PvPNameWidth_L4
		nameLeft = sizeStatFrame.vNameLeft_L4
	end

	button:SetFrameLevel( GVAR.MainFrame.Stats:GetFrameLevel()+1 )
	button:SetWidth(sizeStatFrame.ButtonWidth)
	button:SetHeight(sizeGeneral.BarHeight)

	-- Collapse Button START
	if collapse then
		button.Collapse = button:CreateTexture(nil, "ARTWORK")
		button.Collapse:SetWidth(Textures.PlusMinus.qsize)
		button.Collapse:SetHeight(Textures.PlusMinus.qsize)
		button.Collapse:SetPoint("LEFT", collapseWidth, 0)
		button.Collapse:SetTexture(Textures.CharScannerIcons.path)
		button.Collapse:SetTexCoord(unpack(Textures.PlusMinus.coordsPlus))
	end
	-- Collapse Button END

	-- Name Button START
	button.Name = button:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
	button.Name:SetWidth(nameWidth)
	button.Name:SetHeight(fonts.GameFontNormalSmall.height)
	button.Name:SetPoint("LEFT", nameLeft, 0)
	button.Name:SetJustifyH("LEFT")
	button.Name:SetTextColor(color[col].rgba[1], color[col].rgba[2], color[col].rgba[3], color[col].rgba[4])
	-- Name Button END

	-- Achievement Button START
	if level ~= "title" and level ~= "summary" and level ~= "level3" then
		button.AchievementDetail = CreateFrame("Button", nil, button)
		button.AchievementDetail:SetWidth(achievementWidth + sizeStatFrame.PvPAchievementDetailWidth + sizeGeneral.Space)
		button.AchievementDetail:SetHeight(sizeGeneral.BarHeight)
		button.AchievementDetail:SetPoint("RIGHT", -sizeStatFrame.PvPAchievementDetailRight, 0)
		button.AchievementDetail:SetFrameLevel( GVAR.MainFrame.Stats:GetFrameLevel()+2 )

		button.AchievementDetailHighlight = button.AchievementDetail:CreateTexture(nil, "ARTWORK")
		button.AchievementDetailHighlight:SetPoint("TOPLEFT", button, "TOPRIGHT", -(sizeStatFrame.PvPAchievementRight+achievementWidth), 0)
		button.AchievementDetailHighlight:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", -sizeStatFrame.PvPAchievementRight, 0)
		button.AchievementDetailHighlight:SetTexture(0, 0, 0, 0)

		button.AchievementDetailIcon = button.AchievementDetail:CreateTexture(nil, "ARTWORK")
		button.AchievementDetailIcon:SetWidth(Textures.AchievementDetail.width)
		button.AchievementDetailIcon:SetHeight(Textures.AchievementDetail.height)
		button.AchievementDetailIcon:SetPoint("RIGHT", 0, 0)
		button.AchievementDetailIcon:SetTexture(Textures.CharScannerIcons.path)
		button.AchievementDetailIcon:SetTexCoord(unpack(Textures.AchievementDetail.coords))
		button.AchievementDetailIcon:SetAlpha(0.25) 

		button.Achievement = button.AchievementDetail:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
		button.Achievement:SetWidth(achievementWidth)
		button.Achievement:SetHeight(fonts.GameFontNormalSmall.height)
		button.Achievement:SetPoint("RIGHT", button.AchievementDetailIcon, "LEFT", -sizeGeneral.Space, 0)
		button.Achievement:SetJustifyH("RIGHT")
		button.Achievement:SetTextColor(color[col].rgba[1], color[col].rgba[2], color[col].rgba[3], color[col].rgba[4])

		button.AchievementDetail:SetScript("OnEnter", function() CharScanner:AchievementDetail_OnEnter("PvP", button, l1, l2, l3) end)
		button.AchievementDetail:SetScript("OnLeave", function() CharScanner:AchievementDetail_OnLeave("PvP", button, l1, l2, l3) end)
		button.AchievementDetail:SetScript("OnClick", function() CharScanner:AchievementDetail_OnClick("PvP", button, l1, l2, l3) end)

		button.RaidSelect = CreateFrame("Button", nil, button)
		button.RaidSelect:SetWidth(Textures.RaidSelect.width)
		button.RaidSelect:SetHeight(Textures.RaidSelect.height)
		button.RaidSelect:SetPoint("RIGHT", button, "RIGHT", -sizeStatFrame.RaidSelectRight, 0)
		button.RaidSelect:SetFrameLevel( GVAR.MainFrame.Stats:GetFrameLevel()+2 )
		button.RaidSelectTexture = button.RaidSelect:CreateTexture(nil, "ARTWORK")
		button.RaidSelectTexture:SetWidth(Textures.RaidSelect.width)
		button.RaidSelectTexture:SetHeight(Textures.RaidSelect.height)
		button.RaidSelectTexture:SetPoint("LEFT", 0, 0)
		button.RaidSelectTexture:SetTexture(Textures.CharScannerIcons.path)
		button.RaidSelectTexture:SetTexCoord(unpack(Textures.RaidSelect.coords))
		button.RaidSelectTexture:SetAlpha(0) -- 0.25

		--button.RaidSelect:SetScript("OnEnter", function() button.RaidSelectTexture:SetAlpha(1) end)
		--button.RaidSelect:SetScript("OnLeave", function() button.RaidSelectTexture:SetAlpha(0.25) end)
		--button.RaidSelect:SetScript("OnClick", function() CharScanner:CompareFrameShow() CharScanner:History_GUISort("PVP", GUI_history_SortBy, l1, l2, l3) end)
	end
	-- Achievement Button END

	if level ~= "level3" then
		-- WinLoss Button START
		button.WinLoss = button:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
		button.WinLoss:SetWidth(winlossWidth)
		button.WinLoss:SetHeight(fonts.GameFontNormalSmall.height)
		button.WinLoss:SetPoint("RIGHT", -sizeStatFrame.PvPWinLossRight, 0)
		button.WinLoss:SetJustifyH("RIGHT")
		button.WinLoss:SetTextColor(color[col].rgba[1], color[col].rgba[2], color[col].rgba[3], color[col].rgba[4])
		-- WinLoss Button END

		-- WinLossPercent Button START
		button.WinLossPercent = button:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
		button.WinLossPercent:SetWidth(winlosspercentWidth)
		button.WinLossPercent:SetHeight(fonts.GameFontNormalSmall.height)
		button.WinLossPercent:SetPoint("RIGHT", -sizeStatFrame.PvPWinLossPercentRight, 0)
		button.WinLossPercent:SetJustifyH("RIGHT")
		button.WinLossPercent:SetTextColor(color[col].rgba[1], color[col].rgba[2], color[col].rgba[3], color[col].rgba[4])
		-- WinLossPercent Button END
	end

	-- HonorableKills Button START
	button.HonorableKills = button:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
	button.HonorableKills:SetWidth(sizeStatFrame.PvPHonorableKillsWidth)
	button.HonorableKills:SetHeight(fonts.GameFontNormalSmall.height)
	button.HonorableKills:SetPoint("RIGHT", -honorablekillsRight, 0)
	button.HonorableKills:SetJustifyH("RIGHT")
	button.HonorableKills:SetTextColor(color[col].rgba[1], color[col].rgba[2], color[col].rgba[3], color[col].rgba[4])
	-- HonorableKills Button END

	if level == "title" or level == "summary" then
		button.Achievement = button:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
		button.Achievement:SetWidth(achievementWidth)
		button.Achievement:SetHeight(fonts.GameFontNormalSmall.height)
		button.Achievement:SetPoint("RIGHT", -sizeStatFrame.PvPAchievementRight, 0)
		button.Achievement:SetJustifyH("RIGHT")
		button.Achievement:SetTextColor(color[col].rgba[1], color[col].rgba[2], color[col].rgba[3], color[col].rgba[4])
	end

	if level ~= "title" and level ~= "summary" then
		button.Highlight = button:CreateTexture(nil, "ARTWORK")
		button.Highlight:SetPoint("TOPLEFT", 0, 0)
		button.Highlight:SetPoint("BOTTOMRIGHT", 0, 0)
		button.Highlight:SetTexture(color.ButtonHighlight.rgba[1], color.ButtonHighlight.rgba[2], color.ButtonHighlight.rgba[3], color.ButtonHighlight.rgba[4])
		button:SetHighlightTexture(button.Highlight)
		button.FXtexture = button:CreateTexture(nil, "BACKGROUND")
		button.FXtexture:SetPoint("TOPLEFT", nameLeft, -5)
		button.FXtexture:SetPoint("BOTTOMRIGHT", -sizeStatFrame.PvPHonorableKillsRight, 5)
		button.FXtexture:SetTexture(color.ButtonLineMarker.rgba[1], color.ButtonLineMarker.rgba[2], color.ButtonLineMarker.rgba[3], color.ButtonLineMarker.rgba[4])
	end
--[[ TEST
	if button.Name then
		local texture = button:CreateTexture(nil, "ARTWORK")
		texture:SetPoint("LEFT", button, "LEFT", nameLeft, 0)
		texture:SetPoint("RIGHT", button, "LEFT", nameWidth+nameLeft, 0)
		texture:SetTexture(0.22, 0.42, 0.22, 1)
	end
	if button.Achievement then
		local texture = button:CreateTexture(nil, "ARTWORK")
		texture:SetPoint("LEFT", button, "RIGHT", -(sizeStatFrame.PvPAchievementRight+achievementWidth), 0)
		texture:SetPoint("RIGHT", button, "RIGHT", -sizeStatFrame.PvPAchievementRight, 0)
		texture:SetTexture(0.42, 0.42, 0.42, 1)
	end
	if button.AchievementDetail then
		local texture = button:CreateTexture(nil, "ARTWORK")
		texture:SetPoint("LEFT", button, "RIGHT", -(sizeStatFrame.PvPAchievementDetailRight+sizeStatFrame.PvPAchievementDetailWidth), 0)
		texture:SetPoint("RIGHT", button, "RIGHT", -sizeStatFrame.PvPAchievementDetailRight, 0)
		texture:SetTexture(0.22, 0.32, 0.22, 1)
	end
	if button.WinLoss then
		local texture = button:CreateTexture(nil, "ARTWORK")
		texture:SetPoint("LEFT", button, "RIGHT", -(sizeStatFrame.PvPWinLossRight+winlossWidth), 0)
		texture:SetPoint("RIGHT", button, "RIGHT", -sizeStatFrame.PvPWinLossRight, 0)
		texture:SetTexture(0.82, 0.22, 0.32, 1)
	end
	if button.WinLossPercent then
		local texture = button:CreateTexture(nil, "ARTWORK")
		texture:SetPoint("LEFT", button, "RIGHT", -(sizeStatFrame.PvPWinLossPercentRight+winlosspercentWidth), 0)
		texture:SetPoint("RIGHT", button, "RIGHT", -sizeStatFrame.PvPWinLossPercentRight, 0)
		texture:SetTexture(0.22, 0.22, 0.62, 1)
	end
	if button.HonorableKills then
		local texture = button:CreateTexture(nil, "ARTWORK")
		texture:SetPoint("LEFT", button, "RIGHT", -(sizeStatFrame.PvPHonorableKillsRight+sizeStatFrame.PvPHonorableKillsWidth), 0)
		texture:SetPoint("RIGHT", button, "RIGHT", -sizeStatFrame.PvPHonorableKillsRight, 0)
		texture:SetTexture(0.12, 0.22, 0.22, 1)
	end
	if button.RaidSelect then
		local texture = button:CreateTexture(nil, "ARTWORK")
		texture:SetPoint("LEFT", button, "RIGHT", -(sizeStatFrame.RaidSelectRight+sizeStatFrame.RaidSelectWidth), 0)
		texture:SetPoint("RIGHT", button, "RIGHT", -sizeStatFrame.RaidSelectRight, 0)
		texture:SetTexture(0.52, 0.02, 0.22, 1)
	end
--]]
end
-- Template PvPStats END ----------------------------------------

-- Template SortPosition START ----------------------------------------
TEMPLATE.SortPosition = function(button, text, pos, width)
	button.Background = button:CreateTexture(nil, "BACKGROUND")
	button.Background:SetPoint("TOPLEFT", 0, 0)
	button.Background:SetPoint("BOTTOMRIGHT", 0, 0)
	button.Background:SetTexture(color.ButtonSortBG.rgba[1], color.ButtonSortBG.rgba[2], color.ButtonSortBG.rgba[3], color.ButtonSortBG.rgba[4])

	if text then
		if pos == "right" then
			button.Text = button:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
			button.Text:SetText(text)
			local fw = button.Text:GetStringWidth()
			if fw >= width then
				button.Text:SetWidth( width-10 )
			end
			button.Text:SetHeight(fonts.GameFontNormalSmall.height)
			button.Text:SetPoint("RIGHT", button, "RIGHT", 0, 0)
			button.Text:SetJustifyH("RIGHT")
		elseif pos == "left" then
			button.Text = button:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
			button.Text:SetText(text)
			local fw = button.Text:GetStringWidth()
			if fw >= width then
				button.Text:SetWidth( width-10 )
			end
			button.Text:SetHeight(fonts.GameFontNormalSmall.height)
			button.Text:SetPoint("LEFT", button, "LEFT", 0, 0)
			button.Text:SetJustifyH("LEFT")
		else
			button.Text = button:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
			button.Text:SetText(text)
			local fw = button.Text:GetStringWidth()
			if fw >= width then
				button.Text:SetWidth( width-10 )
			end
			button.Text:SetHeight(fonts.GameFontNormalSmall.height)
			button.Text:SetPoint("CENTER", button, "CENTER", 0, 0)
			button.Text:SetJustifyH("CENTER")
		end
	end

	button.Arrow = button:CreateTexture(nil, "BACKGROUND")
	button.Arrow:SetWidth(Textures.SortArrow.width)
	button.Arrow:SetHeight(Textures.SortArrow.height)
	button.Arrow:SetTexture(Textures.CharScannerIcons.path)
	if text then
		if pos == "right" then
			button.Arrow:SetPoint("RIGHT", button.Text, "LEFT", -1, 0)
		elseif pos == "left" then
			button.Arrow:SetPoint("LEFT", button.Text, "RIGHT", 1, 0)
		else
			button.Arrow:SetPoint("RIGHT", button.Text, "LEFT", -1, 0)
		end
	else
		button.Arrow:SetPoint("CENTER", button, "CENTER", 0, 0)
	end

	button.Highlight = button:CreateTexture(nil, "ARTWORK")
	button.Highlight:SetPoint("TOPLEFT", 0, 0)
	button.Highlight:SetPoint("BOTTOMRIGHT", 0, 0)
	button.Highlight:SetTexture(color.ButtonHighlight.rgba[1], color.ButtonHighlight.rgba[2], color.ButtonHighlight.rgba[3], color.ButtonHighlight.rgba[4])
	button:SetHighlightTexture(button.Highlight)
end
-- Template SortPosition END ----------------------------------------

-- Template TalentButton START ----------------------------------------
TEMPLATE.TalentButton = function(button)
	button:SetWidth(Textures.TalentIconTexture.qsize)
	button:SetHeight(Textures.TalentIconTexture.qsize)

	button.Border = button:CreateTexture(nil, "BACKGROUND")
	button.Border:SetWidth(Textures.TalentIconTexture.qsize)
	button.Border:SetHeight(Textures.TalentIconTexture.qsize)
	button.Border:SetAllPoints(button)
	button.Border:SetTexture(0.2, 0.2, 0.2, 1)

	button.IconTexture = button:CreateTexture(nil, "BORDER")
	button.IconTexture:SetWidth(Textures.TalentIconTexture.qsize-2)
	button.IconTexture:SetHeight(Textures.TalentIconTexture.qsize-2)
	button.IconTexture:SetPoint("TOPLEFT", 1, -1)
	button.IconTexture:SetTexCoord(5/64, 59/64, 5/64, 59/64)

	button.RankBorder = button:CreateTexture(nil, "ARTWORK")
	button.RankBorder:SetWidth(Textures.TalentRankBorder.qsize)
	button.RankBorder:SetHeight(Textures.TalentRankBorder.qsize)
	button.RankBorder:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 2, -2)
	button.RankBorder:SetTexture(Textures.CharScannerIcons.path)
	button.RankBorder:SetTexCoord(unpack(Textures.TalentRankBorder.coords))

	button.RankNumber = button:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
	button.RankNumber:SetWidth(12)
	button.RankNumber:SetHeight(12)
	button.RankNumber:SetPoint("CENTER", button.RankBorder, "CENTER", 0, 0)
	button.RankNumber:SetJustifyH("CENTER")
end
-- Template TalentButton END ----------------------------------------

-- Template CheckButton START ----------------------------------------
TEMPLATE.DisableCheckButton = function(button)
	button.Text:SetTextColor(0.5, 0.5, 0.5)
	button.Border:SetTexture(0.4, 0.4, 0.4, 1)
	button:Disable()
end

TEMPLATE.EnableCheckButton = function(button)
	button.Text:SetTextColor(1, 1, 1)
	button.Border:SetTexture(0.8, 0.2, 0.2, 1)
	button:Enable()
end

TEMPLATE.CheckButton = function(button, size, space, text)
	button.Border = button:CreateTexture(nil, "BACKGROUND")
	button.Border:SetWidth( size )
	button.Border:SetHeight( size )
	button.Border:SetPoint("LEFT", 0, 0)
	button.Border:SetTexture(0.4, 0.4, 0.4, 1)

	button.Background = button:CreateTexture(nil, "BORDER")
	button.Background:SetPoint("TOPLEFT", button.Border, "TOPLEFT", 1, -1)
	button.Background:SetPoint("BOTTOMRIGHT", button.Border, "BOTTOMRIGHT", -1, 1)
	button.Background:SetTexture(0, 0, 0, 1)

	button.Normal = button:CreateTexture(nil, "ARTWORK")
	button.Normal:SetPoint("TOPLEFT", button.Border, "TOPLEFT", 1, -1)
	button.Normal:SetPoint("BOTTOMRIGHT", button.Border, "BOTTOMRIGHT", -1, 1)
	button.Normal:SetTexture(0, 0, 0, 1)
	button:SetNormalTexture(button.Normal)

	button.Push = button:CreateTexture(nil, "ARTWORK")
	button.Push:SetPoint("TOPLEFT", button.Border, "TOPLEFT", 4, -4)
	button.Push:SetPoint("BOTTOMRIGHT", button.Border, "BOTTOMRIGHT", -4, 4)
	button.Push:SetTexture(0.4, 0.4, 0.4, 0.5)
	button:SetPushedTexture(button.Push)

	button.Disabled = button:CreateTexture(nil, "ARTWORK")
	button.Disabled:SetPoint("TOPLEFT", button.Border, "TOPLEFT", 3, -3)
	button.Disabled:SetPoint("BOTTOMRIGHT", button.Border, "BOTTOMRIGHT", -3, 3)
	button.Disabled:SetTexture(0.4, 0.4, 0.4, 0.5)
	button:SetDisabledTexture(button.Disabled)

	button.Checked = button:CreateTexture(nil, "ARTWORK")
	button.Checked:SetWidth( size )
	button.Checked:SetHeight( size )
	button.Checked:SetPoint("LEFT", 0, 0)
	button.Checked:SetTexture("Interface\\Buttons\\UI-CheckBox-Check")
	button:SetCheckedTexture(button.Checked)

	button.Text = button:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
	button.Text:SetHeight( size )
	button.Text:SetPoint("LEFT", button.Normal, "RIGHT", space, 0)
	button.Text:SetJustifyH("LEFT")
	button.Text:SetText(text)
	button.Text:SetTextColor(1, 1, 1, 1)

	button:SetWidth(size + space + button.Text:GetStringWidth() + space)
	button:SetHeight(size)

	button.Highlight = button:CreateTexture(nil, "OVERLAY")
	button.Highlight:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0)
	button.Highlight:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 0, 0)
	button.Highlight:SetTexture(1, 1, 1, 0.1)
	button.Highlight:Hide()

	button:SetScript("OnEnter", function() button.Highlight:Show() end)
	button:SetScript("OnLeave", function() button.Highlight:Hide() end)
end
-- Template CheckButton END ----------------------------------------

-- Template Slider START ----------------------------------------
TEMPLATE.Slider = function(slider, width, step, minVal, maxVal, curVal, func, pro, scale)
	if not scale then scale = 1 end

	slider:SetWidth(width)
	slider:SetHeight(sizeGeneral.SliderSize)
	slider:SetValueStep(step) 
	slider:SetMinMaxValues(minVal, maxVal)
	slider:SetValue(curVal*scale)
	slider:SetOrientation("HORIZONTAL")

	slider.TextMin = slider:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	slider.TextMin:SetPoint("TOP", slider, "BOTTOM", 0, -2)
	slider.TextMin:SetPoint("LEFT", slider, "LEFT", 0, 0)
	slider.TextMin:SetJustifyH("CENTER")
	slider.TextMin:SetTextColor(0.6, 0.6, 0.6, 1)
	slider.TextMin:SetText((minVal/scale))

	slider.TextMax = slider:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	slider.TextMax:SetPoint("TOP", slider, "BOTTOM", 0, -2)
	slider.TextMax:SetPoint("RIGHT", slider, "RIGHT", 0, 0)
	slider.TextMax:SetJustifyH("CENTER")
	slider.TextMax:SetTextColor(0.6, 0.6, 0.6, 1)
	slider.TextMax:SetText((maxVal/scale))

	slider.TextCur = slider:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	slider.TextCur:SetPoint("TOP", slider, "BOTTOM", 0, -2)
	slider.TextCur:SetPoint("CENTER", slider, "CENTER", 0, 0)
	slider.TextCur:SetJustifyH("CENTER")
	slider.TextCur:SetTextColor(1, 1, 1, 1)
	if pro then
		slider.TextCur:SetText(curVal..pro)
	else
		slider.TextCur:SetText(curVal)
	end

	slider.BGL = slider:CreateTexture(nil, "BACKGROUND")
	slider.BGL:SetWidth(5)
	slider.BGL:SetHeight(6)
	slider.BGL:SetPoint("LEFT", slider, "LEFT", 0, 0)
	slider.BGL:SetTexture(Textures.CharScannerIcons.path)
	slider.BGL:SetTexCoord(unpack(Textures.SliderBG.coordsL))
	slider.BGM = slider:CreateTexture(nil, "BACKGROUND")
	slider.BGM:SetWidth(width-5-5)
	slider.BGM:SetHeight(6)
	slider.BGM:SetPoint("LEFT", slider.BGL, "RIGHT", 0, 0)
	slider.BGM:SetTexture(Textures.CharScannerIcons.path)
	slider.BGM:SetTexCoord(unpack(Textures.SliderBG.coordsM))
	slider.BGR = slider:CreateTexture(nil, "BACKGROUND")
	slider.BGR:SetWidth(5)
	slider.BGR:SetHeight(6)
	slider.BGR:SetPoint("LEFT", slider.BGM, "RIGHT", 0, 0)
	slider.BGR:SetTexture(Textures.CharScannerIcons.path)
	slider.BGR:SetTexCoord(unpack(Textures.SliderBG.coordsR))

	slider.Thumb = slider:CreateTexture(nil, "BORDER")
	slider.Thumb:SetWidth(Textures.SliderKnob.width)
	slider.Thumb:SetHeight(Textures.SliderKnob.height)
	slider.Thumb:SetTexture(Textures.CharScannerIcons.path)
	slider.Thumb:SetTexCoord(unpack(Textures.SliderKnob.coords))
	slider:SetThumbTexture(slider.Thumb)

	slider:SetScript("OnValueChanged", function(self, value)
		value = value/scale
		if pro then
			slider.TextCur:SetText(value..pro)
		else
			slider.TextCur:SetText(value)
		end
		if func then
			func(self, value)
		end
	end)
end
-- Template Slider END ----------------------------------------
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function CharScanner:InitializeOptions()
	SlashCmdList["CHARSCANNER"] = SlashHandler
	SLASH_CHARSCANNER1 = "/cs"
	SLASH_CHARSCANNER2 = "/scan"
	SLASH_CHARSCANNER3 = "/charscanner"

	if CharScanner_Options.version == nil then
		CharScanner_Options.version = 4
	end

	if CharScanner_Options.version == 1 then
		table_wipe(CharScanner_Options)
		CharScanner_Options.version = 2
		Print("Options reset.")
	end

	if CharScanner_Options.version == 2 then
		CharScanner_Options.version = 3
		CharScanner_Options.scanButtonTrans = nil
	end

	if CharScanner_Options.version == 3 then
		CharScanner_Options.MainFrame_relP = nil
		CharScanner_Options.MainFrame_posX = nil
		CharScanner_Options.MainFrame_posY = nil
		CharScanner_Options.ScanButton_relP = nil
		CharScanner_Options.ScanButton_posX = nil
		CharScanner_Options.ScanButton_posY = nil
		Print("MainFrame position reset.")
		if CharScanner_Options.scanButton then
			CharScanner_Options.scanButtonLock = false
			Print("SCAN-Button position reset! -> "..L["click & move"].." + "..L["Lock SCAN-Button"])
		end
		CharScanner_Options.version = 4
	end

	if CharScanner_Options.scanButton        == nil then CharScanner_Options.scanButton        = false end
	if CharScanner_Options.scanButtonGrpAll  == nil then CharScanner_Options.scanButtonGrpAll  = false end
	if CharScanner_Options.scanButtonGrpMiss == nil then CharScanner_Options.scanButtonGrpMiss = false end
	if CharScanner_Options.scanButtonLock    == nil then CharScanner_Options.scanButtonLock    = false end
	if CharScanner_Options.scanButtonSize    == nil then CharScanner_Options.scanButtonSize    = 20    end
	if CharScanner_Options.chatoutput        == nil then CharScanner_Options.chatoutput        = false end
	if CharScanner_Options.chatoutputGrp     == nil then CharScanner_Options.chatoutputGrp     = false end
	if CharScanner_Options.scanDuration      == nil then CharScanner_Options.scanDuration      = 1     end
	if CharScanner_Options.MinimapButton     == nil then CharScanner_Options.MinimapButton     = false end
	if CharScanner_Options.MinimapButtonPos  == nil then CharScanner_Options.MinimapButtonPos  = -90   end
	if CharScanner_Options.StatsCurRows == nil then
		CharScanner_Options.StatsCurRows = GUI_ABKsearch_curRows
	else
		GUI_ABKsearch_curRows = CharScanner_Options.StatsCurRows
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function CharScanner:LDBcheck()
	if LibStub and LibStub:GetLibrary("CallbackHandler-1.0", true) and LibStub:GetLibrary("LibDataBroker-1.1", true) then
		LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("CharScanner", {
			type = "launcher",
			icon = Textures.CharScannerGUI.path,
			OnClick = function(self, button)
				CharScanner:MiniOptionsToggle(button, 1)
			end,
		})
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function CharScanner:CreateInterfaceOptions()
	GVAR.InterfaceOptions = CreateFrame("Frame", "CharScanner_InterfaceOptions")
	GVAR.InterfaceOptions.name = "CharScanner"

	GVAR.InterfaceOptions.Title = GVAR.InterfaceOptions:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalLarge.name)
	GVAR.InterfaceOptions.Title:SetPoint("TOPLEFT", 16, -16)
	GVAR.InterfaceOptions.Title:SetText("CharScanner")
	GVAR.InterfaceOptions.Title:SetJustifyH("LEFT")
	GVAR.InterfaceOptions.Title:SetJustifyV("TOP")

	GVAR.InterfaceOptions.GUI = CreateFrame("Button", nil, GVAR.InterfaceOptions)
	TEMPLATE.TextButton(GVAR.InterfaceOptions.GUI, L["Open GUI"], 1)
	GVAR.InterfaceOptions.GUI:SetWidth(150)
	GVAR.InterfaceOptions.GUI:SetHeight(22)
	GVAR.InterfaceOptions.GUI:SetPoint("TOPLEFT", GVAR.InterfaceOptions.Title, "BOTTOMLEFT", 0, -10)
	GVAR.InterfaceOptions.GUI:SetScript("OnClick", function()
		InterfaceOptionsFrame_Show()
		HideUIPanel(GameMenuFrame)
		CharScanner:Frame_Toggle(GVAR.MainFrame)
	end)

	GVAR.InterfaceOptions.SlashCommandText = GVAR.InterfaceOptions:CreateFontString(nil, "ARTWORK", fonts.GameFontWhiteSmall.name)
	GVAR.InterfaceOptions.SlashCommandText:SetPoint("LEFT", GVAR.InterfaceOptions.GUI, "RIGHT", 10, 0)
	GVAR.InterfaceOptions.SlashCommandText:SetText("(|cff"..color.LightYellow.hex.."/cs|r - |cff"..color.LightYellow.hex.."/scan|r - |cff"..color.LightYellow.hex.."/charscanner|r)")
	GVAR.InterfaceOptions.SlashCommandText:SetNonSpaceWrap(true)

	GVAR.InterfaceOptions.OPTIONS = CreateFrame("Button", nil, CharScanner_InterfaceOptions)
	TEMPLATE.TextButton(GVAR.InterfaceOptions.OPTIONS, L["Open Options"], 1)
	GVAR.InterfaceOptions.OPTIONS:SetPoint("TOPLEFT", GVAR.InterfaceOptions.GUI, "BOTTOMLEFT", 0, -10)
	GVAR.InterfaceOptions.OPTIONS:SetWidth(150)
	GVAR.InterfaceOptions.OPTIONS:SetHeight(22)
	GVAR.InterfaceOptions.OPTIONS:SetScript("OnClick", function()
		InterfaceOptionsFrame_Show()
		HideUIPanel(GameMenuFrame)
		CharScanner:Frame_Toggle(GVAR.OptionsFrame)
	end)

	InterfaceOptions_AddCategory(GVAR.InterfaceOptions)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
-- -----------------------------------------------------------------------------
function CharScanner:InitializeFrame()
	-- initial achievement & statistics collapse menu appearance
	for i = 1, #DefaultPvEStatsViewData do
		CharScanner:NodePvE_Click("PvE", DefaultPvEStatsViewData[i].lvl,
		                      DefaultPvEStatsViewData[i].name1, DefaultPvEStatsViewData[i].index1,
		                      DefaultPvEStatsViewData[i].name2, DefaultPvEStatsViewData[i].index2,
		                      DefaultPvEStatsViewData[i].name3, DefaultPvEStatsViewData[i].index3)
	end
	for i = 1, #DefaultPvPStatsViewData do
		CharScanner:NodePvP_Click("PvP", DefaultPvPStatsViewData[i].lvl,
		                      DefaultPvPStatsViewData[i].name1, DefaultPvPStatsViewData[i].index1,
		                      DefaultPvPStatsViewData[i].name2, DefaultPvPStatsViewData[i].index2,
		                      DefaultPvPStatsViewData[i].name3, DefaultPvPStatsViewData[i].index3, true)
	end
	-- initial stats height
	local height = (GUI_ABKsearch_curRows*sizeGeneral.BarHeight) + sizeStatFrame.OffsetTopSearch + sizeGeneral.Offset -- (savedRows*12) + 16 + 22 + 5
	GVAR.MainFrame.Stats:SetHeight(sizeGeneral.TabFrameHeight + height) -- 20 + height
	GVAR.MainFrame.StatsPvE:SetHeight(height) -- height
	GVAR.MainFrame.StatsPvP:SetHeight(height) -- height
	ScrollFrameUpdate()
	-- initialize PvE/PvP tab
	CharScanner:PvEPvPSwitch_OnClick("PvE")
	-- initialize sort buttons: History/Compare, Achievement Detail, ABKsearch
	CharScanner:History_SortButtonUpdate()
	CharScanner:AchievementDetail_SortButtonUpdate()
	CharScanner:ABKsearch_SortButtonUpdate()
	-- initialize ScanButton (force user to lock)
	if CharScanner_Options.scanButton and not CharScanner_Options.scanButtonLock then
		CharScanner:Frame_Toggle(GVAR.ScanButtonFrame, 1)
	end
	--CharScanner:ScanFrameToggle(1) -- MENUDEV
end
-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
function CharScanner:Frame_Toggle(frame, show)
	if show then
		frame:Show()
	else
		if frame:IsShown() then
			frame:Hide()
		else
			frame:Show()
		end
	end
end

function CharScanner:Frame_SetupPosition(frameName)
	if CharScanner_Options.pos and CharScanner_Options.pos[frameName.."_posX"] then
		_G[frameName]:ClearAllPoints()
		_G[frameName]:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", CharScanner_Options.pos[frameName.."_posX"], CharScanner_Options.pos[frameName.."_posY"])
	else
		if frameName == "CharScanner_ScanButtonFrame" then
			_G[frameName]:ClearAllPoints()
			_G[frameName]:SetPoint("TOP", UIParent, "TOP", 0, -100)
		elseif frameName == "CharScanner_MainFrame" then
			_G[frameName]:ClearAllPoints()
			_G[frameName]:SetPoint("TOPRIGHT", UIParent, "CENTER", -20, 150)
		elseif frameName == "CharScanner_OptionsFrame" then
			_G[frameName]:ClearAllPoints()
			_G[frameName]:SetPoint("TOPLEFT", UIParent, "CENTER", 20, 150)
		else
			_G[frameName]:ClearAllPoints()
			_G[frameName]:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
		end
	end
end

function CharScanner:Frame_SavePosition(frameName)
	if not CharScanner_Options.pos then CharScanner_Options.pos = {} end
	CharScanner_Options.pos[frameName.."_posX"] = _G[frameName]:GetLeft()
	CharScanner_Options.pos[frameName.."_posY"] = _G[frameName]:GetTop()
end

function CharScanner:Frame_MoverInfo(frame, action, where)
	if not action then
		frame.MoverInfoTop:Hide()
		frame.MoverInfoBottom:Hide()
		return
	end

	if action == "MOVE" then
		frame.MoverInfoTopText:SetText(L["click & move"])
		frame.MoverInfoBottomText:SetText(L["click & move"])
	else
		frame.MoverInfoTopText:SetText(L["click & resize"])
		frame.MoverInfoBottomText:SetText(L["click & resize"])
	end

	if where == "TOP" then
		frame.MoverInfoTop:Show()
	else
		frame.MoverInfoBottom:Show()
	end
end
-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
function CharScanner:ScanFrameShow()
	GVAR.MainFrame.Base:Show()
	GVAR.MainFrame.Talent:Show()
	GVAR.MainFrame.Equipment:Show()
	GVAR.MainFrame.Stats:Show()
	GVAR.MainFrame.Summary:Show()
	GVAR.MainFrame.Status:Show()
	GVAR.MainFrame.Headline.Collapse.Normal:SetTexCoord(unpack(Textures.Collapse.coords))
	GVAR.MainFrame.Headline.Collapse.Push:SetTexCoord(unpack(Textures.Collapse.coords))
	if NumScannedChars == 0 then
		TEMPLATE.DisableIconButton(GVAR.MainFrame.Headline.Collapse, 1)
		TEMPLATE.DisableIconButton(GVAR.MainFrame.Headline.History)
		TEMPLATE.DisableIconButton(GVAR.MainFrame.Headline.Compare)
		HistoryFrameIsShown = false
		CompareFrameIsShown = false
	else
		TEMPLATE.EnableIconButton(GVAR.MainFrame.Headline.Collapse, 1)
		TEMPLATE.EnableIconButton(GVAR.MainFrame.Headline.History)
		TEMPLATE.EnableIconButton(GVAR.MainFrame.Headline.Compare)
	end
end

function CharScanner:ScanFrameHide()
	GVAR.MainFrame.Base:Hide()
	GVAR.MainFrame.Talent:Hide()
	GVAR.MainFrame.Equipment:Hide()
	GVAR.MainFrame.Stats:Hide()
	GVAR.MainFrame.Summary:Hide()
	GVAR.MainFrame.Status:Hide()
	GVAR.HistoryFrame:Hide()
	GVAR.MainFrame.Headline.Collapse.Background:SetTexture(color.ButtonBG.rgba[1], color.ButtonBG.rgba[2], color.ButtonBG.rgba[3], color.ButtonBG.rgba[4])
	TEMPLATE.DisableIconButton(GVAR.MainFrame.Headline.History, 1)
	TEMPLATE.DisableIconButton(GVAR.MainFrame.Headline.Compare, 1)
	GVAR.MainFrame.Headline.Collapse.Normal:SetTexCoord(unpack(Textures.Expand.coords))
	GVAR.MainFrame.Headline.Collapse.Push:SetTexCoord(unpack(Textures.Expand.coords))
	HistoryFrameIsShown = false
	CompareFrameIsShown = false
	if NumScannedChars == 0 then
		TEMPLATE.DisableIconButton(GVAR.MainFrame.Headline.Collapse)
	else
		TEMPLATE.EnableIconButton(GVAR.MainFrame.Headline.Collapse)
	end
end

function CharScanner:ScanFrameToggle(show, hide)
	if show then
		CharScanner:ScanFrameShow()
		CharScanner:MainFrame_CalculateHeight()--MHIGH
		return
	end
	if hide then
		CharScanner:ScanFrameHide()
		CharScanner:MainFrame_CalculateHeight(1)--MHIGH
		return
	end
	if GVAR.MainFrame.Base:IsShown() then
		CharScanner:ScanFrameHide()
		CharScanner:MainFrame_CalculateHeight(1)--MHIGH
	else
		CharScanner:ScanFrameShow()
		CharScanner:MainFrame_CalculateHeight()--MHIGH
	end
end
-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
function CharScanner:StatusFrame_CheckPosition()
	if GVAR.MainFrame.Base:IsShown() then
		GVAR.MainFrame.Status:SetPoint("TOPLEFT", GVAR.MainFrame.Summary, "BOTTOMLEFT", 0, sizeGeneral.Border)
	else
		GVAR.MainFrame.Status:SetPoint("TOPLEFT", GVAR.MainFrame.Headline, "BOTTOMLEFT", 0, sizeGeneral.Border)
	end
end
-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
function CharScanner:DeleteButtonToggle(show)
	if show then
		TEMPLATE.EnableIconButton(GVAR.MainFrame.Headline.Delete, nil, 1)
	else
		TEMPLATE.DisableIconButton(GVAR.MainFrame.Headline.Delete, nil, 1)
	end
end
-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
function CharScanner:SingleScanShow()
	TEMPLATE.EnableIconButton(GVAR.MainFrame.Headline.SingleScan)
	TEMPLATE.EnableIconButton(GVAR.ScanButtonFrame.SingleScan)
	GVAR.MiniOptionsFrame.SingleScan:Enable()
	GVAR.MiniOptionsFrame.SingleScan.Text:SetTextColor(color.Yellow.rgba[1], color.Yellow.rgba[2], color.Yellow.rgba[3], color.Yellow.rgba[4])
end

function CharScanner:SingleScanHide()
	TEMPLATE.DisableIconButton(GVAR.MainFrame.Headline.SingleScan)
	TEMPLATE.DisableIconButton(GVAR.ScanButtonFrame.SingleScan)
	GVAR.MiniOptionsFrame.SingleScan:Disable()
	GVAR.MiniOptionsFrame.SingleScan.Text:SetTextColor(color.Grey.rgba[1], color.Grey.rgba[2], color.Grey.rgba[3], color.Grey.rgba[4])
end
-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
function CharScanner:GroupScanToggle()
	if not GroupScanIsActive then
		if inPartyOrRaid then
			CharScanner:GroupScanShow()
		else
			CharScanner:GroupScanHide()
		end
	end
end

function CharScanner:GroupScanShow()
	if inPartyOrRaid then
		TEMPLATE.EnableIconButton(GVAR.ScanButtonFrame.GroupScanAll)
		TEMPLATE.EnableIconButton(GVAR.MainFrame.Headline.GroupScanAll)
		GVAR.MiniOptionsFrame.GroupScanAll:Enable()
		GVAR.MiniOptionsFrame.GroupScanAll.Text:SetTextColor(color.Yellow.rgba[1], color.Yellow.rgba[2], color.Yellow.rgba[3], color.Yellow.rgba[4])
		if GroupMissingNum > 0 then
			TEMPLATE.EnableIconButton(GVAR.ScanButtonFrame.GroupScanMissing)
			TEMPLATE.EnableIconButton(GVAR.MainFrame.Headline.GroupScanMissing)
			GVAR.MiniOptionsFrame.GroupScanMissing:Enable()
			GVAR.MiniOptionsFrame.GroupScanMissing.Text:SetTextColor(color.Yellow.rgba[1], color.Yellow.rgba[2], color.Yellow.rgba[3], color.Yellow.rgba[4])
		end
	end
end

function CharScanner:GroupScanHide()
	TEMPLATE.DisableIconButton(GVAR.ScanButtonFrame.GroupScanAll)
	TEMPLATE.DisableIconButton(GVAR.ScanButtonFrame.GroupScanMissing)
	TEMPLATE.DisableIconButton(GVAR.MainFrame.Headline.GroupScanAll)
	GVAR.MiniOptionsFrame.GroupScanAll:Disable()
	GVAR.MiniOptionsFrame.GroupScanAll.Text:SetTextColor(color.Grey.rgba[1], color.Grey.rgba[2], color.Grey.rgba[3], color.Grey.rgba[4])
	TEMPLATE.DisableIconButton(GVAR.MainFrame.Headline.GroupScanMissing)
	GVAR.MiniOptionsFrame.GroupScanMissing:Disable()
	GVAR.MiniOptionsFrame.GroupScanMissing.Text:SetTextColor(color.Grey.rgba[1], color.Grey.rgba[2], color.Grey.rgba[3], color.Grey.rgba[4])
end

function CharScanner:GroupMissingNumUpdate()
	if inPartyOrRaid then
		if GroupMissingNum > 0 then
			GVAR.ScanButtonFrame.GroupScanMissingNum:Show()
			GVAR.ScanButtonFrame.GroupScanMissingNumText:SetText(GroupMissingNum)
			GVAR.MainFrame.Headline.GroupScanMissingNum:Show()
			GVAR.MainFrame.Headline.GroupScanMissingNumText:SetText(GroupMissingNum)
		else
			GVAR.ScanButtonFrame.GroupScanMissingNum:Hide()
			GVAR.MainFrame.Headline.GroupScanMissingNum:Hide()
		end
	else
		GVAR.ScanButtonFrame.GroupScanMissingNum:Hide()
		GVAR.MainFrame.Headline.GroupScanMissingNum:Hide()
	end
end
-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
function CharScanner:HistoryFrameToggle()
	if GVAR.HistoryFrame:IsShown() then
		if CompareFrameIsShown then
			HistoryFrameIsShown = true
			CompareFrameIsShown = false
			CharScanner:HistoryCompareReorganize()
		else
			HistoryFrameIsShown = false
			GVAR.MainFrame.Headline.History.Background:SetTexture(color.ButtonBG.rgba[1], color.ButtonBG.rgba[2], color.ButtonBG.rgba[3], color.ButtonBG.rgba[4])
			GVAR.MainFrame.Headline.Compare.Background:SetTexture(color.ButtonBG.rgba[1], color.ButtonBG.rgba[2], color.ButtonBG.rgba[3], color.ButtonBG.rgba[4])
			GVAR.HistoryFrame:Hide()
		end
	else
		HistoryFrameIsShown = true
		GVAR.HistoryFrame:Show()
		CharScanner:HistoryCompareReorganize()
	end
end

function CharScanner:HistoryFrameShow()
	HistoryFrameIsShown = true
	CompareFrameIsShown = false
	GVAR.HistoryFrame:Show()
	CharScanner:HistoryCompareReorganize()
end

function CharScanner:CompareFrameToggle()
	if GVAR.HistoryFrame:IsShown() then
		if HistoryFrameIsShown then
			CompareFrameIsShown = true
			HistoryFrameIsShown = false
			CharScanner:HistoryCompareReorganize(true)
		else
			CompareFrameIsShown = false
			GVAR.MainFrame.Headline.History.Background:SetTexture(color.ButtonBG.rgba[1], color.ButtonBG.rgba[2], color.ButtonBG.rgba[3], color.ButtonBG.rgba[4])
			GVAR.MainFrame.Headline.Compare.Background:SetTexture(color.ButtonBG.rgba[1], color.ButtonBG.rgba[2], color.ButtonBG.rgba[3], color.ButtonBG.rgba[4])
			GVAR.HistoryFrame:Hide()
		end
	else
		CompareFrameIsShown = true
		--GVAR.HistoryFrame:Show()
		CharScanner:HistoryCompareReorganize(true)
	end
end

function CharScanner:CompareFrameShow()
	HistoryFrameIsShown = false
	CompareFrameIsShown = true
	--GVAR.HistoryFrame:Show()
	CharScanner:HistoryCompareReorganize(true)
end

function CharScanner:HistoryCompareReorganize(show)
	if show then
		GVAR.MainFrame.Headline.History.Background:SetTexture(color.ButtonBG.rgba[1], color.ButtonBG.rgba[2], color.ButtonBG.rgba[3], color.ButtonBG.rgba[4])
		GVAR.MainFrame.Headline.Compare.Background:SetTexture(color.ButtonBGActive.rgba[1], color.ButtonBGActive.rgba[2], color.ButtonBGActive.rgba[3], color.ButtonBGActive.rgba[4])
		GVAR.HistoryFrame:SetWidth(sizeCompareFrame.CompareWidth)
		GVAR.HistoryFrame.Sort:SetWidth(sizeCompareFrame.CompareWidth)
		GVAR.HistoryFrame.Status:SetWidth(sizeCompareFrame.CompareWidth)
		GVAR.HistoryFrame.GroupStatusTabTex:SetWidth(sizeCompareFrame.CompareWidth)
		for i = 1, GUI_history_maxRows do
			GVAR.HistoryFrame.ScrollFrameButton[i]:SetWidth(sizeCompareFrame.CompareButtonWidth)
			GVAR.HistoryFrame.ScrollFrameButton[i].CompareFrame:Show()
		end
		if not GVAR.HistoryFrame:IsShown() then
			GVAR.HistoryFrame:Hide()
			GVAR.HistoryFrame:Show()
		end
		GVAR.HistoryFrame.Sort.CompareFrame:Show()
	else
		GVAR.MainFrame.Headline.History.Background:SetTexture(color.ButtonBGActive.rgba[1], color.ButtonBGActive.rgba[2], color.ButtonBGActive.rgba[3], color.ButtonBGActive.rgba[4])
		GVAR.MainFrame.Headline.Compare.Background:SetTexture(color.ButtonBG.rgba[1], color.ButtonBG.rgba[2], color.ButtonBG.rgba[3], color.ButtonBG.rgba[4])
		GVAR.HistoryFrame:SetWidth(sizeCompareFrame.HistoryWidth)
		GVAR.HistoryFrame.Sort:SetWidth(sizeCompareFrame.HistoryWidth)
		GVAR.HistoryFrame.Status:SetWidth(sizeCompareFrame.HistoryWidth)
		GVAR.HistoryFrame.GroupStatusTabTex:SetWidth(sizeCompareFrame.HistoryWidth)
		for i = 1, GUI_history_maxRows do
			GVAR.HistoryFrame.ScrollFrameButton[i]:SetWidth(sizeCompareFrame.HistoryButtonWidth)
			GVAR.HistoryFrame.ScrollFrameButton[i].CompareFrame:Hide()
		end
		GVAR.HistoryFrame.Sort.CompareFrame:Hide()
	end
end

function CharScanner:HistoryCompareFrameHide()
	HistoryFrameIsShown = false
	CompareFrameIsShown = false
	GVAR.MainFrame.Headline.History.Background:SetTexture(color.ButtonBG.rgba[1], color.ButtonBG.rgba[2], color.ButtonBG.rgba[3], color.ButtonBG.rgba[4])
	GVAR.MainFrame.Headline.Compare.Background:SetTexture(color.ButtonBG.rgba[1], color.ButtonBG.rgba[2], color.ButtonBG.rgba[3], color.ButtonBG.rgba[4])
	GVAR.HistoryFrame:Hide()
end
-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
function CharScanner:PvEPvPSwitch_OnClick(mode)
	if mode == "PvE" then
		TEMPLATE.SetTabButton(GVAR.MainFrame.PvEPvPSwitch.Button1, true)
		TEMPLATE.SetTabButton(GVAR.MainFrame.PvEPvPSwitch.Button2, false)

		GVAR.MainFrame.StatsPvE:Show()
		GVAR.MainFrame.StatsPvP:Hide()
		GVAR.MainFrame.Summary1PvE:Show()
		GVAR.MainFrame.Summary2PvE:Show()
		GVAR.MainFrame.Summary1PvP:Hide()
		GVAR.MainFrame.Summary2PvP:Hide()
		AchievementDetailFrameMode = "PvE"
		if GVAR.MainFrame.StatsPvE.Title.AchievementDetailShow then
			CharScanner:AchievementDetail_Update("PvE", GUI_achievementdetail_L1, GUI_achievementdetail_L2, GUI_achievementdetail_L3)
		else
			CharScanner:AchievementDetail_Hide(AchievementDetailFrameMode)
		end
	else
		TEMPLATE.SetTabButton(GVAR.MainFrame.PvEPvPSwitch.Button1, false)
		TEMPLATE.SetTabButton(GVAR.MainFrame.PvEPvPSwitch.Button2, true)

		GVAR.MainFrame.StatsPvE:Hide()
		GVAR.MainFrame.StatsPvP:Show()
		GVAR.MainFrame.Summary1PvE:Hide()
		GVAR.MainFrame.Summary2PvE:Hide()
		GVAR.MainFrame.Summary1PvP:Show()
		GVAR.MainFrame.Summary2PvP:Show()
		AchievementDetailFrameMode = "PvP"
		if GVAR.MainFrame.StatsPvP.Title.AchievementDetailShow then
			CharScanner:AchievementDetail_Update("PvP", GUI_PvPachievementdetail_L1, GUI_PvPachievementdetail_L2, GUI_PvPachievementdetail_L3)
		else
			CharScanner:AchievementDetail_Hide(AchievementDetailFrameMode)
		end
	end
end
-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
function CharScanner:AchievementDetail_OnEnter(mode, button, l1, l2, l3)
	if LatestScanIsComplete then
		local buttonref = GVAR.MainFrame.StatsPvE.Title
		if mode == "PvP" then
			buttonref = GVAR.MainFrame.StatsPvP.Title
		end
		if buttonref.AchievementClicked ~= button then
			CharScanner:AchievementDetail_Highlight(button, true)
			CharScanner:AchievementDetail_Update(mode, l1, l2, l3)
		end
	end
end

function CharScanner:AchievementDetail_OnLeave(mode, button, l1, l2, l3)
	local buttonref = GVAR.MainFrame.StatsPvE.Title
	if mode == "PvP" then
		buttonref = GVAR.MainFrame.StatsPvP.Title
	end
	if buttonref.AchievementClicked ~= button then -- XFIX
		CharScanner:AchievementDetail_Highlight(button, false) -- XFIX
	end -- XFIX
	if LatestScanIsComplete then
		if buttonref.AchievementClicked ~= button then
			--CharScanner:AchievementDetail_Highlight(button, false) -- XFIX
			GVAR.AchievementDetailFrame:Hide()
			table_wipe(GUI_achievementdetail_data) -- TEST TODO
		end
		if buttonref.AchievementClicked ~= "" then
			CharScanner:AchievementDetail_Update(mode, buttonref.AchievementL1, buttonref.AchievementL2, buttonref.AchievementL3)
		end
	end
end

function CharScanner:AchievementDetail_OnClick(mode, button, l1, l2, l3)
	if LatestScanIsComplete then
		local buttonref = GVAR.MainFrame.StatsPvE.Title
		if mode == "PvP" then
			buttonref = GVAR.MainFrame.StatsPvP.Title
		end
		if buttonref.AchievementClicked == "" then
			buttonref.AchievementClicked = button
			buttonref.AchievementL1 = l1
			buttonref.AchievementL2 = l2
			buttonref.AchievementL3 = l3
			CharScanner:AchievementDetail_Highlight(button, true)
			buttonref.AchievementDetailShow = true
			CharScanner:AchievementDetail_Update(mode, l1, l2, l3)
		elseif buttonref.AchievementClicked == button then
			buttonref.AchievementClicked = ""
			buttonref.AchievementL1 = nil
			buttonref.AchievementL2 = nil
			buttonref.AchievementL3 = nil
			CharScanner:AchievementDetail_Highlight(button, false)
			buttonref.AchievementDetailShow = false
			GVAR.AchievementDetailFrame:Hide()
		elseif  buttonref.AchievementClicked ~= button then
			CharScanner:AchievementDetail_Highlight(buttonref.AchievementClicked, false)
			buttonref.AchievementClicked = button
			buttonref.AchievementL1 = l1
			buttonref.AchievementL2 = l2
			buttonref.AchievementL3 = l3
			CharScanner:AchievementDetail_Highlight(button, true)
			buttonref.AchievementDetailShow = true
			CharScanner:AchievementDetail_Update(mode, l1, l2, l3)
		end
	end
end

function CharScanner:AchievementDetail_Highlight(button, show)
	if show then
		button.AchievementDetailIcon:SetAlpha(1)
		button.AchievementDetailHighlight:SetTexture(color.ButtonHighlight.rgba[1], color.ButtonHighlight.rgba[2], color.ButtonHighlight.rgba[3], color.ButtonHighlight.rgba[4])
	else
		button.AchievementDetailIcon:SetAlpha(0.25)
		button.AchievementDetailHighlight:SetTexture(0, 0, 0, 0)
	end
end

--function CharScanner:AchievementDetail_Show(mode)
--	GVAR.AchievementDetailFrame:Show()
--end

function CharScanner:AchievementDetail_Hide(mode)
	local buttonref = GVAR.MainFrame.StatsPvE.Title
	if mode == "PvP" then
		buttonref = GVAR.MainFrame.StatsPvP.Title
	end
	if buttonref.AchievementClicked ~= "" then
		CharScanner:AchievementDetail_Highlight(buttonref.AchievementClicked, false)
		buttonref.AchievementClicked = ""
	end
	buttonref.AchievementDetailShow = false

	GVAR.AchievementDetailFrame:Hide()
end

function CharScanner:AchievementDetail_HideAll()
	if GVAR.MainFrame.StatsPvE.Title.AchievementClicked ~= "" then
		CharScanner:AchievementDetail_Highlight(GVAR.MainFrame.StatsPvE.Title.AchievementClicked, false)
		GVAR.MainFrame.StatsPvE.Title.AchievementClicked = ""
	end
	GVAR.MainFrame.StatsPvE.Title.AchievementDetailShow = false

	if GVAR.MainFrame.StatsPvP.Title.AchievementClicked ~= "" then
		CharScanner:AchievementDetail_Highlight(GVAR.MainFrame.StatsPvP.Title.AchievementClicked, false)
		GVAR.MainFrame.StatsPvP.Title.AchievementClicked = ""
	end
	GVAR.MainFrame.StatsPvP.Title.AchievementDetailShow = false

	GVAR.AchievementDetailFrame:Hide()
end
-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
function CharScanner:GearFrame_OnClick()
	if GearFrameIsShown then
		GVAR.GearFrame:Hide()
		GVAR.MainFrame.Equipment.Collapse.Background:SetTexture(color.ButtonBG.rgba[1], color.ButtonBG.rgba[2], color.ButtonBG.rgba[3], color.ButtonBG.rgba[4])
		GVAR.MainFrame.Equipment.Collapse.Normal:SetTexCoord(unpack(Textures.RightSide.coords))
		GVAR.MainFrame.Equipment.Collapse.Push:SetTexCoord(unpack(Textures.RightSide.coords))
		GearFrameIsShown = false
	else
		GVAR.GearFrame:Show()
		GVAR.MainFrame.Equipment.Collapse.Background:SetTexture(color.ButtonBGActive.rgba[1], color.ButtonBGActive.rgba[2], color.ButtonBGActive.rgba[3], color.ButtonBGActive.rgba[4])
		GVAR.MainFrame.Equipment.Collapse.Normal:SetTexCoord(unpack(Textures.RightSide.coordsOff))
		GVAR.MainFrame.Equipment.Collapse.Push:SetTexCoord(unpack(Textures.RightSide.coordsOff))
		GearFrameIsShown = true
	end
end

function CharScanner:GearFrame_OnEnter()
	if not GearFrameIsShown then
		GVAR.GearFrame:Show()
	else
		GVAR.GearFrame:Hide() -- for SetToplevel
		GVAR.GearFrame:Show()
	end
end

function CharScanner:GearFrame_OnLeave()
	if not GearFrameIsShown then
		GVAR.GearFrame:Hide()
	end
end

-- gearframe scripts START
function CharScanner:GearFrame_IconButton_OnEnter(i, guid)
	if not SCAN[ guid ].gear[i] then return end
	if not SCAN[ guid ].gear[i].iLink then return end
	GVAR.GearFrame_Slot[i].IconButtonBG:SetTexture(1, 1, 0.49, 1)
	GameTooltip:SetOwner(GVAR.GearFrame_Slot[i], "ANCHOR_BOTTOMLEFT", -sizeGeneral.Offset, Textures.GearIconTexture.qsize)
	GameTooltip:SetHyperlink( SCAN[ guid ].gear[i].iLink )
	GVAR.GearFrame_Slot[i].UpdateTooltip = GearTooltipHandler
end
function CharScanner:GearFrame_IconButton_OnLeave(i, guid)
	if not SCAN[ guid ].gear[i] then return end
	if not SCAN[ guid ].gear[i].iLink then return end
	GVAR.GearFrame_Slot[i].IconButtonBG:SetTexture(0.2, 0.2, 0.2, 1)
	GameTooltip:Hide()
	ResetCursor()
	GVAR.GearFrame_Slot[i].comparing = false
end
function CharScanner:GearFrame_IconButton_OnClick(i, guid)
	if not SCAN[ guid ].gear[i] then return end
	if not SCAN[ guid ].gear[i].iLink then return end
	HandleModifiedItemClick( SCAN[ guid ].gear[i].iLink )
end

function CharScanner:GearFrame_EnchButton_OnEnter(i, guid)
	if not SCAN[ guid ].gear[i] then return end
	if not SCAN[ guid ].gear[i].iEnch then return end
	GVAR.GearFrame_Slot[i].UpdateTooltip = nil
	GVAR.GearFrame_Slot[i].EnchButtonBG:SetTexture(1, 1, 0.49, 1)
	GameTooltip:SetOwner(GVAR.GearFrame_Slot[i], "ANCHOR_BOTTOMLEFT", -sizeGeneral.Offset, Textures.GearIconTexture.qsize)
	if SCAN[ guid ].gear[i].iEnch > 0 then
		local aid, hid = CharScanner_GearStats_Enchantment( SCAN[ guid ].gear[i].iEnch )
		if aid then
			if hid then
				if SCAN[ guid ].faction == "Horde" then
					aid = hid
				end
			end
			if aid > 0 then -- item
				GameTooltip:SetHyperlink("item:"..aid)
			else -- spell
				GameTooltip:SetSpellByID(-aid)
			end
		else
			GameTooltip:ClearLines()
			GameTooltip:AddLine("|TInterface\\DialogFrame\\UI-Dialog-Icon-AlertNew:24|t "..UNKNOWN, 1, 0, 0)
			GameTooltip:AddLine("enchantID: "..SCAN[ guid ].gear[i].iEnch)
			GameTooltip:AddLine("TODO")
			GameTooltip:Show()
		end
	end
end
function CharScanner:GearFrame_EnchButton_OnLeave(i, guid)
	if not SCAN[ guid ].gear[i] then return end
	if not SCAN[ guid ].gear[i].iEnch then return end
	GVAR.GearFrame_Slot[i].EnchButtonBG:SetTexture(0.2, 0.2, 0.2, 1)
	GameTooltip:Hide()
end
function CharScanner:GearFrame_EnchButton_OnClick(i, guid)
	if not SCAN[ guid ].gear[i] then return end
	if not SCAN[ guid ].gear[i].iEnch then return end
	local link
	if SCAN[ guid ].gear[i].iEnch > 0 then
		local aid, hid = CharScanner_GearStats_Enchantment( SCAN[ guid ].gear[i].iEnch )
		if aid then
			if hid then
				if SCAN[ guid ].faction == "Horde" then
					aid = hid
				end
			end
			if aid > 0 then -- item
				link = select(2, GetItemInfo(aid))
			else -- spell
				local name = GetSpellInfo(-aid)
				if name then
					link = "|cff71d5ff|Hspell:"..(-aid).."|h["..name.."]|h|r"
				end
			end
		end
	end
	if link then
		HandleModifiedItemClick( link )
	end
end

function CharScanner:GearFrame_ReforgeButton_OnEnter(i, guid)
	if not SCAN[ guid ].gear[i] then return end
	if not SCAN[ guid ].gear[i].iRefoTxt then return end
	GVAR.GearFrame_Slot[i].UpdateTooltip = nil
	GVAR.GearFrame_Slot[i].ReforgeButtonBG:SetTexture(1, 1, 0.49, 1)
	GameTooltip:SetOwner(GVAR.GearFrame_Slot[i], "ANCHOR_BOTTOMLEFT", -sizeGeneral.Offset, Textures.GearIconTexture.qsize)
	GameTooltip:AddLine( SCAN[ guid ].gear[i].iRefoTxt )
	GameTooltip:Show()
end
function CharScanner:GearFrame_ReforgeButton_OnLeave(i, guid)
	if not SCAN[ guid ].gear[i] then return end
	if not SCAN[ guid ].gear[i].iRefoTxt then return end
	GVAR.GearFrame_Slot[i].ReforgeButtonBG:SetTexture(0.2, 0.2, 0.2, 1)
	GameTooltip:Hide()
end

function CharScanner:GearFrame_GemButton_OnEnter(gem, i, guid)
	if not SCAN[ guid ].gear[i] then return end
	if not SCAN[ guid ].gear[i]["iGem"..gem] then return end
	GVAR.GearFrame_Slot[i].UpdateTooltip = nil
	GVAR.GearFrame_Slot[i]["Gem"..gem.."ButtonBG"]:SetTexture(1, 1, 0.49, 1)
	GameTooltip:SetOwner(GVAR.GearFrame_Slot[i], "ANCHOR_BOTTOMLEFT", -sizeGeneral.Offset, Textures.GearIconTexture.qsize)
	GameTooltip:SetHyperlink("item:"..SCAN[ guid ].gear[i]["iGem"..gem])
end
function CharScanner:GearFrame_GemButton_OnLeave(gem, i, guid)
	if not SCAN[ guid ].gear[i] then return end
	if not SCAN[ guid ].gear[i]["iGem"..gem] then return end
	GVAR.GearFrame_Slot[i]["Gem"..gem.."ButtonBG"]:SetTexture(0.2, 0.2, 0.2, 1)
	GameTooltip:Hide()
end
function CharScanner:GearFrame_GemButton_OnClick(gem, i, guid)
	if not SCAN[ guid ].gear[i] then return end
	if not SCAN[ guid ].gear[i]["iGem"..gem] then return end
	local link = select(2, GetItemInfo(SCAN[ guid ].gear[i]["iGem"..gem]))
	if link then
		HandleModifiedItemClick( link )
	end
end
-- gearframe scripts END
-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
--[[
function CharScanner:BorderPolice(style, frame)
	--             T             --
	--        -----------        --
	--      L |         | R      --
	--        -----------        --
	--             B             --
	frame.Border:ClearAllPoints()
	if style == "TBL" then
	-- TBL:   -----------        --
	--        |                  --
	--        -----------        --
		frame.Border:SetPoint("TOPLEFT",     sizeGeneral.Border, -sizeGeneral.Border)
		frame.Border:SetPoint("BOTTOMLEFT",  sizeGeneral.Border, sizeGeneral.Border)
		frame.Border:SetPoint("TOPRIGHT",    0, -sizeGeneral.Border)
		frame.Border:SetPoint("BOTTOMRIGHT", 0, sizeGeneral.Border)
	elseif style == "TRBL" then
	-- TRBL:  -----------        --
	--        |         |        --
	--        -----------        --
		frame.Border:SetPoint("TOPLEFT",     sizeGeneral.Border, -sizeGeneral.Border)
		frame.Border:SetPoint("BOTTOMRIGHT", -sizeGeneral.Border, sizeGeneral.Border)
	elseif style == "TRB" then
	-- TRB:   -----------        --
	--                  |        --
	--        -----------        --
		frame.Border:SetPoint("TOPLEFT",     0, -sizeGeneral.Border)
		frame.Border:SetPoint("BOTTOMLEFT",  0, sizeGeneral.Border)
		frame.Border:SetPoint("TOPRIGHT",    -sizeGeneral.Border, -sizeGeneral.Border)
		frame.Border:SetPoint("BOTTOMRIGHT", -sizeGeneral.Border, sizeGeneral.Border)
	end
end
--]]
-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
function CharScanner:TalentTreeFrame_OnClick()
	GVAR.TalentTreeFrame.GroupFrame.Button2:EnableMouse(true) -- fast MouseOver Button2 hackfix
	if TalentTreeFrameIsShown then
		GVAR.TalentTreeFrame:Hide()
		GVAR.MainFrame.Talent.Collapse.Background:SetTexture(color.ButtonBG.rgba[1], color.ButtonBG.rgba[2], color.ButtonBG.rgba[3], color.ButtonBG.rgba[4])
		GVAR.MainFrame.Talent.Collapse.Normal:SetTexCoord(unpack(Textures.RightSide.coords))
		GVAR.MainFrame.Talent.Collapse.Push:SetTexCoord(unpack(Textures.RightSide.coords))
		TalentTreeFrameIsShown = false
	else
		GVAR.TalentTreeFrame:Show()
		GVAR.MainFrame.Talent.Collapse.Background:SetTexture(color.ButtonBGActive.rgba[1], color.ButtonBGActive.rgba[2], color.ButtonBGActive.rgba[3], color.ButtonBGActive.rgba[4])
		GVAR.MainFrame.Talent.Collapse.Normal:SetTexCoord(unpack(Textures.RightSide.coordsOff))
		GVAR.MainFrame.Talent.Collapse.Push:SetTexCoord(unpack(Textures.RightSide.coordsOff))
		TalentTreeFrameIsShown = true
	end
end

function CharScanner:TalentTreeFrame_OnEnter()
	if not TalentTreeFrameIsShown then
		GVAR.TalentTreeFrame:Show()
		GVAR.TalentTreeFrame.GroupFrame.Button2:EnableMouse(false) -- fast MouseOver Button2 hackfix
	else
		GVAR.TalentTreeFrame:Hide() -- for SetToplevel
		GVAR.TalentTreeFrame:Show()
	end
end

function CharScanner:TalentTreeFrame_OnLeave()
	if not TalentTreeFrameIsShown then
		GVAR.TalentTreeFrame:Hide()
		GVAR.TalentTreeFrame.GroupFrame.Button2:EnableMouse(true) -- fast MouseOver Button2 hackfix
	end
end
-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
function CharScanner:ProgressDisplay(scantype, progress, restriction)
	if progress then
		if scantype == "SINGLE" then
			GVAR.MainFrame.Status.ProgressText:SetPoint("TOP", GVAR.MainFrame.Status, "TOP", 0, -7)
			GVAR.MainFrame.Status.ProgressBackground:SetPoint("TOP", GVAR.MainFrame.Status, "TOP", 0, -7)
			GVAR.MainFrame.Status.ProgressButton:Hide()
		else
			GVAR.MainFrame.Status.ProgressText:SetPoint("TOP", GVAR.MainFrame.Status.ProgressButton, "BOTTOM", 0, -2)
			GVAR.MainFrame.Status.ProgressBackground:SetPoint("TOP", GVAR.MainFrame.Status.ProgressButton, "BOTTOM", 0, -2)
			GVAR.MainFrame.Status.ProgressButton:Show()
		end
		GVAR.MainFrame.Status.ProgressText:Show()
		GVAR.MainFrame.Status.ProgressBackground:Show()
		GVAR.MainFrame.Status.ProgressDone:Show()
		GVAR.MainFrame.Status.Text:Hide()
	else
		GVAR.MainFrame.Status.ProgressButton:Hide()
		GVAR.MainFrame.Status.ProgressText:Hide()
		GVAR.MainFrame.Status.ProgressBackground:Hide()
		GVAR.MainFrame.Status.ProgressDone:Hide()
		GVAR.MainFrame.Status.Text:Show()
	end
	if restriction then
		GVAR.MainFrame.Status.ThrottleText1:Show()
		GVAR.MainFrame.Status.ThrottleText2:Show()
		GVAR.MainFrame.Status.ThrottleBackground:Show()
		GVAR.MainFrame.Status.ThrottleDone:Show()
	else
		GVAR.MainFrame.Status.ThrottleText1:Hide()
		GVAR.MainFrame.Status.ThrottleText2:Hide()
		GVAR.MainFrame.Status.ThrottleBackground:Hide()
		GVAR.MainFrame.Status.ThrottleDone:Hide()
	end

	if scantype == "SINGLE" then
		GVAR.MainFrame.Status:SetHeight(sizeGeneral.StatusHeight)
	else
		if progress then
			GVAR.MainFrame.Status:SetHeight(46)
			if restriction then
				GVAR.MainFrame.Status:SetHeight(81)
			end
		else
			GVAR.MainFrame.Status:SetHeight(sizeGeneral.StatusHeight)
		end
	end
end
-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
function CharScanner:MiniOptionsToggle(button, pos)
	if button == "RightButton" then
		if GVAR.MiniOptionsFrame:IsShown() then
			if GVAR.MiniOptionsFrame.position and GVAR.MiniOptionsFrame.position ~= pos then
				if pos == 1 then
					local xpos, ypos = GetCursorPosition()
					GVAR.MiniOptionsFrame:ClearAllPoints()
					GVAR.MiniOptionsFrame:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", xpos, ypos)
				else
					GVAR.MiniOptionsFrame:ClearAllPoints()
					GVAR.MiniOptionsFrame:SetPoint("TOP", CharScanner_MinimapButton, "BOTTOM", 0, 0)	
				end
				GVAR.MiniOptionsFrame.position = pos
				MiniOptions_TimerStart = GetTime()
				return
			end

			GVAR.MiniOptionsFrame:Hide()
			GVAR.MiniOptionsFrame:SetScript("OnUpdate", nil)
		else
			if pos == 1 then
				local xpos, ypos = GetCursorPosition()
				GVAR.MiniOptionsFrame:ClearAllPoints()
				GVAR.MiniOptionsFrame:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", xpos, ypos)
			else
				GVAR.MiniOptionsFrame:ClearAllPoints()
				GVAR.MiniOptionsFrame:SetPoint("TOP", CharScanner_MinimapButton, "BOTTOM", 0, 0)	
			end

			GVAR.MiniOptionsFrame.position = pos

			GVAR.MiniOptionsFrame:Show()
			MiniOptions_TimerStart = GetTime()
			GVAR.MiniOptionsFrame:SetScript("OnUpdate", MiniOptionsTimer)
		end
	else
		GVAR.MiniOptionsFrame:Hide()
		GVAR.MiniOptionsFrame:SetScript("OnUpdate", nil)
		CharScanner:Frame_Toggle(GVAR.MainFrame)
	end
end
-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
function CharScanner:MainFrame_Hide()
	ResizeEnd() -- STOP resize
	PlaySound("igQuestListClose")
	TEMPLATE.EnableTextButton(GVAR.InterfaceOptions.GUI, 1)
	TEMPLATE.EnableTextButton(GVAR.OptionsFrame.OpenGUI, 1)
	TEMPLATE.EnableTextButton(GVAR.ScanButtonFrame.Mover.GUI, 1)
end

function CharScanner:MainFrame_Show()
	PlaySound("igQuestListOpen")
	TEMPLATE.DisableTextButton(GVAR.InterfaceOptions.GUI)
	TEMPLATE.DisableTextButton(GVAR.OptionsFrame.OpenGUI)
	TEMPLATE.DisableTextButton(GVAR.ScanButtonFrame.Mover.GUI)
	CharScanner:Frame_SetupPosition("CharScanner_MainFrame")
	--CharScanner:History_Update()
end
-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
function CharScanner:BossSearch()
	BossSearchQuery = GVAR.MainFrame.StatsPvE.SearchBox.EditBox:GetText()
	if BossSearchQuery == "" then
		BossSearchQuery = nil
	end
	if BossSearchQuery then
		GVAR.MainFrame.StatsPvE.SearchBox.EditBoxClear:Enable()
	else
		GVAR.MainFrame.StatsPvE.SearchBox.EditBoxClear:Disable()
	end
	CharScanner:ABKsearch_Update()
end

function CharScanner:BossSearchClear()
	BossSearchQuery = nil
	GVAR.MainFrame.StatsPvE.SearchBox.EditBox:SetText("")
	GVAR.MainFrame.StatsPvE.SearchBox.EditBox:ClearFocus()
	CharScanner:ABKsearch_Update()
end
-- -----------------------------------------------------------------------------
-- ---------------------------------------------------------------------------------------------------------------------


-- ---------------------------------------------------------------------------------------------------------------------
function CharScanner:MainFrame_CalculateHeight(basic)--MHIGH
	if basic then
		GVAR.MainFrame:SetHeight(sizeGeneral.MainHeight)
		GVAR.MainFrame:SetClampRectInsets(0, 0, 0, 0)
		CharScanner:Frame_SetupPosition("CharScanner_MainFrame")
	else
		local height =
		sizeGeneral.MainHeight + -- Headline
		(sizeBaseCharFrame_BaseHeight - sizeGeneral.Border) + -- Base Char Data
		(sizeTalentFrame.BaseHeight - sizeGeneral.Border) + -- Talent
		(GVAR.MainFrame.Equipment:GetHeight() - sizeGeneral.Border) + -- Equipment
		(GVAR.MainFrame.Stats:GetHeight() - sizeGeneral.Border) + -- Stats
		(sizeSummaryFrame_BaseHeight - sizeGeneral.Border) + -- Summary
		(sizeGeneral.StatusHeight - sizeGeneral.Border); -- Status

		CharScanner:Frame_SetupPosition("CharScanner_MainFrame")

		GVAR.MainFrame:SetHeight(height)
		GVAR.MainFrame:SetClampRectInsets(0, 0, 0, (height-sizeGeneral.MainHeight))
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function CharScanner:CreateMainFrame()
	GVAR.MainFrame = CreateFrame("Frame", "CharScanner_MainFrame", UIParent)
	GVAR.MainFrame:EnableMouse(true)
	GVAR.MainFrame:SetMovable(true)
	GVAR.MainFrame:SetToplevel(true)
	GVAR.MainFrame:SetClampedToScreen(true)
	GVAR.MainFrame:SetWidth(sizeGeneral.MainWidth)
	GVAR.MainFrame:Hide()
	CharScanner:MainFrame_CalculateHeight(1)--MHIGH
	-- GVAR.MainFrame:SetScript at end of function



	-- Headline Frame START
	GVAR.MainFrame.Headline = CreateFrame("Frame", nil, GVAR.MainFrame)
	TEMPLATE.BorderTRBL(GVAR.MainFrame.Headline)
	GVAR.MainFrame.Headline:SetWidth(sizeGeneral.MainWidth)
	GVAR.MainFrame.Headline:SetHeight(sizeGeneral.MainHeight)
	GVAR.MainFrame.Headline:SetPoint("TOPLEFT", GVAR.MainFrame, "TOPLEFT", 0, 0)
	GVAR.MainFrame.Headline:EnableMouse(true)

	GVAR.MainFrame.Headline.SingleScan = CreateFrame("Button", nil, GVAR.MainFrame.Headline)
	TEMPLATE.IconButton(GVAR.MainFrame.Headline.SingleScan, 7)
	GVAR.MainFrame.Headline.SingleScan:SetPoint("LEFT", GVAR.MainFrame.Headline, "LEFT", sizeGeneral.Offset, 0)
	GVAR.MainFrame.Headline.SingleScan:SetScript("OnClick", function() CharScanner:ScanUnit("target") end)

	GVAR.MainFrame.Headline.GroupScanAll = CreateFrame("Button", nil, GVAR.MainFrame.Headline)
	TEMPLATE.IconButton(GVAR.MainFrame.Headline.GroupScanAll, 1)
	GVAR.MainFrame.Headline.GroupScanAll.BorderColor:SetTexture(color.ButtonBorderDisable.rgba[1], color.ButtonBorderDisable.rgba[2], color.ButtonBorderDisable.rgba[3], color.ButtonBorderDisable.rgba[4])
	GVAR.MainFrame.Headline.GroupScanAll:SetPoint("TOPLEFT", GVAR.MainFrame.Headline.SingleScan, "TOPRIGHT", sizeGeneral.Offset, 0)
	GVAR.MainFrame.Headline.GroupScanAll:Disable()
	GVAR.MainFrame.Headline.GroupScanAll:SetScript("OnClick", function() CharScanner:ScanGroup(false) end)

	GVAR.MainFrame.Headline.GroupScanMissing = CreateFrame("Button", nil, GVAR.MainFrame.Headline)
	TEMPLATE.IconButton(GVAR.MainFrame.Headline.GroupScanMissing, 5)
	GVAR.MainFrame.Headline.GroupScanMissing.BorderColor:SetTexture(color.ButtonBorderDisable.rgba[1], color.ButtonBorderDisable.rgba[2], color.ButtonBorderDisable.rgba[3], color.ButtonBorderDisable.rgba[4])
	GVAR.MainFrame.Headline.GroupScanMissing:SetPoint("TOPLEFT", GVAR.MainFrame.Headline.GroupScanAll, "TOPRIGHT", sizeGeneral.Offset, 0)
	GVAR.MainFrame.Headline.GroupScanMissing:Disable()
	GVAR.MainFrame.Headline.GroupScanMissing:SetScript("OnClick", function() CharScanner:ScanGroup(true) end)

		GVAR.MainFrame.Headline.GroupScanMissingNum = CreateFrame("Button", nil, GVAR.MainFrame.Headline)
		GVAR.MainFrame.Headline.GroupScanMissingNum:SetWidth(20)
		GVAR.MainFrame.Headline.GroupScanMissingNum:SetHeight(fonts.GameFontNormalSmall.height + sizeGeneral.Space)
		GVAR.MainFrame.Headline.GroupScanMissingNum:SetPoint("BOTTOM", GVAR.MainFrame.Headline.GroupScanMissing, "TOP", 0, sizeGeneral.Space)
		GVAR.MainFrame.Headline.GroupScanMissingNum:Hide()

		GVAR.MainFrame.Headline.GroupScanMissingNumTex = GVAR.MainFrame.Headline.GroupScanMissingNum:CreateTexture(nil, "BACKGROUND")
		GVAR.MainFrame.Headline.GroupScanMissingNumTex:SetPoint("TOPLEFT", GVAR.MainFrame.Headline.GroupScanMissingNum, "TOPLEFT", 0, 0)
		GVAR.MainFrame.Headline.GroupScanMissingNumTex:SetPoint("BOTTOMRIGHT", GVAR.MainFrame.Headline.GroupScanMissingNum, "BOTTOMRIGHT", 0, 0)
		GVAR.MainFrame.Headline.GroupScanMissingNumTex:SetTexture(0, 0, 0, 1)

		GVAR.MainFrame.Headline.GroupScanMissingNumText = GVAR.MainFrame.Headline.GroupScanMissingNum:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
		GVAR.MainFrame.Headline.GroupScanMissingNumText:SetPoint("CENTER", 0, 0)
		GVAR.MainFrame.Headline.GroupScanMissingNumText:SetJustifyH("CENTER")
		GVAR.MainFrame.Headline.GroupScanMissingNumText:SetTextColor(color.Red.rgba[1], color.Red.rgba[2], color.Red.rgba[3], color.Red.rgba[4])

	GVAR.MainFrame.Headline.Collapse = CreateFrame("Button", nil, GVAR.MainFrame.Headline)
	TEMPLATE.IconButton(GVAR.MainFrame.Headline.Collapse, 6)
	GVAR.MainFrame.Headline.Collapse.BorderColor:SetTexture(color.ButtonBorderDisable.rgba[1], color.ButtonBorderDisable.rgba[2], color.ButtonBorderDisable.rgba[3], color.ButtonBorderDisable.rgba[4])
	GVAR.MainFrame.Headline.Collapse:SetPoint("TOPLEFT", GVAR.MainFrame.Headline.GroupScanMissing, "TOPRIGHT", sizeGeneral.Offset, 0)
	GVAR.MainFrame.Headline.Collapse:Disable()
	GVAR.MainFrame.Headline.Collapse:SetScript("OnClick", function() CharScanner:ScanFrameToggle() end)

	GVAR.MainFrame.Headline.History = CreateFrame("Button", nil, GVAR.MainFrame.Headline)
	TEMPLATE.IconButton(GVAR.MainFrame.Headline.History, 2)
	GVAR.MainFrame.Headline.History.BorderColor:SetTexture(color.ButtonBorderDisable.rgba[1], color.ButtonBorderDisable.rgba[2], color.ButtonBorderDisable.rgba[3], color.ButtonBorderDisable.rgba[4])
	GVAR.MainFrame.Headline.History:SetPoint("LEFT", GVAR.MainFrame.Headline.Collapse, "RIGHT", sizeGeneral.Offset, 0)
	GVAR.MainFrame.Headline.History:Disable()
	GVAR.MainFrame.Headline.History:SetScript("OnClick", function() CharScanner:HistoryFrameToggle() end)

	GVAR.MainFrame.Headline.Compare = CreateFrame("Button", nil, GVAR.MainFrame.Headline)
	TEMPLATE.IconButton(GVAR.MainFrame.Headline.Compare, 3)
	GVAR.MainFrame.Headline.Compare.BorderColor:SetTexture(color.ButtonBorderDisable.rgba[1], color.ButtonBorderDisable.rgba[2], color.ButtonBorderDisable.rgba[3], color.ButtonBorderDisable.rgba[4])
	GVAR.MainFrame.Headline.Compare:SetPoint("LEFT", GVAR.MainFrame.Headline.History, "RIGHT", sizeGeneral.Offset, 0)
	GVAR.MainFrame.Headline.Compare:Disable()
	GVAR.MainFrame.Headline.Compare:SetScript("OnClick", function() CharScanner:CompareFrameToggle() end)

	GVAR.MainFrame.Headline.Close = CreateFrame("Button", nil, GVAR.MainFrame.Headline)
	TEMPLATE.IconButton(GVAR.MainFrame.Headline.Close, 9)
	GVAR.MainFrame.Headline.Close:SetPoint("RIGHT", GVAR.MainFrame.Headline, "RIGHT", -sizeGeneral.Offset, 0)
	GVAR.MainFrame.Headline.Close:SetScript("OnClick", function() CharScanner:Frame_Toggle(GVAR.MainFrame) end)

	GVAR.MainFrame.Headline.Options = CreateFrame("Button", nil, GVAR.MainFrame.Headline)
	TEMPLATE.IconButton(GVAR.MainFrame.Headline.Options, 8)
	GVAR.MainFrame.Headline.Options:SetPoint("RIGHT", GVAR.MainFrame.Headline.Close, "LEFT", -sizeGeneral.Offset, 0)
	GVAR.MainFrame.Headline.Options:SetScript("OnClick", function() CharScanner:Frame_Toggle(GVAR.OptionsFrame, 1) end)

	GVAR.MainFrame.Headline.Delete = CreateFrame("Button", nil, GVAR.MainFrame.Headline)
	TEMPLATE.IconButton(GVAR.MainFrame.Headline.Delete, 4)
	TEMPLATE.DisableIconButton(GVAR.MainFrame.Headline.Delete, nil, 1)
	GVAR.MainFrame.Headline.Delete:SetPoint("RIGHT", GVAR.MainFrame.Headline.Options, "LEFT", -sizeGeneral.Offset, 0)
	GVAR.MainFrame.Headline.Delete:SetScript("OnClick", function() CharScanner:DeleteCharacter() end)

	GVAR.MainFrame.Headline.ButtonMover = CreateFrame("Button", nil, GVAR.MainFrame.Headline)
	GVAR.MainFrame.Headline.ButtonMover:SetWidth(22)
	GVAR.MainFrame.Headline.ButtonMover:SetHeight(22)
	GVAR.MainFrame.Headline.ButtonMover:SetPoint("TOPLEFT", GVAR.MainFrame.Headline.Compare, "TOPRIGHT", sizeGeneral.Offset, 0)
	GVAR.MainFrame.Headline.ButtonMover:SetPoint("BOTTOMRIGHT", GVAR.MainFrame.Headline.Delete, "BOTTOMLEFT", -sizeGeneral.Offset, 0)
	GVAR.MainFrame.Headline.ButtonMoverTexture = GVAR.MainFrame.Headline.ButtonMover:CreateTexture(nil, "BACKGROUND")
	GVAR.MainFrame.Headline.ButtonMoverTexture:SetWidth(1)
	GVAR.MainFrame.Headline.ButtonMoverTexture:SetHeight(1)
	GVAR.MainFrame.Headline.ButtonMoverTexture:SetPoint("TOPLEFT", GVAR.MainFrame.Headline.ButtonMover, "TOPLEFT", 0, 0)
	GVAR.MainFrame.Headline.ButtonMoverTexture:SetPoint("BOTTOMRIGHT", GVAR.MainFrame.Headline.ButtonMover, "BOTTOMRIGHT", 0, 0)
	GVAR.MainFrame.Headline.ButtonMoverTexture:SetTexture(1, 1, 1, 0)
	GVAR.MainFrame.Headline.ButtonMover:SetScript("OnEnter", function() GVAR.MainFrame.Headline.ButtonMoverTexture:SetTexture(1, 1, 1, 0.075) CharScanner:Frame_MoverInfo(GVAR.MainFrame, "MOVE", "TOP") end)
	GVAR.MainFrame.Headline.ButtonMover:SetScript("OnLeave", function() GVAR.MainFrame.Headline.ButtonMoverTexture:SetTexture(1, 1, 1, 0) CharScanner:Frame_MoverInfo(GVAR.MainFrame) end)
	GVAR.MainFrame.Headline.ButtonMover:SetScript("OnMouseDown", MoveStart)
	GVAR.MainFrame.Headline.ButtonMover:SetScript("OnMouseUp", MoveStop)
	-- Headline Frame END



	-- Base Character Information Frame START
	GVAR.MainFrame.Base = CreateFrame("Frame", nil, GVAR.MainFrame)
	TEMPLATE.BorderTRBL(GVAR.MainFrame.Base)
	GVAR.MainFrame.Base:SetWidth(sizeGeneral.MainWidth)
	GVAR.MainFrame.Base:SetHeight(sizeBaseCharFrame_BaseHeight)
	GVAR.MainFrame.Base:SetPoint("TOPLEFT", GVAR.MainFrame.Headline, "BOTTOMLEFT", 0, sizeGeneral.Border)
	GVAR.MainFrame.Base:EnableMouse(true)
	GVAR.MainFrame.Base:SetScript("OnShow", function() CharScanner:StatusFrame_CheckPosition() end)
	GVAR.MainFrame.Base:Hide()

	GVAR.MainFrame.Base.ClassIcon = GVAR.MainFrame.Base:CreateTexture(nil, "ARTWORK")
	GVAR.MainFrame.Base.ClassIcon:SetWidth(28)
	GVAR.MainFrame.Base.ClassIcon:SetHeight(28)
	GVAR.MainFrame.Base.ClassIcon:SetPoint("LEFT", sizeGeneral.Offset, 0)
	GVAR.MainFrame.Base.ClassIcon:SetTexture(Textures.ClassIcon.path)

	GVAR.MainFrame.Base.FactionIcon = GVAR.MainFrame.Base:CreateTexture(nil, "ARTWORK")
	GVAR.MainFrame.Base.FactionIcon:SetWidth(18)
	GVAR.MainFrame.Base.FactionIcon:SetHeight(18)
	GVAR.MainFrame.Base.FactionIcon:SetPoint("LEFT", GVAR.MainFrame.Base.ClassIcon, "RIGHT", 0, 0)

	GVAR.MainFrame.Base.Name = GVAR.MainFrame.Base:CreateFontString(nil, "ARTWORK", fonts.GameFontNormal.name)
	GVAR.MainFrame.Base.Name:SetPoint("TOPLEFT", GVAR.MainFrame.Base, "TOPLEFT", sizeGeneral.Offset+28+18+sizeGeneral.Space, -sizeGeneral.Offset)
	GVAR.MainFrame.Base.Name:SetJustifyH("LEFT")
	GVAR.MainFrame.Base.Name:SetTextColor(1, 1, 1, 1)

	GVAR.MainFrame.Base.GUID = GVAR.MainFrame.Base:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.MainFrame.Base.GUID:SetPoint("BOTTOM", GVAR.MainFrame.Base.Name, "BOTTOM")
	GVAR.MainFrame.Base.GUID:SetPoint("RIGHT", GVAR.MainFrame.Base, "RIGHT", -sizeGeneral.Offset, 0)
	GVAR.MainFrame.Base.GUID:SetJustifyH("LEFT")
	GVAR.MainFrame.Base.GUID:SetTextColor(color.ButtonBGActive.rgba[1], color.ButtonBGActive.rgba[2], color.ButtonBGActive.rgba[3], color.ButtonBGActive.rgba[4])

	GVAR.MainFrame.Base.Level = GVAR.MainFrame.Base:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.MainFrame.Base.Level:SetPoint("TOPLEFT", GVAR.MainFrame.Base.Name, "BOTTOMLEFT", 0, -sizeGeneral.Space)
	GVAR.MainFrame.Base.Level:SetJustifyH("LEFT")
	GVAR.MainFrame.Base.Level:SetTextColor(1, 1, 1, 1)

	GVAR.MainFrame.Base.Guild = GVAR.MainFrame.Base:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.MainFrame.Base.Guild:SetPoint("TOPLEFT", GVAR.MainFrame.Base.Level, "BOTTOMLEFT", 0, -sizeGeneral.Space)
	GVAR.MainFrame.Base.Guild:SetJustifyH("LEFT")
	GVAR.MainFrame.Base.Guild:SetTextColor(1, 1, 1, 1)
	-- Base Character Information Frame END



	-- Talent Frame START
	GVAR.MainFrame.Talent = CreateFrame("Frame", nil, GVAR.MainFrame)
	TEMPLATE.BorderTRBL(GVAR.MainFrame.Talent)
	GVAR.MainFrame.Talent:SetWidth(sizeGeneral.MainWidth)
	GVAR.MainFrame.Talent:SetHeight(sizeTalentFrame.BaseHeight)
	GVAR.MainFrame.Talent:SetPoint("TOPLEFT", GVAR.MainFrame.Base, "BOTTOMLEFT", 0, sizeGeneral.Border)
	GVAR.MainFrame.Talent:EnableMouse(true)
	GVAR.MainFrame.Talent:Hide()

	GVAR.MainFrame.Talent.Title = GVAR.MainFrame.Talent:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.MainFrame.Talent.Title:SetPoint("TOPLEFT", GVAR.MainFrame.Talent, "TOPLEFT", sizeGeneral.Offset, -sizeGeneral.Offset)
	GVAR.MainFrame.Talent.Title:SetJustifyH("LEFT")
	GVAR.MainFrame.Talent.Title:SetText(TALENTS..":")

	GVAR.MainFrame.Talent.MainRole = GVAR.MainFrame.Talent:CreateTexture(nil, "ARTWORK")
	GVAR.MainFrame.Talent.MainRole:SetWidth(Textures.TalentMiniIcons.qsize*2)
	GVAR.MainFrame.Talent.MainRole:SetHeight(Textures.TalentMiniIcons.qsize*2)
	GVAR.MainFrame.Talent.MainRole:SetPoint("TOPLEFT", GVAR.MainFrame.Talent, "TOPLEFT", sizeGeneral.Offset + GVAR.MainFrame.Talent.Title:GetStringWidth() + sizeGeneral.Offset, -sizeGeneral.Offset)

	GVAR.MainFrame.Talent.MainRoleActive = GVAR.MainFrame.Talent:CreateTexture(nil, "ARTWORK")
	GVAR.MainFrame.Talent.MainRoleActive:SetWidth(Textures.SelectionArrow.width)
	GVAR.MainFrame.Talent.MainRoleActive:SetHeight(Textures.SelectionArrow.height)
	GVAR.MainFrame.Talent.MainRoleActive:SetTexture(Textures.CharScannerIcons.path)
	GVAR.MainFrame.Talent.MainRoleActive:SetTexCoord(unpack(Textures.SelectionArrow.coords))

	GVAR.MainFrame.Talent.Group1Icon = GVAR.MainFrame.Talent:CreateTexture(nil, "ARTWORK")
	GVAR.MainFrame.Talent.Group1Icon:SetWidth(Textures.TalentMiniIcons.qsize)
	GVAR.MainFrame.Talent.Group1Icon:SetHeight(Textures.TalentMiniIcons.qsize)
	GVAR.MainFrame.Talent.Group1Icon:SetPoint("TOPLEFT", GVAR.MainFrame.Talent.MainRole, "TOPRIGHT", sizeGeneral.Offset+Textures.SelectionArrow.width+sizeGeneral.Offset, 0)
	GVAR.MainFrame.Talent.Group1Icon:SetTexCoord(unpack(Textures.TalentMiniIcons.coords))
	GVAR.MainFrame.Talent.Group1Text = GVAR.MainFrame.Talent:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.MainFrame.Talent.Group1Text:SetPoint("BOTTOM", GVAR.MainFrame.Talent.Title, "BOTTOM", 0, 0)
	GVAR.MainFrame.Talent.Group1Text:SetPoint("LEFT", GVAR.MainFrame.Talent.Group1Icon, "RIGHT", sizeGeneral.Offset, 0)
	GVAR.MainFrame.Talent.Group1Text:SetJustifyH("LEFT")

	GVAR.MainFrame.Talent.Group2Icon = GVAR.MainFrame.Talent:CreateTexture(nil, "ARTWORK")
	GVAR.MainFrame.Talent.Group2Icon:SetWidth(Textures.TalentMiniIcons.qsize)
	GVAR.MainFrame.Talent.Group2Icon:SetHeight(Textures.TalentMiniIcons.qsize)
	GVAR.MainFrame.Talent.Group2Icon:SetPoint("TOPLEFT", GVAR.MainFrame.Talent.Group1Icon, "BOTTOMLEFT", 0, 0)
	GVAR.MainFrame.Talent.Group2Icon:SetTexCoord(unpack(Textures.TalentMiniIcons.coords))
	GVAR.MainFrame.Talent.Group2Text = GVAR.MainFrame.Talent:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.MainFrame.Talent.Group2Text:SetPoint("LEFT", GVAR.MainFrame.Talent.Group2Icon, "RIGHT", sizeGeneral.Offset, 0)
	GVAR.MainFrame.Talent.Group2Text:SetJustifyH("LEFT")

	GVAR.MainFrame.Talent.Collapse = CreateFrame("Button", nil, GVAR.MainFrame.Talent)
	TEMPLATE.IconButton(GVAR.MainFrame.Talent.Collapse, 10)
	GVAR.MainFrame.Talent.Collapse:SetPoint("RIGHT", GVAR.MainFrame.Talent, "RIGHT", 0, 0)
	GVAR.MainFrame.Talent.Collapse:SetScript("OnClick", function() CharScanner:TalentTreeFrame_OnClick() end)
	GVAR.MainFrame.Talent.Collapse:SetScript("OnEnter", function() CharScanner:TalentTreeFrame_OnEnter() end)
	GVAR.MainFrame.Talent.Collapse:SetScript("OnLeave", function() CharScanner:TalentTreeFrame_OnLeave() end)
	-- Talent Frame END



	-- TalentTree Frame START
	GVAR.TalentTreeFrame = CreateFrame("Frame", "CharScanner_TalentTreeFrame", GVAR.MainFrame.Talent)
	TEMPLATE.BorderTRBL(GVAR.TalentTreeFrame)
	GVAR.TalentTreeFrame:SetWidth(sizeTalentFrame.TreeFrameWidth)
	GVAR.TalentTreeFrame:SetHeight(sizeTalentFrame.TreeFrameHeight)
	GVAR.TalentTreeFrame:SetPoint("TOPLEFT", GVAR.MainFrame.Talent, "TOPRIGHT", -sizeGeneral.Border, sizeGeneral.TabFrameHeight-sizeGeneral.Border)
	GVAR.TalentTreeFrame:EnableMouse(true)
	GVAR.TalentTreeFrame:SetToplevel(true)
	GVAR.TalentTreeFrame:Hide()

	GVAR.TalentTreeFrame.Close = CreateFrame("Button", nil, GVAR.TalentTreeFrame)
	TEMPLATE.IconButton(GVAR.TalentTreeFrame.Close, 9, 12)
	GVAR.TalentTreeFrame.Close:SetPoint("BOTTOMRIGHT", GVAR.TalentTreeFrame, "TOPRIGHT", 0, -sizeGeneral.Border)
	GVAR.TalentTreeFrame.Close:SetScript("OnClick", function() CharScanner:TalentTreeFrame_OnClick() end)

	GVAR.TalentTreeFrame.GroupFrame = CreateFrame("Frame", nil, GVAR.TalentTreeFrame)
	TEMPLATE.BorderTRBL(GVAR.TalentTreeFrame.GroupFrame)
	GVAR.TalentTreeFrame.GroupFrame:SetWidth(sizeTalentFrame.TreeFrameWidth)
	GVAR.TalentTreeFrame.GroupFrame:SetHeight(sizeGeneral.TabFrameHeight)
	GVAR.TalentTreeFrame.GroupFrame:SetPoint("TOPLEFT", 0, 0)

	GVAR.TalentTreeFrame.GroupFrame.Button1 = CreateFrame("Button", nil, GVAR.TalentTreeFrame.GroupFrame)
	TEMPLATE.TabButton(GVAR.TalentTreeFrame.GroupFrame.Button1, TALENT_SPEC_PRIMARY, sizeTalentFrame.GroupSelectionWidth)
	GVAR.TalentTreeFrame.GroupFrame.Button1:SetWidth(sizeTalentFrame.GroupSelectionWidth)
	GVAR.TalentTreeFrame.GroupFrame.Button1:SetHeight(sizeGeneral.TabButtonHeight)
	GVAR.TalentTreeFrame.GroupFrame.Button1:SetPoint("BOTTOMLEFT", GVAR.TalentTreeFrame.GroupFrame, "BOTTOMLEFT", sizeTalentFrame.GroupSelection1Left, -sizeGeneral.Border)
	GVAR.TalentTreeFrame.GroupFrame.Button1:Hide()

	GVAR.TalentTreeFrame.GroupFrame.Button2 = CreateFrame("Button", nil, GVAR.TalentTreeFrame.GroupFrame)
	TEMPLATE.TabButton(GVAR.TalentTreeFrame.GroupFrame.Button2, TALENT_SPEC_SECONDARY, sizeTalentFrame.GroupSelectionWidth)
	GVAR.TalentTreeFrame.GroupFrame.Button2:SetWidth(sizeTalentFrame.GroupSelectionWidth)
	GVAR.TalentTreeFrame.GroupFrame.Button2:SetHeight(sizeGeneral.TabButtonHeight)
	GVAR.TalentTreeFrame.GroupFrame.Button2:SetPoint("BOTTOMLEFT", GVAR.TalentTreeFrame.GroupFrame, "BOTTOMLEFT", sizeTalentFrame.GroupSelection2Left, -sizeGeneral.Border)
	GVAR.TalentTreeFrame.GroupFrame.Button2:Hide()

	if not GVAR.TalentTreeFrame.Tree then GVAR.TalentTreeFrame.Tree = {} end
	GVAR.TalentTreeFrame.Tree[1] = CreateFrame("Frame", nil, GVAR.TalentTreeFrame)
	GVAR.TalentTreeFrame.Tree[1]:SetWidth(sizeTalentFrame.TreeWidth)
	GVAR.TalentTreeFrame.Tree[1]:SetHeight(sizeTalentFrame.TreeHeight)
	GVAR.TalentTreeFrame.Tree[1]:SetPoint("TOPLEFT", GVAR.TalentTreeFrame.GroupFrame, "BOTTOMLEFT", sizeTalentFrame.Tree1Left, -sizeGeneral.Offset)
	GVAR.TalentTreeFrame.Tree[1].Name = GVAR.TalentTreeFrame.Tree[1]:CreateFontString(nil, "ARTWORK", fonts.GameFontWhiteSmall.name)
	GVAR.TalentTreeFrame.Tree[1].Name:SetPoint("TOP", 0, 0)
	GVAR.TalentTreeFrame.Tree[1].Name:SetJustifyH("CENTER")
	GVAR.TalentTreeFrame.Tree[1].Points = GVAR.TalentTreeFrame.Tree[1]:CreateFontString(nil, "ARTWORK", fonts.GameFontWhiteSmall.name)
	GVAR.TalentTreeFrame.Tree[1].Points:SetPoint("TOP", GVAR.TalentTreeFrame.Tree[1].Name, "BOTTOM", 0, -sizeGeneral.Space)
	GVAR.TalentTreeFrame.Tree[1].Points:SetJustifyH("CENTER")

	GVAR.TalentTreeFrame.Tree[2] = CreateFrame("Frame", nil, GVAR.TalentTreeFrame)
	GVAR.TalentTreeFrame.Tree[2]:SetWidth(sizeTalentFrame.TreeWidth)
	GVAR.TalentTreeFrame.Tree[2]:SetHeight(sizeTalentFrame.TreeHeight)
	GVAR.TalentTreeFrame.Tree[2]:SetPoint("TOPLEFT", GVAR.TalentTreeFrame.GroupFrame, "BOTTOMLEFT", sizeTalentFrame.Tree2Left, -sizeGeneral.Offset)
	GVAR.TalentTreeFrame.Tree[2].Name = GVAR.TalentTreeFrame.Tree[2]:CreateFontString(nil, "ARTWORK", fonts.GameFontWhiteSmall.name)
	GVAR.TalentTreeFrame.Tree[2].Name:SetPoint("TOP", 0, 0)
	GVAR.TalentTreeFrame.Tree[2].Name:SetJustifyH("CENTER")
	GVAR.TalentTreeFrame.Tree[2].Points = GVAR.TalentTreeFrame.Tree[2]:CreateFontString(nil, "ARTWORK", fonts.GameFontWhiteSmall.name)
	GVAR.TalentTreeFrame.Tree[2].Points:SetPoint("TOP", GVAR.TalentTreeFrame.Tree[2].Name, "BOTTOM", 0, -sizeGeneral.Space)
	GVAR.TalentTreeFrame.Tree[2].Points:SetJustifyH("CENTER")

	GVAR.TalentTreeFrame.Tree[3] = CreateFrame("Frame", nil, GVAR.TalentTreeFrame)
	GVAR.TalentTreeFrame.Tree[3]:SetWidth(sizeTalentFrame.TreeWidth)
	GVAR.TalentTreeFrame.Tree[3]:SetHeight(sizeTalentFrame.TreeHeight)
	GVAR.TalentTreeFrame.Tree[3]:SetPoint("TOPLEFT", GVAR.TalentTreeFrame.GroupFrame, "BOTTOMLEFT", sizeTalentFrame.Tree3Left, -sizeGeneral.Offset)
	GVAR.TalentTreeFrame.Tree[3].Name = GVAR.TalentTreeFrame.Tree[3]:CreateFontString(nil, "ARTWORK", fonts.GameFontWhiteSmall.name)
	GVAR.TalentTreeFrame.Tree[3].Name:SetPoint("TOP", 0, 0)
	GVAR.TalentTreeFrame.Tree[3].Name:SetJustifyH("CENTER")
	GVAR.TalentTreeFrame.Tree[3].Points = GVAR.TalentTreeFrame.Tree[3]:CreateFontString(nil, "ARTWORK", fonts.GameFontWhiteSmall.name)
	GVAR.TalentTreeFrame.Tree[3].Points:SetPoint("TOP", GVAR.TalentTreeFrame.Tree[3].Name, "BOTTOM", 0, -sizeGeneral.Space)
	GVAR.TalentTreeFrame.Tree[3].Points:SetJustifyH("CENTER")

	local tier = 1
	local column = 1
	for tab = 1, myMAX_TALENT_TABS do
		for talent = 1, myMAX_NUM_TALENTS do
			if talent == 1 then
				tier = 1
				column = 1
				if not GVAR.TalentTreeFrame.Slot then GVAR.TalentTreeFrame.Slot = {} end
				if not GVAR.TalentTreeFrame.Slot[tab] then GVAR.TalentTreeFrame.Slot[tab] = {} end
				if not GVAR.TalentTreeFrame.Slot[tab][tier] then GVAR.TalentTreeFrame.Slot[tab][tier] = {} end
				GVAR.TalentTreeFrame.Slot[tab][tier][column] = CreateFrame("Button", nil, GVAR.TalentTreeFrame)
				GVAR.TalentTreeFrame.Slot[tab][tier][column]:SetPoint("TOPLEFT", GVAR.TalentTreeFrame.Tree[tab], "TOPLEFT", 0, -((2*fonts.GameFontWhiteSmall.height)+(2*sizeGeneral.Space))) -- -(12*-2) font height
			elseif mod(talent, myMAX_COLUMN) == 1 then
				tier = tier + 1
				column = 1
				if not GVAR.TalentTreeFrame.Slot then GVAR.TalentTreeFrame.Slot = {} end
				if not GVAR.TalentTreeFrame.Slot[tab] then GVAR.TalentTreeFrame.Slot[tab] = {} end
				if not GVAR.TalentTreeFrame.Slot[tab][tier] then GVAR.TalentTreeFrame.Slot[tab][tier] = {} end
				GVAR.TalentTreeFrame.Slot[tab][tier][column] = CreateFrame("Button", nil, GVAR.TalentTreeFrame)
				GVAR.TalentTreeFrame.Slot[tab][tier][column]:SetPoint("TOP", GVAR.TalentTreeFrame.Slot[tab][(tier-1)][column], "BOTTOM", 0, -sizeGeneral.Offset)
			else
				column = column + 1
				if not GVAR.TalentTreeFrame.Slot then GVAR.TalentTreeFrame.Slot = {} end
				if not GVAR.TalentTreeFrame.Slot[tab] then GVAR.TalentTreeFrame.Slot[tab] = {} end
				if not GVAR.TalentTreeFrame.Slot[tab][tier] then GVAR.TalentTreeFrame.Slot[tab][tier] = {} end
				GVAR.TalentTreeFrame.Slot[tab][tier][column] = CreateFrame("Button", nil, GVAR.TalentTreeFrame)
				GVAR.TalentTreeFrame.Slot[tab][tier][column]:SetPoint("LEFT", GVAR.TalentTreeFrame.Slot[tab][tier][(column-1)], "RIGHT", sizeGeneral.Offset, 0)
			end
			TEMPLATE.TalentButton(GVAR.TalentTreeFrame.Slot[tab][tier][column])
		end
	end
	-- TalentTree Frame END



	-- Equipment Frame START
	GVAR.MainFrame.Equipment = CreateFrame("Frame", nil, GVAR.MainFrame)
	TEMPLATE.BorderTRBL(GVAR.MainFrame.Equipment)
	GVAR.MainFrame.Equipment:SetWidth(sizeGeneral.MainWidth)
	GVAR.MainFrame.Equipment:SetHeight(sizeEquipmentFrame.BaseHeight)
	GVAR.MainFrame.Equipment:SetPoint("TOPLEFT", GVAR.MainFrame.Talent, "BOTTOMLEFT", 0, sizeGeneral.Border)
	GVAR.MainFrame.Equipment:EnableMouse(true)
	GVAR.MainFrame.Equipment:Hide()

	GVAR.MainFrame.Equipment.Title = GVAR.MainFrame.Equipment:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.MainFrame.Equipment.Title:SetPoint("TOPLEFT", GVAR.MainFrame.Equipment, "TOPLEFT", sizeGeneral.Offset, -sizeGeneral.Offset)
	GVAR.MainFrame.Equipment.Title:SetJustifyH("LEFT")
	GVAR.MainFrame.Equipment.Text1 = GVAR.MainFrame.Equipment:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.MainFrame.Equipment.Text1:SetPoint("LEFT", GVAR.MainFrame.Equipment.Title, "RIGHT", sizeGeneral.Offset, 0)
	GVAR.MainFrame.Equipment.Text1:SetJustifyH("LEFT")
	GVAR.MainFrame.Equipment.Text1:SetTextColor(1, 1, 1, 1)
	GVAR.MainFrame.Equipment.Text2 = GVAR.MainFrame.Equipment:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.MainFrame.Equipment.Text2:SetPoint("TOPLEFT", GVAR.MainFrame.Equipment.Text1, "BOTTOMLEFT", 0, -(sizeGeneral.Space+sizeGeneral.Space))
	GVAR.MainFrame.Equipment.Text2:SetJustifyH("LEFT")
	GVAR.MainFrame.Equipment.Text2:SetTextColor(1, 1, 1, 1)
	GVAR.MainFrame.Equipment.Text3 = GVAR.MainFrame.Equipment:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.MainFrame.Equipment.Text3:SetPoint("TOPLEFT", GVAR.MainFrame.Equipment.Text2, "BOTTOMLEFT", 0, -(sizeGeneral.Space+sizeGeneral.Space))
	GVAR.MainFrame.Equipment.Text3:SetJustifyH("LEFT")

	GVAR.MainFrame.Equipment.Collapse = CreateFrame("Button", nil, GVAR.MainFrame.Equipment)
	TEMPLATE.IconButton(GVAR.MainFrame.Equipment.Collapse, 10)
	GVAR.MainFrame.Equipment.Collapse:SetPoint("RIGHT", GVAR.MainFrame.Equipment, "RIGHT", 0, 0)
	GVAR.MainFrame.Equipment.Collapse:SetScript("OnClick", function() CharScanner:GearFrame_OnClick() end)
	GVAR.MainFrame.Equipment.Collapse:SetScript("OnEnter", function() CharScanner:GearFrame_OnEnter() end)
	GVAR.MainFrame.Equipment.Collapse:SetScript("OnLeave", function() CharScanner:GearFrame_OnLeave() end)
	-- Equipment Frame END



	-- Gear Frame START
	GVAR.GearFrame = CreateFrame("Frame", "CharScanner_GearFrame", GVAR.MainFrame.Equipment)
	TEMPLATE.BorderTRBL(GVAR.GearFrame)
	GVAR.GearFrame:SetWidth(sizeEquipmentFrame.GearWidth)
	GVAR.GearFrame:SetHeight(sizeEquipmentFrame.GearHeight+sizeGeneral.TabFrameHeight+sizeEquipmentFrame.GearSummaryHeight)
	GVAR.GearFrame:SetPoint("TOPLEFT", GVAR.MainFrame.Headline, "TOPRIGHT", -sizeGeneral.Border, 0)
	GVAR.GearFrame:EnableMouse(true)
	GVAR.GearFrame:SetToplevel(true)
	GVAR.GearFrame:Hide()

	GVAR.GearFrame.Close = CreateFrame("Button", nil, GVAR.GearFrame)
	TEMPLATE.IconButton(GVAR.GearFrame.Close, 9, 12)
	GVAR.GearFrame.Close:SetPoint("BOTTOMRIGHT", GVAR.GearFrame, "TOPRIGHT", 0, -sizeGeneral.Border)
	GVAR.GearFrame.Close:SetScript("OnClick", function() CharScanner:GearFrame_OnClick() end)

	-- - Gear Slot START
	for i = 1, #slotButton do
		if not GVAR.GearFrame_Slot then GVAR.GearFrame_Slot = {} end
		if not GVAR.GearFrame_Slot[i] then GVAR.GearFrame_Slot[i] = {} end
		GVAR.GearFrame_Slot[i] = CreateFrame("Button", nil, GVAR.GearFrame)
		GVAR.GearFrame_Slot[i]:SetWidth(sizeEquipmentFrame.GearButtonWidth)
		GVAR.GearFrame_Slot[i]:SetHeight(Textures.GearIconTexture.qsize)
		if i == 1 then
			GVAR.GearFrame_Slot[i]:SetPoint("TOPLEFT", GVAR.GearFrame, "TOPLEFT", sizeGeneral.Offset, -sizeGeneral.Offset)
		elseif i == 7 then
			GVAR.GearFrame_Slot[i]:SetPoint("TOPLEFT", GVAR.GearFrame_Slot[(i-1)], "BOTTOMLEFT", 0, -(3*sizeGeneral.Offset))
		elseif i == 15 then
			GVAR.GearFrame_Slot[i]:SetPoint("TOPLEFT", GVAR.GearFrame_Slot[(i-1)], "BOTTOMLEFT", 0, -(3*sizeGeneral.Offset))
		else
			GVAR.GearFrame_Slot[i]:SetPoint("TOPLEFT", GVAR.GearFrame_Slot[(i-1)], "BOTTOMLEFT", 0, -sizeGeneral.Space)
		end

		GVAR.GearFrame_Slot[i].DefaultIconTexture = GVAR.GearFrame_Slot[i]:CreateTexture(nil, "BORDER")
		GVAR.GearFrame_Slot[i].DefaultIconTexture:SetWidth(Textures.GearIconTexture.qsize)
		GVAR.GearFrame_Slot[i].DefaultIconTexture:SetHeight(Textures.GearIconTexture.qsize)
		GVAR.GearFrame_Slot[i].DefaultIconTexture:SetPoint("LEFT", 0, 0)
		GVAR.GearFrame_Slot[i].DefaultIconTexture:SetTexCoord(5/64, 59/64, 5/64, 59/64)
		GVAR.GearFrame_Slot[i].DefaultIconTexture:SetTexture(slotButton[i].texture)

		GVAR.GearFrame_Slot[i].PvPTexture = GVAR.GearFrame_Slot[i]:CreateTexture(nil, "ARTWORK")
		GVAR.GearFrame_Slot[i].PvPTexture:SetWidth(Textures.PvPIcon.qsize)
		GVAR.GearFrame_Slot[i].PvPTexture:SetHeight(Textures.PvPIcon.qsize)
		GVAR.GearFrame_Slot[i].PvPTexture:SetPoint("LEFT", GVAR.GearFrame_Slot[i].DefaultIconTexture, Textures.GearIconTexture.qsize+30+sizeGeneral.Space, 0)
		GVAR.GearFrame_Slot[i].PvPTexture:SetTexture(Textures.CharScannerIcons.path)
		GVAR.GearFrame_Slot[i].PvPTexture:SetTexCoord(unpack(Textures.PvPIcon.coords))
		GVAR.GearFrame_Slot[i].PvPTexture:Hide()

		GVAR.GearFrame_Slot[i].ItemLevel = GVAR.GearFrame_Slot[i]:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
		GVAR.GearFrame_Slot[i].ItemLevel:SetWidth(30)
		GVAR.GearFrame_Slot[i].ItemLevel:SetPoint("LEFT", GVAR.GearFrame_Slot[i].DefaultIconTexture, "RIGHT", 0, 0)
		GVAR.GearFrame_Slot[i].ItemLevel:SetJustifyH("RIGHT")

		GVAR.GearFrame_Slot[i].IconButton = CreateFrame("Button", nil, GVAR.GearFrame_Slot[i])
		GVAR.GearFrame_Slot[i].IconButton:SetWidth(Textures.GearIconTexture.qsize)
		GVAR.GearFrame_Slot[i].IconButton:SetHeight(Textures.GearIconTexture.qsize)
		GVAR.GearFrame_Slot[i].IconButton:SetPoint("LEFT", GVAR.GearFrame_Slot[i].DefaultIconTexture, "LEFT", Textures.GearIconTexture.qsize+30+sizeGeneral.Space+Textures.PvPIcon.qsize+sizeGeneral.Space, 0)
		GVAR.GearFrame_Slot[i].IconButtonBG = GVAR.GearFrame_Slot[i].IconButton:CreateTexture(nil, "BACKGROUND")
		GVAR.GearFrame_Slot[i].IconButtonBG:SetWidth(Textures.GearIconTexture.qsize)
		GVAR.GearFrame_Slot[i].IconButtonBG:SetHeight(Textures.GearIconTexture.qsize)
		GVAR.GearFrame_Slot[i].IconButtonBG:SetPoint("TOPLEFT", 0, 0)
		GVAR.GearFrame_Slot[i].IconButtonBG:SetTexture(0.2, 0.2, 0.2, 1)
		GVAR.GearFrame_Slot[i].IconTexture = GVAR.GearFrame_Slot[i].IconButton:CreateTexture(nil, "BORDER")
		GVAR.GearFrame_Slot[i].IconTexture:SetWidth(Textures.GearIconTexture.qsize-2)
		GVAR.GearFrame_Slot[i].IconTexture:SetHeight(Textures.GearIconTexture.qsize-2)
		GVAR.GearFrame_Slot[i].IconTexture:SetPoint("CENTER", 0, 0)
		GVAR.GearFrame_Slot[i].IconTexture:SetTexCoord(5/64, 59/64, 5/64, 59/64)
		GVAR.GearFrame_Slot[i].IconTexture:SetTexture(slotButton[i].texture)

		GVAR.GearFrame_Slot[i].IsSetTexture = GVAR.GearFrame_Slot[i].IconButton:CreateTexture(nil, "ARTWORK")
		GVAR.GearFrame_Slot[i].IsSetTexture:SetWidth(3)
		GVAR.GearFrame_Slot[i].IsSetTexture:SetHeight(Textures.GearIconTexture.qsize)
		GVAR.GearFrame_Slot[i].IsSetTexture:SetPoint("RIGHT", GVAR.GearFrame_Slot[i].IconButton, "RIGHT", 0, 0)
		GVAR.GearFrame_Slot[i].IsSetTexture:SetTexture(1, 1, 0.49, 1)
		GVAR.GearFrame_Slot[i].IsSetTexture:Hide()

		GVAR.GearFrame_Slot[i].EnchButton = CreateFrame("Button", nil, GVAR.GearFrame_Slot[i])
		GVAR.GearFrame_Slot[i].EnchButton:SetWidth(Textures.EnchantmentIcon.qsize)
		GVAR.GearFrame_Slot[i].EnchButton:SetHeight(Textures.EnchantmentIcon.qsize)
		GVAR.GearFrame_Slot[i].EnchButton:SetPoint("LEFT", GVAR.GearFrame_Slot[i].IconButton, "RIGHT", sizeGeneral.Space, 0)
		GVAR.GearFrame_Slot[i].EnchButton:Hide()
		GVAR.GearFrame_Slot[i].EnchButtonBG = GVAR.GearFrame_Slot[i].EnchButton:CreateTexture(nil, "BACKGROUND")
		GVAR.GearFrame_Slot[i].EnchButtonBG:SetWidth(Textures.EnchantmentIcon.qsize)
		GVAR.GearFrame_Slot[i].EnchButtonBG:SetHeight(Textures.EnchantmentIcon.qsize)
		GVAR.GearFrame_Slot[i].EnchButtonBG:SetPoint("TOPLEFT", 0, 0)
		GVAR.GearFrame_Slot[i].EnchButtonBG:SetTexture(0.2, 0.2, 0.2, 1)
		GVAR.GearFrame_Slot[i].EnchTexture = GVAR.GearFrame_Slot[i].EnchButton:CreateTexture(nil, "ARTWORK")
		GVAR.GearFrame_Slot[i].EnchTexture:SetWidth(Textures.EnchantmentIcon.qsize-2)
		GVAR.GearFrame_Slot[i].EnchTexture:SetHeight(Textures.EnchantmentIcon.qsize-2)
		GVAR.GearFrame_Slot[i].EnchTexture:SetPoint("CENTER", 0, 0)
		GVAR.GearFrame_Slot[i].EnchTexture:SetTexture(Textures.CharScannerIcons.path)
		GVAR.GearFrame_Slot[i].EnchTexture:SetTexCoord(unpack(Textures.EnchantmentIcon.coords))

		GVAR.GearFrame_Slot[i].ReforgeButton = CreateFrame("Button", nil, GVAR.GearFrame_Slot[i])
		GVAR.GearFrame_Slot[i].ReforgeButton:SetWidth(Textures.ReforgeIcon.qsize)
		GVAR.GearFrame_Slot[i].ReforgeButton:SetHeight(Textures.ReforgeIcon.qsize)
		GVAR.GearFrame_Slot[i].ReforgeButton:SetPoint("LEFT", GVAR.GearFrame_Slot[i].EnchButton, "RIGHT", sizeGeneral.Space, 0)
		GVAR.GearFrame_Slot[i].ReforgeButton:Hide()
		GVAR.GearFrame_Slot[i].ReforgeButtonBG = GVAR.GearFrame_Slot[i].ReforgeButton:CreateTexture(nil, "BACKGROUND")
		GVAR.GearFrame_Slot[i].ReforgeButtonBG:SetWidth(Textures.ReforgeIcon.qsize)
		GVAR.GearFrame_Slot[i].ReforgeButtonBG:SetHeight(Textures.ReforgeIcon.qsize)
		GVAR.GearFrame_Slot[i].ReforgeButtonBG:SetPoint("TOPLEFT", 0, 0)
		GVAR.GearFrame_Slot[i].ReforgeButtonBG:SetTexture(0.2, 0.2, 0.2, 1)
		GVAR.GearFrame_Slot[i].ReforgeTexture = GVAR.GearFrame_Slot[i].ReforgeButton:CreateTexture(nil, "ARTWORK")
		GVAR.GearFrame_Slot[i].ReforgeTexture:SetWidth(Textures.ReforgeIcon.qsize-2)
		GVAR.GearFrame_Slot[i].ReforgeTexture:SetHeight(Textures.ReforgeIcon.qsize-2)
		GVAR.GearFrame_Slot[i].ReforgeTexture:SetPoint("CENTER", 0, 0)
		GVAR.GearFrame_Slot[i].ReforgeTexture:SetTexture(Textures.CharScannerIcons.path)
		GVAR.GearFrame_Slot[i].ReforgeTexture:SetTexCoord(unpack(Textures.ReforgeIcon.coords))

		GVAR.GearFrame_Slot[i].Gem1Button = CreateFrame("Button", nil, GVAR.GearFrame_Slot[i])
		GVAR.GearFrame_Slot[i].Gem1Button:SetWidth(Textures.GemIcon.qsize)
		GVAR.GearFrame_Slot[i].Gem1Button:SetHeight(Textures.GemIcon.qsize)
		GVAR.GearFrame_Slot[i].Gem1Button:SetPoint("LEFT", GVAR.GearFrame_Slot[i].ReforgeButton, "RIGHT", sizeGeneral.Space, 0)
		GVAR.GearFrame_Slot[i].Gem1Button:Hide()
		GVAR.GearFrame_Slot[i].Gem1ButtonBG = GVAR.GearFrame_Slot[i].Gem1Button:CreateTexture(nil, "BACKGROUND")
		GVAR.GearFrame_Slot[i].Gem1ButtonBG:SetWidth(Textures.GemIcon.qsize)
		GVAR.GearFrame_Slot[i].Gem1ButtonBG:SetHeight(Textures.GemIcon.qsize)
		GVAR.GearFrame_Slot[i].Gem1ButtonBG:SetPoint("TOPLEFT", 0, 0)
		GVAR.GearFrame_Slot[i].Gem1ButtonBG:SetTexture(0.2, 0.2, 0.2, 1)
		GVAR.GearFrame_Slot[i].Gem1Texture = GVAR.GearFrame_Slot[i].Gem1Button:CreateTexture(nil, "ARTWORK")
		GVAR.GearFrame_Slot[i].Gem1Texture:SetWidth(Textures.GemIcon.qsize-2)
		GVAR.GearFrame_Slot[i].Gem1Texture:SetHeight(Textures.GemIcon.qsize-2)
		GVAR.GearFrame_Slot[i].Gem1Texture:SetPoint("CENTER", 0, 0)
		GVAR.GearFrame_Slot[i].Gem1Texture:SetTexture(Textures.CharScannerIcons.path)
		GVAR.GearFrame_Slot[i].Gem1Texture:SetTexCoord(0, 0, 0, 0)

		GVAR.GearFrame_Slot[i].Gem2Button = CreateFrame("Button", nil, GVAR.GearFrame_Slot[i])
		GVAR.GearFrame_Slot[i].Gem2Button:SetWidth(Textures.GemIcon.qsize)
		GVAR.GearFrame_Slot[i].Gem2Button:SetHeight(Textures.GemIcon.qsize)
		GVAR.GearFrame_Slot[i].Gem2Button:SetPoint("LEFT", GVAR.GearFrame_Slot[i].Gem1Button, "RIGHT", (sizeGeneral.Space/2), 0)
		GVAR.GearFrame_Slot[i].Gem2Button:Hide()
		GVAR.GearFrame_Slot[i].Gem2ButtonBG = GVAR.GearFrame_Slot[i].Gem2Button:CreateTexture(nil, "BACKGROUND")
		GVAR.GearFrame_Slot[i].Gem2ButtonBG:SetWidth(Textures.GemIcon.qsize)
		GVAR.GearFrame_Slot[i].Gem2ButtonBG:SetHeight(Textures.GemIcon.qsize)
		GVAR.GearFrame_Slot[i].Gem2ButtonBG:SetPoint("TOPLEFT", 0, 0)
		GVAR.GearFrame_Slot[i].Gem2ButtonBG:SetTexture(0.2, 0.2, 0.2, 1)
		GVAR.GearFrame_Slot[i].Gem2Texture = GVAR.GearFrame_Slot[i].Gem2Button:CreateTexture(nil, "ARTWORK")
		GVAR.GearFrame_Slot[i].Gem2Texture:SetWidth(Textures.GemIcon.qsize-2)
		GVAR.GearFrame_Slot[i].Gem2Texture:SetHeight(Textures.GemIcon.qsize-2)
		GVAR.GearFrame_Slot[i].Gem2Texture:SetPoint("CENTER", 0, 0)
		GVAR.GearFrame_Slot[i].Gem2Texture:SetTexture(Textures.CharScannerIcons.path)
		GVAR.GearFrame_Slot[i].Gem2Texture:SetTexCoord(0, 0, 0, 0)

		GVAR.GearFrame_Slot[i].Gem3Button = CreateFrame("Button", nil, GVAR.GearFrame_Slot[i])
		GVAR.GearFrame_Slot[i].Gem3Button:SetWidth(Textures.GemIcon.qsize)
		GVAR.GearFrame_Slot[i].Gem3Button:SetHeight(Textures.GemIcon.qsize)
		GVAR.GearFrame_Slot[i].Gem3Button:SetPoint("LEFT", GVAR.GearFrame_Slot[i].Gem2Button, "RIGHT", (sizeGeneral.Space/2), 0)
		GVAR.GearFrame_Slot[i].Gem3Button:Hide()
		GVAR.GearFrame_Slot[i].Gem3ButtonBG = GVAR.GearFrame_Slot[i].Gem3Button:CreateTexture(nil, "BACKGROUND")
		GVAR.GearFrame_Slot[i].Gem3ButtonBG:SetWidth(Textures.GemIcon.qsize)
		GVAR.GearFrame_Slot[i].Gem3ButtonBG:SetHeight(Textures.GemIcon.qsize)
		GVAR.GearFrame_Slot[i].Gem3ButtonBG:SetPoint("TOPLEFT", 0, 0)
		GVAR.GearFrame_Slot[i].Gem3ButtonBG:SetTexture(0.2, 0.2, 0.2, 1)
		GVAR.GearFrame_Slot[i].Gem3Texture = GVAR.GearFrame_Slot[i].Gem3Button:CreateTexture(nil, "ARTWORK")
		GVAR.GearFrame_Slot[i].Gem3Texture:SetWidth(Textures.GemIcon.qsize-2)
		GVAR.GearFrame_Slot[i].Gem3Texture:SetHeight(Textures.GemIcon.qsize-2)
		GVAR.GearFrame_Slot[i].Gem3Texture:SetPoint("CENTER", 0, 0)
		GVAR.GearFrame_Slot[i].Gem3Texture:SetTexture(Textures.CharScannerIcons.path)
		GVAR.GearFrame_Slot[i].Gem3Texture:SetTexCoord(0, 0, 0, 0)

		GVAR.GearFrame_Slot[i].Gem4Button = CreateFrame("Button", nil, GVAR.GearFrame_Slot[i])
		GVAR.GearFrame_Slot[i].Gem4Button:SetWidth(Textures.GemIcon.qsize)
		GVAR.GearFrame_Slot[i].Gem4Button:SetHeight(Textures.GemIcon.qsize)
		GVAR.GearFrame_Slot[i].Gem4Button:SetPoint("LEFT", GVAR.GearFrame_Slot[i].Gem3Button, "RIGHT", (sizeGeneral.Space/2), 0)
		GVAR.GearFrame_Slot[i].Gem4Button:Hide()
		GVAR.GearFrame_Slot[i].Gem4ButtonBG = GVAR.GearFrame_Slot[i].Gem4Button:CreateTexture(nil, "BACKGROUND")
		GVAR.GearFrame_Slot[i].Gem4ButtonBG:SetWidth(Textures.GemIcon.qsize)
		GVAR.GearFrame_Slot[i].Gem4ButtonBG:SetHeight(Textures.GemIcon.qsize)
		GVAR.GearFrame_Slot[i].Gem4ButtonBG:SetPoint("TOPLEFT", 0, 0)
		GVAR.GearFrame_Slot[i].Gem4ButtonBG:SetTexture(0.2, 0.2, 0.2, 1)
		GVAR.GearFrame_Slot[i].Gem4Texture = GVAR.GearFrame_Slot[i].Gem4Button:CreateTexture(nil, "ARTWORK")
		GVAR.GearFrame_Slot[i].Gem4Texture:SetWidth(Textures.GemIcon.qsize-2)
		GVAR.GearFrame_Slot[i].Gem4Texture:SetHeight(Textures.GemIcon.qsize-2)
		GVAR.GearFrame_Slot[i].Gem4Texture:SetPoint("CENTER", 0, 0)
		GVAR.GearFrame_Slot[i].Gem4Texture:SetTexture(Textures.CharScannerIcons.path)
		GVAR.GearFrame_Slot[i].Gem4Texture:SetTexCoord(0, 0, 0, 0)

		GVAR.GearFrame_Slot[i].GemBonusTexture = GVAR.GearFrame_Slot[i]:CreateTexture(nil, "BORDER")
		GVAR.GearFrame_Slot[i].GemBonusTexture:SetWidth(2)
		GVAR.GearFrame_Slot[i].GemBonusTexture:SetHeight(2)
		GVAR.GearFrame_Slot[i].GemBonusTexture:SetPoint("TOPLEFT", GVAR.GearFrame_Slot[i].Gem1Button, "BOTTOMLEFT", 0, -2)
		GVAR.GearFrame_Slot[i].GemBonusTexture:SetTexture(0, 0, 0, 0)
--[[
		GVAR.GearFrame_Slot[i].Highlight = GVAR.GearFrame_Slot[i]:CreateTexture(nil, "OVERLAY")
		GVAR.GearFrame_Slot[i].Highlight:SetPoint("TOPLEFT", GVAR.GearFrame_Slot[i], "TOPLEFT", 0, 0)
		GVAR.GearFrame_Slot[i].Highlight:SetPoint("BOTTOMRIGHT", GVAR.GearFrame_Slot[i], "BOTTOMRIGHT", 0, 0)
		GVAR.GearFrame_Slot[i].Highlight:SetTexture(1, 1, 1, 0.1)
		GVAR.GearFrame_Slot[i]:SetHighlightTexture(GVAR.GearFrame_Slot[i].Highlight)
--]]
	end
	-- - Gear Slot END

	-- - Gear Summary START
	GVAR.GearFrame_Summary = CreateFrame("Frame", nil, GVAR.GearFrame)
	TEMPLATE.BorderTRBL(GVAR.GearFrame_Summary)
	GVAR.GearFrame_Summary:SetWidth(sizeEquipmentFrame.GearWidth)
	GVAR.GearFrame_Summary:SetHeight(sizeGeneral.TabFrameHeight+sizeEquipmentFrame.GearSummaryHeight)
	GVAR.GearFrame_Summary:SetPoint("BOTTOMLEFT", GVAR.GearFrame, "BOTTOMLEFT", 0, 0)
	GVAR.GearFrame_Summary:SetToplevel(true)

	local tabspace = 4
	local tabwidth = (sizeEquipmentFrame.GearWidth-tabspace-tabspace-(3*tabspace ) ) / 4

	GVAR.GearFrame_Summary.TabSelection = CreateFrame("Frame", nil, GVAR.GearFrame_Summary)
	TEMPLATE.BorderTRBL(GVAR.GearFrame_Summary.TabSelection)
	GVAR.GearFrame_Summary.TabSelection:SetWidth(sizeEquipmentFrame.GearWidth)
	GVAR.GearFrame_Summary.TabSelection:SetHeight(sizeGeneral.TabFrameHeight)
	GVAR.GearFrame_Summary.TabSelection:SetPoint("TOPLEFT", GVAR.GearFrame_Summary, "TOPLEFT", 0, 0)

	GVAR.GearFrame_Summary.TabButton1 = CreateFrame("Button", nil, GVAR.GearFrame_Summary.TabSelection)
	TEMPLATE.TabButton(GVAR.GearFrame_Summary.TabButton1, L["Enchantments"], tabwidth)
	GVAR.GearFrame_Summary.TabButton1:SetWidth(tabwidth)
	GVAR.GearFrame_Summary.TabButton1:SetHeight(sizeGeneral.TabButtonHeight)
	GVAR.GearFrame_Summary.TabButton1:SetPoint("BOTTOMLEFT", GVAR.GearFrame_Summary.TabSelection, "BOTTOMLEFT", tabspace, -sizeGeneral.Border)
	GVAR.GearFrame_Summary.TabButton1Frame = CreateFrame("Frame", nil, GVAR.GearFrame_Summary)
	GVAR.GearFrame_Summary.TabButton1Frame:SetWidth(sizeEquipmentFrame.GearWidth)
	GVAR.GearFrame_Summary.TabButton1Frame:SetHeight(sizeEquipmentFrame.GearSummaryHeight)
	GVAR.GearFrame_Summary.TabButton1Frame:SetPoint("TOPLEFT", GVAR.GearFrame_Summary.TabSelection, "BOTTOMLEFT", 0, sizeGeneral.Border)
	--GVAR.GearFrame_Summary.TabButton1Frame:Hide()

	GVAR.GearFrame_Summary.TabButton2 = CreateFrame("Button", nil, GVAR.GearFrame_Summary.TabSelection)
	TEMPLATE.TabButton(GVAR.GearFrame_Summary.TabButton2, L["Reforged"], tabwidth)
	GVAR.GearFrame_Summary.TabButton2:SetWidth(tabwidth)
	GVAR.GearFrame_Summary.TabButton2:SetHeight(sizeGeneral.TabButtonHeight)
	GVAR.GearFrame_Summary.TabButton2:SetPoint("BOTTOMLEFT", GVAR.GearFrame_Summary.TabSelection, "BOTTOMLEFT", tabspace+tabwidth+tabspace, -sizeGeneral.Border)
	GVAR.GearFrame_Summary.TabButton2Frame = CreateFrame("Frame", nil, GVAR.GearFrame_Summary)
	GVAR.GearFrame_Summary.TabButton2Frame:SetWidth(sizeEquipmentFrame.GearWidth)
	GVAR.GearFrame_Summary.TabButton2Frame:SetHeight(sizeEquipmentFrame.GearSummaryHeight)
	GVAR.GearFrame_Summary.TabButton2Frame:SetPoint("TOPLEFT", GVAR.GearFrame_Summary.TabSelection, "BOTTOMLEFT", 0, sizeGeneral.Border)
	GVAR.GearFrame_Summary.TabButton2Frame:Hide()

	GVAR.GearFrame_Summary.TabButton3 = CreateFrame("Button", nil, GVAR.GearFrame_Summary.TabSelection)
	TEMPLATE.TabButton(GVAR.GearFrame_Summary.TabButton3, L["Gems"], tabwidth)
	GVAR.GearFrame_Summary.TabButton3:SetWidth(tabwidth)
	GVAR.GearFrame_Summary.TabButton3:SetHeight(sizeGeneral.TabButtonHeight)
	GVAR.GearFrame_Summary.TabButton3:SetPoint("BOTTOMLEFT", GVAR.GearFrame_Summary.TabSelection, "BOTTOMLEFT", tabspace+tabwidth+tabspace+tabwidth+tabspace, -sizeGeneral.Border)
	GVAR.GearFrame_Summary.TabButton3Frame = CreateFrame("Frame", nil, GVAR.GearFrame_Summary)
	GVAR.GearFrame_Summary.TabButton3Frame:SetWidth(sizeEquipmentFrame.GearWidth)
	GVAR.GearFrame_Summary.TabButton3Frame:SetHeight(sizeEquipmentFrame.GearSummaryHeight)
	GVAR.GearFrame_Summary.TabButton3Frame:SetPoint("TOPLEFT", GVAR.GearFrame_Summary.TabSelection, "BOTTOMLEFT", 0, sizeGeneral.Border)
	GVAR.GearFrame_Summary.TabButton3Frame:Hide()

	GVAR.GearFrame_Summary.TabButton4 = CreateFrame("Button", nil, GVAR.GearFrame_Summary.TabSelection)
	TEMPLATE.TabButton(GVAR.GearFrame_Summary.TabButton4, L["Sets"], tabwidth)
	GVAR.GearFrame_Summary.TabButton4:SetWidth(tabwidth)
	GVAR.GearFrame_Summary.TabButton4:SetHeight(sizeGeneral.TabButtonHeight)
	GVAR.GearFrame_Summary.TabButton4:SetPoint("BOTTOMLEFT", GVAR.GearFrame_Summary.TabSelection, "BOTTOMLEFT", tabspace+tabwidth+tabspace+tabwidth+tabspace+tabwidth+tabspace, -sizeGeneral.Border)
	GVAR.GearFrame_Summary.TabButton4Frame = CreateFrame("Frame", nil, GVAR.GearFrame_Summary)
	GVAR.GearFrame_Summary.TabButton4Frame:SetWidth(sizeEquipmentFrame.GearWidth)
	GVAR.GearFrame_Summary.TabButton4Frame:SetHeight(sizeEquipmentFrame.GearSummaryHeight)
	GVAR.GearFrame_Summary.TabButton4Frame:SetPoint("TOPLEFT", GVAR.GearFrame_Summary.TabSelection, "BOTTOMLEFT", 0, sizeGeneral.Border)
	GVAR.GearFrame_Summary.TabButton4Frame:Hide()

	for i = 1, 16 do
		-- 1 enchantment
		if not GVAR.GearFrame_Summary.TabButton1Frame[i] then GVAR.GearFrame_Summary.TabButton1Frame[i] = {} end
		GVAR.GearFrame_Summary.TabButton1Frame[i].Button = CreateFrame("Button", nil, GVAR.GearFrame_Summary.TabButton1Frame)
		GVAR.GearFrame_Summary.TabButton1Frame[i].Button:SetWidth(sizeEquipmentFrame.GearSummaryButtonWidth)
		GVAR.GearFrame_Summary.TabButton1Frame[i].Button:SetHeight(fonts.GameFontNormalSmall.height + sizeGeneral.Space)
		if i == 1 then
			GVAR.GearFrame_Summary.TabButton1Frame[i].Button:SetPoint("TOPLEFT", GVAR.GearFrame_Summary.TabButton1Frame, "TOPLEFT", sizeGeneral.Offset, -sizeGeneral.Offset)
		elseif i == 9 then
			GVAR.GearFrame_Summary.TabButton1Frame[i].Button:SetPoint("LEFT", GVAR.GearFrame_Summary.TabButton1Frame[1].Button, "RIGHT", (sizeGeneral.Offset*2), 0)
		else
			GVAR.GearFrame_Summary.TabButton1Frame[i].Button:SetPoint("TOPLEFT", GVAR.GearFrame_Summary.TabButton1Frame[(i-1)].Button, "BOTTOMLEFT", 0, 0)
		end

		GVAR.GearFrame_Summary.TabButton1Frame[i].Text = GVAR.GearFrame_Summary.TabButton1Frame[i].Button:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
		GVAR.GearFrame_Summary.TabButton1Frame[i].Text:SetWidth(sizeEquipmentFrame.GearStatKeyWidth)
		GVAR.GearFrame_Summary.TabButton1Frame[i].Text:SetHeight(fonts.GameFontNormalSmall.height)
		GVAR.GearFrame_Summary.TabButton1Frame[i].Text:SetPoint("LEFT", 0, 0)
		GVAR.GearFrame_Summary.TabButton1Frame[i].Text:SetJustifyH("LEFT")
		--GVAR.GearFrame_Summary.TabButton1Frame[i].Text:SetText("Enchantment"..i) -- TEST
		GVAR.GearFrame_Summary.TabButton1Frame[i].Text:SetTextColor(color.LightYellow.rgba[1], color.LightYellow.rgba[2], color.LightYellow.rgba[3], color.LightYellow.rgba[4])

		--GVAR.GearFrame_Summary.TabButton1Frame[i].testBG1 = GVAR.GearFrame_Summary.TabButton1Frame[i].Button:CreateTexture(nil, "OVERLAY")
		--GVAR.GearFrame_Summary.TabButton1Frame[i].testBG1:SetWidth(sizeEquipmentFrame.GearStatKeyWidth)
		--GVAR.GearFrame_Summary.TabButton1Frame[i].testBG1:SetHeight(fonts.GameFontNormalSmall.height)
		--GVAR.GearFrame_Summary.TabButton1Frame[i].testBG1:SetPoint("LEFT", 0, 0)
		--GVAR.GearFrame_Summary.TabButton1Frame[i].testBG1:SetTexture(0.63, 0.18, 0.76, 1)

		GVAR.GearFrame_Summary.TabButton1Frame[i].Value = GVAR.GearFrame_Summary.TabButton1Frame[i].Button:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
		GVAR.GearFrame_Summary.TabButton1Frame[i].Value:SetWidth(sizeEquipmentFrame.GearStatValueWidth)
		GVAR.GearFrame_Summary.TabButton1Frame[i].Value:SetPoint("LEFT", GVAR.GearFrame_Summary.TabButton1Frame[i].Text, "RIGHT", 0, 0)
		GVAR.GearFrame_Summary.TabButton1Frame[i].Value:SetJustifyH("RIGHT")
		--GVAR.GearFrame_Summary.TabButton1Frame[i].Value:SetText("ID 999999") -- TEST
		GVAR.GearFrame_Summary.TabButton1Frame[i].Value:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])

		--GVAR.GearFrame_Summary.TabButton1Frame[i].testBG2 = GVAR.GearFrame_Summary.TabButton1Frame[i].Button:CreateTexture(nil, "OVERLAY")
		--GVAR.GearFrame_Summary.TabButton1Frame[i].testBG2:SetWidth(sizeEquipmentFrame.GearStatValueWidth)
		--GVAR.GearFrame_Summary.TabButton1Frame[i].testBG2:SetHeight(fonts.GameFontNormalSmall.height)
		--GVAR.GearFrame_Summary.TabButton1Frame[i].testBG2:SetPoint("LEFT", GVAR.GearFrame_Summary.TabButton1Frame[i].Text, "RIGHT", 0, 0)
		--GVAR.GearFrame_Summary.TabButton1Frame[i].testBG2:SetTexture(0.38, 0.78, 0.26, 1)

		GVAR.GearFrame_Summary.TabButton1Frame[i].Highlight = GVAR.GearFrame_Summary.TabButton1Frame[i].Button:CreateTexture(nil, "OVERLAY")
		GVAR.GearFrame_Summary.TabButton1Frame[i].Highlight:SetPoint("TOPLEFT", GVAR.GearFrame_Summary.TabButton1Frame[i].Button, "TOPLEFT", 0, 0)
		GVAR.GearFrame_Summary.TabButton1Frame[i].Highlight:SetPoint("BOTTOMRIGHT", GVAR.GearFrame_Summary.TabButton1Frame[i].Button, "BOTTOMRIGHT", 0, 0)
		GVAR.GearFrame_Summary.TabButton1Frame[i].Highlight:SetTexture(1, 1, 1, 0.1)
		GVAR.GearFrame_Summary.TabButton1Frame[i].Button:SetHighlightTexture(GVAR.GearFrame_Summary.TabButton1Frame[i].Highlight)

		-- 3 gem
		if not GVAR.GearFrame_Summary.TabButton3Frame[i] then GVAR.GearFrame_Summary.TabButton3Frame[i] = {} end
		GVAR.GearFrame_Summary.TabButton3Frame[i].Button = CreateFrame("Button", nil, GVAR.GearFrame_Summary.TabButton3Frame)
		GVAR.GearFrame_Summary.TabButton3Frame[i].Button:SetWidth(sizeEquipmentFrame.GearSummaryButtonWidth)
		GVAR.GearFrame_Summary.TabButton3Frame[i].Button:SetHeight(fonts.GameFontNormalSmall.height + sizeGeneral.Space)
		if i == 1 then
			GVAR.GearFrame_Summary.TabButton3Frame[i].Button:SetPoint("TOPLEFT", GVAR.GearFrame_Summary.TabButton3Frame, "TOPLEFT", sizeGeneral.Offset, -sizeGeneral.Offset)
		elseif i == 9 then
			GVAR.GearFrame_Summary.TabButton3Frame[i].Button:SetPoint("LEFT", GVAR.GearFrame_Summary.TabButton3Frame[1].Button, "RIGHT", (sizeGeneral.Offset*2), 0)
		else
			GVAR.GearFrame_Summary.TabButton3Frame[i].Button:SetPoint("TOPLEFT", GVAR.GearFrame_Summary.TabButton3Frame[(i-1)].Button, "BOTTOMLEFT", 0, 0)
		end

		GVAR.GearFrame_Summary.TabButton3Frame[i].Text = GVAR.GearFrame_Summary.TabButton3Frame[i].Button:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
		GVAR.GearFrame_Summary.TabButton3Frame[i].Text:SetWidth(sizeEquipmentFrame.GearStatKeyWidth)
		GVAR.GearFrame_Summary.TabButton3Frame[i].Text:SetHeight(fonts.GameFontNormalSmall.height)
		GVAR.GearFrame_Summary.TabButton3Frame[i].Text:SetPoint("LEFT", 0, 0)
		GVAR.GearFrame_Summary.TabButton3Frame[i].Text:SetJustifyH("LEFT")
		--GVAR.GearFrame_Summary.TabButton3Frame[i].Text:SetText("Enchantment"..i) -- TEST
		GVAR.GearFrame_Summary.TabButton3Frame[i].Text:SetTextColor(color.LightYellow.rgba[1], color.LightYellow.rgba[2], color.LightYellow.rgba[3], color.LightYellow.rgba[4])

		GVAR.GearFrame_Summary.TabButton3Frame[i].Value = GVAR.GearFrame_Summary.TabButton3Frame[i].Button:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
		GVAR.GearFrame_Summary.TabButton3Frame[i].Value:SetWidth(sizeEquipmentFrame.GearStatValueWidth)
		GVAR.GearFrame_Summary.TabButton3Frame[i].Value:SetPoint("LEFT", GVAR.GearFrame_Summary.TabButton3Frame[i].Text, "RIGHT", 0, 0)
		GVAR.GearFrame_Summary.TabButton3Frame[i].Value:SetJustifyH("RIGHT")
		--GVAR.GearFrame_Summary.TabButton3Frame[i].Value:SetText("Value"..i) -- TEST
		GVAR.GearFrame_Summary.TabButton3Frame[i].Value:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])

		GVAR.GearFrame_Summary.TabButton3Frame[i].Highlight = GVAR.GearFrame_Summary.TabButton3Frame[i].Button:CreateTexture(nil, "OVERLAY")
		GVAR.GearFrame_Summary.TabButton3Frame[i].Highlight:SetPoint("TOPLEFT", GVAR.GearFrame_Summary.TabButton3Frame[i].Button, "TOPLEFT", 0, 0)
		GVAR.GearFrame_Summary.TabButton3Frame[i].Highlight:SetPoint("BOTTOMRIGHT", GVAR.GearFrame_Summary.TabButton3Frame[i].Button, "BOTTOMRIGHT", 0, 0)
		GVAR.GearFrame_Summary.TabButton3Frame[i].Highlight:SetTexture(1, 1, 1, 0.1)
		GVAR.GearFrame_Summary.TabButton3Frame[i].Button:SetHighlightTexture(GVAR.GearFrame_Summary.TabButton3Frame[i].Highlight)

		-- 4 set
		if not GVAR.GearFrame_Summary.TabButton4Frame[i] then GVAR.GearFrame_Summary.TabButton4Frame[i] = {} end
		GVAR.GearFrame_Summary.TabButton4Frame[i].Button = CreateFrame("Button", nil, GVAR.GearFrame_Summary.TabButton4Frame)
		GVAR.GearFrame_Summary.TabButton4Frame[i].Button:SetWidth(sizeEquipmentFrame.GearSummaryButtonWidth)
		GVAR.GearFrame_Summary.TabButton4Frame[i].Button:SetHeight(fonts.GameFontNormalSmall.height + sizeGeneral.Space)
		if i == 1 then
			GVAR.GearFrame_Summary.TabButton4Frame[i].Button:SetPoint("TOPLEFT", GVAR.GearFrame_Summary.TabButton4Frame, "TOPLEFT", sizeGeneral.Offset, -sizeGeneral.Offset)
		elseif i == 9 then
			GVAR.GearFrame_Summary.TabButton4Frame[i].Button:SetPoint("LEFT", GVAR.GearFrame_Summary.TabButton4Frame[1].Button, "RIGHT", (sizeGeneral.Offset*2), 0)
		else
			GVAR.GearFrame_Summary.TabButton4Frame[i].Button:SetPoint("TOPLEFT", GVAR.GearFrame_Summary.TabButton4Frame[(i-1)].Button, "BOTTOMLEFT", 0, 0)
		end

		GVAR.GearFrame_Summary.TabButton4Frame[i].Text = GVAR.GearFrame_Summary.TabButton4Frame[i].Button:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
		GVAR.GearFrame_Summary.TabButton4Frame[i].Text:SetWidth(sizeEquipmentFrame.GearStatKeyWidth)
		GVAR.GearFrame_Summary.TabButton4Frame[i].Text:SetHeight(fonts.GameFontNormalSmall.height)
		GVAR.GearFrame_Summary.TabButton4Frame[i].Text:SetPoint("LEFT", 0, 0)
		GVAR.GearFrame_Summary.TabButton4Frame[i].Text:SetJustifyH("LEFT")
		--GVAR.GearFrame_Summary.TabButton4Frame[i].Text:SetText("Enchantment"..i) -- TEST
		GVAR.GearFrame_Summary.TabButton4Frame[i].Text:SetTextColor(color.LightYellow.rgba[1], color.LightYellow.rgba[2], color.LightYellow.rgba[3], color.LightYellow.rgba[4])

		GVAR.GearFrame_Summary.TabButton4Frame[i].Value = GVAR.GearFrame_Summary.TabButton4Frame[i].Button:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
		GVAR.GearFrame_Summary.TabButton4Frame[i].Value:SetWidth(sizeEquipmentFrame.GearStatValueWidth)
		GVAR.GearFrame_Summary.TabButton4Frame[i].Value:SetPoint("LEFT", GVAR.GearFrame_Summary.TabButton4Frame[i].Text, "RIGHT", 0, 0)
		GVAR.GearFrame_Summary.TabButton4Frame[i].Value:SetJustifyH("RIGHT")
		--GVAR.GearFrame_Summary.TabButton4Frame[i].Value:SetText("Value"..i) -- TEST
		GVAR.GearFrame_Summary.TabButton4Frame[i].Value:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])

		GVAR.GearFrame_Summary.TabButton4Frame[i].Highlight = GVAR.GearFrame_Summary.TabButton4Frame[i].Button:CreateTexture(nil, "OVERLAY")
		GVAR.GearFrame_Summary.TabButton4Frame[i].Highlight:SetPoint("TOPLEFT", GVAR.GearFrame_Summary.TabButton4Frame[i].Button, "TOPLEFT", 0, 0)
		GVAR.GearFrame_Summary.TabButton4Frame[i].Highlight:SetPoint("BOTTOMRIGHT", GVAR.GearFrame_Summary.TabButton4Frame[i].Button, "BOTTOMRIGHT", 0, 0)
		GVAR.GearFrame_Summary.TabButton4Frame[i].Highlight:SetTexture(1, 1, 1, 0.1)
		GVAR.GearFrame_Summary.TabButton4Frame[i].Button:SetHighlightTexture(GVAR.GearFrame_Summary.TabButton4Frame[i].Highlight)
	end

	for i = 1, 7 do
		-- 2 reforge
		if not GVAR.GearFrame_Summary.TabButton2Frame[i] then GVAR.GearFrame_Summary.TabButton2Frame[i] = {} end
		GVAR.GearFrame_Summary.TabButton2Frame[i].Text = GVAR.GearFrame_Summary.TabButton2Frame:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
		GVAR.GearFrame_Summary.TabButton2Frame[i].Text:SetWidth(sizeEquipmentFrame.GearStatKeyWidth)
		if i == 1 then
			GVAR.GearFrame_Summary.TabButton2Frame[i].Text:SetPoint("TOPLEFT", GVAR.GearFrame_Summary.TabButton2Frame, "TOPLEFT", sizeGeneral.Offset, -sizeGeneral.Offset)
		else
			GVAR.GearFrame_Summary.TabButton2Frame[i].Text:SetPoint("TOPLEFT", GVAR.GearFrame_Summary.TabButton2Frame[(i-1)].Text, "BOTTOMLEFT", 0, -sizeGeneral.Space)
		end
		GVAR.GearFrame_Summary.TabButton2Frame[i].Text:SetJustifyH("LEFT")
		--GVAR.GearFrame_Summary.TabButton2Frame[i].Text:SetText("Reforge"..i) -- TEST
		GVAR.GearFrame_Summary.TabButton2Frame[i].Text:SetTextColor(color.Red.rgba[1], color.Red.rgba[2], color.Red.rgba[3], color.Red.rgba[4])

		GVAR.GearFrame_Summary.TabButton2Frame[i].Value = GVAR.GearFrame_Summary.TabButton2Frame:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
		GVAR.GearFrame_Summary.TabButton2Frame[i].Value:SetWidth(sizeEquipmentFrame.GearStatKeyWidth)
		if i == 1 then
			GVAR.GearFrame_Summary.TabButton2Frame[i].Value:SetPoint("TOPLEFT", GVAR.GearFrame_Summary.TabButton2Frame, "TOPLEFT", sizeEquipmentFrame.GearStatKeyWidth + sizeGeneral.Offset + sizeEquipmentFrame.GearStatValueWidth + sizeGeneral.Offset, -sizeGeneral.Offset)
		else
			GVAR.GearFrame_Summary.TabButton2Frame[i].Value:SetPoint("TOPLEFT", GVAR.GearFrame_Summary.TabButton2Frame[(i-1)].Value, "BOTTOMLEFT", 0, -sizeGeneral.Space)
		end
		GVAR.GearFrame_Summary.TabButton2Frame[i].Value:SetJustifyH("LEFT")
		--GVAR.GearFrame_Summary.TabButton2Frame[i].Value:SetText("Value"..i) -- TEST
		GVAR.GearFrame_Summary.TabButton2Frame[i].Value:SetTextColor(color.Green.rgba[1], color.Green.rgba[2], color.Green.rgba[3], color.Green.rgba[4])
	end

	GVAR.GearFrame_Summary.TabButton1:SetScript("OnClick", function()
		TEMPLATE.SetTabButton(GVAR.GearFrame_Summary.TabButton1, true)
		TEMPLATE.SetTabButton(GVAR.GearFrame_Summary.TabButton2)
		TEMPLATE.SetTabButton(GVAR.GearFrame_Summary.TabButton3)
		TEMPLATE.SetTabButton(GVAR.GearFrame_Summary.TabButton4)
		GVAR.GearFrame_Summary.TabButton1Frame:Show()
		GVAR.GearFrame_Summary.TabButton2Frame:Hide()
		GVAR.GearFrame_Summary.TabButton3Frame:Hide()
		GVAR.GearFrame_Summary.TabButton4Frame:Hide()
	end)

	GVAR.GearFrame_Summary.TabButton2:SetScript("OnClick", function()
		TEMPLATE.SetTabButton(GVAR.GearFrame_Summary.TabButton1)
		TEMPLATE.SetTabButton(GVAR.GearFrame_Summary.TabButton2, true)
		TEMPLATE.SetTabButton(GVAR.GearFrame_Summary.TabButton3)
		TEMPLATE.SetTabButton(GVAR.GearFrame_Summary.TabButton4)
		GVAR.GearFrame_Summary.TabButton1Frame:Hide()
		GVAR.GearFrame_Summary.TabButton2Frame:Show()
		GVAR.GearFrame_Summary.TabButton3Frame:Hide()
		GVAR.GearFrame_Summary.TabButton4Frame:Hide()
	end)

	GVAR.GearFrame_Summary.TabButton3:SetScript("OnClick", function()
		TEMPLATE.SetTabButton(GVAR.GearFrame_Summary.TabButton1)
		TEMPLATE.SetTabButton(GVAR.GearFrame_Summary.TabButton2)
		TEMPLATE.SetTabButton(GVAR.GearFrame_Summary.TabButton3, true)
		TEMPLATE.SetTabButton(GVAR.GearFrame_Summary.TabButton4)
		GVAR.GearFrame_Summary.TabButton1Frame:Hide()
		GVAR.GearFrame_Summary.TabButton2Frame:Hide()
		GVAR.GearFrame_Summary.TabButton3Frame:Show()
		GVAR.GearFrame_Summary.TabButton4Frame:Hide()
	end)

	GVAR.GearFrame_Summary.TabButton4:SetScript("OnClick", function()
		TEMPLATE.SetTabButton(GVAR.GearFrame_Summary.TabButton1)
		TEMPLATE.SetTabButton(GVAR.GearFrame_Summary.TabButton2)
		TEMPLATE.SetTabButton(GVAR.GearFrame_Summary.TabButton3)
		TEMPLATE.SetTabButton(GVAR.GearFrame_Summary.TabButton4, true)
		GVAR.GearFrame_Summary.TabButton1Frame:Hide()
		GVAR.GearFrame_Summary.TabButton2Frame:Hide()
		GVAR.GearFrame_Summary.TabButton3Frame:Hide()
		GVAR.GearFrame_Summary.TabButton4Frame:Show()
	end)

	TEMPLATE.SetTabButton(GVAR.GearFrame_Summary.TabButton1, true)
	TEMPLATE.SetTabButton(GVAR.GearFrame_Summary.TabButton2)
	TEMPLATE.SetTabButton(GVAR.GearFrame_Summary.TabButton3)
	TEMPLATE.SetTabButton(GVAR.GearFrame_Summary.TabButton4)
	-- - Gear Summary END

	-- - Gear Stats START
	GVAR.GearFrame_Stats = CreateFrame("Frame", nil, GVAR.GearFrame)
	TEMPLATE.BorderTRBL(GVAR.GearFrame_Stats)
	GVAR.GearFrame_Stats:SetWidth(sizeEquipmentFrame.GearPanelWidth)
	GVAR.GearFrame_Stats:SetHeight(sizeEquipmentFrame.GearHeight + sizeGeneral.Border)
	GVAR.GearFrame_Stats:SetPoint("TOPRIGHT", GVAR.GearFrame, "TOPRIGHT", 0, 0)
	GVAR.GearFrame_Stats:SetToplevel(true)

	for i = 1, #statGlobalName do
		if not GVAR.GearFrame_Stats[i] then GVAR.GearFrame_Stats[i] = {} end

		GVAR.GearFrame_Stats[i].Button = CreateFrame("Button", nil, GVAR.GearFrame_Stats)
		if i == 1 then
			GVAR.GearFrame_Stats[i].Button:SetPoint("TOPLEFT", GVAR.GearFrame_Stats, "TOPLEFT", sizeGeneral.Offset, -sizeGeneral.Offset)
		--elseif i == 6 or i == 16 or i == 18 or i == 25 or i == 27 then
		elseif i == 6 or i == 7 or i == 9 or i == 10 or i == 16 or i == 20 or i == 22 or i == 28 then
			GVAR.GearFrame_Stats[i].Button:SetPoint("TOPLEFT", GVAR.GearFrame_Stats[(i-1)].Button, "BOTTOMLEFT", 0, -sizeGeneral.Offset)
		else
			GVAR.GearFrame_Stats[i].Button:SetPoint("TOPLEFT", GVAR.GearFrame_Stats[(i-1)].Button, "BOTTOMLEFT", 0, 0)
		end
		GVAR.GearFrame_Stats[i].Button:SetWidth(sizeEquipmentFrame.GearButtonWidth)
		GVAR.GearFrame_Stats[i].Button:SetHeight(fonts.GameFontNormalSmall.height + sizeGeneral.Space)

		GVAR.GearFrame_Stats[i].Stat = GVAR.GearFrame_Stats[i].Button:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
		GVAR.GearFrame_Stats[i].Stat:SetWidth(sizeEquipmentFrame.GearStatKeyWidth)
		GVAR.GearFrame_Stats[i].Stat:SetPoint("LEFT", GVAR.GearFrame_Stats[i].Button, "LEFT", 0, 0)
		GVAR.GearFrame_Stats[i].Stat:SetHeight(fonts.GameFontNormalSmall.height)
		GVAR.GearFrame_Stats[i].Stat:SetJustifyH("LEFT")
		GVAR.GearFrame_Stats[i].Stat:SetTextColor(color.LightYellow.rgba[1], color.LightYellow.rgba[2], color.LightYellow.rgba[3], color.LightYellow.rgba[4])
		GVAR.GearFrame_Stats[i].Stat:SetText(statShortLocName[ statDisplay[i] ]..":")

		GVAR.GearFrame_Stats[i].Value = GVAR.GearFrame_Stats[i].Button:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
		GVAR.GearFrame_Stats[i].Value:SetPoint("LEFT", GVAR.GearFrame_Stats[i].Stat, "RIGHT", 0, 0)
		GVAR.GearFrame_Stats[i].Value:SetWidth(sizeEquipmentFrame.GearStatValueWidth)
		GVAR.GearFrame_Stats[i].Value:SetJustifyH("RIGHT")
		GVAR.GearFrame_Stats[i].Value:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])

		GVAR.GearFrame_Stats[i].Highlight = GVAR.GearFrame_Stats[i].Button:CreateTexture(nil, "OVERLAY")
		GVAR.GearFrame_Stats[i].Highlight:SetPoint("TOPLEFT", GVAR.GearFrame_Stats[i].Button, "TOPLEFT", 0, 0)
		GVAR.GearFrame_Stats[i].Highlight:SetPoint("BOTTOMRIGHT", GVAR.GearFrame_Stats[i].Button, "BOTTOMRIGHT", 0, 0)
		GVAR.GearFrame_Stats[i].Highlight:SetTexture(1, 1, 1, 0.1)
		GVAR.GearFrame_Stats[i].Button:SetHighlightTexture(GVAR.GearFrame_Stats[i].Highlight)

	end

	GVAR.GearFrame_Stats.GemBlueTexture = GVAR.GearFrame_Stats:CreateTexture(nil, "ARTWORK")
	GVAR.GearFrame_Stats.GemBlueTexture:SetWidth(Textures.GemColorIcon.qsize)
	GVAR.GearFrame_Stats.GemBlueTexture:SetHeight(Textures.GemColorIcon.qsize)
	GVAR.GearFrame_Stats.GemBlueTexture:SetPoint("TOP", GVAR.GearFrame_Stats[#statGlobalName].Value, "BOTTOM", 0, -20)
	GVAR.GearFrame_Stats.GemBlueTexture:SetPoint("LEFT", GVAR.GearFrame_Stats, "LEFT", 5, 0)
	GVAR.GearFrame_Stats.GemBlueTexture:SetTexture(Textures.GemColorIcon.path)
	GVAR.GearFrame_Stats.GemBlueTexture:SetTexCoord(unpack(Textures.GemColorIcon.coordsBlu))
	GVAR.GearFrame_Stats.GemBlueText = GVAR.GearFrame_Stats:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
	GVAR.GearFrame_Stats.GemBlueText:SetPoint("LEFT", GVAR.GearFrame_Stats.GemBlueTexture, "RIGHT", 2, 0)
	GVAR.GearFrame_Stats.GemBlueText:SetJustifyH("LEFT")
	GVAR.GearFrame_Stats.GemBlueText:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])

	GVAR.GearFrame_Stats.GemRedTexture = GVAR.GearFrame_Stats:CreateTexture(nil, "ARTWORK")
	GVAR.GearFrame_Stats.GemRedTexture:SetWidth(Textures.GemColorIcon.qsize)
	GVAR.GearFrame_Stats.GemRedTexture:SetHeight(Textures.GemColorIcon.qsize)
	GVAR.GearFrame_Stats.GemRedTexture:SetPoint("LEFT", GVAR.GearFrame_Stats.GemBlueTexture, "RIGHT", 30, 0)
	GVAR.GearFrame_Stats.GemRedTexture:SetTexture(Textures.GemColorIcon.path)
	GVAR.GearFrame_Stats.GemRedTexture:SetTexCoord(unpack(Textures.GemColorIcon.coordsRed))
	GVAR.GearFrame_Stats.GemRedText = GVAR.GearFrame_Stats:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
	GVAR.GearFrame_Stats.GemRedText:SetPoint("LEFT", GVAR.GearFrame_Stats.GemRedTexture, "RIGHT", 2, 0)
	GVAR.GearFrame_Stats.GemRedText:SetJustifyH("LEFT")
	GVAR.GearFrame_Stats.GemRedText:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])

	GVAR.GearFrame_Stats.GemYellowTexture = GVAR.GearFrame_Stats:CreateTexture(nil, "ARTWORK")
	GVAR.GearFrame_Stats.GemYellowTexture:SetWidth(Textures.GemColorIcon.qsize)
	GVAR.GearFrame_Stats.GemYellowTexture:SetHeight(Textures.GemColorIcon.qsize)
	GVAR.GearFrame_Stats.GemYellowTexture:SetPoint("LEFT", GVAR.GearFrame_Stats.GemRedTexture, "RIGHT", 30, 0)
	GVAR.GearFrame_Stats.GemYellowTexture:SetTexture(Textures.GemColorIcon.path)
	GVAR.GearFrame_Stats.GemYellowTexture:SetTexCoord(unpack(Textures.GemColorIcon.coordsYel))
	GVAR.GearFrame_Stats.GemYellowText = GVAR.GearFrame_Stats:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
	GVAR.GearFrame_Stats.GemYellowText:SetPoint("LEFT", GVAR.GearFrame_Stats.GemYellowTexture, "RIGHT", 2, 0)
	GVAR.GearFrame_Stats.GemYellowText:SetJustifyH("LEFT")
	GVAR.GearFrame_Stats.GemYellowText:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])

	GVAR.GearFrame_Stats.GemMetaTexture = GVAR.GearFrame_Stats:CreateTexture(nil, "ARTWORK")
	GVAR.GearFrame_Stats.GemMetaTexture:SetWidth(Textures.GemColorIcon.qsize)
	GVAR.GearFrame_Stats.GemMetaTexture:SetHeight(Textures.GemColorIcon.qsize)
	GVAR.GearFrame_Stats.GemMetaTexture:SetPoint("TOP", GVAR.GearFrame_Stats.GemBlueTexture, "BOTTOM", 0, -5)
	GVAR.GearFrame_Stats.GemMetaTexture:SetTexture(Textures.GemColorIcon.path)
	GVAR.GearFrame_Stats.GemMetaTexture:SetTexCoord(unpack(Textures.GemColorIcon.coordsMet))
	GVAR.GearFrame_Stats.GemMetaText = GVAR.GearFrame_Stats:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
	GVAR.GearFrame_Stats.GemMetaText:SetPoint("LEFT", GVAR.GearFrame_Stats.GemMetaTexture, "RIGHT", 2, 0)
	GVAR.GearFrame_Stats.GemMetaText:SetJustifyH("LEFT")
	GVAR.GearFrame_Stats.GemMetaText:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])
	-- - Gear Stats END
	-- Gear Frame END



	-- PvE / PvP Achievement & Statistics Frame START
	GVAR.MainFrame.Stats = CreateFrame("Frame", nil, GVAR.MainFrame)
	TEMPLATE.BorderTRBL(GVAR.MainFrame.Stats)
	GVAR.MainFrame.Stats:SetWidth(sizeGeneral.MainWidth)
	GVAR.MainFrame.Stats:SetHeight(sizeStatFrame.BaseHeight)
	GVAR.MainFrame.Stats:SetMinResize(sizeGeneral.MainWidth, sizeStatFrame.Stats_minHeight) -- (minWidth, minHeight)
	GVAR.MainFrame.Stats:SetMaxResize(sizeGeneral.MainWidth, sizeStatFrame.Stats_maxHeight) -- (maxWidth, maxHeight)
	GVAR.MainFrame.Stats:SetPoint("TOPLEFT", GVAR.MainFrame.Equipment, "BOTTOMLEFT", 0, sizeGeneral.Border)
	GVAR.MainFrame.Stats:EnableMouse(true)
	GVAR.MainFrame.Stats:Hide()

	-- - PvE / PvP Switch Frame START
	GVAR.MainFrame.PvEPvPSwitch = CreateFrame("Frame", nil, GVAR.MainFrame.Stats)
	TEMPLATE.BorderTRBL(GVAR.MainFrame.PvEPvPSwitch)
	GVAR.MainFrame.PvEPvPSwitch:SetWidth(sizeGeneral.MainWidth)
	GVAR.MainFrame.PvEPvPSwitch:SetHeight(sizeGeneral.TabFrameHeight)
	GVAR.MainFrame.PvEPvPSwitch:SetPoint("TOPLEFT", GVAR.MainFrame.Stats, "TOPLEFT", 0, 0)
	GVAR.MainFrame.PvEPvPSwitch:EnableMouse(true)

	GVAR.MainFrame.PvEPvPSwitch.Title = GVAR.MainFrame.PvEPvPSwitch:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.MainFrame.PvEPvPSwitch.Title:SetPoint("TOPLEFT", GVAR.MainFrame.PvEPvPSwitch, "TOPLEFT", sizeGeneral.Offset, -sizeGeneral.Offset)
	GVAR.MainFrame.PvEPvPSwitch.Title:SetJustifyH("LEFT")
	GVAR.MainFrame.PvEPvPSwitch.Title:SetText(L["Statistics"]..":")

	GVAR.MainFrame.PvEPvPSwitch.Button1 = CreateFrame("Button", nil, GVAR.MainFrame.PvEPvPSwitch)
	TEMPLATE.TabButton(GVAR.MainFrame.PvEPvPSwitch.Button1, L["PvE"], sizeStatFrame.TabButtonWidth)
	GVAR.MainFrame.PvEPvPSwitch.Button1:SetWidth(sizeStatFrame.TabButtonWidth)
	GVAR.MainFrame.PvEPvPSwitch.Button1:SetHeight(sizeGeneral.TabButtonHeight)
	GVAR.MainFrame.PvEPvPSwitch.Button1:SetPoint("BOTTOMLEFT", GVAR.MainFrame.PvEPvPSwitch, "BOTTOMLEFT", sizeStatFrame.TabButton1Left, -sizeGeneral.Border)
	GVAR.MainFrame.PvEPvPSwitch.Button1:SetScript("OnClick", function() CharScanner:PvEPvPSwitch_OnClick("PvE") end)

	GVAR.MainFrame.PvEPvPSwitch.Button2 = CreateFrame("Button", nil, GVAR.MainFrame.PvEPvPSwitch)
	TEMPLATE.TabButton(GVAR.MainFrame.PvEPvPSwitch.Button2, L["PvP"], sizeStatFrame.TabButtonWidth)
	GVAR.MainFrame.PvEPvPSwitch.Button2:SetWidth(sizeStatFrame.TabButtonWidth)
	GVAR.MainFrame.PvEPvPSwitch.Button2:SetHeight(sizeGeneral.TabButtonHeight)
	GVAR.MainFrame.PvEPvPSwitch.Button2:SetPoint("BOTTOMLEFT", GVAR.MainFrame.PvEPvPSwitch, "BOTTOMLEFT", sizeStatFrame.TabButton2Left, -sizeGeneral.Border)
	GVAR.MainFrame.PvEPvPSwitch.Button2:SetScript("OnClick", function() CharScanner:PvEPvPSwitch_OnClick("PvP") end)

	GVAR.MainFrame.StatsPvE = CreateFrame("Frame", nil, GVAR.MainFrame.Stats)
	GVAR.MainFrame.StatsPvE:SetWidth(sizeGeneral.MainWidth)
	GVAR.MainFrame.StatsPvE:SetHeight(sizeStatFrame.BaseHeight)
	GVAR.MainFrame.StatsPvE:SetPoint("TOPLEFT", GVAR.MainFrame.PvEPvPSwitch, "BOTTOMLEFT", 0, 0)
	GVAR.MainFrame.StatsPvE:Hide()
	GVAR.MainFrame.StatsPvP = CreateFrame("Frame", nil, GVAR.MainFrame.Stats)
	GVAR.MainFrame.StatsPvP:SetWidth(sizeGeneral.MainWidth)
	GVAR.MainFrame.StatsPvP:SetHeight(sizeStatFrame.BaseHeight)
	GVAR.MainFrame.StatsPvP:SetPoint("TOPLEFT", GVAR.MainFrame.PvEPvPSwitch, "BOTTOMLEFT", 0, 0)
	GVAR.MainFrame.StatsPvP:Hide()
	-- - PvE / PvP Switch Frame END


	-- - PvE START
	GVAR.MainFrame.StatsPvE.SearchBox = CreateFrame("Button", nil, GVAR.MainFrame.StatsPvE)
	TEMPLATE.EditBox(GVAR.MainFrame.StatsPvE.SearchBox, 120, CharScanner.BossSearch, CharScanner.BossSearchClear)
	GVAR.MainFrame.StatsPvE.SearchBox:SetWidth(120)
	GVAR.MainFrame.StatsPvE.SearchBox:SetHeight(Textures.SearchBoxIcons.height)
	GVAR.MainFrame.StatsPvE.SearchBox:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvE, "TOPLEFT", sizeGeneral.Offset, -(sizeGeneral.Space+sizeGeneral.Border))

	GVAR.MainFrame.StatsPvE.Title = CreateFrame("Button", nil, GVAR.MainFrame.StatsPvE)
	TEMPLATE.PvEStats(GVAR.MainFrame.StatsPvE.Title, false, "title")
	GVAR.MainFrame.StatsPvE.Title:SetPoint("BOTTOMLEFT", GVAR.MainFrame.StatsPvE.SearchBox, "BOTTOMLEFT", 0, 0)
	GVAR.MainFrame.StatsPvE.Title.AchievementClicked = ""
	GVAR.MainFrame.StatsPvE.Title.AchievementDetailShow = false

	-- - - PvE scrollframe normal START
	GVAR.MainFrame.StatsPvE.ScrollFrame = CreateFrame("ScrollFrame", nil, GVAR.MainFrame.StatsPvE)
	TEMPLATE.ScrollFrame(GVAR.MainFrame.StatsPvE.ScrollFrame, true)
	GVAR.MainFrame.StatsPvE.ScrollFrame:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvE.Title, "BOTTOMLEFT", 0, -sizeGeneral.Space)
	GVAR.MainFrame.StatsPvE.ScrollFrame:SetPoint("BOTTOMRIGHT", GVAR.MainFrame.StatsPvE, "BOTTOMRIGHT", -(sizeGeneral.Space+sizeGeneral.SliderSize+sizeGeneral.Space+sizeGeneral.Border), sizeGeneral.Offset) -- -(2+17+2+1), 5
	GVAR.MainFrame.StatsPvE.ScrollFrame:SetScript("OnVerticalScroll", function(self, value)
		GVAR.MainFrame.StatsPvE.ScrollFrame.ScrollBar:SetValue(value)
		GVAR.MainFrame.StatsPvE.ScrollFrame.offset = value / sizeGeneral.BarHeight--math_floor((value / sizeGeneral.BarHeight) + 0.5)
		GVAR.MainFrame.StatsPvE.ScrollFrame["L1"..PVEDUMP[1].name]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvE.Title, "BOTTOMLEFT", 0, value-2)
		ScrollFrameUpdate()
	end)
	-- - - PvE scrollframe normal END

	-- - - PvE scrollframe search Sort START
	GVAR.MainFrame.StatsPvE.SearchSort = CreateFrame("Frame", nil, GVAR.MainFrame.StatsPvE)
	GVAR.MainFrame.StatsPvE.SearchSort:SetWidth(sizeStatFrame.ButtonWidth)
	GVAR.MainFrame.StatsPvE.SearchSort:SetHeight(sizeGeneral.BarHeight)
	GVAR.MainFrame.StatsPvE.SearchSort:SetPoint("BOTTOMLEFT", GVAR.MainFrame.StatsPvE.SearchBox, "BOTTOMLEFT", 0, 0)
	GVAR.MainFrame.StatsPvE.SearchSort:Hide()

	GVAR.MainFrame.StatsPvE.SearchSort.Standard = CreateFrame("Button", nil, GVAR.MainFrame.StatsPvE.SearchSort)
	TEMPLATE.SortPosition(GVAR.MainFrame.StatsPvE.SearchSort.Standard, L["Standard"], "left", sizeStatFrame.SearchSortStdWidth)
	GVAR.MainFrame.StatsPvE.SearchSort.Standard:SetWidth(sizeStatFrame.SearchSortStdWidth)
	GVAR.MainFrame.StatsPvE.SearchSort.Standard:SetHeight(sizeGeneral.BarHeight)
	GVAR.MainFrame.StatsPvE.SearchSort.Standard:SetPoint("LEFT", GVAR.MainFrame.StatsPvE.SearchSort, "LEFT", sizeStatFrame.SearchSortStdLeft, 0)
	GVAR.MainFrame.StatsPvE.SearchSort.Standard:SetScript("OnClick", function() CharScanner:ABKsearch_GUISort(1) end)

	GVAR.MainFrame.StatsPvE.SearchSort.Name = CreateFrame("Button", nil, GVAR.MainFrame.StatsPvE.SearchSort)
	TEMPLATE.SortPosition(GVAR.MainFrame.StatsPvE.SearchSort.Name, NAME, "left", sizeStatFrame.SearchSortNameWidth)
	GVAR.MainFrame.StatsPvE.SearchSort.Name:SetWidth(sizeStatFrame.SearchSortNameWidth)
	GVAR.MainFrame.StatsPvE.SearchSort.Name:SetHeight(sizeGeneral.BarHeight)
	GVAR.MainFrame.StatsPvE.SearchSort.Name:SetPoint("LEFT", GVAR.MainFrame.StatsPvE.SearchSort, "LEFT", sizeStatFrame.SearchSortNameLeft, 0)
	GVAR.MainFrame.StatsPvE.SearchSort.Name:SetScript("OnClick", function() CharScanner:ABKsearch_GUISort(2) end)

	GVAR.MainFrame.StatsPvE.SearchSort.Kills = CreateFrame("Button", nil, GVAR.MainFrame.StatsPvE.SearchSort)
	TEMPLATE.SortPosition(GVAR.MainFrame.StatsPvE.SearchSort.Kills, L["Kills"], "right", sizeStatFrame.SearchSortKillsWidth)
	GVAR.MainFrame.StatsPvE.SearchSort.Kills:SetWidth(sizeStatFrame.SearchSortKillsWidth)
	GVAR.MainFrame.StatsPvE.SearchSort.Kills:SetHeight(sizeGeneral.BarHeight)
	GVAR.MainFrame.StatsPvE.SearchSort.Kills:SetPoint("RIGHT", GVAR.MainFrame.StatsPvE.SearchSort, "RIGHT", -sizeStatFrame.BossesTotalRight, 0)
	GVAR.MainFrame.StatsPvE.SearchSort.Kills:SetScript("OnClick", function() CharScanner:ABKsearch_GUISort(3) end)
	-- - - PvE scrollframe search Sort END

	-- - - PvE scrollframe search START
	GVAR.MainFrame.StatsPvE.SearchScrollFrame = CreateFrame("ScrollFrame", nil, GVAR.MainFrame.StatsPvE)
	TEMPLATE.ScrollFrame(GVAR.MainFrame.StatsPvE.SearchScrollFrame)
	GVAR.MainFrame.StatsPvE.SearchScrollFrame:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvE.Title, "BOTTOMLEFT", 0, -sizeGeneral.Space)
	GVAR.MainFrame.StatsPvE.SearchScrollFrame:SetPoint("BOTTOMRIGHT", GVAR.MainFrame.StatsPvE, "BOTTOMRIGHT", -(sizeGeneral.Space+sizeGeneral.SliderSize+sizeGeneral.Space+sizeGeneral.Border), sizeGeneral.Offset) -- -(2+17+2+1), 5
	GVAR.MainFrame.StatsPvE.SearchScrollFrame:SetScript("OnVerticalScroll", function(self, value) CharScanner:ABKsearch_OnVerticalScroll(value) end)
	GVAR.MainFrame.StatsPvE.SearchScrollFrame:Hide()
	for i = 1, GUI_ABKsearch_maxRows do
		if not GVAR.MainFrame.StatsPvE.SearchScrollFrameButton then GVAR.MainFrame.StatsPvE.SearchScrollFrameButton = {} end
		GVAR.MainFrame.StatsPvE.SearchScrollFrameButton[i] = CreateFrame("Button", nil, GVAR.MainFrame.StatsPvE.SearchScrollFrame)
		TEMPLATE.PvEStats(GVAR.MainFrame.StatsPvE.SearchScrollFrameButton[i], false, "search", i)
		GVAR.MainFrame.StatsPvE.SearchScrollFrameButton[i]:SetWidth(sizeStatFrame.ButtonWidth)
		GVAR.MainFrame.StatsPvE.SearchScrollFrameButton[i]:SetHeight(sizeGeneral.BarHeight)
		if i == 1 then
			GVAR.MainFrame.StatsPvE.SearchScrollFrameButton[i]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvE.SearchScrollFrame, "TOPLEFT", 0, 0)
		else
			GVAR.MainFrame.StatsPvE.SearchScrollFrameButton[i]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvE.SearchScrollFrameButton[(i-1)], "BOTTOMLEFT", 0, 0)
		end
	end
	-- - - PvE scrollframe search END

	local TEMPpvestats = {}
	-- - - PvE level 1 START
	for i = 1, #PVEDUMP do
		GVAR.MainFrame.StatsPvE.ScrollFrame["L1"..PVEDUMP[i].name] = CreateFrame("Button", nil, GVAR.MainFrame.StatsPvE.ScrollFrame.ScrollChildFrame)
		TEMPLATE.PvEStats(GVAR.MainFrame.StatsPvE.ScrollFrame["L1"..PVEDUMP[i].name], true, "level1", i)
		if i == 1 then
			GVAR.MainFrame.StatsPvE.ScrollFrame["L1"..PVEDUMP[i].name]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvE.Title, "BOTTOMLEFT", 0, -sizeGeneral.Space)
		else
			GVAR.MainFrame.StatsPvE.ScrollFrame["L1"..PVEDUMP[i].name]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvE.ScrollFrame["L1"..PVEDUMP[(i-1)].name], "BOTTOMLEFT", 0, 0)
		end
		GVAR.MainFrame.StatsPvE.ScrollFrame["L1"..PVEDUMP[i].name]:SetScript("OnClick", function() CharScanner:NodePvE_Click("PvE", 1, PVEDUMP[i].name, i) end)
		--GVAR.MainFrame.StatsPvE.ScrollFrame["L1"..PVEDUMP[i].name].Name:SetText("StatsLevel1_"..PVEDUMP[i].name) -- MENUDEV

		local n = "L1"..PVEDUMP[i].name
		TEMPpvestats[n] = {}
		TEMPpvestats[n].slevel = 1
		TEMPpvestats[n].index1 = i
		TEMPpvestats[n].index2 = 0
		TEMPpvestats[n].index3 = 0
		TEMPpvestats[n].height = sizeGeneral.BarHeight
		TEMPpvestats[n].collapse = false
	end
	-- - - PvE level 1 END

	-- - - PvE level 2 START
	for i = 1, #PVEDUMP do
		if PVEDUMP[i].statistic then
			local height = 0
			GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name] = CreateFrame("Frame", nil, GVAR.MainFrame.StatsPvE.ScrollFrame["L1"..PVEDUMP[i].name])
			GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name]:SetWidth(sizeGeneral.MainWidth)
			GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvE.ScrollFrame["L1"..PVEDUMP[i].name], "BOTTOMLEFT", 0, 0)
			for j = 1, #PVEDUMP[i].statistic do
				GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..j] = CreateFrame("Button", nil, GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name])
				TEMPLATE.PvEStats(GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..j], false, "level2")
				if j == 1 then
					GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..j]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name], "TOPLEFT", 0, 0)
				else
					GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..j]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..(j-1)], "BOTTOMLEFT", 0, 0)
				end
				--GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..j].Name:SetText("L2"..PVEDUMP[i].name..j) -- MENUDEV
				height = height + 1
			end
			GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name]:SetHeight(height*sizeGeneral.BarHeight)
			GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name]:Hide()

			local n = "L2"..PVEDUMP[i].name
			TEMPpvestats[n] = {}
			TEMPpvestats[n].slevel = 2
			TEMPpvestats[n].index1 = i
			TEMPpvestats[n].index2 = 0
			TEMPpvestats[n].index3 = 0
			TEMPpvestats[n].height = height*sizeGeneral.BarHeight
			TEMPpvestats[n].collapse = false
		else
			local height = 0
			GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name] = CreateFrame("Frame", nil, GVAR.MainFrame.StatsPvE.ScrollFrame["L1"..PVEDUMP[i].name])
			GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name]:SetWidth(sizeGeneral.MainWidth)
			GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvE.ScrollFrame["L1"..PVEDUMP[i].name], "BOTTOMLEFT", 0, 0)
			for j = 1, #PVEDUMP[i] do
				GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..PVEDUMP[i][j].name] = CreateFrame("Button", nil, GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name])
				TEMPLATE.PvEStats(GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..PVEDUMP[i][j].name], true, "level2", i, j)
				if j == 1 then
					GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..PVEDUMP[i][j].name]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name], "TOPLEFT", 0, 0)
				else
					GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..PVEDUMP[i][j].name]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..PVEDUMP[i][(j-1)].name], "BOTTOMLEFT", 0, 0)
				end
				GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..PVEDUMP[i][j].name]:SetScript("OnClick", function() CharScanner:NodePvE_Click("PvE", 2, PVEDUMP[i].name, i, PVEDUMP[i][j].name, j) end)
				--GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..PVEDUMP[i][j].name].Name:SetText("L2"..PVEDUMP[i].name..PVEDUMP[i][j].name) -- MENUDEV
				height = height + 1
			end
			GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name]:SetHeight(height*sizeGeneral.BarHeight)
			GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name]:Hide()

			local n = "L2"..PVEDUMP[i].name
			TEMPpvestats[n] = {}
			TEMPpvestats[n].slevel = 2
			TEMPpvestats[n].index1 = i
			TEMPpvestats[n].index2 = 0
			TEMPpvestats[n].index3 = 0
			TEMPpvestats[n].height = height*sizeGeneral.BarHeight
			TEMPpvestats[n].collapse = false
		end
	end
	-- - - PvE level 2 END

	-- - - PvE level 3 START
	for i = 1, #PVEDUMP do
		if not PVEDUMP[i].statistic then
			for j = 1, #PVEDUMP[i] do

				if PVEDUMP[i][j].statistic then
					local height = 0
					GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name] = CreateFrame("Frame", nil, GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name])
					GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name]:SetWidth(sizeGeneral.MainWidth)
					GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..PVEDUMP[i][j].name], "BOTTOMLEFT", 0, 0)
					for k = 1, #PVEDUMP[i][j].statistic do
						GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k] = CreateFrame("Button", nil, GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name])
						TEMPLATE.PvEStats(GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k], false, "level3")
						if k == 1 then
							GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name], "TOPLEFT", 0, 0)
						else
							GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..(k-1)], "BOTTOMLEFT", 0, 0)
						end
						--GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k].Name:SetText("L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k) -- MENUDEV
						height = height + 1
					end
					GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name]:SetHeight(height*sizeGeneral.BarHeight)
					GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name]:Hide()

					local n = "L3"..PVEDUMP[i].name..PVEDUMP[i][j].name
					TEMPpvestats[n] = {}
					TEMPpvestats[n].slevel = 3
					TEMPpvestats[n].index1 = i
					TEMPpvestats[n].index2 = j
					TEMPpvestats[n].index3 = 0
					TEMPpvestats[n].height = height*sizeGeneral.BarHeight
					TEMPpvestats[n].collapse = false
				else
					local height = 0
					GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name] = CreateFrame("Frame", nil, GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name])
					GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name]:SetWidth(sizeGeneral.MainWidth)
					GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..PVEDUMP[i][j].name], "BOTTOMLEFT", 0, 0)
					for k = 1, #PVEDUMP[i][j] do
						GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k] = CreateFrame("Button", nil, GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name])
						if PVEDUMP[i][j][k].noboss then --NOBOSS
							TEMPLATE.PvEStats(GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k], true, "level3", i, j, k, "NOBOSS")
						else
							TEMPLATE.PvEStats(GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k], true, "level3", i, j, k)
						end
						if k == 1 then
							GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name], "TOPLEFT", 0, 0)
						else
							GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..(k-1)], "BOTTOMLEFT", 0, 0)
						end
						if not PVEDUMP[i][j][k].noboss then --NOBOSS
							GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k]:SetScript("OnClick", function() CharScanner:NodePvE_Click("PvE", 3, PVEDUMP[i].name, i, PVEDUMP[i][j].name, j, PVEDUMP[i][j][k].name, k) end)
						end
						--GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k].Name:SetText("L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k) -- MENUDEV
						height = height + 1
					end
					GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name]:SetHeight(height*sizeGeneral.BarHeight)
					GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name]:Hide()

					local n = "L3"..PVEDUMP[i].name..PVEDUMP[i][j].name
					TEMPpvestats[n] = {}
					TEMPpvestats[n].slevel = 3
					TEMPpvestats[n].index1 = i
					TEMPpvestats[n].index2 = j
					TEMPpvestats[n].index3 = 0
					TEMPpvestats[n].height = height*sizeGeneral.BarHeight
					TEMPpvestats[n].collapse = false
				end

			end
		end
	end
	-- - - PvE level 3 END

	-- - - PvE level 4 START
	for i = 1, #PVEDUMP do
		if not PVEDUMP[i].statistic then
			for j = 1, #PVEDUMP[i] do

				if not PVEDUMP[i][j].statistic then
					for k = 1, #PVEDUMP[i][j] do

						if PVEDUMP[i][j][k].statistic then
							local height = 0
							GVAR.MainFrame.StatsPvE.ScrollFrame["L4"..PVEDUMP[i].name..PVEDUMP[i][j].name..PVEDUMP[i][j][k].name] = CreateFrame("Frame", nil, GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name])
							GVAR.MainFrame.StatsPvE.ScrollFrame["L4"..PVEDUMP[i].name..PVEDUMP[i][j].name..PVEDUMP[i][j][k].name]:SetWidth(sizeGeneral.MainWidth)
							GVAR.MainFrame.StatsPvE.ScrollFrame["L4"..PVEDUMP[i].name..PVEDUMP[i][j].name..PVEDUMP[i][j][k].name]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k], "BOTTOMLEFT", 0, 0)
							for l = 1, #PVEDUMP[i][j][k].statistic do
								GVAR.MainFrame.StatsPvE.ScrollFrame["L4"..PVEDUMP[i].name..PVEDUMP[i][j].name..PVEDUMP[i][j][k].name..l] = CreateFrame("Button", nil, GVAR.MainFrame.StatsPvE.ScrollFrame["L4"..PVEDUMP[i].name..PVEDUMP[i][j].name..PVEDUMP[i][j][k].name])
								TEMPLATE.PvEStats(GVAR.MainFrame.StatsPvE.ScrollFrame["L4"..PVEDUMP[i].name..PVEDUMP[i][j].name..PVEDUMP[i][j][k].name..l], false, "level4")
								if l == 1 then
									GVAR.MainFrame.StatsPvE.ScrollFrame["L4"..PVEDUMP[i].name..PVEDUMP[i][j].name..PVEDUMP[i][j][k].name..l]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvE.ScrollFrame["L4"..PVEDUMP[i].name..PVEDUMP[i][j].name..PVEDUMP[i][j][k].name], "TOPLEFT", 0, 0)
								else
									GVAR.MainFrame.StatsPvE.ScrollFrame["L4"..PVEDUMP[i].name..PVEDUMP[i][j].name..PVEDUMP[i][j][k].name..l]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvE.ScrollFrame["L4"..PVEDUMP[i].name..PVEDUMP[i][j].name..PVEDUMP[i][j][k].name..(l-1)], "BOTTOMLEFT", 0, 0)
								end
								--GVAR.MainFrame.StatsPvE.ScrollFrame["L4"..PVEDUMP[i].name..PVEDUMP[i][j].name..PVEDUMP[i][j][k].name..l].Name:SetText("L4"..PVEDUMP[i].name..PVEDUMP[i][j].name..PVEDUMP[i][j][k].name..l) -- MENUDEV
								height = height + 1
							end
							GVAR.MainFrame.StatsPvE.ScrollFrame["L4"..PVEDUMP[i].name..PVEDUMP[i][j].name..PVEDUMP[i][j][k].name]:SetHeight(height*sizeGeneral.BarHeight)
							GVAR.MainFrame.StatsPvE.ScrollFrame["L4"..PVEDUMP[i].name..PVEDUMP[i][j].name..PVEDUMP[i][j][k].name]:Hide()

							local n = "L4"..PVEDUMP[i].name..PVEDUMP[i][j].name..PVEDUMP[i][j][k].name
							TEMPpvestats[n] = {}
							TEMPpvestats[n].slevel = 4
							TEMPpvestats[n].index1 = i
							TEMPpvestats[n].index2 = j
							TEMPpvestats[n].index3 = k
							TEMPpvestats[n].height = height*sizeGeneral.BarHeight
							TEMPpvestats[n].collapse = false
						end

					end
				end

			end
		end
	end
	-- - - PvE level 4 END

	-- - - PvE stats sort START
	local ni = 1
	for key, value in SortedPairs(TEMPpvestats, 1) do
		GUI_PvEstats_menu[ni] = {}
		GUI_PvEstats_menu[ni].name     = key
		GUI_PvEstats_menu[ni].slevel   = value.slevel
		GUI_PvEstats_menu[ni].index1   = value.index1
		GUI_PvEstats_menu[ni].index2   = value.index2
		GUI_PvEstats_menu[ni].index3   = value.index3
		GUI_PvEstats_menu[ni].height   = value.height
		GUI_PvEstats_menu[ni].collapse = value.collapse
		ni = ni + 1
	end
	table_wipe(TEMPpvestats)
	-- - - PvE stats sort END
	-- - PvE END

	-- - PvP START
	GVAR.MainFrame.StatsPvP.Title = CreateFrame("Button", nil, GVAR.MainFrame.StatsPvP)
	TEMPLATE.PvPStats(GVAR.MainFrame.StatsPvP.Title, false, "title")
	GVAR.MainFrame.StatsPvP.Title:SetPoint("BOTTOMLEFT", GVAR.MainFrame.StatsPvE.SearchBox, "BOTTOMLEFT", 0, 0)
	GVAR.MainFrame.StatsPvP.Title.AchievementClicked = ""
	GVAR.MainFrame.StatsPvP.Title.AchievementDetailShow = false

	-- - - PvP scrollframe normal START
	GVAR.MainFrame.StatsPvP.ScrollFrame = CreateFrame("ScrollFrame", nil, GVAR.MainFrame.StatsPvP)
	TEMPLATE.ScrollFrame(GVAR.MainFrame.StatsPvP.ScrollFrame, true)
	GVAR.MainFrame.StatsPvP.ScrollFrame:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvP.Title, "BOTTOMLEFT", 0, -sizeGeneral.Space)
	GVAR.MainFrame.StatsPvP.ScrollFrame:SetPoint("BOTTOMRIGHT", GVAR.MainFrame.StatsPvP, "BOTTOMRIGHT", -(sizeGeneral.Space+sizeGeneral.SliderSize+sizeGeneral.Space+sizeGeneral.Border), sizeGeneral.Offset) -- -(2+17+2+1), 5
	GVAR.MainFrame.StatsPvP.ScrollFrame:SetScript("OnVerticalScroll", function(self, value)
		GVAR.MainFrame.StatsPvP.ScrollFrame.ScrollBar:SetValue(value)
		GVAR.MainFrame.StatsPvP.ScrollFrame.offset = value / sizeGeneral.BarHeight--math_floor((value / sizeGeneral.BarHeight) + 0.5)
		GVAR.MainFrame.StatsPvP.ScrollFrame["L1"..PVPDUMP[1].name]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvP.Title, "BOTTOMLEFT", 0, value-2)
		ScrollFrameUpdate()
	end)
	-- - - PvP scrollframe normal END

	local TEMPpvpstats = {}
	-- - - PvP level 1 START
	for i = 1, #PVPDUMP do
		GVAR.MainFrame.StatsPvP.ScrollFrame["L1"..PVPDUMP[i].name] = CreateFrame("Button", nil, GVAR.MainFrame.StatsPvP.ScrollFrame.ScrollChildFrame)
		if PVPDUMP[i].statisticHide then
			TEMPLATE.PvPStats(GVAR.MainFrame.StatsPvP.ScrollFrame["L1"..PVPDUMP[i].name], false, "level1", i)
		else
			TEMPLATE.PvPStats(GVAR.MainFrame.StatsPvP.ScrollFrame["L1"..PVPDUMP[i].name], true, "level1", i)
		end
		if i == 1 then
			GVAR.MainFrame.StatsPvP.ScrollFrame["L1"..PVPDUMP[i].name]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvP.Title, "BOTTOMLEFT", 0, -sizeGeneral.Space)
		else
			GVAR.MainFrame.StatsPvP.ScrollFrame["L1"..PVPDUMP[i].name]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvP.ScrollFrame["L1"..PVPDUMP[(i-1)].name], "BOTTOMLEFT", 0, 0)
		end
		if not PVPDUMP[i].statisticHide then
			GVAR.MainFrame.StatsPvP.ScrollFrame["L1"..PVPDUMP[i].name]:SetScript("OnClick", function() CharScanner:NodePvP_Click("PvP", 1, PVPDUMP[i].name, i) end)
		end
		--GVAR.MainFrame.StatsPvP.ScrollFrame["L1"..PVPDUMP[i].name].Name:SetText("level1".."StatsLevel1_"..PVPDUMP[i].name) -- TEST

		local n = "L1"..PVPDUMP[i].name
		TEMPpvpstats[n] = {}
		TEMPpvpstats[n].slevel = 1
		TEMPpvpstats[n].index1 = i
		TEMPpvpstats[n].index2 = 0
		TEMPpvpstats[n].index3 = 0
		TEMPpvpstats[n].height = sizeGeneral.BarHeight
		TEMPpvpstats[n].collapse = false
	end
	-- - - PvP level 1 END

	-- - - PvP level 2 START
	for i = 1, #PVPDUMP do
		if PVPDUMP[i].statistic then
			local height = 0
			GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name] = CreateFrame("Frame", nil, GVAR.MainFrame.StatsPvP.ScrollFrame["L1"..PVPDUMP[i].name])
			GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name]:SetWidth(sizeGeneral.MainWidth)
			GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvP.ScrollFrame["L1"..PVPDUMP[i].name], "BOTTOMLEFT", 0, 0)
			for j = 1, PVPDUMP[i].statisticTotal do
				GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name..j] = CreateFrame("Button", nil, GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name])
				TEMPLATE.PvPStats(GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name..j], false, "level2")
				if j == 1 then
					GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name..j]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name], "TOPLEFT", 0, 0)
				else
					GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name..j]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name..(j-1)], "BOTTOMLEFT", 0, 0)
				end
				--GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name..j].Name:SetText("level2".."StatsLevel2_"..PVPDUMP[i].name..j) -- TEST
				height = height + 1
			end
			GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name]:SetHeight(height*sizeGeneral.BarHeight)
			GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name]:Hide()

			local n = "L2"..PVPDUMP[i].name
			TEMPpvpstats[n] = {}
			TEMPpvpstats[n].slevel = 2
			TEMPpvpstats[n].index1 = i
			TEMPpvpstats[n].index2 = 0
			TEMPpvpstats[n].index3 = 0
			TEMPpvpstats[n].height = height*sizeGeneral.BarHeight
			TEMPpvpstats[n].collapse = false
		else
			local height = 0
			GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name] = CreateFrame("Frame", nil, GVAR.MainFrame.StatsPvP.ScrollFrame["L1"..PVPDUMP[i].name])
			GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name]:SetWidth(sizeGeneral.MainWidth)
			GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvP.ScrollFrame["L1"..PVPDUMP[i].name], "BOTTOMLEFT", 0, 0)
			for j = 1, #PVPDUMP[i] do
				GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name..PVPDUMP[i][j].name] = CreateFrame("Button", nil, GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name])
				if PVPDUMP[i][j].statisticHide then
					TEMPLATE.PvPStats(GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name..PVPDUMP[i][j].name], false, "level2", i, j)
				else
					TEMPLATE.PvPStats(GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name..PVPDUMP[i][j].name], true, "level2", i, j)
				end
				if j == 1 then
					GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name..PVPDUMP[i][j].name]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name], "TOPLEFT", 0, 0)
				else
					GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name..PVPDUMP[i][j].name]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name..PVPDUMP[i][(j-1)].name], "BOTTOMLEFT", 0, 0)
				end
				if not PVPDUMP[i][j].statisticHide then
					GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name..PVPDUMP[i][j].name]:SetScript("OnClick", function() CharScanner:NodePvP_Click("PvP", 2, PVPDUMP[i].name, i, PVPDUMP[i][j].name, j) end)
				end
				--GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name..PVPDUMP[i][j].name].Name:SetText("level2".."StatsLevel2_"..PVPDUMP[i].name..PVPDUMP[i][j].name) -- TEST
				height = height + 1
			end
			GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name]:SetHeight(height*sizeGeneral.BarHeight)
			GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name]:Hide()

			local n = "L2"..PVPDUMP[i].name
			TEMPpvpstats[n] = {}
			TEMPpvpstats[n].slevel = 2
			TEMPpvpstats[n].index1 = i
			TEMPpvpstats[n].index2 = 0
			TEMPpvpstats[n].index3 = 0
			TEMPpvpstats[n].height = height*sizeGeneral.BarHeight
			TEMPpvpstats[n].collapse = false
		end
	end
	-- - - PvP level 2 END

	-- - - PvP level 3 START
	for i = 1, #PVPDUMP do
		if not PVPDUMP[i].statistic then
			for j = 1, #PVPDUMP[i] do

				if PVPDUMP[i][j].statistic then
					local height = 0
					GVAR.MainFrame.StatsPvP.ScrollFrame["L3"..PVPDUMP[i].name..PVPDUMP[i][j].name] = CreateFrame("Frame", nil, GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name])
					GVAR.MainFrame.StatsPvP.ScrollFrame["L3"..PVPDUMP[i].name..PVPDUMP[i][j].name]:SetWidth(sizeGeneral.MainWidth)
					GVAR.MainFrame.StatsPvP.ScrollFrame["L3"..PVPDUMP[i].name..PVPDUMP[i][j].name]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name..PVPDUMP[i][j].name], "BOTTOMLEFT", 0, 0)
					for k = 1, PVPDUMP[i][j].statisticTotal do
						GVAR.MainFrame.StatsPvP.ScrollFrame["L3"..PVPDUMP[i].name..PVPDUMP[i][j].name..k] = CreateFrame("Button", nil, GVAR.MainFrame.StatsPvP.ScrollFrame["L3"..PVPDUMP[i].name..PVPDUMP[i][j].name])
						TEMPLATE.PvPStats(GVAR.MainFrame.StatsPvP.ScrollFrame["L3"..PVPDUMP[i].name..PVPDUMP[i][j].name..k], false, "level3")
						if k == 1 then
							GVAR.MainFrame.StatsPvP.ScrollFrame["L3"..PVPDUMP[i].name..PVPDUMP[i][j].name..k]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvP.ScrollFrame["L3"..PVPDUMP[i].name..PVPDUMP[i][j].name], "TOPLEFT", 0, 0)
						else
							GVAR.MainFrame.StatsPvP.ScrollFrame["L3"..PVPDUMP[i].name..PVPDUMP[i][j].name..k]:SetPoint("TOPLEFT", GVAR.MainFrame.StatsPvP.ScrollFrame["L3"..PVPDUMP[i].name..PVPDUMP[i][j].name..(k-1)], "BOTTOMLEFT", 0, 0)
						end
						--GVAR.MainFrame.StatsPvP.ScrollFrame["L3"..PVPDUMP[i].name..PVPDUMP[i][j].name..k].Name:SetText("level3".."StatsLevel3_"..PVPDUMP[i].name..PVPDUMP[i][j].name..k) -- TEST
						height = height + 1
					end
					GVAR.MainFrame.StatsPvP.ScrollFrame["L3"..PVPDUMP[i].name..PVPDUMP[i][j].name]:SetHeight(height*sizeGeneral.BarHeight)
					GVAR.MainFrame.StatsPvP.ScrollFrame["L3"..PVPDUMP[i].name..PVPDUMP[i][j].name]:Hide()

					local n = "L3"..PVPDUMP[i].name..PVPDUMP[i][j].name
					TEMPpvpstats[n] = {}
					TEMPpvpstats[n].slevel = 3
					TEMPpvpstats[n].index1 = i
					TEMPpvpstats[n].index2 = j
					TEMPpvpstats[n].index3 = 0
					TEMPpvpstats[n].height = height*sizeGeneral.BarHeight
					TEMPpvpstats[n].collapse = false
				end

			end
		end
	end
	-- - - PvP level 3 END

	-- - - PvP stats sort START
	local ni = 1
	for key, value in SortedPairs(TEMPpvpstats, 1) do
		GUI_PvPstats_menu[ni] = {}
		GUI_PvPstats_menu[ni].name     = key
		GUI_PvPstats_menu[ni].slevel   = value.slevel
		GUI_PvPstats_menu[ni].index1   = value.index1
		GUI_PvPstats_menu[ni].index2   = value.index2
		GUI_PvPstats_menu[ni].index3   = value.index3
		GUI_PvPstats_menu[ni].height   = value.height
		GUI_PvPstats_menu[ni].collapse = value.collapse
		ni = ni + 1
	end
	table_wipe(TEMPpvpstats)
	-- - - PvP stats sort END
	-- - PvP END
	-- PvE / PvP Achievement & Statistics Frame END



	-- AchievementDetailFrame START
	GVAR.AchievementDetailFrame = CreateFrame("Frame", "CharScanner_AchievementDetailFrame", GVAR.MainFrame.Stats)
	TEMPLATE.BorderTRBL(GVAR.AchievementDetailFrame)
	GVAR.AchievementDetailFrame:SetWidth(sizeAchievementDetail.Width)
	GVAR.AchievementDetailFrame:SetHeight(sizeAchievementDetail.Height)
	GVAR.AchievementDetailFrame:SetPoint("TOPLEFT", GVAR.MainFrame.Base, "TOPRIGHT", -sizeGeneral.Border, 0)
	GVAR.AchievementDetailFrame:EnableMouse(true)
	GVAR.AchievementDetailFrame:SetToplevel(true)
	GVAR.AchievementDetailFrame:Hide()

	-- - AchievementDetail headline START
	--GVAR.AchievementDetailFrame.Headline = CreateFrame("Frame", nil, GVAR.AchievementDetailFrame)
	--TEMPLATE.BorderTRBL(headline)
	--GVAR.AchievementDetailFrame.Headline:SetWidth(sizeAchievementDetail.Width)
	--GVAR.AchievementDetailFrame.Headline:SetHeight(sizeGeneral.FreeBarHeight)
	--GVAR.AchievementDetailFrame.Headline:SetPoint("BOTTOMLEFT", GVAR.AchievementDetailFrame, "TOPLEFT", 0, -sizeGeneral.Border)
	--GVAR.AchievementDetailFrame.Headline:EnableMouse(true)
	--GVAR.AchievementDetailFrame.Headline:Hide()

	GVAR.AchievementDetailFrame.Close = CreateFrame("Button", nil, GVAR.AchievementDetailFrame)
	TEMPLATE.IconButton(GVAR.AchievementDetailFrame.Close, 9, 12)
	GVAR.AchievementDetailFrame.Close:SetPoint("BOTTOMRIGHT", GVAR.AchievementDetailFrame, "TOPRIGHT", 0, -sizeGeneral.Border)
	GVAR.AchievementDetailFrame.Close:SetScript("OnClick", function() CharScanner:AchievementDetail_Hide(AchievementDetailFrameMode) end)
	--GVAR.AchievementDetailFrame.Headline.Free = CreateFrame("Button", nil, GVAR.AchievementDetailFrame)
	--TEMPLATE.IconButton(GVAR.AchievementDetailFrame.Headline.Free, 11, 12)
	--GVAR.AchievementDetailFrame.Headline.Free:SetPoint("TOPRIGHT", ???, "TOPLEFT", 0, 0)
	--GVAR.AchievementDetailFrame.Headline.Free:SetScript("OnClick", function() CharScanner:AchievementDetail_FreeBarToggle() end)
	-- - AchievementDetail headline END

	-- - AchievementDetail Sort START
	GVAR.AchievementDetailFrame.Sort = CreateFrame("Frame", nil, GVAR.AchievementDetailFrame)
	GVAR.AchievementDetailFrame.Sort:SetWidth(sizeAchievementDetail.ButtonWidth)
	GVAR.AchievementDetailFrame.Sort:SetHeight(sizeGeneral.BarHeight)
	GVAR.AchievementDetailFrame.Sort:SetPoint("TOPLEFT", GVAR.AchievementDetailFrame, "TOPLEFT", sizeGeneral.Offset, -sizeAchievementDetail.SortLine2Top)

	GVAR.AchievementDetailFrame.Sort.ACI = CreateFrame("Button", nil, GVAR.AchievementDetailFrame.Sort)
	TEMPLATE.PullDownMenu(GVAR.AchievementDetailFrame.Sort.ACI, "ACI", PullDownMenu_ACI[1], 150, #PullDownMenu_ACI, PullDownMenuFunc_ACI)
	GVAR.AchievementDetailFrame.Sort.ACI:SetWidth(150)
	GVAR.AchievementDetailFrame.Sort.ACI:SetHeight(sizeAchievementDetail.SortLine1Height)
	GVAR.AchievementDetailFrame.Sort.ACI:SetPoint("TOPLEFT", GVAR.AchievementDetailFrame.Sort, "TOPLEFT", 0, sizeAchievementDetail.SortLine1Height+sizeGeneral.Space)

	GVAR.AchievementDetailFrame.Sort.Id = CreateFrame("Button", nil, GVAR.AchievementDetailFrame.Sort)
	TEMPLATE.SortPosition(GVAR.AchievementDetailFrame.Sort.Id, "ID", "right", sizeAchievementDetail.IdWidth)
	GVAR.AchievementDetailFrame.Sort.Id:SetWidth(sizeAchievementDetail.IdWidth)
	GVAR.AchievementDetailFrame.Sort.Id:SetHeight(sizeGeneral.BarHeight)
	GVAR.AchievementDetailFrame.Sort.Id:SetPoint("LEFT", GVAR.AchievementDetailFrame.Sort, "LEFT", sizeAchievementDetail.IdLeft, 0)
	GVAR.AchievementDetailFrame.Sort.Id:SetScript("OnClick", function() CharScanner:AchievementDetail_GUISort(1) end)

	GVAR.AchievementDetailFrame.Sort.Name = CreateFrame("Button", nil, GVAR.AchievementDetailFrame.Sort)
	TEMPLATE.SortPosition(GVAR.AchievementDetailFrame.Sort.Name, NAME, "left", sizeAchievementDetail.NameWidth + sizeGeneral.Space + sizeAchievementDetail.CompareWidth)
	GVAR.AchievementDetailFrame.Sort.Name:SetWidth(sizeAchievementDetail.NameWidth + sizeGeneral.Space + sizeAchievementDetail.CompareWidth)
	GVAR.AchievementDetailFrame.Sort.Name:SetHeight(sizeGeneral.BarHeight)
	GVAR.AchievementDetailFrame.Sort.Name:SetPoint("LEFT", GVAR.AchievementDetailFrame.Sort, "LEFT", sizeAchievementDetail.NameLeft, 0)
	GVAR.AchievementDetailFrame.Sort.Name:SetScript("OnClick", function() CharScanner:AchievementDetail_GUISort(2) end)
	-- - AchievementDetail Sort END

	-- - AchievementDetail ScrollFrame START
	GVAR.AchievementDetailFrame.ScrollFrame = CreateFrame("ScrollFrame", nil, GVAR.AchievementDetailFrame)
	TEMPLATE.ScrollFrame(GVAR.AchievementDetailFrame.ScrollFrame)
	GVAR.AchievementDetailFrame.ScrollFrame:SetPoint("TOPLEFT", GVAR.AchievementDetailFrame, "TOPLEFT", sizeGeneral.Offset, -sizeAchievementDetail.SortLineXTop)
	GVAR.AchievementDetailFrame.ScrollFrame:SetPoint("BOTTOMRIGHT", GVAR.AchievementDetailFrame, "BOTTOMRIGHT", -(sizeGeneral.Space+sizeGeneral.SliderSize+sizeGeneral.Space+sizeGeneral.Border), sizeGeneral.Offset) -- -(2+17+2+1), 5
	GVAR.AchievementDetailFrame.ScrollFrame:SetScript("OnVerticalScroll", function(self, value) CharScanner:AchievementDetail_OnVerticalScroll(value) end)

	for i = 1, GUI_achievementdetail_maxRows do
		if not GVAR.AchievementDetailFrame.ScrollFrameButton then GVAR.AchievementDetailFrame.ScrollFrameButton = {} end

		GVAR.AchievementDetailFrame.ScrollFrameButton[i] = CreateFrame("Button", nil, GVAR.AchievementDetailFrame)
		GVAR.AchievementDetailFrame.ScrollFrameButton[i]:SetWidth(sizeAchievementDetail.ButtonWidth)
		GVAR.AchievementDetailFrame.ScrollFrameButton[i]:SetHeight(sizeGeneral.BarHeight)
		if i == 1 then
			GVAR.AchievementDetailFrame.ScrollFrameButton[i]:SetPoint("TOPLEFT", GVAR.AchievementDetailFrame.ScrollFrame, "TOPLEFT", 0, 0)
		else
			GVAR.AchievementDetailFrame.ScrollFrameButton[i]:SetPoint("TOPLEFT", GVAR.AchievementDetailFrame.ScrollFrameButton[(i-1)], "BOTTOMLEFT", 0, 0)
		end

		GVAR.AchievementDetailFrame.ScrollFrameButton[i].Id = GVAR.AchievementDetailFrame.ScrollFrameButton[i]:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
		GVAR.AchievementDetailFrame.ScrollFrameButton[i].Id:SetWidth(sizeAchievementDetail.IdWidth)
		GVAR.AchievementDetailFrame.ScrollFrameButton[i].Id:SetHeight(fonts.GameFontNormalSmall.height)
		GVAR.AchievementDetailFrame.ScrollFrameButton[i].Id:SetPoint("LEFT", GVAR.AchievementDetailFrame.ScrollFrameButton[i], "LEFT", sizeAchievementDetail.IdLeft, 0)
		GVAR.AchievementDetailFrame.ScrollFrameButton[i].Id:SetJustifyH("RIGHT")
		GVAR.AchievementDetailFrame.ScrollFrameButton[i].Id:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])

		GVAR.AchievementDetailFrame.ScrollFrameButton[i].Name = GVAR.AchievementDetailFrame.ScrollFrameButton[i]:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
		GVAR.AchievementDetailFrame.ScrollFrameButton[i].Name:SetWidth(sizeAchievementDetail.NameWidth)
		GVAR.AchievementDetailFrame.ScrollFrameButton[i].Name:SetHeight(fonts.GameFontNormalSmall.height)
		GVAR.AchievementDetailFrame.ScrollFrameButton[i].Name:SetPoint("LEFT", GVAR.AchievementDetailFrame.ScrollFrameButton[i], "LEFT", sizeAchievementDetail.NameLeft, 0)
		GVAR.AchievementDetailFrame.ScrollFrameButton[i].Name:SetJustifyH("LEFT")

		GVAR.AchievementDetailFrame.ScrollFrameButton[i].CompareButton = CreateFrame("Button", nil, GVAR.AchievementDetailFrame.ScrollFrameButton[i])
		GVAR.AchievementDetailFrame.ScrollFrameButton[i].CompareButton:SetWidth(Textures.RaidSelect.width)
		GVAR.AchievementDetailFrame.ScrollFrameButton[i].CompareButton:SetHeight(Textures.RaidSelect.height)
		GVAR.AchievementDetailFrame.ScrollFrameButton[i].CompareButton:SetPoint("LEFT", GVAR.AchievementDetailFrame.ScrollFrameButton[i], "LEFT", sizeAchievementDetail.CompareLeft, 0)
		GVAR.AchievementDetailFrame.ScrollFrameButton[i].CompareButton:SetFrameLevel( GVAR.AchievementDetailFrame:GetFrameLevel()+2 )
		GVAR.AchievementDetailFrame.ScrollFrameButton[i].CompareButtonTexture = GVAR.AchievementDetailFrame.ScrollFrameButton[i].CompareButton:CreateTexture(nil, "ARTWORK")
		GVAR.AchievementDetailFrame.ScrollFrameButton[i].CompareButtonTexture:SetWidth(Textures.RaidSelect.width)
		GVAR.AchievementDetailFrame.ScrollFrameButton[i].CompareButtonTexture:SetHeight(Textures.RaidSelect.height)
		GVAR.AchievementDetailFrame.ScrollFrameButton[i].CompareButtonTexture:SetPoint("LEFT", 0, 0)
		GVAR.AchievementDetailFrame.ScrollFrameButton[i].CompareButtonTexture:SetTexture(Textures.CharScannerIcons.path)
		GVAR.AchievementDetailFrame.ScrollFrameButton[i].CompareButtonTexture:SetTexCoord(unpack(Textures.RaidSelect.coords))
		GVAR.AchievementDetailFrame.ScrollFrameButton[i].CompareButtonTexture:SetAlpha(0.25) 
		GVAR.AchievementDetailFrame.ScrollFrameButton[i].CompareButton:SetScript("OnEnter", function() GVAR.AchievementDetailFrame.ScrollFrameButton[i].CompareButtonTexture:SetAlpha(1) end)
		GVAR.AchievementDetailFrame.ScrollFrameButton[i].CompareButton:SetScript("OnLeave", function() GVAR.AchievementDetailFrame.ScrollFrameButton[i].CompareButtonTexture:SetAlpha(0.25) end)
		GVAR.AchievementDetailFrame.ScrollFrameButton[i].CompareButton:SetScript("OnClick", function() CharScanner:CompareFrameShow() CharScanner:History_GUISort("ACHIEVEMENT", GUI_history_SortBy, GVAR.AchievementDetailFrame.ScrollFrameButton[i].achievementID) end)

		GVAR.AchievementDetailFrame.ScrollFrameButton[i].Highlight = GVAR.AchievementDetailFrame.ScrollFrameButton[i]:CreateTexture(nil, "ARTWORK")
		GVAR.AchievementDetailFrame.ScrollFrameButton[i].Highlight:SetPoint("TOPLEFT", 0, 0)
		GVAR.AchievementDetailFrame.ScrollFrameButton[i].Highlight:SetPoint("BOTTOMRIGHT", 0, 0)
		GVAR.AchievementDetailFrame.ScrollFrameButton[i].Highlight:SetTexture(color.ButtonHighlight.rgba[1], color.ButtonHighlight.rgba[2], color.ButtonHighlight.rgba[3], color.ButtonHighlight.rgba[4])
		GVAR.AchievementDetailFrame.ScrollFrameButton[i]:SetHighlightTexture(GVAR.AchievementDetailFrame.ScrollFrameButton[i].Highlight)

		GVAR.AchievementDetailFrame.ScrollFrameButton[i]:SetScript("OnEnter", function()
			local _, _, _, _, _, _, _, description, _, icon = GetAchievementInfo(GVAR.AchievementDetailFrame.ScrollFrameButton[i].achievementID)
			GVAR.AchievementDetailFrame.InfoBox.Title:SetText(AchievementIDTable[GVAR.AchievementDetailFrame.ScrollFrameButton[i].achievementID])
			GVAR.AchievementDetailFrame.InfoBox.InfoText:SetText(description)
			GVAR.AchievementDetailFrame.InfoBox.Texture:SetTexture(icon)
			local textheight = GVAR.AchievementDetailFrame.InfoBox.InfoText:GetStringHeight()
			if textheight < sizeAchievementDetail.InfoTexture then
				textheight = sizeAchievementDetail.InfoTexture
			end
			local height = sizeGeneral.Offset + GVAR.AchievementDetailFrame.InfoBox.Title:GetStringHeight() + sizeGeneral.Offset + textheight + sizeGeneral.Offset
			GVAR.AchievementDetailFrame.InfoBox:SetHeight(height)
			GVAR.AchievementDetailFrame.InfoBox:Show()
		end)
		GVAR.AchievementDetailFrame.ScrollFrameButton[i]:SetScript("OnLeave", function() GVAR.AchievementDetailFrame.InfoBox:Hide() end)
	end
	-- - AchievementDetail ScrollFrame END

	-- - AchievementDetail StatusFrame START
	GVAR.AchievementDetailFrame.Status = CreateFrame("Frame", nil, GVAR.AchievementDetailFrame)
	TEMPLATE.BorderTRBL(GVAR.AchievementDetailFrame.Status)
	GVAR.AchievementDetailFrame.Status:SetWidth(sizeAchievementDetail.Width)
	GVAR.AchievementDetailFrame.Status:SetHeight(sizeGeneral.StatusHeight)
	GVAR.AchievementDetailFrame.Status:SetPoint("TOPLEFT", GVAR.AchievementDetailFrame, "BOTTOMLEFT", 0, sizeGeneral.Border)
	GVAR.AchievementDetailFrame.Status:EnableMouse(true)

	GVAR.AchievementDetailFrame.StatusText = GVAR.AchievementDetailFrame.Status:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.AchievementDetailFrame.StatusText:SetPoint("CENTER", GVAR.AchievementDetailFrame.Status, "CENTER", 0, 0)
	GVAR.AchievementDetailFrame.StatusText:SetTextColor(1, 1, 1, 1)
	-- - AchievementDetail StatusFrame END

	-- - AchievementDetail Info START
	GVAR.AchievementDetailFrame.InfoBox = CreateFrame("Frame", nil, GVAR.AchievementDetailFrame)
	TEMPLATE.BorderTRBL(GVAR.AchievementDetailFrame.InfoBox)
	GVAR.AchievementDetailFrame.InfoBox:SetWidth(sizeAchievementDetail.Width)
	GVAR.AchievementDetailFrame.InfoBox:SetPoint("TOPLEFT", GVAR.AchievementDetailFrame, "BOTTOMLEFT", 0, -(sizeGeneral.StatusHeight-(2*sizeGeneral.Border)))
	GVAR.AchievementDetailFrame.InfoBox:Hide()
	GVAR.AchievementDetailFrame.InfoBox.Title = GVAR.AchievementDetailFrame.InfoBox:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.AchievementDetailFrame.InfoBox.Title:SetWidth(sizeAchievementDetail.Width-sizeGeneral.Offset-sizeGeneral.Offset)
	GVAR.AchievementDetailFrame.InfoBox.Title:SetPoint("TOPLEFT", GVAR.AchievementDetailFrame.InfoBox, "TOPLEFT", sizeGeneral.Offset, -sizeGeneral.Offset)
	GVAR.AchievementDetailFrame.InfoBox.Title:SetJustifyH("LEFT")
	GVAR.AchievementDetailFrame.InfoBox.Texture = GVAR.AchievementDetailFrame.InfoBox:CreateTexture(nil, "ARTWORK")
	GVAR.AchievementDetailFrame.InfoBox.Texture:SetWidth(sizeAchievementDetail.InfoTexture)
	GVAR.AchievementDetailFrame.InfoBox.Texture:SetHeight(sizeAchievementDetail.InfoTexture)
	GVAR.AchievementDetailFrame.InfoBox.Texture:SetPoint("TOPLEFT", GVAR.AchievementDetailFrame.InfoBox.Title, "BOTTOMLEFT", 0, -sizeGeneral.Offset)
	GVAR.AchievementDetailFrame.InfoBox.InfoText = GVAR.AchievementDetailFrame.InfoBox:CreateFontString(nil, "OVERLAY", fonts.GameFontWhiteSmall.name)
	GVAR.AchievementDetailFrame.InfoBox.InfoText:SetWidth(sizeAchievementDetail.InfoTextWidth)
	GVAR.AchievementDetailFrame.InfoBox.InfoText:SetPoint("TOPLEFT", GVAR.AchievementDetailFrame.InfoBox.Texture, "TOPRIGHT", sizeGeneral.Offset, 0)
	GVAR.AchievementDetailFrame.InfoBox.InfoText:SetJustifyH("LEFT")
	-- - AchievementDetail Info END
	-- AchievementDetailFrame END



	-- Summary Frame START
	--GVAR.MainFrame.Summary
	GVAR.MainFrame.Summary = CreateFrame("Frame", nil, GVAR.MainFrame)
	TEMPLATE.BorderTcRBL(GVAR.MainFrame.Summary)
	GVAR.MainFrame.Summary:SetWidth(sizeGeneral.MainWidth)
	GVAR.MainFrame.Summary:SetHeight(sizeSummaryFrame_BaseHeight)
	GVAR.MainFrame.Summary:SetPoint("TOPLEFT", GVAR.MainFrame.Stats, "BOTTOMLEFT", 0, sizeGeneral.Border)
	GVAR.MainFrame.Summary:EnableMouse(true)
	GVAR.MainFrame.Summary:Hide()

	-- - resize START
	GVAR.MainFrame.Summary.ResizeL = CreateFrame("Button", nil, GVAR.MainFrame.Summary)
	GVAR.MainFrame.Summary.ResizeL:SetPoint("TOPLEFT", GVAR.MainFrame.Summary, "TOPLEFT", 3, -3)
	GVAR.MainFrame.Summary.ResizeL:SetWidth(sizeGeneral.SliderSize)
	GVAR.MainFrame.Summary.ResizeL:SetHeight(sizeGeneral.SliderSize)
	GVAR.MainFrame.Summary.ResizeL.Normal = GVAR.MainFrame.Summary.ResizeL:CreateTexture(nil, "ARTWORK")
	GVAR.MainFrame.Summary.ResizeL.Normal:SetPoint("TOPLEFT", 0, 0)
	GVAR.MainFrame.Summary.ResizeL.Normal:SetPoint("BOTTOMRIGHT", 0, 0)
	GVAR.MainFrame.Summary.ResizeL.Normal:SetTexture(Textures.CharScannerIcons.path)
	GVAR.MainFrame.Summary.ResizeL.Normal:SetTexCoord(unpack(Textures.ResizeUpDown.coords))
	GVAR.MainFrame.Summary.ResizeL:SetNormalTexture(GVAR.MainFrame.Summary.ResizeL.Normal)
	GVAR.MainFrame.Summary.ResizeL.Push = GVAR.MainFrame.Summary.ResizeL:CreateTexture(nil, "ARTWORK")
	GVAR.MainFrame.Summary.ResizeL.Push:SetPoint("TOPLEFT", 1, -1)
	GVAR.MainFrame.Summary.ResizeL.Push:SetPoint("BOTTOMRIGHT", -1, 1)
	GVAR.MainFrame.Summary.ResizeL.Push:SetTexture(Textures.CharScannerIcons.path)
	GVAR.MainFrame.Summary.ResizeL.Push:SetTexCoord(unpack(Textures.ResizeUpDown.coords))
	GVAR.MainFrame.Summary.ResizeL:SetPushedTexture(GVAR.MainFrame.Summary.ResizeL.Push)
	GVAR.MainFrame.Summary.ResizeL.Highlight = GVAR.MainFrame.Summary.ResizeL:CreateTexture(nil, "ARTWORK")
	GVAR.MainFrame.Summary.ResizeL.Highlight:SetPoint("TOPLEFT", 0, 0)
	GVAR.MainFrame.Summary.ResizeL.Highlight:SetPoint("BOTTOMRIGHT", 0, 0)
	GVAR.MainFrame.Summary.ResizeL.Highlight:SetTexture(Textures.CharScannerIcons.path)
	GVAR.MainFrame.Summary.ResizeL.Highlight:SetTexCoord(unpack(Textures.ButtonHL.coords))
	GVAR.MainFrame.Summary.ResizeL:SetHighlightTexture(GVAR.MainFrame.Summary.ResizeL.Highlight)
	GVAR.MainFrame.Summary.ResizeL:SetScript("OnMouseDown", ResizeStart)
	GVAR.MainFrame.Summary.ResizeL:SetScript("OnMouseUp", ResizeEnd)
	GVAR.MainFrame.Summary.ResizeL:SetScript("OnEnter", function() CharScanner:Frame_MoverInfo(GVAR.MainFrame, "RESIZE", "BOTTOM") end)
	GVAR.MainFrame.Summary.ResizeL:SetScript("OnLeave", function() CharScanner:Frame_MoverInfo(GVAR.MainFrame) end)

	GVAR.MainFrame.Summary.ResizeR = CreateFrame("Button", nil, GVAR.MainFrame.Summary)
	GVAR.MainFrame.Summary.ResizeR:SetPoint("TOPRIGHT", GVAR.MainFrame.Summary, "TOPRIGHT", -3, -3)
	GVAR.MainFrame.Summary.ResizeR:SetWidth(sizeGeneral.SliderSize)
	GVAR.MainFrame.Summary.ResizeR:SetHeight(sizeGeneral.SliderSize)
	GVAR.MainFrame.Summary.ResizeR.Normal = GVAR.MainFrame.Summary.ResizeR:CreateTexture(nil, "ARTWORK")
	GVAR.MainFrame.Summary.ResizeR.Normal:SetPoint("TOPLEFT", 0, 0)
	GVAR.MainFrame.Summary.ResizeR.Normal:SetPoint("BOTTOMRIGHT", 0, 0)
	GVAR.MainFrame.Summary.ResizeR.Normal:SetTexture(Textures.CharScannerIcons.path)
	GVAR.MainFrame.Summary.ResizeR.Normal:SetTexCoord(unpack(Textures.ResizeUpDown.coords))
	GVAR.MainFrame.Summary.ResizeR:SetNormalTexture(GVAR.MainFrame.Summary.ResizeR.Normal)
	GVAR.MainFrame.Summary.ResizeR.Push = GVAR.MainFrame.Summary.ResizeR:CreateTexture(nil, "ARTWORK")
	GVAR.MainFrame.Summary.ResizeR.Push:SetPoint("TOPLEFT", 1, -1)
	GVAR.MainFrame.Summary.ResizeR.Push:SetPoint("BOTTOMRIGHT", -1, 1)
	GVAR.MainFrame.Summary.ResizeR.Push:SetTexture(Textures.CharScannerIcons.path)
	GVAR.MainFrame.Summary.ResizeR.Push:SetTexCoord(unpack(Textures.ResizeUpDown.coords))
	GVAR.MainFrame.Summary.ResizeR:SetPushedTexture(GVAR.MainFrame.Summary.ResizeR.Push)
	GVAR.MainFrame.Summary.ResizeR.Highlight = GVAR.MainFrame.Summary.ResizeR:CreateTexture(nil, "ARTWORK")
	GVAR.MainFrame.Summary.ResizeR.Highlight:SetPoint("TOPLEFT", 0, 0)
	GVAR.MainFrame.Summary.ResizeR.Highlight:SetPoint("BOTTOMRIGHT", 0, 0)
	GVAR.MainFrame.Summary.ResizeR.Highlight:SetTexture(Textures.CharScannerIcons.path)
	GVAR.MainFrame.Summary.ResizeR.Highlight:SetTexCoord(unpack(Textures.ButtonHL.coords))
	GVAR.MainFrame.Summary.ResizeR:SetHighlightTexture(GVAR.MainFrame.Summary.ResizeR.Highlight)
	GVAR.MainFrame.Summary.ResizeR:SetScript("OnMouseDown", ResizeStart)
	GVAR.MainFrame.Summary.ResizeR:SetScript("OnMouseUp", ResizeEnd)
	GVAR.MainFrame.Summary.ResizeR:SetScript("OnEnter", function() CharScanner:Frame_MoverInfo(GVAR.MainFrame, "RESIZE", "BOTTOM") end)
	GVAR.MainFrame.Summary.ResizeR:SetScript("OnLeave", function() CharScanner:Frame_MoverInfo(GVAR.MainFrame) end)
	-- - resize END

	GVAR.MainFrame.Summary1PvE = CreateFrame("Button", nil, GVAR.MainFrame.Summary)
	TEMPLATE.PvEStats(GVAR.MainFrame.Summary1PvE, false, "summary")
	GVAR.MainFrame.Summary1PvE:SetPoint("TOPLEFT", GVAR.MainFrame.Summary, "TOPLEFT", sizeGeneral.Offset, -sizeGeneral.Offset)
	GVAR.MainFrame.Summary2PvE = CreateFrame("Button", nil, GVAR.MainFrame.Summary)
	TEMPLATE.PvEStats(GVAR.MainFrame.Summary2PvE, false, "summary")
	GVAR.MainFrame.Summary2PvE:SetPoint("TOPLEFT", GVAR.MainFrame.Summary1PvE, "BOTTOMLEFT", 0, 0)

	GVAR.MainFrame.Summary1PvP = CreateFrame("Button", nil, GVAR.MainFrame.Summary)
	TEMPLATE.PvPStats(GVAR.MainFrame.Summary1PvP, false, "summary")
	GVAR.MainFrame.Summary1PvP:SetPoint("TOPLEFT", GVAR.MainFrame.Summary, "TOPLEFT", sizeGeneral.Offset, -sizeGeneral.Offset)
	GVAR.MainFrame.Summary1PvP:Hide()
	GVAR.MainFrame.Summary2PvP = CreateFrame("Button", nil, GVAR.MainFrame.Summary)
	TEMPLATE.PvPStats(GVAR.MainFrame.Summary2PvP, false, "summary")
	GVAR.MainFrame.Summary2PvP:SetPoint("TOPLEFT", GVAR.MainFrame.Summary1PvP, "BOTTOMLEFT", 0, 0)
	GVAR.MainFrame.Summary2PvP:Hide()
	-- Summary Frame END



	-- Status Frame START
	GVAR.MainFrame.Status = CreateFrame("Frame", nil, GVAR.MainFrame)
	TEMPLATE.BorderTRBL(GVAR.MainFrame.Status)
	GVAR.MainFrame.Status:SetWidth(sizeGeneral.MainWidth)
	GVAR.MainFrame.Status:SetHeight(sizeGeneral.StatusHeight)
	GVAR.MainFrame.Status:SetPoint("TOPLEFT", GVAR.MainFrame.Summary, "BOTTOMLEFT", 0, sizeGeneral.Border)
	GVAR.MainFrame.Status:EnableMouse(true)
	GVAR.MainFrame.Status:SetScript("OnShow", function() CharScanner:StatusFrame_CheckPosition() end)
	GVAR.MainFrame.Status:Hide()

	GVAR.MainFrame.Status:SetScript("OnEnter", function() CharScanner:Frame_MoverInfo(GVAR.MainFrame, "MOVE", "BOTTOM") end)
	GVAR.MainFrame.Status:SetScript("OnLeave", function() CharScanner:Frame_MoverInfo(GVAR.MainFrame) end)
	GVAR.MainFrame.Status:SetScript("OnMouseDown", MoveStart)
	GVAR.MainFrame.Status:SetScript("OnMouseUp", MoveStop)

	GVAR.MainFrame.Status.Text = GVAR.MainFrame.Status:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.MainFrame.Status.Text:SetPoint("CENTER", GVAR.MainFrame.Status, "CENTER", 0, 0)
	GVAR.MainFrame.Status.Text:SetTextColor(1, 1, 1, 1)

	GVAR.MainFrame.Status.ProgressButton = CreateFrame("Button", nil, GVAR.MainFrame.Status)
	TEMPLATE.TextButton(GVAR.MainFrame.Status.ProgressButton, L["STOP SCAN"], 1)
	GVAR.MainFrame.Status.ProgressButton:SetWidth(150)
	GVAR.MainFrame.Status.ProgressButton:SetHeight(22)
	GVAR.MainFrame.Status.ProgressButton:SetPoint("TOP", GVAR.MainFrame.Status, "TOP", 0, -5)
	GVAR.MainFrame.Status.ProgressButton:SetScript("OnClick", function() CharScanner:StopGroupScan() end)
	GVAR.MainFrame.Status.ProgressButton:Hide()
	GVAR.MainFrame.Status.ProgressText = GVAR.MainFrame.Status:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.MainFrame.Status.ProgressText:SetPoint("TOP", GVAR.MainFrame.Status.ProgressButton, "BOTTOM", 0, -2)
	GVAR.MainFrame.Status.ProgressText:SetJustifyH("CENTER")
	GVAR.MainFrame.Status.ProgressText:SetWidth(100)
	GVAR.MainFrame.Status.ProgressText:SetText("-/-")
	GVAR.MainFrame.Status.ProgressText:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])
	GVAR.MainFrame.Status.ProgressText:Hide()
	GVAR.MainFrame.Status.ProgressBackground = GVAR.MainFrame.Status:CreateTexture(nil, "BORDER")
	GVAR.MainFrame.Status.ProgressBackground:SetPoint("TOP", GVAR.MainFrame.Status.ProgressButton, "BOTTOM", 0, -2)
	GVAR.MainFrame.Status.ProgressBackground:SetWidth(sizeGeneral.StatusProgressBarWidth)
	GVAR.MainFrame.Status.ProgressBackground:SetHeight(12)
	GVAR.MainFrame.Status.ProgressBackground:SetTexture(color.DarkGrey.rgba[1], color.DarkGrey.rgba[2], color.DarkGrey.rgba[3], color.DarkGrey.rgba[4])
	GVAR.MainFrame.Status.ProgressBackground:Hide()
	GVAR.MainFrame.Status.ProgressDone = GVAR.MainFrame.Status:CreateTexture(nil, "ARTWORK")
	GVAR.MainFrame.Status.ProgressDone:SetPoint("LEFT", GVAR.MainFrame.Status.ProgressBackground, "LEFT", 0, 0)
	GVAR.MainFrame.Status.ProgressDone:SetWidth(1)
	GVAR.MainFrame.Status.ProgressDone:SetHeight(12)
	GVAR.MainFrame.Status.ProgressDone:SetBlendMode("BLEND")
	GVAR.MainFrame.Status.ProgressDone:SetTexture(color.Green.rgba[1], color.Green.rgba[2], color.Green.rgba[3], color.Green.rgba[4])
	GVAR.MainFrame.Status.ProgressDone:Hide()

	GVAR.MainFrame.Status.ThrottleText1 = GVAR.MainFrame.Status:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.MainFrame.Status.ThrottleText1:SetPoint("TOP", GVAR.MainFrame.Status.ProgressBackground, "BOTTOM", 0, -5)
	GVAR.MainFrame.Status.ThrottleText1:SetJustifyH("CENTER")
	GVAR.MainFrame.Status.ThrottleText1:SetWidth(sizeGeneral.MainWidth-20)
	GVAR.MainFrame.Status.ThrottleText1:SetText(L["Achievement/Talent scan restriction from Blizzard:"])
	GVAR.MainFrame.Status.ThrottleText1:SetTextColor(color.Grey.rgba[1], color.Grey.rgba[2], color.Grey.rgba[3], color.Grey.rgba[4])
	GVAR.MainFrame.Status.ThrottleText1:Hide()
	GVAR.MainFrame.Status.ThrottleText2 = GVAR.MainFrame.Status:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.MainFrame.Status.ThrottleText2:SetPoint("TOP", GVAR.MainFrame.Status.ThrottleText1, "BOTTOM", 0, -2)
	GVAR.MainFrame.Status.ThrottleText2:SetJustifyH("CENTER")
	GVAR.MainFrame.Status.ThrottleText2:SetWidth(sizeGeneral.MainWidth-20)
	GVAR.MainFrame.Status.ThrottleText2:SetText( string_format(L["Please wait %.2f seconds."], RestrictionMaxTime) )
	GVAR.MainFrame.Status.ThrottleText2:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])
	GVAR.MainFrame.Status.ThrottleText2:Hide()
	GVAR.MainFrame.Status.ThrottleBackground = GVAR.MainFrame.Status:CreateTexture(nil, "BORDER")
	GVAR.MainFrame.Status.ThrottleBackground:SetPoint("TOP", GVAR.MainFrame.Status.ThrottleText2, "BOTTOM", 0, -2)
	GVAR.MainFrame.Status.ThrottleBackground:SetWidth(sizeGeneral.StatusProgressBarWidth)
	GVAR.MainFrame.Status.ThrottleBackground:SetHeight(6)
	GVAR.MainFrame.Status.ThrottleBackground:SetTexture(color.DarkGrey.rgba[1], color.DarkGrey.rgba[2], color.DarkGrey.rgba[3], color.DarkGrey.rgba[4])
	GVAR.MainFrame.Status.ThrottleBackground:Hide()
	GVAR.MainFrame.Status.ThrottleDone = GVAR.MainFrame.Status:CreateTexture(nil, "ARTWORK")
	GVAR.MainFrame.Status.ThrottleDone:SetPoint("LEFT", GVAR.MainFrame.Status.ThrottleBackground, "LEFT", 0, 0)
	GVAR.MainFrame.Status.ThrottleDone:SetWidth(1)
	GVAR.MainFrame.Status.ThrottleDone:SetHeight(6)
	GVAR.MainFrame.Status.ThrottleDone:SetBlendMode("BLEND")
	GVAR.MainFrame.Status.ThrottleDone:SetTexture(color.Red.rgba[1], color.Red.rgba[2], color.Red.rgba[3], color.Red.rgba[4])
	GVAR.MainFrame.Status.ThrottleDone:Hide()
	-- Status Frame END



	-- Mover Info START
	GVAR.MainFrame.MoverInfoTop = CreateFrame("Frame", nil, GVAR.MainFrame)
	GVAR.MainFrame.MoverInfoTop:SetWidth(150)
	GVAR.MainFrame.MoverInfoTop:SetHeight(14)
	GVAR.MainFrame.MoverInfoTop:SetPoint("BOTTOM", GVAR.MainFrame, "TOP", 0, 0)
	GVAR.MainFrame.MoverInfoTop:EnableMouse(true)
	GVAR.MainFrame.MoverInfoTop:Hide()
	GVAR.MainFrame.MoverInfoTopTexture = GVAR.MainFrame.MoverInfoTop:CreateTexture(nil, "BACKGROUND")
	GVAR.MainFrame.MoverInfoTopTexture:SetAllPoints(GVAR.MainFrame.MoverInfoTop)
	GVAR.MainFrame.MoverInfoTopTexture:SetTexture(0, 0, 0, 0.5)
	GVAR.MainFrame.MoverInfoTopText = GVAR.MainFrame.MoverInfoTop:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.MainFrame.MoverInfoTopText:SetPoint("CENTER", GVAR.MainFrame.MoverInfoTop, "CENTER", 0, 0)
	GVAR.MainFrame.MoverInfoTopText:SetJustifyH("CENTER")
	GVAR.MainFrame.MoverInfoTopText:SetTextColor(0.6, 0.6, 0.6, 1)

	GVAR.MainFrame.MoverInfoBottom = CreateFrame("Frame", nil, GVAR.MainFrame)
	GVAR.MainFrame.MoverInfoBottom:SetWidth(150)
	GVAR.MainFrame.MoverInfoBottom:SetHeight(14)
	GVAR.MainFrame.MoverInfoBottom:SetPoint("TOP", GVAR.MainFrame.Status, "BOTTOM", 0, 0)
	GVAR.MainFrame.MoverInfoBottom:EnableMouse(true)
	GVAR.MainFrame.MoverInfoBottom:Hide()
	GVAR.MainFrame.MoverInfoBottomTexture = GVAR.MainFrame.MoverInfoBottom:CreateTexture(nil, "BACKGROUND")
	GVAR.MainFrame.MoverInfoBottomTexture:SetAllPoints(GVAR.MainFrame.MoverInfoBottom)
	GVAR.MainFrame.MoverInfoBottomTexture:SetTexture(0, 0, 0, 0.5)
	GVAR.MainFrame.MoverInfoBottomText = GVAR.MainFrame.MoverInfoBottom:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.MainFrame.MoverInfoBottomText:SetPoint("CENTER", GVAR.MainFrame.MoverInfoBottom, "CENTER", 0, 0)
	GVAR.MainFrame.MoverInfoBottomText:SetJustifyH("CENTER")
	GVAR.MainFrame.MoverInfoBottomText:SetTextColor(0.6, 0.6, 0.6, 1)
	-- Mover Info END



	-- History Frame START
	GVAR.HistoryFrame = CreateFrame("Frame", "CharScanner_HistoryFrame", GVAR.MainFrame)
	TEMPLATE.BorderTRBL(GVAR.HistoryFrame)
	GVAR.HistoryFrame:SetWidth(sizeCompareFrame.HistoryWidth)
	GVAR.HistoryFrame:SetHeight(sizeCompareFrame.HistoryHeight)
	GVAR.HistoryFrame:SetPoint("TOPLEFT", GVAR.MainFrame.Base, "TOPLEFT", -(sizeCompareFrame.HistoryWidth-sizeGeneral.Border), 0)
	GVAR.HistoryFrame:EnableMouse(true)
	GVAR.HistoryFrame:SetMovable(true)
	GVAR.HistoryFrame:SetToplevel(true)
	GVAR.HistoryFrame:Hide()
	GVAR.HistoryFrame:SetScript("OnShow", function()
		CharScanner:History_Update()
	end)

	GVAR.HistoryFrame.GroupStatusTabAll = CreateFrame("Button", nil, GVAR.HistoryFrame)
	TEMPLATE.TabButton(GVAR.HistoryFrame.GroupStatusTabAll, L["All"], sizeStatFrame.TabButtonWidth)
	GVAR.HistoryFrame.GroupStatusTabAll:SetWidth( (sizeCompareFrame.HistoryWidth-(4*sizeGeneral.Offset))/3 )
	GVAR.HistoryFrame.GroupStatusTabAll:SetHeight(sizeGeneral.TabButtonHeight)
	GVAR.HistoryFrame.GroupStatusTabAll:SetPoint("BOTTOMLEFT", GVAR.HistoryFrame, "TOPLEFT", sizeGeneral.Offset, -(sizeCompareFrame.SortLine1Height + sizeCompareFrame.SortLine1Top+sizeGeneral.Border))
	GVAR.HistoryFrame.GroupStatusTabAll:SetScript("OnClick", function()
		if GroupTabStatus == 1 then return end
		GroupTabStatus = 1
		CharScanner:History_Update()
	end)

	GVAR.HistoryFrame.GroupStatusTabGroup = CreateFrame("Button", nil, GVAR.HistoryFrame)
	TEMPLATE.TabButton(GVAR.HistoryFrame.GroupStatusTabGroup, L["Group"], sizeStatFrame.TabButtonWidth)
	GVAR.HistoryFrame.GroupStatusTabGroup:SetWidth( (sizeCompareFrame.HistoryWidth-(4*sizeGeneral.Offset))/3 )
	GVAR.HistoryFrame.GroupStatusTabGroup:SetHeight(sizeGeneral.TabButtonHeight)
	GVAR.HistoryFrame.GroupStatusTabGroup:SetPoint("BOTTOMLEFT", GVAR.HistoryFrame.GroupStatusTabAll, "BOTTOMRIGHT", sizeGeneral.Offset, 0)
	GVAR.HistoryFrame.GroupStatusTabGroup:SetScript("OnClick", function()
		if GroupTabStatus == 2 then return end
		GroupTabStatus = 2
		CharScanner:History_Update()
	end)

	GVAR.HistoryFrame.GroupStatusTabOthers = CreateFrame("Button", nil, GVAR.HistoryFrame)
	TEMPLATE.TabButton(GVAR.HistoryFrame.GroupStatusTabOthers, L["Others"], sizeStatFrame.TabButtonWidth)
	GVAR.HistoryFrame.GroupStatusTabOthers:SetWidth( (sizeCompareFrame.HistoryWidth-(4*sizeGeneral.Offset))/3 )
	GVAR.HistoryFrame.GroupStatusTabOthers:SetHeight(sizeGeneral.TabButtonHeight)
	GVAR.HistoryFrame.GroupStatusTabOthers:SetPoint("BOTTOMLEFT", GVAR.HistoryFrame.GroupStatusTabGroup, "BOTTOMRIGHT", sizeGeneral.Offset, 0)
	GVAR.HistoryFrame.GroupStatusTabOthers:SetScript("OnClick", function()
		if GroupTabStatus == 3 then return end
		GroupTabStatus = 3
		CharScanner:History_Update()
	end)

	GVAR.HistoryFrame.GroupStatusTabTex = GVAR.HistoryFrame:CreateTexture(nil, "ARTWORK")
	GVAR.HistoryFrame.GroupStatusTabTex:SetWidth(sizeCompareFrame.HistoryWidth)
	GVAR.HistoryFrame.GroupStatusTabTex:SetHeight(sizeGeneral.Border)
	GVAR.HistoryFrame.GroupStatusTabTex:SetTexture(color.ButtonBorder.rgba[1], color.ButtonBorder.rgba[2], color.ButtonBorder.rgba[3], color.ButtonBorder.rgba[4])
	GVAR.HistoryFrame.GroupStatusTabTex:SetPoint("BOTTOM", GVAR.HistoryFrame.GroupStatusTabAll, "BOTTOM", 0, sizeGeneral.Border)
	GVAR.HistoryFrame.GroupStatusTabTex:SetPoint("LEFT", GVAR.HistoryFrame, "LEFT", 0, 0)

	GVAR.HistoryFrame.Close = CreateFrame("Button", nil, GVAR.HistoryFrame)
	TEMPLATE.IconButton(GVAR.HistoryFrame.Close, 9, 12)
	GVAR.HistoryFrame.Close:SetPoint("BOTTOMRIGHT", GVAR.HistoryFrame, "TOPRIGHT", 0, -sizeGeneral.Border)
	GVAR.HistoryFrame.Close:SetScript("OnClick", function() CharScanner:HistoryCompareFrameHide() end)

	-- - History Sort START
	GVAR.HistoryFrame.Sort = CreateFrame("Frame", nil, GVAR.HistoryFrame)
	GVAR.HistoryFrame.Sort:SetWidth(sizeCompareFrame.HistoryWidth)
	GVAR.HistoryFrame.Sort:SetHeight(sizeGeneral.BarHeight)
	GVAR.HistoryFrame.Sort:SetPoint("TOPLEFT", GVAR.HistoryFrame, "TOPLEFT", sizeGeneral.Offset, -sizeCompareFrame.SortLine2Top)

	GVAR.HistoryFrame.Sort.Icon = CreateFrame("Button", nil, GVAR.HistoryFrame.Sort)
	TEMPLATE.SortPosition(GVAR.HistoryFrame.Sort.Icon, nil, nil)
	GVAR.HistoryFrame.Sort.Icon:SetWidth(sizeGeneral.BarHeight)
	GVAR.HistoryFrame.Sort.Icon:SetHeight(sizeGeneral.BarHeight)
	GVAR.HistoryFrame.Sort.Icon:SetPoint("LEFT", GVAR.HistoryFrame.Sort, "LEFT", sizeCompareFrame.ClassiconLeft, 0)
	GVAR.HistoryFrame.Sort.Icon:SetScript("OnClick", function() CharScanner:History_GUISort("NORMAL", 1) end)

	GVAR.HistoryFrame.Sort.Name = CreateFrame("Button", nil, GVAR.HistoryFrame.Sort)
	TEMPLATE.SortPosition(GVAR.HistoryFrame.Sort.Name, NAME, "left", sizeCompareFrame.NameWidth)
	GVAR.HistoryFrame.Sort.Name:SetWidth(sizeCompareFrame.NameWidth)
	GVAR.HistoryFrame.Sort.Name:SetHeight(sizeGeneral.BarHeight)
	GVAR.HistoryFrame.Sort.Name:SetPoint("LEFT", GVAR.HistoryFrame.Sort, "LEFT", sizeCompareFrame.NameLeft, 0)
	GVAR.HistoryFrame.Sort.Name:SetScript("OnClick", function() CharScanner:History_GUISort("NORMAL", 2) end)

	GVAR.HistoryFrame.Sort.Time = CreateFrame("Button", nil, GVAR.HistoryFrame.Sort)
	TEMPLATE.SortPosition(GVAR.HistoryFrame.Sort.Time, L["Time"], "left", sizeCompareFrame.TimeWidth)
	GVAR.HistoryFrame.Sort.Time:SetWidth(sizeCompareFrame.TimeWidth)
	GVAR.HistoryFrame.Sort.Time:SetHeight(sizeGeneral.BarHeight)
	GVAR.HistoryFrame.Sort.Time:SetPoint("LEFT", GVAR.HistoryFrame.Sort, "LEFT", sizeCompareFrame.TimeLeft, 0)
	GVAR.HistoryFrame.Sort.Time:SetScript("OnClick", function() CharScanner:History_GUISort("NORMAL", 3) end)

	GVAR.HistoryFrame.Sort.MainRole = CreateFrame("Button", nil, GVAR.HistoryFrame.Sort)
	TEMPLATE.SortPosition(GVAR.HistoryFrame.Sort.MainRole, nil, nil)
	GVAR.HistoryFrame.Sort.MainRole:SetWidth(sizeCompareFrame.MainroleWidth)
	GVAR.HistoryFrame.Sort.MainRole:SetHeight(sizeGeneral.BarHeight)
	GVAR.HistoryFrame.Sort.MainRole:SetPoint("LEFT", GVAR.HistoryFrame.Sort, "LEFT", sizeCompareFrame.MainroleLeft, 0)
	GVAR.HistoryFrame.Sort.MainRole:SetScript("OnClick", function() CharScanner:History_GUISort("NORMAL", 4) end)

	GVAR.HistoryFrame.Sort.CompareFrame = CreateFrame("Frame", nil, GVAR.HistoryFrame.Sort)
	GVAR.HistoryFrame.Sort.CompareFrame:SetToplevel(true)

	GVAR.HistoryFrame.Sort.Talents1 = CreateFrame("Button", nil, GVAR.HistoryFrame.Sort.CompareFrame)
	TEMPLATE.SortPosition(GVAR.HistoryFrame.Sort.Talents1, L["T1 (Current Talents)"], "left", sizeCompareFrame.Talents1Width)
	GVAR.HistoryFrame.Sort.Talents1:SetWidth(sizeCompareFrame.Talents1Width)
	GVAR.HistoryFrame.Sort.Talents1:SetHeight(sizeGeneral.BarHeight)
	GVAR.HistoryFrame.Sort.Talents1:SetPoint("LEFT", GVAR.HistoryFrame.Sort, "LEFT", sizeCompareFrame.Talents1Left, 0)
	GVAR.HistoryFrame.Sort.Talents1:SetScript("OnClick", function() CharScanner:History_GUISort("NORMAL", 5) end)

	GVAR.HistoryFrame.Sort.Talents2 = CreateFrame("Button", nil, GVAR.HistoryFrame.Sort.CompareFrame)
	TEMPLATE.SortPosition(GVAR.HistoryFrame.Sort.Talents2, L["T2"], "left", sizeCompareFrame.Talents2Width)
	GVAR.HistoryFrame.Sort.Talents2:SetWidth(sizeCompareFrame.Talents2Width)
	GVAR.HistoryFrame.Sort.Talents2:SetHeight(sizeGeneral.BarHeight)
	GVAR.HistoryFrame.Sort.Talents2:SetPoint("LEFT", GVAR.HistoryFrame.Sort, "LEFT", sizeCompareFrame.Talents2Left, 0)
	GVAR.HistoryFrame.Sort.Talents2:SetScript("OnClick", function() CharScanner:History_GUISort("NORMAL", 6) end)

	GVAR.HistoryFrame.Sort.EnchantmentCount = CreateFrame("Button", nil, GVAR.HistoryFrame.Sort.CompareFrame)
	TEMPLATE.SortPosition(GVAR.HistoryFrame.Sort.EnchantmentCount, L["E"], "right", sizeCompareFrame.EnchWidth)
	GVAR.HistoryFrame.Sort.EnchantmentCount:SetWidth(sizeCompareFrame.EnchWidth)
	GVAR.HistoryFrame.Sort.EnchantmentCount:SetHeight(sizeGeneral.BarHeight)
	GVAR.HistoryFrame.Sort.EnchantmentCount:SetPoint("LEFT", GVAR.HistoryFrame.Sort, "LEFT", sizeCompareFrame.EnchLeft, 0)
	GVAR.HistoryFrame.Sort.EnchantmentCount:SetScript("OnClick", function() CharScanner:History_GUISort("NORMAL", 7) end)

	GVAR.HistoryFrame.Sort.PvPCount = CreateFrame("Button", nil, GVAR.HistoryFrame.Sort.CompareFrame)
	TEMPLATE.SortPosition(GVAR.HistoryFrame.Sort.PvPCount, L["P"], "right", sizeCompareFrame.PvPWidth)
	GVAR.HistoryFrame.Sort.PvPCount:SetWidth(sizeCompareFrame.PvPWidth)
	GVAR.HistoryFrame.Sort.PvPCount:SetHeight(sizeGeneral.BarHeight)
	GVAR.HistoryFrame.Sort.PvPCount:SetPoint("LEFT", GVAR.HistoryFrame.Sort, "LEFT", sizeCompareFrame.PvPLeft, 0)
	GVAR.HistoryFrame.Sort.PvPCount:SetScript("OnClick", function() CharScanner:History_GUISort("NORMAL", 8) end)

	GVAR.HistoryFrame.Sort.ItemLevel = CreateFrame("Button", nil, GVAR.HistoryFrame.Sort.CompareFrame)
	TEMPLATE.SortPosition(GVAR.HistoryFrame.Sort.ItemLevel, L["iLvl"], "right", sizeCompareFrame.EquipWidth)
	GVAR.HistoryFrame.Sort.ItemLevel:SetWidth(sizeCompareFrame.EquipWidth)
	GVAR.HistoryFrame.Sort.ItemLevel:SetHeight(sizeGeneral.BarHeight)
	GVAR.HistoryFrame.Sort.ItemLevel:SetPoint("LEFT", GVAR.HistoryFrame.Sort, "LEFT", sizeCompareFrame.EquipLeft, 0)
	GVAR.HistoryFrame.Sort.ItemLevel:SetScript("OnClick", function() CharScanner:History_GUISort("NORMAL", 9) end)

	GVAR.HistoryFrame.Sort.Achievement = CreateFrame("Button", nil, GVAR.HistoryFrame.Sort.CompareFrame)
	TEMPLATE.SortPosition(GVAR.HistoryFrame.Sort.Achievement, L["Achievements"], "right", sizeCompareFrame.AchievementWidth)
	GVAR.HistoryFrame.Sort.Achievement:SetWidth(sizeCompareFrame.AchievementWidth)
	GVAR.HistoryFrame.Sort.Achievement:SetHeight(sizeGeneral.BarHeight)
	GVAR.HistoryFrame.Sort.Achievement:SetPoint("LEFT", GVAR.HistoryFrame.Sort, "LEFT", sizeCompareFrame.AchievementLeft, 0)
	GVAR.HistoryFrame.Sort.Achievement:SetScript("OnClick", function() CharScanner:History_GUISort("NORMAL", 10) end)

	GVAR.HistoryFrame.Sort.Boss = CreateFrame("Button", nil, GVAR.HistoryFrame.Sort.CompareFrame)
	TEMPLATE.SortPosition(GVAR.HistoryFrame.Sort.Boss, L["Bosses"], "right", sizeCompareFrame.BossWidth)
	GVAR.HistoryFrame.Sort.Boss:SetWidth(sizeCompareFrame.BossWidth)
	GVAR.HistoryFrame.Sort.Boss:SetHeight(sizeGeneral.BarHeight)
	GVAR.HistoryFrame.Sort.Boss:SetPoint("LEFT", GVAR.HistoryFrame.Sort, "LEFT", sizeCompareFrame.BossLeft, 0)
	GVAR.HistoryFrame.Sort.Boss:SetScript("OnClick", function() CharScanner:History_GUISort("NORMAL", 11) end)

	GVAR.HistoryFrame.Sort.Kills = CreateFrame("Button", nil, GVAR.HistoryFrame.Sort.CompareFrame)
	TEMPLATE.SortPosition(GVAR.HistoryFrame.Sort.Kills, L["Kills"], "right", sizeCompareFrame.KillsWidth)
	GVAR.HistoryFrame.Sort.Kills:SetWidth(sizeCompareFrame.KillsWidth)
	GVAR.HistoryFrame.Sort.Kills:SetHeight(sizeGeneral.BarHeight)
	GVAR.HistoryFrame.Sort.Kills:SetPoint("LEFT", GVAR.HistoryFrame.Sort, "LEFT", sizeCompareFrame.KillsLeft, 0)
	GVAR.HistoryFrame.Sort.Kills:SetScript("OnClick", function() CharScanner:History_GUISort("NORMAL", 12) end)

	GVAR.HistoryFrame.Sort.SingleAchievementCompare = CreateFrame("Button", nil, GVAR.HistoryFrame.Sort.CompareFrame)
	TEMPLATE.SortPosition(GVAR.HistoryFrame.Sort.SingleAchievementCompare, L["Single Achievement Compare"], "left", sizeCompareFrame.SingleAchievementCompareWidth)
	GVAR.HistoryFrame.Sort.SingleAchievementCompare:SetWidth(sizeCompareFrame.SingleAchievementCompareWidth)
	GVAR.HistoryFrame.Sort.SingleAchievementCompare:SetHeight(sizeGeneral.BarHeight)
	GVAR.HistoryFrame.Sort.SingleAchievementCompare:SetPoint("LEFT", GVAR.HistoryFrame.Sort, "LEFT", sizeCompareFrame.SingleAchievementCompareLeft, 0)
	GVAR.HistoryFrame.Sort.SingleAchievementCompare:SetScript("OnClick", function() CharScanner:History_GUISort("NORMAL", 13) end)
	GVAR.HistoryFrame.Sort.SingleAchievementCompare:Hide()

	GVAR.HistoryFrame.Sort.PvEdump = CreateFrame("Button", nil, GVAR.HistoryFrame.Sort.CompareFrame)
	TEMPLATE.PullDownMenu(GVAR.HistoryFrame.Sort.PvEdump, "PvEdump", PullDownMenu_PvEdump[1].name, 250, #PullDownMenu_PvEdump, PullDownMenuFunc_PvEdump)
	GVAR.HistoryFrame.Sort.PvEdump:SetWidth(250)
	GVAR.HistoryFrame.Sort.PvEdump:SetHeight(sizeCompareFrame.SortLine1Height)
	GVAR.HistoryFrame.Sort.PvEdump:SetPoint("TOPRIGHT", GVAR.HistoryFrame, "TOPRIGHT", -sizeCompareFrame.RaidSelectLeft, -sizeGeneral.Offset)
	-- - History Sort END

	-- - History ScrollFrame START
	GVAR.HistoryFrame.ScrollFrame = CreateFrame("ScrollFrame", nil, GVAR.HistoryFrame)
	TEMPLATE.ScrollFrame(GVAR.HistoryFrame.ScrollFrame)
	GVAR.HistoryFrame.ScrollFrame:SetPoint("TOPLEFT", GVAR.HistoryFrame, "TOPLEFT", sizeGeneral.Offset, -sizeCompareFrame.SortLineXTop)
	GVAR.HistoryFrame.ScrollFrame:SetPoint("BOTTOMRIGHT", GVAR.HistoryFrame, "BOTTOMRIGHT", -(sizeGeneral.Space+sizeGeneral.SliderSize+sizeGeneral.Space+sizeGeneral.Border), sizeGeneral.Offset) -- -(2+17+2+1), 5
	GVAR.HistoryFrame.ScrollFrame:SetScript("OnVerticalScroll", function(self, value) CharScanner:History_OnVerticalScroll(value) end)
	for i = 1, GUI_history_maxRows do
		if not GVAR.HistoryFrame.ScrollFrameButton then GVAR.HistoryFrame.ScrollFrameButton = {} end

		GVAR.HistoryFrame.ScrollFrameButton[i] = CreateFrame("Button", nil, GVAR.HistoryFrame)
		GVAR.HistoryFrame.ScrollFrameButton[i]:SetWidth(sizeCompareFrame.HistoryButtonWidth)
		GVAR.HistoryFrame.ScrollFrameButton[i]:SetHeight(sizeGeneral.BarHeight)
		if i == 1 then
			GVAR.HistoryFrame.ScrollFrameButton[i]:SetPoint("TOPLEFT", GVAR.HistoryFrame.ScrollFrame, "TOPLEFT", 0, 0)
		else
			GVAR.HistoryFrame.ScrollFrameButton[i]:SetPoint("TOPLEFT", GVAR.HistoryFrame.ScrollFrameButton[(i-1)], "BOTTOMLEFT", 0, 0)
		end

		GVAR.HistoryFrame.ScrollFrameButton[i].Delete = CreateFrame("Button", nil, GVAR.HistoryFrame.ScrollFrameButton[i])
		GVAR.HistoryFrame.ScrollFrameButton[i].Delete:SetWidth(Textures.Trashcan.width)
		GVAR.HistoryFrame.ScrollFrameButton[i].Delete:SetHeight(Textures.Trashcan.height)
		GVAR.HistoryFrame.ScrollFrameButton[i].Delete:SetPoint("LEFT", GVAR.HistoryFrame.ScrollFrameButton[i], "LEFT", sizeCompareFrame.TrashcanLeft, 0)
		GVAR.HistoryFrame.ScrollFrameButton[i].DeleteTexture = GVAR.HistoryFrame.ScrollFrameButton[i].Delete:CreateTexture(nil, "ARTWORK")
		GVAR.HistoryFrame.ScrollFrameButton[i].DeleteTexture:SetPoint("LEFT", 0, 0)
		GVAR.HistoryFrame.ScrollFrameButton[i].DeleteTexture:SetWidth(Textures.Trashcan.width)
		GVAR.HistoryFrame.ScrollFrameButton[i].DeleteTexture:SetHeight(Textures.Trashcan.height)
		GVAR.HistoryFrame.ScrollFrameButton[i].DeleteTexture:SetTexture(Textures.CharScannerIcons.path)
		GVAR.HistoryFrame.ScrollFrameButton[i].DeleteTexture:SetTexCoord(unpack(Textures.Trashcan.coords))
		GVAR.HistoryFrame.ScrollFrameButton[i].DeleteTexture:SetAlpha(0.5)
		GVAR.HistoryFrame.ScrollFrameButton[i].Delete:SetScript("OnEnter", function() GVAR.HistoryFrame.ScrollFrameButton[i].DeleteTexture:SetAlpha(1) end)
		GVAR.HistoryFrame.ScrollFrameButton[i].Delete:SetScript("OnLeave", function() GVAR.HistoryFrame.ScrollFrameButton[i].DeleteTexture:SetAlpha(0.5) end)
		GVAR.HistoryFrame.ScrollFrameButton[i].Delete:SetScript("OnClick", function() CharScanner:DeleteCharacter(GVAR.HistoryFrame.ScrollFrameButton[i].guid, i) end)

		GVAR.HistoryFrame.ScrollFrameButton[i].Arrow = GVAR.HistoryFrame.ScrollFrameButton[i]:CreateTexture(nil, "BACKGROUND")
		GVAR.HistoryFrame.ScrollFrameButton[i].Arrow:SetWidth(Textures.SelectionArrow.width)
		GVAR.HistoryFrame.ScrollFrameButton[i].Arrow:SetHeight(Textures.SelectionArrow.height)
		GVAR.HistoryFrame.ScrollFrameButton[i].Arrow:SetTexture(Textures.CharScannerIcons.path)
		GVAR.HistoryFrame.ScrollFrameButton[i].Arrow:SetTexCoord(unpack(Textures.SelectionArrow.coords))
		GVAR.HistoryFrame.ScrollFrameButton[i].Arrow:SetPoint("LEFT", GVAR.HistoryFrame.ScrollFrameButton[i], "LEFT", sizeCompareFrame.ArrowLeft, 0)
		GVAR.HistoryFrame.ScrollFrameButton[i].Arrow:Hide()

		GVAR.HistoryFrame.ScrollFrameButton[i].Icon = GVAR.HistoryFrame.ScrollFrameButton[i]:CreateTexture(nil, "ARTWORK")
		GVAR.HistoryFrame.ScrollFrameButton[i].Icon:SetWidth(sizeGeneral.BarHeight)
		GVAR.HistoryFrame.ScrollFrameButton[i].Icon:SetHeight(sizeGeneral.BarHeight)
		GVAR.HistoryFrame.ScrollFrameButton[i].Icon:SetPoint("LEFT", GVAR.HistoryFrame.ScrollFrameButton[i], "LEFT", sizeCompareFrame.ClassiconLeft, 0)
		GVAR.HistoryFrame.ScrollFrameButton[i].Icon:SetTexture(Textures.ClassIcon.path)

		GVAR.HistoryFrame.ScrollFrameButton[i].Name = GVAR.HistoryFrame.ScrollFrameButton[i]:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
		GVAR.HistoryFrame.ScrollFrameButton[i].Name:SetWidth(sizeCompareFrame.NameWidth)
		GVAR.HistoryFrame.ScrollFrameButton[i].Name:SetHeight(fonts.GameFontNormalSmall.height)
		GVAR.HistoryFrame.ScrollFrameButton[i].Name:SetPoint("LEFT", GVAR.HistoryFrame.ScrollFrameButton[i], "LEFT", sizeCompareFrame.NameLeft, 0)
		GVAR.HistoryFrame.ScrollFrameButton[i].Name:SetJustifyH("LEFT")
		GVAR.HistoryFrame.ScrollFrameButton[i].Name:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])

		GVAR.HistoryFrame.ScrollFrameButton[i].Time = GVAR.HistoryFrame.ScrollFrameButton[i]:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
		GVAR.HistoryFrame.ScrollFrameButton[i].Time:SetWidth(sizeCompareFrame.TimeWidth)
		GVAR.HistoryFrame.ScrollFrameButton[i].Time:SetHeight(fonts.GameFontWhiteSmall.height)
		GVAR.HistoryFrame.ScrollFrameButton[i].Time:SetPoint("LEFT", GVAR.HistoryFrame.ScrollFrameButton[i], "LEFT", sizeCompareFrame.TimeLeft, 0)
		GVAR.HistoryFrame.ScrollFrameButton[i].Time:SetJustifyH("LEFT")
		GVAR.HistoryFrame.ScrollFrameButton[i].Time:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])

		GVAR.HistoryFrame.ScrollFrameButton[i].MainRole = GVAR.HistoryFrame.ScrollFrameButton[i]:CreateTexture(nil, "ARTWORK")
		GVAR.HistoryFrame.ScrollFrameButton[i].MainRole:SetWidth(sizeCompareFrame.MainroleWidth)
		GVAR.HistoryFrame.ScrollFrameButton[i].MainRole:SetHeight(sizeCompareFrame.MainroleWidth)
		GVAR.HistoryFrame.ScrollFrameButton[i].MainRole:SetPoint("LEFT", GVAR.HistoryFrame.ScrollFrameButton[i], "LEFT", sizeCompareFrame.MainroleLeft, 0)

		GVAR.HistoryFrame.ScrollFrameButton[i].CompareFrame = CreateFrame("Frame", nil, GVAR.HistoryFrame.ScrollFrameButton[i])
		GVAR.HistoryFrame.ScrollFrameButton[i].CompareFrame:SetToplevel(true)

		GVAR.HistoryFrame.ScrollFrameButton[i].Talents1 = GVAR.HistoryFrame.ScrollFrameButton[i].CompareFrame:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
		GVAR.HistoryFrame.ScrollFrameButton[i].Talents1:SetWidth(sizeCompareFrame.Talents1Width)
		GVAR.HistoryFrame.ScrollFrameButton[i].Talents1:SetHeight(fonts.GameFontWhiteSmall.height)
		GVAR.HistoryFrame.ScrollFrameButton[i].Talents1:SetPoint("LEFT", GVAR.HistoryFrame.ScrollFrameButton[i], "LEFT", sizeCompareFrame.Talents1Left, 0)
		GVAR.HistoryFrame.ScrollFrameButton[i].Talents1:SetJustifyH("LEFT")
		GVAR.HistoryFrame.ScrollFrameButton[i].Talents1:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])

		GVAR.HistoryFrame.ScrollFrameButton[i].Talents2 = GVAR.HistoryFrame.ScrollFrameButton[i].CompareFrame:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
		GVAR.HistoryFrame.ScrollFrameButton[i].Talents2:SetWidth(sizeCompareFrame.Talents2Width)
		GVAR.HistoryFrame.ScrollFrameButton[i].Talents2:SetHeight(fonts.GameFontWhiteSmall.height)
		GVAR.HistoryFrame.ScrollFrameButton[i].Talents2:SetPoint("LEFT", GVAR.HistoryFrame.ScrollFrameButton[i], "LEFT", sizeCompareFrame.Talents2Left, 0)
		GVAR.HistoryFrame.ScrollFrameButton[i].Talents2:SetJustifyH("LEFT")
		GVAR.HistoryFrame.ScrollFrameButton[i].Talents2:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])

		GVAR.HistoryFrame.ScrollFrameButton[i].EnchantmentCount = GVAR.HistoryFrame.ScrollFrameButton[i].CompareFrame:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
		GVAR.HistoryFrame.ScrollFrameButton[i].EnchantmentCount:SetWidth(sizeCompareFrame.EnchWidth)
		GVAR.HistoryFrame.ScrollFrameButton[i].EnchantmentCount:SetHeight(fonts.GameFontWhiteSmall.height)
		GVAR.HistoryFrame.ScrollFrameButton[i].EnchantmentCount:SetPoint("LEFT", GVAR.HistoryFrame.ScrollFrameButton[i], "LEFT", sizeCompareFrame.EnchLeft, 0)
		GVAR.HistoryFrame.ScrollFrameButton[i].EnchantmentCount:SetJustifyH("RIGHT")
		GVAR.HistoryFrame.ScrollFrameButton[i].EnchantmentCount:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])

		GVAR.HistoryFrame.ScrollFrameButton[i].PvPCount = GVAR.HistoryFrame.ScrollFrameButton[i].CompareFrame:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
		GVAR.HistoryFrame.ScrollFrameButton[i].PvPCount:SetWidth(sizeCompareFrame.PvPWidth)
		GVAR.HistoryFrame.ScrollFrameButton[i].PvPCount:SetHeight(fonts.GameFontWhiteSmall.height)
		GVAR.HistoryFrame.ScrollFrameButton[i].PvPCount:SetPoint("LEFT", GVAR.HistoryFrame.ScrollFrameButton[i], "LEFT", sizeCompareFrame.PvPLeft, 0)
		GVAR.HistoryFrame.ScrollFrameButton[i].PvPCount:SetJustifyH("RIGHT")
		GVAR.HistoryFrame.ScrollFrameButton[i].PvPCount:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])

		GVAR.HistoryFrame.ScrollFrameButton[i].ItemLevel = GVAR.HistoryFrame.ScrollFrameButton[i].CompareFrame:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
		GVAR.HistoryFrame.ScrollFrameButton[i].ItemLevel:SetWidth(sizeCompareFrame.EquipWidth)
		GVAR.HistoryFrame.ScrollFrameButton[i].ItemLevel:SetHeight(fonts.GameFontWhiteSmall.height)
		GVAR.HistoryFrame.ScrollFrameButton[i].ItemLevel:SetPoint("LEFT", GVAR.HistoryFrame.ScrollFrameButton[i], "LEFT", sizeCompareFrame.EquipLeft, 0)
		GVAR.HistoryFrame.ScrollFrameButton[i].ItemLevel:SetJustifyH("RIGHT")
		GVAR.HistoryFrame.ScrollFrameButton[i].ItemLevel:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])

		GVAR.HistoryFrame.ScrollFrameButton[i].Achievement = GVAR.HistoryFrame.ScrollFrameButton[i].CompareFrame:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
		GVAR.HistoryFrame.ScrollFrameButton[i].Achievement:SetWidth(sizeCompareFrame.AchievementWidth)
		GVAR.HistoryFrame.ScrollFrameButton[i].Achievement:SetHeight(fonts.GameFontWhiteSmall.height)
		GVAR.HistoryFrame.ScrollFrameButton[i].Achievement:SetPoint("LEFT", GVAR.HistoryFrame.ScrollFrameButton[i], "LEFT", sizeCompareFrame.AchievementLeft, 0)
		GVAR.HistoryFrame.ScrollFrameButton[i].Achievement:SetJustifyH("RIGHT")
		GVAR.HistoryFrame.ScrollFrameButton[i].Achievement:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])

		GVAR.HistoryFrame.ScrollFrameButton[i].Boss = GVAR.HistoryFrame.ScrollFrameButton[i].CompareFrame:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
		GVAR.HistoryFrame.ScrollFrameButton[i].Boss:SetWidth(sizeCompareFrame.BossWidth)
		GVAR.HistoryFrame.ScrollFrameButton[i].Boss:SetHeight(fonts.GameFontWhiteSmall.height)
		GVAR.HistoryFrame.ScrollFrameButton[i].Boss:SetPoint("LEFT", GVAR.HistoryFrame.ScrollFrameButton[i], "LEFT", sizeCompareFrame.BossLeft, 0)
		GVAR.HistoryFrame.ScrollFrameButton[i].Boss:SetJustifyH("RIGHT")
		GVAR.HistoryFrame.ScrollFrameButton[i].Boss:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])

		GVAR.HistoryFrame.ScrollFrameButton[i].Kills = GVAR.HistoryFrame.ScrollFrameButton[i].CompareFrame:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
		GVAR.HistoryFrame.ScrollFrameButton[i].Kills:SetWidth(sizeCompareFrame.KillsWidth)
		GVAR.HistoryFrame.ScrollFrameButton[i].Kills:SetHeight(fonts.GameFontWhiteSmall.height)
		GVAR.HistoryFrame.ScrollFrameButton[i].Kills:SetPoint("LEFT", GVAR.HistoryFrame.ScrollFrameButton[i], "LEFT", sizeCompareFrame.KillsLeft, 0)
		GVAR.HistoryFrame.ScrollFrameButton[i].Kills:SetJustifyH("RIGHT")
		GVAR.HistoryFrame.ScrollFrameButton[i].Kills:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])

		GVAR.HistoryFrame.ScrollFrameButton[i].SingleAchievementCompare = GVAR.HistoryFrame.ScrollFrameButton[i].CompareFrame:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
		GVAR.HistoryFrame.ScrollFrameButton[i].SingleAchievementCompare:SetWidth(sizeCompareFrame.SingleAchievementCompareWidth)
		GVAR.HistoryFrame.ScrollFrameButton[i].SingleAchievementCompare:SetHeight(fonts.GameFontWhiteSmall.height)
		GVAR.HistoryFrame.ScrollFrameButton[i].SingleAchievementCompare:SetPoint("LEFT", GVAR.HistoryFrame.ScrollFrameButton[i], "LEFT", sizeCompareFrame.SingleAchievementCompareLeft, 0)
		GVAR.HistoryFrame.ScrollFrameButton[i].SingleAchievementCompare:SetJustifyH("LEFT")
		GVAR.HistoryFrame.ScrollFrameButton[i].SingleAchievementCompare:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])
		GVAR.HistoryFrame.ScrollFrameButton[i].SingleAchievementCompare:Hide()

		GVAR.HistoryFrame.ScrollFrameButton[i].Highlight = GVAR.HistoryFrame.ScrollFrameButton[i]:CreateTexture(nil, "ARTWORK")
		GVAR.HistoryFrame.ScrollFrameButton[i].Highlight:SetPoint("TOPLEFT", GVAR.HistoryFrame.ScrollFrameButton[i], "TOPLEFT", sizeCompareFrame.ArrowLeft, 0)--sizeCompareFrame.ClassiconLeft, 0)
		GVAR.HistoryFrame.ScrollFrameButton[i].Highlight:SetPoint("BOTTOMRIGHT", GVAR.HistoryFrame.ScrollFrameButton[i], "BOTTOMRIGHT", 0, 0)
		GVAR.HistoryFrame.ScrollFrameButton[i].Highlight:SetTexture(color.ButtonHighlight.rgba[1], color.ButtonHighlight.rgba[2], color.ButtonHighlight.rgba[3], color.ButtonHighlight.rgba[4])
		GVAR.HistoryFrame.ScrollFrameButton[i]:SetHighlightTexture(GVAR.HistoryFrame.ScrollFrameButton[i].Highlight)
		GVAR.HistoryFrame.ScrollFrameButton[i].BlankBG = GVAR.HistoryFrame.ScrollFrameButton[i]:CreateTexture(nil, "BACKGROUND")
		GVAR.HistoryFrame.ScrollFrameButton[i].BlankBG:SetPoint("TOPLEFT", GVAR.HistoryFrame.ScrollFrameButton[i], "TOPLEFT", sizeCompareFrame.NameLeft, -5)
		GVAR.HistoryFrame.ScrollFrameButton[i].BlankBG:SetPoint("BOTTOMRIGHT", GVAR.HistoryFrame.ScrollFrameButton[i], "BOTTOMRIGHT", 0, 5)
		GVAR.HistoryFrame.ScrollFrameButton[i].BlankBG:SetTexture(color.ButtonLineMarker.rgba[1], color.ButtonLineMarker.rgba[2], color.ButtonLineMarker.rgba[3], color.ButtonLineMarker.rgba[4])

		GVAR.HistoryFrame.ScrollFrameButton[i]:SetScript("OnClick", function() CharScanner:PreShowSavedData(GVAR.HistoryFrame.ScrollFrameButton[i].guid, true) end)
	end
	-- - History ScrollFrame END

	-- - History StatusFrame START
	GVAR.HistoryFrame.Status = CreateFrame("Frame", nil, GVAR.HistoryFrame)
	TEMPLATE.BorderTRBL(GVAR.HistoryFrame.Status)
	GVAR.HistoryFrame.Status:SetWidth(sizeCompareFrame.HistoryWidth)
	GVAR.HistoryFrame.Status:SetHeight(sizeGeneral.StatusHeight)
	GVAR.HistoryFrame.Status:SetPoint("TOPLEFT", GVAR.HistoryFrame, "BOTTOMLEFT", 0, sizeGeneral.Border)
	GVAR.HistoryFrame.Status:EnableMouse(true)

	GVAR.HistoryFrame.Status.FakeTextBG = GVAR.HistoryFrame.Status:CreateTexture(nil, "ARTWORK")
	GVAR.HistoryFrame.Status.FakeTextBG:SetWidth(150)
	GVAR.HistoryFrame.Status.FakeTextBG:SetHeight(sizeGeneral.StatusHeight)
	GVAR.HistoryFrame.Status.FakeTextBG:SetPoint("TOPLEFT", GVAR.HistoryFrame.Status, "TOPLEFT", (sizeCompareFrame.HistoryWidth-150)/2, 0)
	GVAR.HistoryFrame.Status.FakeTextBG:SetTexture(0, 0, 0, 0)
	GVAR.HistoryFrame.Status.Text = GVAR.HistoryFrame.Status:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.HistoryFrame.Status.Text:SetPoint("CENTER", GVAR.HistoryFrame.Status.FakeTextBG, "CENTER", 0, 0)
	GVAR.HistoryFrame.Status.Text:SetTextColor(1, 1, 1, 1)
	GVAR.HistoryFrame.Status.FakeTexture = GVAR.HistoryFrame.Sort.CompareFrame:CreateTexture(nil, "ARTWORK")
	GVAR.HistoryFrame.Status.FakeTexture:SetPoint("TOPLEFT", GVAR.HistoryFrame.Status, "TOPLEFT", sizeCompareFrame.Talents1Left, -sizeGeneral.Space)
	GVAR.HistoryFrame.Status.FakeTexture:SetPoint("BOTTOMRIGHT", GVAR.HistoryFrame.Status, "TOPLEFT", sizeCompareFrame.Talents2Left+sizeCompareFrame.Talents1Width, -(sizeGeneral.StatusHeight-sizeGeneral.Space))
	GVAR.HistoryFrame.Status.FakeTexture:SetTexture(1, 1, 1, 0.075)

	-- 4 = 1 = TANK, 2 = HEAL, 3 = DAMAGE, 4 = UNKNOWN (lvl>9 with all talentpoints unspent)
	for i = 1, 4 do
		if not GVAR.HistoryFrame.Status.Role then GVAR.HistoryFrame.Status.Role = {} end
		if not GVAR.HistoryFrame.Status.Role[i] then GVAR.HistoryFrame.Status.Role[i] = {} end
		GVAR.HistoryFrame.Status.Role[i].Icon = GVAR.HistoryFrame.Sort.CompareFrame:CreateTexture(nil, "ARTWORK")
		GVAR.HistoryFrame.Status.Role[i].Icon:SetWidth(Textures.RoleIcon.sWidth)
		GVAR.HistoryFrame.Status.Role[i].Icon:SetHeight(Textures.RoleIcon.sHeight)
		GVAR.HistoryFrame.Status.Role[i].Icon:SetTexture(Textures.RoleIcon[i].path)
		GVAR.HistoryFrame.Status.Role[i].Icon:SetTexCoord(Textures.RoleIcon[i].coords[1], Textures.RoleIcon[i].coords[2], Textures.RoleIcon[i].coords[3], Textures.RoleIcon[i].coords[4])
		GVAR.HistoryFrame.Status.Role[i].Icon:SetAlpha(0.25)
		GVAR.HistoryFrame.Status.Role[i].Text = GVAR.HistoryFrame.Sort.CompareFrame:CreateFontString(nil, "OVERLAY", fonts.GameFontNormal.name)
		GVAR.HistoryFrame.Status.Role[i].Text:SetText("- (-)")
		GVAR.HistoryFrame.Status.Role[i].Text:SetJustifyH("LEFT")
		GVAR.HistoryFrame.Status.Role[i].Text:SetTextColor(color.Grey.rgba[1], color.Grey.rgba[2], color.Grey.rgba[3], color.Grey.rgba[4])

		if i == 1 then
			GVAR.HistoryFrame.Status.Role[i].Icon:SetPoint("LEFT", GVAR.HistoryFrame.Status.FakeTexture, "LEFT", (2*sizeGeneral.Offset), 0)
			GVAR.HistoryFrame.Status.Role[i].Text:SetPoint("LEFT", GVAR.HistoryFrame.Status.Role[i].Icon, "RIGHT", sizeGeneral.Space, 0)
		elseif i == 2 then
			GVAR.HistoryFrame.Status.Role[i].Icon:SetPoint("LEFT", GVAR.HistoryFrame.Status.Role[(i-1)].Text, "RIGHT", (2*sizeGeneral.Offset), 0)
			GVAR.HistoryFrame.Status.Role[i].Text:SetPoint("LEFT", GVAR.HistoryFrame.Status.Role[i].Icon, "RIGHT", sizeGeneral.Space, 0)
		elseif i == 3 then
			GVAR.HistoryFrame.Status.Role[i].Icon:SetPoint("LEFT", GVAR.HistoryFrame.Status.Role[(i-1)].Text, "RIGHT", (2*sizeGeneral.Offset), 0)
			GVAR.HistoryFrame.Status.Role[i].Text:SetPoint("LEFT", GVAR.HistoryFrame.Status.Role[i].Icon, "RIGHT", sizeGeneral.Space, 0)
		else
			GVAR.HistoryFrame.Status.Role[i].Text:SetPoint("RIGHT", GVAR.HistoryFrame.Status.FakeTexture, "RIGHT", -(2*sizeGeneral.Offset), 0)
			GVAR.HistoryFrame.Status.Role[i].Icon:SetPoint("RIGHT", GVAR.HistoryFrame.Status.Role[i].Text, "LEFT", -sizeGeneral.Space, 0)
		end
	end
	-- - History StatusFrame END
	-- History Frame END



	-- OnUpdate-UIParent Buttons START
	CharScanner.OnUpdateSingleScan  = CreateFrame("Button", nil, UIParent)
	CharScanner.OnUpdateGroupScan   = CreateFrame("Button", nil, UIParent)
	CharScanner.OnUpdateRestriction = CreateFrame("Button", nil, UIParent)
	-- OnUpdate-UIParent Buttons END

	-- Main Frame SetSript START
	GVAR.MainFrame:SetScript("OnShow", function() CharScanner:MainFrame_Show() end)
	GVAR.MainFrame:SetScript("OnHide", function() CharScanner:MainFrame_Hide() end)
	-- Main Frame SetSript END
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function CharScanner:CreateOptionsFrame()
	if CharScanner_OptionsFrame then return end

	-- Main Frame START
	GVAR.OptionsFrame = CreateFrame("Frame", "CharScanner_OptionsFrame", UIParent)
	TEMPLATE.BorderTRBL(GVAR.OptionsFrame)
	GVAR.OptionsFrame:EnableMouse(true)
	GVAR.OptionsFrame:SetMovable(true)
	GVAR.OptionsFrame:SetToplevel(true)
	GVAR.OptionsFrame:SetClampedToScreen(true)
	GVAR.OptionsFrame:SetWidth(350)
	GVAR.OptionsFrame:SetScript("OnShow", function()
		CharScanner:Frame_SetupPosition("CharScanner_OptionsFrame")
		TEMPLATE.DisableIconButton(GVAR.MainFrame.Headline.Options)
		TEMPLATE.DisableTextButton(GVAR.ScanButtonFrame.Mover.OPTIONS)
		TEMPLATE.DisableTextButton(GVAR.InterfaceOptions.OPTIONS)
		GVAR.MiniOptionsFrame.Options:Disable()
		GVAR.MiniOptionsFrame.Options.Text:SetTextColor(color.Grey.rgba[1], color.Grey.rgba[2], color.Grey.rgba[3], color.Grey.rgba[4])
	end)
	GVAR.OptionsFrame:SetScript("OnHide", function()
		TEMPLATE.EnableIconButton(GVAR.MainFrame.Headline.Options)
		TEMPLATE.EnableTextButton(GVAR.ScanButtonFrame.Mover.OPTIONS, 1)
		TEMPLATE.EnableTextButton(GVAR.InterfaceOptions.OPTIONS, 1)
		GVAR.MiniOptionsFrame.Options:Enable()
		GVAR.MiniOptionsFrame.Options.Text:SetTextColor(color.Yellow.rgba[1], color.Yellow.rgba[2], color.Yellow.rgba[3], color.Yellow.rgba[4])
	end)
	GVAR.OptionsFrame:Hide()
	-- Main Frame END

	-- Headline Frame START
	GVAR.OptionsFrame.Headline = CreateFrame("Frame", nil, GVAR.OptionsFrame)
	TEMPLATE.BorderTRBL(GVAR.OptionsFrame.Headline)
	GVAR.OptionsFrame.Headline:SetWidth(350)
	GVAR.OptionsFrame.Headline:SetHeight(sizeGeneral.MainHeight)
	GVAR.OptionsFrame.Headline:SetPoint("TOPLEFT", GVAR.OptionsFrame, "TOPLEFT", 0, 0)
	GVAR.OptionsFrame.Headline:EnableMouse(true)

	GVAR.OptionsFrame.Headline.Text = GVAR.OptionsFrame.Headline:CreateFontString(nil, "ARTWORK", fonts.GameFontNormal.name)
	GVAR.OptionsFrame.Headline.Text:SetPoint("LEFT", GVAR.OptionsFrame.Headline, "LEFT", 10, 0)
	GVAR.OptionsFrame.Headline.Text:SetJustifyH("LEFT")
	GVAR.OptionsFrame.Headline.Text:SetText("CharScanner "..L["Options"])

	GVAR.OptionsFrame.Headline.Close = CreateFrame("Button", nil, GVAR.OptionsFrame.Headline)
	TEMPLATE.IconButton(GVAR.OptionsFrame.Headline.Close, 9)
	GVAR.OptionsFrame.Headline.Close:SetPoint("RIGHT", GVAR.OptionsFrame.Headline, "RIGHT", -sizeGeneral.Offset, 0)
	GVAR.OptionsFrame.Headline.Close:SetScript("OnClick", function() CharScanner:Frame_Toggle(GVAR.OptionsFrame) end)

	GVAR.OptionsFrame.Headline.ButtonMover = CreateFrame("Button", nil, GVAR.OptionsFrame.Headline)
	GVAR.OptionsFrame.Headline.ButtonMover:SetWidth(22)
	GVAR.OptionsFrame.Headline.ButtonMover:SetHeight(22)
	GVAR.OptionsFrame.Headline.ButtonMover:SetPoint("TOPLEFT", GVAR.OptionsFrame.Headline, "TOPLEFT", sizeGeneral.Offset, 0)
	GVAR.OptionsFrame.Headline.ButtonMover:SetPoint("BOTTOMRIGHT", GVAR.OptionsFrame.Headline.Close, "BOTTOMLEFT", -sizeGeneral.Offset, 0)
	GVAR.OptionsFrame.Headline.ButtonMoverTexture = GVAR.OptionsFrame.Headline.ButtonMover:CreateTexture(nil, "BACKGROUND")
	GVAR.OptionsFrame.Headline.ButtonMoverTexture:SetWidth(1)
	GVAR.OptionsFrame.Headline.ButtonMoverTexture:SetHeight(1)
	GVAR.OptionsFrame.Headline.ButtonMoverTexture:SetPoint("TOPLEFT", GVAR.OptionsFrame.Headline.ButtonMover, "TOPLEFT", 0, 0)
	GVAR.OptionsFrame.Headline.ButtonMoverTexture:SetPoint("BOTTOMRIGHT", GVAR.OptionsFrame.Headline.ButtonMover, "BOTTOMRIGHT", 0, 0)
	GVAR.OptionsFrame.Headline.ButtonMoverTexture:SetTexture(1, 1, 1, 0)
	GVAR.OptionsFrame.Headline.ButtonMover:SetScript("OnEnter", function() GVAR.OptionsFrame.Headline.ButtonMoverTexture:SetTexture(1, 1, 1, 0.075) CharScanner:Frame_MoverInfo(GVAR.OptionsFrame, "MOVE", "TOP") end)
	GVAR.OptionsFrame.Headline.ButtonMover:SetScript("OnLeave", function() GVAR.OptionsFrame.Headline.ButtonMoverTexture:SetTexture(1, 1, 1, 0) CharScanner:Frame_MoverInfo(GVAR.OptionsFrame) end)
	GVAR.OptionsFrame.Headline.ButtonMover:SetScript("OnMouseDown", MoveStart)
	GVAR.OptionsFrame.Headline.ButtonMover:SetScript("OnMouseUp", MoveStop)
	-- Headline Frame END

	-- Base Frame START
	GVAR.OptionsFrame.Base = CreateFrame("Frame", nil, GVAR.OptionsFrame)
	TEMPLATE.BorderTRBL(GVAR.OptionsFrame.Base)
	GVAR.OptionsFrame.Base:SetWidth(350)
	GVAR.OptionsFrame.Base:SetHeight(500)
	GVAR.OptionsFrame.Base:SetPoint("TOPLEFT", GVAR.OptionsFrame.Headline, "BOTTOMLEFT", 0, sizeGeneral.Border)
	GVAR.OptionsFrame.Base:EnableMouse(true)
	-- Base Frame END

	-- Status Frame START
	GVAR.OptionsFrame.Status = CreateFrame("Frame", nil, GVAR.OptionsFrame)
	TEMPLATE.BorderTRBL(GVAR.OptionsFrame.Status)
	GVAR.OptionsFrame.Status:SetWidth(350)
	GVAR.OptionsFrame.Status:SetHeight(sizeGeneral.StatusHeight)
	GVAR.OptionsFrame.Status:SetPoint("TOPLEFT", GVAR.OptionsFrame.Base, "BOTTOMLEFT", 0, sizeGeneral.Border)
	GVAR.OptionsFrame.Status:EnableMouse(true)
	GVAR.OptionsFrame.Status:SetScript("OnEnter", function() CharScanner:Frame_MoverInfo(GVAR.OptionsFrame, "MOVE", "BOTTOM") end)
	GVAR.OptionsFrame.Status:SetScript("OnLeave", function() CharScanner:Frame_MoverInfo(GVAR.OptionsFrame) end)
	GVAR.OptionsFrame.Status:SetScript("OnMouseDown", MoveStart)
	GVAR.OptionsFrame.Status:SetScript("OnMouseUp", MoveStop)
	-- Status Frame END

	-- Mover Info START
	GVAR.OptionsFrame.MoverInfoTop = CreateFrame("Frame", nil, GVAR.OptionsFrame)
	GVAR.OptionsFrame.MoverInfoTop:SetWidth(150)
	GVAR.OptionsFrame.MoverInfoTop:SetHeight(14)
	GVAR.OptionsFrame.MoverInfoTop:SetPoint("BOTTOM", GVAR.OptionsFrame, "TOP", 0, 0)
	GVAR.OptionsFrame.MoverInfoTop:EnableMouse(true)
	GVAR.OptionsFrame.MoverInfoTop:Hide()
	GVAR.OptionsFrame.MoverInfoTopTexture = GVAR.OptionsFrame.MoverInfoTop:CreateTexture(nil, "BACKGROUND")
	GVAR.OptionsFrame.MoverInfoTopTexture:SetAllPoints(GVAR.OptionsFrame.MoverInfoTop)
	GVAR.OptionsFrame.MoverInfoTopTexture:SetTexture(0, 0, 0, 0.5)
	GVAR.OptionsFrame.MoverInfoTopText = GVAR.OptionsFrame.MoverInfoTop:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.OptionsFrame.MoverInfoTopText:SetPoint("CENTER", GVAR.OptionsFrame.MoverInfoTop, "CENTER", 0, 0)
	GVAR.OptionsFrame.MoverInfoTopText:SetJustifyH("CENTER")
	GVAR.OptionsFrame.MoverInfoTopText:SetTextColor(0.6, 0.6, 0.6, 1)

	GVAR.OptionsFrame.MoverInfoBottom = CreateFrame("Frame", nil, GVAR.OptionsFrame)
	GVAR.OptionsFrame.MoverInfoBottom:SetWidth(150)
	GVAR.OptionsFrame.MoverInfoBottom:SetHeight(14)
	GVAR.OptionsFrame.MoverInfoBottom:SetPoint("TOP", GVAR.OptionsFrame.Status, "BOTTOM", 0, 0)
	GVAR.OptionsFrame.MoverInfoBottom:EnableMouse(true)
	GVAR.OptionsFrame.MoverInfoBottom:Hide()
	GVAR.OptionsFrame.MoverInfoBottomTexture = GVAR.OptionsFrame.MoverInfoBottom:CreateTexture(nil, "BACKGROUND")
	GVAR.OptionsFrame.MoverInfoBottomTexture:SetAllPoints(GVAR.OptionsFrame.MoverInfoBottom)
	GVAR.OptionsFrame.MoverInfoBottomTexture:SetTexture(0, 0, 0, 0.5)
	GVAR.OptionsFrame.MoverInfoBottomText = GVAR.OptionsFrame.MoverInfoBottom:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.OptionsFrame.MoverInfoBottomText:SetPoint("CENTER", GVAR.OptionsFrame.MoverInfoBottom, "CENTER", 0, 0)
	GVAR.OptionsFrame.MoverInfoBottomText:SetJustifyH("CENTER")
	GVAR.OptionsFrame.MoverInfoBottomText:SetTextColor(0.6, 0.6, 0.6, 1)
	-- Mover Info END

	local height = GVAR.OptionsFrame.Headline:GetHeight() + (GVAR.OptionsFrame.Base:GetHeight()-sizeGeneral.Border) + (GVAR.OptionsFrame.Status:GetHeight()-sizeGeneral.Border)
	GVAR.OptionsFrame:SetHeight(height)
	GVAR.OptionsFrame:SetClampRectInsets(0, 0, 0, height-sizeGeneral.MainHeight)

	-- Options START
	GVAR.OptionsFrame.OpenGUI = CreateFrame("Button", nil, GVAR.OptionsFrame.Base)
	TEMPLATE.TextButton(GVAR.OptionsFrame.OpenGUI, L["Open GUI"], 1)
	GVAR.OptionsFrame.OpenGUI:SetWidth(150)
	GVAR.OptionsFrame.OpenGUI:SetHeight(22)
	GVAR.OptionsFrame.OpenGUI:SetPoint("TOP", GVAR.OptionsFrame.Base, "TOP", 0, -4)
	GVAR.OptionsFrame.OpenGUI:SetScript("OnClick", function()
		CharScanner:Frame_Toggle(GVAR.MainFrame, 1)
	end)

	-- - scanbutton START
	GVAR.OptionsFrame.ScanButtonText = GVAR.OptionsFrame.Base:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.OptionsFrame.ScanButtonText:SetPoint("LEFT", GVAR.OptionsFrame.Base, "LEFT", 10, -10)
	GVAR.OptionsFrame.ScanButtonText:SetPoint("TOP", GVAR.OptionsFrame.OpenGUI, "BOTTOM", 0, -10)
	GVAR.OptionsFrame.ScanButtonText:SetJustifyH("LEFT")
	GVAR.OptionsFrame.ScanButtonText:SetText(L["SCAN-Button"]..":")

	GVAR.OptionsFrame.ScanButton = CreateFrame("CheckButton", nil, GVAR.OptionsFrame.Base)
	TEMPLATE.CheckButton(GVAR.OptionsFrame.ScanButton, 16, 4, L["Show SCAN-Button"])
	GVAR.OptionsFrame.ScanButton:SetPoint("LEFT", GVAR.OptionsFrame.Base, "LEFT", 10, 0)
	GVAR.OptionsFrame.ScanButton:SetPoint("TOP", GVAR.OptionsFrame.ScanButtonText, "BOTTOM", 0, -10)
	GVAR.OptionsFrame.ScanButton:SetChecked(CharScanner_Options.scanButton)
	TEMPLATE.EnableCheckButton(GVAR.OptionsFrame.ScanButton)

	GVAR.OptionsFrame.ScanButtonInfo = GVAR.OptionsFrame.Base:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.OptionsFrame.ScanButtonInfo:SetWidth(310)
	GVAR.OptionsFrame.ScanButtonInfo:SetPoint("LEFT", GVAR.OptionsFrame.ScanButton, "LEFT", 16+4, 0)
	GVAR.OptionsFrame.ScanButtonInfo:SetPoint("TOP", GVAR.OptionsFrame.ScanButton, "BOTTOM", 0, -5)
	GVAR.OptionsFrame.ScanButtonInfo:SetJustifyH("LEFT")
	GVAR.OptionsFrame.ScanButtonInfo:SetText(L["buttonshow_desc"])
	GVAR.OptionsFrame.ScanButtonInfo:SetTextColor(0.6, 0.6, 0.6, 1)

	GVAR.OptionsFrame.ScanButton.GroupScanAll = CreateFrame("CheckButton", nil, GVAR.OptionsFrame.Base)
	TEMPLATE.CheckButton(GVAR.OptionsFrame.ScanButton.GroupScanAll, 16, 4, L["Show Button for GroupScan (all)"])
	GVAR.OptionsFrame.ScanButton.GroupScanAll:SetPoint("TOP", GVAR.OptionsFrame.ScanButtonInfo, "BOTTOM", 0, -10)
	GVAR.OptionsFrame.ScanButton.GroupScanAll:SetPoint("LEFT", GVAR.OptionsFrame.ScanButton, "LEFT", 16+4, 0)
	GVAR.OptionsFrame.ScanButton.GroupScanAll:SetChecked(CharScanner_Options.scanButtonGrpAll)
	if CharScanner_Options.scanButton then
		TEMPLATE.EnableCheckButton(GVAR.OptionsFrame.ScanButton.GroupScanAll)
	else
		TEMPLATE.DisableCheckButton(GVAR.OptionsFrame.ScanButton.GroupScanAll)
	end

	GVAR.OptionsFrame.ScanButton.GroupScanMissing = CreateFrame("CheckButton", nil, GVAR.OptionsFrame.Base)
	TEMPLATE.CheckButton(GVAR.OptionsFrame.ScanButton.GroupScanMissing, 16, 4, L["Show Button for GroupScan (missing)"])
	GVAR.OptionsFrame.ScanButton.GroupScanMissing:SetPoint("TOP", GVAR.OptionsFrame.ScanButton.GroupScanAll, "BOTTOM", 0, -2)
	GVAR.OptionsFrame.ScanButton.GroupScanMissing:SetPoint("LEFT", GVAR.OptionsFrame.ScanButton, "LEFT", 16+4, 0)
	GVAR.OptionsFrame.ScanButton.GroupScanMissing:SetChecked(CharScanner_Options.scanButtonGrpMiss)
	if CharScanner_Options.scanButton then
		TEMPLATE.EnableCheckButton(GVAR.OptionsFrame.ScanButton.GroupScanMissing)
	else
		TEMPLATE.DisableCheckButton(GVAR.OptionsFrame.ScanButton.GroupScanMissing)
	end

	GVAR.OptionsFrame.ScanButtonGroupInfo = GVAR.OptionsFrame.Base:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.OptionsFrame.ScanButtonGroupInfo:SetWidth(290)
	GVAR.OptionsFrame.ScanButtonGroupInfo:SetPoint("LEFT", GVAR.OptionsFrame.ScanButton.GroupScanMissing, "LEFT", 16+4, 0)
	GVAR.OptionsFrame.ScanButtonGroupInfo:SetPoint("TOP", GVAR.OptionsFrame.ScanButton.GroupScanMissing, "BOTTOM", 0, -5)
	GVAR.OptionsFrame.ScanButtonGroupInfo:SetJustifyH("LEFT")
	GVAR.OptionsFrame.ScanButtonGroupInfo:SetText(L["groupbutton_desc"])
	GVAR.OptionsFrame.ScanButtonGroupInfo:SetTextColor(0.6, 0.6, 0.6, 1)

	GVAR.OptionsFrame.ScanButton.Lock = CreateFrame("CheckButton", nil, GVAR.OptionsFrame.Base)
	TEMPLATE.CheckButton(GVAR.OptionsFrame.ScanButton.Lock, 16, 4, L["Lock SCAN-Button"])
	GVAR.OptionsFrame.ScanButton.Lock:SetPoint("TOP", GVAR.OptionsFrame.ScanButtonGroupInfo, "BOTTOM", 0, -10)
	GVAR.OptionsFrame.ScanButton.Lock:SetPoint("LEFT", GVAR.OptionsFrame.ScanButton, "LEFT", 16+4, 0)
	GVAR.OptionsFrame.ScanButton.Lock:SetChecked(CharScanner_Options.scanButtonLock)
	if CharScanner_Options.scanButton then
		TEMPLATE.EnableCheckButton(GVAR.OptionsFrame.ScanButton.Lock)
	else
		TEMPLATE.DisableCheckButton(GVAR.OptionsFrame.ScanButton.Lock)
	end

	GVAR.OptionsFrame.ScanButton:SetScript("OnClick", function(self)
		CharScanner_Options.scanButton = not CharScanner_Options.scanButton
		if CharScanner_Options.scanButton then
			GVAR.ScanButtonFrame:Show()
			GVAR.ScanButtonFrame.Mover:Show()
			TEMPLATE.EnableCheckButton(GVAR.OptionsFrame.ScanButton.GroupScanAll)
			TEMPLATE.EnableCheckButton(GVAR.OptionsFrame.ScanButton.GroupScanMissing)
			TEMPLATE.EnableCheckButton(GVAR.OptionsFrame.ScanButton.Lock)
		else
			GVAR.ScanButtonFrame:Hide()
			GVAR.ScanButtonFrame.GroupScanAll:Hide()
			GVAR.ScanButtonFrame.GroupScanMissing:Hide()
			CharScanner_Options.scanButtonGrpAll = false
			CharScanner_Options.scanButtonGrpMiss = false
			CharScanner_Options.scanButtonLock = false
			GVAR.OptionsFrame.ScanButton.GroupScanAll:SetChecked(false)
			GVAR.OptionsFrame.ScanButton.GroupScanMissing:SetChecked(false)
			GVAR.OptionsFrame.ScanButton.Lock:SetChecked(false)
			TEMPLATE.DisableCheckButton(GVAR.OptionsFrame.ScanButton.GroupScanAll)
			TEMPLATE.DisableCheckButton(GVAR.OptionsFrame.ScanButton.GroupScanMissing)
			TEMPLATE.DisableCheckButton(GVAR.OptionsFrame.ScanButton.Lock)
		end
	end)

	GVAR.OptionsFrame.ScanButton.GroupScanAll:SetScript("OnClick", function(self)
		CharScanner_Options.scanButtonGrpAll = not CharScanner_Options.scanButtonGrpAll
		if CharScanner_Options.scanButtonGrpAll then
			if CharScanner_Options.scanButtonGrpMiss then
				GVAR.ScanButtonFrame.GroupScanMissing:SetPoint("TOPLEFT", GVAR.ScanButtonFrame.GroupScanAll, "TOPRIGHT", sizeGeneral.Offset, 0)
			end
			if not CharScanner_Options.scanButtonLock then
				GVAR.ScanButtonFrame.GroupScanAll:Show()
			end
		else
			if CharScanner_Options.scanButtonGrpMiss then
				GVAR.ScanButtonFrame.GroupScanMissing:SetPoint("TOPLEFT", GVAR.ScanButtonFrame.SingleScan, "TOPRIGHT", sizeGeneral.Offset, 0)
			end
			GVAR.ScanButtonFrame.GroupScanAll:Hide()
		end
	end)

	GVAR.OptionsFrame.ScanButton.GroupScanMissing:SetScript("OnClick", function(self)
		CharScanner_Options.scanButtonGrpMiss = not CharScanner_Options.scanButtonGrpMiss
		if CharScanner_Options.scanButtonGrpMiss then
			if CharScanner_Options.scanButtonGrpAll then
				GVAR.ScanButtonFrame.GroupScanMissing:SetPoint("TOPLEFT", GVAR.ScanButtonFrame.GroupScanAll, "TOPRIGHT", sizeGeneral.Offset, 0)
			else
				GVAR.ScanButtonFrame.GroupScanMissing:SetPoint("TOPLEFT", GVAR.ScanButtonFrame.SingleScan, "TOPRIGHT", sizeGeneral.Offset, 0)
			end
			if not CharScanner_Options.scanButtonLock then
				GVAR.ScanButtonFrame.GroupScanMissing:Show()
			end
		else
			GVAR.ScanButtonFrame.GroupScanMissing:Hide()
		end
	end)

	GVAR.OptionsFrame.ScanButton.Lock:SetScript("OnClick", function(self)
		CharScanner_Options.scanButtonLock = not CharScanner_Options.scanButtonLock
		if CharScanner_Options.scanButtonLock then
			GVAR.ScanButtonFrame.Mover:Hide()
			if UnitIsPlayer("target") then
				GVAR.ScanButtonFrame:Show()
				if inPartyOrRaid then
					if CharScanner_Options.scanButtonGrpAll then
						GVAR.ScanButtonFrame.GroupScanAll:Show()
					end
					if CharScanner_Options.scanButtonGrpMiss then
						GVAR.ScanButtonFrame.GroupScanMissing:Show()
					end
				else
					GVAR.ScanButtonFrame.GroupScanAll:Hide()
					GVAR.ScanButtonFrame.GroupScanMissing:Hide()
				end
			else
				GVAR.ScanButtonFrame:Hide()
			end
		else
			GVAR.ScanButtonFrame:Show()
			GVAR.ScanButtonFrame.Mover:Show()
			if CharScanner_Options.scanButtonGrpAll then
				GVAR.ScanButtonFrame.GroupScanAll:Show()
			end
			if CharScanner_Options.scanButtonGrpMiss then
				GVAR.ScanButtonFrame.GroupScanMissing:Show()
			end
		end
	end)
	-- - scanbutton END

	-- - scanduration START
	GVAR.OptionsFrame.ScanDuration = GVAR.OptionsFrame.Base:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.OptionsFrame.ScanDuration:SetHeight(20)
	GVAR.OptionsFrame.ScanDuration:SetPoint("LEFT", GVAR.OptionsFrame.Base, "LEFT", 10, 0)
	GVAR.OptionsFrame.ScanDuration:SetPoint("TOP", GVAR.OptionsFrame.ScanButton.Lock, "BOTTOM", 0, -20)
	GVAR.OptionsFrame.ScanDuration:SetJustifyH("LEFT")
	GVAR.OptionsFrame.ScanDuration:SetText(L["Maximum duration for one scan"]..":")

	GVAR.OptionsFrame.ScanDurationSlider = CreateFrame("Slider", nil, GVAR.OptionsFrame.Base)
	TEMPLATE.Slider(GVAR.OptionsFrame.ScanDurationSlider, 150, 1, 10, 20, CharScanner_Options.scanDuration,
	function(self, value)
		CharScanner_Options.scanDuration = value
	end,
	nil,
	10)
	GVAR.OptionsFrame.ScanDurationSlider:SetPoint("LEFT", GVAR.OptionsFrame.ScanDuration, "LEFT", 0, 0)
	GVAR.OptionsFrame.ScanDurationSlider:SetPoint("TOP", GVAR.OptionsFrame.ScanDuration, "BOTTOM", 0, -5)

	GVAR.OptionsFrame.ScanDurationInfo = GVAR.OptionsFrame.Base:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.OptionsFrame.ScanDurationInfo:SetWidth(330)
	GVAR.OptionsFrame.ScanDurationInfo:SetPoint("LEFT", GVAR.OptionsFrame.ScanDurationSlider, "LEFT", 0, 0)
	GVAR.OptionsFrame.ScanDurationInfo:SetPoint("TOP", GVAR.OptionsFrame.ScanDurationSlider, "BOTTOM", 0, -20)
	GVAR.OptionsFrame.ScanDurationInfo:SetJustifyH("LEFT")
	GVAR.OptionsFrame.ScanDurationInfo:SetText(L["scanduration_desc"])
	GVAR.OptionsFrame.ScanDurationInfo:SetTextColor(0.6, 0.6, 0.6, 1)
	-- - scanduration END

	-- - chatoutput START
	GVAR.OptionsFrame.ChatOutputText = GVAR.OptionsFrame.Base:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.OptionsFrame.ChatOutputText:SetPoint("LEFT", GVAR.OptionsFrame.Base, "LEFT", 10, 0)
	GVAR.OptionsFrame.ChatOutputText:SetPoint("TOP", GVAR.OptionsFrame.ScanDurationInfo, "BOTTOM", 0, -20)
	GVAR.OptionsFrame.ChatOutputText:SetJustifyH("LEFT")
	GVAR.OptionsFrame.ChatOutputText:SetText(L["Chat output"]..":")

	GVAR.OptionsFrame.ChatOutput = CreateFrame("CheckButton", nil, GVAR.OptionsFrame.Base)
	TEMPLATE.CheckButton(GVAR.OptionsFrame.ChatOutput, 16, 4, L["Chat output"])
	GVAR.OptionsFrame.ChatOutput:SetPoint("LEFT", GVAR.OptionsFrame.Base, "LEFT", 10, 0)
	GVAR.OptionsFrame.ChatOutput:SetPoint("TOP", GVAR.OptionsFrame.ChatOutputText, "BOTTOM", 0, -10)
	GVAR.OptionsFrame.ChatOutput:SetChecked(CharScanner_Options.chatoutput)
	TEMPLATE.EnableCheckButton(GVAR.OptionsFrame.ChatOutput)

	GVAR.OptionsFrame.ChatOutputInfo = GVAR.OptionsFrame.Base:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.OptionsFrame.ChatOutputInfo:SetWidth(310)
	GVAR.OptionsFrame.ChatOutputInfo:SetPoint("LEFT", GVAR.OptionsFrame.ChatOutput, "LEFT", 16+4, 0)
	GVAR.OptionsFrame.ChatOutputInfo:SetPoint("TOP", GVAR.OptionsFrame.ChatOutput, "BOTTOM", 0, -5)
	GVAR.OptionsFrame.ChatOutputInfo:SetJustifyH("LEFT")
	GVAR.OptionsFrame.ChatOutputInfo:SetText(L["chat_desc"])
	GVAR.OptionsFrame.ChatOutputInfo:SetTextColor(0.6, 0.6, 0.6, 1)

	GVAR.OptionsFrame.ChatOutputGrp = CreateFrame("CheckButton", nil, GVAR.OptionsFrame.Base)
	TEMPLATE.CheckButton(GVAR.OptionsFrame.ChatOutputGrp, 16, 4, L["GroupScan: Show only one chat message"])
	GVAR.OptionsFrame.ChatOutputGrp:SetPoint("LEFT", GVAR.OptionsFrame.ChatOutput, "LEFT", 16+4, 0)
	GVAR.OptionsFrame.ChatOutputGrp:SetPoint("TOP", GVAR.OptionsFrame.ChatOutputInfo, "BOTTOM", 0, -10)
	GVAR.OptionsFrame.ChatOutputGrp:SetChecked(CharScanner_Options.chatoutputGrp)
	if CharScanner_Options.chatoutput then
		TEMPLATE.EnableCheckButton(GVAR.OptionsFrame.ChatOutputGrp)
	else
		TEMPLATE.DisableCheckButton(GVAR.OptionsFrame.ChatOutputGrp)
	end

	GVAR.OptionsFrame.ChatOutputGrpInfo = GVAR.OptionsFrame.Base:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.OptionsFrame.ChatOutputGrpInfo:SetWidth(290)
	GVAR.OptionsFrame.ChatOutputGrpInfo:SetPoint("LEFT", GVAR.OptionsFrame.ChatOutputGrp, "LEFT", 16+4, 0)
	GVAR.OptionsFrame.ChatOutputGrpInfo:SetPoint("TOP", GVAR.OptionsFrame.ChatOutputGrp, "BOTTOM", 0, -5)
	GVAR.OptionsFrame.ChatOutputGrpInfo:SetJustifyH("LEFT")
	GVAR.OptionsFrame.ChatOutputGrpInfo:SetText(L["chatgrp_desc"])
	GVAR.OptionsFrame.ChatOutputGrpInfo:SetTextColor(0.6, 0.6, 0.6, 1)

	GVAR.OptionsFrame.ChatOutput:SetScript("OnClick", function(self)
		CharScanner_Options.chatoutput = not CharScanner_Options.chatoutput
		if CharScanner_Options.chatoutput then
			TEMPLATE.EnableCheckButton(GVAR.OptionsFrame.ChatOutputGrp)
		else
			CharScanner_Options.chatoutputGrp = false
			GVAR.OptionsFrame.ChatOutputGrp:SetChecked(false)
			TEMPLATE.DisableCheckButton(GVAR.OptionsFrame.ChatOutputGrp)
		end
	end)

	GVAR.OptionsFrame.ChatOutputGrp:SetScript("OnClick", function(self)
		CharScanner_Options.chatoutputGrp = not CharScanner_Options.chatoutputGrp
	end)
	-- - chatoutput END

	-- - minimap button START
	GVAR.OptionsFrame.MinimapText = GVAR.OptionsFrame.Base:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.OptionsFrame.MinimapText:SetPoint("LEFT", GVAR.OptionsFrame.Base, "LEFT", 10, 0)
	GVAR.OptionsFrame.MinimapText:SetPoint("TOP", GVAR.OptionsFrame.ChatOutputGrpInfo, "BOTTOM", 0, -20)
	GVAR.OptionsFrame.MinimapText:SetJustifyH("LEFT")
	GVAR.OptionsFrame.MinimapText:SetText(L["Minimap-Button"]..":")

	GVAR.OptionsFrame.Minimap = CreateFrame("CheckButton", nil, GVAR.OptionsFrame.Base)
	TEMPLATE.CheckButton(GVAR.OptionsFrame.Minimap, 16, 4, L["Minimap-Button"])
	GVAR.OptionsFrame.Minimap:SetPoint("LEFT", GVAR.OptionsFrame.Base, "LEFT", 10, 0)
	GVAR.OptionsFrame.Minimap:SetPoint("TOP", GVAR.OptionsFrame.MinimapText, "BOTTOM", 0, -10)
	GVAR.OptionsFrame.Minimap:SetChecked(CharScanner_Options.MinimapButton)
	TEMPLATE.EnableCheckButton(GVAR.OptionsFrame.Minimap)

	GVAR.OptionsFrame.Minimap:SetScript("OnClick", function(self)
		CharScanner_Options.MinimapButton = not CharScanner_Options.MinimapButton
		CharScanner:CreateMinimapButton()
	end)
	-- - minimap button END
	-- Options END
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function CharScanner:CreateScanButtonFrame()
	if CharScanner_ScanButtonFrame then return end

	-- scan button START
	GVAR.ScanButtonFrame = CreateFrame("Frame", "CharScanner_ScanButtonFrame", UIParent)
	TEMPLATE.BorderTRBL(GVAR.ScanButtonFrame)
	GVAR.ScanButtonFrame:EnableMouse(true)
	GVAR.ScanButtonFrame:SetMovable(true)
	GVAR.ScanButtonFrame:SetToplevel(true)
	GVAR.ScanButtonFrame:SetClampedToScreen(true)
	GVAR.ScanButtonFrame:Hide()
	GVAR.ScanButtonFrame:SetWidth(1)
	GVAR.ScanButtonFrame:SetHeight(1)

	GVAR.ScanButtonFrame.SingleScan = CreateFrame("Button", nil, GVAR.ScanButtonFrame)
	TEMPLATE.IconButton(GVAR.ScanButtonFrame.SingleScan, 7, CharScanner_Options.scanButtonSize)
	GVAR.ScanButtonFrame.SingleScan:SetPoint("TOP", GVAR.ScanButtonFrame, "TOP", 0, 0)
	GVAR.ScanButtonFrame.SingleScan:SetScript("OnClick", function()
		if CharScanner_Options.scanButton and CharScanner_Options.scanButtonLock then
			GVAR.MainFrame:Show()
			CharScanner:ScanUnit("target")
		end
	end)

	GVAR.ScanButtonFrame.GroupScanAll = CreateFrame("Button", nil, GVAR.ScanButtonFrame)
	TEMPLATE.IconButton(GVAR.ScanButtonFrame.GroupScanAll, 1, CharScanner_Options.scanButtonSize)
	GVAR.ScanButtonFrame.GroupScanAll:SetPoint("TOPLEFT", GVAR.ScanButtonFrame.SingleScan, "TOPRIGHT", sizeGeneral.Offset, 0)
	GVAR.ScanButtonFrame.GroupScanAll:SetScript("OnClick", function()
		if CharScanner_Options.scanButton and CharScanner_Options.scanButtonLock then
			GVAR.MainFrame:Show()
			CharScanner:ScanGroup(false)
		end
	end)
	if CharScanner_Options.scanButtonGrpAll then
		GVAR.ScanButtonFrame.GroupScanAll:Show()
	else
		GVAR.ScanButtonFrame.GroupScanAll:Hide()
	end

	GVAR.ScanButtonFrame.GroupScanMissing = CreateFrame("Button", nil, GVAR.ScanButtonFrame)
	TEMPLATE.IconButton(GVAR.ScanButtonFrame.GroupScanMissing, 5, CharScanner_Options.scanButtonSize)
	GVAR.ScanButtonFrame.GroupScanMissing:SetPoint("TOPLEFT", GVAR.ScanButtonFrame.GroupScanAll, "TOPRIGHT", sizeGeneral.Offset, 0)
	GVAR.ScanButtonFrame.GroupScanMissing:SetScript("OnClick", function()
		if CharScanner_Options.scanButton and CharScanner_Options.scanButtonLock then
			GVAR.MainFrame:Show()
			CharScanner:ScanGroup(true)
		end
	end)
	if CharScanner_Options.scanButtonGrpMiss then
		GVAR.ScanButtonFrame.GroupScanMissing:Show()
	else
		GVAR.ScanButtonFrame.GroupScanMissing:Hide()
	end

	if CharScanner_Options.scanButtonGrpAll then
		GVAR.ScanButtonFrame.GroupScanMissing:SetPoint("TOPLEFT", GVAR.ScanButtonFrame.GroupScanAll, "TOPRIGHT", sizeGeneral.Offset, 0)
	else
		GVAR.ScanButtonFrame.GroupScanMissing:SetPoint("TOPLEFT", GVAR.ScanButtonFrame.SingleScan, "TOPRIGHT", sizeGeneral.Offset, 0)
	end

		GVAR.ScanButtonFrame.GroupScanMissingNum = CreateFrame("Button", nil, GVAR.ScanButtonFrame)
		GVAR.ScanButtonFrame.GroupScanMissingNum:SetWidth(20)
		GVAR.ScanButtonFrame.GroupScanMissingNum:SetHeight(fonts.GameFontNormalSmall.height + sizeGeneral.Space)
		GVAR.ScanButtonFrame.GroupScanMissingNum:SetPoint("BOTTOM", GVAR.ScanButtonFrame.GroupScanMissing, "TOP", 0, 0)
		GVAR.ScanButtonFrame.GroupScanMissingNum:Hide()

		GVAR.ScanButtonFrame.GroupScanMissingNumTex = GVAR.ScanButtonFrame.GroupScanMissingNum:CreateTexture(nil, "BACKGROUND")
		GVAR.ScanButtonFrame.GroupScanMissingNumTex:SetPoint("TOPLEFT", GVAR.ScanButtonFrame.GroupScanMissingNum, "TOPLEFT", 0, 0)
		GVAR.ScanButtonFrame.GroupScanMissingNumTex:SetPoint("BOTTOMRIGHT", GVAR.ScanButtonFrame.GroupScanMissingNum, "BOTTOMRIGHT", 0, 0)
		GVAR.ScanButtonFrame.GroupScanMissingNumTex:SetTexture(0, 0, 0, 1)

		GVAR.ScanButtonFrame.GroupScanMissingNumText = GVAR.ScanButtonFrame.GroupScanMissingNum:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
		GVAR.ScanButtonFrame.GroupScanMissingNumText:SetPoint("CENTER", 0, 0)
		GVAR.ScanButtonFrame.GroupScanMissingNumText:SetJustifyH("CENTER")
		GVAR.ScanButtonFrame.GroupScanMissingNumText:SetTextColor(color.Red.rgba[1], color.Red.rgba[2], color.Red.rgba[3], color.Red.rgba[4])
	-- scan button END

	-- mover START
	GVAR.ScanButtonFrame.Mover = CreateFrame("Frame", nil, GVAR.ScanButtonFrame)
	GVAR.ScanButtonFrame.Mover:EnableMouse(true)
	GVAR.ScanButtonFrame.Mover:SetToplevel(true)
	GVAR.ScanButtonFrame.Mover:SetFrameStrata("LOW")
	GVAR.ScanButtonFrame.Mover:SetWidth(330)
	GVAR.ScanButtonFrame.Mover:SetHeight(80)
	GVAR.ScanButtonFrame.Mover:SetPoint("TOP", GVAR.ScanButtonFrame, "TOP", 0, 32)
	GVAR.ScanButtonFrame.Mover.BGtop = GVAR.ScanButtonFrame.Mover:CreateTexture(nil, "ARTWORK")
	GVAR.ScanButtonFrame.Mover.BGtop:SetPoint("TOPLEFT", GVAR.ScanButtonFrame.Mover, "TOPLEFT", 0, 0)
	GVAR.ScanButtonFrame.Mover.BGtop:SetPoint("BOTTOMRIGHT", GVAR.ScanButtonFrame.Mover, "TOPRIGHT", 0, -28)
	GVAR.ScanButtonFrame.Mover.BGtop:SetTexture(0, 0, 0, 0.8)
	GVAR.ScanButtonFrame.Mover.BGbottom = GVAR.ScanButtonFrame.Mover:CreateTexture(nil, "ARTWORK")
	GVAR.ScanButtonFrame.Mover.BGbottom:SetPoint("TOPLEFT", GVAR.ScanButtonFrame.Mover, "TOPLEFT", 0, -90)
	GVAR.ScanButtonFrame.Mover.BGbottom:SetPoint("BOTTOMRIGHT", 0, 0)
	GVAR.ScanButtonFrame.Mover.BGbottom:SetTexture(0, 0, 0, 0.8)
	GVAR.ScanButtonFrame.Mover:SetScript("OnEnter", function() GVAR.ScanButtonFrame.Mover.BGtop:SetTexture(0, 0, 0, 0.5) GVAR.ScanButtonFrame.Mover.BGbottom:SetTexture(0, 0, 0, 0.5) end)
	GVAR.ScanButtonFrame.Mover:SetScript("OnLeave", function() GVAR.ScanButtonFrame.Mover.BGtop:SetTexture(0, 0, 0, 0.8) GVAR.ScanButtonFrame.Mover.BGbottom:SetTexture(0, 0, 0, 0.8) end)
	GVAR.ScanButtonFrame.Mover:SetScript("OnMouseDown", MoveStart)
	GVAR.ScanButtonFrame.Mover:SetScript("OnMouseUp", MoveStop)

	GVAR.ScanButtonFrame.MoverText = GVAR.ScanButtonFrame.Mover:CreateFontString(nil, "ARTWORK", fonts.GameFontNormal.name)
	GVAR.ScanButtonFrame.MoverText:SetPoint("BOTTOM", GVAR.ScanButtonFrame.Mover, "TOP", 0, 4)
	GVAR.ScanButtonFrame.MoverText:SetJustifyH("CENTER")
	GVAR.ScanButtonFrame.MoverText:SetWidth(330)
	GVAR.ScanButtonFrame.MoverText:SetText(L["SCAN-Button"]..":")
	GVAR.ScanButtonFrame.MoverText:SetTextColor(1, 0.82, 0, 1)

	GVAR.ScanButtonFrame.Mover.Scale = CreateFrame("Slider", nil, GVAR.ScanButtonFrame.Mover)
	TEMPLATE.Slider(GVAR.ScanButtonFrame.Mover.Scale, 200, 2, 10, 48, CharScanner_Options.scanButtonSize,
	function(self, value)
		CharScanner_Options.scanButtonSize = value
		GVAR.ScanButtonFrame.SingleScan:SetWidth(sizeGeneral.Border + sizeGeneral.Space + (Textures.SingleScan.width/(Textures.SingleScan.height/value)) + sizeGeneral.Space + sizeGeneral.Border)
		GVAR.ScanButtonFrame.SingleScan:SetHeight(sizeGeneral.Border + sizeGeneral.Space + value + sizeGeneral.Space + sizeGeneral.Border)
		GVAR.ScanButtonFrame.GroupScanAll:SetWidth(sizeGeneral.Border + sizeGeneral.Space + (Textures.GroupScanAll.width/(Textures.GroupScanAll.height/value)) + sizeGeneral.Space + sizeGeneral.Border)
		GVAR.ScanButtonFrame.GroupScanAll:SetHeight(sizeGeneral.Border + sizeGeneral.Space + value + sizeGeneral.Space + sizeGeneral.Border)
		GVAR.ScanButtonFrame.GroupScanMissing:SetWidth(sizeGeneral.Border + sizeGeneral.Space + (Textures.GroupScanMissing.width/(Textures.GroupScanMissing.height/value)) + sizeGeneral.Space + sizeGeneral.Border)
		GVAR.ScanButtonFrame.GroupScanMissing:SetHeight(sizeGeneral.Border + sizeGeneral.Space + value + sizeGeneral.Space + sizeGeneral.Border)
	end,
	"px",
	1)
	GVAR.ScanButtonFrame.Mover.Scale:SetPoint("TOP", GVAR.ScanButtonFrame.Mover, "TOP", 0, -88)

	GVAR.ScanButtonFrame.MoverText1 = GVAR.ScanButtonFrame.Mover:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.ScanButtonFrame.MoverText1:SetPoint("TOP", GVAR.ScanButtonFrame.Mover.Scale, "BOTTOM", 0, -10)
	GVAR.ScanButtonFrame.MoverText1:SetJustifyH("CENTER")
	GVAR.ScanButtonFrame.MoverText1:SetWidth(330)
	GVAR.ScanButtonFrame.MoverText1:SetText("\n"..L["click & move"].."\n ")
	GVAR.ScanButtonFrame.MoverText1:SetTextColor(0.6, 0.6, 0.6, 1)
	GVAR.ScanButtonFrame.MoverText2 = GVAR.ScanButtonFrame.Mover:CreateFontString(nil, "ARTWORK", fonts.GameFontNormalSmall.name)
	GVAR.ScanButtonFrame.MoverText2:SetPoint("CENTER", GVAR.ScanButtonFrame.Mover.BGtop, "CENTER", 0, 0)
	GVAR.ScanButtonFrame.MoverText2:SetJustifyH("CENTER")
	GVAR.ScanButtonFrame.MoverText2:SetWidth(330)
	GVAR.ScanButtonFrame.MoverText2:SetText("\n"..L["click & move"].."\n ")
	GVAR.ScanButtonFrame.MoverText2:SetTextColor(0.6, 0.6, 0.6, 1)

	GVAR.ScanButtonFrame.Mover.Lock = CreateFrame("Button", nil, GVAR.ScanButtonFrame.Mover)
	TEMPLATE.TextButton(GVAR.ScanButtonFrame.Mover.Lock, L["Lock SCAN-Button"], 1)
	GVAR.ScanButtonFrame.Mover.Lock:SetWidth(175)
	GVAR.ScanButtonFrame.Mover.Lock:SetHeight(22)
	GVAR.ScanButtonFrame.Mover.Lock:SetPoint("TOP", GVAR.ScanButtonFrame.MoverText1, "BOTTOM", 0, -4)
	GVAR.ScanButtonFrame.Mover.Lock:SetScript("OnClick", function()
		CharScanner_Options.scanButtonLock = true
		GVAR.ScanButtonFrame.Mover:Hide()
		if UnitIsPlayer("target") then
			GVAR.ScanButtonFrame:Show()
			if inPartyOrRaid then
				if CharScanner_Options.scanButtonGrpAll then
					GVAR.ScanButtonFrame.GroupScanAll:Show()
				end
				if CharScanner_Options.scanButtonGrpMiss then
					GVAR.ScanButtonFrame.GroupScanMissing:Show()
				end
			else
				GVAR.ScanButtonFrame.GroupScanAll:Hide()
				GVAR.ScanButtonFrame.GroupScanMissing:Hide()
			end
		else
			GVAR.ScanButtonFrame:Hide()
		end
		GVAR.OptionsFrame.ScanButton.Lock:SetChecked(CharScanner_Options.scanButtonLock)
	end)

	GVAR.ScanButtonFrame.Mover.GUI = CreateFrame("Button", nil, GVAR.ScanButtonFrame.Mover)
	TEMPLATE.TextButton(GVAR.ScanButtonFrame.Mover.GUI, L["Open GUI"], 1)
	GVAR.ScanButtonFrame.Mover.GUI:SetWidth(150)
	GVAR.ScanButtonFrame.Mover.GUI:SetHeight(22)
	GVAR.ScanButtonFrame.Mover.GUI:SetPoint("TOP", GVAR.ScanButtonFrame.Mover.Lock, "BOTTOM", 0, -20)
	GVAR.ScanButtonFrame.Mover.GUI:SetPoint("LEFT", GVAR.ScanButtonFrame.Mover, "LEFT", 10, 0)
	GVAR.ScanButtonFrame.Mover.GUI:SetScript("OnClick", function()
		CharScanner:Frame_Toggle(GVAR.MainFrame)
	end)

	GVAR.ScanButtonFrame.Mover.OPTIONS = CreateFrame("Button", nil, GVAR.ScanButtonFrame.Mover)
	TEMPLATE.TextButton(GVAR.ScanButtonFrame.Mover.OPTIONS, L["Open Options"], 1)
	GVAR.ScanButtonFrame.Mover.OPTIONS:SetWidth(150)
	GVAR.ScanButtonFrame.Mover.OPTIONS:SetHeight(22)
	GVAR.ScanButtonFrame.Mover.OPTIONS:SetPoint("LEFT", GVAR.ScanButtonFrame.Mover.GUI, "RIGHT", 10, 0)
	GVAR.ScanButtonFrame.Mover.OPTIONS:SetScript("OnClick", function()
		CharScanner:Frame_Toggle(GVAR.OptionsFrame)
	end)

	GVAR.ScanButtonFrame.Mover:SetHeight( 28   + 56 +   4 + 16 + 30 + 4 + fonts.GameFontNormalSmall.height + 4 + 22 + 20 + 22 + 4 )
	-- mover END

	if CharScanner_Options.scanButtonLock then
		GVAR.ScanButtonFrame.Mover:Hide()
	end
	CharScanner:Frame_SetupPosition("CharScanner_ScanButtonFrame")
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function CharScanner:CreateMinimapButton()
	if not CharScanner_Options.MinimapButton then
		if CharScanner_MinimapButton then
			CharScanner_MinimapButton:Hide()
		end
		return
	else
		if CharScanner_MinimapButton then
			CharScanner_MinimapButton:Show()
			return
		end
	end

	if CharScanner_MinimapButton then return end

	local function MoveMinimapButton()
		local xpos
		local ypos
		local minimapShape = GetMinimapShape and GetMinimapShape() or "ROUND"
		if minimapShape == "SQUARE" then
			xpos = 110 * cos(CharScanner_Options.MinimapButtonPos or 0)
			ypos = 110 * sin(CharScanner_Options.MinimapButtonPos or 0)
			xpos = math.max(-82, math.min(xpos, 84))
			ypos = math.max(-86, math.min(ypos, 82))
		else
			xpos = 80 * cos(CharScanner_Options.MinimapButtonPos or 0)
			ypos = 80 * sin(CharScanner_Options.MinimapButtonPos or 0)
		end
		CharScanner_MinimapButton:SetPoint("TOPLEFT", Minimap, "TOPLEFT", 52-xpos, ypos-52)
	end

	local function DragMinimapButton()
		local xpos, ypos = GetCursorPosition()
		local xmin, ymin = Minimap:GetLeft() or 400, Minimap:GetBottom() or 400
		local scale = Minimap:GetEffectiveScale()
		xpos = xmin-xpos/scale+70
		ypos = ypos/scale-ymin-70
		CharScanner_Options.MinimapButtonPos = math.deg(math.atan2(ypos, xpos))
		MoveMinimapButton()
	end

	local MinimapButton = CreateFrame("Button", "CharScanner_MinimapButton", Minimap)
	MinimapButton:EnableMouse(true)
	MinimapButton:SetMovable(true)
	MinimapButton:SetToplevel(true)
	MinimapButton:SetWidth(Textures.MinimapButton.width)
	MinimapButton:SetHeight(Textures.MinimapButton.height)
	MinimapButton:SetPoint("TOPLEFT")
	MinimapButton:SetFrameStrata("LOW")
	MinimapButton:RegisterForClicks("AnyUp")
	MinimapButton:RegisterForDrag("LeftButton")

	MinimapButton.NormalTexture = MinimapButton:CreateTexture(nil, "ARTWORK")
	MinimapButton.NormalTexture:SetPoint("TOPLEFT", 0, 0)
	MinimapButton.NormalTexture:SetPoint("BOTTOMRIGHT", 0, 0)
	MinimapButton.NormalTexture:SetTexture(Textures.CharScannerIcons.path)
	MinimapButton.NormalTexture:SetTexCoord(Textures.MinimapButton.coords[1], Textures.MinimapButton.coords[2], Textures.MinimapButton.coords[3], Textures.MinimapButton.coords[4])
	MinimapButton:SetNormalTexture(MinimapButton.NormalTexture)

	MinimapButton.PushedTexture = MinimapButton:CreateTexture(nil, "ARTWORK")
	MinimapButton.PushedTexture:SetPoint("TOPLEFT", 1, -1)
	MinimapButton.PushedTexture:SetPoint("BOTTOMRIGHT", -1, 1)
	MinimapButton.PushedTexture:SetTexture(Textures.CharScannerIcons.path)
	MinimapButton.PushedTexture:SetTexCoord(Textures.MinimapButton.coords[1], Textures.MinimapButton.coords[2], Textures.MinimapButton.coords[3], Textures.MinimapButton.coords[4])
	MinimapButton:SetPushedTexture(MinimapButton.PushedTexture)

	MinimapButton.HighlightTexture = MinimapButton:CreateTexture(nil, "ARTWORK")
	MinimapButton.HighlightTexture:SetPoint("TOPLEFT", 0, 0)
	MinimapButton.HighlightTexture:SetPoint("BOTTOMRIGHT", 0, 0)
	MinimapButton.HighlightTexture:SetTexture("Interface\\Minimap\\UI-Minimap-ZoomButton-Highlight")
	MinimapButton:SetHighlightTexture(MinimapButton.HighlightTexture)

	MinimapButton:SetScript("OnEnter", function(self)
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
		GameTooltip:AddLine("CharScanner", color.Yellow.rgba[1], color.Yellow.rgba[2], color.Yellow.rgba[3], color.Yellow.rgba[4])
		GameTooltip:Show()
	end)
	MinimapButton:SetScript("OnLeave", function(self) GameTooltip:Hide() end)
	MinimapButton:SetScript("OnClick", function(self, button) CharScanner:MiniOptionsToggle(button, 2) end)
	MinimapButton:SetScript("OnDragStart", function(self) self:LockHighlight() self:SetScript("OnUpdate", DragMinimapButton) end)
	MinimapButton:SetScript("OnDragStop", function(self) self:SetScript("OnUpdate", nil) self:UnlockHighlight() end)

	MoveMinimapButton()
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function CharScanner:CreateMiniOptionsFrame()
	if GVAR.MiniOptionsFrame then return end

	local buttondata = {}
	buttondata[1] = "SingleScan"
	buttondata[2] = "GroupScanAll"
	buttondata[3] = "GroupScanMissing"
	buttondata[4] = "Options"

	local maxiconwidth = 0
	local maxtextwidth = 0

	local iw = 0
	for i = 1, #buttondata do
		iw = Textures[ buttondata[i] ].width
		if iw > maxiconwidth then
			maxiconwidth = iw
		end
	end
	
	local buttonheight = Textures.SingleScan.height
	local frameheight = sizeGeneral.Offset + (#buttondata*buttonheight) + sizeGeneral.Offset

	GVAR.MiniOptionsFrame = CreateFrame("Frame", nil, UIParent)
	TEMPLATE.BorderTRBL(GVAR.MiniOptionsFrame)
	GVAR.MiniOptionsFrame:EnableMouse(true)
	GVAR.MiniOptionsFrame:SetMovable(true)
	GVAR.MiniOptionsFrame:SetToplevel(true)
	GVAR.MiniOptionsFrame:SetClampedToScreen(true)
	GVAR.MiniOptionsFrame:SetFrameStrata("DIALOG")
	GVAR.MiniOptionsFrame:SetHeight(frameheight)
	GVAR.MiniOptionsFrame:Hide()

	for i = 1, #buttondata do
		-- GVAR.MiniOptionsFrame.SingleScan
		-- GVAR.MiniOptionsFrame.GroupScanAll
		-- GVAR.MiniOptionsFrame.GroupScanMissing
		-- GVAR.MiniOptionsFrame.Options

		GVAR.MiniOptionsFrame[ buttondata[i] ] = CreateFrame("Button", nil, GVAR.MiniOptionsFrame)
		GVAR.MiniOptionsFrame[ buttondata[i] ]:SetHeight(buttonheight)
		if i == 1 then
			GVAR.MiniOptionsFrame[ buttondata[i] ]:SetPoint("TOPLEFT", GVAR.MiniOptionsFrame, "TOPLEFT", sizeGeneral.Offset, -sizeGeneral.Offset)
		else
			GVAR.MiniOptionsFrame[ buttondata[i] ]:SetPoint("TOPLEFT", GVAR.MiniOptionsFrame[ buttondata[(i-1)] ], "BOTTOMLEFT", 0, 0)
		end

		GVAR.MiniOptionsFrame[ buttondata[i] ].Text = GVAR.MiniOptionsFrame[ buttondata[i] ]:CreateFontString(nil, "OVERLAY", fonts.GameFontNormalSmall.name)
		GVAR.MiniOptionsFrame[ buttondata[i] ].Text:SetHeight(fonts.GameFontNormalSmall.height)
		GVAR.MiniOptionsFrame[ buttondata[i] ].Text:SetPoint("LEFT", GVAR.MiniOptionsFrame[ buttondata[i] ], "LEFT", maxiconwidth+sizeGeneral.Space, 0)
		GVAR.MiniOptionsFrame[ buttondata[i] ].Text:SetJustifyH("LEFT")
		GVAR.MiniOptionsFrame[ buttondata[i] ].Text:SetText(L[ buttondata[i] ])
		local tw = GVAR.MiniOptionsFrame[ buttondata[i] ].Text:GetStringWidth()
		if tw > maxtextwidth then
			maxtextwidth = tw
		end
	end

	local framewidth = sizeGeneral.Offset + maxiconwidth + sizeGeneral.Offset + maxtextwidth + sizeGeneral.Offset
	GVAR.MiniOptionsFrame:SetWidth(framewidth)

	for i = 1, #buttondata do
		-- GVAR.MiniOptionsFrame.SingleScan
		-- GVAR.MiniOptionsFrame.GroupScanAll
		-- GVAR.MiniOptionsFrame.GroupScanMissing
		-- GVAR.MiniOptionsFrame.Options

		GVAR.MiniOptionsFrame[ buttondata[i] ]:SetWidth(framewidth-(2*sizeGeneral.Offset))
		
		local bottomright = (framewidth-(2*sizeGeneral.Offset)) - Textures[ buttondata[i] ].width

		GVAR.MiniOptionsFrame[ buttondata[i] ].Normal = GVAR.MiniOptionsFrame[ buttondata[i] ]:CreateTexture(nil, "ARTWORK")
		GVAR.MiniOptionsFrame[ buttondata[i] ].Normal:SetPoint("TOPLEFT", 2, -2)
		GVAR.MiniOptionsFrame[ buttondata[i] ].Normal:SetPoint("BOTTOMRIGHT", -(bottomright + 2), 2)
		GVAR.MiniOptionsFrame[ buttondata[i] ].Normal:SetTexture(Textures.CharScannerIcons.path)
		GVAR.MiniOptionsFrame[ buttondata[i] ].Normal:SetTexCoord(Textures[ buttondata[i] ].coords[1], Textures[ buttondata[i] ].coords[2], Textures[ buttondata[i] ].coords[3], Textures[ buttondata[i] ].coords[4])
		GVAR.MiniOptionsFrame[ buttondata[i] ]:SetNormalTexture(GVAR.MiniOptionsFrame[ buttondata[i] ].Normal)

		GVAR.MiniOptionsFrame[ buttondata[i] ].Push = GVAR.MiniOptionsFrame[ buttondata[i] ]:CreateTexture(nil, "ARTWORK")
		GVAR.MiniOptionsFrame[ buttondata[i] ].Push:SetPoint("TOPLEFT", 3, -3)
		GVAR.MiniOptionsFrame[ buttondata[i] ].Push:SetPoint("BOTTOMRIGHT", -(bottomright + 3), 3)
		GVAR.MiniOptionsFrame[ buttondata[i] ].Push:SetTexture(Textures.CharScannerIcons.path)
		GVAR.MiniOptionsFrame[ buttondata[i] ].Push:SetTexCoord(Textures[ buttondata[i] ].coords[1], Textures[ buttondata[i] ].coords[2], Textures[ buttondata[i] ].coords[3], Textures[ buttondata[i] ].coords[4])
		GVAR.MiniOptionsFrame[ buttondata[i] ]:SetPushedTexture(GVAR.MiniOptionsFrame[ buttondata[i] ].Push)

		GVAR.MiniOptionsFrame[ buttondata[i] ].Disabled = GVAR.MiniOptionsFrame[ buttondata[i] ]:CreateTexture(nil, "ARTWORK")
		GVAR.MiniOptionsFrame[ buttondata[i] ].Disabled:SetPoint("TOPLEFT", 2, -2)
		GVAR.MiniOptionsFrame[ buttondata[i] ].Disabled:SetPoint("BOTTOMRIGHT", -(bottomright + 2), 2)
		GVAR.MiniOptionsFrame[ buttondata[i] ].Disabled:SetTexture(Textures.CharScannerIcons.path)
		GVAR.MiniOptionsFrame[ buttondata[i] ].Disabled:SetTexCoord(Textures[ buttondata[i] ].coords[1], Textures[ buttondata[i] ].coords[2], Textures[ buttondata[i] ].coords[3], Textures[ buttondata[i] ].coords[4])
		GVAR.MiniOptionsFrame[ buttondata[i] ]:SetDisabledTexture(GVAR.MiniOptionsFrame[ buttondata[i] ].Disabled)

		Desaturation(GVAR.MiniOptionsFrame[ buttondata[i] ].Disabled, true)

		GVAR.MiniOptionsFrame[ buttondata[i] ].Highlight = GVAR.MiniOptionsFrame[ buttondata[i] ]:CreateTexture(nil, "ARTWORK")
		GVAR.MiniOptionsFrame[ buttondata[i] ].Highlight:SetPoint("TOPLEFT", 0, 0)
		GVAR.MiniOptionsFrame[ buttondata[i] ].Highlight:SetPoint("BOTTOMRIGHT", 0, 0)
		GVAR.MiniOptionsFrame[ buttondata[i] ].Highlight:SetTexture(0.2, 0.2, 0.2, 1)
		GVAR.MiniOptionsFrame[ buttondata[i] ]:SetHighlightTexture(GVAR.MiniOptionsFrame[ buttondata[i] ].Highlight)

		if (buttondata[i] == "GroupScanAll" or buttondata[i] == "GroupScanMissing") and (not inPartyOrRaid or GroupScanIsActive) then
			GVAR.MiniOptionsFrame[ buttondata[i] ]:Disable()
			GVAR.MiniOptionsFrame[ buttondata[i] ].Text:SetTextColor(color.Grey.rgba[1], color.Grey.rgba[2], color.Grey.rgba[3], color.Grey.rgba[4])
		end
		if buttondata[i] == "SingleScan" and SingleScanIsActive then
			GVAR.MiniOptionsFrame[ buttondata[i] ]:Disable()
			GVAR.MiniOptionsFrame[ buttondata[i] ].Text:SetTextColor(color.Grey.rgba[1], color.Grey.rgba[2], color.Grey.rgba[3], color.Grey.rgba[4])
		end

		if i == 1 then -- SingleScan
			GVAR.MiniOptionsFrame[ buttondata[i] ]:SetScript("OnClick", function()
				GVAR.MiniOptionsFrame:SetScript("OnUpdate", nil)
				GVAR.MainFrame:Show()
				CharScanner:ScanUnit("target")
				GVAR.MiniOptionsFrame:Hide()
			end)
		elseif i == 2 then -- GroupScanAll
			GVAR.MiniOptionsFrame[ buttondata[i] ]:SetScript("OnClick", function()
				GVAR.MiniOptionsFrame:SetScript("OnUpdate", nil)
				GVAR.MainFrame:Show()
				CharScanner:ScanGroup(false)
				GVAR.MiniOptionsFrame:Hide()
			end)
		elseif i == 3 then -- GroupScanMissing
			GVAR.MiniOptionsFrame[ buttondata[i] ]:SetScript("OnClick", function()
				GVAR.MiniOptionsFrame:SetScript("OnUpdate", nil)
				GVAR.MainFrame:Show()
				CharScanner:ScanGroup(true)
				GVAR.MiniOptionsFrame:Hide()
			end)
		elseif i == 4 then -- Options
			GVAR.MiniOptionsFrame[ buttondata[i] ]:SetScript("OnClick", function()
				GVAR.MiniOptionsFrame:SetScript("OnUpdate", nil)
				CharScanner:Frame_Toggle(GVAR.OptionsFrame, 1)
				GVAR.MiniOptionsFrame:Hide()
			end)
		end

		GVAR.MiniOptionsFrame[ buttondata[i] ]:SetScript("OnEnter", function() MiniOptions_TimerStart = GetTime() end)
		GVAR.MiniOptionsFrame[ buttondata[i] ]:SetScript("OnLeave", function() MiniOptions_TimerStart = GetTime() end)
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function CharScanner:NodePvE_Click(mode, lvl, name1, index1, name2, index2, name3, index3)
	--print("lvl:", lvl, "index1:", index1, "index2:", index2, "index3:", index3) -- MENUDEV
	local baseName = GVAR.MainFrame.StatsPvE.ScrollFrame
	if lvl == 1 then
	-- level 1 START
		if baseName["L2"..name1]:IsVisible() then
			baseName["L1"..name1].Collapse:SetTexCoord(unpack(Textures.PlusMinus.coordsPlus))

			NodeDisplay(mode, "L2"..name1, false)

			if index1 == 1 then
				baseName["L1"..PVEDUMP[2].name]:SetPoint("TOPLEFT", baseName["L1"..PVEDUMP[1].name], "BOTTOMLEFT", 0, 0)
			elseif index1 > 1 and index1 < #PVEDUMP then
				baseName["L1"..PVEDUMP[index1+1].name]:SetPoint("TOPLEFT", baseName["L1"..PVEDUMP[index1].name], "BOTTOMLEFT", 0, 0)
			end
		else
			baseName["L1"..name1].Collapse:SetTexCoord(unpack(Textures.PlusMinus.coordsMinus))
			
			NodeDisplay(mode, "L2"..name1, true)

			if index1 == 1 then
				baseName["L1"..PVEDUMP[2].name]:SetPoint("TOPLEFT", baseName["L2"..name1], "BOTTOMLEFT", 0, 0)
			elseif index1 > 1 and index1 < #PVEDUMP then
				baseName["L1"..PVEDUMP[index1+1].name]:SetPoint("TOPLEFT", baseName["L2"..name1], "BOTTOMLEFT", 0, 0)
			end
		end
	-- level 1 END
	elseif lvl == 2 then
	-- level 2 START
		if baseName["L3"..name1..name2]:IsVisible() then
			baseName["L2"..name1..name2].Collapse:SetTexCoord(unpack(Textures.PlusMinus.coordsPlus))

			NodeDisplay(mode, "L3"..name1..name2, false)

			if index2 == 1 then
				baseName["L2"..PVEDUMP[index1].name..PVEDUMP[index1][2].name]:SetPoint("TOPLEFT", baseName["L2"..PVEDUMP[index1].name..PVEDUMP[index1][1].name], "BOTTOMLEFT", 0, 0)
			elseif index2 > 1 and index2 < #PVEDUMP[index1] then
				baseName["L2"..PVEDUMP[index1].name..PVEDUMP[index1][index2+1].name]:SetPoint("TOPLEFT", baseName["L2"..PVEDUMP[index1].name..PVEDUMP[index1][index2].name], "BOTTOMLEFT", 0, 0)
			end
		else
			baseName["L2"..name1..name2].Collapse:SetTexCoord(unpack(Textures.PlusMinus.coordsMinus))

			NodeDisplay(mode, "L3"..name1..name2, true)

			if index2 == 1 then
				baseName["L2"..PVEDUMP[index1].name..PVEDUMP[index1][2].name]:SetPoint("TOPLEFT", baseName["L3"..PVEDUMP[index1].name..name2], "BOTTOMLEFT", 0, 0)
			elseif index2 > 1 and index2 < #PVEDUMP[index1] then
				baseName["L2"..PVEDUMP[index1].name..PVEDUMP[index1][index2+1].name]:SetPoint("TOPLEFT", baseName[ "L3"..PVEDUMP[index1].name..name2..#PVEDUMP[index1][index2] ], "BOTTOMLEFT", 0, 0)
				local c1 = #PVEDUMP[index1][index2]
				local c2 = #PVEDUMP[index1][index2][c1].statistic
				if baseName["L4"..PVEDUMP[index1].name..PVEDUMP[index1][index2].name..PVEDUMP[index1][index2][c1].name..c2]:IsVisible() then
					baseName["L2"..PVEDUMP[index1].name..PVEDUMP[index1][index2+1].name]:SetPoint("TOPLEFT", baseName["L4"..PVEDUMP[index1].name..PVEDUMP[index1][index2].name..PVEDUMP[index1][index2][c1].name..c2], "BOTTOMLEFT", 0, 0)
				end
			end
		end
	-- level 2 END
	elseif lvl == 3 then
	-- level 3 START
		if baseName["L4"..name1..name2..name3]:IsVisible() then
			baseName["L3"..name1..name2..index3].Collapse:SetTexCoord(unpack(Textures.PlusMinus.coordsPlus))

			NodeDisplay(mode, "L4"..name1..name2..name3, false)

			if index3 == 1 then
				baseName["L3"..PVEDUMP[index1].name..PVEDUMP[index1][index2].name.."2"]:SetPoint("TOPLEFT", baseName["L3"..PVEDUMP[index1].name..PVEDUMP[index1][index2].name.."1"], "BOTTOMLEFT", 0, 0)
			elseif index3 > 1 and index3 < #PVEDUMP[index1][index2] then
				baseName["L3"..PVEDUMP[index1].name..PVEDUMP[index1][index2].name..(index3+1)]:SetPoint("TOPLEFT", baseName["L3"..PVEDUMP[index1].name..PVEDUMP[index1][index2].name..index3], "BOTTOMLEFT", 0, 0)
			elseif index3 == #PVEDUMP[index1][index2] and index2 < #PVEDUMP[index1] then
				baseName["L2"..PVEDUMP[index1].name..PVEDUMP[index1][index2+1].name]:SetPoint("TOPLEFT", baseName[ "L3"..PVEDUMP[index1].name..PVEDUMP[index1][index2].name..#PVEDUMP[index1][index2] ], "BOTTOMLEFT", 0, 0)
			end
		else
			baseName["L3"..name1..name2..index3].Collapse:SetTexCoord(unpack(Textures.PlusMinus.coordsMinus))

			NodeDisplay(mode, "L4"..name1..name2..name3, true)

			if index3 == 1 then
				baseName["L3"..PVEDUMP[index1].name..PVEDUMP[index1][index2].name.."2"]:SetPoint("TOPLEFT", baseName["L4"..PVEDUMP[index1].name..PVEDUMP[index1][index2].name..PVEDUMP[index1][index2][1].name], "BOTTOMLEFT", 0, 0)
			elseif index3 > 1 and index3 < #PVEDUMP[index1][index2] then
				baseName["L3"..PVEDUMP[index1].name..PVEDUMP[index1][index2].name..(index3+1)]:SetPoint("TOPLEFT", baseName["L4"..PVEDUMP[index1].name..PVEDUMP[index1][index2].name..PVEDUMP[index1][index2][index3].name..#PVEDUMP[index1][index2][index3].statistic], "BOTTOMLEFT", 0, 0)
			elseif index3 == #PVEDUMP[index1][index2] and index2 < #PVEDUMP[index1] then
				baseName["L2"..PVEDUMP[index1].name..PVEDUMP[index1][index2+1].name]:SetPoint("TOPLEFT", baseName["L4"..PVEDUMP[index1].name..PVEDUMP[index1][index2].name..PVEDUMP[index1][index2][index3].name..#PVEDUMP[index1][index2][index3].statistic], "BOTTOMLEFT", 0, 0)
			end
		end
	-- level 3 END
	end

	if index1 == 3 and PVEDUMP[index1+1] and PVEDUMP[index1+1].name then -- MENUDEV
		local c1 = #PVEDUMP[index1]
		local c2 = #PVEDUMP[index1][c1]
		local c3 = #PVEDUMP[index1][c1][c2].statistic
		if baseName["L4"..PVEDUMP[index1].name..PVEDUMP[index1][c1].name..PVEDUMP[index1][c1][c2].name..c3]:IsVisible() then
			--print("MenuDev lvl4", "L4"..PVEDUMP[index1].name..PVEDUMP[index1][c1].name..PVEDUMP[index1][c1][c2].name..c3) -- MENUDEV
			baseName["L1"..PVEDUMP[index1+1].name]:SetPoint("TOPLEFT", baseName["L4"..PVEDUMP[index1].name..PVEDUMP[index1][c1].name..PVEDUMP[index1][c1][c2].name..c3], "BOTTOMLEFT", 0, 0)
		elseif baseName["L3"..PVEDUMP[index1].name..PVEDUMP[index1][c1].name]:IsVisible() then
			--print("MenuDev lvl3", "L3"..PVEDUMP[index1].name..PVEDUMP[index1][c1].name) -- MENUDEV
			baseName["L1"..PVEDUMP[index1+1].name]:SetPoint("TOPLEFT", baseName["L3"..PVEDUMP[index1].name..PVEDUMP[index1][c1].name..c2], "BOTTOMLEFT", 0, 0)
		elseif baseName["L2"..PVEDUMP[index1].name..PVEDUMP[index1][c1].name]:IsVisible() then
			--print("MenuDev lvl2", "L2"..PVEDUMP[index1].name..PVEDUMP[index1][c1].name) -- MENUDEV
			baseName["L1"..PVEDUMP[index1+1].name]:SetPoint("TOPLEFT", baseName["L2"..PVEDUMP[index1].name..PVEDUMP[index1][c1].name], "BOTTOMLEFT", 0, 0)
		else
			--print("MenuDev lvl1", "L1"..PVEDUMP[index1].name) -- MENUDEV
			baseName["L1"..PVEDUMP[index1+1].name]:SetPoint("TOPLEFT", baseName["L1"..PVEDUMP[index1].name], "BOTTOMLEFT", 0, 0)
		end
	end -- MENUDEV

end

function CharScanner:NodePvP_Click(mode, lvl, name1, index1, name2, index2, name3, index3, init)
	local baseName = GVAR.MainFrame.StatsPvP.ScrollFrame
	if lvl == 1 then
	-- level 1 START
		local childName      = "L2"..name1
		local childButton    = baseName["L2"..name1]
		local collapseButton = baseName["L1"..name1].Collapse

		if childButton:IsVisible() then
			collapseButton:SetTexCoord(unpack(Textures.PlusMinus.coordsPlus))

			NodeDisplay(mode, childName, false)

			if index1 == 1 then
				if PVPDUMP[2] and #PVPDUMP[2].name then
					baseName["L1"..PVPDUMP[2].name]:SetPoint("TOPLEFT", baseName["L1"..PVPDUMP[1].name], "BOTTOMLEFT", 0, 0)
				end
			elseif index1 > 1 and index1 < #PVPDUMP then
				baseName["L1"..PVPDUMP[index1+1].name]:SetPoint("TOPLEFT", baseName["L1"..PVPDUMP[index1].name], "BOTTOMLEFT", 0, 0)
			end
		else
			collapseButton:SetTexCoord(unpack(Textures.PlusMinus.coordsMinus))
			
			NodeDisplay(mode, childName, true)

			if index1 == 1 then
				if PVPDUMP[2] and #PVPDUMP[2].name then
					baseName["L1"..PVPDUMP[2].name]:SetPoint("TOPLEFT", baseName["L2"..name1], "BOTTOMLEFT", 0, 0)
				end
			elseif index1 > 1 and index1 < #PVPDUMP then
				baseName["L1"..PVPDUMP[index1+1].name]:SetPoint("TOPLEFT", baseName["L2"..name1], "BOTTOMLEFT", 0, 0)
			end
		end

		if PVPDUMP[index1].statistic then return end
		if init then return end

		local c1 = #PVPDUMP[index1]
		local c2 = PVPDUMP[index1][c1].statisticTotal
		local AnchorIfLastChildCollapsed = baseName["L3"..PVPDUMP[index1].name..PVPDUMP[index1][c1].name..c2]
		local AnchorIfLastCollapsed      = baseName["L2"..PVPDUMP[index1].name..PVPDUMP[index1][c1].name]
		local AnchorIfLastNotCollapsed   = baseName["L1"..PVPDUMP[index1].name]

		if PVPDUMP[index1+1] then
			if AnchorIfLastChildCollapsed and AnchorIfLastChildCollapsed:IsVisible() then
				local toAnchor = baseName["L1"..PVPDUMP[index1+1].name]
				if toAnchor then
					toAnchor:SetPoint("TOPLEFT", AnchorIfLastChildCollapsed, "BOTTOMLEFT", 0, 0)
				end
				return
			elseif AnchorIfLastCollapsed:IsVisible() then
				local toAnchor = baseName["L1"..PVPDUMP[index1+1].name]
				if toAnchor then
					toAnchor:SetPoint("TOPLEFT", AnchorIfLastCollapsed, "BOTTOMLEFT", 0, 0)
				end
				return
			else
				local toAnchor = baseName["L1"..PVPDUMP[index1+1].name]
				if toAnchor then
					toAnchor:SetPoint("TOPLEFT", AnchorIfLastNotCollapsed, "BOTTOMLEFT", 0, 0)
				end
				return
			end
		end

	-- level 1 END
	elseif lvl == 2 then
	-- level 2 START
		local childName      = "L3"..name1..name2
		local childButton    = baseName["L3"..name1..name2]
		local collapseButton = baseName["L2"..name1..name2].Collapse

		if childButton:IsVisible() then
			collapseButton:SetTexCoord(unpack(Textures.PlusMinus.coordsPlus))

			NodeDisplay(mode, childName, false)

			if index2 == 1 then
				if PVPDUMP[index1][2] and PVPDUMP[index1][2].statisticTotal > 1 then
					baseName["L2"..PVPDUMP[index1].name..PVPDUMP[index1][2].name]:SetPoint("TOPLEFT", baseName["L2"..PVPDUMP[index1].name..PVPDUMP[index1][1].name], "BOTTOMLEFT", 0, 0)
				end
			elseif index2 > 1 and index2 < #PVPDUMP[index1] then
				baseName["L2"..PVPDUMP[index1].name..PVPDUMP[index1][index2+1].name]:SetPoint("TOPLEFT", baseName["L2"..PVPDUMP[index1].name..PVPDUMP[index1][index2].name], "BOTTOMLEFT", 0, 0)
			end
		else
			collapseButton:SetTexCoord(unpack(Textures.PlusMinus.coordsMinus))

			NodeDisplay(mode, childName, true)

			if index2 == 1 then
				if PVPDUMP[index1][2] and PVPDUMP[index1][2].statisticTotal > 1 then
					baseName["L2"..PVPDUMP[index1].name..PVPDUMP[index1][2].name]:SetPoint("TOPLEFT", baseName["L3"..PVPDUMP[index1].name..name2..PVPDUMP[index1][index2].statisticTotal], "BOTTOMLEFT", 0, 0)
				end
			elseif index2 > 1 and index2 < #PVPDUMP[index1] then
				baseName["L2"..PVPDUMP[index1].name..PVPDUMP[index1][index2+1].name]:SetPoint("TOPLEFT", baseName["L3"..PVPDUMP[index1].name..name2..PVPDUMP[index1][index2].statisticTotal], "BOTTOMLEFT", 0, 0)
			end
		end

		local c1 = #PVPDUMP[index1]
		local c2 = PVPDUMP[index1][c1].statisticTotal
		local AnchorIfLastCollapsed    = baseName["L3"..PVPDUMP[index1].name..PVPDUMP[index1][c1].name..c2]
		local AnchorIfLastNotCollapsed = baseName["L2"..PVPDUMP[index1].name..PVPDUMP[index1][c1].name]

		if PVPDUMP[index1][c1] and PVPDUMP[index1][c1].statisticTotal > 1 then
			if AnchorIfLastCollapsed:IsVisible() then
				local toAnchor = baseName["L1"..PVPDUMP[index1+1].name]
				if toAnchor then
					toAnchor:SetPoint("TOPLEFT", AnchorIfLastCollapsed, "BOTTOMLEFT", 0, 0)
				end
			else
				local toAnchor = baseName["L1"..PVPDUMP[index1+1].name]
				if toAnchor then
					toAnchor:SetPoint("TOPLEFT", AnchorIfLastNotCollapsed, "BOTTOMLEFT", 0, 0)
				end
			end
		end
	-- level 2 END
	end
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function CharScanner:PullDdownMenu_OnVerticalScroll(pulldownmenu, value)
	pulldownmenu.ScrollFrame.ScrollBar:SetValue(value)
	pulldownmenu.ScrollFrame.offset = math_floor((value / sizeGeneral.BarHeight) + 0.5)
	CharScanner:PullDdownMenu_FillList(pulldownmenu)
end

function CharScanner:PullDdownMenu_FillList(pulldownmenu)
	local which = pulldownmenu.content

	-- frameName dependent START ----------
	local maxNum = 0
	local countNum = 0
	local numToDisplay = 0
	if which == "ACI" then
		maxNum = #PullDownMenu_ACI
		if maxNum > 10 then
			maxNum = 10
		end
		countNum = #PullDownMenu_ACI
		numToDisplay = maxNum
	elseif which == "PvEdump" then
		maxNum = #PullDownMenu_PvEdump
		if maxNum > 25 then
			maxNum = 25
		end
		countNum = #PullDownMenu_PvEdump
		numToDisplay = maxNum
	end
	-- frameName dependent END ----------

	local scrollFrame = pulldownmenu.ScrollFrame
	local buttonName = pulldownmenu.ScrollFrame.Button
	local scrollFrameHeight = 0
	--local numToDisplay = maxNum
	local valueStep = sizeGeneral.BarHeight

	if countNum < numToDisplay then
		numToDisplay = countNum
	end

	if countNum > 0 then
		scrollFrameHeight = (countNum - numToDisplay) * valueStep
		if scrollFrameHeight < 0 then
			scrollFrameHeight = 0
		end
	end
	scrollFrame.ScrollBar:SetMinMaxValues(0, scrollFrameHeight)
	scrollFrame.ScrollBar:SetValueStep(valueStep)

	if countNum <= numToDisplay then
		scrollFrame.ScrollBar:Hide()
	else
		scrollFrame.ScrollBar:Show()
	end

	if scrollFrame.ScrollBar:GetValue() == 0 then
		scrollFrame.ScrollBarScrollUpButton:Disable()
	else
		scrollFrame.ScrollBarScrollUpButton:Enable()
	end
	if ((scrollFrame.ScrollBar:GetValue() - scrollFrameHeight) == 0) then
		scrollFrame.ScrollBarScrollDownButton:Disable()
	else
		scrollFrame.ScrollBarScrollDownButton:Enable()
	end

	for i=1, maxNum do
		buttonName[i]:Hide()
	end
	if countNum == 0 then return end

	local j = 1
	local sliderPos = scrollFrame.offset
	for i=sliderPos+1, sliderPos+numToDisplay do
		if sliderPos < countNum then
			-- frameName dependent START ----------
			if which == "ACI" then
				buttonName[j].Text:SetText(PullDownMenu_ACI[i])
				buttonName[j].value1 = i
			elseif which == "PvEdump" then
				buttonName[j].Text:SetText(PullDownMenu_PvEdump[i].name)
				if PullDownMenu_PvEdump[i].lvl == 1 then
					buttonName[j].Text:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])
					buttonName[j].Text:SetPoint("LEFT", sizeGeneral.Space, 0)
				elseif PullDownMenu_PvEdump[i].lvl == 2 then
					buttonName[j].Text:SetTextColor(color.Level2.rgba[1], color.Level2.rgba[2], color.Level2.rgba[3], color.Level2.rgba[4])
					buttonName[j].Text:SetPoint("LEFT", sizeStatFrame.LevelModifier, 0)
				else
					buttonName[j].Text:SetTextColor(color.Level3.rgba[1], color.Level3.rgba[2], color.Level3.rgba[3], color.Level3.rgba[4])
					buttonName[j].Text:SetPoint("LEFT", sizeStatFrame.LevelModifier * 2, 0)
				end
				buttonName[j].value1 = PullDownMenu_PvEdump[i].v1
				buttonName[j].value2 = PullDownMenu_PvEdump[i].v2
				buttonName[j].value3 = PullDownMenu_PvEdump[i].v3
			end
			-- frameName dependent END ----------
			buttonName[j]:Show()
			j = j + 1
		end
	end
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function CharScanner:ABKsearch_Update(guid)
	CharScanner:ABKsearch_GUICreate(guid)
	CharScanner:ABKsearch_FillList()
end

function CharScanner:ABKsearch_GUICreate(guid)
	table_wipe(GUI_ABKsearch_data)
	GUI_ABKsearch_Count = 0

	if not guid then
		guid = GUIselectedGUID
	end

	if BossSearchQuery then

		GVAR.MainFrame.StatsPvE.ScrollFrame:Hide()
		GVAR.MainFrame.StatsPvE.SearchScrollFrame:Show()
		GVAR.MainFrame.StatsPvE.SearchSort:Show()
		GVAR.MainFrame.StatsPvE.Title.Achievement:Hide()
		GVAR.MainFrame.StatsPvE.Title.Statistic:Hide()
		GVAR.MainFrame.StatsPvE.Title.BossesTotal:Hide()

		GVAR.MainFrame.Summary1PvE.Achievement:Hide()
		GVAR.MainFrame.Summary2PvE.Achievement:Hide()
		GVAR.MainFrame.Summary1PvE.Statistic:Hide()
		GVAR.MainFrame.Summary2PvE.Statistic:Hide()
		--GVAR.MainFrame.Summary1PvE.BossesTotal:Hide()

		local lowerSearchQuery = string_lower(BossSearchQuery)
		local x = 1
		local totalCount = 0

		for i = 1, #PVEDUMP do
			if PVEDUMP[i].statistic then
				for j = 1, #PVEDUMP[i].statistic do
					if string_find(string_lower(StatisticIDTable[ PVEDUMP[i].statistic[j] ]), lowerSearchQuery, 1, true) then
						GUI_ABKsearch_data[x] = {}
						GUI_ABKsearch_data[x].Num = x
						GUI_ABKsearch_data[x].Name = StatisticIDTable[ PVEDUMP[i].statistic[j] ]
						GUI_ABKsearch_data[x].BossesTotal = SCAN[guid].PVEDATA[i].statisticBossEach[j]
						totalCount = totalCount + SCAN[guid].PVEDATA[i].statisticBossEach[j]
						x = x + 1
					end
				end
			else
				for j = 1, #PVEDUMP[i] do
					if PVEDUMP[i][j].statistic then
						for k = 1, #PVEDUMP[i][j].statistic do
							if string_find(string_lower(StatisticIDTable[ PVEDUMP[i][j].statistic[k] ]), lowerSearchQuery, 1, true) then
								GUI_ABKsearch_data[x] = {}
								GUI_ABKsearch_data[x].Num = x
								GUI_ABKsearch_data[x].Name = StatisticIDTable[ PVEDUMP[i][j].statistic[k] ]
								GUI_ABKsearch_data[x].BossesTotal = SCAN[guid].PVEDATA[i][j].statisticBossEach[k]
								totalCount = totalCount + SCAN[guid].PVEDATA[i][j].statisticBossEach[k]
								x = x + 1
							end
						end
					else
						for k = 1, #PVEDUMP[i][j] do
							if PVEDUMP[i][j][k].statistic then
								for l = 1, #PVEDUMP[i][j][k].statistic do
									if string_find(string_lower(StatisticIDTable[ PVEDUMP[i][j][k].statistic[l] ]), lowerSearchQuery, 1, true) then
										GUI_ABKsearch_data[x] = {}
										GUI_ABKsearch_data[x].Num = x
										GUI_ABKsearch_data[x].Name = StatisticIDTable[ PVEDUMP[i][j][k].statistic[l] ]
										GUI_ABKsearch_data[x].BossesTotal = SCAN[guid].PVEDATA[i][j][k].statisticBossEach[l]
										totalCount = totalCount + SCAN[guid].PVEDATA[i][j][k].statisticBossEach[l]
										x = x + 1
									end
								end
							end
						end
					end
				end
			end
		end
		GUI_ABKsearch_Count = #GUI_ABKsearch_data

		local sortfunc
		if GUI_ABKsearch_SortBy == 1 then -- 1 = standard
			if GUI_ABKsearch_SortOrder == 1 then
				sortfunc = function(a, b)
					if a.Num < b.Num then
						return true
					end
				end
			else
				sortfunc = function(a, b)
					if a.Num > b.Num then
						return true
					end
				end
			end
		elseif GUI_ABKsearch_SortBy == 2 then -- 2 = name
			if GUI_ABKsearch_SortOrder == 1 then
				sortfunc = function(a, b)
					if a.Name < b.Name then
						return true
					end
				end
			else
				sortfunc = function(a, b)
					if a.Name > b.Name then
						return true
					end
				end
			end
		elseif GUI_ABKsearch_SortBy == 3 then -- 3 = kills
			if GUI_ABKsearch_SortOrder == 1 then
				sortfunc = function(a, b)
					if a.BossesTotal > b.BossesTotal then
						return true
					end
				end
			else
				sortfunc = function(a, b)
					if a.BossesTotal < b.BossesTotal then
						return true
					end
				end
			end
		end
		table_sort(GUI_ABKsearch_data, sortfunc)

		GVAR.MainFrame.Summary1PvE.BossesTotal:SetText(totalCount)
	else
		GVAR.MainFrame.StatsPvE.ScrollFrame:Show()
		GVAR.MainFrame.StatsPvE.SearchScrollFrame:Hide()
		GVAR.MainFrame.StatsPvE.SearchSort:Hide()
		GVAR.MainFrame.StatsPvE.Title.Achievement:Show()
		GVAR.MainFrame.StatsPvE.Title.Statistic:Show()
		GVAR.MainFrame.StatsPvE.Title.BossesTotal:Show()

		GVAR.MainFrame.Summary1PvE.Achievement:Show()
		GVAR.MainFrame.Summary2PvE.Achievement:Show()
		GVAR.MainFrame.Summary1PvE.Statistic:Show()
		GVAR.MainFrame.Summary2PvE.Statistic:Show()
		--GVAR.MainFrame.Summary1PvE.BossesTotal:Show()

		if guid and SCAN[guid] then
			SCAN[guid].DataStep = 0
			CharScanner:ShowData(4, GUIselectedGUID, false)
			SCAN[guid].DataStep = 0
		end
	end
end

function CharScanner:ABKsearch_GUISort(sortBy)
	if GUI_ABKsearch_SortBy == sortBy then
		if GUI_ABKsearch_SortOrder == 1 then
			GUI_ABKsearch_SortOrder = 0
		else
			GUI_ABKsearch_SortOrder = 1
		end
	else
		GUI_ABKsearch_SortOrder = 1
	end
	GUI_ABKsearch_SortBy = sortBy
	CharScanner:ABKsearch_SortButtonUpdate()
	CharScanner:ABKsearch_GUICreate()
	CharScanner:ABKsearch_FillList()
end

function CharScanner:ABKsearch_SortButtonUpdate()
	local arrow
	for i=1, 3 do
		if i == 1 then -- 1 = standard
			arrow = GVAR.MainFrame.StatsPvE.SearchSort.Standard.Arrow
		elseif i == 2 then -- 2 = name
			arrow = GVAR.MainFrame.StatsPvE.SearchSort.Name.Arrow
		elseif i == 3 then -- 3 = kills
			arrow = GVAR.MainFrame.StatsPvE.SearchSort.Kills.Arrow
		end
		if GUI_ABKsearch_SortBy == i then
			if GUI_ABKsearch_SortOrder == 1 then arrow:SetTexCoord(unpack(Textures.SortArrow.coords1)) end
			if GUI_ABKsearch_SortOrder == 0 then arrow:SetTexCoord(unpack(Textures.SortArrow.coords2)) end
		else
			arrow:SetTexCoord(0, 0, 0, 0)
		end
	end
end

function CharScanner:ABKsearch_OnVerticalScroll(value)
	GVAR.MainFrame.StatsPvE.SearchScrollFrame.ScrollBar:SetValue(value)
	GVAR.MainFrame.StatsPvE.SearchScrollFrame.offset = math_floor((value / sizeGeneral.BarHeight) + 0.5)
	CharScanner:ABKsearch_FillList()
end

function CharScanner:ABKsearch_FillList()
	local scrollFrame = GVAR.MainFrame.StatsPvE.SearchScrollFrame
	local scrollFrameHeight = 0
	local numToDisplay = GUI_ABKsearch_curRows
	local valueStep = 12

	if GUI_ABKsearch_Count < numToDisplay then
		numToDisplay = GUI_ABKsearch_Count
	end

	if GUI_ABKsearch_Count > 0 then
		scrollFrameHeight = (GUI_ABKsearch_Count - numToDisplay) * valueStep
		if scrollFrameHeight < 0 then
			scrollFrameHeight = 0
		end
	end
	scrollFrame.ScrollBar:SetMinMaxValues(0, scrollFrameHeight)
	scrollFrame.ScrollBar:SetValueStep(valueStep)

	if GUI_ABKsearch_Count <= GUI_ABKsearch_curRows then
		ScrollFrameDisable(scrollFrame)
	else
		ScrollFrameEnable(scrollFrame)
	end

	if scrollFrame.ScrollBar:GetValue() == 0 then
		scrollFrame.ScrollBarScrollUpButton:Disable()
	else
		scrollFrame.ScrollBarScrollUpButton:Enable()
	end
	if ((scrollFrame.ScrollBar:GetValue() - scrollFrameHeight) == 0) then
		scrollFrame.ScrollBarScrollDownButton:Disable()
	else
		scrollFrame.ScrollBarScrollDownButton:Enable()
	end

	for i=1, GUI_ABKsearch_maxRows do
		GVAR.MainFrame.StatsPvE.SearchScrollFrameButton[i]:Hide()
	end

	local j = 1
	local sliderPos = scrollFrame.offset
	for i=sliderPos+1, sliderPos+numToDisplay do
		if sliderPos < GUI_ABKsearch_Count then
			if GUI_ABKsearch_data[i].BossesTotal > 0 then
				GVAR.MainFrame.StatsPvE.SearchScrollFrameButton[j].Name:SetTextColor(color.Green.rgba[1], color.Green.rgba[2], color.Green.rgba[3], color.Green.rgba[4])
			else
				GVAR.MainFrame.StatsPvE.SearchScrollFrameButton[j].Name:SetTextColor(color.Red.rgba[1], color.Red.rgba[2], color.Red.rgba[3], color.Red.rgba[4])
			end
			GVAR.MainFrame.StatsPvE.SearchScrollFrameButton[j].Name:SetText(GUI_ABKsearch_data[i].Name)
			GVAR.MainFrame.StatsPvE.SearchScrollFrameButton[j].BossesTotal:SetText(GUI_ABKsearch_data[i].BossesTotal)
			GVAR.MainFrame.StatsPvE.SearchScrollFrameButton[j]:Show()
			j = j + 1
		end
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function CharScanner:History_Update()
	if inPartyOrRaid then
		TEMPLATE.EnableTabButton(GVAR.HistoryFrame.GroupStatusTabAll)
		TEMPLATE.EnableTabButton(GVAR.HistoryFrame.GroupStatusTabGroup)
		TEMPLATE.EnableTabButton(GVAR.HistoryFrame.GroupStatusTabOthers)
		if GroupTabStatus == 1 then
			TEMPLATE.SetTabButton(GVAR.HistoryFrame.GroupStatusTabAll, true)
			TEMPLATE.SetTabButton(GVAR.HistoryFrame.GroupStatusTabGroup)
			TEMPLATE.SetTabButton(GVAR.HistoryFrame.GroupStatusTabOthers)
		elseif GroupTabStatus == 2 then
			TEMPLATE.SetTabButton(GVAR.HistoryFrame.GroupStatusTabAll)
			TEMPLATE.SetTabButton(GVAR.HistoryFrame.GroupStatusTabGroup, true)
			TEMPLATE.SetTabButton(GVAR.HistoryFrame.GroupStatusTabOthers)
		elseif GroupTabStatus == 3 then
			TEMPLATE.SetTabButton(GVAR.HistoryFrame.GroupStatusTabAll)
			TEMPLATE.SetTabButton(GVAR.HistoryFrame.GroupStatusTabGroup)
			TEMPLATE.SetTabButton(GVAR.HistoryFrame.GroupStatusTabOthers, true)
		end
	else
		CleanOORData() -- XFIX
		TEMPLATE.DisableTabButton(GVAR.HistoryFrame.GroupStatusTabGroup, true)
		TEMPLATE.DisableTabButton(GVAR.HistoryFrame.GroupStatusTabOthers, true)
		TEMPLATE.SetTabButton(GVAR.HistoryFrame.GroupStatusTabAll, true)
		GroupTabStatus = 1
	end
	CharScanner:History_GUICreate()
	CharScanner:History_FillList()
	CharScanner:History_StatusUpdate()
end

function CharScanner:History_GUICreate()
	if not GroupScanIsActive and not SingleScanIsActive then -- XFIX
		CleanTempData()
	end -- XFIX

	table_wipe(GUI_history_data)
	NumScannedChars = 0

	local x = 1
	local addThis
	for guid, v in pairs(SCAN) do
		if v.time then -- XFIX
			
			NumScannedChars = NumScannedChars + 1
			
			addThis = nil
			if GroupTabStatus == 1 then
				if not v.oorScan then
					addThis = true
				end
			elseif GroupTabStatus == 2 then
				if GroupMembers[guid] then
					addThis = true
				end
			elseif GroupTabStatus == 3 then
				if not GroupMembers[guid] and not v.oorScan then
					addThis = true
				end
			end

			if addThis then

				GUI_history_data[x] = {}
				GUI_history_data[x].guid = guid
				GUI_history_data[x].classEN = v.classEN
				GUI_history_data[x].class = v.class
				GUI_history_data[x].name = v.name
				GUI_history_data[x].time = v.time
				GUI_history_data[x].mainrole = v.mainRole

				-- talents START
				if v.activeTalentGroup then

					if v.activeTalentGroup == 1 then

						if v.talentGroup1name then
							local unspent = ""
							if v.talentGroup1pointsUnspent > 0 then
								unspent = "|cff"..color.Red.hex.."("..v.talentGroup1pointsUnspent..")|r"
							end

							local text = RoleTextShort[v.talentGroup1role].." |cff"..color.White.hex..v.talentGroup1name.." "..v.talentGroup1pointsString.."|r"
							local num = v.talentGroup1role..v.talentGroup1name..v.talentGroup1pointsString
							if v.talentGroup1pointsString == "0/0/0" then
								text = RoleTextShort[v.talentGroup1role].." "..unspent
								num = v.talentGroup1role..v.talentGroup1pointsUnspent
							else
								if unspent ~= "" then
									text = text.." "..unspent
								end
							end
							GUI_history_data[x].talents1 = text
							GUI_history_data[x].talents1Num = num
						else
							GUI_history_data[x].talents1 = ""
							GUI_history_data[x].talents1Num = "zzz"
						end
						if v.talentGroup2name then
							local unspent = ""
							if v.talentGroup2pointsUnspent > 0 then
								unspent = "|cff"..color.Red.hex.."("..v.talentGroup2pointsUnspent..")|r"
							end

							local text = RoleTextShort[v.talentGroup2role]--.." |cff"..col..v.talentGroup2name.." "..v.talentGroup2pointsString.."|r"
							local num = v.talentGroup2role..v.talentGroup2name..v.talentGroup2pointsString
							if v.talentGroup2pointsString == "0/0/0" then
								text = RoleTextShort[v.talentGroup2role].." "..unspent
								num = v.talentGroup2role..v.talentGroup2pointsUnspent
							else
								if unspent ~= "" then
									text = text.." "..unspent
								end
							end
							GUI_history_data[x].talents2 = text
							GUI_history_data[x].talents2Num = num
						else
							GUI_history_data[x].talents2 = ""
							GUI_history_data[x].talents2Num = "zzz"
						end

					elseif v.activeTalentGroup == 2 then

						if v.talentGroup2name then
							local unspent = ""
							if v.talentGroup2pointsUnspent > 0 then
								unspent = "|cff"..color.Red.hex.."("..v.talentGroup2pointsUnspent..")|r"
							end

							local text = RoleTextShort[v.talentGroup2role].." |cff"..color.White.hex..v.talentGroup2name.." "..v.talentGroup2pointsString.."|r"
							local num = v.talentGroup2role..v.talentGroup2name..v.talentGroup2pointsString
							if v.talentGroup2pointsString == "0/0/0" then
								text = RoleTextShort[v.talentGroup2role].." "..unspent
								num = v.talentGroup2role..v.talentGroup2pointsUnspent
							else
								if unspent ~= "" then
									text = text.." "..unspent
								end
							end
							GUI_history_data[x].talents1 = text
							GUI_history_data[x].talents1Num = num
						else
							GUI_history_data[x].talents1 = ""
							GUI_history_data[x].talents1Num = "zzz"
						end
						if v.talentGroup1name then
							local unspent = ""
							if v.talentGroup1pointsUnspent > 0 then
								unspent = "|cff"..color.Red.hex.."("..v.talentGroup1pointsUnspent..")|r"
							end

							local text = RoleTextShort[v.talentGroup1role]--.." |cff"..col..v.talentGroup1name.." "..v.talentGroup1pointsString.."|r"
							local num = v.talentGroup1role..v.talentGroup1name..v.talentGroup1pointsString
							if v.talentGroup1pointsString == "0/0/0" then
								text = RoleTextShort[v.talentGroup1role].." "..unspent
								num = v.talentGroup1role..v.talentGroup1pointsUnspent
							else
								if unspent ~= "" then
									text = text.." "..unspent
								end
							end
							GUI_history_data[x].talents2 = text
							GUI_history_data[x].talents2Num = num
						else
							GUI_history_data[x].talents2 = ""
							GUI_history_data[x].talents2Num = "zzz"
						end

					end

				else
					GUI_history_data[x].talents1 = ""
					GUI_history_data[x].talents1Num = "zzz"
					GUI_history_data[x].talents2 = ""
					GUI_history_data[x].talents2Num = "zzz"
				end
				-- talents END

				GUI_history_data[x].enchcount = v.gearEnchCount
				GUI_history_data[x].pvpcount = v.gearPvPCount
				GUI_history_data[x].itemlevel = cnum(v.itemlevelAvg)
				GUI_history_data[x].itemlevelNum = v.itemlevelAvg

				-- raid selection START
				local col
				if GUI_history_RaidL3 then
					local achievement = math_floor((v.PVEDATA[GUI_history_RaidL1][GUI_history_RaidL2][GUI_history_RaidL3].achievementCount/PVEDUMP[GUI_history_RaidL1][GUI_history_RaidL2][GUI_history_RaidL3].achievementTotal*100) * mult2 + 0.5) / mult2
					if PVEDUMP[GUI_history_RaidL1][GUI_history_RaidL2][GUI_history_RaidL3].achievementTotal == v.PVEDATA[GUI_history_RaidL1][GUI_history_RaidL2][GUI_history_RaidL3].achievementCount then
						col="|cff"..color.Green.hex else col="|cff"..color.Red.hex
					end
					GUI_history_data[x].achievement = col..v.PVEDATA[GUI_history_RaidL1][GUI_history_RaidL2][GUI_history_RaidL3].achievementCount.."|r / |cff"..color.Level3.hex..PVEDUMP[GUI_history_RaidL1][GUI_history_RaidL2][GUI_history_RaidL3].achievementTotal.."|r"
					GUI_history_data[x].achievementNum = achievement
					local boss = math_floor((v.PVEDATA[GUI_history_RaidL1][GUI_history_RaidL2][GUI_history_RaidL3].statisticBoss/PVEDUMP[GUI_history_RaidL1][GUI_history_RaidL2][GUI_history_RaidL3].statisticTotal*100) * mult2 + 0.5) / mult2
					if PVEDUMP[GUI_history_RaidL1][GUI_history_RaidL2][GUI_history_RaidL3].statisticTotal == v.PVEDATA[GUI_history_RaidL1][GUI_history_RaidL2][GUI_history_RaidL3].statisticBoss then
						col="|cff"..color.Green.hex else col="|cff"..color.Red.hex
					end
					GUI_history_data[x].boss = col..v.PVEDATA[GUI_history_RaidL1][GUI_history_RaidL2][GUI_history_RaidL3].statisticBoss.."|r / |cff"..color.Level3.hex..PVEDUMP[GUI_history_RaidL1][GUI_history_RaidL2][GUI_history_RaidL3].statisticTotal.."|r"
					GUI_history_data[x].bossNum = boss
					GUI_history_data[x].kills = v.PVEDATA[GUI_history_RaidL1][GUI_history_RaidL2][GUI_history_RaidL3].statisticBossAll
				elseif GUI_history_RaidL2 then
					local achievement = math_floor((v.PVEDATA[GUI_history_RaidL1][GUI_history_RaidL2].achievementCount/PVEDUMP[GUI_history_RaidL1][GUI_history_RaidL2].achievementTotal*100) * mult2 + 0.5) / mult2
					if PVEDUMP[GUI_history_RaidL1][GUI_history_RaidL2].achievementTotal == v.PVEDATA[GUI_history_RaidL1][GUI_history_RaidL2].achievementCount then
						col="|cff"..color.Green.hex else col="|cff"..color.Red.hex
					end
					GUI_history_data[x].achievement = col..v.PVEDATA[GUI_history_RaidL1][GUI_history_RaidL2].achievementCount.."|r / |cff"..color.Level2.hex..PVEDUMP[GUI_history_RaidL1][GUI_history_RaidL2].achievementTotal.."|r"
					GUI_history_data[x].achievementNum = achievement
					local boss = math_floor((v.PVEDATA[GUI_history_RaidL1][GUI_history_RaidL2].statisticBoss/PVEDUMP[GUI_history_RaidL1][GUI_history_RaidL2].statisticTotal*100) * mult2 + 0.5) / mult2
					if PVEDUMP[GUI_history_RaidL1][GUI_history_RaidL2].statisticTotal == v.PVEDATA[GUI_history_RaidL1][GUI_history_RaidL2].statisticBoss then
						col="|cff"..color.Green.hex else col="|cff"..color.Red.hex
					end
					GUI_history_data[x].boss = col..v.PVEDATA[GUI_history_RaidL1][GUI_history_RaidL2].statisticBoss.."|r / |cff"..color.Level2.hex..PVEDUMP[GUI_history_RaidL1][GUI_history_RaidL2].statisticTotal.."|r"
					GUI_history_data[x].bossNum = boss
					GUI_history_data[x].kills = v.PVEDATA[GUI_history_RaidL1][GUI_history_RaidL2].statisticBossAll
				else
					if GUI_history_RaidL1 == 0 then -- Total
						local achievement = math_floor((v.PvEAchievementCount/v.PvEAchievementTotal*100) * mult2 + 0.5) / mult2
						GUI_history_data[x].achievement = "|cff"..color.LightYellow.hex..cnum(achievement).."%|r"
						GUI_history_data[x].achievementNum = achievement
						local boss = math_floor((v.PvEStatisticCount/v.PvEStatisticTotal*100) * mult2 + 0.5) / mult2
						GUI_history_data[x].boss = "|cff"..color.LightYellow.hex..cnum(boss).."%|r"
						GUI_history_data[x].bossNum = boss
						GUI_history_data[x].kills = v.PvEStatisticBossKills
					else
						local achievement = math_floor((v.PVEDATA[GUI_history_RaidL1].achievementCount/PVEDUMP[GUI_history_RaidL1].achievementTotal*100) * mult2 + 0.5) / mult2
						if PVEDUMP[GUI_history_RaidL1].achievementTotal == v.PVEDATA[GUI_history_RaidL1].achievementCount then
							col="|cff"..color.Green.hex else col="|cff"..color.Red.hex
						end
						GUI_history_data[x].achievement = col..v.PVEDATA[GUI_history_RaidL1].achievementCount.."|r / |cff"..color.Level1.hex..PVEDUMP[GUI_history_RaidL1].achievementTotal.."|r"
						GUI_history_data[x].achievementNum = achievement
						local boss = math_floor((v.PVEDATA[GUI_history_RaidL1].statisticBoss/PVEDUMP[GUI_history_RaidL1].statisticTotal*100) * mult2 + 0.5) / mult2
						if PVEDUMP[GUI_history_RaidL1].statisticTotal == v.PVEDATA[GUI_history_RaidL1].statisticBoss then
							col="|cff"..color.Green.hex else col="|cff"..color.Red.hex
						end
						GUI_history_data[x].boss = col..v.PVEDATA[GUI_history_RaidL1].statisticBoss.."|r / |cff"..color.Level1.hex..PVEDUMP[GUI_history_RaidL1].statisticTotal.."|r"
						GUI_history_data[x].bossNum = boss
						GUI_history_data[x].kills = v.PVEDATA[GUI_history_RaidL1].statisticBossAll
					end
				end
				-- raid selection END

				-- single achievement compare START
				if GUI_achievementdetail_Compare then
					if AchievementDetailFrameMode == "PvE" then
						if SCAN[guid].PVEDATA.achievementID and SCAN[guid].PVEDATA.achievementID[ GUI_achievementdetail_Compare ] then
							col="|cff"..color.Green.hex
						else
							col="|cff"..color.Red.hex
						end
						GUI_history_data[x].achievementCompare = col..AchievementIDTable[ GUI_achievementdetail_Compare ].."|r"
					elseif AchievementDetailFrameMode == "PvP" then
						if SCAN[guid].PVPDATA.achievementID and SCAN[guid].PVPDATA.achievementID[ GUI_achievementdetail_Compare ] then
							col="|cff"..color.Green.hex
						else
							col="|cff"..color.Red.hex
						end
						GUI_history_data[x].achievementCompare = col..AchievementIDTable[ GUI_achievementdetail_Compare ].."|r"
					end
				else
					GUI_history_data[x].achievementCompare = ""
				end
				-- single achievement compare END
				x = x + 1

			end

		end -- XFIX
	end
	GUI_history_Count = #GUI_history_data

	local sortfunc
	if GUI_history_SortBy == 1 then -- 1 = class
		if GUI_history_SortOrder == 1 then
			sortfunc = function(a, b)
				if a.class == b.class then
					if a.name < b.name then
						return true
					end
				elseif a.class < b.class then
					return true
				end
			end
		else
			sortfunc = function(a, b)
				if a.class == b.class then
					if a.name < b.name then
						return true
					end
				elseif a.class > b.class then
					return true
				end
			end
		end
	elseif GUI_history_SortBy == 2 then -- 2 = name
		if GUI_history_SortOrder == 1 then
			sortfunc = function(a, b)
				if a.name < b.name then
					return true
				end
			end
		else
			sortfunc = function(a, b)
				if a.name > b.name then
					return true
				end
			end
		end
	elseif GUI_history_SortBy == 3 then -- 3 = time
		if GUI_history_SortOrder == 1 then
			sortfunc = function(a, b)
				if a.time == b.time then
					if a.name < b.name then
						return true
					end
				elseif a.time < b.time then
					return true
				end
			end
		else
			sortfunc = function(a, b)
				if a.time == b.time then
					if a.name < b.name then
						return true
					end
				elseif a.time > b.time then
					return true
				end
			end
		end
	elseif GUI_history_SortBy == 4 then -- 4 = mainrole
		if GUI_history_SortOrder == 1 then
			sortfunc = function(a, b)
				if a.mainrole == b.mainrole then
					if a.class == b.class then
						if a.name < b.name then
							return true
						end
					elseif a.class < b.class then
						return true
					end
				elseif a.mainrole < b.mainrole then
					return true
				end
			end
		else
			sortfunc = function(a, b)
				if a.mainrole == b.mainrole then
					if a.class == b.class then
						if a.name < b.name then
							return true
						end
					elseif a.class < b.class then
						return true
					end
				elseif a.mainrole > b.mainrole then
					return true
				end
			end
		end
	elseif GUI_history_SortBy == 5 then -- 5 = talents1
		if GUI_history_SortOrder == 1 then
			sortfunc = function(a, b)
				if a.talents1Num == b.talents1Num then
					if a.name < b.name then
						return true
					end
				elseif a.talents1Num < b.talents1Num then
					return true
				end
			end
		else
			sortfunc = function(a, b)
				if a.talents1Num == b.talents1Num then
					if a.name < b.name then
						return true
					end
				elseif a.talents1Num > b.talents1Num then
					return true
				end
			end
		end
	elseif GUI_history_SortBy == 6 then -- 6 = talents2
		if GUI_history_SortOrder == 1 then
			sortfunc = function(a, b)
				if a.talents2Num == b.talents2Num then
					if a.name < b.name then
						return true
					end
				elseif a.talents2Num < b.talents2Num then
					return true
				end
			end
		else
			sortfunc = function(a, b)
				if a.talents2Num == b.talents2Num then
					if a.name < b.name then
						return true
					end
				elseif a.talents2Num > b.talents2Num then
					return true
				end
			end
		end
	elseif GUI_history_SortBy == 7 then -- 7 = EnchantmentCount
		if GUI_history_SortOrder == 1 then
			sortfunc = function(a, b)
				if a.enchcount == b.enchcount then
					if a.itemlevelNum == b.itemlevelNum then
						if a.name < b.name then
							return true
						end
					elseif a.itemlevelNum > b.itemlevelNum then
						return true
					end
				elseif a.enchcount > b.enchcount then
					return true
				end
			end
		else
			sortfunc = function(a, b)
				if a.enchcount == b.enchcount then
					if a.itemlevelNum == b.itemlevelNum then
						if a.name < b.name then
							return true
						end
					elseif a.itemlevelNum > b.itemlevelNum then
						return true
					end
				elseif a.enchcount < b.enchcount then
					return true
				end
			end
		end
	elseif GUI_history_SortBy == 8 then -- 8 = PvPCount
		if GUI_history_SortOrder == 1 then
			sortfunc = function(a, b)
				if a.pvpcount == b.pvpcount then
					if a.itemlevelNum == b.itemlevelNum then
						if a.name < b.name then
							return true
						end
					elseif a.itemlevelNum > b.itemlevelNum then
						return true
					end
				elseif a.pvpcount > b.pvpcount then
					return true
				end
			end
		else
			sortfunc = function(a, b)
				if a.pvpcount == b.pvpcount then
					if a.itemlevelNum == b.itemlevelNum then
						if a.name < b.name then
							return true
						end
					elseif a.itemlevelNum > b.itemlevelNum then
						return true
					end
				elseif a.pvpcount < b.pvpcount then
					return true
				end
			end
		end
	elseif GUI_history_SortBy == 9 then -- 9 = itemlevel (itemlevelNum)
		if GUI_history_SortOrder == 1 then
			sortfunc = function(a, b)
				if a.itemlevelNum == b.itemlevelNum then
					if a.class == b.class then
						if a.name < b.name then
							return true
						end
					elseif a.class < b.class then
						return true
					end
				elseif a.itemlevelNum > b.itemlevelNum then
					return true
				end
			end
		else
			sortfunc = function(a, b)
				if a.itemlevelNum == b.itemlevelNum then
					if a.class == b.class then
						if a.name < b.name then
							return true
						end
					elseif a.class < b.class then
						return true
					end
				elseif a.itemlevelNum < b.itemlevelNum then
					return true
				end
			end
		end
	elseif GUI_history_SortBy == 10 then -- 10 = achievement (achievementNum)
		if GUI_history_SortOrder == 1 then
			sortfunc = function(a, b)
				if a.achievementNum == b.achievementNum then
					if a.bossNum == b.bossNum then
						if a.kills == b.kills then
							if a.name < b.name then
								return true
							end
						elseif a.kills > b.kills then
							return true
						end
					elseif a.bossNum > b.bossNum then
						return true
					end
				elseif a.achievementNum > b.achievementNum then
					return true
				end
			end
		else
			sortfunc = function(a, b)
				if a.achievementNum == b.achievementNum then
					if a.bossNum == b.bossNum then
						if a.kills == b.kills then
							if a.name < b.name then
								return true
							end
						elseif a.kills < b.kills then
							return true
						end
					elseif a.bossNum < b.bossNum then
						return true
					end
				elseif a.achievementNum < b.achievementNum then
					return true
				end
			end
		end
	elseif GUI_history_SortBy == 11 then -- 11 = boss (bossNum)
		if GUI_history_SortOrder == 1 then
			sortfunc = function(a, b)
				if a.bossNum == b.bossNum then
					if a.achievementNum == b.achievementNum then
						if a.kills == b.kills then
							if a.name < b.name then
								return true
							end
						elseif a.kills > b.kills then
							return true
						end
					elseif a.achievementNum > b.achievementNum then
						return true
					end
				elseif a.bossNum > b.bossNum then
					return true
				end
			end
		else
			sortfunc = function(a, b)
				if a.bossNum == b.bossNum then
					if a.achievementNum == b.achievementNum then
						if a.kills == b.kills then
							if a.name < b.name then
								return true
							end
						elseif a.kills < b.kills then
							return true
						end
					elseif a.achievementNum < b.achievementNum then
						return true
					end
				elseif a.bossNum < b.bossNum then
					return true
				end
			end
		end
	elseif GUI_history_SortBy == 12 then -- 12 = kills (kills)
		if GUI_history_SortOrder == 1 then
			sortfunc = function(a, b)
				if a.kills == b.kills then
					if a.bossNum == b.bossNum then
						if a.achievementNum == b.achievementNum then
							if a.name < b.name then
								return true
							end
						elseif a.achievementNum > b.achievementNum then
							return true
						end
					elseif a.bossNum > b.bossNum then
						return true
					end
				elseif a.kills > b.kills then
					return true
				end
			end
		else
			sortfunc = function(a, b)
				if a.kills == b.kills then
					if a.bossNum == b.bossNum then
						if a.achievementNum == b.achievementNum then
							if a.name < b.name then
								return true
							end
						elseif a.achievementNum < b.achievementNum then
							return true
						end
					elseif a.bossNum < b.bossNum then
						return true
					end
				elseif a.kills < b.kills then
					return true
				end
			end
		end
	elseif GUI_history_SortBy == 13 then -- 13 = achievementCompare (SingleAchievementCompare)
		if GUI_history_SortOrder == 1 then
			sortfunc = function(a, b)
				if a.achievementCompare == b.achievementCompare then
					if a.name < b.name then
						return true
					end
				elseif a.achievementCompare < b.achievementCompare then
					return true
				end
			end
		else
			sortfunc = function(a, b)
				if a.achievementCompare == b.achievementCompare then
					if a.name < b.name then
						return true
					end
				elseif a.achievementCompare > b.achievementCompare then
					return true
				end
			end
		end

	end

	table_sort(GUI_history_data, sortfunc)
end

function CharScanner:History_GUISort(what, sortBy, l1, l2, l3)
	if what == "RAID" then

		GVAR.HistoryFrame.Sort.Achievement:Show()
		GVAR.HistoryFrame.Sort.Boss:Show()
		GVAR.HistoryFrame.Sort.Kills:Show()
		GVAR.HistoryFrame.Sort.SingleAchievementCompare:Hide()
		for i = 1, GUI_history_maxRows do
			GVAR.HistoryFrame.ScrollFrameButton[i].Achievement:Show()
			GVAR.HistoryFrame.ScrollFrameButton[i].Boss:Show()
			GVAR.HistoryFrame.ScrollFrameButton[i].Kills:Show()
			GVAR.HistoryFrame.ScrollFrameButton[i].SingleAchievementCompare:Hide()
		end

		GUI_history_RaidL1 = l1
		GUI_history_RaidL2 = l2
		GUI_history_RaidL3 = l3
		local name
		if GUI_history_RaidL3 then
			name = L[PVEDUMP[l1][l2][l3].name].." ("..L[PVEDUMP[l1][l2].name]..")"
		elseif GUI_history_RaidL2 then
			name = L[PVEDUMP[l1][l2].name]
		else
			if l1 == 0 then
				name = L["Total"]
			else
				name = L[PVEDUMP[l1].name]
			end
		end
		GVAR.HistoryFrame.Sort.PvEdump.PullDownButtonText:SetText(name)
		GVAR.HistoryFrame.Sort.PvEdump.PullDownButtonExpand:SetTexCoord(unpack(Textures.Expand.coords))
		GVAR.HistoryFrame.Sort.PvEdump.PullDownMenu:Hide()
		I_history_SortOrder = 1
		GUI_history_SortBy = 11 -- Boss
	elseif what == "ACHIEVEMENT" then
		GUI_achievementdetail_Compare = l1
		GVAR.HistoryFrame.Sort.Achievement:Hide()
		GVAR.HistoryFrame.Sort.Boss:Hide()
		GVAR.HistoryFrame.Sort.Kills:Hide()
		GVAR.HistoryFrame.Sort.SingleAchievementCompare:Show()
		for i = 1, GUI_history_maxRows do
			GVAR.HistoryFrame.ScrollFrameButton[i].Achievement:Hide()
			GVAR.HistoryFrame.ScrollFrameButton[i].Boss:Hide()
			GVAR.HistoryFrame.ScrollFrameButton[i].Kills:Hide()
			GVAR.HistoryFrame.ScrollFrameButton[i].SingleAchievementCompare:Show()
		end
		GVAR.HistoryFrame.Sort.PvEdump.PullDownButtonText:SetText(AchievementIDTable[l1])
		GVAR.HistoryFrame.Sort.PvEdump.PullDownButtonExpand:SetTexCoord(unpack(Textures.Expand.coords))
		GVAR.HistoryFrame.Sort.PvEdump.PullDownMenu:Hide()
		GUI_history_SortOrder = 1
		GUI_history_SortBy = 13 -- SingleAchievementCompare
	elseif what == "PVP" then
		--print(what, sortBy, l1, l2, l3) -- TODO
	else
		if GUI_history_SortBy == sortBy then
			if GUI_history_SortOrder == 1 then
				GUI_history_SortOrder = 0
			else
				GUI_history_SortOrder = 1
			end
		else
			GUI_history_SortOrder = 1
		end
		GUI_history_SortBy = sortBy
	end
	CharScanner:History_SortButtonUpdate()
	CharScanner:History_GUICreate()
	CharScanner:History_FillList()
end

function CharScanner:History_SortButtonUpdate()
	local arrow
	for i=1, 13 do
		if i == 1 then     -- 1 = class
			arrow = GVAR.HistoryFrame.Sort.Icon.Arrow
		elseif i == 2 then -- 2 = name
			arrow = GVAR.HistoryFrame.Sort.Name.Arrow
		elseif i == 3 then -- 3 = time
			arrow = GVAR.HistoryFrame.Sort.Time.Arrow
		elseif i == 4 then -- 4 = mainrole
			arrow = GVAR.HistoryFrame.Sort.MainRole.Arrow
		elseif i == 5 then -- 5 = talents1
			arrow = GVAR.HistoryFrame.Sort.Talents1.Arrow
		elseif i == 6 then -- 6 = talents2
			arrow = GVAR.HistoryFrame.Sort.Talents2.Arrow
		elseif i == 7 then -- 7 = EnchantmentCount
			arrow = GVAR.HistoryFrame.Sort.EnchantmentCount.Arrow
		elseif i == 8 then -- 8 = PvPCount
			arrow = GVAR.HistoryFrame.Sort.PvPCount.Arrow
		elseif i == 9 then -- 9 = ItemLevel
			arrow = GVAR.HistoryFrame.Sort.ItemLevel.Arrow
		elseif i == 10 then -- 10 = Achievement
			arrow = GVAR.HistoryFrame.Sort.Achievement.Arrow
		elseif i == 11 then -- 11 = Boss
			arrow = GVAR.HistoryFrame.Sort.Boss.Arrow
		elseif i == 12 then -- 12 = Kills
			arrow = GVAR.HistoryFrame.Sort.Kills.Arrow
		elseif i == 13 then -- 13 = SingleAchievementCompare
			arrow = GVAR.HistoryFrame.Sort.SingleAchievementCompare.Arrow
		end
		if GUI_history_SortBy == i then
			if GUI_history_SortOrder == 1 then arrow:SetTexCoord(unpack(Textures.SortArrow.coords1)) end
			if GUI_history_SortOrder == 0 then arrow:SetTexCoord(unpack(Textures.SortArrow.coords2)) end
		else
			arrow:SetTexCoord(0, 0, 0, 0)
		end
	end
end

function CharScanner:History_OnVerticalScroll(value)
	GVAR.HistoryFrame.ScrollFrame.ScrollBar:SetValue(value)
	GVAR.HistoryFrame.ScrollFrame.offset = math_floor((value / sizeGeneral.BarHeight) + 0.5)
	CharScanner:History_FillList()
end

function CharScanner:History_FillList()
	local scrollFrame = GVAR.HistoryFrame.ScrollFrame
	local scrollFrameHeight = 0
	local numToDisplay = GUI_history_curRows
	local valueStep = 12

	if GUI_history_Count < numToDisplay then
		numToDisplay = GUI_history_Count
	end

	if GUI_history_Count > 0 then
		scrollFrameHeight = (GUI_history_Count - numToDisplay) * valueStep
		if scrollFrameHeight < 0 then
			scrollFrameHeight = 0
		end
	end
	scrollFrame.ScrollBar:SetMinMaxValues(0, scrollFrameHeight)
	scrollFrame.ScrollBar:SetValueStep(valueStep)

	if GUI_history_Count <= GUI_history_curRows then
		ScrollFrameDisable(scrollFrame)
	else
		ScrollFrameEnable(scrollFrame)
	end

	if scrollFrame.ScrollBar:GetValue() == 0 then
		scrollFrame.ScrollBarScrollUpButton:Disable()
	else
		scrollFrame.ScrollBarScrollUpButton:Enable()
	end
	if ((scrollFrame.ScrollBar:GetValue() - scrollFrameHeight) == 0) then
		scrollFrame.ScrollBarScrollDownButton:Disable()
	else
		scrollFrame.ScrollBarScrollDownButton:Enable()
	end

	for i=1, GUI_history_maxRows do
		GVAR.HistoryFrame.ScrollFrameButton[i]:Hide()
	end

	local j = 1
	local sliderPos = scrollFrame.offset
	for i=sliderPos+1, sliderPos+numToDisplay do
		if sliderPos < GUI_history_Count then
			if GroupTabStatus == 1 then
				GVAR.HistoryFrame.ScrollFrameButton[j].Delete:Show()
			else
				GVAR.HistoryFrame.ScrollFrameButton[j].Delete:Hide()
			end

			if SCAN[ GUI_history_data[i].guid ].oorScan then
				GVAR.HistoryFrame.ScrollFrameButton[j]:SetAlpha(0.3)
			else
				GVAR.HistoryFrame.ScrollFrameButton[j]:SetAlpha(1)
			end

			GVAR.HistoryFrame.ScrollFrameButton[j].Icon:SetTexCoord(classes[GUI_history_data[i].classEN].icon[1], classes[GUI_history_data[i].classEN].icon[2], classes[GUI_history_data[i].classEN].icon[3], classes[GUI_history_data[i].classEN].icon[4])
			GVAR.HistoryFrame.ScrollFrameButton[j].Name:SetText(GUI_history_data[i].name)
			GVAR.HistoryFrame.ScrollFrameButton[j].Name:SetTextColor(classcolors[GUI_history_data[i].classEN].r, classcolors[GUI_history_data[i].classEN].g, classcolors[GUI_history_data[i].classEN].b, 1)
			GVAR.HistoryFrame.ScrollFrameButton[j].Time:SetText(GUI_history_data[i].time)

			GVAR.HistoryFrame.ScrollFrameButton[j].MainRole:SetTexture(Textures.RoleIcon[GUI_history_data[i].mainrole].path)
			GVAR.HistoryFrame.ScrollFrameButton[j].MainRole:SetTexCoord(Textures.RoleIcon[GUI_history_data[i].mainrole].coords[1], Textures.RoleIcon[GUI_history_data[i].mainrole].coords[2], Textures.RoleIcon[GUI_history_data[i].mainrole].coords[3], Textures.RoleIcon[GUI_history_data[i].mainrole].coords[4])

			GVAR.HistoryFrame.ScrollFrameButton[j].Talents1:SetText(GUI_history_data[i].talents1)
			GVAR.HistoryFrame.ScrollFrameButton[j].Talents2:SetText(GUI_history_data[i].talents2)

			GVAR.HistoryFrame.ScrollFrameButton[j].EnchantmentCount:SetText(GUI_history_data[i].enchcount)
			GVAR.HistoryFrame.ScrollFrameButton[j].PvPCount:SetText(GUI_history_data[i].pvpcount)
			GVAR.HistoryFrame.ScrollFrameButton[j].ItemLevel:SetText(GUI_history_data[i].itemlevel)

			GVAR.HistoryFrame.ScrollFrameButton[j].Achievement:SetText(GUI_history_data[i].achievement)
			GVAR.HistoryFrame.ScrollFrameButton[j].Boss:SetText(GUI_history_data[i].boss)
			GVAR.HistoryFrame.ScrollFrameButton[j].Kills:SetText(GUI_history_data[i].kills)

			GVAR.HistoryFrame.ScrollFrameButton[j].SingleAchievementCompare:SetText(GUI_history_data[i].achievementCompare)

			GVAR.HistoryFrame.ScrollFrameButton[j]:Show()
			GVAR.HistoryFrame.ScrollFrameButton[j].guid = GUI_history_data[i].guid

			GVAR.HistoryFrame.ScrollFrameButton[j].Arrow:Hide()
			if GUIselectedGUID == GUI_history_data[i].guid then
				GVAR.HistoryFrame.ScrollFrameButton[j].Arrow:Show()
			end
			j = j + 1
		end
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function CharScanner:AchievementDetail_Update(mode, l1, l2, l3, sel)
	-- do not show AchievementDetail if last scan is incomplete
	if not LatestScanIsComplete then
		CharScanner:AchievementDetail_HideAll()
		return
	end

	if sel then
		GUI_achievementdetail_Select = sel
		GVAR.AchievementDetailFrame.Sort.ACI.PullDownButtonText:SetText( PullDownMenu_ACI[sel] )
		GVAR.AchievementDetailFrame.Sort.ACI.PullDownButtonExpand:SetTexCoord(unpack(Textures.Expand.coords))
		GVAR.AchievementDetailFrame.Sort.ACI.PullDownMenu:Hide()
	else
		if mode == "PvE" then
			GUI_achievementdetail_L1 = l1
			GUI_achievementdetail_L2 = l2
			GUI_achievementdetail_L3 = l3
		else
			GUI_PvPachievementdetail_L1 = l1
			GUI_PvPachievementdetail_L2 = l2
			GUI_PvPachievementdetail_L3 = l3
		end
	end

	GVAR.AchievementDetailFrame:Hide() -- for SetToplevel
	GVAR.AchievementDetailFrame:Show()
	CharScanner:AchievementDetail_GUICreate(mode)
	CharScanner:AchievementDetail_FillList()
	CharScanner:AchievementDetail_StatusUpdate(mode)
end

function CharScanner:AchievementDetail_StatusUpdate(mode)
	local all = 0
	local complete = 0
	local incomplete = 0

	if mode == "PvE" then
		if GUI_achievementdetail_L3 then
			all        = PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2][GUI_achievementdetail_L3].achievementTotal
			complete   = SCAN[GUIselectedGUID].PVEDATA[GUI_achievementdetail_L1][GUI_achievementdetail_L2][GUI_achievementdetail_L3].achievementCount
			incomplete = all-complete
		elseif GUI_achievementdetail_L2 then
			all        = PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2].achievementTotal
			complete   = SCAN[GUIselectedGUID].PVEDATA[GUI_achievementdetail_L1][GUI_achievementdetail_L2].achievementCount
			incomplete = all-complete
		else
			all        = PVEDUMP[GUI_achievementdetail_L1].achievementTotal
			complete   = SCAN[GUIselectedGUID].PVEDATA[GUI_achievementdetail_L1].achievementCount
			incomplete = all-complete
		end
	else
		if GUI_PvPachievementdetail_L3 then
			all        = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][GUI_PvPachievementdetail_L3].achievementTotal
			complete   = SCAN[GUIselectedGUID].PVPDATA[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][GUI_PvPachievementdetail_L3].achievementCount
			incomplete = all-complete
		elseif GUI_PvPachievementdetail_L2 then
			all        = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2].achievementTotal
			complete   = SCAN[GUIselectedGUID].PVPDATA[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2].achievementCount
			incomplete = all-complete
		else
			all        = PVPDUMP[GUI_PvPachievementdetail_L1].achievementTotal
			complete   = SCAN[GUIselectedGUID].PVPDATA[GUI_PvPachievementdetail_L1].achievementCount
			incomplete = all-complete
		end
	end

	GVAR.AchievementDetailFrame.StatusText:SetText("|cff"..color.Green.hex..complete.."|r | |cff"..color.Red.hex..incomplete.."|r | |cff"..color.White.hex..all.."|r")
end

function CharScanner:AchievementDetail_GUICreate(mode)
	if mode == "PvE" then
		-- PvE START  ----------------------
		-- DUMMY_FactionFix START
		if SCAN[GUIselectedGUID].faction == "Horde" then
			PVEDUMP[3][1].achievement[54] = 4297
		else
			PVEDUMP[3][1].achievement[54] = 4298
		end
		-- DUMMY_FactionFix END

		table_wipe(GUI_achievementdetail_data)
		local x = 1
		if GUI_achievementdetail_L3 then

			if GUI_achievementdetail_Select == 1 then
				for i = 1, #PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2][GUI_achievementdetail_L3].achievement do
					GUI_achievementdetail_data[x] = {}
					GUI_achievementdetail_data[x].id = PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2][GUI_achievementdetail_L3].achievement[i]
					GUI_achievementdetail_data[x].name = AchievementIDTable[ PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2][GUI_achievementdetail_L3].achievement[i] ]
					if SCAN[GUIselectedGUID].PVEDATA.achievementID and SCAN[GUIselectedGUID].PVEDATA.achievementID[ PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2][GUI_achievementdetail_L3].achievement[i] ] then
						GUI_achievementdetail_data[x].color = 1
					end
					x = x + 1
				end
				GUI_achievementdetail_Count = x - 1
			elseif GUI_achievementdetail_Select == 2 then
				for i = 1, #PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2][GUI_achievementdetail_L3].achievement do
					if SCAN[GUIselectedGUID].PVEDATA.achievementID and SCAN[GUIselectedGUID].PVEDATA.achievementID[ PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2][GUI_achievementdetail_L3].achievement[i] ] then
						GUI_achievementdetail_data[x] = {}
						GUI_achievementdetail_data[x].id = PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2][GUI_achievementdetail_L3].achievement[i]
						GUI_achievementdetail_data[x].name = AchievementIDTable[ PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2][GUI_achievementdetail_L3].achievement[i] ]
						GUI_achievementdetail_data[x].color = 1
						x = x + 1
					end
				end
				GUI_achievementdetail_Count = x - 1
			elseif GUI_achievementdetail_Select == 3 then
				for i = 1, #PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2][GUI_achievementdetail_L3].achievement do
					if not SCAN[GUIselectedGUID].PVEDATA.achievementID or not SCAN[GUIselectedGUID].PVEDATA.achievementID[ PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2][GUI_achievementdetail_L3].achievement[i] ] then
						GUI_achievementdetail_data[x] = {}
						GUI_achievementdetail_data[x].id = PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2][GUI_achievementdetail_L3].achievement[i]
						GUI_achievementdetail_data[x].name = AchievementIDTable[ PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2][GUI_achievementdetail_L3].achievement[i] ]
						x = x + 1
					end
				end
				GUI_achievementdetail_Count = x - 1
			end

		elseif GUI_achievementdetail_L2 then

			if GUI_achievementdetail_Select == 1 then
				if not PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2].achievement then
					for k = 1, #PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2] do
						if PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2][k].achievement then
							for i = 1, #PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2][k].achievement do
								GUI_achievementdetail_data[x] = {}
								GUI_achievementdetail_data[x].id = PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2][k].achievement[i]
								GUI_achievementdetail_data[x].name = AchievementIDTable[ PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2][k].achievement[i] ]
								if SCAN[GUIselectedGUID].PVEDATA.achievementID and SCAN[GUIselectedGUID].PVEDATA.achievementID[ PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2][k].achievement[i] ] then
									GUI_achievementdetail_data[x].color = 1
								end
								x = x + 1
							end
						end
					end
				else
					for i = 1, #PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2].achievement do
						GUI_achievementdetail_data[x] = {}
						GUI_achievementdetail_data[x].id = PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2].achievement[i]
						GUI_achievementdetail_data[x].name = AchievementIDTable[ PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2].achievement[i] ]
						if SCAN[GUIselectedGUID].PVEDATA.achievementID and SCAN[GUIselectedGUID].PVEDATA.achievementID[ PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2].achievement[i] ] then
							GUI_achievementdetail_data[x].color = 1
						end
						x = x + 1
					end
				end
				GUI_achievementdetail_Count = x - 1
			elseif GUI_achievementdetail_Select == 2 then
				if not PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2].achievement then
					for k = 1, #PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2] do
						if PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2][k].achievement then
							for i = 1, #PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2][k].achievement do
								if SCAN[GUIselectedGUID].PVEDATA.achievementID and SCAN[GUIselectedGUID].PVEDATA.achievementID[ PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2][k].achievement[i] ] then
									GUI_achievementdetail_data[x] = {}
									GUI_achievementdetail_data[x].id = PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2][k].achievement[i]
									GUI_achievementdetail_data[x].name = AchievementIDTable[ PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2][k].achievement[i] ]
									GUI_achievementdetail_data[x].color = 1
									x = x + 1
								end
							end
						end
					end
				else
					for i = 1, #PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2].achievement do
						if SCAN[GUIselectedGUID].PVEDATA.achievementID and SCAN[GUIselectedGUID].PVEDATA.achievementID[ PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2].achievement[i] ] then
							GUI_achievementdetail_data[x] = {}
							GUI_achievementdetail_data[x].id = PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2].achievement[i]
							GUI_achievementdetail_data[x].name = AchievementIDTable[ PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2].achievement[i] ]
							GUI_achievementdetail_data[x].color = 1
							x = x + 1
						end
					end
				end
				GUI_achievementdetail_Count = x - 1
			elseif GUI_achievementdetail_Select == 3 then
				if not PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2].achievement then
					for k = 1, #PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2] do
						if PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2][k].achievement then
							for i = 1, #PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2][k].achievement do
								if not SCAN[GUIselectedGUID].PVEDATA.achievementID or not SCAN[GUIselectedGUID].PVEDATA.achievementID[ PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2][k].achievement[i] ] then
									GUI_achievementdetail_data[x] = {}
									GUI_achievementdetail_data[x].id = PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2][k].achievement[i]
									GUI_achievementdetail_data[x].name = AchievementIDTable[ PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2][k].achievement[i] ]
									x = x + 1
								end
							end
						end
					end
				else
					for i = 1, #PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2].achievement do
						if not SCAN[GUIselectedGUID].PVEDATA.achievementID or not SCAN[GUIselectedGUID].PVEDATA.achievementID[ PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2].achievement[i] ] then
							GUI_achievementdetail_data[x] = {}
							GUI_achievementdetail_data[x].id = PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2].achievement[i]
							GUI_achievementdetail_data[x].name = AchievementIDTable[ PVEDUMP[GUI_achievementdetail_L1][GUI_achievementdetail_L2].achievement[i] ]
							x = x + 1
						end
					end
				end
				GUI_achievementdetail_Count = x - 1
			end

		else

			if GUI_achievementdetail_Select == 1 then
				if not PVEDUMP[GUI_achievementdetail_L1].achievement then
					for j = 1, #PVEDUMP[GUI_achievementdetail_L1] do
						if PVEDUMP[GUI_achievementdetail_L1][j].achievement then
							for i = 1, #PVEDUMP[GUI_achievementdetail_L1][j].achievement do
								GUI_achievementdetail_data[x] = {}
								GUI_achievementdetail_data[x].id = PVEDUMP[GUI_achievementdetail_L1][j].achievement[i]
								GUI_achievementdetail_data[x].name = AchievementIDTable[ PVEDUMP[GUI_achievementdetail_L1][j].achievement[i] ]
								if SCAN[GUIselectedGUID].PVEDATA.achievementID and SCAN[GUIselectedGUID].PVEDATA.achievementID[ PVEDUMP[GUI_achievementdetail_L1][j].achievement[i] ] then
									GUI_achievementdetail_data[x].color = 1
								end
								x = x + 1
							end
						end
						for k = 1, #PVEDUMP[GUI_achievementdetail_L1][j] do
							if PVEDUMP[GUI_achievementdetail_L1][j][k].achievement then
								for i = 1, #PVEDUMP[GUI_achievementdetail_L1][j][k].achievement do
									GUI_achievementdetail_data[x] = {}
									GUI_achievementdetail_data[x].id = PVEDUMP[GUI_achievementdetail_L1][j][k].achievement[i]
									GUI_achievementdetail_data[x].name = AchievementIDTable[ PVEDUMP[GUI_achievementdetail_L1][j][k].achievement[i] ]
									if SCAN[GUIselectedGUID].PVEDATA.achievementID and SCAN[GUIselectedGUID].PVEDATA.achievementID[ PVEDUMP[GUI_achievementdetail_L1][j][k].achievement[i] ] then
										GUI_achievementdetail_data[x].color = 1
									end
									x = x + 1
								end
							end
						end
					end
				else
					for i = 1, #PVEDUMP[GUI_achievementdetail_L1].achievement do
						GUI_achievementdetail_data[x] = {}
						GUI_achievementdetail_data[x].id = PVEDUMP[GUI_achievementdetail_L1].achievement[i]
						GUI_achievementdetail_data[x].name = AchievementIDTable[ PVEDUMP[GUI_achievementdetail_L1].achievement[i] ]
						if SCAN[GUIselectedGUID].PVEDATA.achievementID and SCAN[GUIselectedGUID].PVEDATA.achievementID[ PVEDUMP[GUI_achievementdetail_L1].achievement[i] ] then
							GUI_achievementdetail_data[x].color = 1
						end
						x = x + 1
					end
				end
				GUI_achievementdetail_Count = x - 1
			elseif GUI_achievementdetail_Select == 2 then
				if not PVEDUMP[GUI_achievementdetail_L1].achievement then
					for j = 1, #PVEDUMP[GUI_achievementdetail_L1] do
						if PVEDUMP[GUI_achievementdetail_L1][j].achievement then
							for i = 1, #PVEDUMP[GUI_achievementdetail_L1][j].achievement do
								if SCAN[GUIselectedGUID].PVEDATA.achievementID and SCAN[GUIselectedGUID].PVEDATA.achievementID[ PVEDUMP[GUI_achievementdetail_L1][j].achievement[i] ] then
									GUI_achievementdetail_data[x] = {}
									GUI_achievementdetail_data[x].id = PVEDUMP[GUI_achievementdetail_L1][j].achievement[i]
									GUI_achievementdetail_data[x].name = AchievementIDTable[ PVEDUMP[GUI_achievementdetail_L1][j].achievement[i] ]
									GUI_achievementdetail_data[x].color = 1
									x = x + 1
								end
							end
						end
						for k = 1, #PVEDUMP[GUI_achievementdetail_L1][j] do
							if PVEDUMP[GUI_achievementdetail_L1][j][k].achievement then
								for i = 1, #PVEDUMP[GUI_achievementdetail_L1][j][k].achievement do
									if SCAN[GUIselectedGUID].PVEDATA.achievementID and SCAN[GUIselectedGUID].PVEDATA.achievementID[ PVEDUMP[GUI_achievementdetail_L1][j][k].achievement[i] ] then
										GUI_achievementdetail_data[x] = {}
										GUI_achievementdetail_data[x].id = PVEDUMP[GUI_achievementdetail_L1][j][k].achievement[i]
										GUI_achievementdetail_data[x].name = AchievementIDTable[ PVEDUMP[GUI_achievementdetail_L1][j][k].achievement[i] ]
										GUI_achievementdetail_data[x].color = 1
										x = x + 1
									end
								end
							end
						end
					end
				else
					for i = 1, #PVEDUMP[GUI_achievementdetail_L1].achievement do
						if SCAN[GUIselectedGUID].PVEDATA.achievementID and SCAN[GUIselectedGUID].PVEDATA.achievementID[ PVEDUMP[GUI_achievementdetail_L1].achievement[i] ] then
							GUI_achievementdetail_data[x] = {}
							GUI_achievementdetail_data[x].id = PVEDUMP[GUI_achievementdetail_L1].achievement[i]
							GUI_achievementdetail_data[x].name = AchievementIDTable[ PVEDUMP[GUI_achievementdetail_L1].achievement[i] ]
							GUI_achievementdetail_data[x].color = 1
							x = x + 1
						end
					end
				end
				GUI_achievementdetail_Count = x - 1
			elseif GUI_achievementdetail_Select == 3 then
				if not PVEDUMP[GUI_achievementdetail_L1].achievement then
					for j = 1, #PVEDUMP[GUI_achievementdetail_L1] do
						if PVEDUMP[GUI_achievementdetail_L1][j].achievement then
							for i = 1, #PVEDUMP[GUI_achievementdetail_L1][j].achievement do
								if not SCAN[GUIselectedGUID].PVEDATA.achievementID or not SCAN[GUIselectedGUID].PVEDATA.achievementID[ PVEDUMP[GUI_achievementdetail_L1][j].achievement[i] ] then
									GUI_achievementdetail_data[x] = {}
									GUI_achievementdetail_data[x].id = PVEDUMP[GUI_achievementdetail_L1][j].achievement[i]
									GUI_achievementdetail_data[x].name = AchievementIDTable[ PVEDUMP[GUI_achievementdetail_L1][j].achievement[i] ]
									x = x + 1
								end
							end
						end
						for k = 1, #PVEDUMP[GUI_achievementdetail_L1][j] do
							if PVEDUMP[GUI_achievementdetail_L1][j][k].achievement then
								for i = 1, #PVEDUMP[GUI_achievementdetail_L1][j][k].achievement do
									if not SCAN[GUIselectedGUID].PVEDATA.achievementID or not SCAN[GUIselectedGUID].PVEDATA.achievementID[ PVEDUMP[GUI_achievementdetail_L1][j][k].achievement[i] ] then
										GUI_achievementdetail_data[x] = {}
										GUI_achievementdetail_data[x].id = PVEDUMP[GUI_achievementdetail_L1][j][k].achievement[i]
										GUI_achievementdetail_data[x].name = AchievementIDTable[ PVEDUMP[GUI_achievementdetail_L1][j][k].achievement[i] ]
										x = x + 1
									end
								end
							end
						end
					end
				else
					for i = 1, #PVEDUMP[GUI_achievementdetail_L1].achievement do
						if not SCAN[GUIselectedGUID].PVEDATA.achievementID or not SCAN[GUIselectedGUID].PVEDATA.achievementID[ PVEDUMP[GUI_achievementdetail_L1].achievement[i] ] then
							GUI_achievementdetail_data[x] = {}
							GUI_achievementdetail_data[x].id = PVEDUMP[GUI_achievementdetail_L1].achievement[i]
							GUI_achievementdetail_data[x].name = AchievementIDTable[ PVEDUMP[GUI_achievementdetail_L1].achievement[i] ]
							x = x + 1
						end
					end
				end
				GUI_achievementdetail_Count = x - 1
			end

		end
		-- PvE END -------------------------
	else
		-- PvP START -----------------------
		table_wipe(GUI_achievementdetail_data)

		local x = 1
		if GUI_PvPachievementdetail_L3 then
			if GUI_achievementdetail_Select == 1 then
				for i = 1, #PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][GUI_PvPachievementdetail_L3].achievement do
					GUI_achievementdetail_data[x] = {}
					local valID
					if PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][GUI_PvPachievementdetail_L3].achievement[i].id then
						valID = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][GUI_PvPachievementdetail_L3].achievement[i].id
					elseif PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][GUI_PvPachievementdetail_L3].achievement[i].Aid then
						if SCAN[GUIselectedGUID].faction == "Horde" then
							valID = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][GUI_PvPachievementdetail_L3].achievement[i].Hid
						else
							valID = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][GUI_PvPachievementdetail_L3].achievement[i].Aid
						end
					end
					GUI_achievementdetail_data[x].id = valID
					GUI_achievementdetail_data[x].name = AchievementIDTable[ valID ]
					if SCAN[GUIselectedGUID].PVPDATA.achievementID and SCAN[GUIselectedGUID].PVPDATA.achievementID[ valID ] then
						GUI_achievementdetail_data[x].color = 1
					end
					x = x + 1
				end
				GUI_achievementdetail_Count = x - 1
			elseif GUI_achievementdetail_Select == 2 then
				for i = 1, #PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][GUI_PvPachievementdetail_L3].achievement do

					local valID
					if PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][GUI_PvPachievementdetail_L3].achievement[i].id then
						valID = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][GUI_PvPachievementdetail_L3].achievement[i].id
					elseif PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][GUI_PvPachievementdetail_L3].achievement[i].Aid then
						if SCAN[GUIselectedGUID].faction == "Horde" then
							valID = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][GUI_PvPachievementdetail_L3].achievement[i].Hid
						else
							valID = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][GUI_PvPachievementdetail_L3].achievement[i].Aid
						end
					end

					if SCAN[GUIselectedGUID].PVPDATA.achievementID and SCAN[GUIselectedGUID].PVPDATA.achievementID[ valID ] then
						GUI_achievementdetail_data[x] = {}

						GUI_achievementdetail_data[x].id = valID
						GUI_achievementdetail_data[x].name = AchievementIDTable[ valID ]
						GUI_achievementdetail_data[x].color = 1
						x = x + 1
					end
				end
				GUI_achievementdetail_Count = x - 1
			elseif GUI_achievementdetail_Select == 3 then
				for i = 1, #PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][GUI_PvPachievementdetail_L3].achievement do

					local valID
					if PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][GUI_PvPachievementdetail_L3].achievement[i].id then
						valID = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][GUI_PvPachievementdetail_L3].achievement[i].id
					elseif PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][GUI_PvPachievementdetail_L3].achievement[i].Aid then
						if SCAN[GUIselectedGUID].faction == "Horde" then
							valID = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][GUI_PvPachievementdetail_L3].achievement[i].Hid
						else
							valID = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][GUI_PvPachievementdetail_L3].achievement[i].Aid
						end
					end

					if not SCAN[GUIselectedGUID].PVPDATA.achievementID or not SCAN[GUIselectedGUID].PVPDATA.achievementID[ valID ] then
						GUI_achievementdetail_data[x] = {}

						GUI_achievementdetail_data[x].id = valID
						GUI_achievementdetail_data[x].name = AchievementIDTable[ valID ]
						x = x + 1
					end
				end
				GUI_achievementdetail_Count = x - 1
			end

		elseif GUI_PvPachievementdetail_L2 then

			if GUI_achievementdetail_Select == 1 then
				if not PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2].achievement then
					for k = 1, #PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2] do
						if PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][k].achievement then
							for i = 1, #PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][k].achievement do
								GUI_achievementdetail_data[x] = {}
								local valID
								if PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][k].achievement[i].id then
									valID = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][k].achievement[i].id
								elseif PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][k].achievement[i].Aid then
									if SCAN[GUIselectedGUID].faction == "Horde" then
										valID = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][k].achievement[i].Hid
									else
										valID = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][k].achievement[i].Aid
									end
								end
								GUI_achievementdetail_data[x].id = valID
								GUI_achievementdetail_data[x].name = AchievementIDTable[ valID ]
								if SCAN[GUIselectedGUID].PVPDATA.achievementID and SCAN[GUIselectedGUID].PVPDATA.achievementID[ valID ] then
									GUI_achievementdetail_data[x].color = 1
								end
								x = x + 1
							end
						end
					end
				else
					for i = 1, #PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2].achievement do
						GUI_achievementdetail_data[x] = {}
						local valID
						if PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2].achievement[i].id then
							valID = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2].achievement[i].id
						elseif PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2].achievement[i].Aid then
							if SCAN[GUIselectedGUID].faction == "Horde" then
								valID = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2].achievement[i].Hid
							else
								valID = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2].achievement[i].Aid
							end
						end
						GUI_achievementdetail_data[x].id = valID
						GUI_achievementdetail_data[x].name = AchievementIDTable[ valID ]
						if SCAN[GUIselectedGUID].PVPDATA.achievementID and SCAN[GUIselectedGUID].PVPDATA.achievementID[ valID ] then
							GUI_achievementdetail_data[x].color = 1
						end
						x = x + 1
					end
				end
				GUI_achievementdetail_Count = x - 1
			elseif GUI_achievementdetail_Select == 2 then
				if not PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2].achievement then
					for k = 1, #PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2] do
						if PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][k].achievement then
							for i = 1, #PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][k].achievement do

								local valID
								if PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][k].achievement[i].id then
									valID = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][k].achievement[i].id
								elseif PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][k].achievement[i].Aid then
									if SCAN[GUIselectedGUID].faction == "Horde" then
										valID = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][k].achievement[i].Hid
									else
										valID = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][k].achievement[i].Aid
									end
								end

								if SCAN[GUIselectedGUID].PVPDATA.achievementID and SCAN[GUIselectedGUID].PVPDATA.achievementID[ valID ] then
									GUI_achievementdetail_data[x] = {}

									GUI_achievementdetail_data[x].id = valID
									GUI_achievementdetail_data[x].name = AchievementIDTable[ valID ]
									GUI_achievementdetail_data[x].color = 1
									x = x + 1
								end
							end
						end
					end
				else
					for i = 1, #PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2].achievement do

						local valID
						if PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2].achievement[i].id then
							valID = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2].achievement[i].id
						elseif PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2].achievement[i].Aid then
							if SCAN[GUIselectedGUID].faction == "Horde" then
								valID = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2].achievement[i].Hid
							else
								valID = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2].achievement[i].Aid
							end
						end

						if SCAN[GUIselectedGUID].PVPDATA.achievementID and SCAN[GUIselectedGUID].PVPDATA.achievementID[ valID ] then
							GUI_achievementdetail_data[x] = {}

							GUI_achievementdetail_data[x].id = valID
							GUI_achievementdetail_data[x].name = AchievementIDTable[ valID ]
							GUI_achievementdetail_data[x].color = 1
							x = x + 1
						end
					end
				end
				GUI_achievementdetail_Count = x - 1
			elseif GUI_achievementdetail_Select == 3 then
				if not PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2].achievement then
					for k = 1, #PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2] do
						if PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][k].achievement then
							for i = 1, #PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][k].achievement do

								local valID
								if PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][k].achievement[i].id then
									valID = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][k].achievement[i].id
								elseif PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][k].achievement[i].Aid then
									if SCAN[GUIselectedGUID].faction == "Horde" then
										valID = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][k].achievement[i].Hid
									else
										valID = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2][k].achievement[i].Aid
									end
								end

								if not SCAN[GUIselectedGUID].PVPDATA.achievementID or not SCAN[GUIselectedGUID].PVPDATA.achievementID[ valID ] then
									GUI_achievementdetail_data[x] = {}

									GUI_achievementdetail_data[x].id = valID
									GUI_achievementdetail_data[x].name = AchievementIDTable[ valID ]
									x = x + 1
								end
							end
						end
					end
				else
					for i = 1, #PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2].achievement do

						local valID
						if PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2].achievement[i].id then
							valID = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2].achievement[i].id
						elseif PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2].achievement[i].Aid then
							if SCAN[GUIselectedGUID].faction == "Horde" then
								valID = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2].achievement[i].Hid
							else
								valID = PVPDUMP[GUI_PvPachievementdetail_L1][GUI_PvPachievementdetail_L2].achievement[i].Aid
							end
						end

						if not SCAN[GUIselectedGUID].PVPDATA.achievementID or not SCAN[GUIselectedGUID].PVPDATA.achievementID[ valID ] then
							GUI_achievementdetail_data[x] = {}

							GUI_achievementdetail_data[x].id = valID
							GUI_achievementdetail_data[x].name = AchievementIDTable[ valID ]
							x = x + 1
						end
					end
				end
				GUI_achievementdetail_Count = x - 1
			end

		else

			if GUI_achievementdetail_Select == 1 then
				if not PVPDUMP[GUI_PvPachievementdetail_L1].achievement then
					for j = 1, #PVPDUMP[GUI_PvPachievementdetail_L1] do
						if PVPDUMP[GUI_PvPachievementdetail_L1][j].achievement then
							for i = 1, #PVPDUMP[GUI_PvPachievementdetail_L1][j].achievement do
								GUI_achievementdetail_data[x] = {}
								local valID
								if PVPDUMP[GUI_PvPachievementdetail_L1][j].achievement[i].id then
									valID = PVPDUMP[GUI_PvPachievementdetail_L1][j].achievement[i].id
								elseif PVPDUMP[GUI_PvPachievementdetail_L1][j].achievement[i].Aid then
									if SCAN[GUIselectedGUID].faction == "Horde" then
										valID = PVPDUMP[GUI_PvPachievementdetail_L1][j].achievement[i].Hid
									else
										valID = PVPDUMP[GUI_PvPachievementdetail_L1][j].achievement[i].Aid
									end
								end
								GUI_achievementdetail_data[x].id = valID
								GUI_achievementdetail_data[x].name = AchievementIDTable[ valID ]
								if SCAN[GUIselectedGUID].PVPDATA.achievementID and SCAN[GUIselectedGUID].PVPDATA.achievementID[ valID ] then
									GUI_achievementdetail_data[x].color = 1
								end
								x = x + 1
							end
						end
						for k = 1, #PVPDUMP[GUI_PvPachievementdetail_L1][j] do
							if PVPDUMP[GUI_PvPachievementdetail_L1][j][k].achievement then
								for i = 1, #PVPDUMP[GUI_PvPachievementdetail_L1][j][k].achievement do
									GUI_achievementdetail_data[x] = {}
									local valID
									if PVPDUMP[GUI_PvPachievementdetail_L1][j][k].achievement[i].id then
										valID = PVPDUMP[GUI_PvPachievementdetail_L1][j][k].achievement[i].id
									elseif PVPDUMP[GUI_PvPachievementdetail_L1][j][k].achievement[i].Aid then
										if SCAN[GUIselectedGUID].faction == "Horde" then
											valID = PVPDUMP[GUI_PvPachievementdetail_L1][j][k].achievement[i].Hid
										else
											valID = PVPDUMP[GUI_PvPachievementdetail_L1][j][k].achievement[i].Aid
										end
									end
									GUI_achievementdetail_data[x].id = valID
									GUI_achievementdetail_data[x].name = AchievementIDTable[ valID ]
									if SCAN[GUIselectedGUID].PVPDATA.achievementID and SCAN[GUIselectedGUID].PVPDATA.achievementID[ valID ] then
										GUI_achievementdetail_data[x].color = 1
									end
									x = x + 1
								end
							end
						end
					end
				else
					for i = 1, #PVPDUMP[GUI_PvPachievementdetail_L1].achievement do
						GUI_achievementdetail_data[x] = {}
						local valID
						if PVPDUMP[GUI_PvPachievementdetail_L1].achievement[i].id then
							valID = PVPDUMP[GUI_PvPachievementdetail_L1].achievement[i].id
						elseif PVPDUMP[GUI_PvPachievementdetail_L1].achievement[i].Aid then
							if SCAN[GUIselectedGUID].faction == "Horde" then
								valID = PVPDUMP[GUI_PvPachievementdetail_L1].achievement[i].Hid
							else
								valID = PVPDUMP[GUI_PvPachievementdetail_L1].achievement[i].Aid
							end
						end
						GUI_achievementdetail_data[x].id = valID
						GUI_achievementdetail_data[x].name = AchievementIDTable[ valID ]
						if SCAN[GUIselectedGUID].PVPDATA.achievementID and SCAN[GUIselectedGUID].PVPDATA.achievementID[ valID ] then
							GUI_achievementdetail_data[x].color = 1
						end
						x = x + 1
					end
				end
				GUI_achievementdetail_Count = x - 1
			elseif GUI_achievementdetail_Select == 2 then
				if not PVPDUMP[GUI_PvPachievementdetail_L1].achievement then
					for j = 1, #PVPDUMP[GUI_PvPachievementdetail_L1] do
						if PVPDUMP[GUI_PvPachievementdetail_L1][j].achievement then
							for i = 1, #PVPDUMP[GUI_PvPachievementdetail_L1][j].achievement do

								local valID
								if PVPDUMP[GUI_PvPachievementdetail_L1][j].achievement[i].id then
									valID = PVPDUMP[GUI_PvPachievementdetail_L1][j].achievement[i].id
								elseif PVPDUMP[GUI_PvPachievementdetail_L1][j].achievement[i].Aid then
									if SCAN[GUIselectedGUID].faction == "Horde" then
										valID = PVPDUMP[GUI_PvPachievementdetail_L1][j].achievement[i].Hid
									else
										valID = PVPDUMP[GUI_PvPachievementdetail_L1][j].achievement[i].Aid
									end
								end

								if SCAN[GUIselectedGUID].PVPDATA.achievementID and SCAN[GUIselectedGUID].PVPDATA.achievementID[ valID ] then
									GUI_achievementdetail_data[x] = {}

									GUI_achievementdetail_data[x].id = valID
									GUI_achievementdetail_data[x].name = AchievementIDTable[ valID ]
									GUI_achievementdetail_data[x].color = 1
									x = x + 1
								end
							end
						end
						for k = 1, #PVPDUMP[GUI_PvPachievementdetail_L1][j] do
							if PVPDUMP[GUI_PvPachievementdetail_L1][j][k].achievement then
								for i = 1, #PVPDUMP[GUI_PvPachievementdetail_L1][j][k].achievement do

									local valID
									if PVPDUMP[GUI_PvPachievementdetail_L1][j][k].achievement[i].id then
										valID = PVPDUMP[GUI_PvPachievementdetail_L1][j][k].achievement[i].id
									elseif PVPDUMP[GUI_PvPachievementdetail_L1][j][k].achievement[i].Aid then
										if SCAN[GUIselectedGUID].faction == "Horde" then
											valID = PVPDUMP[GUI_PvPachievementdetail_L1][j][k].achievement[i].Hid
										else
											valID = PVPDUMP[GUI_PvPachievementdetail_L1][j][k].achievement[i].Aid
										end
									end

									if SCAN[GUIselectedGUID].PVPDATA.achievementID and SCAN[GUIselectedGUID].PVPDATA.achievementID[ valID ] then
										GUI_achievementdetail_data[x] = {}

										GUI_achievementdetail_data[x].id = valID
										GUI_achievementdetail_data[x].name = AchievementIDTable[ valID ]
										GUI_achievementdetail_data[x].color = 1
										x = x + 1
									end
								end
							end
						end
					end
				else
					for i = 1, #PVPDUMP[GUI_PvPachievementdetail_L1].achievement do

						local valID
						if PVPDUMP[GUI_PvPachievementdetail_L1].achievement[i].id then
							valID = PVPDUMP[GUI_PvPachievementdetail_L1].achievement[i].id
						elseif PVPDUMP[GUI_PvPachievementdetail_L1].achievement[i].Aid then
							if SCAN[GUIselectedGUID].faction == "Horde" then
								valID = PVPDUMP[GUI_PvPachievementdetail_L1].achievement[i].Hid
							else
								valID = PVPDUMP[GUI_PvPachievementdetail_L1].achievement[i].Aid
							end
						end

						if SCAN[GUIselectedGUID].PVPDATA.achievementID and SCAN[GUIselectedGUID].PVPDATA.achievementID[ valID ] then
							GUI_achievementdetail_data[x] = {}

							GUI_achievementdetail_data[x].id = valID
							GUI_achievementdetail_data[x].name = AchievementIDTable[ valID ]
							GUI_achievementdetail_data[x].color = 1
							x = x + 1
						end
					end
				end
				GUI_achievementdetail_Count = x - 1
			elseif GUI_achievementdetail_Select == 3 then
				if not PVPDUMP[GUI_PvPachievementdetail_L1].achievement then
					for j = 1, #PVPDUMP[GUI_PvPachievementdetail_L1] do
						if PVPDUMP[GUI_PvPachievementdetail_L1][j].achievement then
							for i = 1, #PVPDUMP[GUI_PvPachievementdetail_L1][j].achievement do

								local valID
								if PVPDUMP[GUI_PvPachievementdetail_L1][j].achievement[i].id then
									valID = PVPDUMP[GUI_PvPachievementdetail_L1][j].achievement[i].id
								elseif PVPDUMP[GUI_PvPachievementdetail_L1][j].achievement[i].Aid then
									if SCAN[GUIselectedGUID].faction == "Horde" then
										valID = PVPDUMP[GUI_PvPachievementdetail_L1][j].achievement[i].Hid
									else
										valID = PVPDUMP[GUI_PvPachievementdetail_L1][j].achievement[i].Aid
									end
								end

								if not SCAN[GUIselectedGUID].PVPDATA.achievementID or not SCAN[GUIselectedGUID].PVPDATA.achievementID[ valID ] then
									GUI_achievementdetail_data[x] = {}

									GUI_achievementdetail_data[x].id = valID
									GUI_achievementdetail_data[x].name = AchievementIDTable[ valID ]
									x = x + 1
								end
							end
						end
						for k = 1, #PVPDUMP[GUI_PvPachievementdetail_L1][j] do
							if PVPDUMP[GUI_PvPachievementdetail_L1][j][k].achievement then
								for i = 1, #PVPDUMP[GUI_PvPachievementdetail_L1][j][k].achievement do

									local valID
									if PVPDUMP[GUI_PvPachievementdetail_L1][j][k].achievement[i].id then
										valID = PVPDUMP[GUI_PvPachievementdetail_L1][j][k].achievement[i].id
									elseif PVPDUMP[GUI_PvPachievementdetail_L1][j][k].achievement[i].Aid then
										if SCAN[GUIselectedGUID].faction == "Horde" then
											valID = PVPDUMP[GUI_PvPachievementdetail_L1][j][k].achievement[i].Hid
										else
											valID = PVPDUMP[GUI_PvPachievementdetail_L1][j][k].achievement[i].Aid
										end
									end

									if not SCAN[GUIselectedGUID].PVPDATA.achievementID or not SCAN[GUIselectedGUID].PVPDATA.achievementID[ valID ] then
										GUI_achievementdetail_data[x] = {}

										GUI_achievementdetail_data[x].id = valID
										GUI_achievementdetail_data[x].name = AchievementIDTable[ valID ]
										x = x + 1
									end
								end
							end
						end
					end
				else
					for i = 1, #PVPDUMP[GUI_PvPachievementdetail_L1].achievement do

						local valID
						if PVPDUMP[GUI_PvPachievementdetail_L1].achievement[i].id then
							valID = PVPDUMP[GUI_PvPachievementdetail_L1].achievement[i].id
						elseif PVPDUMP[GUI_PvPachievementdetail_L1].achievement[i].Aid then
							if SCAN[GUIselectedGUID].faction == "Horde" then
								valID = PVPDUMP[GUI_PvPachievementdetail_L1].achievement[i].Hid
							else
								valID = PVPDUMP[GUI_PvPachievementdetail_L1].achievement[i].Aid
							end
						end

						if not SCAN[GUIselectedGUID].PVPDATA.achievementID or not SCAN[GUIselectedGUID].PVPDATA.achievementID[ valID ] then
							GUI_achievementdetail_data[x] = {}

							GUI_achievementdetail_data[x].id = valID
							GUI_achievementdetail_data[x].name = AchievementIDTable[ valID ]
							x = x + 1
						end
					end
				end
				GUI_achievementdetail_Count = x - 1
			end

		end
		-- PvP END -------------------------
	end

	local sortfunc
	if GUI_achievementdetail_SortBy == 1 then -- 1 = id
		if GUI_achievementdetail_SortOrder == 1 then
			sortfunc = function(a, b)
				if a.id < b.id then
					return true
				end
			end
		else
			sortfunc = function(a, b)
				if a.id > b.id then
					return true
				end
			end
		end
	elseif GUI_achievementdetail_SortBy == 2 then -- 2 = name
		if GUI_achievementdetail_SortOrder == 1 then
			sortfunc = function(a, b)
				if a.name < b.name then
					return true
				end
			end
		else
			sortfunc = function(a, b)
				if a.name > b.name then
					return true
				end
			end
		end
	end

	table_sort(GUI_achievementdetail_data, sortfunc)
end

function CharScanner:AchievementDetail_GUISort(sortBy)
	if GUI_achievementdetail_SortBy == sortBy then
		if GUI_achievementdetail_SortOrder == 1 then
			GUI_achievementdetail_SortOrder = 0
		else
			GUI_achievementdetail_SortOrder = 1
		end
	else
		GUI_achievementdetail_SortOrder = 1
	end
	GUI_achievementdetail_SortBy = sortBy
	CharScanner:AchievementDetail_SortButtonUpdate()
	CharScanner:AchievementDetail_GUICreate(AchievementDetailFrameMode)
	CharScanner:AchievementDetail_FillList()
end

function CharScanner:AchievementDetail_SortButtonUpdate()
	local arrow
	for i=1, 2 do
		if i == 1 then -- 1 = id
			arrow = GVAR.AchievementDetailFrame.Sort.Id.Arrow
		elseif i == 2 then -- 2 = name
			arrow = GVAR.AchievementDetailFrame.Sort.Name.Arrow
		end
		if GUI_achievementdetail_SortBy == i then
			if GUI_achievementdetail_SortOrder == 1 then arrow:SetTexCoord(unpack(Textures.SortArrow.coords1)) end
			if GUI_achievementdetail_SortOrder == 0 then arrow:SetTexCoord(unpack(Textures.SortArrow.coords2)) end
		else
			arrow:SetTexCoord(0, 0, 0, 0)
		end
	end
end

function CharScanner:AchievementDetail_OnVerticalScroll(value)
	GVAR.AchievementDetailFrame.ScrollFrame.ScrollBar:SetValue(value)
	GVAR.AchievementDetailFrame.ScrollFrame.offset = math_floor((value / sizeGeneral.BarHeight) + 0.5)
	CharScanner:AchievementDetail_FillList()
end

function CharScanner:AchievementDetail_FillList()
	local scrollFrame = GVAR.AchievementDetailFrame.ScrollFrame
	local scrollFrameHeight = 0
	local numToDisplay = GUI_achievementdetail_curRows
	local valueStep = 12

	if GUI_achievementdetail_Count < numToDisplay then
		numToDisplay = GUI_achievementdetail_Count
	end

	if GUI_achievementdetail_Count > 0 then
		scrollFrameHeight = (GUI_achievementdetail_Count - numToDisplay) * valueStep
		if scrollFrameHeight < 0 then
			scrollFrameHeight = 0
		end
	end
	scrollFrame.ScrollBar:SetMinMaxValues(0, scrollFrameHeight)
	scrollFrame.ScrollBar:SetValueStep(valueStep)

	if GUI_achievementdetail_Count <= GUI_achievementdetail_curRows then
		ScrollFrameDisable(scrollFrame)
	else
		ScrollFrameEnable(scrollFrame)
	end

	if scrollFrame.ScrollBar:GetValue() == 0 then
		scrollFrame.ScrollBarScrollUpButton:Disable()
	else
		scrollFrame.ScrollBarScrollUpButton:Enable()
	end
	if ((scrollFrame.ScrollBar:GetValue() - scrollFrameHeight) == 0) then
		scrollFrame.ScrollBarScrollDownButton:Disable()
	else
		scrollFrame.ScrollBarScrollDownButton:Enable()
	end

	for i=1, GUI_achievementdetail_maxRows do
		GVAR.AchievementDetailFrame.ScrollFrameButton[i]:Hide()
	end

	local j = 1
	local sliderPos = scrollFrame.offset
	for i=sliderPos+1, sliderPos+numToDisplay do
		if sliderPos < GUI_achievementdetail_Count then
			GVAR.AchievementDetailFrame.ScrollFrameButton[j].Id:SetText(GUI_achievementdetail_data[i].id)
			GVAR.AchievementDetailFrame.ScrollFrameButton[j].Name:SetText(GUI_achievementdetail_data[i].name)
			if GUI_achievementdetail_data[i].color then
				GVAR.AchievementDetailFrame.ScrollFrameButton[j].Name:SetTextColor(color.Green.rgba[1], color.Green.rgba[2], color.Green.rgba[3], color.Green.rgba[4])
			else
				GVAR.AchievementDetailFrame.ScrollFrameButton[j].Name:SetTextColor(color.Red.rgba[1], color.Red.rgba[2], color.Red.rgba[3], color.Red.rgba[4])
			end
			GVAR.AchievementDetailFrame.ScrollFrameButton[j].achievementID = GUI_achievementdetail_data[i].id

			GVAR.AchievementDetailFrame.ScrollFrameButton[j]:Show()
			j = j + 1
		end
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function CharScanner:PreShowSavedData(guid, switch)
	CharScanner:ClearData()
	CharScanner:ScanFrameToggle(true)
	if CompareFrameIsShown and switch then
		CharScanner:HistoryFrameToggle()
	end
	SCAN[guid].DataStep = 0
	CharScanner:ShowData(1, guid, false)
	CharScanner:ShowData(2, guid, false)
	CharScanner:ShowData(3, guid, false)
	CharScanner:ShowData(4, guid, false)
end

function CharScanner:DeleteCharacter(guid, id)
	if guid then -- delete one GUID
		NumScannedChars = NumScannedChars - 1
		if NumScannedChars == 0 then
			if GroupScanIsActive then
				CharScanner:StopGroupScan()
			end
			table_wipe(SCAN)
			GUIselectedGUID = nil
			CharScanner:DeleteButtonToggle(false)
			CharScanner:ScanFrameToggle(nil, true)
			CharScanner:AchievementDetail_HideAll()
			CharScanner:ClearData()
			CharScanner:History_Update()
		elseif NumScannedChars == 1 then
			SCAN[guid] = nil
			if guid == GUIselectedGUID then
				if id == 1 then
					CharScanner:PreShowSavedData(GVAR.HistoryFrame.ScrollFrameButton[2].guid)
				else
					CharScanner:PreShowSavedData(GVAR.HistoryFrame.ScrollFrameButton[1].guid)
				end
			end
			CharScanner:History_Update()
		elseif NumScannedChars > 1 then
			SCAN[guid] = nil
			if guid == GUIselectedGUID then
				if id == 1 then
					CharScanner:PreShowSavedData(GVAR.HistoryFrame.ScrollFrameButton[2].guid)
				else
					CharScanner:PreShowSavedData(GVAR.HistoryFrame.ScrollFrameButton[(id-1)].guid)
				end
			end
			CharScanner:History_Update()
		end
	else -- delete all GUIDs
		if GroupScanIsActive then
			CharScanner:StopGroupScan()
		end
		table_wipe(SCAN)
		GUIselectedGUID = nil
		NumScannedChars = 0
		CharScanner:DeleteButtonToggle(false)
		CharScanner:ScanFrameToggle(nil, true)
		CharScanner:AchievementDetail_HideAll()
		CharScanner:ClearData()
		CharScanner:History_Update()
	end
	CharScanner:UpdateGroupMissingMembers()
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function CharScanner:History_StatusUpdate()
	-- average scan time START
	local avgScanTime = 0
	local addThis
	local scans = 0
	for guid, value in pairs(SCAN) do
		if value.scanTime and not value.oorScan then
			addThis = nil
			if GroupTabStatus == 1 then
				addThis = true
			elseif GroupTabStatus == 2 then
				if GroupMembers[guid] then
					addThis = true
				end
			elseif GroupTabStatus == 3 then
				if not GroupMembers[guid] then
					addThis = true
				end
			end
			if addThis then
				avgScanTime = avgScanTime + value.scanTime
				scans = scans + 1
			end
		end
	end
	local text = string_format("%s %s", scans, L["|4Char:Chars;"])
	if scans > 0 then
		avgScanTime = math_floor((avgScanTime/scans) * mult4 + 0.5) / mult4
	end
	-- average scan time END
	GVAR.HistoryFrame.Status.Text:SetText(text.." - |cff"..color.Green.hex..L["Avg"].." "..avgScanTime.."|r")

	local roleTable = {}
	local addThis
	for guid, value in pairs(SCAN) do
		if value.time and not value.oorScan then -- XFIX

			addThis = nil
			if GroupTabStatus == 1 then
				addThis = true
			elseif GroupTabStatus == 2 then
				if GroupMembers[guid] then
					addThis = true
				end
			elseif GroupTabStatus == 3 then
				if not GroupMembers[guid] then
					addThis = true
				end
			end

			if addThis then

				if SCAN[guid].talentGroup1role then
					if SCAN[guid].activeTalentGroup == 1 then
						--checker START
						if not roleTable[ SCAN[guid].talentGroup1role ] then
							roleTable[ SCAN[guid].talentGroup1role ] = {}
							roleTable[ SCAN[guid].talentGroup1role ].active = 1
							roleTable[ SCAN[guid].talentGroup1role ].count = 1
						else
							roleTable[ SCAN[guid].talentGroup1role ].active = roleTable[ SCAN[guid].talentGroup1role ].active + 1
							roleTable[ SCAN[guid].talentGroup1role ].count = roleTable[ SCAN[guid].talentGroup1role ].count + 1
						end
						--checker END
					else
						--checker START
						if not roleTable[ SCAN[guid].talentGroup1role ] then
							roleTable[ SCAN[guid].talentGroup1role ] = {}
							roleTable[ SCAN[guid].talentGroup1role ].active = 0
							roleTable[ SCAN[guid].talentGroup1role ].count = 1
						else
							--roleTable[ SCAN[guid].talentGroup1role ].active = roleTable[ SCAN[guid].talentGroup1role ].active
							roleTable[ SCAN[guid].talentGroup1role ].count = roleTable[ SCAN[guid].talentGroup1role ].count + 1
						end
						--checker END
					end

					if SCAN[guid].talentGroup2role then
						if SCAN[guid].activeTalentGroup == 2 then
							--checker START
							if not roleTable[ SCAN[guid].talentGroup2role ] then
								roleTable[ SCAN[guid].talentGroup2role ] = {}
								roleTable[ SCAN[guid].talentGroup2role ].active = 1
								roleTable[ SCAN[guid].talentGroup2role ].count = 1
							else
								roleTable[ SCAN[guid].talentGroup2role ].active = roleTable[ SCAN[guid].talentGroup2role ].active + 1
								if SCAN[guid].talentGroup1role ~= SCAN[guid].talentGroup2role then
									roleTable[ SCAN[guid].talentGroup2role ].count = roleTable[ SCAN[guid].talentGroup2role ].count + 1
								end
							end
							--checker END
						else
							--checker START
							if not roleTable[ SCAN[guid].talentGroup2role ] then
								roleTable[ SCAN[guid].talentGroup2role ] = {}
								roleTable[ SCAN[guid].talentGroup2role ].active = 0
								roleTable[ SCAN[guid].talentGroup2role ].count = 1
							else
								--roleTable[ SCAN[guid].talentGroup2role ].active = roleTable[ SCAN[guid].talentGroup2role ].active
								if SCAN[guid].talentGroup1role ~= SCAN[guid].talentGroup2role then
									roleTable[ SCAN[guid].talentGroup2role ].count = roleTable[ SCAN[guid].talentGroup2role ].count + 1
								end
							end
							--checker END
						end
					end
				else
					--checker START
					if not roleTable[ SCAN[guid].mainRole ] then
						roleTable[ SCAN[guid].mainRole ] = {}
						roleTable[ SCAN[guid].mainRole ].active = 1
						roleTable[ SCAN[guid].mainRole ].count = 1
					else
						roleTable[ SCAN[guid].mainRole ].active = roleTable[ SCAN[guid].mainRole ].active + 1
						roleTable[ SCAN[guid].mainRole ].count = roleTable[ SCAN[guid].mainRole ].count + 1
					end
					--checker END
				end

			end

		end -- XFIX
	end

	for i = 1, 4 do
		if roleTable[i] then
			GVAR.HistoryFrame.Status.Role[i].Icon:SetAlpha(1)
			GVAR.HistoryFrame.Status.Role[i].Text:SetText(roleTable[i].active.." |cff"..color.Grey.hex.."("..roleTable[i].count..")|r")
			GVAR.HistoryFrame.Status.Role[i].Text:SetTextColor(color.Role[ i ].rgba[1], color.Role[ i ].rgba[2], color.Role[ i ].rgba[3], color.Role[ i ].rgba[4])
		else
			GVAR.HistoryFrame.Status.Role[i].Icon:SetAlpha(0.25)
			GVAR.HistoryFrame.Status.Role[i].Text:SetText("- (-)")
			GVAR.HistoryFrame.Status.Role[i].Text:SetTextColor(color.Grey.rgba[1], color.Grey.rgba[2], color.Grey.rgba[3], color.Grey.rgba[4])
		end
	end
	table_wipe(roleTable)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function CharScanner:ScanGroup(missing)
	if inCombat then
		GVAR.MainFrame.Status.Text:SetText(L["Scan is deactivated in combat!"])
		GVAR.MainFrame.Status.Text:SetTextColor(color.Red.rgba[1], color.Red.rgba[2], color.Red.rgba[3], color.Red.rgba[4])
		GVAR.MainFrame.Status:Show()
		return
	end

	-- ScanGroup-func-is-only-available-if-we-are-in-group,-but-a-paranoia-check-is-good! START
	local partymember = GetNumPartyMembers()
	local raidmember  = GetNumRaidMembers()
	local inParty
	local inRaid
	if partymember > 0 then
		inRaid  = false
		inParty = true
	end
	if raidmember > 0 then
		inRaid  = true
		inParty = false
	end
	if not inRaid and not inParty then return end
	-- ScanGroup-func-is-only-available-if-we-are-in-group,-but-a-paranoia-check-is-good! END

	GroupScanTimerStart = GetTime()
	GroupScanIsActive = true
	GroupScanNext = false
	CharScanner:SingleScanHide()
	CharScanner:GroupScanHide()
	CharScanner:ProgressDisplay("GROUP", true, false)

	-- group member table START
	local grpmem = {}
	if missing then
		local temp = {}
		temp = GroupMissingMembers
		local x = 1
		for guid, unit in pairs(temp) do
			grpmem[x] = {}
			grpmem[x].guid = guid
			grpmem[x].unit = unit
			x = x + 1
		end
		table_wipe(temp)
	else
		-- we build our own membertable, so we are independent from PARTY_MEMBERS_CHANGED/RAID_ROSTER_UPDATE changes
		-- raid roster changes between button click and end of groupscan are discarded!
		local x = 1
		if inParty then
			for i = 1, partymember do
				local guid = UnitGUID(unitIDparty[i])
				if guid then
					grpmem[x] = {}
					grpmem[x].guid = guid
					grpmem[x].unit = unitIDparty[i]
					x = x + 1
				end
			end
			grpmem[x] = {}
			grpmem[x].guid = playerGUID
			grpmem[x].unit = "player"
		end
		if inRaid then
			for i = 1, raidmember do
				local guid = UnitGUID(unitIDraid[i])
				if guid then
					grpmem[x] = {}
					grpmem[x].guid = guid
					grpmem[x].unit = unitIDraid[i]
					x = x + 1
				end
			end
		end
	end
	local numMember = #grpmem
	-- group member table END

	if numMember == 0 then
		CharScanner:StopGroupScan()
		return
	end

	local gsStart = GetTime()
	local currentScanNumber = 1
	GroupScanFirstGUID = nil

	local function scanNextMember()
		local guid = UnitGUID(grpmem[currentScanNumber].unit)
		if guid and guid == grpmem[currentScanNumber].guid then
			GVAR.MainFrame.Status.ProgressText:SetText(currentScanNumber.."/"..numMember)
			GVAR.MainFrame.Status.ProgressDone:SetWidth(currentScanNumber*sizeGeneral.StatusProgressBarWidth/numMember)
			CharScanner:ScanUnit(grpmem[currentScanNumber].unit)
			currentScanNumber = currentScanNumber + 1
		else
			GVAR.MainFrame.Status.ProgressText:SetText(currentScanNumber.."/"..numMember)
			GVAR.MainFrame.Status.ProgressDone:SetWidth(currentScanNumber*sizeGeneral.StatusProgressBarWidth/numMember)
			currentScanNumber = currentScanNumber + 1
		end
	end

	local function gsTimeCheck()
		if inCombat then
			CharScanner:StopGroupScan()
			return
		end
		if not inPartyOrRaid then
			CharScanner:StopGroupScan()
			return
		end
		if RestrictionTalNum >= RestrictionCountMax or RestrictionAchNum >= RestrictionCountMax then
			if currentScanNumber > numMember then
				CharScanner:StopGroupScan()
				return
			end
			local elapsedTime = GetTime() - RestrictionTime
			local throttlewidth = (sizeGeneral.StatusProgressBarWidth/RestrictionMaxTime)*elapsedTime
			if throttlewidth > sizeGeneral.StatusProgressBarWidth then
				throttlewidth = sizeGeneral.StatusProgressBarWidth
			end
			CharScanner:ProgressDisplay("GROUP", true, true)
			GVAR.MainFrame.Status.ThrottleText2:SetText( string_format(L["Please wait %.2f seconds."], (RestrictionMaxTime-elapsedTime)) )
			GVAR.MainFrame.Status.ThrottleDone:SetWidth(throttlewidth)
			if elapsedTime >= RestrictionMaxTime then
				CharScanner:ProgressDisplay("GROUP", true, false)
				RestrictionAchNum = 0
				RestrictionTalNum = 0
			end
		end
		if RestrictionTalNum < RestrictionCountMax and RestrictionAchNum < RestrictionCountMax and
		   ( (GetTime() - gsStart) >= CharScanner_Options.scanDuration or GroupScanNext ) then
			CharScanner:ProgressDisplay("GROUP", true, false)
			if currentScanNumber > numMember then
				CharScanner:StopGroupScan()
				return
			end
			-- next scan
			gsStart = GetTime()
			GroupScanNext = false
			scanNextMember()
		end
	end

	CharScanner.OnUpdateGroupScan:SetScript("OnUpdate", gsTimeCheck)
	if RestrictionTalNum < RestrictionCountMax and RestrictionAchNum < RestrictionCountMax then
		scanNextMember()
	end
end

function CharScanner:StopGroupScan()
	CharScanner.OnUpdateGroupScan:SetScript("OnUpdate", nil)
	CharScanner:Unregister_INSPECT_READY()
	CharScanner:Unregister_INSPECT_ACHIEVEMENT_READY()
	GroupScanNext = false
	GroupScanIsActive = false
	CleanTempData()

	if not GUIselectedGUID then
		CharScanner:ClearData()
		CharScanner:History_Update()
		if NumScannedChars == 0 then
			table_wipe(SCAN)
			CharScanner:DeleteButtonToggle(false)
			CharScanner:ScanFrameToggle(nil, true)
		end
	end

	if CharScanner_Options.chatoutput then
		local time = 0
		local TimeElapsed = math_floor((GetTime()-GroupScanTimerStart) * mult4 + 0.5) / mult4
		if type(TimeElapsed) == "number" and TimeElapsed > 0 then
			time = TimeElapsed
		end
		if time == 1 then
			time = string_format("%s %s", time, SECONDS)
		else
			time = time.." "..SECONDS
		end
		Print(L["GroupScan finished!"].." |cff"..color.Green.hex.."("..time..")|r")
	end

	CharScanner:UpdateGroupMissingMembers()
	CharScanner:SingleScanShow()
	CharScanner:GroupScanShow()
	CharScanner:ProgressDisplay("GROUP", false, false)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function CharScanner:ScanUnit(unit)
	if inCombat then
		GVAR.MainFrame.Status.Text:SetText(L["Scan is deactivated in combat!"])
		GVAR.MainFrame.Status.Text:SetTextColor(color.Red.rgba[1], color.Red.rgba[2], color.Red.rgba[3], color.Red.rgba[4])
		GVAR.MainFrame.Status:Show()
		GroupScanNext = true
		return
	end
	if AchievementFrame and AchievementFrame:IsVisible() then
		GVAR.MainFrame.Status.Text:SetText(L["Please close AchievementFrame!"])
		GVAR.MainFrame.Status.Text:SetTextColor(color.Red.rgba[1], color.Red.rgba[2], color.Red.rgba[3], color.Red.rgba[4])
		GVAR.MainFrame.Status:Show()
		GroupScanNext = true
		return
	end
	if not UnitExists(unit) or not UnitIsPlayer(unit) then
		GVAR.MainFrame.Status.Text:SetText(L["Target is NOT a player!"])
		GVAR.MainFrame.Status.Text:SetTextColor(color.Red.rgba[1], color.Red.rgba[2], color.Red.rgba[3], color.Red.rgba[4])
		GVAR.MainFrame.Status:Show()
		GroupScanNext = true
		return
	end
	if not CheckInteractDistance(unit, 1) then
		
		if GroupScanIsActive then
			SCANGUID = UnitGUID(unit)
			if SCANGUID and GroupMembers[SCANGUID] then

				SingleScanTimerStart = GetTime()
				LatestScanIsComplete = false
				CharScanner:ScanFrameToggle(true)

				local name, realm = UnitName(unit)
				if name then
					if realm and realm ~= "" then
						name = name.."-"..realm
					end
				else
					name = SCANGUID
				end
		
				SCANGUID = "_"..SCANGUID -- previewGUID -> if a succesfuly scanned unit is scanned again with an uncomplete scan the original (complete) scan is untouched
				CreateScanDataByGUID(SCANGUID)
				SCAN[SCANGUID].DataStep = 0
				SCAN[SCANGUID].oorScan = 1

				-- DUMMY_FactionFix START
				local faction
				if playerFaction == "Horde" then
					PVEDUMP[3][1].achievement[54] = 4297
					faction = "Horde"
				else
					PVEDUMP[3][1].achievement[54] = 4298
					faction = "Alliance"
				end
				-- DUMMY_FactionFix END

				SCAN[SCANGUID].name = name
				SCAN[SCANGUID].level = UnitLevel(unit) or "?"
				SCAN[SCANGUID].race = UnitRace(unit) or UNKNOWN
				SCAN[SCANGUID].faction = faction

				SCAN[SCANGUID].class, SCAN[SCANGUID].classEN = UnitClass(unit)
				if not SCAN[SCANGUID].class then
					SCAN[SCANGUID].class = UNKNOWN
					SCAN[SCANGUID].classEN = "OORCLASS"
				end

				SCAN[SCANGUID].guildName, SCAN[SCANGUID].guildRankName, SCAN[SCANGUID].guildRankIndex = GetGuildInfo(unit)
				
				CharScanner:ClearData()
				CharScanner:ShowData(1, SCANGUID, true)
				CharScanner:ShowData(2, SCANGUID, true)
				CharScanner:ShowData(3, SCANGUID, true)

				CharScanner:Function_INSPECT_ACHIEVEMENT_READY(SCANGUID)
				CharScanner:ShowData(4, SCANGUID, true)
	
				GroupScanNext = true
				return
			end
		end

		GVAR.MainFrame.Status.Text:SetText(L["Target is NOT in inspect range!"])
		GVAR.MainFrame.Status.Text:SetTextColor(color.Red.rgba[1], color.Red.rgba[2], color.Red.rgba[3], color.Red.rgba[4])
		GVAR.MainFrame.Status:Show()
		GroupScanNext = true
		return
	end
	if not CanInspect(unit) then
		GVAR.MainFrame.Status.Text:SetText(L["Can NOT inspect target!"])
		GVAR.MainFrame.Status.Text:SetTextColor(color.Red.rgba[1], color.Red.rgba[2], color.Red.rgba[3], color.Red.rgba[4])
		GVAR.MainFrame.Status:Show()
		GroupScanNext = true
		return
	end

	if not GroupScanIsActive then
		CharScanner:ProgressDisplay("SINGLE", true, false)
		if RestrictionTalNum >= RestrictionCountMax or RestrictionAchNum >= RestrictionCountMax then
			if (GetTime() - RestrictionTime) < RestrictionMaxTime then
				GVAR.MainFrame.Status.Text:SetText( L["Achievement/Talent scan restriction from Blizzard:"].." "..string_format(L["Please wait %.2f seconds."], RestrictionMaxTime) )
				GVAR.MainFrame.Status.Text:SetTextColor(color.Red.rgba[1], color.Red.rgba[2], color.Red.rgba[3], color.Red.rgba[4])
				GVAR.MainFrame.Status:Show()
				CharScanner:SingleScanHide()
				CharScanner:GroupScanHide()

				local function restrictionTimeCheck()
					if inCombat then
						CharScanner.OnUpdateRestriction:SetScript("OnUpdate", nil)
						CharScanner:StopSingleScan(true)
						return
					end
					local elapsedWaitTime = GetTime() - RestrictionTime
					if elapsedWaitTime < RestrictionMaxTime then
						CharScanner:ProgressDisplay("SINGLE", false, false)
						GVAR.MainFrame.Status.Text:SetText( L["Achievement/Talent scan restriction from Blizzard:"].." "..string_format(L["Please wait %.2f seconds."], (RestrictionMaxTime-elapsedWaitTime)) )
					else
						CharScanner:SingleScanShow()
						CharScanner:GroupScanShow()
						GVAR.MainFrame.Status.Text:SetText("")
						CharScanner.OnUpdateRestriction:SetScript("OnUpdate", nil)
						return
					end
				end
				CharScanner.OnUpdateRestriction:SetScript("OnUpdate", restrictionTimeCheck)
				return
			end
			RestrictionTalNum = 0
			RestrictionAchNum = 0
		end
	end

	-- DUMMY_FactionFix START
	local faction
	if UnitFactionGroup(unit) == "Horde" then
		PVEDUMP[3][1].achievement[54] = 4297
		faction = "Horde"
	else
		PVEDUMP[3][1].achievement[54] = 4298
		faction = "Alliance"
	end
	-- DUMMY_FactionFix END

	SingleScanTimerStart = GetTime()
	LatestScanIsComplete = false

	if not GroupScanIsActive then
		CharScanner:ProgressDisplay("SINGLE", true, false)
		CharScanner:ClearData()
	end
	CharScanner:ScanFrameToggle(true)

	if UnitIsUnit("player", unit) then
		IsUnitInpected = false -- "Character" -> it's me
		unit = "player"
	else
		IsUnitInpected = true -- "Inspect"
	end

	SCANGUID = UnitGUID(unit) --..GetTime() -- TEST
	if not SCANGUID then Print("GUID ERROR!", SCANGUID) return end -- TEST
	SCANGUID = "_"..SCANGUID -- previewGUID -> if a succesfuly scanned unit is scanned again with an uncomplete scan the original (complete) scan is untouched
	CreateScanDataByGUID(SCANGUID)
	SCAN[SCANGUID].DataStep = 0

	if not GroupScanIsActive then
		SingleScanIsActive = true
		CharScanner:SingleScanHide()
		CharScanner:GroupScanHide()

		local function ssTimeCheck()
			if inCombat then
				CharScanner:StopSingleScan(true)
				return
			end
			if (GetTime() - SingleScanTimerStart) >= CharScanner_Options.scanDuration then
				CharScanner:StopSingleScan(true)
				return
			end
		end

		CharScanner.OnUpdateSingleScan:SetScript("OnUpdate", ssTimeCheck)
	end

	local name, realm = UnitName(unit)
	if realm and realm ~= "" then
		name = name.."-"..realm
	end
	SCAN[SCANGUID].name = name
	SCAN[SCANGUID].level = UnitLevel(unit)
	SCAN[SCANGUID].race = UnitRace(unit)
	SCAN[SCANGUID].faction = faction
	SCAN[SCANGUID].class, SCAN[SCANGUID].classEN = UnitClass(unit)
	SCAN[SCANGUID].guildName, SCAN[SCANGUID].guildRankName, SCAN[SCANGUID].guildRankIndex = GetGuildInfo(unit)

	CharScanner:ShowData(1, SCANGUID, true)

	ClearAchievementComparisonUnit()
	SetAchievementComparisonUnit(unit)
	Event_INSPECT_ACHIEVEMENT_READY = true

	NotifyInspect(unit)
	InspectFrame.unit = unit
	SCANUNIT = unit

	CharScanner:GearScanPost(unit)
	Event_INSPECT_READY = true
end

function CharScanner:StopSingleScan(statusupdate)
	CharScanner.OnUpdateSingleScan:SetScript("OnUpdate", nil)
	CharScanner:Unregister_INSPECT_READY()
	CharScanner:Unregister_INSPECT_ACHIEVEMENT_READY()
	SingleScanIsActive = false
	CleanTempData()
	CharScanner:UpdateGroupMissingMembers()
	CharScanner:SingleScanShow()
	CharScanner:GroupScanShow()
	CharScanner:ProgressDisplay("SINGLE", false, false)
	if statusupdate then
		CharScanner:AchievementDetail_HideAll()
		GVAR.MainFrame.Status.Text:SetText(L["Scan incomplete!"])
		GVAR.MainFrame.Status.Text:SetTextColor(color.Red.rgba[1], color.Red.rgba[2], color.Red.rgba[3], color.Red.rgba[4])
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function CharScanner:ShowData(step, guid, scan)
	SCAN[guid].DataStep = SCAN[guid].DataStep + 1

	if (GroupScanIsActive and not GroupScanFirstGUID) or SingleScanIsActive or not scan then

		if step == 1 then

			for i = 1, GUI_history_maxRows do
				GVAR.HistoryFrame.ScrollFrameButton[i].Arrow:Hide()
			end
			TEMPLATE.EnableIconButton(GVAR.MainFrame.Headline.Collapse, 1)
			CharScanner:DeleteButtonToggle(true)

			-- ClassIcon
			GVAR.MainFrame.Base.ClassIcon:SetTexCoord(classes[SCAN[guid].classEN].icon[1], classes[SCAN[guid].classEN].icon[2], classes[SCAN[guid].classEN].icon[3], classes[SCAN[guid].classEN].icon[4])
			-- FactionIcon
			if SCAN[guid].faction == "Horde" then
				GVAR.MainFrame.Base.FactionIcon:SetTexture("Interface\\TargetingFrame\\UI-PVP-Horde")
			else
				GVAR.MainFrame.Base.FactionIcon:SetTexture("Interface\\TargetingFrame\\UI-PVP-Alliance")
			end
			GVAR.MainFrame.Base.FactionIcon:SetTexCoord(0.046875, 0.609375, 0.03125, 0.59375) -- (3/64, 39/64, 2/64, 38/64)
			-- Name
			GVAR.MainFrame.Base.Name:SetText(SCAN[guid].name)
			GVAR.MainFrame.Base.Name:SetTextColor(classcolors[SCAN[guid].classEN].r, classcolors[SCAN[guid].classEN].g, classcolors[SCAN[guid].classEN].b, 1)
			-- GUID
			if scan then
				GVAR.MainFrame.Base.GUID:SetText("(GUID: "..string_sub(guid, 2)..")") -- save preview guid (with leading '_')
			else
				GVAR.MainFrame.Base.GUID:SetText("(GUID: "..guid..")") -- real guid (without leading '_')
			end
			-- Guild
			local guild = ""
			if SCAN[guid].guildName then
				guild = GUILD..": "..SCAN[guid].guildName
				if SCAN[guid].guildRankIndex and SCAN[guid].guildRankIndex < 5 then
					guild = guild.." | "..RANK..": "..guildRank[SCAN[guid].guildRankIndex+1]
				else
					guild = guild.." | "..RANK..": -"
				end
			else
				guild = GUILD..": - "
			end
			GVAR.MainFrame.Base.Guild:SetText(guild)
			GVAR.MainFrame.Base.Guild:SetTextColor(color.Level2.rgba[1], color.Level2.rgba[2], color.Level2.rgba[3], color.Level2.rgba[4])
			-- Level/Race/Class
			GVAR.MainFrame.Base.Level:SetText(LEVEL.." "..SCAN[guid].level.." - "..SCAN[guid].race..", "..SCAN[guid].class)
			GVAR.MainFrame.Base.Level:SetTextColor(classcolors[SCAN[guid].classEN].r, classcolors[SCAN[guid].classEN].g, classcolors[SCAN[guid].classEN].b, 1)

		elseif step == 2 then

			-- Equipment
			GVAR.MainFrame.Equipment.Title:SetText(L["Equipment"]..":")

			local miss = ""
			if SCAN[guid].itemlevelMiss > 0 then
				miss = " - |cff"..color.Red.hex..string_format(L["%s %s not used"], SCAN[guid].itemlevelMiss, L["|4Slot:Slots;"]).."|r"
			end
			local enchcount = L["Enchantments"]..": |cff"..color.White.hex..SCAN[guid].gearEnchCount.."|r"
			local pvpcount = " | "..L["PvP Items"]..": |cff"..color.White.hex..SCAN[guid].gearPvPCount.."|r"
			local gemcount = ""
			if SCAN[guid].gemTotal > 0 then
				if SCAN[guid].gemMissing > 0 then
					gemcount = " | "..L["Gems"]..": |cff"..color.Red.hex..(SCAN[guid].gemTotal-SCAN[guid].gemMissing).."/"..SCAN[guid].gemTotal.." ("..SCAN[guid].gemMissing..")|r"
				else
					gemcount = " | "..L["Gems"]..": |cff"..color.Green.hex..(SCAN[guid].gemTotal-SCAN[guid].gemMissing).."/"..SCAN[guid].gemTotal.."|r"
				end
			else
				gemcount = " | "..L["Gems"]..": |cff"..color.White.hex..SCAN[guid].gemTotal.."|r"
			end

			local txt = L["Avg"].." "..cnum(SCAN[guid].itemlevelAvg).." ("..L["min."].." "..SCAN[guid].itemlevelMin.." - "..L["max."].." "..SCAN[guid].itemlevelMax..")"..miss

			GVAR.MainFrame.Equipment.Text1:SetText(txt)
			GVAR.MainFrame.Equipment.Text2:SetText(SCAN[guid].itemlevelQuality)
			GVAR.MainFrame.Equipment.Text3:SetText(enchcount..pvpcount..gemcount)

			local w1 = ceil(GVAR.MainFrame.Equipment.Title:GetStringWidth())
			local w2 = ceil(GVAR.MainFrame.Equipment.Text2:GetStringWidth())
			local maxwidth = sizeGeneral.MainWidth - (sizeGeneral.Offset + w1 + sizeGeneral.Offset + sizeGeneral.Offset + 25) -- ~25px for buttonwidth

			if w2 >= maxwidth then
				GVAR.MainFrame.Equipment.Text2:SetWidth(maxwidth)
				GVAR.MainFrame.Equipment:SetHeight(sizeEquipmentFrame.BaseHeightXXL)
			else
				GVAR.MainFrame.Equipment.Text2:SetWidth(maxwidth)
				GVAR.MainFrame.Equipment:SetHeight(sizeEquipmentFrame.BaseHeight)
			end

			-- Gear Slots START
			for i = 1, #SCAN[guid].gear do
				local itemTexture = GetItemIcon(SCAN[guid].gear[i].iID)
				if itemTexture then
					local rcolor = SCAN[guid].gear[i].iRar+1

					GVAR.GearFrame_Slot[i].IconButton:Enable()
					GVAR.GearFrame_Slot[i].IconButton:Show()

					GVAR.GearFrame_Slot[i].IconTexture:SetTexture(itemTexture)

					if SCAN[guid].gear[i].iLvl == 0 then
						GVAR.GearFrame_Slot[i].ItemLevel:SetText("|cff"..itemQuality[rcolor].color.."?|r")
					else
						GVAR.GearFrame_Slot[i].ItemLevel:SetText("|cff"..itemQuality[rcolor].color..SCAN[guid].gear[i].iLvl.."|r")
					end
					GVAR.GearFrame_Slot[i].ItemLevel:Show()

					if SCAN[guid].gear[i].iEnch > 0 then
						GVAR.GearFrame_Slot[i].EnchButton:Show()
					end

					if SCAN[guid].gear[i].iRefo == 1 then
						GVAR.GearFrame_Slot[i].ReforgeButton:Show()
					end

					if SCAN[guid].gear[i].iPvP == 1 then
						GVAR.GearFrame_Slot[i].PvPTexture:Show()
					end

					if SCAN[guid].gear[i].isSet then
						GVAR.GearFrame_Slot[i].IsSetTexture:Show()
					end

					local gemCount = 0
					if SCAN[guid].gear[i].iGem1 then
						gemCount = gemCount + 1
						GVAR.GearFrame_Slot[i].Gem1Button:Show()
						if SCAN[guid].gear[i].iGem1 > 0 then
							GVAR.GearFrame_Slot[i].Gem1Texture:SetTexture( GetItemIcon(SCAN[guid].gear[i].iGem1) )
							GVAR.GearFrame_Slot[i].Gem1Texture:SetTexCoord(5/64, 59/64, 5/64, 59/64)
						elseif SCAN[guid].gear[i].iGem1 == 0 then
							GVAR.GearFrame_Slot[i].Gem1Texture:SetTexture(Textures.CharScannerIcons.path)
							GVAR.GearFrame_Slot[i].Gem1Texture:SetTexCoord(unpack(Textures.GemIcon.coordsUK))
						else
							GVAR.GearFrame_Slot[i].Gem1Texture:SetTexture(Textures.CharScannerIcons.path)
							GVAR.GearFrame_Slot[i].Gem1Texture:SetTexCoord(unpack(Textures.GemIcon.coordsNO))
						end
					end
					if SCAN[guid].gear[i].iGem2 then
						gemCount = gemCount + 1
						GVAR.GearFrame_Slot[i].Gem2Button:Show()
						if SCAN[guid].gear[i].iGem2 > 0 then
							GVAR.GearFrame_Slot[i].Gem2Texture:SetTexture( GetItemIcon(SCAN[guid].gear[i].iGem2) )
							GVAR.GearFrame_Slot[i].Gem2Texture:SetTexCoord(5/64, 59/64, 5/64, 59/64)
						elseif SCAN[guid].gear[i].iGem2 == 0 then
							GVAR.GearFrame_Slot[i].Gem2Texture:SetTexture(Textures.CharScannerIcons.path)
							GVAR.GearFrame_Slot[i].Gem2Texture:SetTexCoord(unpack(Textures.GemIcon.coordsUK))
						else
							GVAR.GearFrame_Slot[i].Gem2Texture:SetTexture(Textures.CharScannerIcons.path)
							GVAR.GearFrame_Slot[i].Gem2Texture:SetTexCoord(unpack(Textures.GemIcon.coordsNO))
						end
					end
					if SCAN[guid].gear[i].iGem3 then
						gemCount = gemCount + 1
						GVAR.GearFrame_Slot[i].Gem3Button:Show()
						if SCAN[guid].gear[i].iGem3 > 0 then
							GVAR.GearFrame_Slot[i].Gem3Texture:SetTexture( GetItemIcon(SCAN[guid].gear[i].iGem3) )
							GVAR.GearFrame_Slot[i].Gem3Texture:SetTexCoord(5/64, 59/64, 5/64, 59/64)
						elseif SCAN[guid].gear[i].iGem3 == 0 then
							GVAR.GearFrame_Slot[i].Gem3Texture:SetTexture(Textures.CharScannerIcons.path)
							GVAR.GearFrame_Slot[i].Gem3Texture:SetTexCoord(unpack(Textures.GemIcon.coordsUK))
						else
							GVAR.GearFrame_Slot[i].Gem3Texture:SetTexture(Textures.CharScannerIcons.path)
							GVAR.GearFrame_Slot[i].Gem3Texture:SetTexCoord(unpack(Textures.GemIcon.coordsNO))
						end
					end
					if SCAN[guid].gear[i].iGem4 then
						gemCount = gemCount + 1
						GVAR.GearFrame_Slot[i].Gem4Button:Show()
						if SCAN[guid].gear[i].iGem4 > 0 then
							GVAR.GearFrame_Slot[i].Gem4Texture:SetTexture( GetItemIcon(SCAN[guid].gear[i].iGem4) )
							GVAR.GearFrame_Slot[i].Gem4Texture:SetTexCoord(5/64, 59/64, 5/64, 59/64)
						elseif SCAN[guid].gear[i].iGem4 == 0 then
							GVAR.GearFrame_Slot[i].Gem4Texture:SetTexture(Textures.CharScannerIcons.path)
							GVAR.GearFrame_Slot[i].Gem4Texture:SetTexCoord(unpack(Textures.GemIcon.coordsUK))
						else
							GVAR.GearFrame_Slot[i].Gem4Texture:SetTexture(Textures.CharScannerIcons.path)
							GVAR.GearFrame_Slot[i].Gem4Texture:SetTexCoord(unpack(Textures.GemIcon.coordsNO))
						end
					end
					if SCAN[guid].gear[i].iGemBonusVal then
						GVAR.GearFrame_Slot[i].GemBonusTexture:SetWidth( (gemCount*Textures.GemIcon.qsize) + ((gemCount-1)*1) )
						if SCAN[guid].gear[i].iGemBonusVal > 0 then
							GVAR.GearFrame_Slot[i].GemBonusTexture:SetTexture(color.Green.rgba[1], color.Green.rgba[2], color.Green.rgba[3], color.Green.rgba[4])
						else
							GVAR.GearFrame_Slot[i].GemBonusTexture:SetTexture(color.Red.rgba[1], color.Red.rgba[2], color.Red.rgba[3], color.Red.rgba[4])
						end
					end

					GVAR.GearFrame_Slot[i].IconButton:SetScript("OnEnter", function() CharScanner:GearFrame_IconButton_OnEnter(i, guid) end)
					GVAR.GearFrame_Slot[i].IconButton:SetScript("OnLeave", function() CharScanner:GearFrame_IconButton_OnLeave(i, guid) end)
					GVAR.GearFrame_Slot[i].IconButton:SetScript("OnClick", function() CharScanner:GearFrame_IconButton_OnClick(i, guid) end)
					GVAR.GearFrame_Slot[i].EnchButton:SetScript("OnEnter", function() CharScanner:GearFrame_EnchButton_OnEnter(i, guid) end)
					GVAR.GearFrame_Slot[i].EnchButton:SetScript("OnLeave", function() CharScanner:GearFrame_EnchButton_OnLeave(i, guid) end)
					GVAR.GearFrame_Slot[i].EnchButton:SetScript("OnClick", function() CharScanner:GearFrame_EnchButton_OnClick(i, guid) end)
					GVAR.GearFrame_Slot[i].ReforgeButton:SetScript("OnEnter", function() CharScanner:GearFrame_ReforgeButton_OnEnter(i, guid) end)
					GVAR.GearFrame_Slot[i].ReforgeButton:SetScript("OnLeave", function() CharScanner:GearFrame_ReforgeButton_OnLeave(i, guid) end)
					GVAR.GearFrame_Slot[i].Gem1Button:SetScript("OnEnter", function() CharScanner:GearFrame_GemButton_OnEnter(1, i, guid) end)
					GVAR.GearFrame_Slot[i].Gem1Button:SetScript("OnLeave", function() CharScanner:GearFrame_GemButton_OnLeave(1, i, guid) end)
					GVAR.GearFrame_Slot[i].Gem1Button:SetScript("OnClick", function() CharScanner:GearFrame_GemButton_OnClick(1, i, guid) end)
					GVAR.GearFrame_Slot[i].Gem2Button:SetScript("OnEnter", function() CharScanner:GearFrame_GemButton_OnEnter(2, i, guid) end)
					GVAR.GearFrame_Slot[i].Gem2Button:SetScript("OnLeave", function() CharScanner:GearFrame_GemButton_OnLeave(2, i, guid) end)
					GVAR.GearFrame_Slot[i].Gem2Button:SetScript("OnClick", function() CharScanner:GearFrame_GemButton_OnClick(2, i, guid) end)
					GVAR.GearFrame_Slot[i].Gem3Button:SetScript("OnEnter", function() CharScanner:GearFrame_GemButton_OnEnter(3, i, guid) end)
					GVAR.GearFrame_Slot[i].Gem3Button:SetScript("OnLeave", function() CharScanner:GearFrame_GemButton_OnLeave(3, i, guid) end)
					GVAR.GearFrame_Slot[i].Gem3Button:SetScript("OnClick", function() CharScanner:GearFrame_GemButton_OnClick(3, i, guid) end)
					GVAR.GearFrame_Slot[i].Gem4Button:SetScript("OnEnter", function() CharScanner:GearFrame_GemButton_OnEnter(4, i, guid) end)
					GVAR.GearFrame_Slot[i].Gem4Button:SetScript("OnLeave", function() CharScanner:GearFrame_GemButton_OnLeave(4, i, guid) end)
					GVAR.GearFrame_Slot[i].Gem4Button:SetScript("OnClick", function() CharScanner:GearFrame_GemButton_OnClick(4, i, guid) end)

				else
					GVAR.GearFrame_Slot[i].IconButton:Disable()
				end
			end

			if not SCAN[guid].oorScan then
				if SCAN[guid].gearTwoHandWeapon then
					GVAR.GearFrame_Slot[16].IconButtonBG:SetTexture(0, 0, 0, 0)
					GVAR.GearFrame_Slot[16].IconTexture:SetTexture("Interface\\PaperDollInfoFrame\\UI-GearManager-LeaveItem-Opaque")
					GVAR.GearFrame_Slot[16].IconTexture:SetAlpha(0.4)
					GVAR.GearFrame_Slot[16].IconButton:Disable()
				else
					GVAR.GearFrame_Slot[16].IconButtonBG:SetTexture(0.2, 0.2, 0.2, 1)
					GVAR.GearFrame_Slot[16].IconTexture:SetAlpha(1)
					GVAR.GearFrame_Slot[16].IconButton:Enable()
				end
			end
			-- Gear Slots END

			-- GearStats START
			for i = 1, #statDisplay do
				if SCAN[guid].gearStats[ statGlobalName[ statDisplay[i] ] ] then
					GVAR.GearFrame_Stats[i].Value:SetText( "+"..SCAN[guid].gearStats[ statGlobalName[ statDisplay[i] ] ] )
					GVAR.GearFrame_Stats[i].Value:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])
					GVAR.GearFrame_Stats[i].Stat:SetTextColor(color.LightYellow.rgba[1], color.LightYellow.rgba[2], color.LightYellow.rgba[3], color.LightYellow.rgba[4])
				else
					GVAR.GearFrame_Stats[i].Value:SetTextColor(color.Grey.rgba[1], color.Grey.rgba[2], color.Grey.rgba[3], color.Grey.rgba[4])
					GVAR.GearFrame_Stats[i].Stat:SetTextColor(color.Grey.rgba[1], color.Grey.rgba[2], color.Grey.rgba[3], color.Grey.rgba[4])
				end
			end
			GVAR.GearFrame_Stats.GemBlueText:SetText(": "..SCAN[guid].gemBlue)
			GVAR.GearFrame_Stats.GemRedText:SetText(": "..SCAN[guid].gemRed)
			GVAR.GearFrame_Stats.GemYellowText:SetText(": "..SCAN[guid].gemYellow)
			if SCAN[guid].gemMeta then
				if SCAN[guid].gemMeta > 0 then
					GVAR.GearFrame_Stats.GemMetaText:SetText(" "..META_GEM..": |cff"..color.Green.hex..L["OK"].."|r")
				else
					GVAR.GearFrame_Stats.GemMetaText:SetText(" "..META_GEM..": |cff"..color.Red.hex..L["NOT OK"].."|r")
				end
			end
			-- GearStats END

			-- 1 enchantment summary START
			for i = 1, #SCAN[guid].enchStats do
				local stat  = SCAN[guid].enchStats[i].stat
				local value = SCAN[guid].enchStats[i].value
				if i <= 15 then
					GVAR.GearFrame_Summary.TabButton1Frame[i].Button:Show()
					if type(stat) == "string" then
						GVAR.GearFrame_Summary.TabButton1Frame[i].Text:SetText( statShortLocName[ statName[stat] ] )
						GVAR.GearFrame_Summary.TabButton1Frame[i].Text:SetTextColor(color.LightYellow.rgba[1], color.LightYellow.rgba[2], color.LightYellow.rgba[3], color.LightYellow.rgba[4])
						GVAR.GearFrame_Summary.TabButton1Frame[i].Value:SetText("+"..value)
						GVAR.GearFrame_Summary.TabButton1Frame[i].Value:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])
					else
						if value > 1000000000 then -- item
							value = value/1000000000
							GVAR.GearFrame_Summary.TabButton1Frame[i].Text:SetText(L["Effect"])
							GVAR.GearFrame_Summary.TabButton1Frame[i].Text:SetTextColor(color.SpellColor.rgba[1], color.SpellColor.rgba[2], color.SpellColor.rgba[3], color.SpellColor.rgba[4])
							GVAR.GearFrame_Summary.TabButton1Frame[i].Value:SetText("ID "..value)
							GVAR.GearFrame_Summary.TabButton1Frame[i].Value:SetTextColor(color.SpellColor.rgba[1], color.SpellColor.rgba[2], color.SpellColor.rgba[3], color.SpellColor.rgba[4])
							GVAR.GearFrame_Summary.TabButton1Frame[i].Button:SetScript("OnEnter", function()
								GameTooltip:SetOwner(GVAR.GearFrame_Summary, "ANCHOR_BOTTOMLEFT", 0, sizeEquipmentFrame.GearSummaryHeight)
								GameTooltip:SetHyperlink("item:"..value)
							end)
							GVAR.GearFrame_Summary.TabButton1Frame[i].Button:SetScript("OnLeave", function()
								GameTooltip:Hide()
							end)
							GVAR.GearFrame_Summary.TabButton1Frame[i].Button:SetScript("OnClick", function()
								local link = select(2, GetItemInfo(value))
								if link then
									HandleModifiedItemClick( link )
								end
							end)
						else -- spell
							value = value/-100
							GVAR.GearFrame_Summary.TabButton1Frame[i].Text:SetText(L["Effect"])
							GVAR.GearFrame_Summary.TabButton1Frame[i].Text:SetTextColor(color.SpellColor.rgba[1], color.SpellColor.rgba[2], color.SpellColor.rgba[3], color.SpellColor.rgba[4])
							GVAR.GearFrame_Summary.TabButton1Frame[i].Value:SetText("ID "..-value)
							GVAR.GearFrame_Summary.TabButton1Frame[i].Value:SetTextColor(color.SpellColor.rgba[1], color.SpellColor.rgba[2], color.SpellColor.rgba[3], color.SpellColor.rgba[4])
							GVAR.GearFrame_Summary.TabButton1Frame[i].Button:SetScript("OnEnter", function()
								GameTooltip:SetOwner(GVAR.GearFrame_Summary, "ANCHOR_BOTTOMLEFT", 0, sizeEquipmentFrame.GearSummaryHeight)
								GameTooltip:SetSpellByID(-value)
							end)
							GVAR.GearFrame_Summary.TabButton1Frame[i].Button:SetScript("OnLeave", function()
								GameTooltip:Hide()
							end)
							GVAR.GearFrame_Summary.TabButton1Frame[i].Button:SetScript("OnClick", function()
								local name = GetSpellInfo(-value)
								if name then
									HandleModifiedItemClick( "|cff71d5ff|Hspell:"..(-value).."|h["..name.."]|h|r" )
								end
							end)
						end
					end
				elseif i == 16 then
					GVAR.GearFrame_Summary.TabButton1Frame[i].Button:Show()
					GVAR.GearFrame_Summary.TabButton1Frame[i].Text:SetText("...")
					GVAR.GearFrame_Summary.TabButton1Frame[i].Text:SetTextColor(color.LightYellow.rgba[1], color.LightYellow.rgba[2], color.LightYellow.rgba[3], color.LightYellow.rgba[4])
					GVAR.GearFrame_Summary.TabButton1Frame[i].Value:SetText("")
				end
			end
			-- 1 enchantment summary END

			-- 2 reforge summary START
			local x = 1
			for key, value in pairs(SCAN[guid].refoMinStats) do
				GVAR.GearFrame_Summary.TabButton2Frame[x].Text:Show()
				GVAR.GearFrame_Summary.TabButton2Frame[x].Text:SetText("-"..value.." "..statShortLocName[ statName[key] ])
				x = x + 1
			end
			local x = 1
			for key, value in pairs(SCAN[guid].refoPluStats) do
				GVAR.GearFrame_Summary.TabButton2Frame[x].Value:Show()
				GVAR.GearFrame_Summary.TabButton2Frame[x].Value:SetText("+"..value.." "..statShortLocName[ statName[key] ])
				x = x + 1
			end
			-- 2 reforge summary END

			-- 3 gem summary START
			for i = 1, #SCAN[guid].gemStats do
				local stat  = SCAN[guid].gemStats[i].stat
				local value = SCAN[guid].gemStats[i].value
				if i <= 15 then
					GVAR.GearFrame_Summary.TabButton3Frame[i].Button:Show()
					if type(stat) == "string" then
						GVAR.GearFrame_Summary.TabButton3Frame[i].Text:SetText(statShortLocName[ statName[stat] ])
						GVAR.GearFrame_Summary.TabButton3Frame[i].Text:SetTextColor(color.LightYellow.rgba[1], color.LightYellow.rgba[2], color.LightYellow.rgba[3], color.LightYellow.rgba[4])
						GVAR.GearFrame_Summary.TabButton3Frame[i].Value:SetText("+"..value)
						GVAR.GearFrame_Summary.TabButton3Frame[i].Value:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])
					else
						value = value/-100
						GVAR.GearFrame_Summary.TabButton3Frame[i].Text:SetText(L["Effect"])
						GVAR.GearFrame_Summary.TabButton3Frame[i].Text:SetTextColor(color.SpellColor.rgba[1], color.SpellColor.rgba[2], color.SpellColor.rgba[3], color.SpellColor.rgba[4])
						GVAR.GearFrame_Summary.TabButton3Frame[i].Value:SetText("ID "..-value)
						GVAR.GearFrame_Summary.TabButton3Frame[i].Value:SetTextColor(color.SpellColor.rgba[1], color.SpellColor.rgba[2], color.SpellColor.rgba[3], color.SpellColor.rgba[4])
						GVAR.GearFrame_Summary.TabButton3Frame[i].Button:SetScript("OnEnter", function()
							GameTooltip:SetOwner(GVAR.GearFrame_Summary, "ANCHOR_BOTTOMLEFT", 0, sizeEquipmentFrame.GearSummaryHeight)
							GameTooltip:SetSpellByID(-value)
						end)
						GVAR.GearFrame_Summary.TabButton3Frame[i].Button:SetScript("OnLeave", function()
							GameTooltip:Hide()
						end)
						GVAR.GearFrame_Summary.TabButton3Frame[i].Button:SetScript("OnClick", function()
							local name = GetSpellInfo(-value)
							if name then
								HandleModifiedItemClick( "|cff71d5ff|Hspell:"..(-value).."|h["..name.."]|h|r" )
							end
						end)
					end
				elseif i == 16 then
					GVAR.GearFrame_Summary.TabButton3Frame[i].Button:Show()
					GVAR.GearFrame_Summary.TabButton3Frame[i].Text:SetText("...")
					GVAR.GearFrame_Summary.TabButton3Frame[i].Text:SetTextColor(color.LightYellow.rgba[1], color.LightYellow.rgba[2], color.LightYellow.rgba[3], color.LightYellow.rgba[4])
					GVAR.GearFrame_Summary.TabButton3Frame[i].Value:SetText("")
				end
			end
			-- 3 gem summary END

			-- 4 set summary START
			for i = 1, #SCAN[guid].setStats do
				local stat  = SCAN[guid].setStats[i].stat
				local value = SCAN[guid].setStats[i].value
				if i <= 15 then
					GVAR.GearFrame_Summary.TabButton4Frame[i].Button:Show()
					if type(stat) == "string" then
						GVAR.GearFrame_Summary.TabButton4Frame[i].Text:SetText(statShortLocName[ statName[stat] ])
						GVAR.GearFrame_Summary.TabButton4Frame[i].Text:SetTextColor(color.LightYellow.rgba[1], color.LightYellow.rgba[2], color.LightYellow.rgba[3], color.LightYellow.rgba[4])
						GVAR.GearFrame_Summary.TabButton4Frame[i].Value:SetText("+"..value)
						GVAR.GearFrame_Summary.TabButton4Frame[i].Value:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])
					else
						value = value/-100
						GVAR.GearFrame_Summary.TabButton4Frame[i].Text:SetText(L["Effect"])
						GVAR.GearFrame_Summary.TabButton4Frame[i].Text:SetTextColor(color.SpellColor.rgba[1], color.SpellColor.rgba[2], color.SpellColor.rgba[3], color.SpellColor.rgba[4])
						GVAR.GearFrame_Summary.TabButton4Frame[i].Value:SetText("ID "..-value)
						GVAR.GearFrame_Summary.TabButton4Frame[i].Value:SetTextColor(color.SpellColor.rgba[1], color.SpellColor.rgba[2], color.SpellColor.rgba[3], color.SpellColor.rgba[4])
						GVAR.GearFrame_Summary.TabButton4Frame[i].Button:SetScript("OnEnter", function()
							GameTooltip:SetOwner(GVAR.GearFrame_Summary, "ANCHOR_BOTTOMLEFT", 0, sizeEquipmentFrame.GearSummaryHeight)
							GameTooltip:SetSpellByID(-value)
						end)
						GVAR.GearFrame_Summary.TabButton4Frame[i].Button:SetScript("OnLeave", function()
							GameTooltip:Hide()
						end)
						GVAR.GearFrame_Summary.TabButton4Frame[i].Button:SetScript("OnClick", function()
							local name = GetSpellInfo(-value)
							if name then
								HandleModifiedItemClick( "|cff71d5ff|Hspell:"..(-value).."|h["..name.."]|h|r" )
							end
						end)
					end
				elseif i == 16 then
					GVAR.GearFrame_Summary.TabButton4Frame[i].Button:Show()
					GVAR.GearFrame_Summary.TabButton4Frame[i].Text:SetText("...")
					GVAR.GearFrame_Summary.TabButton4Frame[i].Text:SetTextColor(color.LightYellow.rgba[1], color.LightYellow.rgba[2], color.LightYellow.rgba[3], color.LightYellow.rgba[4])
					GVAR.GearFrame_Summary.TabButton4Frame[i].Value:SetText("")
				end
			end
			-- 4 set summary END

			--TEST code for GearStats START
			--for key, value in pairs(SCAN[guid].gearStats) do
			--	if not CharScanner_Options.statsTest then CharScanner_Options.statsTest = {} end
			--	if not CharScanner_Options.statsTest[key] then CharScanner_Options.statsTest[key] = _G[key] or "noglobal" end
			--end
			--TEST code for GearStats END
			-- GearStats END

		elseif step == 3 then

			-- Talent
			local col
			GVAR.MainFrame.Talent.Title:SetText(TALENTS..":")
			if SCAN[guid].talentGroups then

				local text = RoleTextShort[SCAN[guid].talentGroup1role].." "..SCAN[guid].talentGroup1name.." "..SCAN[guid].talentGroup1pointsString
				local unspent = ""
				if SCAN[guid].talentGroup1pointsUnspent > 0 then
					if SCAN[guid].activeTalentGroup == 1 then
						col = "|cff"..color.Red.hex
					else
						col = "|cff"..color.Grey.hex
					end
					unspent = col.."("..string_format(UNSPENT_TALENT_POINTS, SCAN[guid].talentGroup1pointsUnspent)..")|r"
				end

				local texture = SCAN[guid].talentGroup1icon
				if SCAN[guid].talentGroup1pointsString == "0/0/0" then
					texture = nil
					text = RoleTextShort[SCAN[guid].talentGroup1role].." "..unspent
				else
					if unspent ~= "" then
						text = text.." "..unspent
					end
				end

				GVAR.MainFrame.Talent.Group1Icon:SetTexture(texture)
				GVAR.MainFrame.Talent.Group1Text:SetText(text)
				GVAR.MainFrame.Talent.Group1Text:SetPoint("LEFT", GVAR.MainFrame.Talent.Group1Icon, "RIGHT", sizeGeneral.Offset, 0)

				if SCAN[guid].activeTalentGroup == 1 then
					GVAR.MainFrame.Talent.MainRoleActive:SetTexture(Textures.CharScannerIcons.path)
					GVAR.MainFrame.Talent.MainRoleActive:SetPoint("RIGHT", GVAR.MainFrame.Talent.Group1Icon, "LEFT", -sizeGeneral.Offset, 0)
					GVAR.MainFrame.Talent.MainRoleActive:SetVertexColor(color.Role[ SCAN[guid].talentGroup1role ].rgba[1], color.Role[ SCAN[guid].talentGroup1role ].rgba[2], color.Role[ SCAN[guid].talentGroup1role ].rgba[3], color.Role[ SCAN[guid].talentGroup1role ].rgba[4])
					GVAR.MainFrame.Talent.Group1Text:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])
				else
					GVAR.MainFrame.Talent.Group1Text:SetTextColor(color.Grey.rgba[1], color.Grey.rgba[2], color.Grey.rgba[3], color.Grey.rgba[4])
				end

				if SCAN[guid].talentGroups > 1 then

					local text = RoleTextShort[SCAN[guid].talentGroup2role].." "..SCAN[guid].talentGroup2name.." "..SCAN[guid].talentGroup2pointsString
					local unspent = ""
					if SCAN[guid].talentGroup2pointsUnspent > 0 then
						if SCAN[guid].activeTalentGroup == 2 then
							col = "|cff"..color.Red.hex
						else
							col = "|cff"..color.Grey.hex
						end
						unspent = col.."("..string_format(UNSPENT_TALENT_POINTS, SCAN[guid].talentGroup2pointsUnspent)..")|r"
					end

					local texture = SCAN[guid].talentGroup2icon
					if SCAN[guid].talentGroup2pointsString == "0/0/0" then
						texture = nil
						text = RoleTextShort[SCAN[guid].talentGroup2role].." "..unspent
					else
						if unspent ~= "" then
							text = text.." "..unspent
						end
					end

					GVAR.MainFrame.Talent.Group2Icon:SetTexture(texture)
					GVAR.MainFrame.Talent.Group2Text:SetText(text)

					if SCAN[guid].activeTalentGroup == 2 then
						GVAR.MainFrame.Talent.MainRoleActive:SetTexture(Textures.CharScannerIcons.path)
						GVAR.MainFrame.Talent.MainRoleActive:SetPoint("RIGHT", GVAR.MainFrame.Talent.Group2Icon, "LEFT", -sizeGeneral.Offset, 0)
						GVAR.MainFrame.Talent.MainRoleActive:SetVertexColor(color.Role[ SCAN[guid].talentGroup2role ].rgba[1], color.Role[ SCAN[guid].talentGroup2role ].rgba[2], color.Role[ SCAN[guid].talentGroup2role ].rgba[3], color.Role[ SCAN[guid].talentGroup2role ].rgba[4])
						GVAR.MainFrame.Talent.Group2Text:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])
					else
						GVAR.MainFrame.Talent.Group2Text:SetTextColor(color.Grey.rgba[1], color.Grey.rgba[2], color.Grey.rgba[3], color.Grey.rgba[4])
					end

				else
					GVAR.MainFrame.Talent.Group2Icon:SetTexture(nil)
					GVAR.MainFrame.Talent.Group2Text:SetText("")
				end

				GVAR.MainFrame.Talent.MainRole:SetTexture(Textures.RoleIcon[SCAN[guid].mainRole].path)
				GVAR.MainFrame.Talent.MainRole:SetTexCoord(Textures.RoleIcon[SCAN[guid].mainRole].coords[1], Textures.RoleIcon[SCAN[guid].mainRole].coords[2], Textures.RoleIcon[SCAN[guid].mainRole].coords[3], Textures.RoleIcon[SCAN[guid].mainRole].coords[4])
				CharScanner:TalentGroup_Switch(guid, SCAN[guid].activeTalentGroup)
				
			else
				GVAR.MainFrame.Talent.MainRoleActive:SetTexture(nil)
				GVAR.MainFrame.Talent.Group1Icon:SetTexture(nil)
				GVAR.MainFrame.Talent.Group1Text:SetText(L["No Talents"])
				GVAR.MainFrame.Talent.Group1Text:SetPoint("LEFT", GVAR.MainFrame.Talent.Title, "RIGHT", sizeGeneral.Offset, 0)
				GVAR.MainFrame.Talent.Group1Text:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])
				GVAR.MainFrame.Talent.Group2Icon:SetTexture(nil)
				GVAR.MainFrame.Talent.Group2Text:SetText("")

				GVAR.MainFrame.Talent.MainRole:SetTexture(nil)
				CharScanner:TalentGroup_Switch(guid, 0)
			end

		elseif step == 4 then

			GVAR.MainFrame.StatsPvE.Title.Name:SetText("")

			-- PvE START
			if BossSearchQuery then
				CharScanner:ABKsearch_Update(guid)
			else

				GVAR.MainFrame.StatsPvE.Title.BossesTotal:SetText(L["Kills"])
				GVAR.MainFrame.StatsPvE.Title.Achievement:SetText(L["Achievements"])
				GVAR.MainFrame.StatsPvE.Title.Statistic:SetText(L["Bosses"])

				local col
				-- level 1 START
				for i = 1, #PVEDUMP do
					GVAR.MainFrame.StatsPvE.ScrollFrame["L1"..PVEDUMP[i].name].Name:SetText(L[PVEDUMP[i].name])
					if PVEDUMP[i].achievementTotal == SCAN[guid].PVEDATA[i].achievementCount then
						col="|cff"..color.Green.hex else col="|cff"..color.Red.hex
					end
					GVAR.MainFrame.StatsPvE.ScrollFrame["L1"..PVEDUMP[i].name].Achievement:SetText(col..SCAN[guid].PVEDATA[i].achievementCount.."|r / "..PVEDUMP[i].achievementTotal)

					if PVEDUMP[i].statisticTotal == SCAN[guid].PVEDATA[i].statisticBoss then
						col="|cff"..color.Green.hex else col="|cff"..color.Red.hex
					end
					if PVEDUMP[i].statisticTotal == 0 then --NOBOSS
						GVAR.MainFrame.StatsPvE.ScrollFrame["L1"..PVEDUMP[i].name].Statistic:SetText("- / -")
						GVAR.MainFrame.StatsPvE.ScrollFrame["L1"..PVEDUMP[i].name].BossesTotal:SetText("-")
					else
						GVAR.MainFrame.StatsPvE.ScrollFrame["L1"..PVEDUMP[i].name].Statistic:SetText(col..SCAN[guid].PVEDATA[i].statisticBoss.."|r / "..PVEDUMP[i].statisticTotal)
						GVAR.MainFrame.StatsPvE.ScrollFrame["L1"..PVEDUMP[i].name].BossesTotal:SetText(SCAN[guid].PVEDATA[i].statisticBossAll)
					end
				--end
				-- level 1 END

				-- level 2 START
				--for i = 1, #PVEDUMP do
					if PVEDUMP[i].statistic then
						for j = 1, #PVEDUMP[i].statistic do
							if SCAN[guid].PVEDATA[i].statisticBossEach[j] > 0 then
								col="|cff"..color.Green.hex else col="|cff"..color.Red.hex
							end
							GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..j].Name:SetText(col..StatisticIDTable[ PVEDUMP[i].statistic[j] ].."|r")
							GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..j].BossesTotal:SetText(SCAN[guid].PVEDATA[i].statisticBossEach[j])
						end
					else
						for j = 1, #PVEDUMP[i] do
							GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..PVEDUMP[i][j].name].Name:SetText(L[PVEDUMP[i][j].name])
							if PVEDUMP[i][j].achievementTotal == SCAN[guid].PVEDATA[i][j].achievementCount then
								col="|cff"..color.Green.hex else col="|cff"..color.Red.hex
							end
							GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..PVEDUMP[i][j].name].Achievement:SetText(col..SCAN[guid].PVEDATA[i][j].achievementCount.."|r / "..PVEDUMP[i][j].achievementTotal)
							if PVEDUMP[i][j].statisticTotal == SCAN[guid].PVEDATA[i][j].statisticBoss then
								col="|cff"..color.Green.hex else col="|cff"..color.Red.hex
							end
							if PVEDUMP[i][j].statisticTotal == 0 then --NOBOSS
								GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..PVEDUMP[i][j].name].Statistic:SetText("- / -")
								GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..PVEDUMP[i][j].name].BossesTotal:SetText("-")
							else
								GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..PVEDUMP[i][j].name].Statistic:SetText(col..SCAN[guid].PVEDATA[i][j].statisticBoss.."|r / "..PVEDUMP[i][j].statisticTotal)
								GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..PVEDUMP[i][j].name].BossesTotal:SetText(SCAN[guid].PVEDATA[i][j].statisticBossAll)
							end
						--end
					--end
				--end
				-- level 2 END

				-- level 3 START
				--for i = 1, #PVEDUMP do
					--if not PVEDUMP[i].statistic then
						--for j = 1, #PVEDUMP[i] do

							if PVEDUMP[i][j].statistic then
								for k = 1, #PVEDUMP[i][j].statistic do
									if SCAN[guid].PVEDATA[i][j].statisticBossEach[k] > 0 then
										col="|cff"..color.Green.hex else col="|cff"..color.Red.hex
									end
									GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k].Name:SetText(col..StatisticIDTable[ PVEDUMP[i][j].statistic[k] ].."|r")
									GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k].BossesTotal:SetText(SCAN[guid].PVEDATA[i][j].statisticBossEach[k])
								end
							else
								for k = 1, #PVEDUMP[i][j] do
									GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k].Name:SetText(L[PVEDUMP[i][j][k].name])
									if PVEDUMP[i][j][k].achievementTotal == SCAN[guid].PVEDATA[i][j][k].achievementCount then
										col="|cff"..color.Green.hex else col="|cff"..color.Red.hex
									end
									GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k].Achievement:SetText(col..SCAN[guid].PVEDATA[i][j][k].achievementCount.."|r / "..PVEDUMP[i][j][k].achievementTotal)
									if PVEDUMP[i][j][k].statisticTotal == SCAN[guid].PVEDATA[i][j][k].statisticBoss then
										col="|cff"..color.Green.hex else col="|cff"..color.Red.hex
									end
									if PVEDUMP[i][j][k].statisticTotal == 0 then --NOBOSS
										GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k].Statistic:SetText("- / -")
										GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k].BossesTotal:SetText("-")
									else
										GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k].Statistic:SetText(col..SCAN[guid].PVEDATA[i][j][k].statisticBoss.."|r / "..PVEDUMP[i][j][k].statisticTotal)
										GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k].BossesTotal:SetText(SCAN[guid].PVEDATA[i][j][k].statisticBossAll)
									end
								--end
							--end

						--end
					--end
				--end
				-- level 3 END

				-- level 4 START
				--for i = 1, #PVEDUMP do
					--if not PVEDUMP[i].statistic then
						--for j = 1, #PVEDUMP[i] do

							--if not PVEDUMP[i][j].statistic then
								--for k = 1, #PVEDUMP[i][j] do

									if PVEDUMP[i][j][k].statistic then
										for l = 1, #PVEDUMP[i][j][k].statistic do
											if SCAN[guid].PVEDATA[i][j][k].statisticBossEach[l] > 0 then
												col="|cff"..color.Green.hex else col="|cff"..color.Red.hex
											end
											
											GVAR.MainFrame.StatsPvE.ScrollFrame["L4"..PVEDUMP[i].name..PVEDUMP[i][j].name..PVEDUMP[i][j][k].name..l].Name:SetText(col..StatisticIDTable[ PVEDUMP[i][j][k].statistic[l] ].."|r")
											GVAR.MainFrame.StatsPvE.ScrollFrame["L4"..PVEDUMP[i].name..PVEDUMP[i][j].name..PVEDUMP[i][j][k].name..l].BossesTotal:SetText(SCAN[guid].PVEDATA[i][j][k].statisticBossEach[l])
										end
									end

								end
							end

						end
					end
				end
				-- level 4 END

				-- summary
				GVAR.MainFrame.Summary1PvE.Achievement:SetText(SCAN[guid].PvEAchievementCount.." / "..SCAN[guid].PvEAchievementTotal)
				GVAR.MainFrame.Summary1PvE.Achievement:SetTextColor(color.Level1.rgba[1], color.Level1.rgba[2], color.Level1.rgba[3], color.Level1.rgba[4])
				local achievement = "0.00%"
				if SCAN[guid].PvEAchievementTotal > 0 then
					achievement = cnum(math_floor((SCAN[guid].PvEAchievementCount/SCAN[guid].PvEAchievementTotal*100) * mult2 + 0.5) / mult2) .. "%"
				end
				GVAR.MainFrame.Summary2PvE.Achievement:SetText(achievement)
				GVAR.MainFrame.Summary2PvE.Achievement:SetTextColor(color.LightYellow.rgba[1], color.LightYellow.rgba[2], color.LightYellow.rgba[3], color.LightYellow.rgba[4])

				GVAR.MainFrame.Summary1PvE.Statistic:SetText(SCAN[guid].PvEStatisticCount.." / "..SCAN[guid].PvEStatisticTotal)
				GVAR.MainFrame.Summary1PvE.Statistic:SetTextColor(color.Level1.rgba[1], color.Level1.rgba[2], color.Level1.rgba[3], color.Level1.rgba[4])
				local statistic = "0.00%"
				if SCAN[guid].PvEStatisticTotal > 0 then
					statistic = cnum(math_floor((SCAN[guid].PvEStatisticCount/SCAN[guid].PvEStatisticTotal*100) * mult2 + 0.5) / mult2) .. "%"
				end
				GVAR.MainFrame.Summary2PvE.Statistic:SetText(statistic)
				GVAR.MainFrame.Summary2PvE.Statistic:SetTextColor(color.LightYellow.rgba[1], color.LightYellow.rgba[2], color.LightYellow.rgba[3], color.LightYellow.rgba[4])

				GVAR.MainFrame.Summary1PvE.BossesTotal:SetText(SCAN[guid].PvEStatisticBossKills)
				GVAR.MainFrame.Summary1PvE.BossesTotal:SetTextColor(color.Level1.rgba[1], color.Level1.rgba[2], color.Level1.rgba[3], color.Level1.rgba[4])
			end
			-- PvE END



			-- PvP START
			GVAR.MainFrame.StatsPvP.Title.Name:SetText("")
			GVAR.MainFrame.StatsPvP.Title.Achievement:SetText(L["Achievements"])
			--GVAR.MainFrame.StatsPvP.Title.WinLoss:SetText("Win / Matches") -- not needed WinLoss+WinLossPercent=title
			GVAR.MainFrame.StatsPvP.Title.WinLossPercent:SetText(L["Victories / Battles / %"])
			GVAR.MainFrame.StatsPvP.Title.HonorableKills:SetText(L["HK"])

			local col
			-- level 1 START
			for i = 1, #PVPDUMP do
				GVAR.MainFrame.StatsPvP.ScrollFrame["L1"..PVPDUMP[i].name].Name:SetText(L[PVPDUMP[i].name])
				if PVPDUMP[i].achievementTotal == SCAN[guid].PVPDATA[i].achievementCount then
					col="|cff"..color.Green.hex else col="|cff"..color.Red.hex
				end
				GVAR.MainFrame.StatsPvP.ScrollFrame["L1"..PVPDUMP[i].name].Achievement:SetText(col..SCAN[guid].PVPDATA[i].achievementCount.."|r / "..PVPDUMP[i].achievementTotal)
				local percent = ""
				local winloss = ""
				local honorkills = ""
				if PVPDUMP[i].name == "Battlegrounds" then
					percent = "0.00%"
					if SCAN[guid].PvPBattlegroundMatches > 0 then
						percent = cnum( math_floor((SCAN[guid].PvPBattlegroundWins/SCAN[guid].PvPBattlegroundMatches*100) * mult2 + 0.5) / mult2 ) .. "%"
					end
					winloss = SCAN[guid].PvPBattlegroundWins.." / "..SCAN[guid].PvPBattlegroundMatches
					honorkills = SCAN[guid].PvPBattlegroundKills
				elseif PVPDUMP[i].name == "Arena" then
					percent = "0.00%"
					if SCAN[guid].PvPArenaMatches > 0 then
						percent = cnum( math_floor((SCAN[guid].PvPArenaWins/SCAN[guid].PvPArenaMatches*100) * mult2 + 0.5) / mult2 ) .. "%"
					end
					winloss = SCAN[guid].PvPArenaWins.." / "..SCAN[guid].PvPArenaMatches
					honorkills = SCAN[guid].PvPArenaKills
				elseif PVPDUMP[i].name == "PvPGeneral" then
					percent = ""
					winloss = ""
					honorkills = SCAN[guid].PvPStatisticTotalKills
				end
				GVAR.MainFrame.StatsPvP.ScrollFrame["L1"..PVPDUMP[i].name].WinLoss:SetText(winloss)
				GVAR.MainFrame.StatsPvP.ScrollFrame["L1"..PVPDUMP[i].name].WinLossPercent:SetText(percent)
				GVAR.MainFrame.StatsPvP.ScrollFrame["L1"..PVPDUMP[i].name].HonorableKills:SetText(honorkills)
			--end
			-- level 1 END

			-- level 2 START
			--for i = 1, #PVPDUMP do
				if PVPDUMP[i].statistic then
					for j = 1, PVPDUMP[i].statisticTotal do
						local nid = PVPDUMP[i].statistic[j].matches or PVPDUMP[i].statistic[j].wins or PVPDUMP[i].statistic[j].kills or PVPDUMP[i].statistic[j].misc
						GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name..j].Name:SetText(StatisticIDTable[ nid ])
						GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name..j].HonorableKills:SetText("") -- **
					end
				else
					for j = 1, #PVPDUMP[i] do
						GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name..PVPDUMP[i][j].name].Name:SetText(L[PVPDUMP[i][j].name])
						if PVPDUMP[i][j].achievementTotal == SCAN[guid].PVPDATA[i][j].achievementCount then
							col="|cff"..color.Green.hex else col="|cff"..color.Red.hex
						end
						GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name..PVPDUMP[i][j].name].Achievement:SetText(col..SCAN[guid].PVPDATA[i][j].achievementCount.."|r / "..PVPDUMP[i][j].achievementTotal)
						local percent = ""
						local winloss = ""
						local honorkills = ""
						if PVPDUMP[i].name == "Battlegrounds" then
							if PVPDUMP[i][j].name == "RatedBG" then
								percent = ""
								winloss = ""
								honorkills = ""
							else
								percent = "0.00%"
								if SCAN[guid].PVPDATA[i][j].matches > 0 then
									percent = cnum( math_floor((SCAN[guid].PVPDATA[i][j].wins/SCAN[guid].PVPDATA[i][j].matches*100) * mult2 + 0.5) / mult2 ) .. "%"
								end
								winloss = SCAN[guid].PVPDATA[i][j].wins.." / "..SCAN[guid].PVPDATA[i][j].matches
								honorkills = SCAN[guid].PVPDATA[i][j].kills
							end
						elseif PVPDUMP[i].name == "Arena" and PVPDUMP[i][j].name ~= "ArenaGeneral" then
							percent = "0.00%"
							if SCAN[guid].PVPDATA[i][j].matches > 0 then
								percent = cnum( math_floor((SCAN[guid].PVPDATA[i][j].wins/SCAN[guid].PVPDATA[i][j].matches*100) * mult2 + 0.5) / mult2 ) .. "%"
							end
							winloss = SCAN[guid].PVPDATA[i][j].wins.." / "..SCAN[guid].PVPDATA[i][j].matches
							honorkills = SCAN[guid].PVPDATA[i][j].kills
						end
						GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name..PVPDUMP[i][j].name].WinLoss:SetText(winloss)
						GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name..PVPDUMP[i][j].name].WinLossPercent:SetText(percent)
						GVAR.MainFrame.StatsPvP.ScrollFrame["L2"..PVPDUMP[i].name..PVPDUMP[i][j].name].HonorableKills:SetText(honorkills)
					--end
				--end
			--end
			-- level 2 END

			-- level 3 START
			--for i = 1, #PVPDUMP do
				--if not PVPDUMP[i].statistic then
					--for j = 1, #PVPDUMP[i] do

						if PVPDUMP[i][j].statistic then
							for k = 1, PVPDUMP[i][j].statisticTotal do
								local nid = PVPDUMP[i][j].statistic[k].matches or PVPDUMP[i][j].statistic[k].wins or PVPDUMP[i][j].statistic[k].kills or PVPDUMP[i][j].statistic[k].misc
								GVAR.MainFrame.StatsPvP.ScrollFrame["L3"..PVPDUMP[i].name..PVPDUMP[i][j].name..k].Name:SetText(StatisticIDTable[ nid ])
								local text = ""
								if PVPDUMP[i][j].statistic[k].misc and SCAN[guid].PVPDATA[i][j].misc[k] then
									text = SCAN[guid].PVPDATA[i][j].misc[k]
								end
								GVAR.MainFrame.StatsPvP.ScrollFrame["L3"..PVPDUMP[i].name..PVPDUMP[i][j].name..k].HonorableKills:SetText(text) -- **
							end
						end

					end
				end
			end
			-- level 3 END

			-- summary
			GVAR.MainFrame.Summary1PvP.Achievement:SetText(SCAN[guid].PvPAchievementCount.." / "..SCAN[guid].PvPAchievementTotal)
			GVAR.MainFrame.Summary1PvP.Achievement:SetTextColor(color.Level1.rgba[1], color.Level1.rgba[2], color.Level1.rgba[3], color.Level1.rgba[4])
			local achievement = "0.00%"
			if SCAN[guid].PvPAchievementTotal > 0 then
				achievement = cnum(math_floor((SCAN[guid].PvPAchievementCount/SCAN[guid].PvPAchievementTotal*100) * mult2 + 0.5) / mult2) .. "%"
			end
			GVAR.MainFrame.Summary2PvP.Achievement:SetText(achievement)
			GVAR.MainFrame.Summary2PvP.Achievement:SetTextColor(color.LightYellow.rgba[1], color.LightYellow.rgba[2], color.LightYellow.rgba[3], color.LightYellow.rgba[4])

			GVAR.MainFrame.Summary1PvP.WinLoss:SetText(SCAN[guid].PvPStatisticWins.." / "..SCAN[guid].PvPStatisticMatches)
			GVAR.MainFrame.Summary1PvP.WinLoss:SetTextColor(color.Level1.rgba[1], color.Level1.rgba[2], color.Level1.rgba[3], color.Level1.rgba[4])
			local statistic = "0.00%"
			if SCAN[guid].PvPStatisticMatches > 0 then
				statistic = cnum(math_floor((SCAN[guid].PvPStatisticWins/SCAN[guid].PvPStatisticMatches*100) * mult2 + 0.5) / mult2) .. "%"
			end
			GVAR.MainFrame.Summary1PvP.WinLossPercent:SetText(statistic)
			GVAR.MainFrame.Summary1PvP.WinLossPercent:SetTextColor(color.Level1.rgba[1], color.Level1.rgba[2], color.Level1.rgba[3], color.Level1.rgba[4])

			GVAR.MainFrame.Summary1PvP.HonorableKills:SetText(SCAN[guid].PvPStatisticTotalKills)
			GVAR.MainFrame.Summary1PvP.HonorableKills:SetTextColor(color.Level1.rgba[1], color.Level1.rgba[2], color.Level1.rgba[3], color.Level1.rgba[4])
			GVAR.MainFrame.Summary2PvP.HonorableKills:SetText("("..L["Total"]..")")
			GVAR.MainFrame.Summary2PvP.HonorableKills:SetTextColor(color.ButtonBGActive.rgba[1], color.ButtonBGActive.rgba[2], color.ButtonBGActive.rgba[3], color.ButtonBGActive.rgba[4])
			-- PvP END

		end

		if scan and not GroupScanIsActive then
			GVAR.MainFrame.Status.ProgressText:SetText(L["Step"].." "..SCAN[guid].DataStep.."/4")
			GVAR.MainFrame.Status.ProgressDone:SetWidth(SCAN[guid].DataStep*sizeGeneral.StatusProgressBarWidth/4)
		end
	end

	if SCAN[guid].DataStep == 4 then

		if scan then
			-- scan complete: copy&delete preview scandata START
			local realGUID = string_sub(guid, 2)
			CreateScanDataByGUID(realGUID)
			SCAN[realGUID] = SCAN[guid] -- copy preview
			for k, v in pairs(SCAN) do
				if string_sub(k, 0, 1) == "_" then
					SCAN[k] = nil -- delete preview
				end
			end
			guid = realGUID
			-- scan complete: copy&delete preview scandata END

			-- talent group guid adjustment
			if (GroupScanIsActive and not GroupScanFirstGUID) or SingleScanIsActive or not scan then
				GVAR.TalentTreeFrame.GroupFrame.Button1:SetScript("OnClick", function() CharScanner:TalentGroup_Switch(guid, 1) end)
				GVAR.TalentTreeFrame.GroupFrame.Button2:SetScript("OnClick", function() CharScanner:TalentGroup_Switch(guid, 2) end)
			end
			-- gearframe scripts guid adjustment
			for i = 1, #slotButton do
				GVAR.GearFrame_Slot[i].IconButton:SetScript("OnEnter", function() CharScanner:GearFrame_IconButton_OnEnter(i, guid) end)
				GVAR.GearFrame_Slot[i].IconButton:SetScript("OnLeave", function() CharScanner:GearFrame_IconButton_OnLeave(i, guid) end)
				GVAR.GearFrame_Slot[i].IconButton:SetScript("OnClick", function() CharScanner:GearFrame_IconButton_OnClick(i, guid) end)
				GVAR.GearFrame_Slot[i].EnchButton:SetScript("OnEnter", function() CharScanner:GearFrame_EnchButton_OnEnter(i, guid) end)
				GVAR.GearFrame_Slot[i].EnchButton:SetScript("OnLeave", function() CharScanner:GearFrame_EnchButton_OnLeave(i, guid) end)
				GVAR.GearFrame_Slot[i].EnchButton:SetScript("OnClick", function() CharScanner:GearFrame_EnchButton_OnClick(i, guid) end)
				GVAR.GearFrame_Slot[i].ReforgeButton:SetScript("OnEnter", function() CharScanner:GearFrame_ReforgeButton_OnEnter(i, guid) end)
				GVAR.GearFrame_Slot[i].ReforgeButton:SetScript("OnLeave", function() CharScanner:GearFrame_ReforgeButton_OnLeave(i, guid) end)
				GVAR.GearFrame_Slot[i].Gem1Button:SetScript("OnEnter", function() CharScanner:GearFrame_GemButton_OnEnter(1, i, guid) end)
				GVAR.GearFrame_Slot[i].Gem1Button:SetScript("OnLeave", function() CharScanner:GearFrame_GemButton_OnLeave(1, i, guid) end)
				GVAR.GearFrame_Slot[i].Gem1Button:SetScript("OnClick", function() CharScanner:GearFrame_GemButton_OnClick(1, i, guid) end)
				GVAR.GearFrame_Slot[i].Gem2Button:SetScript("OnEnter", function() CharScanner:GearFrame_GemButton_OnEnter(2, i, guid) end)
				GVAR.GearFrame_Slot[i].Gem2Button:SetScript("OnLeave", function() CharScanner:GearFrame_GemButton_OnLeave(2, i, guid) end)
				GVAR.GearFrame_Slot[i].Gem2Button:SetScript("OnClick", function() CharScanner:GearFrame_GemButton_OnClick(2, i, guid) end)
				GVAR.GearFrame_Slot[i].Gem3Button:SetScript("OnEnter", function() CharScanner:GearFrame_GemButton_OnEnter(3, i, guid) end)
				GVAR.GearFrame_Slot[i].Gem3Button:SetScript("OnLeave", function() CharScanner:GearFrame_GemButton_OnLeave(3, i, guid) end)
				GVAR.GearFrame_Slot[i].Gem3Button:SetScript("OnClick", function() CharScanner:GearFrame_GemButton_OnClick(3, i, guid) end)
				GVAR.GearFrame_Slot[i].Gem4Button:SetScript("OnEnter", function() CharScanner:GearFrame_GemButton_OnEnter(4, i, guid) end)
				GVAR.GearFrame_Slot[i].Gem4Button:SetScript("OnLeave", function() CharScanner:GearFrame_GemButton_OnLeave(4, i, guid) end)
				GVAR.GearFrame_Slot[i].Gem4Button:SetScript("OnClick", function() CharScanner:GearFrame_GemButton_OnClick(4, i, guid) end)
			end

			SCAN[guid].scanTime = math_floor((GetTime()-SingleScanTimerStart) * mult4 + 0.5) / mult4

			if (GroupScanIsActive and not GroupScanFirstGUID) or SingleScanIsActive or not scan then
				local sTime = SCAN[guid].scanTime
				if sTime == 1 then
					sTime = string_format("%s %s", sTime, SECONDS)
				else
					sTime = sTime.." "..SECONDS
				end
				GVAR.MainFrame.Status.Text:SetText(sTime)
				GVAR.MainFrame.Status.Text:SetTextColor(color.Green.rgba[1], color.Green.rgba[2], color.Green.rgba[3], color.Green.rgba[4])
			end

			local hour, minute = GetGameTime()
			if hour < 10 then
				hour = "0"..hour
			end
			if minute < 10 then
				minute = "0"..minute
			end
			SCAN[guid].time = hour..":"..minute

			if not SCAN[guid].oorScan and
			   (
			    (CharScanner_Options.chatoutput and not GroupScanIsActive) or
			    (CharScanner_Options.chatoutput and GroupScanIsActive and not CharScanner_Options.chatoutputGrp)
			   )
			then
				local col = ClassHexColor(SCAN[guid].classEN)
				local txt= "|cff"..col..SCAN[guid].name.."|r"				
				txt = txt.." |cff"..color.Level2.hex..LEVEL.." "..SCAN[guid].level.." - "..SCAN[guid].race..",|r"
				txt = txt.." |cff"..col..SCAN[guid].class.."|r"
				if SCAN[guid].guildName then
					txt = txt.." |cff"..color.Level2.hex.."("..GUILD..": "..SCAN[guid].guildName..")|r"
				end
				Print(txt)
			end

			if (GroupScanIsActive and not GroupScanFirstGUID) or SingleScanIsActive or not scan then
				GUIselectedGUID = guid
			end

			LatestScanIsComplete = true
			if HistoryFrameIsShown or CompareFrameIsShown then
				CharScanner:History_Update()
			end
			CharScanner:UpdateGroupMissingMembers()

			if GroupScanIsActive and not GroupScanFirstGUID then
				if not HistoryFrameIsShown and not CompareFrameIsShown then
					CharScanner:HistoryFrameShow()
				end
			end

			if not GroupScanIsActive then
				CharScanner:StopSingleScan(false)
			end
		else
			GUIselectedGUID = guid
			if not GroupScanIsActive and not SingleScanIsActive then -- XFIX
				LatestScanIsComplete = true
			end -- XFIX
			local sTime = SCAN[guid].scanTime
			if sTime == 1 then
				sTime = string_format("%s %s", sTime, SECONDS)
			else
				sTime = sTime.." "..SECONDS
			end
			GVAR.MainFrame.Status.Text:SetText(L["SCANNED AT"]..": "..SCAN[guid].time.."  |cff"..color.Green.hex.."("..sTime..")|r")
			GVAR.MainFrame.Status.Text:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])
		end

		if AchievementDetailFrameMode == "PvE" then
			if GVAR.MainFrame.StatsPvE.Title.AchievementDetailShow then
				CharScanner:AchievementDetail_Update("PvE", GUI_achievementdetail_L1, GUI_achievementdetail_L2, GUI_achievementdetail_L3)
			end
		else
			if GVAR.MainFrame.StatsPvP.Title.AchievementDetailShow then
				CharScanner:AchievementDetail_Update("PvP", GUI_PvPachievementdetail_L1, GUI_PvPachievementdetail_L2, GUI_PvPachievementdetail_L3)
			end
		end

		if (GroupScanIsActive and not GroupScanFirstGUID) or SingleScanIsActive or not scan then
			for i = 1, GUI_history_maxRows do
				if GVAR.HistoryFrame.ScrollFrameButton[i].guid == guid then
					GVAR.HistoryFrame.ScrollFrameButton[i].Arrow:Show()
					break
				end
			end
		end

		TEMPLATE.EnableIconButton(GVAR.MainFrame.Headline.History)
		TEMPLATE.EnableIconButton(GVAR.MainFrame.Headline.Compare)

		GroupScanFirstGUID = guid
		GroupScanNext = true

	end

end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local GTT = CreateFrame("GameTooltip", "CharScanner_ScanningTooltip", nil, "GameTooltipTemplate")
GTT:SetOwner(CharScanner, "ANCHOR_NONE")

local function ItemData(unit, index)
	local slotID = GetInventorySlotInfo(slotButton[index].name)
	local itemLink = GetInventoryItemLink(unit, slotID)
	if not SCAN[SCANGUID].gear[index] then SCAN[SCANGUID].gear[index] = {} end

	if itemLink then
		local baseID, enchantID, gem1ID, gem2ID, gem3ID, gem4ID, suffixID, uniqueID, linkLevel, reforgeID = string_match(itemLink, "item:(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+)")
		local _, _, itemRarity, iLevel = GetItemInfo(baseID)
		--print("baseID:", baseID, type(baseID), "enchantID:", enchantID, type(enchantID), "gem1ID:", gem1ID, type(gem1ID), "gem2ID:", gem2ID, type(gem2ID), "gem3ID:", gem3ID, type(gem3ID), "gem4ID:", gem4ID, type(gem4ID), "suffixID:", suffixID, type(suffixID), "uniqueID:", uniqueID, type(uniqueID), "linkLevel:", linkLevel, type(linkLevel), "reforgeID:", reforgeID, type(reforgeID), "itemRarity:", itemRarity, type(itemRarity), "iLevel:", iLevel, type(iLevel))

		baseID    = tonumber(baseID)
		enchantID = tonumber(enchantID)
		gem1ID    = tonumber(gem1ID)
		gem2ID    = tonumber(gem2ID)
		gem3ID    = tonumber(gem3ID)
		gem4ID    = tonumber(gem4ID)
		suffixID  = tonumber(suffixID)
		uniqueID  = tonumber(uniqueID)
		linkLevel = tonumber(linkLevel)
		reforgeID = tonumber(reforgeID)

		SCAN[SCANGUID].gear[index].iID      = baseID
		SCAN[SCANGUID].gear[index].iLink    = itemLink
		SCAN[SCANGUID].gear[index].iLvl     = iLevel or 0
		SCAN[SCANGUID].gear[index].iRar     = itemRarity or 0
		SCAN[SCANGUID].gear[index].iEnch    = enchantID or 0
		SCAN[SCANGUID].gear[index].iPvP     = 0
		SCAN[SCANGUID].gear[index].iRefo    = reforgeID or 0
		SCAN[SCANGUID].gear[index].iRefoTxt = nil
		SCAN[SCANGUID].gear[index].iGem1    = nil
		SCAN[SCANGUID].gear[index].iGem2    = nil
		SCAN[SCANGUID].gear[index].iGem3    = nil
		SCAN[SCANGUID].gear[index].iGem4    = nil
		SCAN[SCANGUID].gear[index].iGemMeta = nil

		-- iGemBonusVal: <0  = Socketbonus available but wrong Gems
		-- iGemBonusVal: nil = no Socketbonus (eg. Relict or no Gems at all)
		-- iGemBonusVal: >0  = SocketBonus available
		SCAN[SCANGUID].gear[index].iGemBonus    = nil
		SCAN[SCANGUID].gear[index].iGemBonusVal = nil


		-- Gems START
		-- gem status
		-- 1: ok
		-- 2: unknown gem
		-- 3: socket, but no gem
		local gemSocketCount = 0
		local gemtmp = {}
		-- GemColor count
		--local BLUE       = 1
		--local RED        = 2
		--local YELLOW     = 3
		--local BLUEYELLOW = 4 -- GREEN
		--local REDYELLOW  = 5 -- ORANGE
		--local REDBLUE    = 6 -- PURPLE
		--local PRISMATIC  = 7 -- ALL COLORS (expect META, HYDRAULIC, COGWHEEL)
		--local META       = 8
		--local HYDRAULIC  = 9
		--local COGWHEEL   = 10

		local gem1IDid, gem1IDsocket, gem1IDval1, gem1IDstat1, gem1IDval2, gem1IDstat2, gem1IDval3, gem1IDstat3, gem1IDval4, gem1IDstat4, gem1IDval5, gem1IDstat5, gem1IDval6, gem1IDstat6
		if gem1ID > 0 then
			gemSocketCount = gemSocketCount + 1
			gem1IDid, gem1IDsocket, gem1IDval1, gem1IDstat1, gem1IDval2, gem1IDstat2, gem1IDval3, gem1IDstat3, gem1IDval4, gem1IDstat4, gem1IDval5, gem1IDstat5, gem1IDval6, gem1IDstat6 = CharScanner_GearStats_Gem(gem1ID)
			if gem1IDid then
				SCAN[SCANGUID].gear[index].iGem1 = gem1IDid

				if gem1IDsocket == 1 then
					SCAN[SCANGUID].gemBlue   = SCAN[SCANGUID].gemBlue + 1
				elseif gem1IDsocket == 2 then
					SCAN[SCANGUID].gemRed    = SCAN[SCANGUID].gemRed + 1
				elseif gem1IDsocket == 3 then
					SCAN[SCANGUID].gemYellow = SCAN[SCANGUID].gemYellow + 1
				elseif gem1IDsocket == 4 then
					SCAN[SCANGUID].gemBlue   = SCAN[SCANGUID].gemBlue + 1
					SCAN[SCANGUID].gemYellow = SCAN[SCANGUID].gemYellow + 1
				elseif gem1IDsocket == 5 then
					SCAN[SCANGUID].gemRed    = SCAN[SCANGUID].gemRed + 1
					SCAN[SCANGUID].gemYellow = SCAN[SCANGUID].gemYellow + 1
				elseif gem1IDsocket == 6 then
					SCAN[SCANGUID].gemBlue   = SCAN[SCANGUID].gemBlue + 1
					SCAN[SCANGUID].gemRed    = SCAN[SCANGUID].gemRed + 1
				elseif gem1IDsocket == 7 then
					SCAN[SCANGUID].gemBlue   = SCAN[SCANGUID].gemBlue + 1
					SCAN[SCANGUID].gemRed    = SCAN[SCANGUID].gemRed + 1
					SCAN[SCANGUID].gemYellow = SCAN[SCANGUID].gemYellow + 1
				elseif gem1IDsocket == 8 then
					SCAN[SCANGUID].gear[index].iGemMeta = gem1ID
				end

				if gem1IDsocket ~= 8 then -- MetaGem check at end of a complete gear scan

					if gem1IDstat1 then
						local stat = statGlobalName[gem1IDstat1]
						if not SCAN[SCANGUID].gearStats[stat] then
							SCAN[SCANGUID].gearStats[stat] = gem1IDval1
						else
							SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem1IDval1
						end
						if not SCAN[SCANGUID].gemTmpStats[stat] then
							SCAN[SCANGUID].gemTmpStats[stat] = gem1IDval1
						else
							SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem1IDval1
						end
						if gem1IDstat2 then
							local stat = statGlobalName[gem1IDstat2]
							if not SCAN[SCANGUID].gearStats[stat] then
								SCAN[SCANGUID].gearStats[stat] = gem1IDval2
							else
								SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem1IDval2
							end
							if not SCAN[SCANGUID].gemTmpStats[stat] then
								SCAN[SCANGUID].gemTmpStats[stat] = gem1IDval2
							else
								SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem1IDval2
							end
							if gem1IDstat3 then
								local stat = statGlobalName[gem1IDstat3]
								if not SCAN[SCANGUID].gearStats[stat] then
									SCAN[SCANGUID].gearStats[stat] = gem1IDval3
								else
									SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem1IDval3
								end
								if not SCAN[SCANGUID].gemTmpStats[stat] then
									SCAN[SCANGUID].gemTmpStats[stat] = gem1IDval3
								else
									SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem1IDval3
								end
								if gem1IDstat4 then
									local stat = statGlobalName[gem1IDstat4]
									if not SCAN[SCANGUID].gearStats[stat] then
										SCAN[SCANGUID].gearStats[stat] = gem1IDval4
									else
										SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem1IDval4
									end
									if not SCAN[SCANGUID].gemTmpStats[stat] then
										SCAN[SCANGUID].gemTmpStats[stat] = gem1IDval4
									else
										SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem1IDval4
									end
									if gem1IDstat5 then
										local stat = statGlobalName[gem1IDstat5]
										if not SCAN[SCANGUID].gearStats[stat] then
											SCAN[SCANGUID].gearStats[stat] = gem1IDval5
										else
											SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem1IDval5
										end
										if not SCAN[SCANGUID].gemTmpStats[stat] then
											SCAN[SCANGUID].gemTmpStats[stat] = gem1IDval5
										else
											SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem1IDval5
										end
										if gem1IDstat6 then
											local stat = statGlobalName[gem1IDstat6]
											if not SCAN[SCANGUID].gearStats[stat] then
												SCAN[SCANGUID].gearStats[stat] = gem1IDval6
											else
												SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem1IDval6
											end
											if not SCAN[SCANGUID].gemTmpStats[stat] then
												SCAN[SCANGUID].gemTmpStats[stat] = gem1IDval6
											else
												SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem1IDval6
											end
										end
									end
								end
							end
						end
					end

				end
				--print("gem1ID", gem1IDid, gem1IDsocket, gem1IDval1, gem1IDstat1, gem1IDval2, gem1IDstat2, gem1IDval3, gem1IDstat3, gem1IDval4, gem1IDstat4, gem1IDval5, gem1IDstat5, gem1IDval6, gem1IDstat6)
			else
				SCAN[SCANGUID].gear[index].iGem1 = 0
				--TEST code unknownGems START
				--if not CharScanner_Options.unknownGems then CharScanner_Options.unknownGems = {} end
				--if not CharScanner_Options.unknownGems[gem1ID] then CharScanner_Options.unknownGems[gem1ID] = 1 end
				--TEST code unknownGems END
			end
		end
		local gem2IDid, gem2IDsocket, gem2IDval1, gem2IDstat1, gem2IDval2, gem2IDstat2, gem2IDval3, gem2IDstat3, gem2IDval4, gem2IDstat4, gem2IDval5, gem2IDstat5, gem2IDval6, gem2IDstat6
		if gem2ID > 0 then
			gemSocketCount = gemSocketCount + 1
			gem2IDid, gem2IDsocket, gem2IDval1, gem2IDstat1, gem2IDval2, gem2IDstat2, gem2IDval3, gem2IDstat3, gem2IDval4, gem2IDstat4, gem2IDval5, gem2IDstat5, gem2IDval6, gem2IDstat6 = CharScanner_GearStats_Gem(gem2ID)
			if gem2IDid then
				SCAN[SCANGUID].gear[index].iGem2 = gem2IDid

				if gem2IDsocket == 1 then
					SCAN[SCANGUID].gemBlue   = SCAN[SCANGUID].gemBlue + 1
				elseif gem2IDsocket == 2 then
					SCAN[SCANGUID].gemRed    = SCAN[SCANGUID].gemRed + 1
				elseif gem2IDsocket == 3 then
					SCAN[SCANGUID].gemYellow = SCAN[SCANGUID].gemYellow + 1
				elseif gem2IDsocket == 4 then
					SCAN[SCANGUID].gemBlue   = SCAN[SCANGUID].gemBlue + 1
					SCAN[SCANGUID].gemYellow = SCAN[SCANGUID].gemYellow + 1
				elseif gem2IDsocket == 5 then
					SCAN[SCANGUID].gemRed    = SCAN[SCANGUID].gemRed + 1
					SCAN[SCANGUID].gemYellow = SCAN[SCANGUID].gemYellow + 1
				elseif gem2IDsocket == 6 then
					SCAN[SCANGUID].gemBlue   = SCAN[SCANGUID].gemBlue + 1
					SCAN[SCANGUID].gemRed    = SCAN[SCANGUID].gemRed + 1
				elseif gem2IDsocket == 7 then
					SCAN[SCANGUID].gemBlue   = SCAN[SCANGUID].gemBlue + 1
					SCAN[SCANGUID].gemRed    = SCAN[SCANGUID].gemRed + 1
					SCAN[SCANGUID].gemYellow = SCAN[SCANGUID].gemYellow + 1
				elseif gem2IDsocket == 8 then
					SCAN[SCANGUID].gear[index].iGemMeta = gem2ID
				end

				if gem2IDsocket ~= 8 then -- MetaGem check at end of a complete gear scan

					if gem2IDstat1 then
						local stat = statGlobalName[gem2IDstat1]
						if not SCAN[SCANGUID].gearStats[stat] then
							SCAN[SCANGUID].gearStats[stat] = gem2IDval1
						else
							SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem2IDval1
						end
						if not SCAN[SCANGUID].gemTmpStats[stat] then
							SCAN[SCANGUID].gemTmpStats[stat] = gem2IDval1
						else
							SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem2IDval1
						end
						if gem2IDstat2 then
							local stat = statGlobalName[gem2IDstat2]
							if not SCAN[SCANGUID].gearStats[stat] then
								SCAN[SCANGUID].gearStats[stat] = gem2IDval2
							else
								SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem2IDval2
							end
							if not SCAN[SCANGUID].gemTmpStats[stat] then
								SCAN[SCANGUID].gemTmpStats[stat] = gem2IDval2
							else
								SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem2IDval2
							end
							if gem2IDstat3 then
								local stat = statGlobalName[gem2IDstat3]
								if not SCAN[SCANGUID].gearStats[stat] then
									SCAN[SCANGUID].gearStats[stat] = gem2IDval3
								else
									SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem2IDval3
								end
								if not SCAN[SCANGUID].gemTmpStats[stat] then
									SCAN[SCANGUID].gemTmpStats[stat] = gem2IDval3
								else
									SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem2IDval3
								end
								if gem2IDstat4 then
									local stat = statGlobalName[gem2IDstat4]
									if not SCAN[SCANGUID].gearStats[stat] then
										SCAN[SCANGUID].gearStats[stat] = gem2IDval4
									else
										SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem2IDval4
									end
									if not SCAN[SCANGUID].gemTmpStats[stat] then
										SCAN[SCANGUID].gemTmpStats[stat] = gem2IDval4
									else
										SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem2IDval4
									end
									if gem2IDstat5 then
										local stat = statGlobalName[gem2IDstat5]
										if not SCAN[SCANGUID].gearStats[stat] then
											SCAN[SCANGUID].gearStats[stat] = gem2IDval5
										else
											SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem2IDval5
										end
										if not SCAN[SCANGUID].gemTmpStats[stat] then
											SCAN[SCANGUID].gemTmpStats[stat] = gem2IDval5
										else
											SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem2IDval5
										end
										if gem2IDstat6 then
											local stat = statGlobalName[gem2IDstat6]
											if not SCAN[SCANGUID].gearStats[stat] then
												SCAN[SCANGUID].gearStats[stat] = gem2IDval6
											else
												SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem2IDval6
											end
											if not SCAN[SCANGUID].gemTmpStats[stat] then
												SCAN[SCANGUID].gemTmpStats[stat] = gem2IDval6
											else
												SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem2IDval6
											end
										end
									end
								end
							end
						end
					end

				end
				--print("gem2ID", gem2IDid, gem2IDsocket, gem2IDval1, gem2IDstat1, gem2IDval2, gem2IDstat2, gem2IDval3, gem2IDstat3, gem2IDval4, gem2IDstat4, gem2IDval5, gem2IDstat5, gem2IDval6, gem2IDstat6)
			else
				SCAN[SCANGUID].gear[index].iGem2 = 0
				--TEST code unknownGems START
				--if not CharScanner_Options.unknownGems then CharScanner_Options.unknownGems = {} end
				--if not CharScanner_Options.unknownGems[gem2ID] then CharScanner_Options.unknownGems[gem2ID] = 1 end
				--TEST code unknownGems END
			end
		end
		local gem3IDid, gem3IDsocket, gem3IDval1, gem3IDstat1, gem3IDval2, gem3IDstat2, gem3IDval3, gem3IDstat3, gem3IDval4, gem3IDstat4, gem3IDval5, gem3IDstat5, gem3IDval6, gem3IDstat6
		if gem3ID > 0 then
			gemSocketCount = gemSocketCount + 1
			gem3IDid, gem3IDsocket, gem3IDval1, gem3IDstat1, gem3IDval2, gem3IDstat2, gem3IDval3, gem3IDstat3, gem3IDval4, gem3IDstat4, gem3IDval5, gem3IDstat5, gem3IDval6, gem3IDstat6 = CharScanner_GearStats_Gem(gem3ID)
			if gem3IDid then
				SCAN[SCANGUID].gear[index].iGem3 = gem3IDid

				if gem3IDsocket == 1 then
					SCAN[SCANGUID].gemBlue   = SCAN[SCANGUID].gemBlue + 1
				elseif gem3IDsocket == 2 then
					SCAN[SCANGUID].gemRed    = SCAN[SCANGUID].gemRed + 1
				elseif gem3IDsocket == 3 then
					SCAN[SCANGUID].gemYellow = SCAN[SCANGUID].gemYellow + 1
				elseif gem3IDsocket == 4 then
					SCAN[SCANGUID].gemBlue   = SCAN[SCANGUID].gemBlue + 1
					SCAN[SCANGUID].gemYellow = SCAN[SCANGUID].gemYellow + 1
				elseif gem3IDsocket == 5 then
					SCAN[SCANGUID].gemRed    = SCAN[SCANGUID].gemRed + 1
					SCAN[SCANGUID].gemYellow = SCAN[SCANGUID].gemYellow + 1
				elseif gem3IDsocket == 6 then
					SCAN[SCANGUID].gemBlue   = SCAN[SCANGUID].gemBlue + 1
					SCAN[SCANGUID].gemRed    = SCAN[SCANGUID].gemRed + 1
				elseif gem3IDsocket == 7 then
					SCAN[SCANGUID].gemBlue   = SCAN[SCANGUID].gemBlue + 1
					SCAN[SCANGUID].gemRed    = SCAN[SCANGUID].gemRed + 1
					SCAN[SCANGUID].gemYellow = SCAN[SCANGUID].gemYellow + 1
				elseif gem3IDsocket == 8 then
					SCAN[SCANGUID].gear[index].iGemMeta = gem3ID
				end

				if gem3IDsocket ~= 8 then -- MetaGem check at end of a complete gear scan

					if gem3IDstat1 then
						local stat = statGlobalName[gem3IDstat1]
						if not SCAN[SCANGUID].gearStats[stat] then
							SCAN[SCANGUID].gearStats[stat] = gem3IDval1
						else
							SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem3IDval1
						end
						if not SCAN[SCANGUID].gemTmpStats[stat] then
							SCAN[SCANGUID].gemTmpStats[stat] = gem3IDval1
						else
							SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem3IDval1
						end
						if gem3IDstat2 then
							local stat = statGlobalName[gem3IDstat2]
							if not SCAN[SCANGUID].gearStats[stat] then
								SCAN[SCANGUID].gearStats[stat] = gem3IDval2
							else
								SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem3IDval2
							end
							if not SCAN[SCANGUID].gemTmpStats[stat] then
								SCAN[SCANGUID].gemTmpStats[stat] = gem3IDval2
							else
								SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem3IDval2
							end
							if gem3IDstat3 then
								local stat = statGlobalName[gem3IDstat3]
								if not SCAN[SCANGUID].gearStats[stat] then
									SCAN[SCANGUID].gearStats[stat] = gem3IDval3
								else
									SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem3IDval3
								end
								if not SCAN[SCANGUID].gemTmpStats[stat] then
									SCAN[SCANGUID].gemTmpStats[stat] = gem3IDval3
								else
									SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem3IDval3
								end
								if gem3IDstat4 then
									local stat = statGlobalName[gem3IDstat4]
									if not SCAN[SCANGUID].gearStats[stat] then
										SCAN[SCANGUID].gearStats[stat] = gem3IDval4
									else
										SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem3IDval4
									end
									if not SCAN[SCANGUID].gemTmpStats[stat] then
										SCAN[SCANGUID].gemTmpStats[stat] = gem3IDval4
									else
										SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem3IDval4
									end
									if gem3IDstat5 then
										local stat = statGlobalName[gem3IDstat5]
										if not SCAN[SCANGUID].gearStats[stat] then
											SCAN[SCANGUID].gearStats[stat] = gem3IDval5
										else
											SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem3IDval5
										end
										if not SCAN[SCANGUID].gemTmpStats[stat] then
											SCAN[SCANGUID].gemTmpStats[stat] = gem3IDval5
										else
											SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem3IDval5
										end
										if gem3IDstat6 then
											local stat = statGlobalName[gem3IDstat6]
											if not SCAN[SCANGUID].gearStats[stat] then
												SCAN[SCANGUID].gearStats[stat] = gem3IDval6
											else
												SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem3IDval6
											end
											if not SCAN[SCANGUID].gemTmpStats[stat] then
												SCAN[SCANGUID].gemTmpStats[stat] = gem3IDval6
											else
												SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem3IDval6
											end
										end
									end
								end
							end
						end
					end

				end
				--print("gem3ID", gem3IDid, gem3IDsocket, gem3IDval1, gem3IDstat1, gem3IDval2, gem3IDstat2, gem3IDval3, gem3IDstat3, gem3IDval4, gem3IDstat4, gem3IDval5, gem3IDstat5, gem3IDval6, gem3IDstat6)
			else
				SCAN[SCANGUID].gear[index].iGem3 = 0
				--TEST code unknownGems START
				--if not CharScanner_Options.unknownGems then CharScanner_Options.unknownGems = {} end
				--if not CharScanner_Options.unknownGems[gem3ID] then CharScanner_Options.unknownGems[gem3ID] = 1 end
				--TEST code unknownGems END
			end
		end
		local gem4IDid, gem4IDsocket, gem4IDval1, gem4IDstat1, gem4IDval2, gem4IDstat2, gem4IDval3, gem4IDstat3, gem4IDval4, gem4IDstat4, gem4IDval5, gem4IDstat5, gem4IDval6, gem4IDstat6
		if gem4ID > 0 then
			gemSocketCount = gemSocketCount + 1
			gem4IDid, gem4IDsocket, gem4IDval1, gem4IDstat1, gem4IDval2, gem4IDstat2, gem4IDval3, gem4IDstat3, gem4IDval4, gem4IDstat4, gem4IDval5, gem4IDstat5, gem4IDval6, gem4IDstat6 = CharScanner_GearStats_Gem(gem4ID)
			if gem4IDid then
				SCAN[SCANGUID].gear[index].iGem4 = gem4IDid

				if gem4IDsocket == 1 then
					SCAN[SCANGUID].gemBlue   = SCAN[SCANGUID].gemBlue + 1
				elseif gem4IDsocket == 2 then
					SCAN[SCANGUID].gemRed    = SCAN[SCANGUID].gemRed + 1
				elseif gem4IDsocket == 3 then
					SCAN[SCANGUID].gemYellow = SCAN[SCANGUID].gemYellow + 1
				elseif gem4IDsocket == 4 then
					SCAN[SCANGUID].gemBlue   = SCAN[SCANGUID].gemBlue + 1
					SCAN[SCANGUID].gemYellow = SCAN[SCANGUID].gemYellow + 1
				elseif gem4IDsocket == 5 then
					SCAN[SCANGUID].gemRed    = SCAN[SCANGUID].gemRed + 1
					SCAN[SCANGUID].gemYellow = SCAN[SCANGUID].gemYellow + 1
				elseif gem4IDsocket == 6 then
					SCAN[SCANGUID].gemBlue   = SCAN[SCANGUID].gemBlue + 1
					SCAN[SCANGUID].gemRed    = SCAN[SCANGUID].gemRed + 1
				elseif gem4IDsocket == 7 then
					SCAN[SCANGUID].gemBlue   = SCAN[SCANGUID].gemBlue + 1
					SCAN[SCANGUID].gemRed    = SCAN[SCANGUID].gemRed + 1
					SCAN[SCANGUID].gemYellow = SCAN[SCANGUID].gemYellow + 1
				elseif gem4IDsocket == 8 then
					SCAN[SCANGUID].gear[index].iGemMeta = gem4ID
				end

				if gem4IDsocket ~= 8 then -- MetaGem check at end of a complete gear scan

					if gem4IDstat1 then
						local stat = statGlobalName[gem4IDstat1]
						if not SCAN[SCANGUID].gearStats[stat] then
							SCAN[SCANGUID].gearStats[stat] = gem4IDval1
						else
							SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem4IDval1
						end
						if not SCAN[SCANGUID].gemTmpStats[stat] then
							SCAN[SCANGUID].gemTmpStats[stat] = gem4IDval1
						else
							SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem4IDval1
						end
						if gem4IDstat2 then
							local stat = statGlobalName[gem4IDstat2]
							if not SCAN[SCANGUID].gearStats[stat] then
								SCAN[SCANGUID].gearStats[stat] = gem4IDval2
							else
								SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem4IDval2
							end
							if not SCAN[SCANGUID].gemTmpStats[stat] then
								SCAN[SCANGUID].gemTmpStats[stat] = gem4IDval2
							else
								SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem4IDval2
							end
							if gem4IDstat3 then
								local stat = statGlobalName[gem4IDstat3]
								if not SCAN[SCANGUID].gearStats[stat] then
									SCAN[SCANGUID].gearStats[stat] = gem4IDval3
								else
									SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem4IDval3
								end
								if not SCAN[SCANGUID].gemTmpStats[stat] then
									SCAN[SCANGUID].gemTmpStats[stat] = gem4IDval3
								else
									SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem4IDval3
								end
								if gem4IDstat4 then
									local stat = statGlobalName[gem4IDstat4]
									if not SCAN[SCANGUID].gearStats[stat] then
										SCAN[SCANGUID].gearStats[stat] = gem4IDval4
									else
										SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem4IDval4
									end
									if not SCAN[SCANGUID].gemTmpStats[stat] then
										SCAN[SCANGUID].gemTmpStats[stat] = gem4IDval4
									else
										SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem4IDval4
									end
									if gem4IDstat5 then
										local stat = statGlobalName[gem4IDstat5]
										if not SCAN[SCANGUID].gearStats[stat] then
											SCAN[SCANGUID].gearStats[stat] = gem4IDval5
										else
											SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem4IDval5
										end
										if not SCAN[SCANGUID].gemTmpStats[stat] then
											SCAN[SCANGUID].gemTmpStats[stat] = gem4IDval5
										else
											SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem4IDval5
										end
										if gem4IDstat6 then
											local stat = statGlobalName[gem4IDstat6]
											if not SCAN[SCANGUID].gearStats[stat] then
												SCAN[SCANGUID].gearStats[stat] = gem4IDval6
											else
												SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem4IDval6
											end
											if not SCAN[SCANGUID].gemTmpStats[stat] then
												SCAN[SCANGUID].gemTmpStats[stat] = gem4IDval6
											else
												SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem4IDval6
											end
										end
									end
								end
							end
						end
					end

				end
				--print("gem4ID", gem4IDid, gem4IDsocket, gem4IDval1, gem4IDstat1, gem4IDval2, gem4IDstat2, gem4IDval3, gem4IDstat3, gem4IDval4, gem4IDstat4, gem4IDval5, gem4IDstat5, gem4IDval6, gem4IDstat6)
			else
				SCAN[SCANGUID].gear[index].iGem4 = 0
				--TEST code unknownGems START
				--if not CharScanner_Options.unknownGems then CharScanner_Options.unknownGems = {} end
				--if not CharScanner_Options.unknownGems[gem4ID] then CharScanner_Options.unknownGems[gem4ID] = 1 end
				--TEST code unknownGems END
			end
		end
		-- Gems END

		local reforgeMinus, reforgePlus 
		if reforgeID > 0 then
			reforgeMinus, reforgePlus = CharScanner_GearStats_Reforge(reforgeID)
			SCAN[SCANGUID].gear[index].iRefo = 1
		end

		local gemSocketStatsCount = 0
		for key, value in pairs(GetItemStats(itemLink)) do
			if key and value and type(value) == "number" then
				if not SCAN[SCANGUID].gearStats[key] then
					SCAN[SCANGUID].gearStats[key] = value
				else
					SCAN[SCANGUID].gearStats[key] = SCAN[SCANGUID].gearStats[key] + value
				end

				if gemSockets[key] then
					gemSocketStatsCount = gemSocketStatsCount + value
				elseif key == "ITEM_MOD_RESILIENCE_RATING_SHORT" then
					SCAN[SCANGUID].gear[index].iPvP = 1
				elseif key == reforgeMinus then
					local newvalue = math_floor(value/2.5)
					SCAN[SCANGUID].gear[index].iRefoTxt = "|cff"..color.Red.hex.."-"..newvalue.." ".._G[reforgeMinus].."|r -> |cff"..color.Green.hex.."+"..newvalue.." ".._G[reforgePlus].."|r"

					SCAN[SCANGUID].gearStats[key] = SCAN[SCANGUID].gearStats[key] - newvalue

					if not SCAN[SCANGUID].gearStats[reforgePlus] then
						SCAN[SCANGUID].gearStats[reforgePlus] = newvalue
					else
						SCAN[SCANGUID].gearStats[reforgePlus] = SCAN[SCANGUID].gearStats[reforgePlus] + newvalue
					end
					if not SCAN[SCANGUID].refoMinStats[reforgeMinus] then
						SCAN[SCANGUID].refoMinStats[reforgeMinus] = newvalue
					else
						SCAN[SCANGUID].refoMinStats[reforgeMinus] = SCAN[SCANGUID].refoMinStats[reforgeMinus] + newvalue
					end
					if not SCAN[SCANGUID].refoPluStats[reforgePlus] then
						SCAN[SCANGUID].refoPluStats[reforgePlus] = newvalue
					else
						SCAN[SCANGUID].refoPluStats[reforgePlus] = SCAN[SCANGUID].refoPluStats[reforgePlus] + newvalue
					end
				end
			end
		end
		
		-- check if an additional gem socket was applied to hand (blacksmiths only) or waist (all chars)
		-- NOTE:
		-- there is no way to get an empty additional gem socket (expect you wear such an item), because
		-- the itemLink is only changed if this socket has a gem
		if gemSocketCount > gemSocketStatsCount then
			SCAN[SCANGUID].gemTotal = SCAN[SCANGUID].gemTotal + gemSocketCount
		else
			SCAN[SCANGUID].gemTotal = SCAN[SCANGUID].gemTotal + gemSocketStatsCount
		end
		

		-- SocketBonus
		-- :SetHyperlink(itemLink) :GetTextColor() green/grey color does not work 100%. the following method does :)
		local gemSocket1, gemSocket2, gemSocket3, gemSocket4
		local x = 1
		if gemSocketStatsCount > 0 then
			GTT:SetOwner(CharScanner, "ANCHOR_NONE")
			GTT:SetHyperlink("item:"..baseID)

			local lineCount = GTT:NumLines()
			for i = 2, lineCount do
				local glob = _G["CharScanner_ScanningTooltipTextLeft"..i]
				if glob then
					local text = glob:GetText()
					if text then
						if text == gemLocSocketsBLUE then
							if x == 1 then
								gemSocket1 = 1
							elseif x == 2 then
								gemSocket2 = 1
							elseif x == 3 then
								gemSocket3 = 1
							else
								gemSocket4 = 1
							end
							x = x + 1
						elseif text == gemLocSocketsRED then
							if x == 1 then
								gemSocket1 = 2
							elseif x == 2 then
								gemSocket2 = 2
							elseif x == 3 then
								gemSocket3 = 2
							else
								gemSocket4 = 2
							end
							x = x + 1
						elseif text == gemLocSocketsYELLOW then
							if x == 1 then
								gemSocket1 = 3
							elseif x == 2 then
								gemSocket2 = 3
							elseif x == 3 then
								gemSocket3 = 3
							else
								gemSocket4 = 3
							end
							x = x + 1
						elseif text == gemLocSocketsMETA then
							if x == 1 then
								gemSocket1 = 4
							elseif x == 2 then
								gemSocket2 = 4
							elseif x == 3 then
								gemSocket3 = 4
							else
								gemSocket4 = 4
							end
							x = x + 1
						elseif text == gemLocSocketsPRISMATIC then
							if x == 1 then
								gemSocket1 = 4
							elseif x == 2 then
								gemSocket2 = 4
							elseif x == 3 then
								gemSocket3 = 4
							else
								gemSocket4 = 4
							end
							x = x + 1
						elseif text == gemLocSocketsCOGWHEEL then
							if x == 1 then
								gemSocket1 = 4
							elseif x == 2 then
								gemSocket2 = 4
							elseif x == 3 then
								gemSocket3 = 4
							else
								gemSocket4 = 4
							end
							x = x + 1
						elseif text == gemLocSocketsHYDRAULIC then
							if x == 1 then
								gemSocket1 = 4
							elseif x == 2 then
								gemSocket2 = 4
							elseif x == 3 then
								gemSocket3 = 4
							else
								gemSocket4 = 4
							end
							x = x + 1
						elseif text == gemLocSocketsNO_COLOR then
							if x == 1 then
								gemSocket1 = 4
							elseif x == 2 then
								gemSocket2 = 4
							elseif x == 3 then
								gemSocket3 = 4
							else
								gemSocket4 = 4
							end
							x = x + 1
						elseif string_match(text, bonusStr) then

							local purebonus = string_gsub(text, bonusStr, "")
							local value, name = string_match(purebonus, "(%d+) (.+)")

							SCAN[SCANGUID].gear[index].iGemBonus    = "-"
							SCAN[SCANGUID].gear[index].iGemBonusVal = -1

							for ii = 1, #statLocName do
								if statLocName[ii] == name then
									local realvalue = tonumber(value) or -1
									if realvalue > 0 then

										local hasBonus = GemSocketCheck(gemSocket1, gemSocket2, gemSocket3, gemSocket4, gem1IDsocket, gem2IDsocket, gem3IDsocket, gem4IDsocket)

										if hasBonus then
											SCAN[SCANGUID].gear[index].iGemBonus    = statLocName[ii]
											SCAN[SCANGUID].gear[index].iGemBonusVal = realvalue
											if not SCAN[SCANGUID].gearStats[ statGlobalName[ii] ] then
												SCAN[SCANGUID].gearStats[ statGlobalName[ii] ] = realvalue
											else
												SCAN[SCANGUID].gearStats[ statGlobalName[ii] ] = SCAN[SCANGUID].gearStats[ statGlobalName[ii] ] + realvalue
											end
											if not SCAN[SCANGUID].gemTmpStats[ statGlobalName[ii] ] then
												SCAN[SCANGUID].gemTmpStats[ statGlobalName[ii] ] = realvalue
											else
												SCAN[SCANGUID].gemTmpStats[ statGlobalName[ii] ] = SCAN[SCANGUID].gemTmpStats[ statGlobalName[ii] ] + realvalue
											end
										else
											SCAN[SCANGUID].gear[index].iGemBonus    = "-"
											SCAN[SCANGUID].gear[index].iGemBonusVal = -1
										end

									end
									break
								end
							end
							break -- break line scan here (after socket string match)
						end
					end
				end
			end

			GTT:Hide()
		end

		-- gem status
		-- 1: ok
		-- 2: unknown gem
		-- 3: socket, but no gem
		-- get missing gems
		-- major if-elseif-else brain damage inc.
		-- this madness is necessary because the itemLink _can_ contain 0:0:0:xxxx (gem1ID:gem2ID:gem3ID:gem4ID)
		local missingGems = gemSocketStatsCount - gemSocketCount
		if missingGems > 0 then
			-- 1 Gem missing START
			if missingGems == 1 then
				SCAN[SCANGUID].gemMissing = SCAN[SCANGUID].gemMissing + 1
				if gemSocketStatsCount == 1 then -- 1 gem socket available
					SCAN[SCANGUID].gear[index].iGem1 = -1
				elseif gemSocketStatsCount == 2 then -- 2 gem sockets available
					if SCAN[SCANGUID].gear[index].iGem1 then
						SCAN[SCANGUID].gear[index].iGem2 = -1
					else
						SCAN[SCANGUID].gear[index].iGem1 = -1
					end
				elseif gemSocketStatsCount == 3 then -- 3 gem sockets available
					if SCAN[SCANGUID].gear[index].iGem1 then
						if SCAN[SCANGUID].gear[index].iGem2 then
							SCAN[SCANGUID].gear[index].iGem3 = -1
						else
							SCAN[SCANGUID].gear[index].iGem2 = -1
						end
					else
						SCAN[SCANGUID].gear[index].iGem1 = -1
					end
				elseif gemSocketStatsCount == 4 then -- 4 gem sockets available
					if SCAN[SCANGUID].gear[index].iGem1 then
						if SCAN[SCANGUID].gear[index].iGem2 then
							if SCAN[SCANGUID].gear[index].iGem3 then
								SCAN[SCANGUID].gear[index].iGem4 = -1
							else
								SCAN[SCANGUID].gear[index].iGem3 = -1
							end
						else
							if SCAN[SCANGUID].gear[index].iGem3 then
								SCAN[SCANGUID].gear[index].iGem4 = -1
							else
								SCAN[SCANGUID].gear[index].iGem3 = -1
							end
						end
					else
						SCAN[SCANGUID].gear[index].iGem1 = -1
					end
				end
			-- 1 Gem missing END
			-- 2 Gems missing START
			elseif missingGems == 2 then
				SCAN[SCANGUID].gemMissing = SCAN[SCANGUID].gemMissing + 2
				if gemSocketStatsCount == 2 then -- 2 gem sockets available
					SCAN[SCANGUID].gear[index].iGem1 = -1
					SCAN[SCANGUID].gear[index].iGem2 = -1
				elseif gemSocketStatsCount == 3 then -- 3 gem sockets available
					if SCAN[SCANGUID].gear[index].iGem1 then
						SCAN[SCANGUID].gear[index].iGem2 = -1
						SCAN[SCANGUID].gear[index].iGem3 = -1
					else
						SCAN[SCANGUID].gear[index].iGem1 = -1
						if SCAN[SCANGUID].gear[index].iGem2 then
							SCAN[SCANGUID].gear[index].iGem3 = -1
						else
							SCAN[SCANGUID].gear[index].iGem2 = -1
						end
					end
				elseif gemSocketStatsCount == 4 then -- 4 gem sockets available
					if SCAN[SCANGUID].gear[index].iGem1 then
						if SCAN[SCANGUID].gear[index].iGem2 then
							SCAN[SCANGUID].gear[index].iGem3 = -1
							SCAN[SCANGUID].gear[index].iGem4 = -1
						else
							SCAN[SCANGUID].gear[index].iGem2 = -1
							if SCAN[SCANGUID].gear[index].iGem3 then
								SCAN[SCANGUID].gear[index].iGem4 = -1
							else
								SCAN[SCANGUID].gear[index].iGem3 = -1
							end
						end
					else
						SCAN[SCANGUID].gear[index].iGem1 = -1
						if SCAN[SCANGUID].gear[index].iGem2 then
							if SCAN[SCANGUID].gear[index].iGem3 then
								SCAN[SCANGUID].gear[index].iGem4 = -1
							else
								SCAN[SCANGUID].gear[index].iGem3 = -1
							end
						else
							SCAN[SCANGUID].gear[index].iGem2 = -1
						end
					end
				end
			-- 2 Gems missing END
			-- 3 Gems missing START
			elseif missingGems == 3 then
				SCAN[SCANGUID].gemMissing = SCAN[SCANGUID].gemMissing + 3
				if gemSocketStatsCount == 3 then -- 3 gem sockets available
					SCAN[SCANGUID].gear[index].iGem1 = -1
					SCAN[SCANGUID].gear[index].iGem2 = -1
					SCAN[SCANGUID].gear[index].iGem3 = -1
				elseif gemSocketStatsCount == 4 then -- 4 gem sockets available
					if SCAN[SCANGUID].gear[index].iGem1 then
						SCAN[SCANGUID].gear[index].iGem2 = -1
						SCAN[SCANGUID].gear[index].iGem3 = -1
						SCAN[SCANGUID].gear[index].iGem4 = -1
					else
						SCAN[SCANGUID].gear[index].iGem1 = -1
						if SCAN[SCANGUID].gear[index].iGem2 then
							SCAN[SCANGUID].gear[index].iGem3 = -1
							SCAN[SCANGUID].gear[index].iGem4 = -1
						else
							SCAN[SCANGUID].gear[index].iGem2 = -1
							if SCAN[SCANGUID].gear[index].iGem3 then
								SCAN[SCANGUID].gear[index].iGem4 = -1
							else
								SCAN[SCANGUID].gear[index].iGem3 = -1
							end
						end
					end
				end
			-- 3 Gems missing END
			-- 4 Gems missing START
			elseif missingGems == 4 then
				SCAN[SCANGUID].gemMissing = SCAN[SCANGUID].gemMissing + 4
				SCAN[SCANGUID].gear[index].iGem1 = -1
				SCAN[SCANGUID].gear[index].iGem2 = -1
				SCAN[SCANGUID].gear[index].iGem3 = -1
				SCAN[SCANGUID].gear[index].iGem4 = -1
			-- 4 Gems missing END
			end
		end

	else
		SCAN[SCANGUID].gear[index].iID      = nil
		SCAN[SCANGUID].gear[index].iLink    = nil
		SCAN[SCANGUID].gear[index].iLvl     = -1
		SCAN[SCANGUID].gear[index].iRar     = -1
		SCAN[SCANGUID].gear[index].iEnch    = -1
		SCAN[SCANGUID].gear[index].iPvP     = -1
		SCAN[SCANGUID].gear[index].iRefo    = -1
		SCAN[SCANGUID].gear[index].iRefoTxt = nil
		SCAN[SCANGUID].gear[index].iGem1    = nil
		SCAN[SCANGUID].gear[index].iGem2    = nil
		SCAN[SCANGUID].gear[index].iGem3    = nil
		SCAN[SCANGUID].gear[index].iGem4    = nil

		SCAN[SCANGUID].gear[index].iGemBonus    = nil
		SCAN[SCANGUID].gear[index].iGemBonusVal = nil

		local textureName = GetInventoryItemTexture(unit, slotID);
		if textureName then
			GTT:ClearLines()
			GTT:SetInventoryItem(unit, slotID)
			local itemName, itemLink = GTT:GetItem()
			if itemLink then
				local baseID, enchantID, gem1ID, gem2ID, gem3ID, gem4ID, suffixID, uniqueID, linkLevel, reforgeID = string_match(itemLink, "item:(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+)")
				local _, _, itemRarity, iLevel = GetItemInfo(baseID)
				if baseID and enchantID then
					SCAN[SCANGUID].gear[index].iID   = baseID
					SCAN[SCANGUID].gear[index].iLink = "item:"..baseID..":"..enchantID
					SCAN[SCANGUID].gear[index].iLvl  = tonumber(iLevel) or 0
					SCAN[SCANGUID].gear[index].iRar  = tonumber(itemRarity) or 8
					SCAN[SCANGUID].gear[index].iEnch = tonumber(enchantID) or 0
					SCAN[SCANGUID].gear[index].iPvP  = 0
					SCAN[SCANGUID].gear[index].iRefo = tonumber(reforgeID) or 0
				end
			end
		end

	end
	return SCAN[SCANGUID].gear[index].iLvl, SCAN[SCANGUID].gear[index].iRar, SCAN[SCANGUID].gear[index].iEnch, SCAN[SCANGUID].gear[index].iPvP
end

local function TwoHandWeaponCheck(unit, who)
	local id = GetInventoryItemLink(unit, _G[who.."MainHandSlot"]:GetID())
	local twohandweapon = false
	if id then
		local linkID = select(3, string_find(id, "item:(%d+)"))
		local itemEquipLoc = select(9, GetItemInfo(linkID))
		if itemEquipLoc == "INVTYPE_2HWEAPON" then
			local id2 = GetInventoryItemLink(unit, _G[who.."SecondaryHandSlot"]:GetID())
			if not id2 then
				twohandweapon = true
			end
		end
	end
	return twohandweapon
end

function CharScanner:GearScanPost(unit)
	for i = 1, #slotButton do
		local slotID = GetInventorySlotInfo(slotButton[i].name)
		local textureName = GetInventoryItemTexture(unit, slotID);
		if textureName then
			GTT:ClearLines()
			GTT:SetInventoryItem(unit, slotID)
			local itemName, itemLink = GTT:GetItem()
		end
	end
end

function CharScanner:ItemLevelCalc(unit, who)
	CharScanner:GearScanPost(unit)

	local SlotCount = 17
	if TwoHandWeaponCheck(unit, who) then
		SlotCount = 16
	end
	local ItemLevel = 0
	local ItemCount = 0
	local minItem = 0
	local maxItem = 0
	local EnchCount = 0
	local PvPCount = 0
	local MetaGemID

	for i = 1, itemQualityNum do
		itemQuality[i].count = 0
	end

	for i = 1, #slotButton do
		local iLvL, iRarity, iEnch, iPvP = ItemData(unit, i)
		if iLvL >= 0 then
			ItemLevel = ItemLevel + iLvL
			ItemCount = ItemCount + 1
		end
		if iLvL > 0 then
			if minItem == 0 then
				minItem = iLvL
			end
			if maxItem == 0 then
				maxItem = iLvL
			end
			if iLvL < minItem then
				minItem = iLvL
			end
			if iLvL > maxItem then
				maxItem = iLvL
			end
		end
		if iRarity >= 0 then
			itemQuality[iRarity+1].count = itemQuality[iRarity+1].count + 1
		end
		if iEnch > 0 then
			EnchCount = EnchCount + 1
		end
		if iPvP == 1 then
			PvPCount = PvPCount + 1
		end
		if SCAN[SCANGUID].gear[i].iGemMeta then
			MetaGemID = SCAN[SCANGUID].gear[i].iGemMeta
		end
	end


	-- Enchantment Stats START
	local effx = -1
	for i = 1, #slotButton do
		if SCAN[SCANGUID].gear[i].iEnch > 0 then

			local aid, hid, enchval1, enchstat1, enchval2, enchstat2, enchval3, enchstat3, enchval4, enchstat4, enchval5, enchstat5, enchval6, enchstat6 = CharScanner_GearStats_Enchantment(SCAN[SCANGUID].gear[i].iEnch)
			if aid then
				if enchstat1 then

					if enchstat1 > 0 then
						local stat = statGlobalName[enchstat1]
						if not SCAN[SCANGUID].gearStats[stat] then
							SCAN[SCANGUID].gearStats[stat] = enchval1
						else
							SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + enchval1
						end
						if not SCAN[SCANGUID].enchTmpStats[stat] then
							SCAN[SCANGUID].enchTmpStats[stat] = enchval1
						else
							SCAN[SCANGUID].enchTmpStats[stat] = SCAN[SCANGUID].enchTmpStats[stat] + enchval1
						end
					elseif enchstat1 == 0 then
						if hid then
							if SCAN[ SCANGUID ].faction == "Horde" then
								aid = hid
							end
						end
						SCAN[SCANGUID].enchTmpStats[effx] = aid
						effx = effx - 1
					end

					if enchstat2 then

						if enchstat2 > 0 then
							local stat = statGlobalName[enchstat2]
							if not SCAN[SCANGUID].gearStats[stat] then
								SCAN[SCANGUID].gearStats[stat] = enchval2
							else
								SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + enchval2
							end
							if not SCAN[SCANGUID].enchTmpStats[stat] then
								SCAN[SCANGUID].enchTmpStats[stat] = enchval2
							else
								SCAN[SCANGUID].enchTmpStats[stat] = SCAN[SCANGUID].enchTmpStats[stat] + enchval2
							end
						elseif enchstat2 == 0 then
							if hid then
								if SCAN[ SCANGUID ].faction == "Horde" then
									aid = hid
								end
							end
							SCAN[SCANGUID].enchTmpStats[effx] = aid
							effx = effx - 1
						end

						if enchstat3 then

							if enchstat3 > 0 then
								local stat = statGlobalName[enchstat3]
								if not SCAN[SCANGUID].gearStats[stat] then
									SCAN[SCANGUID].gearStats[stat] = enchval3
								else
									SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + enchval3
								end
								if not SCAN[SCANGUID].enchTmpStats[stat] then
									SCAN[SCANGUID].enchTmpStats[stat] = enchval3
								else
									SCAN[SCANGUID].enchTmpStats[stat] = SCAN[SCANGUID].enchTmpStats[stat] + enchval3
								end
							elseif enchstat3 == 0 then
								if hid then
									if SCAN[ SCANGUID ].faction == "Horde" then
										aid = hid
									end
								end
								SCAN[SCANGUID].enchTmpStats[effx] = aid
								effx = effx - 1
							end

							if enchstat4 then

								if enchstat4 > 0 then
									local stat = statGlobalName[enchstat4]
									if not SCAN[SCANGUID].gearStats[stat] then
										SCAN[SCANGUID].gearStats[stat] = enchval4
									else
										SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + enchval4
									end
									if not SCAN[SCANGUID].enchTmpStats[stat] then
										SCAN[SCANGUID].enchTmpStats[stat] = enchval4
									else
										SCAN[SCANGUID].enchTmpStats[stat] = SCAN[SCANGUID].enchTmpStats[stat] + enchval4
									end
								elseif enchstat4 == 0 then
									if hid then
										if SCAN[ SCANGUID ].faction == "Horde" then
											aid = hid
										end
									end
									SCAN[SCANGUID].enchTmpStats[effx] = aid
									effx = effx - 1
								end

								if enchstat5 then

									if enchstat5 > 0 then
										local stat = statGlobalName[enchstat5]
										if not SCAN[SCANGUID].gearStats[stat] then
											SCAN[SCANGUID].gearStats[stat] = enchval5
										else
											SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + enchval5
										end
										if not SCAN[SCANGUID].enchTmpStats[stat] then
											SCAN[SCANGUID].enchTmpStats[stat] = enchval5
										else
											SCAN[SCANGUID].enchTmpStats[stat] = SCAN[SCANGUID].enchTmpStats[stat] + enchval5
										end
									elseif enchstat5 == 0 then
										if hid then
											if SCAN[ SCANGUID ].faction == "Horde" then
												aid = hid
											end
										end
										SCAN[SCANGUID].enchTmpStats[effx] = aid
										effx = effx - 1
									end

									if enchstat6 then

										if enchstat6 > 0 then
											local stat = statGlobalName[enchstat6]
											if not SCAN[SCANGUID].gearStats[stat] then
												SCAN[SCANGUID].gearStats[stat] = enchval6
											else
												SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + enchval6
											end
											if not SCAN[SCANGUID].enchTmpStats[stat] then
												SCAN[SCANGUID].enchTmpStats[stat] = enchval6
											else
												SCAN[SCANGUID].enchTmpStats[stat] = SCAN[SCANGUID].enchTmpStats[stat] + enchval6
											end
										elseif enchstat6 == 0 then
											if hid then
												if SCAN[ SCANGUID ].faction == "Horde" then
													aid = hid
												end
											end
											SCAN[SCANGUID].enchTmpStats[effx] = aid
											effx = effx - 1
										end

									end
								end
							end
						end
					end
				end
			end

		end
	end
	-- Enchantment Stats END


	-- Set Stats START
	local SetBonus,
	      HeadSlotIs, NeckSlotIs, ShoulderSlotIs, BackSlotIs, ChestSlotIs, WristSlotIs, HandsSlotIs, WaistSlotIs, LegsSlotIs,
	      FeetSlotIs, Finger0SlotIs, Finger1SlotIs, Trinket0SlotIs, Trinket1SlotIs, MainHandSlotIs, SecondaryHandSlotIs, RangedSlotIs = CharScanner_GearStats_Set(
		SCAN[SCANGUID].gear[1].iID, SCAN[SCANGUID].gear[2].iID,
		SCAN[SCANGUID].gear[3].iID, SCAN[SCANGUID].gear[4].iID,
		SCAN[SCANGUID].gear[5].iID, SCAN[SCANGUID].gear[6].iID,
		SCAN[SCANGUID].gear[7].iID, SCAN[SCANGUID].gear[8].iID,
		SCAN[SCANGUID].gear[9].iID, SCAN[SCANGUID].gear[10].iID,
		SCAN[SCANGUID].gear[11].iID, SCAN[SCANGUID].gear[12].iID,
		SCAN[SCANGUID].gear[13].iID, SCAN[SCANGUID].gear[14].iID,
		SCAN[SCANGUID].gear[15].iID, SCAN[SCANGUID].gear[16].iID,
		SCAN[SCANGUID].gear[17].iID)

	--print(type(SetBonus), SetBonus)
	--print(HeadSlotIs, NeckSlotIs, ShoulderSlotIs, BackSlotIs, ChestSlotIs, WristSlotIs, HandsSlotIs, WaistSlotIs, LegsSlotIs,
	--	    FeetSlotIs, Finger0SlotIs, Finger1SlotIs, Trinket0SlotIs, Trinket1SlotIs, MainHandSlotIs, SecondaryHandSlotIs, RangedSlotIs)

	if SetBonus ~= "" then
		local x = 1
		local y = 1
		local effx = -1
		local statType
		local statValue
		for k, v in pairs({strsplit(", ", SetBonus)}) do
			if mod(x-1, 2) == 0 then
				if mod(y, 2) == 0 then
					local vnum = tonumber(v)
					if vnum > 0 then
						statType = statGlobalName[ tonumber(v) ]
					else
						statType = "EFFECT"
					end
				else
					statValue = tonumber(v)
				end
				y = y + 1
			end
			if statType and statValue then

				if statType == "EFFECT" then
					SCAN[SCANGUID].setTmpStats[ effx ] = statValue
					effx = effx - 1
				else
					if not SCAN[SCANGUID].gearStats[ statType ] then
						SCAN[SCANGUID].gearStats[ statType ] = statValue
					else
						SCAN[SCANGUID].gearStats[ statType ] = SCAN[SCANGUID].gearStats[ statType ] + statValue
					end
					if not SCAN[SCANGUID].setTmpStats[ statType ] then
						SCAN[SCANGUID].setTmpStats[ statType ] = statValue
					else
						SCAN[SCANGUID].setTmpStats[ statType ] = SCAN[SCANGUID].setTmpStats[ statType ] + statValue
					end
				end

				statType = nil
				statValue = nil
			end
			x = x + 1
		end
	end

	SCAN[SCANGUID].gear[1].isSet  = HeadSlotIs
	SCAN[SCANGUID].gear[2].isSet  = NeckSlotIs
	SCAN[SCANGUID].gear[3].isSet  = ShoulderSlotIs
	SCAN[SCANGUID].gear[4].isSet  = BackSlotIs
	SCAN[SCANGUID].gear[5].isSet  = ChestSlotIs
	SCAN[SCANGUID].gear[6].isSet  = WristSlotIs
	SCAN[SCANGUID].gear[7].isSet  = HandsSlotIs
	SCAN[SCANGUID].gear[8].isSet  = WaistSlotIs
	SCAN[SCANGUID].gear[9].isSet  = LegsSlotIs
	SCAN[SCANGUID].gear[10].isSet = FeetSlotIs
	SCAN[SCANGUID].gear[11].isSet = Finger0SlotIs
	SCAN[SCANGUID].gear[12].isSet = Finger1SlotIs
	SCAN[SCANGUID].gear[13].isSet = Trinket0SlotIs
	SCAN[SCANGUID].gear[14].isSet = Trinket1SlotIs
	SCAN[SCANGUID].gear[15].isSet = MainHandSlotIs
	SCAN[SCANGUID].gear[16].isSet = SecondaryHandSlotIs
	SCAN[SCANGUID].gear[17].isSet = RangedSlotIs
	-- Set Stats END

	-- Meta Socket check START
	if MetaGemID then
		SCAN[SCANGUID].gemMeta = -1
		local gem1IDid, gem1IDsocket, gem1IDval1, gem1IDstat1, gem1IDval2, gem1IDstat2, gem1IDval3, gem1IDstat3, gem1IDval4, gem1IDstat4, gem1IDval5, gem1IDstat5, gem1IDval6, gem1IDstat6, metareq = CharScanner_GearStats_Gem(MetaGemID)
		if metareq then
			local reqOkCount = 0
			local reqCount = 0
			-- check requirements START
			-- eg.: "B>R:2Y:2B" | "B>Y" | "2B:1Y" | "1R:1Y:1B" | ...
			for k, v in pairs({strsplit(":", metareq)}) do
				reqCount = reqCount + 1
				if string_find(v, ">", 1, true) then
					local firstColor = string_sub(v, 0, 1)
					local secondColor = string_sub(v, 3, 3)

					if firstColor == "B" then
						firstColor = SCAN[SCANGUID].gemBlue
					elseif firstColor == "R" then
						firstColor = SCAN[SCANGUID].gemRed
					else
						firstColor = SCAN[SCANGUID].gemYellow
					end

					if secondColor == "B" then
						secondColor = SCAN[SCANGUID].gemBlue
					elseif secondColor== "R" then
						secondColor = SCAN[SCANGUID].gemRed
					else
						secondColor = SCAN[SCANGUID].gemYellow
					end

					if firstColor > secondColor then
						reqOkCount = reqOkCount + 1
					end
				else
					local number = tonumber( string_sub(v, 0, 1) )
					local color = string_sub(v, 2, 2)

					if color == "B" then
						color = SCAN[SCANGUID].gemBlue
					elseif color== "R" then
						color = SCAN[SCANGUID].gemRed
					else
						color = SCAN[SCANGUID].gemYellow
					end

					if color >= number then
						reqOkCount = reqOkCount + 1
					end
				end
			end
			-- check requirements END

			-- add to stats table if all requirements are okay
			if reqOkCount == reqCount then
				SCAN[SCANGUID].gemMeta = 1

				local effx = -1

				if gem1IDstat1 then

					if gem1IDstat1 > 0 then
						local stat = statGlobalName[gem1IDstat1]
						if not SCAN[SCANGUID].gearStats[stat] then
							SCAN[SCANGUID].gearStats[stat] = gem1IDval1
						else
							SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem1IDval1
						end
						if not SCAN[SCANGUID].gemTmpStats[stat] then
							SCAN[SCANGUID].gemTmpStats[stat] = gem1IDval1
						else
							SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem1IDval1
						end
					elseif gem1IDstat1 == 0 then
						SCAN[SCANGUID].gemTmpStats[effx] = gem1IDval1
						effx = effx - 1
					end

					if gem1IDstat2 then

						if gem1IDstat2 > 0 then
							local stat = statGlobalName[gem1IDstat2]
							if not SCAN[SCANGUID].gearStats[stat] then
								SCAN[SCANGUID].gearStats[stat] = gem1IDval2
							else
								SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem1IDval2
							end
							if not SCAN[SCANGUID].gemTmpStats[stat] then
								SCAN[SCANGUID].gemTmpStats[stat] = gem1IDval2
							else
								SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem1IDval2
							end
						elseif gem1IDstat2 == 0 then
							SCAN[SCANGUID].gemTmpStats[effx] = gem1IDval2
							effx = effx - 1
						end

						if gem1IDstat3 then

							if gem1IDstat3 > 0 then
								local stat = statGlobalName[gem1IDstat3]
								if not SCAN[SCANGUID].gearStats[stat] then
									SCAN[SCANGUID].gearStats[stat] = gem1IDval3
								else
									SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem1IDval3
								end
								if not SCAN[SCANGUID].gemTmpStats[stat] then
									SCAN[SCANGUID].gemTmpStats[stat] = gem1IDval3
								else
									SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem1IDval3
								end
							elseif gem1IDstat3 == 0 then
								SCAN[SCANGUID].gemTmpStats[effx] = gem1IDval3
								effx = effx - 1
							end

							if gem1IDstat4 then

								if gem1IDstat4 > 0 then
									local stat = statGlobalName[gem1IDstat4]
									if not SCAN[SCANGUID].gearStats[stat] then
										SCAN[SCANGUID].gearStats[stat] = gem1IDval4
									else
										SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem1IDval4
									end
									if not SCAN[SCANGUID].gemTmpStats[stat] then
										SCAN[SCANGUID].gemTmpStats[stat] = gem1IDval4
									else
										SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem1IDval4
									end
								elseif gem1IDstat4 == 0 then
									SCAN[SCANGUID].gemTmpStats[effx] = gem1IDval4
									effx = effx - 1
								end

								if gem1IDstat5 then

									if gem1IDstat5 > 0 then
										local stat = statGlobalName[gem1IDstat5]
										if not SCAN[SCANGUID].gearStats[stat] then
											SCAN[SCANGUID].gearStats[stat] = gem1IDval5
										else
											SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem1IDval5
										end
										if not SCAN[SCANGUID].gemTmpStats[stat] then
											SCAN[SCANGUID].gemTmpStats[stat] = gem1IDval5
										else
											SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem1IDval5
										end
									elseif gem1IDstat5 == 0 then
										SCAN[SCANGUID].gemTmpStats[effx] = gem1IDval5
										effx = effx - 1
									end

									if gem1IDstat6 then

										if gem1IDstat6 > 0 then
											local stat = statGlobalName[gem1IDstat6]
											if not SCAN[SCANGUID].gearStats[stat] then
												SCAN[SCANGUID].gearStats[stat] = gem1IDval6
											else
												SCAN[SCANGUID].gearStats[stat] = SCAN[SCANGUID].gearStats[stat] + gem1IDval6
											end
											if not SCAN[SCANGUID].gemTmpStats[stat] then
												SCAN[SCANGUID].gemTmpStats[stat] = gem1IDval6
											else
												SCAN[SCANGUID].gemTmpStats[stat] = SCAN[SCANGUID].gemTmpStats[stat] + gem1IDval6
											end
										elseif gem1IDstat6 == 0 then
											SCAN[SCANGUID].gemTmpStats[effx] = gem1IDval6
											effx = effx - 1
										end

									end
								end
							end
						end
					end
				end

			end
		end
	end
	-- Meta Socket check END

	-- temp summary stats table clean & sort START
	local x = 1
	for key, value in pairs(SCAN[SCANGUID].enchTmpStats) do
		SCAN[SCANGUID].enchStats[x] = {}
		SCAN[SCANGUID].enchStats[x].stat = key
		if type(key) == "number" then
			if value > 0 then
				SCAN[SCANGUID].enchStats[x].value = value*1000000000
			else
				SCAN[SCANGUID].enchStats[x].value = value*-100
			end
		else
			SCAN[SCANGUID].enchStats[x].value = value
		end
		x = x + 1
	end
	table_sort(SCAN[SCANGUID].enchStats, function(a,b) return a.value>b.value end)
	SCAN[SCANGUID].enchTmpStats = nil

	local x = 1
	for key, value in pairs(SCAN[SCANGUID].gemTmpStats) do
		SCAN[SCANGUID].gemStats[x] = {}
		SCAN[SCANGUID].gemStats[x].stat = key
		if type(key) == "number" then
			if value > 0 then
				SCAN[SCANGUID].gemStats[x].value = value*1000000000
			else
				SCAN[SCANGUID].gemStats[x].value = value*-100
			end
		else
			SCAN[SCANGUID].gemStats[x].value = value
		end
		x = x + 1
	end
	table_sort(SCAN[SCANGUID].gemStats, function(a,b) return a.value>b.value end)
	SCAN[SCANGUID].gemTmpStats = nil

	local x = 1
	for key, value in pairs(SCAN[SCANGUID].setTmpStats) do
		SCAN[SCANGUID].setStats[x] = {}
		SCAN[SCANGUID].setStats[x].stat = key
		if type(key) == "number" then
			if value > 0 then
				SCAN[SCANGUID].setStats[x].value = value*1000000000
			else
				SCAN[SCANGUID].setStats[x].value = value*-100
			end
		else
			SCAN[SCANGUID].setStats[x].value = value
		end
		x = x + 1
	end
	table_sort(SCAN[SCANGUID].setStats, function(a,b) return a.value>b.value end)
	SCAN[SCANGUID].setTmpStats = nil
	-- temp summary stats table clean & sort END

	local quality = ""
	local x = 0
	for i = 1, itemQualityNum do
		if itemQuality[i].count > 0 then
			if x == 1 then
				quality = quality..", "
				x = 0
			end
			quality = quality.."|cff"..color.White.hex..itemQuality[i].count.."x|r |cff"..itemQuality[i].color..itemQuality[i].name.."|r"
			x = 1
		end
	end

	SCAN[SCANGUID].itemlevelMin     = minItem
	SCAN[SCANGUID].itemlevelMax     = maxItem
	SCAN[SCANGUID].itemlevelQuality = quality
	SCAN[SCANGUID].gearEnchCount    = EnchCount
	SCAN[SCANGUID].gearPvPCount     = PvPCount
	if SlotCount == 16 then
		SCAN[SCANGUID].gearTwoHandWeapon = true
	end

	local ItemIQcount = SlotCount-ItemCount
	if ItemIQcount > 0 then
		SCAN[SCANGUID].itemlevelMiss = ItemIQcount
	end

	local ItemIQ = math_floor((ItemLevel/SlotCount) * mult2 + 0.5) / mult2
	if type(ItemIQ) == "number" and ItemIQ > 0 then
		SCAN[SCANGUID].itemlevelAvg = ItemIQ
	end

	CharScanner:ShowData(2, SCANGUID, true)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function CharScanner:TalentGroup_Switch(guid, group)
	if SCAN[guid] and SCAN[guid].talents[group] then
		local activgroup = SCAN[guid].activeTalentGroup

		GVAR.TalentTreeFrame.GroupFrame.Button1:SetScript("OnClick", function() CharScanner:TalentGroup_Switch(guid, 1) end)
		GVAR.TalentTreeFrame.GroupFrame.Button2:SetScript("OnClick", function() CharScanner:TalentGroup_Switch(guid, 2) end)

		if SCAN[guid].talentGroups == 1 then
			GVAR.TalentTreeFrame.GroupFrame.Button1:Show()
			GVAR.TalentTreeFrame.GroupFrame.Button2:Hide()
		else
			GVAR.TalentTreeFrame.GroupFrame.Button1:Show()
			GVAR.TalentTreeFrame.GroupFrame.Button2:Show()
		end

		if group == 1 then
			TEMPLATE.SetTabButton(GVAR.TalentTreeFrame.GroupFrame.Button1, true)
			TEMPLATE.SetTabButton(GVAR.TalentTreeFrame.GroupFrame.Button2, false)
		else
			TEMPLATE.SetTabButton(GVAR.TalentTreeFrame.GroupFrame.Button1, false)
			TEMPLATE.SetTabButton(GVAR.TalentTreeFrame.GroupFrame.Button2, true)
		end

		if activgroup == 1 then
			GVAR.TalentTreeFrame.GroupFrame.Button1.TabText:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])
			GVAR.TalentTreeFrame.GroupFrame.Button2.TabText:SetTextColor(color.Grey.rgba[1], color.Grey.rgba[2], color.Grey.rgba[3], color.Grey.rgba[4])
		else
			GVAR.TalentTreeFrame.GroupFrame.Button1.TabText:SetTextColor(color.Grey.rgba[1], color.Grey.rgba[2], color.Grey.rgba[3], color.Grey.rgba[4])
			GVAR.TalentTreeFrame.GroupFrame.Button2.TabText:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])
		end

		local tier = 1
		local column = 1
		for tab = 1, myMAX_TALENT_TABS do

			local point
			if group == 1 then
				point = select(tab, string_match(SCAN[guid].talentGroup1pointsString, "(.+)/(.+)/(.+)"))
			else
				point = select(tab, string_match(SCAN[guid].talentGroup2pointsString, "(.+)/(.+)/(.+)"))
			end

			local bName   = GVAR.TalentTreeFrame.Tree[tab].Name
			local bPoints = GVAR.TalentTreeFrame.Tree[tab].Points

			bName:SetText(classes[ SCAN[guid].classEN ].talentTab[tab].tabName)
			bPoints:SetText(point)

			if group == activgroup then
				bName:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])
				bPoints:SetTextColor(color.White.rgba[1], color.White.rgba[2], color.White.rgba[3], color.White.rgba[4])
			else
				bName:SetTextColor(color.Grey.rgba[1], color.Grey.rgba[2], color.Grey.rgba[3], color.Grey.rgba[4])
				bPoints:SetTextColor(color.Grey.rgba[1], color.Grey.rgba[2], color.Grey.rgba[3], color.Grey.rgba[4])
			end

			for talent = 1, myMAX_NUM_TALENTS do

				if talent == 1 then
					tier = 1
					column = 1
				elseif mod(talent, myMAX_COLUMN) == 1 then
					tier = tier + 1
					column = 1
				else
					column = column + 1
				end

				local buttonName = GVAR.TalentTreeFrame.Slot[tab][tier][column]

				if
					classes[ SCAN[guid].classEN ].talents[tab] and
					classes[ SCAN[guid].classEN ].talents[tab][tier] and
					classes[ SCAN[guid].classEN ].talents[tab][tier][column]
					then

					local spellRank    = SCAN[guid].talents[group][tab][tier][column]
					local spellIcon    = classes[ SCAN[guid].classEN ].talents[tab][tier][column].icon
					local spellID      = classes[ SCAN[guid].classEN ].talents[tab][tier][column].id
					local spellName    = classes[ SCAN[guid].classEN ].talents[tab][tier][column].name
					local talentString = "talent:"..spellID..":"..(spellRank-1)
					local talentLink   = "|cff4e96f7|H"..talentString.."|h["..spellName.."]|h|r"
					buttonName.IconTexture:SetTexture(spellIcon)
					buttonName:Show()
					buttonName:SetScript("OnEnter", function(self)
						buttonName.Border:SetTexture(1, 1, 0.49, 1)
						GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
						GameTooltip:SetHyperlink(talentString)
					end)
					buttonName:SetScript("OnLeave", function()
						buttonName.Border:SetTexture(0.2, 0.2, 0.2, 1)
						GameTooltip:Hide()
					end)
					buttonName:SetScript("OnClick", function()
						HandleModifiedItemClick(talentLink)
					end)
					if spellRank == 0 then
						Desaturation(buttonName.IconTexture, true)
						buttonName:SetAlpha(0.3)
						buttonName.RankBorder:Hide()
						buttonName.RankNumber:Hide()
					else
						Desaturation(buttonName.IconTexture, false)
						buttonName:SetAlpha(1)
						buttonName.RankBorder:Show()
						buttonName.RankNumber:SetText(spellRank)
						buttonName.RankNumber:Show()
					end
				else
					buttonName:Hide()
					buttonName:SetScript("OnEnter", nil)
					buttonName:SetScript("OnLeave", nil)
				end

			end
		end
	else
		for tab = 1, myMAX_TALENT_TABS do
			GVAR.TalentTreeFrame.Tree[tab].Name:SetText("")
			GVAR.TalentTreeFrame.Tree[tab].Points:SetText("")
		end
		GVAR.TalentTreeFrame.GroupFrame.Button1:Hide()
		GVAR.TalentTreeFrame.GroupFrame.Button1:SetScript("OnClick", nil)
		GVAR.TalentTreeFrame.GroupFrame.Button2:Hide()
		GVAR.TalentTreeFrame.GroupFrame.Button2:SetScript("OnClick", nil)
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function CharScanner:ClearData()
	-- Base
	GVAR.MainFrame.Base.ClassIcon:SetTexCoord(0, 0, 0, 0)
	GVAR.MainFrame.Base.FactionIcon:SetTexCoord(0, 0, 0, 0)
	GVAR.MainFrame.Base.Name:SetText("")
	GVAR.MainFrame.Base.GUID:SetText("")
	GVAR.MainFrame.Base.Guild:SetText("")
	GVAR.MainFrame.Base.Level:SetText("")

	-- Equipment
	GVAR.MainFrame.Equipment.Title:SetText("")
	GVAR.MainFrame.Equipment.Text1:SetText("")
	GVAR.MainFrame.Equipment.Text2:SetText("")
	GVAR.MainFrame.Equipment.Text3:SetText("")

	-- Gear
	for i = 1, #slotButton do
		GVAR.GearFrame_Slot[i].IconTexture:SetTexture(slotButton[i].texture)
		GVAR.GearFrame_Slot[i].IconButton:Hide()
		GVAR.GearFrame_Slot[i].ItemLevel:Hide()
		GVAR.GearFrame_Slot[i].EnchButton:Hide()
		GVAR.GearFrame_Slot[i].ReforgeButton:Hide()
		GVAR.GearFrame_Slot[i].PvPTexture:Hide()
		GVAR.GearFrame_Slot[i].IsSetTexture:Hide()
		GVAR.GearFrame_Slot[i].Gem1Button:Hide()
		GVAR.GearFrame_Slot[i].Gem2Button:Hide()
		GVAR.GearFrame_Slot[i].Gem3Button:Hide()
		GVAR.GearFrame_Slot[i].Gem4Button:Hide()
		GVAR.GearFrame_Slot[i].GemBonusTexture:SetTexture(0, 0, 0, 0)
	end
	-- Gear Stats
	for i = 1, #statGlobalName do
		GVAR.GearFrame_Stats[i].Value:SetText(0)
	end
	GVAR.GearFrame_Stats.GemBlueText:SetText(": 0")
	GVAR.GearFrame_Stats.GemRedText:SetText(": 0")
	GVAR.GearFrame_Stats.GemYellowText:SetText(": 0")
	GVAR.GearFrame_Stats.GemMetaText:SetText(" "..META_GEM..": -")
	-- GearSlot Detail
	for i = 1, 16 do
		GVAR.GearFrame_Summary.TabButton1Frame[i].Button:Hide()
		GVAR.GearFrame_Summary.TabButton1Frame[i].Button:SetScript("OnEnter", nil)
		GVAR.GearFrame_Summary.TabButton1Frame[i].Button:SetScript("OnLeave", nil)
		GVAR.GearFrame_Summary.TabButton1Frame[i].Button:SetScript("OnClick", nil)
		GVAR.GearFrame_Summary.TabButton3Frame[i].Button:Hide()
		GVAR.GearFrame_Summary.TabButton3Frame[i].Button:SetScript("OnEnter", nil)
		GVAR.GearFrame_Summary.TabButton3Frame[i].Button:SetScript("OnLeave", nil)
		GVAR.GearFrame_Summary.TabButton3Frame[i].Button:SetScript("OnClick", nil)
		GVAR.GearFrame_Summary.TabButton4Frame[i].Button:Hide()
		GVAR.GearFrame_Summary.TabButton4Frame[i].Button:SetScript("OnEnter", nil)
		GVAR.GearFrame_Summary.TabButton4Frame[i].Button:SetScript("OnLeave", nil)
		GVAR.GearFrame_Summary.TabButton4Frame[i].Button:SetScript("OnClick", nil)
	end
	for i = 1, 7 do
		GVAR.GearFrame_Summary.TabButton2Frame[i].Text:Hide()
		GVAR.GearFrame_Summary.TabButton2Frame[i].Value:Hide()
	end

	-- Talent
	GVAR.MainFrame.Talent.Title:SetText("")
	GVAR.MainFrame.Talent.MainRole:SetTexture(nil)
	GVAR.MainFrame.Talent.MainRoleActive:SetTexture(nil)
	GVAR.MainFrame.Talent.Group1Icon:SetTexture(nil)
	GVAR.MainFrame.Talent.Group1Text:SetText("")
	GVAR.MainFrame.Talent.Group2Icon:SetTexture(nil)
	GVAR.MainFrame.Talent.Group2Text:SetText("")

	-- TalentTree
	CharScanner:TalentGroup_Switch(0, 0)

	local tier = 1
	local column = 1
	for tab = 1, myMAX_TALENT_TABS do
		for talent = 1, myMAX_NUM_TALENTS do
			if talent == 1 then
				tier = 1
				column = 1
			elseif mod(talent, myMAX_COLUMN) == 1 then
				tier = tier + 1
				column = 1
			else
				column = column + 1
			end
			GVAR.TalentTreeFrame.Slot[tab][tier][column]:Hide()
		end
	end

	-- PvE Stats
	GVAR.MainFrame.StatsPvE.Title.Name:SetText("")
	GVAR.MainFrame.StatsPvE.Title.Achievement:SetText("")
	GVAR.MainFrame.StatsPvE.Title.Statistic:SetText("")
	GVAR.MainFrame.StatsPvE.Title.BossesTotal:SetText("")

	for i = 1, #PVEDUMP do
		GVAR.MainFrame.StatsPvE.ScrollFrame["L1"..PVEDUMP[i].name].Name:SetText("")
		GVAR.MainFrame.StatsPvE.ScrollFrame["L1"..PVEDUMP[i].name].Achievement:SetText("")
		GVAR.MainFrame.StatsPvE.ScrollFrame["L1"..PVEDUMP[i].name].Statistic:SetText("")
		GVAR.MainFrame.StatsPvE.ScrollFrame["L1"..PVEDUMP[i].name].BossesTotal:SetText("")

		if PVEDUMP[i].statistic then
			for j = 1, #PVEDUMP[i].statistic do
				GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..j].Name:SetText("")
				GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..j].BossesTotal:SetText("")
			end
		else
			for j = 1, #PVEDUMP[i] do
				GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..PVEDUMP[i][j].name].Name:SetText("")
				GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..PVEDUMP[i][j].name].Achievement:SetText("")
				GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..PVEDUMP[i][j].name].Statistic:SetText("")
				GVAR.MainFrame.StatsPvE.ScrollFrame["L2"..PVEDUMP[i].name..PVEDUMP[i][j].name].BossesTotal:SetText("")

				if PVEDUMP[i][j].statistic then
					for k = 1, #PVEDUMP[i][j].statistic do
						GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k].Name:SetText("")
						GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k].BossesTotal:SetText("")
					end
				else
					for k = 1, #PVEDUMP[i][j] do
						GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k].Name:SetText("")
						GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k].Achievement:SetText("")
						GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k].Statistic:SetText("")
						GVAR.MainFrame.StatsPvE.ScrollFrame["L3"..PVEDUMP[i].name..PVEDUMP[i][j].name..k].BossesTotal:SetText("")

						if PVEDUMP[i][j][k].statistic then
							for l = 1, #PVEDUMP[i][j][k].statistic do
								GVAR.MainFrame.StatsPvE.ScrollFrame["L4"..PVEDUMP[i].name..PVEDUMP[i][j].name..PVEDUMP[i][j][k].name..l].Name:SetText("")
								GVAR.MainFrame.StatsPvE.ScrollFrame["L4"..PVEDUMP[i].name..PVEDUMP[i][j].name..PVEDUMP[i][j][k].name..l].BossesTotal:SetText("")
							end
						end

					end
				end

			end
		end
	end

	-- PvE summary
	GVAR.MainFrame.Summary1PvE.Achievement:SetText("")
	GVAR.MainFrame.Summary2PvE.Achievement:SetText("")

	GVAR.MainFrame.Summary1PvE.Statistic:SetText("")
	GVAR.MainFrame.Summary2PvE.Statistic:SetText("")

	GVAR.MainFrame.Summary1PvE.BossesTotal:SetText("")
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function CharScanner:UpdateGroupMembers(inParty, inRaid, groupSize)
	table_wipe(GroupMembers)

	if not inParty and not inRaid then
		table_wipe(GroupMissingMembers)
		CharScanner:UpdateGroupMissingMembers()
		return
	end

	if inParty then
		for i = 1, groupSize do
			local guid = UnitGUID(unitIDparty[i])
			if guid then
				GroupMembers[guid] = unitIDparty[i]
			end
		end
		GroupMembers[playerGUID] = "player"
	end
	if inRaid then
		for i = 1, groupSize do
			local guid = UnitGUID(unitIDraid[i])
			if guid then
				GroupMembers[guid] = unitIDraid[i]
			end
		end
	end

	CharScanner:UpdateGroupMissingMembers()
end

function CharScanner:UpdateGroupMissingMembers()
	table_wipe(GroupMissingMembers)
	GroupMissingNum = 0

	if inPartyOrRaid then
		for guid, unit in pairs(GroupMembers) do
			if not SCAN[guid] or (SCAN[guid] and SCAN[guid].oorScan) then
				GroupMissingMembers[guid] = unit
				GroupMissingNum = GroupMissingNum + 1
			end
		end
	else
		table_wipe(GroupMembers)
	end

	if GroupMissingNum > 0 then
		if not GroupScanIsActive and not SingleScanIsActive then
			TEMPLATE.EnableIconButton(GVAR.ScanButtonFrame.GroupScanMissing)
			TEMPLATE.EnableIconButton(GVAR.MainFrame.Headline.GroupScanMissing)
			GVAR.MiniOptionsFrame.GroupScanMissing:Enable()
			GVAR.MiniOptionsFrame.GroupScanMissing.Text:SetTextColor(color.Yellow.rgba[1], color.Yellow.rgba[2], color.Yellow.rgba[3], color.Yellow.rgba[4])
		end
	else
		TEMPLATE.DisableIconButton(GVAR.ScanButtonFrame.GroupScanMissing)
		TEMPLATE.DisableIconButton(GVAR.MainFrame.Headline.GroupScanMissing)
		GVAR.MiniOptionsFrame.GroupScanMissing:Disable()
		GVAR.MiniOptionsFrame.GroupScanMissing.Text:SetTextColor(color.Grey.rgba[1], color.Grey.rgba[2], color.Grey.rgba[3], color.Grey.rgba[4])
	end
	CharScanner:GroupMissingNumUpdate()
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function CharScanner:Unregister_INSPECT_READY()
	Event_INSPECT_READY = false
	ClearInspectPlayer()
end

function CharScanner:Unregister_INSPECT_ACHIEVEMENT_READY()
	Event_INSPECT_ACHIEVEMENT_READY = false
	-- workaround to avoid error messages because of achievementFunctions.selectedCategory
	-- -1 or "summary" does not work ... 92 is General and does not produce an error, this is not nice but quick and dirty
	if achievementFunctions and achievementFunctions.selectedCategory then
		achievementFunctions.selectedCategory = 92
	end
	ClearAchievementComparisonUnit()
end
-- ---------------------------------------------------------------------------------------------------------------------

function CharScanner:Function_INSPECT_ACHIEVEMENT_READY(guid)
		-- PvE achievement START
		SCAN[guid].PVEDATA.achievementID = {}
		for i = 1, #PVEDUMP do
			SCAN[guid].PVEDATA[i] = {}
			SCAN[guid].PVEDATA[i].achievementCount = 0
			if PVEDUMP[i].achievement then
				if not SCAN[guid].oorScan then
					for key, value in pairs(PVEDUMP[i].achievement) do
						if GetAchievementComparisonInfo(value, 1) then
							SCAN[guid].PVEDATA[i].achievementCount = SCAN[guid].PVEDATA[i].achievementCount + 1
							SCAN[guid].PVEDATA.achievementID[value] = 1
						end
					end
				end
				SCAN[guid].PvEAchievementTotal = SCAN[guid].PvEAchievementTotal + PVEDUMP[i].achievementTotal
				SCAN[guid].PvEAchievementCount = SCAN[guid].PvEAchievementCount + SCAN[guid].PVEDATA[i].achievementCount
			else

				for j = 1, #PVEDUMP[i] do
					SCAN[guid].PVEDATA[i][j] = {}
					SCAN[guid].PVEDATA[i][j].achievementCount = 0
					if PVEDUMP[i][j].achievement then
						if not SCAN[guid].oorScan then
							for key, value in pairs(PVEDUMP[i][j].achievement) do
								if GetAchievementComparisonInfo(value, 1) then
									SCAN[guid].PVEDATA[i].achievementCount = SCAN[guid].PVEDATA[i].achievementCount + 1
									SCAN[guid].PVEDATA[i][j].achievementCount = SCAN[guid].PVEDATA[i][j].achievementCount + 1
									SCAN[guid].PVEDATA.achievementID[value] = 1
								end
							end
						end
						SCAN[guid].PvEAchievementTotal = SCAN[guid].PvEAchievementTotal + PVEDUMP[i][j].achievementTotal
						SCAN[guid].PvEAchievementCount = SCAN[guid].PvEAchievementCount + SCAN[guid].PVEDATA[i][j].achievementCount
					else

						for k = 1, #PVEDUMP[i][j] do
							SCAN[guid].PVEDATA[i][j][k] = {}
							SCAN[guid].PVEDATA[i][j][k].achievementCount = 0
							if PVEDUMP[i][j][k].achievement then
								if not SCAN[guid].oorScan then
									for key, value in pairs(PVEDUMP[i][j][k].achievement) do
										if GetAchievementComparisonInfo(value, 1) then
											SCAN[guid].PVEDATA[i].achievementCount = SCAN[guid].PVEDATA[i].achievementCount + 1
											SCAN[guid].PVEDATA[i][j].achievementCount = SCAN[guid].PVEDATA[i][j].achievementCount + 1
											SCAN[guid].PVEDATA[i][j][k].achievementCount = SCAN[guid].PVEDATA[i][j][k].achievementCount + 1
											SCAN[guid].PVEDATA.achievementID[value] = 1
										end
									end
								end
								SCAN[guid].PvEAchievementTotal = SCAN[guid].PvEAchievementTotal + PVEDUMP[i][j][k].achievementTotal
								SCAN[guid].PvEAchievementCount = SCAN[guid].PvEAchievementCount + SCAN[guid].PVEDATA[i][j][k].achievementCount
							--else
							end
						end

					end

				end

			end
		end
		-- PvE achievement END

		-- PvP achievement START
		SCAN[guid].PVPDATA.achievementID = {}
		for i = 1, #PVPDUMP do
			SCAN[guid].PVPDATA[i] = {}
			SCAN[guid].PVPDATA[i].achievementCount = 0
			if PVPDUMP[i].achievement then
				if not SCAN[guid].oorScan then
					for key, value in pairs(PVPDUMP[i].achievement) do
  					local valID
						if value.id then
							valID = value.id
						else
							if SCAN[guid].faction == "Horde" then
								valID = value.Hid
							else
								valID = value.Aid
							end
						end
						if GetAchievementComparisonInfo(valID, 1) then
							SCAN[guid].PVPDATA[i].achievementCount = SCAN[guid].PVPDATA[i].achievementCount + 1
							SCAN[guid].PVPDATA.achievementID[valID] = 1
						end
					end
				end
				SCAN[guid].PvPAchievementTotal = SCAN[guid].PvPAchievementTotal + PVPDUMP[i].achievementTotal
				SCAN[guid].PvPAchievementCount = SCAN[guid].PvPAchievementCount + SCAN[guid].PVPDATA[i].achievementCount
			else

				for j = 1, #PVPDUMP[i] do
					SCAN[guid].PVPDATA[i][j] = {}
					SCAN[guid].PVPDATA[i][j].achievementCount = 0
					if PVPDUMP[i][j].achievement then
						if not SCAN[guid].oorScan then
							for key, value in pairs(PVPDUMP[i][j].achievement) do
	  						local valID
								if value.id then
									valID = value.id
								else
									if SCAN[guid].faction == "Horde" then
										valID = value.Hid
									else
										valID = value.Aid
									end
								end
								if GetAchievementComparisonInfo(valID, 1) then
									SCAN[guid].PVPDATA[i].achievementCount = SCAN[guid].PVPDATA[i].achievementCount + 1
									SCAN[guid].PVPDATA[i][j].achievementCount = SCAN[guid].PVPDATA[i][j].achievementCount + 1
									SCAN[guid].PVPDATA.achievementID[valID] = 1
								end
							end
						end
						SCAN[guid].PvPAchievementTotal = SCAN[guid].PvPAchievementTotal + PVPDUMP[i][j].achievementTotal
						SCAN[guid].PvPAchievementCount = SCAN[guid].PvPAchievementCount + SCAN[guid].PVPDATA[i][j].achievementCount
					end
				end

			end

		end
		-- PvP achievement END

		-- PvE statistic START
		for i = 1, #PVEDUMP do
			SCAN[guid].PVEDATA[i].statisticBoss = 0
			SCAN[guid].PVEDATA[i].statisticBossAll = 0
			if PVEDUMP[i].statistic then
				SCAN[guid].PVEDATA[i].statisticBossEach = {}
				if SCAN[guid].oorScan then
					for key, value in pairs(PVEDUMP[i].statistic) do
						SCAN[guid].PVEDATA[i].statisticBossEach[key] = 0
					end
				else
					for key, value in pairs(PVEDUMP[i].statistic) do
						local q = BossStatistic(value, IsUnitInpected)
						SCAN[guid].PVEDATA[i].statisticBossEach[key] = q
						if q > 0 then
							SCAN[guid].PVEDATA[i].statisticBoss = SCAN[guid].PVEDATA[i].statisticBoss + 1
							SCAN[guid].PVEDATA[i].statisticBossAll = SCAN[guid].PVEDATA[i].statisticBossAll + q
						end
					end
				end
				SCAN[guid].PvEStatisticTotal = SCAN[guid].PvEStatisticTotal + PVEDUMP[i].statisticTotal
				SCAN[guid].PvEStatisticCount = SCAN[guid].PvEStatisticCount + SCAN[guid].PVEDATA[i].statisticBoss
				SCAN[guid].PvEStatisticBossKills = SCAN[guid].PvEStatisticBossKills + SCAN[guid].PVEDATA[i].statisticBossAll
			else

				for j = 1, #PVEDUMP[i] do
					SCAN[guid].PVEDATA[i][j].statisticBoss = 0
					SCAN[guid].PVEDATA[i][j].statisticBossAll = 0
					if PVEDUMP[i][j].statistic then
						SCAN[guid].PVEDATA[i][j].statisticBossEach = {}
						if SCAN[guid].oorScan then
							for key, value in pairs(PVEDUMP[i][j].statistic) do
								SCAN[guid].PVEDATA[i][j].statisticBossEach[key] = 0
							end
						else
							for key, value in pairs(PVEDUMP[i][j].statistic) do
								local q = BossStatistic(value, IsUnitInpected)
								SCAN[guid].PVEDATA[i][j].statisticBossEach[key] = q
								if q > 0 then
									SCAN[guid].PVEDATA[i].statisticBoss = SCAN[guid].PVEDATA[i].statisticBoss + 1
									SCAN[guid].PVEDATA[i].statisticBossAll = SCAN[guid].PVEDATA[i].statisticBossAll + q
									SCAN[guid].PVEDATA[i][j].statisticBoss = SCAN[guid].PVEDATA[i][j].statisticBoss + 1
									SCAN[guid].PVEDATA[i][j].statisticBossAll = SCAN[guid].PVEDATA[i][j].statisticBossAll + q
								end
							end
						end
						SCAN[guid].PvEStatisticTotal = SCAN[guid].PvEStatisticTotal + PVEDUMP[i][j].statisticTotal
						SCAN[guid].PvEStatisticCount = SCAN[guid].PvEStatisticCount + SCAN[guid].PVEDATA[i][j].statisticBoss
						SCAN[guid].PvEStatisticBossKills = SCAN[guid].PvEStatisticBossKills + SCAN[guid].PVEDATA[i][j].statisticBossAll
					else

						for k = 1, #PVEDUMP[i][j] do
							SCAN[guid].PVEDATA[i][j][k].statisticBoss = 0
							SCAN[guid].PVEDATA[i][j][k].statisticBossAll = 0
							if PVEDUMP[i][j][k].statistic then
								SCAN[guid].PVEDATA[i][j][k].statisticBossEach = {}
								if SCAN[guid].oorScan then
									for key, value in pairs(PVEDUMP[i][j][k].statistic) do
										SCAN[guid].PVEDATA[i][j][k].statisticBossEach[key] = 0
									end
								else
									for key, value in pairs(PVEDUMP[i][j][k].statistic) do
										local q = BossStatistic(value, IsUnitInpected)
										SCAN[guid].PVEDATA[i][j][k].statisticBossEach[key] = q
										if q > 0 then
											SCAN[guid].PVEDATA[i].statisticBoss = SCAN[guid].PVEDATA[i].statisticBoss + 1
											SCAN[guid].PVEDATA[i].statisticBossAll = SCAN[guid].PVEDATA[i].statisticBossAll + q
											SCAN[guid].PVEDATA[i][j].statisticBoss = SCAN[guid].PVEDATA[i][j].statisticBoss + 1
											SCAN[guid].PVEDATA[i][j].statisticBossAll = SCAN[guid].PVEDATA[i][j].statisticBossAll + q
											SCAN[guid].PVEDATA[i][j][k].statisticBoss = SCAN[guid].PVEDATA[i][j][k].statisticBoss + 1
											SCAN[guid].PVEDATA[i][j][k].statisticBossAll = SCAN[guid].PVEDATA[i][j][k].statisticBossAll + q
										end
									end
								end
								SCAN[guid].PvEStatisticTotal = SCAN[guid].PvEStatisticTotal + PVEDUMP[i][j][k].statisticTotal
								SCAN[guid].PvEStatisticCount = SCAN[guid].PvEStatisticCount + SCAN[guid].PVEDATA[i][j][k].statisticBoss
								SCAN[guid].PvEStatisticBossKills = SCAN[guid].PvEStatisticBossKills + SCAN[guid].PVEDATA[i][j][k].statisticBossAll
							--else
							end
						end

					end

				end

			end
		end
		-- PvE statistic END

		-- PvP statistic START
		for i = 1, #PVPDUMP do
			SCAN[guid].PVPDATA[i].wins = 0
			SCAN[guid].PVPDATA[i].matches = 0
			SCAN[guid].PVPDATA[i].kills = 0
			if PVPDUMP[i].statistic then
				if SCAN[guid].oorScan then
					for key, value in pairs(PVPDUMP[i].statistic) do
						if value.wins then
							SCAN[guid].PVPDATA[i].wins = 0
						elseif value.matches then
							SCAN[guid].PVPDATA[i].matches = 0
						elseif value.kills then
							SCAN[guid].PVPDATA[i].kills = 0
						elseif value.totalHK then
							SCAN[guid].PvPStatisticTotalKills = 0
						end
					end
				else
					for key, value in pairs(PVPDUMP[i].statistic) do
						if value.wins then
							local q = BossStatistic(value.wins, IsUnitInpected)
							SCAN[guid].PVPDATA[i].wins = SCAN[guid].PVPDATA[i].wins + q
						elseif value.matches then
							local q = BossStatistic(value.matches, IsUnitInpected)
							SCAN[guid].PVPDATA[i].matches = SCAN[guid].PVPDATA[i].matches + q
						elseif value.kills then
							local q = BossStatistic(value.kills, IsUnitInpected)
							SCAN[guid].PVPDATA[i].kills = SCAN[guid].PVPDATA[i].kills + q
						elseif value.totalHK then
							local q = BossStatistic(value.totalHK, IsUnitInpected)
							SCAN[guid].PvPStatisticTotalKills = q
						end
					end
				end
				SCAN[guid].PvPStatisticWins    = SCAN[guid].PvPStatisticWins    + SCAN[guid].PVPDATA[i].wins
				SCAN[guid].PvPStatisticMatches = SCAN[guid].PvPStatisticMatches + SCAN[guid].PVPDATA[i].matches
				SCAN[guid].PvPStatisticKills   = SCAN[guid].PvPStatisticKills   + SCAN[guid].PVPDATA[i].kills
			else

				for j = 1, #PVPDUMP[i] do
					SCAN[guid].PVPDATA[i][j].wins = 0
					SCAN[guid].PVPDATA[i][j].matches = 0
					SCAN[guid].PVPDATA[i][j].kills = 0
					if PVPDUMP[i][j].statistic then
						if SCAN[guid].oorScan then
							for key, value in pairs(PVPDUMP[i][j].statistic) do
								if value.wins then
									SCAN[guid].PVPDATA[i][j].wins = 0
								elseif value.matches then
									SCAN[guid].PVPDATA[i][j].matches = 0
								elseif value.kills then
									SCAN[guid].PVPDATA[i][j].kills = 0
								elseif value.misc then
									if not SCAN[guid].PVPDATA[i][j].misc then
										SCAN[guid].PVPDATA[i][j].misc = {}
									end
									SCAN[guid].PVPDATA[i][j].misc[key] = 0
								end
							end
						else
							for key, value in pairs(PVPDUMP[i][j].statistic) do
								if value.wins then
									local q = BossStatistic(value.wins, IsUnitInpected)
									SCAN[guid].PVPDATA[i][j].wins = SCAN[guid].PVPDATA[i][j].wins + q
								elseif value.matches then
									local q = BossStatistic(value.matches, IsUnitInpected)
									SCAN[guid].PVPDATA[i][j].matches = SCAN[guid].PVPDATA[i][j].matches + q
								elseif value.kills then
									local q = BossStatistic(value.kills, IsUnitInpected)
									SCAN[guid].PVPDATA[i][j].kills = SCAN[guid].PVPDATA[i][j].kills + q
								elseif value.misc then
									local q = BossStatistic(value.misc, IsUnitInpected)
									if not SCAN[guid].PVPDATA[i][j].misc then
										SCAN[guid].PVPDATA[i][j].misc = {}
									end
									SCAN[guid].PVPDATA[i][j].misc[key] = q
								end
							end
						end
						if PVPDUMP[i].name == "Battlegrounds" then
							SCAN[guid].PvPBattlegroundWins    = SCAN[guid].PvPBattlegroundWins    + SCAN[guid].PVPDATA[i][j].wins
							SCAN[guid].PvPBattlegroundMatches = SCAN[guid].PvPBattlegroundMatches + SCAN[guid].PVPDATA[i][j].matches
							SCAN[guid].PvPBattlegroundKills   = SCAN[guid].PvPBattlegroundKills   + SCAN[guid].PVPDATA[i][j].kills
						elseif PVPDUMP[i].name == "Arena" then
							SCAN[guid].PvPArenaWins    = SCAN[guid].PvPArenaWins    + SCAN[guid].PVPDATA[i][j].wins
							SCAN[guid].PvPArenaMatches = SCAN[guid].PvPArenaMatches + SCAN[guid].PVPDATA[i][j].matches
							SCAN[guid].PvPArenaKills   = SCAN[guid].PvPArenaKills   + SCAN[guid].PVPDATA[i][j].kills
						end
						SCAN[guid].PvPStatisticWins    = SCAN[guid].PvPStatisticWins    + SCAN[guid].PVPDATA[i][j].wins
						SCAN[guid].PvPStatisticMatches = SCAN[guid].PvPStatisticMatches + SCAN[guid].PVPDATA[i][j].matches
						SCAN[guid].PvPStatisticKills   = SCAN[guid].PvPStatisticKills   + SCAN[guid].PVPDATA[i][j].kills
					end
				end

			end
		end
		-- PvP statistic END
end



-- ---------------------------------------------------------------------------------------------------------------------
local function OnEvent(self, event, ...)
	if event == "PLAYER_REGEN_DISABLED" then
		inCombat = true
		GVAR.ScanButtonFrame:Hide()
	elseif event == "PLAYER_REGEN_ENABLED" then
		inCombat = false
		if CharScanner_Options.scanButton and CharScanner_Options.scanButtonLock then
			if UnitIsPlayer("target") then
				GVAR.ScanButtonFrame:Show()
				if inPartyOrRaid then
					if CharScanner_Options.scanButtonGrpAll then
						GVAR.ScanButtonFrame.GroupScanAll:Show()
					end
					if CharScanner_Options.scanButtonGrpMiss then
						GVAR.ScanButtonFrame.GroupScanMissing:Show()
					end
				else
					GVAR.ScanButtonFrame.GroupScanAll:Hide()
					GVAR.ScanButtonFrame.GroupScanMissing:Hide()
				end
				if GroupScanIsActive or SingleScanIsActive then
					TEMPLATE.DisableIconButton(GVAR.ScanButtonFrame.SingleScan)
					TEMPLATE.DisableIconButton(GVAR.ScanButtonFrame.GroupScanAll)
					TEMPLATE.DisableIconButton(GVAR.ScanButtonFrame.GroupScanMissing)
				else
					TEMPLATE.EnableIconButton(GVAR.ScanButtonFrame.SingleScan)
					TEMPLATE.EnableIconButton(GVAR.ScanButtonFrame.GroupScanAll)
					TEMPLATE.EnableIconButton(GVAR.ScanButtonFrame.GroupScanMissing)
				end
			else
				GVAR.ScanButtonFrame:Hide()
			end
		end
		if inCombatGrpChk then
			local partymembers = GetNumPartyMembers()
			local raidmembers  = GetNumRaidMembers()
			if raidmembers > 0 then
				inPartyOrRaid = true
				CharScanner:UpdateGroupMembers(false, true, raidmembers)
				if HistoryFrameIsShown or CompareFrameIsShown then
					CharScanner:History_Update()
				end
			elseif partymembers > 0 then
				inPartyOrRaid = true
				CharScanner:UpdateGroupMembers(true, false, partymembers)
				if HistoryFrameIsShown or CompareFrameIsShown then
					CharScanner:History_Update()
				end
			else
				inPartyOrRaid = false
				CharScanner:UpdateGroupMembers(false, false, 0)
				if HistoryFrameIsShown or CompareFrameIsShown then
					CharScanner:History_Update()
				end
			end
			CharScanner:GroupScanToggle()
			inCombatGrpChk = false
		end
	elseif event == "PLAYER_TARGET_CHANGED" then
		if inCombat then return end
		if CharScanner_Options.scanButton and CharScanner_Options.scanButtonLock then
			if UnitIsPlayer("target") then
				GVAR.ScanButtonFrame:Show()
				if inPartyOrRaid then
					if CharScanner_Options.scanButtonGrpAll then
						GVAR.ScanButtonFrame.GroupScanAll:Show()
					end
					if CharScanner_Options.scanButtonGrpMiss then
						GVAR.ScanButtonFrame.GroupScanMissing:Show()
					end
				else
					GVAR.ScanButtonFrame.GroupScanAll:Hide()
					GVAR.ScanButtonFrame.GroupScanMissing:Hide()
				end
				if GroupScanIsActive or SingleScanIsActive then
					TEMPLATE.DisableIconButton(GVAR.ScanButtonFrame.SingleScan)
					TEMPLATE.DisableIconButton(GVAR.ScanButtonFrame.GroupScanAll)
					TEMPLATE.DisableIconButton(GVAR.ScanButtonFrame.GroupScanMissing)
				else
					TEMPLATE.EnableIconButton(GVAR.ScanButtonFrame.SingleScan)
					TEMPLATE.EnableIconButton(GVAR.ScanButtonFrame.GroupScanAll)
					TEMPLATE.EnableIconButton(GVAR.ScanButtonFrame.GroupScanMissing)
				end
			else
				GVAR.ScanButtonFrame:Hide()
			end
		end
	elseif event == "INSPECT_READY" then
		-- Achievement/Talent scan restriction START
		local currTime = GetTime()
		if (currTime - RestrictionTime) > RestrictionMaxTime or not RestrictionFirstTime then
			RestrictionAchNum = 0
			RestrictionTalNum = 0
		end
		RestrictionTime = currTime
		RestrictionTalNum = RestrictionTalNum + 1
		RestrictionFirstTime = true
		-- Achievement/Talent scan restriction END

		if not Event_INSPECT_READY then return end

		local talentGroup1pointsTotal = nil
		local talentGroup2pointsTotal = nil
		SCAN[SCANGUID].talentGroup1name = nil
		SCAN[SCANGUID].talentGroup1role = nil
		SCAN[SCANGUID].talentGroup1pointsString = nil
		SCAN[SCANGUID].talentGroup2name = nil
		SCAN[SCANGUID].talentGroup2role = nil
		SCAN[SCANGUID].talentGroup2pointsString = nil
		SCAN[SCANGUID].talentGroups = GetNumTalentGroups(IsUnitInpected, nil)
		SCAN[SCANGUID].activeTalentGroup = GetActiveTalentGroup(IsUnitInpected, nil)

		-- save talent information for each class START
		-- this data is saved to the classes table if a class is scanned the first time
		if not classes[SCAN[SCANGUID].classEN].talentsSaved then
			for tab = 1, myMAX_TALENT_TABS do

				local _, tabName = GetTalentTabInfo(tab, IsUnitInpected, nil)
				if not classes[SCAN[SCANGUID].classEN].talentTab then classes[SCAN[SCANGUID].classEN].talentTab = {} end
				if not classes[SCAN[SCANGUID].classEN].talentTab[tab] then
					classes[SCAN[SCANGUID].classEN].talentTab[tab] = {}
					classes[SCAN[SCANGUID].classEN].talentTab[tab].tabName = tabName
				end

				for talent=1, myMAX_NUM_TALENTS do
					local name, iconTexture, tier, column, rank = GetTalentInfo(tab, talent, IsUnitInpected, nil, nil)
					local spellLink = GetTalentLink(tab, talent, IsUnitInpected, nil, nil, nil)

					if iconTexture and spellLink then
						local spellID = select(3, string_find(spellLink, "talent:(%d+)"))

						if not classes[SCAN[SCANGUID].classEN].talents then classes[SCAN[SCANGUID].classEN].talents = {} end
						if not classes[SCAN[SCANGUID].classEN].talents[tab] then classes[SCAN[SCANGUID].classEN].talents[tab] = {} end
						if not classes[SCAN[SCANGUID].classEN].talents[tab][tier] then classes[SCAN[SCANGUID].classEN].talents[tab][tier] = {} end
						if not classes[SCAN[SCANGUID].classEN].talents[tab][tier][column] then classes[SCAN[SCANGUID].classEN].talents[tab][tier][column] = {} end

						classes[SCAN[SCANGUID].classEN].talents[tab][tier][column].id   = spellID
						classes[SCAN[SCANGUID].classEN].talents[tab][tier][column].icon = iconTexture
						classes[SCAN[SCANGUID].classEN].talents[tab][tier][column].name = name or "CharScanner_TalentScan_ERROR"
					end
				end
			end
			classes[SCAN[SCANGUID].classEN].talentsSaved = true
		end
		-- save talent information for each class END

		-- talent scan START
		for group = 1, SCAN[SCANGUID].talentGroups do
			if not SCAN[SCANGUID].talents[group] then SCAN[SCANGUID].talents[group] = {} end
			if group == 1 then
				SCAN[SCANGUID].talentGroup1pointsUnspent = GetUnspentTalentPoints(IsUnitInpected, nil, group)
			else
				SCAN[SCANGUID].talentGroup2pointsUnspent = GetUnspentTalentPoints(IsUnitInpected, nil, group)
			end
			local points = 0
			for tab = 1, myMAX_TALENT_TABS do
				if not SCAN[SCANGUID].talents[group][tab] then SCAN[SCANGUID].talents[group][tab] = {} end

				-- --------------------
				for talent=1, myMAX_NUM_TALENTS do
					local _, iconTexture, tier, column, rank = GetTalentInfo(tab, talent, IsUnitInpected, nil, group)
					if not SCAN[SCANGUID].talents[group][tab][tier] then SCAN[SCANGUID].talents[group][tab][tier] = {} end
					if iconTexture then
						SCAN[SCANGUID].talents[group][tab][tier][column] = rank
					end
				end
				-- --------------------

				-- --------------------
				local _, name, _, icon, pointsSpent = GetTalentTabInfo(tab, IsUnitInpected, nil, group)
				if name and icon and pointsSpent then
					if tab == 1 then
						if group == 1 then
							points = pointsSpent
							talentGroup1pointsTotal = pointsSpent
							SCAN[SCANGUID].talentGroup1pointsString = pointsSpent
							SCAN[SCANGUID].talentGroup1name         = name
							SCAN[SCANGUID].talentGroup1role         = tab
							SCAN[SCANGUID].talentGroup1icon         = icon
						else
							points = pointsSpent
							talentGroup2pointsTotal = pointsSpent
							SCAN[SCANGUID].talentGroup2pointsString = pointsSpent
							SCAN[SCANGUID].talentGroup2name         = name
							SCAN[SCANGUID].talentGroup2role         = tab
							SCAN[SCANGUID].talentGroup2icon         = icon
						end
					else
						if group == 1 then
							talentGroup1pointsTotal = talentGroup1pointsTotal + pointsSpent
							SCAN[SCANGUID].talentGroup1pointsString = SCAN[SCANGUID].talentGroup1pointsString.."/"..pointsSpent
							if pointsSpent > points then
								points = pointsSpent
								SCAN[SCANGUID].talentGroup1name = name
								SCAN[SCANGUID].talentGroup1role = tab
								SCAN[SCANGUID].talentGroup1icon = icon
							end
						else
							talentGroup2pointsTotal = talentGroup2pointsTotal + pointsSpent
							SCAN[SCANGUID].talentGroup2pointsString = SCAN[SCANGUID].talentGroup2pointsString.."/"..pointsSpent
							if pointsSpent > points then
								points = pointsSpent
								SCAN[SCANGUID].talentGroup2name = name
								SCAN[SCANGUID].talentGroup2role = tab
								SCAN[SCANGUID].talentGroup2icon = icon
							end
						end
					end
				end
				-- --------------------

			end
		end
		-- talent scan END

		-- opposite faction level (lvl ??/skull/-1 only!!!) check START
		if SCAN[SCANGUID].level < 0 then
			-- - Since Patch 4.0.1 there is no precise method to get the exact level for chars from the opposite faction that are more
			--   than 9 levels above our level.
			-- - If we scan a char with a skull we can get the exact level for 1,2,3,4,5,6,7,8,9,10 and 81,82,83,84,85.
			-- - All other possible levels between 11/12 and 79/80 are listed below.
			-- - If we are level 1 we can get the level from chars <= 8.
			-- - If we can not see a level and totalPoints = 0 the scanned char is level 9.
			-- NOTE: There is a possibility to get more precise results in conjunction with the gearscan (linkLevel from itemLink). This
			--       method is not 100% precise too...anyway, I think this check here is more than enough...
			local realLevel = "??"
			local totalPoints = talentGroup1pointsTotal + SCAN[SCANGUID].talentGroup1pointsUnspent

			if     totalPoints ==  0 then realLevel = "9"
			elseif totalPoints ==  1 then realLevel = "10"
			elseif totalPoints ==  2 then realLevel = "11/12"
			elseif totalPoints ==  3 then realLevel = "13/14"
			elseif totalPoints ==  4 then realLevel = "15/16"
			elseif totalPoints ==  5 then realLevel = "17/18"
			elseif totalPoints ==  6 then realLevel = "19/20"
			elseif totalPoints ==  7 then realLevel = "21/22"
			elseif totalPoints ==  8 then realLevel = "23/24"
			elseif totalPoints ==  9 then realLevel = "25/26"
			elseif totalPoints == 10 then realLevel = "27/28"
			elseif totalPoints == 11 then realLevel = "29/30"
			elseif totalPoints == 12 then realLevel = "31/32"
			elseif totalPoints == 13 then realLevel = "33/34"
			elseif totalPoints == 14 then realLevel = "35/36"
			elseif totalPoints == 15 then realLevel = "37/38"
			elseif totalPoints == 16 then realLevel = "39/40"
			elseif totalPoints == 17 then realLevel = "41/42"
			elseif totalPoints == 18 then realLevel = "43/44"
			elseif totalPoints == 19 then realLevel = "45/46"
			elseif totalPoints == 20 then realLevel = "47/48"
			elseif totalPoints == 21 then realLevel = "49/50"
			elseif totalPoints == 22 then realLevel = "51/52"
			elseif totalPoints == 23 then realLevel = "53/54"
			elseif totalPoints == 24 then realLevel = "55/56"
			elseif totalPoints == 25 then realLevel = "57/58"
			elseif totalPoints == 26 then realLevel = "59/60"
			elseif totalPoints == 27 then realLevel = "61/62"
			elseif totalPoints == 28 then realLevel = "63/64"
			elseif totalPoints == 29 then realLevel = "65/66"
			elseif totalPoints == 30 then realLevel = "67/68"
			elseif totalPoints == 31 then realLevel = "69/70"
			elseif totalPoints == 32 then realLevel = "71/72"
			elseif totalPoints == 33 then realLevel = "73/74"
			elseif totalPoints == 34 then realLevel = "75/76"
			elseif totalPoints == 35 then realLevel = "77/78"
			elseif totalPoints == 36 then realLevel = "79/80"
			elseif totalPoints == 37 then realLevel = "81"
			elseif totalPoints == 38 then realLevel = "82"
			elseif totalPoints == 39 then realLevel = "83"
			elseif totalPoints == 40 then realLevel = "84"
			elseif totalPoints == 41 then realLevel = "85" end
	
			SCAN[SCANGUID].level = realLevel
			GVAR.MainFrame.Base.Level:SetText(LEVEL.." "..SCAN[SCANGUID].level.." - "..SCAN[SCANGUID].race..", "..SCAN[SCANGUID].class)
		end
		-- opposite faction level (lvl ??/skull/-1 only!!!) check END

		-- low level (level <= 9) check START
		if (talentGroup1pointsTotal + SCAN[SCANGUID].talentGroup1pointsUnspent) == 0 then
			SCAN[SCANGUID].talentGroup1name = nil
			SCAN[SCANGUID].talentGroup1role = nil
			SCAN[SCANGUID].talentGroup1pointsString = nil
			SCAN[SCANGUID].talentGroup2name = nil
			SCAN[SCANGUID].talentGroup2role = nil
			SCAN[SCANGUID].talentGroup2pointsString = nil
			SCAN[SCANGUID].talentGroups = nil
			SCAN[SCANGUID].activeTalentGroup = nil
		end
		-- low level (level <= 9) check END

		-- main role START
		SCAN[SCANGUID].mainRole = 5
		local role
		if SCAN[SCANGUID].talentGroup1name then
			if talentGroup1pointsTotal > 0 then
				role = classes[SCAN[SCANGUID].classEN].roleNum[SCAN[SCANGUID].talentGroup1role]
				if SCAN[SCANGUID].classEN == "DRUID" then
					if role == 1 and SCAN[SCANGUID].talents[1][2][3][4]+SCAN[SCANGUID].talents[1][2][5][4] < 4 then -- Thick Hide + Natural Reaction
						role = 3 -- DAMAGE
					end
				end
				SCAN[SCANGUID].mainRole = role
				SCAN[SCANGUID].talentGroup1role = role
			else
				SCAN[SCANGUID].mainRole = 4
				SCAN[SCANGUID].talentGroup1role = 4
			end
		end

		if SCAN[SCANGUID].talentGroups and SCAN[SCANGUID].talentGroups > 1 then
			if talentGroup2pointsTotal > 0 then
				role = classes[SCAN[SCANGUID].classEN].roleNum[SCAN[SCANGUID].talentGroup2role]
				if SCAN[SCANGUID].classEN == "DRUID" then
					if role == 1 and SCAN[SCANGUID].talents[2][2][3][4]+SCAN[SCANGUID].talents[2][2][5][4] < 4 then -- Thick Hide + Natural Reaction
						role = 3 -- DAMAGE
					end
				end
				SCAN[SCANGUID].talentGroup2role = role
			else
				SCAN[SCANGUID].talentGroup2role = 4
			end
			if SCAN[SCANGUID].activeTalentGroup == 2 then
				SCAN[SCANGUID].mainRole = SCAN[SCANGUID].talentGroup2role
			end
		end
		-- main role END

		if IsUnitInpected then
			CharScanner:ItemLevelCalc(SCANUNIT, "Inspect")
		else
			CharScanner:ItemLevelCalc("player", "Character")
		end

		CharScanner:ShowData(3, SCANGUID, true)
		CharScanner:Unregister_INSPECT_READY()
	elseif event == "INSPECT_ACHIEVEMENT_READY" then
		-- Achievement/Talent scan restriction START
		local currTime = GetTime()
		if (currTime - RestrictionTime) > RestrictionMaxTime or not RestrictionFirstTime then
			RestrictionAchNum = 0
			RestrictionTalNum = 0
		end
		RestrictionTime = currTime
		RestrictionAchNum = RestrictionAchNum + 1
		RestrictionFirstTime = true
		-- Achievement/Talent scan restriction END

		if not Event_INSPECT_ACHIEVEMENT_READY then return end

		CharScanner:Function_INSPECT_ACHIEVEMENT_READY(SCANGUID)

		CharScanner:ShowData(4, SCANGUID, true)
		CharScanner:Unregister_INSPECT_ACHIEVEMENT_READY()
	elseif event == "RAID_ROSTER_UPDATE" then
		if inCombat then
			inCombatGrpChk = true
			return
		end
		local raidmembers = GetNumRaidMembers()
		if rosterhack ~= raidmembers then -- avoid execute this block if raidmembers are just moved around
			if raidmembers > 0 then
				inPartyOrRaid = true
			else
				inPartyOrRaid = false
			end	
			rosterhack = raidmembers
			CharScanner:GroupScanToggle()
			CharScanner:UpdateGroupMembers(false, true, raidmembers)
			if HistoryFrameIsShown or CompareFrameIsShown then
				CharScanner:History_Update()
			end
		end
	elseif event == "PARTY_MEMBERS_CHANGED" then
		if inCombat then
			inCombatGrpChk = true
			return
		end
		if GetNumRaidMembers() > 0 then return end -- avoid execute this block if we are in raid
		local partymembers = GetNumPartyMembers()
		if partymembers > 0 then
			inPartyOrRaid = true
		else
			inPartyOrRaid = false
		end
		CharScanner:UpdateGroupMembers(true, false, partymembers)
		if HistoryFrameIsShown or CompareFrameIsShown then
			CharScanner:History_Update()
		end
		CharScanner:GroupScanToggle()
	elseif event == "PLAYER_ENTERING_WORLD" then
		if GroupScanIsActive then
			CharScanner:StopGroupScan()
		end
		if SingleScanIsActive then
			CharScanner:StopSingleScan(true)
		end
		if not CharScanner_Options.StartMessage then
			Print("----------------------------------------")
			print(L["Read CharScanner.lua for more information."])
			print(L["Scan via SCAN-Button, Minimap-Button, LDB, Blizzard Interface Options or '/scan' resp. '/scan options'."])
			print("|cffaaaaaa"..L["This message is only shown once."].."|r")
			Print("----------------------------------------")
			CharScanner:Frame_Toggle(GVAR.OptionsFrame, 1)
			CharScanner:Frame_Toggle(GVAR.MainFrame, 1)
			CharScanner_Options.StartMessage = true
		end
	elseif event == "PLAYER_LOGIN" then
		LoadAddOn("Blizzard_InspectUI") -- we need this
		playerGUID = UnitGUID("player")
		playerFaction = UnitFactionGroup("player")
		BuildIDTable()
		CharScanner:InitializeOptions()
		CharScanner:CreateInterfaceOptions()
		CharScanner:LDBcheck()
		CharScanner:CreateMinimapButton()
		CharScanner:CreateMainFrame()
		CharScanner:CreateScanButtonFrame()
		CharScanner:CreateMiniOptionsFrame()
		CharScanner:CreateOptionsFrame()
		CharScanner:InitializeFrame()
		table_insert(UISpecialFrames, "CharScanner_MainFrame")
		table_insert(UISpecialFrames, "CharScanner_OptionsFrame")
		CharScanner:UnregisterEvent("PLAYER_LOGIN")
	end
end
-- ---------------------------------------------------------------------------------------------------------------------
CharScanner:RegisterEvent("PLAYER_LOGIN")
CharScanner:RegisterEvent("PLAYER_ENTERING_WORLD")
CharScanner:RegisterEvent("PLAYER_REGEN_DISABLED")
CharScanner:RegisterEvent("PLAYER_REGEN_ENABLED")
CharScanner:RegisterEvent("PLAYER_TARGET_CHANGED")
CharScanner:RegisterEvent("RAID_ROSTER_UPDATE")
CharScanner:RegisterEvent("PARTY_MEMBERS_CHANGED")
CharScanner:RegisterEvent("INSPECT_ACHIEVEMENT_READY")
CharScanner:RegisterEvent("INSPECT_READY")
CharScanner:SetScript("OnEvent", OnEvent) -- start the Mystical Machine